﻿'use strict';
define([
        'modules/common/services/door-agent',
        'modules/common/services/user-agent',
        'modules/common/services/access-control-agent',
        'modules/accessControl/add-schedule'
    ],
    function () {
        var app = angular.module('biostarWebApp');

        app.register.controller('AcGroupInfoController', ['$scope', '$rootScope', '$routeParams', '$location', '$timeout', '$compile', 'API', 'Const', 'Util', 'UserAgent', 'DoorAgent', 'AccessControlAgent', 'MessageProperties', 'MessageService', function ($scope, $rootScope, $routeParams, $location, $timeout, $compile, API, Const, Util, UserAgent, DoorAgent, AccessControlAgent, MessageProperties, MessageService) {

            var userListLimit = Const.SELECT_LIST_ITEM_LIMIT, userListOffset = 0, userOriginList, userOriginTotal;

            var
                i,
                item,
                userGroupTreeHandler,
                acGroupInfo = {},
                pageMode = Const.PAGE_MODE.CREATE,
                AccessLevelItem = { door_id: undefined, schedule_id: undefined };

            //AcGroup 추가를 취소하고 목록페이지로 돌아갑니다.
            $scope.doCancelAddInfo = function doCancelAddInfo() {
                $scope.$emit('CloseAcGroupInfo');
            };

            $scope.doAddAccessGroupInfo = function doAddAccessGroupInfo() {
                if ($scope.acGroupInfo.name == undefined || $scope.acGroupInfo.name.lenght < 1) {
                    MessageService.displayWarningMessage(MessageProperties.get('common.pleaseEnterName'));
                    return false;
                }

                $scope.acGroupInfo.user_groups = userGroupTreeHandler.getUserGroupsChecked();

//                if (!acGroupInfo.access_levels || !acGroupInfo.access_levels.length) {
//                    MessageService.displayWarningMessage(MessageProperties.get('validate.failedToReadAccessLevel'));
//                    return false;
//                }

                if (!$scope.acGroupInfo.users.length && !$scope.acGroupInfo.user_groups.length) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.atLeastOneOfTwoMustBeEntered', "common.userGroup", "common.user"));
                    return false;
                }

                if (pageMode == Const.PAGE_MODE.CREATE) {
//					console.log($scope.acGroupInfo)
                    AccessControlAgent.postOneAcGroup($scope.acGroupInfo).then(function success(data) {
                        $scope.$emit('CloseAcGroupInfo');
                    });
                } else if (pageMode == Const.PAGE_MODE.UPDATE) {
                    AccessControlAgent.putOneAcGroup({"id": $scope.acGroupInfo.id, "acGroupInfo": $scope.acGroupInfo}).then(function success(data) {
                        $scope.$emit('CloseAcGroupInfo');
                    });
                }
            };

            function reLoadAcLevelListData() {
                AccessControlAgent.getAccessLevels({}).then(function success(data) {
                    $scope.AcLevelListData = data.AccessLevelCollection.rows;
                });
            }

            function initPageData() {

                DoorAgent.getDoorGroupRecursiveDataWithDoors().then(function success(data) {
                    $scope.DoorGroupTreeDataWithDoors = data;
                });

                AccessControlAgent.getAccessLevels({}).then(function success(data) {
                    $scope.AcLevelListData = data.AccessLevelCollection.rows;
                });

                acGroupInfo = Util.getSessionObject('acGroupInfo');
                if (angular.isDefined(acGroupInfo.id)) {
					$scope.acGroupInfo = angular.fromJson(acGroupInfo);

					pageMode = Const.PAGE_MODE.UPDATE;
					$scope.currAcGroupId = $scope.acGroupInfo.id;
					$scope.currAcGroupName = $scope.acGroupInfo.name;
                } else {
					pageMode = Const.PAGE_MODE.CREATE;
					window.sessionStorage.removeItem('acGroupCurrPos');
					$scope.acGroupInfo = {};
				}

                var html = angular.element('<ul />', {
                        class: "tree treeGrid treeTypeUser"
                    }),
                    result;
                UserAgent.getUserGroupRecursiveData('inAcGroupInfo').then(function success(data) {
                    $scope.UserGroupTreeData = data;

                    drawRecursiveTree($scope.UserGroupTreeData, html);

                    result = angular.element('#userGroupTreeDiv').append(html);
                    $compile(result)($scope);
                });

                UserAgent.getUserGroupMemoizationData().then(function success(data) {
                    $scope.UserGroupMapData = data;
                });

                userSearchLoad();

                userOriginList = angular.copy($scope.UserListData);
                userOriginTotal = $scope.userListTotal;

				if (!$scope.acGroupInfo.users) {
					$scope.acGroupInfo.users = [];
				}

				if (!$scope.acGroupInfo.user_groups) {
					$scope.acGroupInfo.user_groups = [];
				}

				if (!$scope.acGroupInfo.access_levels) {
					$scope.acGroupInfo.access_levels = [];
				}

				$scope.checkedAccessLevelItem = [];

				for (i = 0; i < $scope.acGroupInfo.access_levels.length; i++) {
					item = $scope.acGroupInfo.access_levels[i];
					$scope.checkedAccessLevelItem.push(item.id);
				}
            }

            function userSearchLoad() {
                API.get('users', 'userSearchLoad', {limit: userListLimit, offset: userListOffset, query: $scope.userSearchWord}, null, null, null, false).then(function success(data) {
                    $scope.UserListData = data.UserCollection.rows;
                    $scope.userListTotal = data.UserCollection.total;
                    if (data.UserCollection.total == 0) {
                        $scope.UserListData = angular.copy(userOriginList);
                    }
                });
            }

            initPageData();

            // Access Level -start-
            // Access Level 검색 상자 활성화
            $scope.doActivateAccessLevelSearchBox = function doActivateAccessLevelSearchBox() {
                angular.element('#accessLevelSearchBox').addClass('typeInput').removeClass('typeClear');
                angular.element('#accessLevelSearchBoxInput').focus();
            };

            // Access Level 검색 상자 비활성화
            $scope.doDeactivateAccessLevelSearchBox = function doDeactivateAccessLevelSearchBox() {
                angular.element('#accessLevelSearchBox').removeClass('typeInput').removeClass('typeClear');
                $scope.accessLevelSearchWord = undefined;
            };

            // Input Control Focus Out 상태일때 검색상자가 공백이라면 비활성화
            angular.element('#accessLevelSearchBoxInput').on('focusout', function () {
                if (!$scope.accessLevelSearchWord || !$scope.accessLevelSearchWord.length) {
                    angular.element('#accessLevelSearchBox').removeClass('typeInput').removeClass('typeClear');
                }
            });




            var accessLevelEditMode = Const.PAGE_MODE.CREATE;
            $scope.doEditAccessLevel = function doEditAccessLevel(acLevelItem) {
                $scope.acLevelInfo = angular.copy(acLevelItem);
                angular.element(".slideLayer").addClass("displayLayer");
                accessLevelEditMode = Const.PAGE_MODE.UPDATE;
            };

            $scope.doDeleteAccessLevelInCheckList = function doDeleteAccessLevelInCheckList(acLevelItem) {

                MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                    if (confirmYn) {
                        var idx = $scope.checkedAccessLevelItem.indexOf(acLevelItem.id);

                        if (idx > -1) {
                            $scope.checkedAccessLevelItem.splice(idx, 1);
							$scope.acGroupInfo.access_levels.splice(idx, 1);
                        }

                        if (!$scope.$root.$$phase) $scope.$apply();
                    }
                });
            };

            $scope.doAddAccessLevel = function doAddAccessLevel() {
                angular.element('.listForAcLevelAddArea').focusin();
                angular.element('.listForAcLevelAddArea').focus();
                accessLevelEditMode = Const.PAGE_MODE.CREATE;
            };

            angular.element('.listForAcLevelAddArea').on('focusin', function () {
                console.log('%cFocus In', 'color:green');
                angular.element('.listForAcLevelAddArea').removeClass('hide');
            });

            angular.element('.listForAcLevelAddArea').on('focusout', function () {
                console.log('%cFocus Out', 'color:red');
                angular.element('.listForAcLevelAddArea').addClass('hide');
                $scope.accessLevelSearchWord = undefined;
            });

            angular.element('.listForAcLevelAddArea').on('mouseleave', function () {
                console.log('focusout');
                angular.element('.listForAcLevelAddArea').addClass('hide');
//                    $scope.accessLevelSearchWord = undefined;
            });

            $scope.doClickAccessLevelItem = function doClickAccessLevelItem(acLevelItem) {

                var event = window.event,
                    target = event.target || (event.srcElement || document);

                if (target.nodeName !== 'INPUT') {
                    var idx = $scope.checkedAccessLevelItem.indexOf(acLevelItem.id);

                    if (idx > -1) {
                        $scope.checkedAccessLevelItem.splice(idx, 1);
						$scope.acGroupInfo.access_levels.splice(idx, 1);
                    } else {
                        $scope.checkedAccessLevelItem.push(acLevelItem.id);
						$scope.acGroupInfo.access_levels.push({id:acLevelItem.id, name:acLevelItem.name});
                    }
                } else {
                    if (event.returnValue) event.returnValue = false;
                    if (event.preventDefault) event.preventDefault();
                }
            };
            // Access Level -end-

            // slideLayer 영역 -start-
            $scope.acLevelInfo = undefined;
            $scope.doOpenAddAccess = function () {
                $scope.acLevelInfo = {};
                angular.element(".slideLayer").addClass("displayLayer");
            };

            $scope.doCloseSlideLayer = function () {
                $scope.acLevelInfo = undefined;
                angular.element(".slideLayer").removeClass("displayLayer");


            };

            // Add Access Level Item
            $scope.doAddAccessLevelAddLine = function () {

                if (!$scope.acLevelInfo.access_level_items) {
                    $scope.acLevelInfo.access_level_items = [];
                }

                $scope.acLevelInfo.access_level_items.push(angular.copy(AccessLevelItem));
            };

            // Delete Access Level Item
            $scope.doDelAccessLevelAddLine = function ($index) {
                $scope.acLevelInfo.access_level_items.splice($index, 1);
            };

            // Post Access Level Item
            $scope.doAddAccessLevelInfo = function doAddAccessLevelInfo() {
                var i,
                    item;
                if ($.trim($scope.acLevelInfo.name).length == 0) {
                    MessageService.displayWarningMessage(MessageProperties.get('common.pleaseEnterName'));
                    return false;
                } else if (!$scope.acLevelInfo.access_level_items || !$scope.acLevelInfo.access_level_items.length) {
                    return false;
                } else {
                    for (i = 0; i < $scope.acLevelInfo.access_level_items.length; i++) {
                        item = $scope.acLevelInfo.access_level_items[i];

                        if (!item.doors || !item.doors.length) {
                            MessageService.displayWarningMessage(MessageProperties.get('validate.failedToReadDoor'));
                            return false;
                        }

                        if (!item.schedule_id || !item.schedule_id.id) {
                            MessageService.displayWarningMessage(MessageProperties.get('validate.failedToReadSchedule'));
                            return false;
                        }
                    }
                }

                if (accessLevelEditMode == Const.PAGE_MODE.CREATE) {
                    AccessControlAgent.postOneAcLevel($scope.acLevelInfo).then(function success(data) {
                        reLoadAcLevelListData();
                        $scope.doCloseSlideLayer();
                    }, function error(reason) {

                    });
                } else if (accessLevelEditMode == Const.PAGE_MODE.UPDATE) {
                    AccessControlAgent.putOneAcLevel({"id": $scope.acLevelInfo.id, "acLevelInfo": $scope.acLevelInfo}).then(function success(data) {
                        reLoadAcLevelListData();
                        $scope.doCloseSlideLayer();
                    }, function error(reason) {

                    });
                }
            };
            // slideLayer 영역 -end-

            // Page 이전/다음 버튼 영역 -start-
            $scope.currPos = window.sessionStorage.getItem('acGroupCurrPos') || undefined;

            $scope.prevAcGroupInfoBtn = function () {
                var currPos = window.sessionStorage.getItem('acGroupCurrPos');
                var prevIndex = currPos - 2;
                var allAcGroupIdList = window.sessionStorage.getItem('getAllAccessGroupIdList');

                if (allAcGroupIdList && currPos) {
                    allAcGroupIdList = angular.fromJson(allAcGroupIdList);
                } else {
                    doCancelAddInfo();
                    return false;
                }

                if (prevIndex < 0) {
                    return false;
                }

                $scope.currPos = parseInt(currPos) - 1;

                window.sessionStorage.setItem('acGroupCurrPos', $scope.currPos);

                if (allAcGroupIdList[prevIndex] && allAcGroupIdList[prevIndex] != null) {
                    getAcGroupInfo(allAcGroupIdList[prevIndex]);
                } else {
                    getAllAccessGroupIdList(prevIndex);
                }
            };

            $scope.nextAcGroupInfoBtn = function () {
                var currPos = window.sessionStorage.getItem('acGroupCurrPos');
                var nextIndex = currPos;
                var allAcGroupIdList = window.sessionStorage.getItem('getAllAccessGroupIdList');

                if (allAcGroupIdList && currPos) {
                    allAcGroupIdList = angular.fromJson(allAcGroupIdList);
                } else {
                    doCancelAddInfo();
                    return false;
                }

                if (currPos >= allAcGroupIdList.length) {
                    return false;
                }

                $scope.currPos = parseInt(currPos) + 1;

                window.sessionStorage.setItem('acGroupCurrPos', $scope.currPos);

                if (allAcGroupIdList[nextIndex] && allAcGroupIdList[nextIndex] != null) {
                    getAcGroupInfo(allAcGroupIdList[nextIndex]);
                } else {
                    getAllAccessGroupIdList(nextIndex);
                }
            };

            function getAllAccessGroupIdList(index) {
                var i, allAcGroupIdList, startIdx;
                var limit = angular.copy($scope.acGroupListQueryStrings.limit);

                allAcGroupIdList = window.sessionStorage.getItem('getAllAccessGroupIdList');

                if (allAcGroupIdList) {
                    allAcGroupIdList = angular.fromJson(allAcGroupIdList);
                } else {
                    allAcGroupIdList = [];
                }

                $scope.acGroupListQueryStrings.offset = Math.floor(index / limit) * limit;

                startIdx = parseInt($scope.acGroupListQueryStrings.offset);

                AccessControlAgent.getAccessGroups($scope.acGroupListQueryStrings, 'getAllAccessGroupIdList').then(function success(data) {
                    if (allAcGroupIdList.length != data.AccessGroupCollection.total) {
                        allAcGroupIdList = [];
                    }

                    limit = parseInt($scope.acGroupListQueryStrings.limit);

                    if (parseInt($scope.acGroupListQueryStrings.offset) + parseInt($scope.acGroupListQueryStrings.limit) > data.AccessGroupCollection.total) {
                        limit = parseInt(data.AccessGroupCollection.total) - $scope.acGroupListQueryStrings.offset;
                    }

                    for (i = 0; i < limit; i++) {
                        console.log(startIdx + i);
                        allAcGroupIdList[startIdx + i] = data.AccessGroupCollection.rows[i].id;
                    }

                    console.log(allAcGroupIdList[index]);
                    getAcGroupInfo(allAcGroupIdList[index]);

                    window.sessionStorage.setItem('getAllAccessGroupIdList', angular.toJson(allAcGroupIdList));
                });
            }

            function getAcGroupInfo(id) {
                var _acGroup;
                AccessControlAgent.getOneAcGroup(id).then(function success(data) {
                    _acGroup = data;

                    if (_acGroup) {
                        window.sessionStorage.setItem('acGroupInfo', angular.toJson(_acGroup));

                        $scope.acGroupInfo = _acGroup;

						if (!$scope.acGroupInfo.users) {
							$scope.acGroupInfo.users = [];
						}

						if (!$scope.acGroupInfo.user_groups) {
							$scope.acGroupInfo.user_groups = [];
						}

						if (!$scope.acGroupInfo.access_levels) {
							$scope.acGroupInfo.access_levels = [];
						}

						$scope.checkedAccessLevelItem = [];

						for (i = 0; i < $scope.acGroupInfo.access_levels.length; i++) {
							item = $scope.acGroupInfo.access_levels[i];
							$scope.checkedAccessLevelItem.push(item.id);
						}

                        userGroupTreeHandler = new UserGroupTreeHandler($scope.acGroupInfo);

                        $scope.currAcGroupId = $scope.acGroupInfo.id;
                        $scope.currAcGroupName = $scope.acGroupInfo.name;


                    }
                });
            };
            // Page 이전/다음 버튼 영역 -end-


            // User Group Tree Handler -start-
            var mapOfAllUserGroups = {};

            function drawRecursiveTree(userGroups, html) {
                var recurPos, item, ul, li, div, indent;

                li = angular.element('<li />', {
                });

                if (userGroups && userGroups.length) {
                    for (recurPos = 0; recurPos < userGroups.length; recurPos++) {
                        item = userGroups[recurPos];
                        mapOfAllUserGroups[item.id] = item;
                        div = angular.element('<div />', {
                            id: '_user_tree_' + item.id,
                            'ng-class': '{\'tree-node-selected\' : selectedUserGroup.id == ' + item.id + '}',
                            'ng-hide': "userGroupSearchWord.length > 0 && getNonameFunction('" + item.name + "')",
                            'ng-click': "doSelectUserGroup(" + item.id + ")",
                            class: 'tree-node'
                        });

                        angular.element(div).append(angular.element('<span/>', {
                            class: 'tree-checkbox'
                        }));
                        for (indent = 0; indent < item.depth; indent++) {
                            angular.element(div).append(angular.element('<span/>', {
                                class: 'tree-indent'
                            }));
                        }

                        var cls = '', ngClass = '';
                        if (item.user_groups.length == 0) {
                            cls = 'tree-indent';
                        } else if (item.user_groups.length > 0) {
                            cls = 'tree-hit';
                            ngClass += '{\'tree-expanded\':treeCollapsedOK[' + item.id + '] != true' + ',';
                            ngClass += '\'tree-collapsed\':treeCollapsedOK[' + item.id + '] == true}';
                        }

                        angular.element(div).append(angular.element('<span/>', {
                            class: cls,
                            'ng-class': ngClass
                        }));
                        angular.element(div).append(angular.element('<span/>', {
                            class: 'tree-icon tree-folder'
                        }));
                        angular.element(div).append(angular.element('<span/>', {
                            class: 'tree-title wordBreak',
//                                style: 'width: ' + (16.5 - item.depth) + 'em;',
                            title: item.path,
                            text: item.name
                        }));

                        angular.element(li).append(div);
                        if (item.user_groups) {
                            ul = angular.element('<ul />', {
                                'ng-show': 'treeCollapsedOK[' + item.id + '] != true'
                            });
                            var retHtml = drawRecursiveTree(item.user_groups, ul);
                            angular.element(li).append(retHtml);
                        } else {

                        }
                    }
                }

                angular.element(html).append(li);

                return html;
            }

            $scope.checkedUserItem = [];
            $scope.checkedUser = [];
            $scope.userCountOfUserGroups = {};
            $scope.treeCollapsedOK = {};


            $scope.getNonameFunction = function (name) {
                return angular.lowercase(name).indexOf(angular.lowercase($scope.userGroupSearchWord)) == -1;
            };

            $scope.checkedUserGroupDataList = [];
            $scope.checkedUserGroupDataObjectList = [];
            function UserGroupTreeHandler() {
                var  item, userGroupList;

                $scope.checkedUserGroupDataList = [];
                $scope.checkedUserGroupDataObjectList = [];

                $(".tree-checkbox").each(function () {
                    angular.element(this).removeAttr("class").addClass('tree-checkbox tree-checkbox0');
                });

                UserAgent.getUserGroupMemoizationData().then(function success(data) {
                    userGroupList = data;
                    setCheckedUserGroupDataList();
                    setCheckedUserDataList();
                    setTreeFoldingMemoization();
                });


                function setTreeFoldingMemoization() {
                    var i, key;
                    for (key in userGroupList) {
                        $scope.treeCollapsedOK[key] = false;
                    }
                }

                function setCheckedUserGroupDataList() {
                    var i, element, isChecked, userGroup;
                    if ($scope.acGroupInfo.user_groups && $scope.acGroupInfo.user_groups.length) {
                        for (i = 0; i < $scope.acGroupInfo.user_groups.length; i++) {
                            userGroup = $scope.acGroupInfo.user_groups[i];
                            userGroup = userGroupList[userGroup.id];

                            if (!userGroup) {
                                continue;
                            }
                            element = angular.element('#_user_tree_' + userGroup.id).find('.tree-checkbox');
                            isChecked = element.is('.tree-checkbox1');

                            if (!isChecked) {
                                checkUserGroupTreeCheckbox(userGroup);
                                removeChildrenUserGroupInCheckedList(userGroup);
                            }
                        }
                    }
                }

                function setCheckedUserDataList() {
                    var i;

                    $scope.checkedUserItem = [];
                    $scope.checkedUser = [];
                    if ($scope.acGroupInfo.users && $scope.acGroupInfo.users.length) {
                        for (i = 0; i < $scope.acGroupInfo.users.length; i++) {
                            item = $scope.acGroupInfo.users[i];
                            $scope.checkedUserItem.push(item.user_id);
                            $scope.checkedUser.push(item);
                        }
                    }
                }

                function addUserGroupItemToCheckList(userGroup) {
                    var isChecked,
                        idx = $scope.checkedUserGroupDataList.indexOf(userGroup.id),
                        element = angular.element('#_user_tree_' + userGroup.id).find('.tree-checkbox');
                    isChecked = element.is('.tree-checkbox1');

                    if (idx == -1) {
                        if (isChecked) {
							$scope.checkedUserGroupDataList.push(userGroup.id);
							$scope.checkedUserGroupDataObjectList.push({id:userGroup.id, name:userGroup.name});
                        }
                    } else if (idx > -1) {
                        if (!isChecked) {
							$scope.checkedUserGroupDataList.splice(idx, 1);
							$scope.checkedUserGroupDataObjectList.splice(idx, 1);
                        }
                    }
                }

                function doSelectUserGroup(group_id) {
                    var userGroup = mapOfAllUserGroups[group_id];

                    var event = window.event,
                        target = event.target || (event.srcElement || document),
                        element = angular.element('#_user_tree_' + userGroup.id).find('.tree-checkbox');

                    if (target.className.indexOf('tree-checkbox') > -1) {
                        checkUserGroupTreeCheckbox(userGroup);
                    } else if (target.className.indexOf('tree-hit') > -1) {
                        $scope.treeCollapsedOK[userGroup.id] = !$scope.treeCollapsedOK[userGroup.id];
                    } else {
                        checkUserGroupTreeCheckbox(userGroup);
                    }

                    $scope.selectedUserGroup = userGroup;
                }

                function checkUserGroupTreeCheckbox(userGroup) {

                    var element = angular.element('#_user_tree_' + userGroup.id).find('.tree-checkbox'),
                        isChecked = element.is('.tree-checkbox1');

                    if (isChecked) {
                        element.removeAttr('class').addClass('tree-checkbox tree-checkbox0');
                    } else {
                        element.removeAttr('class').addClass('tree-checkbox tree-checkbox1');
                    }

                    addUserGroupItemToCheckList(userGroup);

                    childrenCascadeCheck(userGroup);
                    parentCascadeCheck(userGroup);

                }

                function childrenCascadeCheck(userGroup) {
                    var i, item, element,
                        isChecked = angular.element('#_user_tree_' + userGroup.id)
                            .find('.tree-checkbox').is('.tree-checkbox1'),
                        user_groups = userGroup.user_groups;

                    if (user_groups) {
                        for (i = 0; i < user_groups.length; i++) {
                            item = user_groups[i];
                            element = angular.element('#_user_tree_' + item.id).find('.tree-checkbox');


                            if (isChecked) {
                                element.removeAttr('class').addClass('tree-checkbox tree-checkbox1');
                            } else {
                                element.removeAttr('class').addClass('tree-checkbox tree-checkbox0');
                            }
                            childrenCascadeCheck(item);
                        }
                        removeChildrenUserGroupInCheckedList(userGroup);
                    }
                }

                function parentCascadeCheck(userGroup) {
                    var i, item, idx,
                        childElement, parent, element,
                        childIsCheckedCnt,
                        childIsIndeterminateCnt;


                    if (userGroup.parent_id) {
                        parent = userGroupList[userGroup.parent_id.id];
                        element = angular.element('#_user_tree_' + parent.id).find('.tree-checkbox');

                        childIsCheckedCnt = 0;
                        childIsIndeterminateCnt = 0;


                        for (i = 0; i < parent.user_groups.length; i++) {
                            item = parent.user_groups[i];
                            childElement = angular.element('#_user_tree_' + item.id).find('.tree-checkbox');

                            if (childElement.is('.tree-checkbox1')) {
                                childIsCheckedCnt++;
                            }

                            if (childElement.is('.tree-checkbox2')) {
                                childIsIndeterminateCnt++;
                            }
                        }


                        if (parent.user_groups && parent.user_groups.length) {
                            if (childIsCheckedCnt && childIsCheckedCnt == parent.user_groups.length) {

//                                    element.removeAttr('class').addClass('tree-checkbox tree-checkbox1');
//                                    addUserGroupItemToCheckList(parent);
//                                    removeChildrenUserGroupInCheckedList(parent);
                            } else if (childIsCheckedCnt == 0 && parent.user_groups.length > 0) {

                                element.removeAttr('class').addClass('tree-checkbox tree-checkbox0');
                            } else {

                                element.removeAttr('class').addClass('tree-checkbox tree-checkbox2');
                                addUserGroupItemToCheckList(parent);

                                for (i= 0; i < parent.user_groups.length; i++) {
                                    item = parent.user_groups[i];
                                    addUserGroupItemToCheckList(item);
                                }
                            }

                            if (childIsCheckedCnt < parent.user_groups.length && childIsIndeterminateCnt > 0) {
                                element.removeAttr('class').addClass('tree-checkbox tree-checkbox2');
                            }
                        }

                        parentCascadeCheck(parent);
                    }
                }

                function removeChildrenUserGroupInCheckedList(parent) {
                    var i, item, idx;

                    if (parent.user_groups) {
                        for (i = 0; i < parent.user_groups.length; i++) {
                            item = parent.user_groups[i];
                            idx = $scope.checkedUserGroupDataList.indexOf(item.id);
                            if (idx > -1) {
								$scope.checkedUserGroupDataList.splice(idx, 1);
								$scope.checkedUserGroupDataObjectList.splice(idx, 1);
                            }
                        }
                    }
                }

                function doCheckUserItem(user) {
                    var idx,
                        event = window.event,
                        target = event.target || (event.srcElement || document);

                    if (target.nodeName !== 'INPUT') {
                        idx = $scope.checkedUserItem.indexOf(user.user_id);
                        if (idx > -1) {
                            $scope.checkedUserItem.splice(idx, 1);
                        } else if (idx == -1) {
                            $scope.checkedUserItem.push(user.user_id);
                            $scope.checkedUser.push(user);
                        }
                    } else {
                        if (event.returnValue) event.returnValue = false;
                        if (event.preventDefault) event.preventDefault();
                    }
                }

                function doAllCheckUserListItems() {
                    var i, idx, item; // checkedUserItem
                    $scope.userListSelectAll = !$scope.userListSelectAll;
                    for (i = 0; i < $scope.UserListData.length; i++) {
                        item = $scope.UserListData[i];
                        idx = $scope.checkedUserItem.indexOf(item.user_id);
                        if ($scope.userListSelectAll) {
                            if (idx == -1) {
                                $scope.checkedUserItem.push(item.user_id);
                                $scope.checkedUser.push(item);
                            }
                        } else if (!$scope.userListSelectAll) {
                            if (idx > -1) {
                                $scope.checkedUserItem.splice(idx, 1);
                            }
                        }
                    }
                }

                return {
                    checkUserGroupTreeCheckbox: checkUserGroupTreeCheckbox,
                    doSelectUserGroup: doSelectUserGroup,
                    doCheckUserItem: doCheckUserItem,
                    doAllCheckUserListItems: doAllCheckUserListItems,
                    getUserGroupsChecked: function () {
                        var i,
                            item,
                            _checkedItems = [];

                        for (i = 0; i < $scope.checkedUserGroupDataList.length; i++) {
                            item = $scope.checkedUserGroupDataList[i];
                            _checkedItems.push({id: userGroupList[item].id});
                        }

                        return angular.copy(_checkedItems);
                    }
                }
            }


            $timeout(function () {
                userGroupTreeHandler = new UserGroupTreeHandler($scope.acGroupInfo);
                $scope.doSelectUserGroup = userGroupTreeHandler.doSelectUserGroup;
                $scope.doAllCheckUserListItems = userGroupTreeHandler.doAllCheckUserListItems;
                $scope.checkUserGroupTreeCheckbox = userGroupTreeHandler.checkUserGroupTreeCheckbox;
            }, 200);

            // UserGroup Add 버튼
            $scope.doAddUserGroup = function doAddUserGroup() {
                angular.element('.listForUserGroupAddArea').focusin();
                angular.element('.listForUserGroupAddArea').focus();
            };

            // UserGroup Delete 버튼
            $scope.doDeleteUserGroupInCheckList = function doDeleteUserGroupInCheckList(item) {
                MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                    if (confirmYn) {
                        $scope.checkUserGroupTreeCheckbox(item);

                        if (!$scope.$root.$$phase) $scope.$apply();
                    }
                });
            };

            angular.element('.listForUserGroupAddArea').on('focusin', function () {
                angular.element('.listForUserGroupAddArea').removeClass('hide');
            });
            angular.element('.listForUserGroupAddArea').on('focusout', function () {
                angular.element('.listForUserGroupAddArea').addClass('hide');
                $scope.userGroupSearchWord = undefined;
            });
            angular.element('.listForUserGroupAddArea').on('mouseleave', function () {
                console.log('focusout');
//                    angular.element('.listForUserGroupAddArea').addClass('hide');
//                    $scope.userGroupSearchWord = undefined;
            });

            // User Add 버튼
            $scope.doAddUser = function doAddUser() {
                angular.element('.listForUserAddArea').focusin();
                angular.element('.listForUserAddArea').focus();
                $scope.UserListData = angular.copy(userOriginList);
            };

            $scope.doDeleteUserInCheckList = function doDeleteUserInCheckList(item) {
                MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                    if (confirmYn) {
                        $scope.doCheckUserItem(item);

                        if (!$scope.$root.$$phase) $scope.$apply();
                    }
                });
            };

            $scope.doCheckUserItem = function (user) {
                var idx,
                    event = window.event,
                    target = event.target || (event.srcElement || document);

                idx = $scope.checkedUserItem.indexOf(user.user_id);
                if (idx > -1) {
                    $scope.checkedUserItem.splice(idx, 1);
					$scope.acGroupInfo.users.splice(idx, 1);
                } else if (idx == -1) {
                    $scope.checkedUserItem.push(user.user_id);
                    $scope.acGroupInfo.users.push({user_id:user.user_id, name:user.name});
                }
                angular.element('.listForUserAddArea').focus();
                angular.element('#userSearchBox').removeClass('typeInput').removeClass('typeClear');
                $scope.userSearchWord = null;
            };

            angular.element('.listForUserAddArea').on('focusin', function () {
                console.log('focusin');
                angular.element('.listForUserAddArea').removeClass('hide')
            });
            angular.element('.listForUserAddArea').on('focusout', function () {
                console.log('focusout');
                angular.element('.listForUserAddArea').addClass('hide');
                $scope.userSearchWord = undefined;
            });
            angular.element('.listForUserAddArea').on('mouseleave', function () {
                console.log('focusout');
                angular.element('.listForUserAddArea').addClass('hide');
                $scope.userSearchWord = undefined;
            });

            // UserGroup 검색 상자 활성화
            $scope.doActivateUserGroupSearchBox = function doActivateUserGroupSearchBox() {
                angular.element('#userGroupSearchBox').addClass('typeInput').removeClass('typeClear');
                angular.element('#userGroupSearchBoxInput').focus();
            };

            // UserGroup 검색 상자 비활성화
            $scope.doDeactivateUserGroupSearchBox = function doDeactivateUserGroupSearchBox() {
                angular.element('#userGroupSearchBox').removeClass('typeInput').removeClass('typeClear');
                $scope.userGroupSearchWord = undefined;
            };

            // Input Control Focus Out 상태일때 UserGroup 검색상자가 공백이라면 비활성화
            angular.element('#userGroupSearchBoxInput').on('focusout', function () {
                if (!$scope.userGroupSearchWord || !$scope.userGroupSearchWord.length) {
                    angular.element('#userGroupSearchBox').removeClass('typeInput').removeClass('typeClear');
                }
            });

            // User 검색 상자 활성화
            $scope.doActivateUserSearchBox = function doActivateUserSearchBox() {
                angular.element('#userSearchBox').addClass('typeInput').removeClass('typeClear');
                angular.element('#userSearchBoxInput').focus();
            };

            // User 검색 상자 비활성화
            $scope.doDeactivateUserSearchBox = function doDeactivateUserSearchBox() {
                angular.element('#userSearchBox').removeClass('typeInput').removeClass('typeClear');
                $scope.userSearchWord = undefined;
            };

            // Input Control Focus Out 상태일때 User 검색상자가 공백이라면 비활성화
            angular.element('#userSearchBoxInput').on('focusout', function () {
                if (!$scope.userSearchWord || !$scope.userSearchWord.length) {
                    angular.element('#userSearchBox').removeClass('typeInput').removeClass('typeClear');
                }
            });

            // User Group Tree Handler -end-

            $scope.userListFiltered = [];
            $scope.userListTotal = null;
            $scope.$watch('userListFiltered ', function (nv, ov) {
                if (nv != ov) {
                    if ($scope.userListFiltered.length == 0) {
                        API.get('users', 'userSearchLoad', {limit: userListLimit, offset: userListOffset, query: $scope.userSearchWord}, null, null, null, false).then(function success(data) {
                            $scope.UserListData = data.UserCollection.rows;
                            $scope.userListTotal = data.UserCollection.total;
                            if (data.UserCollection.total == 0) {
                                $scope.UserListData = angular.copy(userOriginList);
                            }
                        });
                    }
                }
            }, true);

            $scope.$watch('userSearchWord', function (nv, ov) {
                if (angular.isDefined(nv) && angular.isDefined(ov)) {
                    if (nv.length < ov.length) {
                        API.get('users', 'userSearchLoad', {limit: userListLimit, offset: userListOffset, query: $scope.userSearchWord}, null, null, null, false).then(function success(data) {
                            $scope.UserListData = data.UserCollection.rows;
                            $scope.userListTotal = data.UserCollection.total;
                            if (data.UserCollection.total == 0) {
                                $scope.UserListData = angular.copy(userOriginList);
                            }
                        });
                    }
                }
            });

            $scope.moreDataShowCondition = function () {

                if ($scope.UserListData && $scope.userListTotal != $scope.UserListData.length) {
                    if ($scope.noDataShowCondition()) {
                        return false;
                    } else {
                        return true;
                    }
                } else {
                    return false;
                }
            };

            $scope.noDataShowCondition = function () {
                if ($scope.userListFiltered.length == 0) {
                    return true;
                } else if ($scope.userListFiltered.length > 0 && $scope.checkedUserItem.length > 0) {
                    var checkedUserItemListInFilteredItemsCount = 0;

                    for (var i = 0; i < $scope.userListFiltered.length; i++) {
                        if ($scope.checkedUserItem.indexOf($scope.userListFiltered[i].user_id) > -1) {
                            checkedUserItemListInFilteredItemsCount++;
                        }
                    }

                    return $scope.userListFiltered.length == checkedUserItemListInFilteredItemsCount;
                } else {
                    return false;
                }
            };


        }]);


    });