'use strict';

define([
        'modules/common/services/door-agent',
        'modules/common/services/access-control-agent',
        'modules/accessControl/add-schedule'
    ],
    function () {

        var app = angular.module('biostarWebApp');
        app.register.controller('AcLevelInfoController', ['$scope', '$rootScope', '$routeParams', '$location', '$timeout', 'API', 'Const', 'Util', 'DoorAgent', 'AccessControlAgent', 'MessageProperties', 'MessageService', function ($scope, $rootScope, $routeParams, $location, $timeout, API, Const, Util, DoorAgent, AccessControlAgent, MessageProperties, MessageService) {

            var
                saveData,
                acLevelInfo = {},
                originalAcLevelData = {},
                pageMode = 'create',
                AcLevelListData = [],
                acLevelListQueryStrings = angular.copy($scope.acLevelListQueryStrings),
                AccessLevelItem = {
                    doors: [],
                    schedule_id: undefined
                };


            //AcLevel 추가를 취소하고 목록페이지로 돌아갑니다.
            $scope.doCancelAddInfo = function doCancelAddInfo() {
                $scope.$emit('CloseAcLevelInfo');
            };

            $scope.doAddAccessLevelInfo = function doAddAccessLevelInfo() {
                var i,
                    item;

                if ($.trim($scope.acLevelInfo.name).length <= 0) {
                    MessageService.displayWarningMessage(MessageProperties.get('common.pleaseEnterName'));
                    return false;
                }

                if (!$scope.acLevelInfo.access_level_items || !$scope.acLevelInfo.access_level_items.length) {
                    MessageService.displayWarningMessage(MessageProperties.get('common.noAccessLevelInfo'));
                    return false;
                } else {
                    for (i = $scope.acLevelInfo.access_level_items.length; i--;) {
                        item = $scope.acLevelInfo.access_level_items[i];

                        if (!item.doors || !item.doors.length) {
                            MessageService.displayWarningMessage(MessageProperties.get('validate.failedToReadDoor'));
                            return false;
                        }

                        if (!item.schedule_id || !item.schedule_id.id) {
                            MessageService.displayWarningMessage(MessageProperties.get('validate.failedToReadSchedule'));
                            return false;
                        }
                    }
                }

                if (pageMode == 'create') {
                    AccessControlAgent.postOneAcLevel($scope.acLevelInfo).then(function success(data) {
                        $scope.$emit('CloseAcLevelInfo');
                    }, function error(reason) {

                    });
                } else if (pageMode == 'update') {
                    Util.debug($scope.acLevelInfo.id);
                    AccessControlAgent.putOneAcLevel({"id": $scope.acLevelInfo.id, "acLevelInfo": $scope.acLevelInfo}).then(function success(data) {
                        $scope.$emit('CloseAcLevelInfo');
                    }, function error(reason) {

                    });
                }
            };

            DoorAgent.getDoorGroupRecursiveDataWithDoors().then(function success(data) {
                $scope.DoorGroupTreeDataWithDoors = data;
            });

            acLevelInfo = window.sessionStorage.getItem('acLevelInfo');
            if (acLevelInfo && acLevelInfo != 'undefined') {
				$scope.acLevelInfo = angular.fromJson(acLevelInfo);
            }


            // AcLevel 정보가 sessionStorage에 있다면..
            if ($scope.acLevelInfo.id) {
                pageMode = 'update';

                $scope.currAcLevelId = $scope.acLevelInfo.id;


                // 세션에 AcLevel 정보가 없다면 다시 서버에서 가져온다.
                if (!$scope.acLevelInfo) {
                    AccessControlAgent.getOneAcLevel($scope.currAcLevelId).then(function success(data) {
						$scope.acLevelInfo = data;
                        originalAcLevelData = angular.copy($scope.acLevelInfo);

                    });
                }

                $scope.currAcLevelName = $scope.acLevelInfo.name;

            } else { // acLevelId가 파라미터로 넘어오지 않았다면 추가 모드
                pageMode = 'create';

                window.sessionStorage.removeItem('acLevelCurrPos');
            }

            // 페이지의 모든 초기 작업이 완료된 후 AcLevel 데이터 원본을 저장한다.
            originalAcLevelData = angular.copy($scope.acLevelInfo);

            // Add Access Level Item
            $scope.doAddAccessLevelAddLine = function () {
                if (!$scope.acLevelInfo.access_level_items) {
                    $scope.acLevelInfo.access_level_items = [];
                }

                $scope.acLevelInfo.access_level_items.push(angular.copy(AccessLevelItem));
            };

            // Delete Access Level Item
            $scope.doDelAccessLevelAddLine = function ($index) {
                $scope.acLevelInfo.access_level_items.splice($index, 1);
            };

//                $rootScope.$on('$routeChangeStart', function (evt, next, curr) {
//                    window.sessionStorage.removeItem('acLevelInfo');
//                });

            AcLevelListData = window.sessionStorage.getItem('AcLevelListData') || {};
            AcLevelListData = angular.fromJson(AcLevelListData);

            $scope.currPos = window.sessionStorage.getItem('acLevelCurrPos') || undefined;

            $scope.prevAcLevelInfoBtn = function () {
                var currPos = window.sessionStorage.getItem('acLevelCurrPos');
                var prevIndex = currPos - 2;
                var allAcLevelIdList = window.sessionStorage.getItem('getAllAccessLevelIdList');

                if (allAcLevelIdList && currPos) {
                    allAcLevelIdList = angular.fromJson(allAcLevelIdList);
                } else {
                    doCancelAddInfo();
                    return false;
                }

                if (prevIndex < 0) {
                    return false;
                }

                $scope.currPos = parseInt(currPos) - 1;

                window.sessionStorage.setItem('acLevelCurrPos', $scope.currPos);

                if (allAcLevelIdList[prevIndex] && allAcLevelIdList[prevIndex] != null) {
                    getAcLevelInfo(allAcLevelIdList[prevIndex]);
                } else {
                    getAllAccessLevelIdList(prevIndex);
                }
            };

            $scope.nextAcLevelInfoBtn = function () {
                var currPos = window.sessionStorage.getItem('acLevelCurrPos');
                var nextIndex = currPos;
                var allAcLevelIdList = window.sessionStorage.getItem('getAllAccessLevelIdList');

                if (allAcLevelIdList && currPos) {
                    allAcLevelIdList = angular.fromJson(allAcLevelIdList);
                } else {
                    doCancelAddInfo();
                    return false;
                }

                if (currPos >= allAcLevelIdList.length) {
                    return false;
                }

                $scope.currPos = parseInt(currPos) + 1;

                window.sessionStorage.setItem('acLevelCurrPos', $scope.currPos);

                if (allAcLevelIdList[nextIndex] && allAcLevelIdList[nextIndex] != null) {
                    getAcLevelInfo(allAcLevelIdList[nextIndex]);
                } else {
                    getAllAccessLevelIdList(nextIndex);
                }
            };

            function getAllAccessLevelIdList(index) {
                var i, allAcLevelIdList, startIdx;
                var limit = angular.copy($scope.acLevelListQueryStrings.limit);

                allAcLevelIdList = window.sessionStorage.getItem('getAllAccessLevelIdList');

                if (allAcLevelIdList) {
                    allAcLevelIdList = angular.fromJson(allAcLevelIdList);
                } else {
                    allAcLevelIdList = [];
                }

                $scope.acLevelListQueryStrings.offset = Math.floor(index / limit) * limit;

                startIdx = parseInt($scope.acLevelListQueryStrings.offset);

                AccessControlAgent.getAccessLevels($scope.acLevelListQueryStrings, 'getAllAccessLevelIdList').then(function success(data) {
                    if (allAcLevelIdList.length != data.AccessLevelCollection.total) {
                        allAcLevelIdList = [];
                    }

                    limit = parseInt($scope.acLevelListQueryStrings.limit);

                    if (parseInt($scope.acLevelListQueryStrings.offset) + parseInt($scope.acLevelListQueryStrings.limit) > data.AccessLevelCollection.total) {
                        limit = parseInt(data.AccessLevelCollection.total) - $scope.acLevelListQueryStrings.offset;
                    }

                    for (i = 0; i < limit; i++) {
                        console.log(startIdx + i);
                        allAcLevelIdList[startIdx + i] = data.AccessLevelCollection.rows[i].id;
                    }

                    console.log(allAcLevelIdList[index]);
                    getAcLevelInfo(allAcLevelIdList[index]);

                    window.sessionStorage.setItem('getAllAccessLevelIdList', angular.toJson(allAcLevelIdList));
                });
            }

            function getAcLevelInfo(id) {
                var _acLevel;
                AccessControlAgent.getOneAcLevel(id).then(function success(data) {
                    _acLevel = data;
                    if (_acLevel) {
                        window.sessionStorage.setItem('acLevelInfo', angular.toJson(_acLevel));
                        $scope.acLevelInfo = angular.copy(_acLevel);
                        $scope.currAcLevelId = _acLevel.id;
                        $scope.currAcLevelName = _acLevel.name;
                    }
                });
            }

            var selectedDoorList = [];
            var selectedScheduleList = [];
            $scope.$watch('acLevelInfo.access_level_items', function (nv, ov) {
                /**
                 * 1. 설정 변경한 행을 찾는다.
                 * 2. 변경한 행을 제외한 데이터로 맵을 만든다.
                 * 3. 생성된 맵과 변경된 행을 비교한다.
                 * 4. 무엇이 중복되었는지 표시하고 값을 비운다.
                 */

                if (angular.isDefined(nv) && angular.isDefined(ov) && nv.length == ov.length) {
                    if(angular.equals(nv, ov) == false) {
                        var lastSelectedIdx = findLastSelectAccessLevelItems(nv, ov);
                        Util.info(lastSelectedIdx);
                        if (angular.isDefined(lastSelectedIdx)) {
                            var lastAcItem = angular.copy(ov[lastSelectedIdx]);
                            if (makeAccessLevelItemMapWithoutLastSelectedIdx(lastSelectedIdx)) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.existsConfiguration'));
                                $scope.acLevelInfo.access_level_items[lastSelectedIdx] = angular.copy(lastAcItem);
                            }
                        }
                    }
                }

            }, true);


            function findLastSelectAccessLevelItems(newerAccessLevelItems, olderAccessLevelItems) {
                var lastSelectedIdx;

                for (var i = 0; i < newerAccessLevelItems.length; i++) {
                    if (!angular.equals(newerAccessLevelItems[i], olderAccessLevelItems[i])) {
                        lastSelectedIdx = i;
                    }
                }

                return lastSelectedIdx;
            }

            function makeAccessLevelItemMapWithoutLastSelectedIdx(selectedIdx) {
                var selectedAcItem = $scope.acLevelInfo.access_level_items[selectedIdx];

                var makeAccessLevelItemMapWithoutLastSelectedIdx = {};

                var acItemIter, doorIter, acItem;

                for(acItemIter = 0; acItemIter < $scope.acLevelInfo.access_level_items.length; acItemIter++) {
                    acItem = $scope.acLevelInfo.access_level_items[acItemIter];
                    if (acItemIter != selectedIdx) {
                        makeAccessLevelItemMapWithoutLastSelectedIdx[acItemIter] = {};
                        makeAccessLevelItemMapWithoutLastSelectedIdx[acItemIter].doors = [];

                        if (angular.isDefined(acItem.doors)) {
                            for (doorIter = 0; doorIter < acItem.doors.length; doorIter++) {
                                makeAccessLevelItemMapWithoutLastSelectedIdx[acItemIter].doors.push(acItem.doors[doorIter].id);
                            }
                        }
                        if (angular.isDefined(acItem.schedule_id)) {
                            makeAccessLevelItemMapWithoutLastSelectedIdx[acItemIter].scheduleId = acItem.schedule_id.id
                        }
                    }
                }

                for(var key in makeAccessLevelItemMapWithoutLastSelectedIdx) {
                    var scheduleDuplicationYn = false, doorDuplicationYn = false;

                    if(makeAccessLevelItemMapWithoutLastSelectedIdx.hasOwnProperty(key)) {
                        acItem = makeAccessLevelItemMapWithoutLastSelectedIdx[key];
                        if(angular.isDefined(selectedAcItem)) {
                            if (angular.isDefined(selectedAcItem.schedule_id)) {
                                if (angular.equals(selectedAcItem.schedule_id.id, acItem.scheduleId)) {
                                    scheduleDuplicationYn = true;
                                }
                            }

                            if (angular.isDefined(selectedAcItem.doors)) {
                                var doorExistsCount = 0;
                                for (doorIter = 0; doorIter < selectedAcItem.doors.length; doorIter++) {
                                    if (acItem.doors.indexOf(selectedAcItem.doors[doorIter].id) > -1) {
                                        doorExistsCount++;
                                    }
                                }

                                if(doorExistsCount == acItem.doors.length && selectedAcItem.doors.length == acItem.doors.length && doorExistsCount > 0) {
                                    doorDuplicationYn = true;
                                }
                            }
                        }
                    }

                    if(scheduleDuplicationYn && doorDuplicationYn) {
                        return true;
                    }
                }

                return false;

            }

        }]);
    });