﻿'use strict';

define([
        'modules/common/services/access-control-agent',
        'modules/accessControl/acGroupInfoController',
        'modules/common/datagrid/columnSettingController',
        'modules/common/datagrid/columnSettingService',
        'modules/common/print/printOptionController',
        'modules/common/print/printOptionService'
    ],
    function () {

        var app = angular.module('biostarWebApp');

        app.register.controller('AccessGroupsController', ['$scope', '$rootScope', '$location', '$compile', '$q', '$window', 'API', 'Util', 'Const', 'MessageProperties', 'AccessControlAgent', 'ColumnSettingService', 'PrintOptionService', 'MessageService', 'PrivilegeService', function ($scope, $rootScope, $location, $compile, $q, $window, API, Util, Const, MessageProperties, AccessControlAgent, ColumnSettingService, PrintOptionService, MessageService, PrivilegeService) {

            if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.ACCESS_CONTROL)) {
                $scope.readOnly = true;
            }

            $scope.gridId = "acGroupGrid";
            var menuID = "#accessGroupContextMenu";

            $scope.clearCheckItemArray = function () {
                $scope.checkItemArray = [];
            };

            var acGroupInfo = undefined,
                acGroupListQueryStrings = {
                    query: undefined,
                    offset: 0,
                    limit: 15,
                    total: undefined
                },
                acGroupListDataGridOptions = {
                    scrollbarSize: 0,
                    view: $.extend({}, $.fn.datagrid.defaults.view, {
                        onBeforeRender: function (target, rows) {
                            $("#acGroupGrid").datagrid('resize');
                        }, onAfterRender: function (target) {
                            $.fn.datagrid.defaults.view.onAfterRender.call(this, target);
                            var grdPanel = angular.element(target).datagrid('getPanel');
                            var opts = angular.element(target).datagrid('options');
                            var vc = grdPanel.children('div.datagrid-view');
                            grdPanel.find(".datagrid-empty").remove();

                            if (angular.element(target).datagrid('getRows').length === 0) {
                                var html = '<div style="width:100%; height:360px; font-size:1.333em; line-height: 360px; ' +
                                    'vertical-align: middle; text-align: center;">' + MessageProperties.get('validate.notFound') +
                                    '</div>';
                                grdPanel.find('div.datagrid-view2 div.datagrid-body').html(html);
                            }

                            //헤더에 전체선택 버튼이 없다면 추가
                            if (grdPanel.find('.datagrid-header-check.compiled').length <= 0) {
                                grdPanel.find('.datagrid-header-check').attr({
                                    'select-all-yn': 'SelectAllYN',
                                    'datagrid-id': '"' + target.id + '"',
                                    'clear-function': 'clearCheckItemArray'
                                });
                                $compile(grdPanel.find('.datagrid-header-row'))($scope);
                                grdPanel.find('.datagrid-header-check').addClass('compiled');
                            }
                        }
                    }),
                    fitColumns: true,
                    resize: true,
                    singleSelect: true,
                    selectOnCheck: false,
                    checkOnSelect: false,
                    checkbox: !$scope.readOnly,
                    columns: [
                        [
                            {field: ($scope.readOnly ? 'none' : Const.FIELD_NAME.CHECKBOX), checkbox: !$scope.readOnly, hidden: $scope.readOnly},
                            {
                                field: 'name', title: MessageProperties.get('common.name'), width: 80, align: 'left', sortable: true
                            },
                            {
                                field: 'description', title: MessageProperties.get('common.description'), width: 80, align: 'left', sortable: true
                            },
                            {
                                field: 'access_levels', title: MessageProperties.get('common.accessLevel'), width: 80, align: 'left',
                                formatter: function (value, row, index) {
                                    var count = '';
                                    if (value && value.length) {
                                        if (value.length > 1) {
                                            count = ' + <mark class="number">' + (value.length - 1) + '<mark>';
                                        }
                                        return '<span title="' + value[0].name + '">' + value[0].name + count + '<span>';
                                    }
                                    return '-';

                                }},
                            {
                                field: 'user_groups', title: MessageProperties.get('accessGroup.userGroup'), width: 80, align: 'left',
                                formatter: function (value, row, index) {
                                    var count = '';
                                    if (value && value.length) {
                                        if (value.length > 1) {
                                            count = ' + <mark class="number">' + (value.length - 1) + '<mark>';
                                        }
                                        return '<span title="' + value[0].name + '">' + value[0].name + count + '<span>';
                                    }
                                    return '-';
                                }
                            },
                            {
                                field: 'users', title: MessageProperties.get('common.user'), width: 80, align: 'left',
                                formatter: function (value, row, index) {
                                    var count = '';
                                    if (value && value.length) {
                                        if (value.length > 1) {
                                            count = ' + <mark class="number">' + (value.length - 1) + '<mark>';
                                        }
                                        return '<span title="' + ( value[0].name || value[0].user_id ) + '">' + ( value[0].name || value[0].user_id ) + count + '<span>';
                                    }
                                    return '-';
                                }
                            }
                        ]
                    ],
                    onCheck: function (rowIndex, rowData) {
                        var existsYn = $scope.checkItemArray.indexOf(rowData.id);
                        if (existsYn == -1) {
                            if ($scope.checkItemArray.length >= Const.SELECT_LIST_ITEM_LIMIT) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.canNotSelectMoreItem', Const.SELECT_LIST_ITEM_LIMIT));
                                angular.element('#acGroupGrid').datagrid('uncheckRow', rowIndex);
                            } else {
                                $scope.checkItemArray.push(rowData.id);
                                if (!$scope.$root.$$phase) $scope.$apply();
                            }
                        }
                    },
                    onUncheck: function (rowIndex, rowData) {
                        if ($scope.SelectAllYN == true) {
                            $scope.checkItemArray = [];
                            var data = angular.element('#acGroupGrid').datagrid('getChecked');
                            $scope.SelectAllYN = false;
                            if (!$scope.$root.$$phase) $scope.$apply();
                            for (var i = 0; i < data.length; i++) {
                                var index = angular.element('#acGroupGrid').datagrid('getRowIndex', data[i]);
                                angular.element('#acGroupGrid').datagrid('checkRow', index);
                            }
                        } else {
                            var idx = $scope.checkItemArray.indexOf(rowData.id);
                            if (idx > -1) {
                                $scope.checkItemArray.splice(idx, 1);

                                if (!$scope.$root.$$phase) $scope.$apply();
                            }
                        }
                    },
                    onCheckAll: function (rows) {
                        var i, uncheckList = [];
                        for (i = 0; i < rows.length; i++) {
                            var item = rows[i];
                            var idx = $scope.checkItemArray.indexOf(item.id);

                            if (idx == -1) {
                                if ($scope.checkItemArray.length < Const.SELECT_LIST_ITEM_LIMIT) {
                                    $scope.checkItemArray.push(item.id);
                                } else {
                                    uncheckList.push(item);
                                }
                            }
                        }

                        if (uncheckList.length > 0) {
                            MessageService.displayWarningMessage(MessageProperties.get('validate.canNotSelectMoreItem', Const.SELECT_LIST_ITEM_LIMIT));
                            for (i = uncheckList.length; i--;) {
                                //console.log(uncheckList[i].user_id);
                                angular.element('#acGroupGrid').datagrid('uncheckRow', angular.element('#acGroupGrid').datagrid('getRowIndex', uncheckList[i]));
                            }
                        }

                        if (!$scope.$root.$$phase) $scope.$apply();
                    },
                    onUncheckAll: function (rows) {
                        if ($scope.SelectAllYN == true) {
                            $scope.SelectAllYN = false;
                            $scope.checkItemArray = [];
                        }
                        for (var i = rows.length; i--;) {
                            var item = rows[i];
                            var idx = $scope.checkItemArray.indexOf(item.id);
                            if (idx > -1) {
                                $scope.checkItemArray.splice(idx, 1);
                            }
                        }
                        if (!$scope.$root.$$phase) $scope.$apply();
                    },
                    onClickCell: function (rowIndex, field) {
                        $rootScope.clickCellOnGrid($scope.gridId, rowIndex, field, "id", function (rowData) {
                            AccessControlAgent.getOneAcGroup(rowData.id).then(function success(data) {
                                window.sessionStorage.setItem('acGroupCurrPos', parseInt($scope.acGroupListQueryStrings.offset) + rowIndex + 1);
                                setAcGroupInfoUrl()
                            });
                        });
                    },
                    onSortColumn: function (sort, order) {
                        var i,
                            orderItemArray,
                            descending = order == 'desc',
                            orderItemObject = {};

                        $scope.acGroupListQueryStrings.order_by = sort + ":" + descending;

                        beSearchAcGroupList();
                    }
                };

            function setAcGroupInfoUrl() {
                $scope.selectedAcGroupInfo = 'modules/accessControl/acGroupInfo.html';
            }

            $scope.doClickAcGroupListRow = function (item, $index) {
                AccessControlAgent.getOneAcGroup(item.id).then(function success(data) {
                    window.sessionStorage.setItem('acGroupCurrPos', parseInt($scope.acGroupListQueryStrings.offset) + $index + 1);
                    setAcGroupInfoUrl()
                });
            };

            $scope.deleteTheSelectedAcGroup = function () {
                MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                    if (confirmYn) {
                        AccessControlAgent.deleteAcGroups($scope.checkItemArray).then(function success(data) {
                            $scope.$emit('ReloadAcGroupsDataGrid');
                        }, function error(reason, status) {
                            console.error(reason, status);
                        });
                    }
                });
            };

            $scope.checkItemArray = [];

            $scope.acGroupListQueryStrings = acGroupListQueryStrings;
            $scope.AcGroupListData = [];

            angular.element('#acGroupGrid').datagrid(acGroupListDataGridOptions);

            //헤더에 전체선택 버튼 추가
            $scope.SelectAllYN = false;

            $scope.$watch('SelectAllYN', function (nv) {
                if (nv) {
                    angular.element('#acGroupGrid').datagrid('checkAll');
                } else {
                    angular.element('#acGroupGrid').datagrid('uncheckAll');
                    $scope.checkItemArray = [];
                }
            });

            $scope.cancelSelectAllAcGroup = function () {
                angular.element('#acGroupGrid').datagrid('uncheckAll');
            };

            function reloadAcGroupLNBInfo() {
                API.get('api/access_groups', 'reloadAcGroupLNBInfo').then(function (data) {
                    $scope.AcGroupListAllData = data.AccessGroupCollection.rows || [];
                });
            }

            reloadAcGroupLNBInfo();

            function beSearchAcGroupList(query, deferFunctionName) {
                var i, rowData, allAccessGroupIdList, startIdx, limit;

                if (query) {
                    acGroupListQueryStrings.query = query;
                }

                AccessControlAgent.getAccessGroups($scope.acGroupListQueryStrings, deferFunctionName).then(function success(data) {
                    var index = 0;
                    var index2 = 0;
                    if (data.AccessGroupCollection.rows != undefined) {
                        for (index = 0; index < data.AccessGroupCollection.rows.length; index++) {
                            if (data.AccessGroupCollection != undefined && data.AccessGroupCollection.rows[index] != undefined &&
                                data.AccessGroupCollection.rows[index].user_groups != undefined && data.AccessGroupCollection.rows[index].user_groups.length > 0) {
                                for (index2 = 0; index2 < data.AccessGroupCollection.rows[index].user_groups.length; index2++) {
                                    if (data.AccessGroupCollection.rows[index].user_groups[index2].id == Const.DEFAULT_ALL_GROUP_ID) {
                                        data.AccessGroupCollection.rows[index].user_groups[index2].name = MessageProperties.get("menu.user.allUserGroups");
                                    }
                                }
                            }
                        }
                    }
                    $scope.AcGroupListData = data.AccessGroupCollection.rows || [];
                    window.sessionStorage.setItem('AcGroupListData', angular.toJson($scope.AcGroupListData));

                    angular.element('#acGroupGrid').datagrid('loadData', $scope.AcGroupListData);

                    allAccessGroupIdList = new Array(parseInt(data.AccessGroupCollection.total));

                    startIdx = parseInt($scope.acGroupListQueryStrings.offset);
                    limit = parseInt($scope.acGroupListQueryStrings.limit);

                    if (parseInt($scope.acGroupListQueryStrings.offset) + parseInt($scope.acGroupListQueryStrings.limit) > data.AccessGroupCollection.total) {
                        limit = parseInt(data.AccessGroupCollection.total) - $scope.acGroupListQueryStrings.offset;
                    }

                    for (i = 0; i < limit; i++) {
                        allAccessGroupIdList[startIdx + i] = data.AccessGroupCollection.rows[i].id;
                    }

                    window.sessionStorage.setItem('getAllAccessGroupIdList', angular.toJson(allAccessGroupIdList));

                    if ($scope.SelectAllYN) {
                        for (i = $scope.AcGroupListData.length; i--;) {
                            rowData = $scope.AcGroupListData[i];
                            if ($scope.checkItemArray.indexOf(rowData.id) < 0) {
                                $scope.checkItemArray.push(rowData.id);
                            }
                        }
                    }

                    acGroupListQueryStrings.total = data.AccessGroupCollection.total || 0;

                    if (!$scope.SelectAllYN) {
                        angular.element('.datagrid-header-check').find('input').attr('checked', false);
                    } else {

                    }

                    for (i = $scope.AcGroupListData.length; i--;) {
                        rowData = $scope.AcGroupListData[i];
                        if ($scope.checkItemArray.indexOf(rowData.id) > -1) {
                            angular.element('#acGroupGrid').datagrid('checkRow', i);
                        }
                    }

                });
            }

            beSearchAcGroupList();


            $scope.SearchCallback = {
                fn: beSearchAcGroupList
            };

            $scope.$on('ReloadAcGroupsDataGrid', function (event, data) {
                angular.element('#acGroupGrid').datagrid('uncheckAll');
                acGroupListQueryStrings.offset = 0;
                if (data && data.offset) {
                    acGroupListQueryStrings.offset = data.offset;
                }
                beSearchAcGroupList();
                reloadAcGroupLNBInfo();
            });

            // Delete Access Levels
            $scope.deleteSelectedAccessGroupInfo = function (itemIdx) {
                angular.element(menuID).menu("hide");

                var param = $scope.checkItemArray;
                if ($scope.SelectAllYN) {
                    param = 'ALL';
                }

                if (itemIdx != undefined) {
                    param = [];
                    param.push(itemIdx);
                }

                MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                    if (confirmYn) {
                        AccessControlAgent.deleteAcGroups(param).then(function success(data) {
                            $scope.$emit('ReloadAcGroupsDataGrid');
                        }, function error(reason, status) {
                            console.error(reason, status);
                        });
                    }
                });


            };

            $scope.$watch(function () {
                return  angular.element('.panel.datagrid').width();
            }, function (nv, ov) {
                if (nv !== ov) {
                    angular.element('#acGroupGrid').datagrid('resize');
                }
            }, true);

            angular.element($window).on('resize', function ($event) {
                var event = $event | window.event;
                angular.element('#acGroupGrid').datagrid('resize');
                Util.debug('div#acGroupGrid resize');
            });

            $scope.dropDownMenu = [
//                    {
//                        menu: MessageProperties.get('menu.print'),
//                        func: function () {
//                            PrintOptionService.openPrintOptionPopupDlg();
//                        }
//                    },
                {
                    menu: MessageProperties.get('menu.columnSetting'),
                    func: function () {
                        ColumnSettingService.openColumnSettingPopupDlg();
                    }
                }
            ];

            $scope.doAddAcGroupInfo = function () {
                $scope.$emit('OpenAcGroupInfo');
            };

            function closeAcGroupInfo(event, data) {
                window.sessionStorage.removeItem('acGroupInfo');
                window.sessionStorage.removeItem('AcGroupListData');
                window.sessionStorage.removeItem('acGroupCurrPos');
                $scope.$emit('ReloadAcGroupsDataGrid');
                $scope.selectedAcGroupInfo = undefined;
            }

            var clickMenuInitUnbind = $rootScope.$on('ClickMenuInit', closeAcGroupInfo);
            $scope.$on('$locationChangeStart', clickMenuInitUnbind);

            $scope.$on('CloseAcGroupInfo', closeAcGroupInfo);

            $scope.$on('OpenAcGroupInfo', function (event, data) {
                window.sessionStorage.setItem('acGroupInfo', '{}');
                setAcGroupInfoUrl();
            });

            acGroupInfo = window.sessionStorage.getItem('acGroupInfo');

            if (acGroupInfo && acGroupInfo != 'undefined') {
                acGroupInfo = angular.fromJson(acGroupInfo);
                setAcGroupInfoUrl()
            }

            $scope.doAddAcLevelInfo = function () {
                window.sessionStorage.setItem('acLevelInfo', '{}');
                $location.path('/access-control/ac-levels')
            };


            $scope.$watch('acGroupListQueryStrings', function (nv, ov) {
                if (nv !== ov) {
                    // TODO BSS-195 BSS-192 server api not implemented
//                        PrintOptionService.buildGridQueryFunction(Const.REPORT_COLLECTION_NAME.ACCESS_GROUP, nv);
                }
            }, true);


            $scope.selectedLnbItem = undefined;

            function initContextMenu() {
                angular.element(menuID).menu({});
            }

            function clearMenuContext(divId) {
                angular.element(divId).find(".menu-item").remove();
            }

            function buildContextMenuDiv(contextMenuObject, ngIfCondition) {
                var html = '<div';
                if (ngIfCondition != undefined) {
                    html += ' ng-if="' + ngIfCondition + '" ';
                }
                html += ' class="menu-item" style="height:31px;" ng-click="' + contextMenuObject.clickFunctionName + '()" ng-privilege><div class="menu-text">' + contextMenuObject.menuName + '</div></div>';
                return html;
            }

            var ContextMenu = {
                'DELETE_ACCESS_GROUP' : {
                    menuName: MessageProperties.get('button.deleteAccessGroup'), clickFunctionName: 'deleteSelectedAccessGroupInfo(selectedLnbItem)'
                }
            };

            initContextMenu();

            angular.element('.leftTreeMenu').on('contextmenu', function ($event) {
                var event = $event || window.event,
                    target = event.target;

                var top;

                var idx = angular.element(target).closest('li').attr('data-list-index');

                if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.ACCESS_CONTROL)) {
                    return false;
                }

                $scope.selectedLnbItem = $scope.AcGroupListAllData[idx].id;

                if (event.preventDefault()) event.preventDefault();
                if (event.returnValue) event.returnValue = false;

                event.cancelBubble = true;

                angular.element(menuID).menu("hide");
                clearMenuContext(menuID);

                angular.element(menuID).append(buildContextMenuDiv(ContextMenu.DELETE_ACCESS_GROUP));

                angular.element(menuID).css("height", "auto");
                angular.element(menuID).css("width", "auto");

                $compile(angular.element(menuID))($scope);

                angular.element(menuID).menu('show', {
                    left: event.pageX,
                    top: event.pageY
                });
            });

            PrintOptionService.getReportColumnNameFunction = function (gridColumnName) {
                var reportColumnName = gridColumnName;
                if (gridColumnName == "acGroup_group_id") {
                    return "acGroup_groups.name"
                } else if (gridColumnName == "access_groups") {
                    return "access_groups.name"
                } else if (gridColumnName == "fingerprint_templates") {
                    return "finger-tempates.count"
                } else if (gridColumnName == "cards") {
                    return "cards.count"
                }
                return reportColumnName;
            };
            $scope.reportUrl = Const.API.BASE_URL + 'acGroups/report';
            $scope.openPrintOptionPopupDlg = PrintOptionService.openPrintOptionPopupDlg;
            $scope.openColumnSettingPopupDlg = ColumnSettingService.openColumnSettingPopupDlg;

            ColumnSettingService.setGridId($scope.gridId);
            ColumnSettingService.setCallbackFunction($scope.SearchCallback.fn);
            var customColumnSetting = ColumnSettingService.getColumnSettingFromLocalStroage();
            if (customColumnSetting) {
                ColumnSettingService.applyChangedColumnsToGrid(customColumnSetting, false);
            }
        }]);
    });