'use strict';

define([
        'modules/common/services/access-control-agent',
        'modules/accessControl/acLevelInfoController',
        'modules/common/datagrid/columnSettingController',
        'modules/common/datagrid/columnSettingService',
        'modules/common/print/printOptionController',
        'modules/common/print/printOptionService'
    ],
    function () {

        var app = angular.module('biostarWebApp');

        app.register.controller('AccessLevelsController', ['$scope', '$rootScope', '$location', '$compile', '$window', 'API', 'Util', 'Const', 'MessageProperties', 'AccessControlAgent', 'ColumnSettingService', 'PrintOptionService', 'MessageService', 'PrivilegeService', function ($scope, $rootScope, $location, $compile, $window, API, Util, Const, MessageProperties, AccessControlAgent, ColumnSettingService, PrintOptionService, MessageService, PrivilegeService) {

            if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.ACCESS_CONTROL)) {
                $scope.readOnly = true;
            }

            $scope.gridId = "acLevelGrid";
            var menuID = "#accessLevelContextMenu";

            $scope.clearCheckItemArray = function () {
                $scope.checkItemArray = [];
            };

            var acLevelInfo = undefined,
                acLevelListQueryStrings = {
                    query: undefined,
                    offset: 0,
                    limit: 15,
                    total: undefined
                },
                acLevelListDataGridOptions = {
                    scrollbarSize: 0,
                    view: $.extend({}, $.fn.datagrid.defaults.view, {
                        onBeforeRender: function (target, rows) {
                            $("#acLevelGrid").datagrid('resize');
                        }, onAfterRender: function (target) {
                            $.fn.datagrid.defaults.view.onAfterRender.call(this, target);
                            var grdPanel = angular.element(target).datagrid('getPanel');
                            var opts = angular.element(target).datagrid('options');
                            var vc = grdPanel.children('div.datagrid-view');
                            grdPanel.find(".datagrid-empty").remove();

                            if (angular.element(target).datagrid('getRows').length === 0) {
                                var html = '<div style="width:100%; height:360px; font-size:1.333em; line-height: 360px; ' +
                                    'vertical-align: middle; text-align: center;">' + MessageProperties.get('validate.notFound') +
                                    '</div>';
                                grdPanel.find('div.datagrid-view2 div.datagrid-body').html(html);
                            }

                            //헤더에 전체선택 버튼이 없다면 추가
                            if (grdPanel.find('.datagrid-header-check.compiled').length <= 0) {
                                grdPanel.find('.datagrid-header-check').attr({
                                    'select-all-yn': 'SelectAllYN',
                                    'datagrid-id': '"' + target.id + '"',
                                    'clear-function': 'clearCheckItemArray'
                                });
                                $compile(grdPanel.find('.datagrid-header-row'))($scope);
                                grdPanel.find('.datagrid-header-check').addClass('compiled');
                            }
                        }
                    }),
                    fitColumns: true,
                    resize: true,
                    singleSelect: true,
                    selectOnCheck: false,
                    checkOnSelect: false,
                    checkbox: !$scope.readOnly,
                    columns: [
                        [
                            {field: ($scope.readOnly ? 'none' : Const.FIELD_NAME.CHECKBOX), checkbox: !$scope.readOnly, hidden: $scope.readOnly},
                            {
                                field: 'name', title: MessageProperties.get("common.name"), width: 80, align: 'left', sortable: true
                            },
                            {
                                field: 'description', title: MessageProperties.get("common.description"), width: 140, align: 'left', sortable: true
                            },
                            {
                                field: 'doors', title: MessageProperties.get("common.door"), width: 80, align: 'left',
                                formatter: function (value, row, index) {
                                    var i,
                                        count = 0,
                                        doorTitle = '',
                                        mark = '',
                                        html = '';
                                    if (row.access_level_items && row.access_level_items.length) {
                                        for (i = row.access_level_items.length; i--;) {
                                            if (row.access_level_items[i].doors && row.access_level_items[i].doors.length) {
                                                doorTitle = row.access_level_items[i].doors[0].name;
                                                count += row.access_level_items[i].doors.length;
                                            }
                                        }

                                        if (count > 1) {
                                            mark = ' + <mark class="number">' + parseInt(count - 1) + '</mark>';
                                        }

                                        html = '<span title="' + doorTitle + '">' + doorTitle + mark + '</span>';
                                    } else {
                                        html = '-'
                                    }
                                    return html;
                                }
                            },
                            {
                                field: 'schedules', title: MessageProperties.get("common.schedule"), width: 140, align: 'left',
                                formatter: function (value, row, index) {
                                    var i,
                                        scheduleTitle = '',
                                        mark = '',
                                        html = '';

                                    if (row.access_level_items && row.access_level_items.length) {
                                        for (i = row.access_level_items.length; i--;) {
                                            if (row.access_level_items[i].schedule_id) {
                                                scheduleTitle = row.access_level_items[i].schedule_id.name;
                                            }
                                        }
                                        html = scheduleTitle;

                                        if (row.access_level_items.length > 1) {
                                            mark = ' + <mark class="number">' + parseInt(row.access_level_items.length - 1) + '</mark>';
                                        }
                                        html += mark;
                                    } else {
                                        html = '-';
                                    }

                                    return html;
                                }
                            }
                        ]
                    ],
                    onCheck: function (rowIndex, rowData) {
                        var existsYn = $scope.checkItemArray.indexOf(rowData.id);
                        if (existsYn == -1) {
                            if ($scope.checkItemArray.length >= Const.SELECT_LIST_ITEM_LIMIT) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.canNotSelectMoreItem', Const.SELECT_LIST_ITEM_LIMIT));
                                angular.element('#acLevelGrid').datagrid('uncheckRow', rowIndex);
                            } else {
                                $scope.checkItemArray.push(rowData.id);
                                if (!$scope.$root.$$phase) $scope.$apply();
                            }
                        }
                    },
                    onUncheck: function (rowIndex, rowData) {
                        if ($scope.SelectAllYN == true) {
                            $scope.checkItemArray = [];
                            var data = angular.element('#acLevelGrid').datagrid('getChecked');
                            $scope.SelectAllYN = false;
                            if (!$scope.$root.$$phase) $scope.$apply();
                            for (var i = 0; i < data.length; i++) {
                                var index = angular.element('#acLevelGrid').datagrid('getRowIndex', data[i]);
                                angular.element('#acLevelGrid').datagrid('checkRow', index);
                            }
                        } else {
                            var idx = $scope.checkItemArray.indexOf(rowData.id);
                            if (idx > -1) {
                                $scope.checkItemArray.splice(idx, 1);

                                if (!$scope.$root.$$phase) $scope.$apply();
                            }
                        }
                    },
                    onCheckAll: function (rows) {
                        var i, uncheckList = [];
                        for (i = 0; i < rows.length; i++) {
                            var item = rows[i];
                            var idx = $scope.checkItemArray.indexOf(item.id);

                            if (idx == -1) {
                                if ($scope.checkItemArray.length < Const.SELECT_LIST_ITEM_LIMIT) {
                                    $scope.checkItemArray.push(item.id);
                                } else {
                                    uncheckList.push(item);
                                }
                            }
                        }

                        if (uncheckList.length > 0) {
                            MessageService.displayWarningMessage(MessageProperties.get('validate.canNotSelectMoreItem', Const.SELECT_LIST_ITEM_LIMIT));
                            for (i = uncheckList.length; i--;) {
                                console.log(uncheckList[i].user_id);
                                angular.element('#acLevelGrid').datagrid('uncheckRow', angular.element('#acLevelGrid').datagrid('getRowIndex', uncheckList[i]));
                            }
                        }

                        if (!$scope.$root.$$phase) $scope.$apply();
                    },
                    onUncheckAll: function (rows) {
                        if ($scope.SelectAllYN == true) {
                            $scope.SelectAllYN = false;
                            $scope.checkItemArray = [];
                        }
                        for (var i = rows.length; i--;) {
                            var item = rows[i];
                            var idx = $scope.checkItemArray.indexOf(item.id);
                            if (idx > -1) {
                                $scope.checkItemArray.splice(idx, 1);
                            }
                        }
                        if (!$scope.$root.$$phase) $scope.$apply();
                    },
                    onClickCell: function (rowIndex, field) {
                        $rootScope.clickCellOnGrid($scope.gridId, rowIndex, field, "id", function (rowData) {
                            AccessControlAgent.getOneAcLevel(rowData.id).then(function success(data) {
                                window.sessionStorage.setItem('acLevelCurrPos', parseInt($scope.acLevelListQueryStrings.offset) + rowIndex + 1);
                                setAcLevelInfoUrl()
                            });
                        });
                    },
                    onSortColumn: function (sort, order) {
                        var i,
                            orderItemArray,
                            descending = order == 'desc',
                            orderItemObject = {};

                        $scope.acLevelListQueryStrings.order_by = sort + ":" + descending;

                        beSearchAcLevelList();
                    }
                };

            function setAcLevelInfoUrl() {
                $scope.selectedAcLevelInfo = 'modules/accessControl/acLevelInfo.html';
            }

            $scope.doClickAcLevelListRow = function (item, $index) {
                AccessControlAgent.getOneAcLevel(item.id).then(function success(data) {
                    window.sessionStorage.setItem('acLevelCurrPos', parseInt($scope.acLevelListQueryStrings.offset) + $index + 1);
                    setAcLevelInfoUrl()
                });
            };

            $scope.checkItemArray = [];

            $scope.acLevelListQueryStrings = acLevelListQueryStrings;
            $scope.AcLevelListData = [];

            angular.element('#acLevelGrid').datagrid(acLevelListDataGridOptions);

            //헤더에 전체선택 버튼 추가
            $scope.SelectAllYN = false;

            $scope.$watch('SelectAllYN', function (nv) {
                if (nv) {
                    angular.element('#acLevelGrid').datagrid('checkAll');
                } else {
                    angular.element('#acLevelGrid').datagrid('uncheckAll');
                    $scope.checkItemArray = [];
                }
            });

            $scope.cancelSelectAllAcLevel = function () {
                angular.element('#acLevelGrid').datagrid('uncheckAll');
            };

            function reloadAcLevelLNBInfo() {
                API.get('api/access_levels').then(function (data) {
                    $scope.AcLevelListAllData = data.AccessLevelCollection.rows || [];
                });
            }

            reloadAcLevelLNBInfo();

            function beSearchAcLevelList(query, deferFunctionName) {

                var i, rowData, allAccessLevelIdList, startIdx, limit;

                if (query) {
                    acLevelListQueryStrings.query = query;
                }

                $rootScope.$broadcast('CurrentSelectedAcLevel`sGroup', $scope.acLevel_group);

                AccessControlAgent.getAccessLevels($scope.acLevelListQueryStrings, deferFunctionName).then(function success(data) {
                    $scope.AcLevelListData = data.AccessLevelCollection.rows || [];

                    window.sessionStorage.setItem('AcLevelListData', angular.toJson($scope.AcLevelListData));

                    angular.element('#acLevelGrid').datagrid('loadData', $scope.AcLevelListData);

                    allAccessLevelIdList = new Array(parseInt(data.AccessLevelCollection.total));

                    startIdx = parseInt($scope.acLevelListQueryStrings.offset);
                    limit = parseInt($scope.acLevelListQueryStrings.limit);

                    if (parseInt($scope.acLevelListQueryStrings.offset) + parseInt($scope.acLevelListQueryStrings.limit) > data.AccessLevelCollection.total) {
                        limit = parseInt(data.AccessLevelCollection.total) - $scope.acLevelListQueryStrings.offset;
                    }

                    for (i = 0; i < limit; i++) {
                        allAccessLevelIdList[startIdx + i] = data.AccessLevelCollection.rows[i].id;
                    }

                    window.sessionStorage.setItem('getAllAccessLevelIdList', angular.toJson(allAccessLevelIdList));

                    if ($scope.SelectAllYN) {
                        for (i = $scope.AcLevelListData.length; i--;) {
                            rowData = $scope.AcLevelListData[i];
                            if ($scope.checkItemArray.indexOf(rowData.id) < 0) {
                                $scope.checkItemArray.push(rowData.id);
                            }
                        }
                    }

                    acLevelListQueryStrings.total = data.AccessLevelCollection.total || 0;

                    if (!$scope.SelectAllYN) {
                        angular.element('.datagrid-header-check').find('input').attr('checked', false);
                    } else {

                    }

                    for (i = $scope.AcLevelListData.length; i--;) {
                        rowData = $scope.AcLevelListData[i];
                        if ($scope.checkItemArray.indexOf(rowData.id) > -1) {
                            angular.element('#acLevelGrid').datagrid('checkRow', i);
                        }
                    }

                });
            }

            beSearchAcLevelList();


            $scope.SearchCallback = {
                fn: beSearchAcLevelList
            };

            $scope.$on('ReloadAcLevelsDataGrid', function (event, data) {

                angular.element('#acLevelGrid').datagrid('uncheckAll');

                acLevelListQueryStrings.offset = 0;

                if (data && data.offset) {
                    acLevelListQueryStrings.offset = data.offset;
                }
                beSearchAcLevelList();
                reloadAcLevelLNBInfo();
            });

            // Delete Access Levels
            $scope.deleteSelectedAccessLevelInfo = function (itemIdx) {

                angular.element(menuID).menu('hide');

                var param = $scope.checkItemArray;
                if ($scope.SelectAllYN) {
                    param = 'ALL';
                }

                if (itemIdx != undefined) {
                    param = [];
                    param.push(itemIdx);
                }

                MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                    if (confirmYn) {
                        AccessControlAgent.deleteAcLevels(param).then(function success(data) {
                            $scope.$emit('ReloadAcLevelsDataGrid');
                        }, function error(reason, status) {
                            console.error(reason, status);
                        });
                    }
                });
            };

            $scope.$watch(function () {
                return  angular.element('.panel.datagrid').width();
            }, function (nv, ov) {
                if (nv !== ov) {
                    angular.element('#acLevelGrid').datagrid('resize');
                }
            }, true);

            angular.element($window).on('resize', function ($event) {
                var event = $event | window.event;
                angular.element('#acLevelGrid').datagrid('resize');
                Util.debug('div#acLevelGrid resize');
            });

            $scope.dropDownMenu = [
//                    {
//                        menu: MessageProperties.get('menu.print'),
//                        func: function () {
//                            PrintOptionService.openPrintOptionPopupDlg();
//                        }
//                    },
                {
                    menu: MessageProperties.get('menu.columnSetting'),
                    func: function () {
                        ColumnSettingService.openColumnSettingPopupDlg();
                    }
                }
            ];

            $scope.doAddAcLevelInfo = function () {
                $scope.$emit('OpenAcLevelInfo');
            };


            function closeAcLevelInfo(event, data) {
                window.sessionStorage.removeItem('acLevelInfo');
                window.sessionStorage.removeItem('AcLevelListData');
                window.sessionStorage.removeItem('acLevelCurrPos');
                $scope.selectedAcLevelInfo = undefined;
            }

            var clickMenuInitUnbind = $rootScope.$on('ClickMenuInit', closeAcLevelInfo);
            $scope.$on('$locationChangeStart', clickMenuInitUnbind);

            $scope.$on('CloseAcLevelInfo', closeAcLevelInfo);

            $scope.$on('OpenAcLevelInfo', function (event, data) {
                window.sessionStorage.setItem('acLevelInfo', '{}');
                setAcLevelInfoUrl();
            });

            acLevelInfo = window.sessionStorage.getItem('acLevelInfo');

            if (acLevelInfo && acLevelInfo != 'undefined') {
                acLevelInfo = angular.fromJson(acLevelInfo);
                setAcLevelInfoUrl()
            }

            $scope.doAddAcGroupInfo = function () {
                window.sessionStorage.setItem('acGroupInfo', '{}');
                $location.path('/access-control/ac-groups')
            };

            $scope.$watch('acLevelListQueryStrings', function (nv, ov) {
                if (nv !== ov) {
                    // TODO BSS-195 BSS-192 server api not implemented
//                        PrintOptionService.buildGridQueryFunction(Const.REPORT_COLLECTION_NAME.ACCESS_LEVEL, nv);
                }
            }, true);


            $scope.selectedLnbItem = undefined;

            function initContextMenu() {
                angular.element(menuID).menu({});
            }

            function clearMenuContext(divId) {
                angular.element(divId).find(".menu-item").remove();
            }

            function buildContextMenuDiv(contextMenuObject, ngIfCondition) {
                var html = '<div';
                if (ngIfCondition != undefined) {
                    html += ' ng-if="' + ngIfCondition + '" ';
                }
                html += ' class="menu-item" style="height:31px;" ng-click="' + contextMenuObject.clickFunctionName + '()" ng-privilege><div class="menu-text">' + contextMenuObject.menuName + '</div></div>';
                return html;
            }

            var ContextMenu = {
                'DELETE_ACCESS_LEVEL' : {
                    menuName: MessageProperties.get('button.deleteAccessLevel'), clickFunctionName: 'deleteSelectedAccessLevelInfo(selectedLnbItem)'
                }
            };

            initContextMenu();

            angular.element('.leftTreeMenu').on('contextmenu', function ($event) {
                var event = $event || window.event,
                    target = event.target;

                var idx = angular.element(target).closest('li').attr('data-list-index');

                if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.ACCESS_CONTROL)) {
                    return false;
                }

                $scope.selectedLnbItem = $scope.AcLevelListAllData[idx].id;

                if (event.preventDefault()) event.preventDefault();
                if (event.returnValue) event.returnValue = false;

                event.cancelBubble = true;

                angular.element(menuID).menu("hide");
                clearMenuContext(menuID);

                angular.element(menuID).append(buildContextMenuDiv(ContextMenu.DELETE_ACCESS_LEVEL));

                angular.element(menuID).css("height", "auto");
                angular.element(menuID).css("width", "auto");

                $compile(angular.element(menuID))($scope);

                angular.element(menuID).menu('show', {
                    left: event.pageX,
                    top: event.pageY
                });
            });

            PrintOptionService.getReportColumnNameFunction = function (gridColumnName) {
                var reportColumnName = gridColumnName;
                if (gridColumnName == "acLevel_group_id") {
                    return "acLevel_groups.name"
                } else if (gridColumnName == "access_groups") {
                    return "access_groups.name"
                } else if (gridColumnName == "fingerprint_templates") {
                    return "finger-tempates.count"
                } else if (gridColumnName == "cards") {
                    return "cards.count"
                }
                return reportColumnName;
            };
            $scope.reportUrl = Const.API.BASE_URL + 'acLevels/report';
            $scope.openPrintOptionPopupDlg = PrintOptionService.openPrintOptionPopupDlg;
            $scope.openColumnSettingPopupDlg = ColumnSettingService.openColumnSettingPopupDlg;

            ColumnSettingService.setGridId($scope.gridId);
            ColumnSettingService.setCallbackFunction($scope.SearchCallback.fn);
            var customColumnSetting = ColumnSettingService.getColumnSettingFromLocalStroage();
            if (customColumnSetting) {
                ColumnSettingService.applyChangedColumnsToGrid(customColumnSetting, false);
            }

        }]);

    });