'use strict';

define(function () {
    var app = angular.module('biostarWebApp');

    app.register.controller('AccessAddScheduleCtrl',
        ['$scope', '$rootScope', '$window', '$parse', 'API', 'Const', 'Util', 'MessageProperties', 'MessageService',
            function ($scope, $rootScope, $window, $parse, API, Const, Util, MessageProperties, MessageService) {

                var page_mode = 'create',
                    keepHolidaySchedulesValue = undefined;


                $scope.FULL_DAYS_TEXT = Const.FULL_DAYS_TEXT;
                $scope.SIMPLE_DAYS_TEXT = Const.SIMPLE_DAYS_TEXT;

                var Schedules = (function (_schedules) {
                    var i,
                        id,
                        name,
                        description,
                        schedules = angular.copy(_schedules),
                        daily_schedules = [],
                        holiday_schedules = [],
                        days_of_iteration = 7,
                        start_date = new Date().buildCurrentTimeWithNoTimeZoneInUTCString(),
                        use_daily_iteration = false;


                    if (schedules) {
                        id = schedules.id;
                        name = schedules.name;
                        description = schedules.description;
                        daily_schedules = schedules.daily_schedules;
                        holiday_schedules = schedules.holiday_schedules;
                        days_of_iteration = parseInt(schedules.days_of_iteration);
                        start_date = schedules.start_date || start_date;
                        use_daily_iteration =
                                schedules.use_daily_iteration.constructor == String ? JSON.parse(schedules.use_daily_iteration) : schedules.use_daily_iteration;

                        if (use_daily_iteration == false && daily_schedules && daily_schedules.length < days_of_iteration) {
                            var idx,
                                item,
                                _daily_schedules = [];
                            for (i = days_of_iteration; i--;) {
                                _daily_schedules.push({
                                    day_index: i,
                                    time_segments: []
                                });
                            }
                            for (i = 0; i < daily_schedules.length; i++) {
                                idx = daily_schedules[i].day_index;
                                if (!daily_schedules[i].time_segments) {
                                    daily_schedules[i].time_segments = [];
                                }
                                _daily_schedules[idx] = angular.copy(daily_schedules[i]);
                            }
                            daily_schedules = angular.copy(_daily_schedules);
                        } else if (use_daily_iteration == true) {
                            if (!daily_schedules) {
                                daily_schedules = [];
                            }

                            if (daily_schedules && daily_schedules.length > days_of_iteration) {
                                daily_schedules.splice(days_of_iteration, daily_schedules.length);

                            } else if (daily_schedules && daily_schedules.length < days_of_iteration) {

                                for (i = daily_schedules.length; i < days_of_iteration; i++) {
                                    daily_schedules.push({
                                        day_index: i,
                                        time_segments: []
                                    });
                                }
                            }
                        }
                    }

                    if (daily_schedules != undefined && !daily_schedules.length) {
                        for (i = 0; i < days_of_iteration; i++) {
                            daily_schedules.push({
                                day_index: i,
                                time_segments: []
                            });
                        }
                    }

                    return {
                        id: id,
                        name: name,
                        description: description,
                        daily_schedules: daily_schedules,
                        holiday_schedules: holiday_schedules,
                        days_of_iteration: days_of_iteration,
                        start_date: start_date,
                        use_daily_iteration: use_daily_iteration
                    }
                });

                $scope.DEFAULT_SCHEDULE = Const.DEFAULT_SCHEDULE;

                $scope.holidayScheduleYN = false;
                $scope.schedules = new Schedules();

                $scope.HolidayGroupList = undefined;
                // Holiday 그룹 정보 조회
                API.get('holiday_groups').then(function success(data) {
                    $scope.HolidayGroupList = data.HolidayGroupCollection.rows;
                });

                $scope.changeUseDailyIterationRadio = function () {
                    var _schedules = new Schedules();
                    $scope.schedules.daily_schedules = angular.copy(_schedules.daily_schedules);
                    $scope.schedules.days_of_iteration = $scope.schedules.daily_schedules.length;
                };

                $scope.changeDailyIterationValue = function () {
                    if ($scope.schedules.days_of_iteration <= 90 && $scope.schedules.days_of_iteration >= 1) {
                        $scope.schedules = new Schedules($scope.schedules);
                    }
                };
                $scope.$watch('schedules.days_of_iteration', function (newValue, oldValue) {

                    if (newValue != oldValue) {
                        if (newValue > 90 || newValue < 0) {
                            $scope.schedules.days_of_iteration = oldValue;
                        }
                    }
                    if (newValue == 0 && oldValue == 0) {
                        $scope.schedules.days_of_iteration = 1;
                    } else if (newValue == '' && oldValue == 0) {

                    } else if (newValue == 0 && oldValue == '') {
                        $scope.schedules.days_of_iteration = 1;
                    }
                });

                $scope.doCopyYesterdayTimeLine = function ($index) {
                    $scope.schedules.daily_schedules[$index].time_segments = angular.copy($scope.schedules.daily_schedules[$index - 1].time_segments);
                };

                $scope.changeHolidayScheduleYNCheckBox = function () {

                    if ($scope.holidayScheduleYN) {
                        if (keepHolidaySchedulesValue) {
                            $scope.schedules.holiday_schedules = angular.copy(keepHolidaySchedulesValue);
                        } else {
                            $scope.schedules.holiday_schedules = [];
                            $scope.schedules.holiday_schedules.push({
                                holiday_group_id: {},
                                time_segments: []

                            })
                        }
                    } else {
                        keepHolidaySchedulesValue = angular.copy($scope.schedules.holiday_schedules);
                        $scope.schedules.holiday_schedules = [];
                    }
                };

                $scope.addHoliday = function () {
                    if ($scope.schedules.holiday_schedules.length < 4) {
                        $scope.schedules.holiday_schedules.push({
                            holiday_group_id: {},
                            time_segments: []
                        });
                    }
                };

                $scope.deleteHoliday = function ($index) {
                    MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                        if (confirmYn) {
                            $scope.schedules.holiday_schedules.splice($index, 1);
                            if (!$scope.schedules.holiday_schedules.length) {
                                $scope.holidayScheduleYN = false;
                            }
                            if (!$scope.$root.$$phase) $scope.$apply();
                        }
                    });

                };

                $scope.doApplyScheduleInfo = function () {
                    var i,
                        holiday_schedules = [],
                        call_api = API.post,
                        id = '';

                    if ($.trim($scope.schedules.name).length <= 0) {
                        MessageService.displayWarningMessage(MessageProperties.get('common.pleaseEnterName'));
                        return false;
                    }

                    if ($scope.schedules.holiday_schedules) {
                        for (i = $scope.schedules.holiday_schedules.length; i--;) {
                            var item = $scope.schedules.holiday_schedules[i];
                            if (!item.holiday_group_id || Object.keys(item.holiday_group_id).length == 0) {
                                MessageService.displayWarningMessage(MessageProperties.get('schedule.pleaseInputHolidayGroup'));
                                return false;
                            }

                            //if (!item.time_segments.length) {
                            //    MessageService.displayWarningMessage(MessageProperties.get('schedule.pleaseInputTimeSegment'));
                            //    return false;
                            //}

                            holiday_schedules.push(item);
                        }
                        $scope.schedules.holiday_schedules = angular.copy(holiday_schedules);
                    }

                    $scope.schedules.days_of_iteration = $scope.schedules.daily_schedules.length;
                    call_api('schedules' + id, undefined, {"Schedule": $scope.schedules}).then(function (data) {
                        var options = angular.element("#addScheduleDlg").dialog('options');
                        options.onClose();
                    });
                };

                $scope.doCancelScheduleInfo = function () {
                    var options = angular.element("#addScheduleDlg").dialog('options');
                    options.onClose();
                };
            }]);
});