'use strict';

define(['modules/common/services/access-control-agent', 'modules/common/services/user-agent'], function () {
    var app = angular.module('biostarWebApp');
    app.register.controller('DoorAlarmEditDlgController',
        ['$scope','$timeout','API','Const','Util','MessageProperties','MessageService','DeviceAgent','DoorAgent',
        function ($scope, $timeout, API, Const, Util, MessageProperties, MessageService, DeviceAgent, DoorAgent) {
            var ACTION_LIST = angular.copy(Const.ACTION_LIST);
            var OUTPUT_TYPES = angular.copy(Const.OUTPUT_TYPES);
            var pageMode = Const.PAGE_MODE.CREATE;

            var DeviceList = [];
            var DeviceListWithoutSIO = [];

            var selectedAlarmItem =  window.sessionStorage.getItem('DoorAlarmInfo');

            var TriggerActions = function() {
                this.door_id = $scope.door.id;
                this.input_event_id = {};
                this.input_event_id.code = window.sessionStorage.getItem('DoorAlarmTypeInfo');

//                this.output_device_id = undefined;
                this.output_device_id = $scope.door.entry_device_id;

                this.output_type = Const.DEFAULT_OUTPUT_TYPE;
                this.output_function_mask = Const.DEFAULT_OUTPUT_FUNCTION_MASK;

                this.output_relay = undefined;
                this.output_signal_id= undefined;
                this.output_priority = undefined;

                return this;
            };

            function setRelayPortList(output_device_id) {

                DeviceAgent.getDeviceSpec().then(function success(data) {
                    var i, relayCnt;

                    for(i = 0; i < DeviceList.length; i++) {
                        if(output_device_id.id == DeviceList[i].id) {
                            output_device_id = DeviceList[i];
                        }
                    }

                    relayCnt = data[output_device_id.device_type_id.id].relay_num;

                    $scope.TriggerActionsRelayList = {};

                    for(i = 0; i < relayCnt; i++) {
                        var item = {};
                        item[i] = MessageProperties.get('door.relay', i);
                        $scope.TriggerActionsRelayList[i] = MessageProperties.get('door.relay', i);
                    }
                }, function error() {

                });
            }


            $scope.DOOR_ALARM_TYPE = window.sessionStorage.getItem('DoorAlarmTypeInfo');


//            부모 스코프에 정의 되어 있음.
//            $scope.SOUND = 2;
//            $scope.OUTPUT = 1;
//            $scope.OTHER_EVENT = 0;

            if($scope.familyDevices) {
                DeviceList = DeviceList.concat($scope.familyDevices);
                for(var i = 0; i < DeviceList.length; i++) {
                    if (DeviceList[i].device_type_id.id != Const.DEVICE_TYPE_ID.SECURE_IO && DeviceList[i].device_type_id.id != Const.DEVICE_TYPE_ID.SECURE_IO2) {
                        DeviceListWithoutSIO.push(DeviceList[i]);
                    }
                }
            }

            $scope.triggerActions = new TriggerActions();

            if(selectedAlarmItem) {
                pageMode = Const.PAGE_MODE.UPDATE;
                selectedAlarmItem = angular.fromJson(selectedAlarmItem);
                $scope.triggerActions = selectedAlarmItem.triggerActions;

                if($scope.triggerActions.output_type == $scope.OUTPUT) {
                    setRelayPortList($scope.triggerActions.output_device_id);
                }
            } else {
                pageMode = Const.PAGE_MODE.CREATE;
                $scope.triggerActions.output_device_id = $scope.door.entry_device_id;
                setRelayPortList($scope.triggerActions.output_device_id);
            }

            $scope.DeviceList = DeviceList;
            $scope.DeviceListWithoutSIO = DeviceListWithoutSIO;
            $scope.ACTION_LIST = ACTION_LIST;

            $scope.selectedActionItem = function(item) {
                $scope.triggerActions.output_function_mask = item.output_function_mask;
                $scope.triggerActions.output_type = item.output_type;
                $scope.triggerActions.output_device_id = $scope.door.entry_device_id;

                if(item.output_type == OUTPUT_TYPES.FUNCTION) {
//                    $scope.triggerActions.output_device_id = $scope.door.entry_device_id;
                } else {
                    $scope.output_relay = undefined;
//                    $scope.triggerActions.output_device_id = undefined;
                    $scope.TriggerActionsRelayList = undefined;
                }
            };


            $scope.doAddTriggerActions = function() {
                var i;
                if(!$scope.door.trigger_actions) {
                    $scope.door.trigger_actions = [];
                }

                if($scope.triggerActions.output_type == $scope.OUTPUT) {
                    if($scope.triggerActions.output_device_id == undefined) {
                        MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseSelectOutputDevice'));
                        return false;
                    }
                    if($scope.triggerActions.output_relay == undefined) {
                        MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseSelectOutputRelay'));
                        return false;
                    }
                    if($scope.triggerActions.output_signal_id == undefined) {
                        MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseSelectOutputSignal'));
                        return false;
                    }
                }

                if(pageMode == Const.PAGE_MODE.UPDATE) {
                    if(selectedAlarmItem) {
                        $scope.door.trigger_actions.splice(selectedAlarmItem.selectedItemIndex, 1);
                    }
                } else {
                    for(i = 0; i < $scope.door.trigger_actions.length; i++) {
                        var item = $scope.door.trigger_actions[i];

                        if(item.input_event_id.code == $scope.triggerActions.input_event_id.code) {
                            if (item.output_type == $scope.triggerActions.output_type && item.output_type == $scope.OUTPUT) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.canNotSelectExistsAction'));
                                return false;
                            } else if (item.output_function_mask == $scope.triggerActions.output_function_mask) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.canNotSelectExistsAction'));
                                return false;
                            }
                        }

                    }
                }

                $scope.door.trigger_actions.push($scope.triggerActions);

                $scope.$emit('InitDoorAlarmEditDlg');
            };

            $scope.doCancelAddTriggerActions = function() {
                $scope.$emit('InitDoorAlarmEditDlg');
            };

            $scope.TriggerActionsRelayList = undefined;
            $scope.$watch('triggerActions.output_device_id', function(nv, ov) {
                if(nv != ov && angular.isDefined(nv)) {
                    $scope.triggerActions.output_relay = undefined;
                    setRelayPortList(nv);
                }
            }, true);

            $scope.focusCombo = function() {
                angular.element('#relayCombo').addClass('open');
            };

            $scope.blurCombo = function() {
                angular.element('#relayCombo').removeClass('open');
            };

            console.log($scope.triggerActions);
        }]);
});
