'use strict';

define([
    'modules/common/services/door-agent'
], function () {
    var app = angular.module('biostarWebApp');

    app.register.controller('DoorGroupTreeController', ['$scope', '$rootScope', '$location', '$compile', '$timeout', 'Const', 'Util', 'DoorAgent', 'MessageProperties', 'MessageService', 'PrivilegeService',
        function ($scope, $rootScope, $location, $compile, $timeout, Const, Util, DoorAgent, MessageProperties, MessageService, PrivilegeService) {

            var DoorGroupTreeOption,
                contextMenuDiv,
                selectedNode,
                clickedNode,
                contextNode,
                renameYn,
                body = document.getElementsByTagName('body')[0];

            var cursorStatus;
            var menuID = "#doorGroupTreeContextMenu";

            function initContextMenu() {
                angular.element(menuID).menu({});
            }

            function clearMenuContext(divId) {
//            angular.element(divId).attr("class", "");
//            angular.element(divId).attr("style", "");
                angular.element(divId).find(".menu-item").remove();
            }

            function buildContextMenuDiv(contextMenuObject, ngIfCondition) {
                var html = '<div';
                if (ngIfCondition != undefined) {
                    html += ' ng-if="' + ngIfCondition + '" ';
                }
                html += ' class="menu-item" style="height:31px;" ng-click="' + contextMenuObject.clickFunctionName + '()" ng-privilege><div class="menu-text">' + contextMenuObject.menuName + '</div></div>';
                return html;
            }

			function doorGroupSelector(nodeId) {
				var findNode;
				var doorTreeArrays = angular.element('#doorGroupsTree').tree('getChildren', angular.element('#doorGroupsTree').tree('getRoot').target);

				for(var i = 0; i < doorTreeArrays.length; i++) {
					if(nodeId == doorTreeArrays[i].id && doorTreeArrays[i].isDoorGroups == true) {
						findNode = doorTreeArrays[i];
						break;
					}
				}

				if(findNode && findNode.target) {
					return findNode;
				}

				return undefined;
			}

            Util.removeSessionObject('selectedDoorGroupId');

            initContextMenu();

            var onContextMenu = function (e, node) {

                if (e.preventDefault) e.preventDefault();
                if (e.returnValue) e.returnValue = false;

                if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.ACCESS_CONTROL)) {
                    return false;
                }

                angular.element(menuID).menu("hide");
                clearMenuContext(menuID);


                if (node.isDoorGroups) {
                    angular.element(menuID).append(buildContextMenuDiv(ContextMenu.ADD_DOOR_GROUP));
                    if (node.id != 1) {
                        angular.element(menuID).append(buildContextMenuDiv(ContextMenu.RENAME_DOOR_GROUP));
                        angular.element(menuID).append(buildContextMenuDiv(ContextMenu.DELETE_DOOR_GROUP));
                    }
                } else {
                    angular.element(menuID).append(buildContextMenuDiv(ContextMenu.DELETE_DOOR));
                }
//
                angular.element(menuID).css("height", "auto");
                angular.element(menuID).css("width", "auto");

                $compile(angular.element(menuID))($scope);

                angular.element(menuID).menu('options').ownerTreeNode = node;
                angular.element(menuID).menu('show', {
                    left: e.pageX,
                    top: e.pageY
                });
            };

			function doorGroupMaxNumbers() {

				var newDoorGroupPrefix = MessageProperties.get("menu.door.newDoorGroup");
				var newDoorGroupsArray = angular.element('#doorGroupsTree').find('span[title*="' + newDoorGroupPrefix + '"]');
				var numbers = [];
				var regExp = /[\d]+/gi;

				for(var i = 0; i < newDoorGroupsArray.length; i++) {
					var text = angular.element(newDoorGroupsArray[i]).html();
					numbers = numbers.concat(text.match(regExp));
				}

				numbers.sort(function (a, b) {
					return parseInt(a) > parseInt(b) ? -1 : 1;
				});

//				console.log(numbers);

				return numbers[0] ? parseInt(numbers[0]) + 1: 1;

			}

            $scope.addDoorGroup = function addDoorGroup(_node) {
                var name;

                var newDoorGroupPrefix = MessageProperties.get("menu.door.newDoorGroup");

                var newDoorGroupsMaxCount = doorGroupMaxNumbers();

                if(_node == null) {
                    _node = angular.element(menuID).menu('options').ownerTreeNode;
                    if(_node == null) {
                        return false;
                    }
                }

                angular.element('#doorGroupTreeContextMenu').menu('hide');

                name = newDoorGroupPrefix + ' ' + newDoorGroupsMaxCount;

                if (_node.depth == 8) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.doorGroupCanNotExceedSteps'));
                } else {
                    var newNode = {
                        "parent_id": {
                            "id": _node.id
                        },
                        "isDoorGroups": true,
                        "depth": parseInt(_node.depth || 0) + 1,
                        "sync_device_groups": _node.sync_device_groups || [],
                        "sync_devices": _node.sync_devices || [],
                        "inherited": true,
                        "iconCls": "doorGroupIcon",
                        "text": name,
                        "name": name
                    };

                    if (_node.state == 'closed') {
                        angular.element('#doorGroupsTree').tree('expand', _node.target);
                        angular.element('#doorGroupsTree').tree('scrollTo', _node.target);
                    }

                    writeTreeExpandedState(_node.id, 'open');

                    DoorAgent.postOneDoorGroup(newNode).then(function success(data) {

						newNode.id = data.DoorGroup.id;

						angular.element('#doorGroupsTree').tree('append', {
							"parent": _node.target,
							"data": newNode
						});

						newNode.target = doorGroupSelector(newNode.id).target;
						angular.element('#doorGroupsTree').tree('beginEdit', newNode.target);
						angular.element(newNode.target).find('input.tree-editor').focus()[0].setSelectionRange(0, newNode.name.length);
                    });
                }
            };

            $scope.renameDoorGroupName = function renameDoorGroupName(contextNode) {

                angular.element('#doorGroupTreeContextMenu').menu('hide');

                    var node  = angular.element(menuID).menu('options').ownerTreeNode;
                    if(node == null) {
                        return false;
                    }

                if (node.id != 1) {
                    angular.element('#doorGroupsTree').tree('beginEdit', node.target);
                    angular.element(node.target).find('input.tree-editor').focus()[0].setSelectionRange(0, node.name.length);
                }
            };

            $scope.deleteDoorGroup = function deleteDoorGroup(contextNode) {
                angular.element('#doorGroupTreeContextMenu').menu('hide');

				var node = angular.element(menuID).menu('options').ownerTreeNode;
				if(node == null) {
					return false;
				}

                if (node.id != 1) {

                    MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                        if (confirmYn) {
                            DoorAgent.deleteDoorGroups(node.id);
                        }
                    });
                }
            };

            $scope.deleteDoor = function deleteDoor(contextNode) {
                angular.element('#doorGroupTreeContextMenu').menu('hide');

                if (contextNode == null) {
                    contextNode = angular.element(menuID).menu('options').ownerTreeNode;
                    if (contextNode == null) {
                        return false;
                    }
                }

                MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                    if (confirmYn) {
                        DoorAgent.deleteDoors([contextNode.id], function() {});
                    }
                });
            };

            function writeTreeExpandedState(group_id, state) {
                var expandedState = window.sessionStorage.getItem('DoorGroupTreeExpandedState');
                if (expandedState) {
                    expandedState = angular.fromJson(expandedState);
                } else {
                    expandedState = {};
                }
                expandedState[group_id] = state;

                window.sessionStorage.setItem('DoorGroupTreeExpandedState', angular.toJson(expandedState));
            }

            var ContextMenu = {
                'ADD_DOOR_GROUP': {
                    menuName: MessageProperties.get('door.addDoorGroup'), clickFunctionName: 'addDoorGroup'
                },
                'RENAME_DOOR_GROUP': {
                    menuName: MessageProperties.get('door.renameDoorGroup'), clickFunctionName: 'renameDoorGroupName'
                },
                'DELETE_DOOR_GROUP': {
                    menuName: MessageProperties.get('door.deleteDoorGroup'), clickFunctionName: 'deleteDoorGroup'
                },
                'DELETE_DOOR': {
                    menuName: MessageProperties.get('button.deleteDoor'), clickFunctionName: 'deleteDoor'
                }
            };
            var isAndroid = (/android/gi).test(navigator.appVersion);
            DoorGroupTreeOption = {
                animate: true,
                dnd: !isAndroid,
                formatter: function (node) {
                    if (node.isDoorGroups == true && node.id == Const.DEFAULT_ALL_GROUP_ID) {
                        node.name = MessageProperties.get("menu.door.allDoorGroups");
                    }
                    if (node.depth === '0') {
                        return '<span title="' + node.name + '">' + node.name + '</span>';
                    } else {
                        return '<span title="' + node.name + '">' + node.name + '</span>';
                    }
                },
                onClick: function (node) {
                    var parentNode = angular.element('#doorGroupsTree').tree('getParent', node.target);

                    if (node) {
                        if (node.isDoorGroups != true) {
                            selectedNode = parentNode;
                            clickedNode = parentNode;
//							Util.setSessionObject('selectedDoorGroupId', parentNode.id);
//                            angular.element('#doorGroupsTree').tree('select', parentNode.target);
                            $rootScope.$broadcast('SelectedDoorsInDoorGroupsTree', node);
//                            $rootScope.$broadcast('CurrentSelectedDoorGroupName', parentNode);
//                            $rootScope.$broadcast('ReloadDoorsDataGrid', parentNode);
                        } else {
                            selectedNode = node;
                            clickedNode = node;

                            Util.setSessionObject('selectedDoorGroupId', node.id);
                            $rootScope.$broadcast('CurrentSelectedDoorGroupName', node);


                            if ($scope.contextNode && $scope.contextNode.target) {
                                if (angular.element($scope.contextNode.target).find('input')[0]) {
                                    angular.element('#doorGroupsTree').tree('endEdit', $scope.contextNode.target);
                                }
                            }

                            $rootScope.$broadcast('ReloadDoorsDataGrid', node);
                        }
                    }
                },
                onExpand: function (node) {
                    writeTreeExpandedState(node.id, 'open');
                },
                onCollapse: function (node) {
                    writeTreeExpandedState(node.id, 'closed');
                },
				onDragOver: function (target, source) {
                    var parent = angular.element('#doorGroupsTree').tree('getData', target);

                    if (source.isDoorGroups) { // 소스가 도어 그룹일때
                        if (parent.isDoorGroups) { // 타겟이 도어 그룹일때

                        } else { // 타겟이 도어일때
//                        $rootScope.$broadcast('ReloadDoorGroupData');
                            return false;
                        }
                    } else { // 소스가 도어일때
                        if (parent.isDoorGroups) { // 타겟이 도어 그룹일때

                        } else { // 타겟이 도어일때
                            return false;
                        }

                    }
                },
                onBeforeDrop: function (target, source, point) {
                    var parent = angular.element('#doorGroupsTree').tree('getData', target);
                    var nodeChangeData;
                    var depth = 0;

                    function findLastChildrenDepth(node) {
                        var i;
                        if (depth < node.depth) {
                            depth = node.depth;
                        }
                        if (node.children && node.children.length) {
                            for (i = node.children.length; i--;) {
                                findLastChildrenDepth(node.children[i]);
                            }
                        }
                        return false;
                    }

                    if (source.isDoorGroups) { // 소스가 도어 그룹일때
                        if (parent.isDoorGroups) { // 타겟이 도어 그룹일때
                            findLastChildrenDepth(source);

                            depth = depth - source.depth + 1;

                            depth = parseInt(parent.depth) + parseInt(depth);

                            if (depth > 8) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.doorGroupCanNotExceedSteps'));
                                $scope.$broadcast('ReloadDoorGroupData');
                                return false;
                            } else {
                                nodeChangeData = {
                                    "id": source.id,
                                    "parent_id": {
                                        "id": parent.id
                                    }
                                };

								writeTreeExpandedState(parent.id, 'open');

                                DoorAgent.putOneDoorGroup(nodeChangeData).then(function success(data) {

                                }, function error(reason, status) {
//                                return false;
                                });
                            }
                        }
                    } else { // 소스가 도어일때

						var p_node = angular.element('#doorGroupsTree').tree('pop', source.target);

                        if (parent.isDoorGroups) { // 타겟이 도어 그룹일때
                            Util.debug(parent.id);

							writeTreeExpandedState(parent.id, 'open');

							if (parent.children && parent.children.length) {
								angular.element('#doorGroupsTree').tree('insert', {
									before: parent.children[0].target,
									data : p_node
								});
							} else {
								angular.element('#doorGroupsTree').tree('append', {
									parent: target,
									data: p_node
								});

							}

                            DoorAgent.putOneDoor({id: source.id, door: {"door_group_id": {id: parent.id}}}, function success(data) {
                            });
							return false;
                        }
                    }
                },
                onDrop: function (target, source, point) {
                },
                onBeforeEdit: function (node) {
                    node.text = node.name;
                },
                onCancelEdit: function (node) {
                    if (node.id == undefined) {
                        angular.element('#doorGroupsTree').tree('remove', node.target);
                    }
                },
                onAfterEdit: function (node) {

                    if (!node.text) {
                        MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseDoorGroupName'));
                        return false;
                    } else if (node.text.length == 0) {
                        MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseDoorGroupName'));
                        return false;
                    }

                    if (!angular.element(node.target).is(':visible')) {
                        return false;
                    }

                    if ($.trim(node.text).length <= 0) {
                        return false;
                    }

                    var nodeChangeData = {
                        "name": node.text.trim()
                    };

                    // 수정
                    nodeChangeData.id = node.id;
                    var selectedDoorGroup = doorGroupSelector(Util.getSessionObject('selectedDoorGroupId'));
                    DoorAgent.putOneDoorGroup(nodeChangeData).then(function success(data) {
                    }, function error(data) {
                    }).finally(function (data) {
                        angular.element('#lnbDoorGroupTree').scrollLeft(0);
                    });
                },
                onContextMenu: onContextMenu,
                onLoadSuccess: function (node, data) {
					var nodeId = Util.getSessionObject('selectedDoorGroupId');
					var findNode = doorGroupSelector(nodeId);

					if (!nodeId || !findNode) {
						findNode = angular.element('#doorGroupsTree').tree('getRoot');
					}

					if (findNode && findNode.target) {
//						angular.element('#doorGroupsTree').tree('select', findNode.target);
						DoorGroupTreeOption.onClick(findNode);

					}
                }
            };

            angular.element('#doorGroupsTree').tree(DoorGroupTreeOption);

            $scope.bindDoorTreeData = function () {
                Util.debug('%cBindDoorTreeData', 'background: skyblue; color: green');
                DoorAgent.getDoorGroupRecursiveDataWithDoors().then(function success(data) {
                    if (data != undefined) {
                        angular.element('#doorGroupsTree').tree('loadData', data);
                    }
                });
            };

            $scope.$on('ReloadDoorGroupData', function (event, data) {
                var reload = false;

                $scope.contextNode = undefined;

                Util.debug('%cReloadDoorGroupData', 'background: skyblue; color: green');

                if (data && data.reload) {
                    reload = data.reload
                }

                DoorAgent.getDoorGroupRecursiveDataWithDoors(reload).then(function success(data) {
                    if (data != undefined) {
                        angular.element('#doorGroupsTree').tree('loadData', data);
                    }
                });

                $scope.SelectAllYN = false;
            });

            $scope.doAddDoor = function () {
                $scope.$emit('OpenDoorInfo');
            };

            $scope.init = function () {
                $scope.bindDoorTreeData();
            };

            angular.element(menuID).on('contextmenu', function ($event) {
                var event = $event || window.event;

                if (event.preventDefault) event.preventDefault();
                if (event.returnValue) event.returnValue = false;

                event.cancelBubble = true;
            })
        }]);


});