'use strict';

define([
        'modules/common/services/door-agent',
        'modules/common/services/device-agent',
        'modules/common/services/access-control-agent',
        "modules/accessControl/add-schedule",
        "modules/accessControl/dialog/doorAlarmEditDlgController",
        "modules/accessControl/dialog/door-output-signal-add-dlg"
    ],
    function () {

        var app = angular.module('biostarWebApp');
        app.register.controller('DoorInfoController', ['$scope', '$rootScope', '$routeParams', '$location', '$timeout', 'API', 'Const', 'MessageProperties', 'DoorAgent', 'MessageService', 'DeviceAgent', 'Util', function ($scope, $rootScope, $routeParams, $location, $timeout, API, Const, MessageProperties, DoorAgent, MessageService, DeviceAgent, Util) {

            var
                saveData,
                door = {},
                pageMode = Const.PAGE_MODE.CREATE,
                DoorListData = [],
                DoorQueryString = angular.copy($scope.doorListQueryStrings),
                DoorTreeData = [];

            $scope.ENTRY_DEVICE = 0;
            $scope.EXIT_DEVICE = 1;
            $scope.SECURE_IO = 2;

            $scope.OPEN_TIME_LIST = Const.OPEN_TIME_LIST;
            $scope.RESET_TIME_LIST = Const.RESET_TIME_LIST;

            $scope.SOUND = 2;
            $scope.OUTPUT = 1;
            $scope.OTHER_EVENT = 0;

            $scope.DeviceSpecList = [];
            $scope.FAMILY = 'FAMILY';

            $scope.RelayList = {};
            $scope.ExitButtonList = {};
            $scope.DoorSensorList = {};
            $scope.RelayList[$scope.FAMILY] = [];
            $scope.ExitButtonList[$scope.FAMILY] = [];
            $scope.DoorSensorList[$scope.FAMILY] = [];

            var fnEntryDeviceWatcher = function (nv, ov) {

                if (angular.isDefined(ov) && angular.isDefined(nv) && nv.id == ov.id) {
                    return false;
                }

                $scope.door.trigger_actions = [];

                getFamilyData(nv);
                $scope.RelayList[$scope.FAMILY] = [];
                $scope.ExitButtonList[$scope.FAMILY] = [];
                $scope.DoorSensorList[$scope.FAMILY] = [];

                getInputPortList(nv, $scope.ENTRY_DEVICE);
                $scope.door.relay_output_id = undefined;

                $scope.door.exit_device_id = {};

                if (!$scope.door.exit_button_input_id) {
                    $scope.door.exit_button_input_id = {};
                }
                $scope.door.exit_button_input_id.type = Const.SWITCH_TYPE.NORMALLY_OPEN;
                $scope.door.exit_button_input_id.input_index = undefined;
                $scope.door.exit_button_input_id.device_id = {};
                $scope.door.exit_button_input_id.device_id.id = undefined;

                if (!$scope.door.sensor_input_id) {
                    $scope.door.sensor_input_id = {};
                }
                $scope.door.sensor_input_id.type = Const.SWITCH_TYPE.NORMALLY_OPEN;
                $scope.door.sensor_input_id.input_index = undefined;
                $scope.door.sensor_input_id.device_id = {};
                $scope.door.sensor_input_id.device_id.id = undefined;

            };
            var fnExitDeviceWatcher = function (nv, ov) {
                if (angular.isDefined(ov) == false && angular.isDefined(nv) == true) {
                    $scope.door.door_anti_passback = {};
                    $scope.door.door_anti_passback.apb_type = Const.APB_TYPE_TEXT.NONE;
                    $scope.door.door_anti_passback.reset_time = Const.DEFAULT_DOOR_APB_RESET_TIME;
                    $scope.door.door_anti_passback.open_when_disconnected = 0;

                } else if (angular.isDefined(ov) == true && angular.isDefined(nv) == true) {

                    if (Object.keys(nv).length == 0 && Object.keys(ov).length > 0) {
                        $scope.door.door_anti_passback = {};

                    } else if (Object.keys(nv).length > 0 && Object.keys(ov).length == 0) {
                        $scope.door.door_anti_passback = {};
                        $scope.door.door_anti_passback.apb_type = Const.APB_TYPE_TEXT.NONE;
                        $scope.door.door_anti_passback.reset_time = Const.DEFAULT_DOOR_APB_RESET_TIME;
                        $scope.door.door_anti_passback.open_when_disconnected = 0;
                    }
                }
            };

            function checkObject(edit, origin) {
                var item;
                for (item in origin) {
                    if (edit[item] != undefined) {
                        if (edit[item].constructor == Array) {

                        } else if (edit[item].constructor == Object) {
                            checkObject(edit[item], origin[item]);
                            if (Object.keys(edit[item]).length <= 0) {
                                delete edit[item];
                            }
                        } else {
                            if (edit[item] == origin[item]) {
                                delete edit[item];
                            }
                        }
                    }
                }
            }

            //Door 추가를 취소하고 목록페이지로 돌아갑니다.
            function cancelAddDoor() {
                saveData = {};
                $scope.$emit('CloseDoorInfo');
            }

            function addDoor() {

                if ($.trim($scope.door.name).length <= 0) {
                    MessageService.displayWarningMessage(MessageProperties.get('common.pleaseEnterName'));
                    return false;
                }

                if (angular.isUndefined($scope.door.relay_output_id) ||
                    angular.isUndefined($scope.door.relay_output_id.relay_index)) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseInputRelay'));
                    return false;
                }

                if ($scope.door.exit_button_input_id && $scope.door.exit_button_input_id.input_index == Const.PORT_UNSELECT) {
                    $scope.door.exit_button_input_id.device_id = {};
                }
                if ($scope.door.sensor_input_id && $scope.door.sensor_input_id.input_index == Const.PORT_UNSELECT) {
                    $scope.door.sensor_input_id.device_id = {};
                }

//                if($scope.door.authentication && $scope.door.authentication.approval_groups) {
//                    if($scope.door.authentication.approval_groups.length == 0) {
//                        delete $scope.door.authentication.approval_groups;
//                    }
//                }

                if (pageMode == Const.PAGE_MODE.CREATE) {

                    DoorAgent.postOneDoor($scope.door).then(function success(data) {
                        cancelAddDoor();
                    }, function error(data) {

                    });

                } else if (pageMode == Const.PAGE_MODE.UPDATE) {

                    var originInfo = angular.fromJson(window.sessionStorage.getItem('doorInfo')),
                        saveData = angular.copy($scope.door);

                    if (originInfo.name == saveData.name) {
                        delete saveData.name
                    }
                    if (saveData.door_group_id && saveData.door_group_id.id == originInfo.door_group_id.id) {
                        delete saveData.door_group_id
                    }

                    if ($scope.doorInfoForm.$valid) {
                        DoorAgent.putOneDoor({"id": $scope.door.id, "door": saveData}).then(function success(data) {
                            cancelAddDoor();
                        }, function error(data) {

                        });
                    }
                }
            }

            function loadDoorInfo(door) {
                pageMode = Const.PAGE_MODE.UPDATE;

                $scope.currDoorId = door.id;
                $scope.currDoorName = door.name;

                if (!door.exit_button_input_id || !door.exit_button_input_id.input_index) {
                    door.exit_button_input_id = {"name": "None", "device_id": {}, "input_index": undefined};
                }
                if (!door.sensor_input_id || !door.sensor_input_id.input_index) {
                    door.sensor_input_id = {"name": "None", "device_id": {}, "input_index": undefined};
                }

                if (DeviceAgent.getAllDevicesMapWithStatus().then) {
                    DeviceAgent.getAllDevicesMapWithStatus().then(function success(data) {
                        if (door.entry_device_id) {
                            door.entry_device_id = data[door.entry_device_id.id];
                            getInputPortList(door.entry_device_id, $scope.ENTRY_DEVICE);
                        }

                        if (door.exit_device_id) {
                            door.exit_device_id = data[door.exit_device_id.id];
                            getInputPortList(door.exit_device_id, $scope.EXIT_DEVICE);
                        }
                    })
                } else {
                    var data = DeviceAgent.getAllDevicesMapWithStatus();
                    if (door.entry_device_id) {
                        door.entry_device_id = data[door.entry_device_id.id];
                        getInputPortList(door.entry_device_id, $scope.ENTRY_DEVICE);
                    }

                    if (door.exit_device_id) {
                        door.exit_device_id = data[door.exit_device_id.id];
                        getInputPortList(door.exit_device_id, $scope.EXIT_DEVICE);
                    }
                }

                getFamilyData(door.entry_device_id);
            }

            $scope.getAllDevicesMapWithStatus = {};
            var getFamilyData = function (device) {
                var i, item, family = [];
                if (DeviceAgent.getAllDevicesMapWithStatus().then) {
                    DeviceAgent.getAllDevicesMapWithStatus().then(function success(data) {
                        makeFamilyData(device, data);
                    });

                } else {
                    var data = DeviceAgent.getAllDevicesMapWithStatus();
                    makeFamilyData(device, angular.copy(data));
                }
            };

            var makeFamilyData = function (device, data) {
                var i, item, j;
                if (device != undefined) {
                    var crew = data[device.id];
                    var family = [], parent, childrens;
                    if (crew.parent_device_id) {
                        family.push(data[crew.parent_device_id.id]);
                        parent = data[crew.parent_device_id.id];
                        childrens = getSlaveDevices(parent, data);

                        for (i = 0; i < childrens.length; i++) {
                            if (childrens[i].id != device.id) {
                                family.push(childrens[i]);
                            }
                        }

                    } else {
                        childrens = getSlaveDevices(crew, data);
                        family = childrens;
                    }

                    $scope.familyDevices = angular.copy(family);
                    $scope.exitDevices = [];
                    if ($scope.familyDevices && $scope.familyDevices.length) {
                        for (i = $scope.familyDevices.length; i--;) {
                            item = $scope.familyDevices[i];
                            if (item.device_type_id.id != Const.DEVICE_TYPE_ID.SECURE_IO && item.device_type_id.id != Const.DEVICE_TYPE_ID.SECURE_IO2) {
                                $scope.exitDevices.push(item);
                            }
                        }
                        $scope.familyDevices = $scope.familyDevices.concat([$scope.door.entry_device_id]);
                    } else {
                        $scope.familyDevices = [$scope.door.entry_device_id];
                    }

                    $scope.familyDevicesMaps = {};
                    for (i = $scope.familyDevices.length; i--;) {
                        item = $scope.familyDevices[i];
                        $scope.familyDevicesMaps[item.id] = item;
                    }

                }
            };

            var getSlaveDevices = function (parent, data) {
                var family = [], childrens;
                if (parent.rs485 && parent.rs485.channels) {
                    for (var ch = 0; ch < parent.rs485.channels.length; ch++) {
                        var channel = parent.rs485.channels[ch];
                        if (channel.slave_devices) {
                            childrens = channel.slave_devices;
                            for (var sIdx = 0; sIdx < childrens.length; sIdx++) {
                                var slave = childrens[sIdx];
                                slave = data[slave.id];
                                family.push(slave);
                            }
                        }

                    }
                }

                return family;
            };

            function getAction(output_function_mask, output_type) {
                var i, item;
                for (i = Const.ACTION_LIST.length; i--;) {
                    item = Const.ACTION_LIST[i];

                    if (item.output_type == output_type && output_type == $scope.OUTPUT) {
                        return item.name;
                    } else if (item.output_type == output_type && output_type == $scope.SOUND) {
                        return item.name;
                    } else if (item.output_type == output_type && output_type == $scope.OTHER_EVENT &&
                        item.output_function_mask == output_function_mask) {
                        return item.name;
                    }
                }
            }

            $scope.getAction = getAction;

            //Door Group Tree Data 호출
            if (DoorTreeData.then) {
                DoorTreeData.then(function success(data) {
                    $scope.DoorTreeData = data;
                });
            } else {
                $scope.DoorTreeData = DoorTreeData;
            }

//            $scope.RELAY_OPEN_CONDITION = Const.RELAY_OPEN_CONDITION;
            $scope.APB_TYPE = Const.APB_TYPE;
            $scope.ACTION_WHEN_FAILED = Const.ACTION_WHEN_FAILED;

            door = window.sessionStorage.getItem('doorInfo');
            if (door && door != 'undefined') {
                door = angular.fromJson(door);
            }

            var entryDeviceWatcher, exitDeviceWatcher;

            function watchInfo() {
                entryDeviceWatcher = $scope.$watch('door.entry_device_id', fnEntryDeviceWatcher, true);
                exitDeviceWatcher = $scope.$watch('door.exit_device_id', fnExitDeviceWatcher, true);
            }

            function watchRemover() {
                entryDeviceWatcher();
                exitDeviceWatcher();
            }

            function createTreeNode(originalNode, newNode){
//            newNode.status = originalNode.status;
                newNode.name = originalNode.name;
                newNode.text = originalNode.text;
                newNode.id = originalNode.id;
                newNode.device_type_id = originalNode.device_type_id;
                newNode.isDevice = originalNode.isDevice;
                newNode.iconCls = originalNode.iconCls.replace('disconnect', '').replace('sync_error', '');
                newNode.rs485 = originalNode.rs485;
//            newNode.status = originalNode.status;
                return newNode;
            }
            function removeDeviceCls(newTreeNode, originalTreeNode) {
                var item, newChildren = [];

                newTreeNode = createTreeNode(originalTreeNode, newTreeNode);

                if (originalTreeNode.children != undefined){
                    for (var i  = 0; i < originalTreeNode.children.length; i++){
                        var originalChildNode = originalTreeNode.children[i];
                        var newChildNode = {};

                        if (newTreeNode.children == undefined){
                            newTreeNode.children = [];
                        }

                        if(angular.isUndefined(originalChildNode.device_type_id)) {
                            newTreeNode.children.push(newChildNode);
                        } else if(originalChildNode.rs485.mode == Const.RS485_MODE.SLAVE) {
                            if(originalChildNode.parent_node_id != undefined) {
                                newTreeNode.children.push(newChildNode);
                            }
                        } else {
                            newTreeNode.children.push(newChildNode);
                        }

                        removeDeviceCls(newChildNode, originalChildNode);
                    }
                }
            }

            $scope.DeviceTreeData = [{}];
            if (DeviceAgent.getDeviceGroupRecursiveData().then) {
                DeviceAgent.getDeviceGroupRecursiveData().then(function success(data) {
                    removeDeviceCls($scope.DeviceTreeData[0], data);
                })
            } else {
                removeDeviceCls($scope.DeviceTreeData[0], DeviceAgent.getDeviceGroupRecursiveData());
            }

            $scope.$watch('door.open_once', function (nv, ov) {
                Util.info('open_once', 'nvalue', nv, 'ovalue', ov);
            })
            $scope.$watch('door.sensor_input_id.type', function (nv, ov) {
                Util.info('senson', 'nvalue', nv, 'ovalue', ov);
            })

            // Door 정보가 sessionStorage에 있다면..
            if (door.id) {
                $scope.door = door;
                loadDoorInfo($scope.door);

                watchInfo();

                if (!door.exit_button_input_id || door.exit_button_input_id.input_index == undefined) {
                    $scope.door.exit_button_input_id = {name: MessageProperties.get('common.none'), device_id: {}, input_index: Const.PORT_UNSELECT, type: Const.SWITCH_TYPE.NORMALLY_OPEN};
                }
                Util.log($scope.door.exit_button_input_id);

                if (!door.sensor_input_id || door.sensor_input_id.input_index == undefined) {
                    $scope.door.sensor_input_id = {name: MessageProperties.get('common.none'), device_id: {}, input_index: Const.PORT_UNSELECT, type: Const.SWITCH_TYPE.NORMALLY_OPEN};
                }

                if (!door.dual_authentication) {
                    door.dual_authentication = {};
                    door.dual_authentication.device = Const.DUAL_AUTHENTICATION_DEVICE_DEFAULT_VALUE;
                    door.dual_authentication.approval_type = Const.DUAL_AUTHENTICATION_APPROVAL_TYPE_DEFAULT_VALUE;
                    door.dual_authentication.timeout = Const.DUAL_AUTHENTICATION_TIMEOUT_DEFAULT_VALUE;
                }

                if (!$scope.door.open_once) {
                    Util.info('set open once');
                    $scope.door.open_once = Const.OPEN_ONCE.TRUE_VALUE;
                }

            } else { // doorId가 파라미터로 넘어오지 않았다면 추가 모드
                pageMode = Const.PAGE_MODE.CREATE;

                door.exit_button_input_id = {name: MessageProperties.get('common.none'), device_id: {}, input_index: undefined, type: Const.SWITCH_TYPE.NORMALLY_OPEN};
                door.sensor_input_id = {name: MessageProperties.get('common.none'), device_id: {}, input_index: undefined, type: Const.SWITCH_TYPE.NORMALLY_OPEN};

                window.sessionStorage.removeItem('doorCurrPos');

                $scope.door = door;

                $scope.door.open_duration = Const.DEFAULT_DOOR_OPEN_DURATION;

                $scope.door.door_group_id = {id: Util.getSessionObject('selectedDoorGroupId')};

                $scope.door.dual_authentication = {};

                $scope.door.dual_authentication.device = Const.DUAL_AUTHENTICATION_DEVICE_DEFAULT_VALUE;
                $scope.door.dual_authentication.approval_type = Const.DUAL_AUTHENTICATION_APPROVAL_TYPE_DEFAULT_VALUE;
                $scope.door.dual_authentication.timeout = Const.DUAL_AUTHENTICATION_TIMEOUT_DEFAULT_VALUE;

                $scope.door.open_once = Const.OPEN_ONCE.TRUE_VALUE;

                watchInfo();
            }

            $scope.addDoor = addDoor;
            $scope.cancelAddDoor = cancelAddDoor;


            $rootScope.$on('$routeChangeStart', function (evt, next, curr) {
                window.sessionStorage.removeItem('doorInfo');
                window.sessionStorage.removeItem('DoorAlarmTypeInfo');
                window.sessionStorage.removeItem('DoorAlarmInfo');
            });

            $scope.currPos = window.sessionStorage.getItem('doorCurrPos') || undefined;

            $scope.prevDoorInfoBtn = function () {
                var currPos = window.sessionStorage.getItem('doorCurrPos');
                var prevIndex = currPos - 2;
                var allDoorIdList = window.sessionStorage.getItem('getAllDoorIdList');

                if (allDoorIdList && currPos) {
                    allDoorIdList = angular.fromJson(allDoorIdList);
                } else {
                    cancelAddDoor();
                    return false;
                }

                if (prevIndex < 0) {
                    return false;
                }

                $scope.currPos = parseInt(currPos) - 1;

                window.sessionStorage.setItem('doorCurrPos', $scope.currPos);

                if (allDoorIdList[prevIndex] && allDoorIdList[prevIndex] != null) {
                    getDoorInfo(allDoorIdList[prevIndex]);
                } else {
                    getAllDoorIdList(prevIndex);
                }
            };

            $scope.nextDoorInfoBtn = function () {
                var currPos = window.sessionStorage.getItem('doorCurrPos');
                var nextIndex = currPos;
                var allDoorIdList = window.sessionStorage.getItem('getAllDoorIdList');

                if (allDoorIdList && currPos) {
                    allDoorIdList = angular.fromJson(allDoorIdList);
                } else {
                    cancelAddDoor();
                    return false;
                }

                if (currPos >= allDoorIdList.length) {
                    return false;
                }

                $scope.currPos = parseInt(currPos) + 1;

                window.sessionStorage.setItem('doorCurrPos', $scope.currPos);

                if (allDoorIdList[nextIndex] && allDoorIdList[nextIndex] != null) {
                    getDoorInfo(allDoorIdList[nextIndex]);
                } else {
                    getAllDoorIdList(nextIndex);
                }
            };

            function getAllDoorIdList(index) {
                var i, allDoorIdList, startIdx;
                var limit = angular.copy($scope.doorListQueryStrings.limit);

                allDoorIdList = window.sessionStorage.getItem('getAllDoorIdList');

                if (allDoorIdList) {
                    allDoorIdList = angular.fromJson(allDoorIdList);
                } else {
                    allDoorIdList = [];
                }

                $scope.doorListQueryStrings.offset = Math.floor(index / limit) * limit;

                startIdx = parseInt($scope.doorListQueryStrings.offset);

                DoorAgent.getDoors($scope.doorListQueryStrings, function success(data) {
                    if (allDoorIdList.length != data.DoorCollection.total) {
                        allDoorIdList = [];
                    }

                    limit = parseInt($scope.doorListQueryStrings.limit);

                    if (parseInt($scope.doorListQueryStrings.offset) + parseInt($scope.doorListQueryStrings.limit) > data.DoorCollection.total) {
                        limit = parseInt(data.DoorCollection.total) - $scope.doorListQueryStrings.offset;
                    }

                    for (i = 0; i < limit; i++) {
                        allDoorIdList[startIdx + i] = data.DoorCollection.rows[i].id;
                    }

                    getDoorInfo(allDoorIdList[index]);

                    window.sessionStorage.setItem('getAllDoorIdList', angular.toJson(allDoorIdList));
                });
            }

            function getDoorInfo(id) {
                DoorAgent.getOneDoor(id).then(function success(data) {
                    $scope.door = angular.copy(data);
                    window.sessionStorage.setItem('doorInfo', angular.toJson($scope.door));
                    $scope.currDoorId = $scope.door.id;
                    $scope.currDoorName = $scope.door.name;
                    watchRemover();
                    loadDoorInfo($scope.door);
                    watchInfo();
                });
            }

            function getInputPortList(device, role) {
                var i, item, relayCount, inputPortCnt;

                $scope.RelayList[role] = [];
                $scope.ExitButtonList[role] = [];
                $scope.DoorSensorList[role] = [];

                DeviceAgent.getDeviceSpec().then(function success(data) {
                    $scope.DeviceSpecList = data;
//                    setInputPortList(data, device, role);

                    if (device && role == $scope.ENTRY_DEVICE && $scope.familyDevices && $scope.familyDevices.length) {
                        $scope.RelayList[$scope.FAMILY] = [];
						$scope.DoorSensorList[$scope.FAMILY] = [];
						$scope.ExitButtonList[$scope.FAMILY] = [];
                        for (i = $scope.familyDevices.length; i--;) {
                            item = $scope.familyDevices[i];
                            relayCount = $scope.DeviceSpecList[item.device_type_id.id].relay_num;
                            inputPortCnt = $scope.DeviceSpecList[device.device_type_id.id].input_port_num;
                            for (var j = 0; j < relayCount; j++) {
                                $scope.RelayList[$scope.FAMILY].push({
                                    device_id: {id: item.id},
                                    relay_index: j
                                });
                            }
                            for (j = 0; j < inputPortCnt; j++) {
                                $scope.ExitButtonList[$scope.FAMILY].push({
                                    device_id: {id: item.id},
                                    input_index: j,
                                    type: Const.SWITCH_TYPE.NORMALLY_CLOSE

                                });
                                $scope.DoorSensorList[$scope.FAMILY].push({
                                    device_id: {id: item.id},
                                    input_index: j,
                                    type: Const.SWITCH_TYPE.NORMALLY_CLOSE
                                });
                            }
                        }
                    }
                }, function error(data) {
                });
            }

            function setInputPortList(DeviceSpecList, device, role) {

                if (device && device.id && device.device_type_id) {

                    var i,
                        relayCnt = DeviceSpecList[device.device_type_id.id].relay_num,
                        inputPortCnt = DeviceSpecList[device.device_type_id.id].input_port_num;

                    for (i = 0; i < relayCnt; i++) {
                        $scope.RelayList[role].push({
                            device_index: role,
                            relay_index: i
                        });
                    }
                    for (i = 0; i < inputPortCnt; i++) {
                        $scope.ExitButtonList[role].push({
                            device_index: role,
                            input_index: i,
                            type: 1

                        });
                        $scope.DoorSensorList[role].push({
                            device_index: role,
                            input_index: i,
                            type: 1
                        });
                    }
                }
            }

            $scope.selectRelayItem = function (item) {
                $scope.door.relay_output_id = angular.copy(item);
                angular.element('.doorRelayComboBox').blur();
            };
            $scope.selectExitButtonItem = function (item) {

                if (item == Const.PORT_UNSELECT) {
                    $scope.door.exit_button_input_id = {};
                    $scope.door.exit_button_input_id.input_index = Const.PORT_UNSELECT;
                    $scope.door.exit_button_input_id.device_id = {};
                } else if (item && $scope.door.sensor_input_id && item.input_index == $scope.door.sensor_input_id.input_index && $scope.door.sensor_input_id.device_id && $scope.door.sensor_input_id.device_id.id == item.device_id.id) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.alreadySelectPort'));
                    return false;
                } else {
                    if (!$scope.door.exit_button_input_id) {
                        $scope.door.exit_button_input_id = {};
                    }
                    if (item) {
                        item = angular.copy(item);
                        $scope.door.exit_button_input_id.type = Const.SWITCH_TYPE.NORMALLY_OPEN;
                        $scope.door.exit_button_input_id.input_index = item.input_index;
                        $scope.door.exit_button_input_id.device_id = {};
                        $scope.door.exit_button_input_id.device_id.id = item.device_id.id;
                    } else {
                        $scope.door.exit_button_input_id.type = undefined;
                        $scope.door.exit_button_input_id.input_index = undefined;
                        $scope.door.exit_button_input_id.device_id = {}
                    }
                }

                angular.element('.exitButtonComboBox').blur();
            };
            $scope.selectDoorSensorItem = function (item) {

                if (item == Const.PORT_UNSELECT) {
                    $scope.door.sensor_input_id = {};
                    $scope.door.sensor_input_id.input_index = Const.PORT_UNSELECT;
                    $scope.door.sensor_input_id.device_id = {};
                } else if (item && $scope.door.exit_button_input_id && item.input_index == $scope.door.exit_button_input_id.input_index && $scope.door.exit_button_input_id.device_id && $scope.door.exit_button_input_id.device_id.id == item.device_id.id) {

                    MessageService.displayWarningMessage(MessageProperties.get('validate.alreadySelectPort'));
                    return false;
                } else {
                    if (!$scope.door.sensor_input_id) {
                        $scope.door.sensor_input_id = {};
                    }
                    if (item) {
                        item = angular.copy(item);
                        $scope.door.sensor_input_id.type = Const.SWITCH_TYPE.NORMALLY_OPEN;
                        $scope.door.sensor_input_id.input_index = item.input_index;
                        $scope.door.sensor_input_id.device_id = {};
                        $scope.door.sensor_input_id.device_id.id = item.device_id.id;
                    } else {
                        $scope.door.sensor_input_id.type = undefined;
                        $scope.door.sensor_input_id.input_index = undefined;
                        $scope.door.sensor_input_id.device_id = {}
                    }
                }

                angular.element('.doorSensorComboBox').blur();
            };

            $scope.getVisibleCheck = function (item) {
//                if(angular.isUndefined(item)) {
//                    return true;
//                }
//
//                if($scope.door.exit_button_input_id &&
//                    item.input_index == $scope.door.exit_button_input_id.input_index &&
//                    $scope.door.exit_button_input_id.device_index == item.device_index) {
//                    return false;
//                }
//
//                if($scope.door.sensor_input_id &&
//                    item.input_index == $scope.door.sensor_input_id.input_index &&
//                    $scope.door.sensor_input_id.device_index == item.device_index) {
//                    return false;
//                }
//
//                if( $scope.door.relay_output_id &&
//                    item.relay_index == $scope.door.relay_output_id.relay_index &&
//                    $scope.door.relay_output_id.device_index == item.device_index) {
//                    return false;
//                }

                return true;
            };

            $scope.doorAlarmEditDlg = {};
            $scope.doOpenDoorHeldOpenAlarmEditDlg = function (item, $index) {
                var doorAlarmEditDlg = {};
                doorAlarmEditDlg.url = 'modules/accessControl/dialog/doorAlarmEditDlg.html';
                window.sessionStorage.setItem('DoorAlarmTypeInfo', Const.DOOR_ALARM.HELD_OPEN);

                if (item) {
                    var doorAlarmInfo = {};
                    doorAlarmInfo.triggerActions = item;
                    doorAlarmInfo.selectedItemIndex = $index;
                    window.sessionStorage.setItem('DoorAlarmInfo', angular.toJson(doorAlarmInfo));
                }

                $scope.doorAlarmEditDlg = doorAlarmEditDlg;
            };

            $scope.doOpenDoorForcedOpenAlarmEditDlg = function (item, $index) {
                var doorAlarmEditDlg = {};
                doorAlarmEditDlg.url = 'modules/accessControl/dialog/doorAlarmEditDlg.html';
                window.sessionStorage.setItem('DoorAlarmTypeInfo', Const.DOOR_ALARM.FORCED_OPEN);

                if (item) {
                    var doorAlarmInfo = {};
                    doorAlarmInfo.triggerActions = item;
                    doorAlarmInfo.selectedItemIndex = $index;
                    window.sessionStorage.setItem('DoorAlarmInfo', angular.toJson(doorAlarmInfo));
                }

                $scope.doorAlarmEditDlg = doorAlarmEditDlg;
            };

            $scope.doOpenDoorAntiPassBackAlarmEditDlg = function (item, $index) {
                var doorAlarmEditDlg = {};
                doorAlarmEditDlg.url = 'modules/accessControl/dialog/doorAlarmEditDlg.html';
                window.sessionStorage.setItem('DoorAlarmTypeInfo', Const.DOOR_ALARM.ANTI_PASS_BACK);

                if (item) {
                    var doorAlarmInfo = {};
                    doorAlarmInfo.triggerActions = item;
                    doorAlarmInfo.selectedItemIndex = $index;
                    window.sessionStorage.setItem('DoorAlarmInfo', angular.toJson(doorAlarmInfo));
                }

                $scope.doorAlarmEditDlg = doorAlarmEditDlg;
            };

            $scope.doDeleteDoorAlarmActionItem = function ($index) {
                MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                    if (confirmYn) {
                        $scope.door.trigger_actions.splice($index, 1);
                        if (!$scope.$root.$$phase) $scope.$apply();
                    }
                });
            };

            $scope.$on('InitDoorAlarmEditDlg', function () {
                $scope.doorAlarmEditDlg = {};
                angular.element('#doorAlarmEditDlg').dialog('destroy');
                window.sessionStorage.removeItem('DoorAlarmTypeInfo');
                window.sessionStorage.removeItem('DoorAlarmInfo');

                if (!$scope.$root.$$phase) $scope.$apply();
            });

            $scope.doAfterOpenDoorAlarmEditDlg = function () {
                var options = {};
                options.modal = true;
                options.shadow = false;
                options.border = false;
                options.title = MessageProperties.get("door.addAction");
                options.width = 600;
                options.height = 'auto';

                options.onClose = function () {
                    $scope.$emit('InitDoorAlarmEditDlg');
                };

                angular.element("#doorAlarmEditDlg").dialog(options);
            };

            $scope.$watch('door.door_anti_passback.apb_type', function (nv, ov) {
                Util.log(ov, nv);
                if (angular.isDefined(nv) && nv != Const.APB_TYPE_TEXT.NONE) {
                    if (ov == undefined || ov == Const.APB_TYPE_TEXT.NONE) {
                        $scope.door.door_anti_passback.reset_time = Const.DEFAULT_DOOR_APB_RESET_TIME;
                        $scope.door.door_anti_passback.open_when_disconnected = 0;
                    }
                }
            });

            $scope.$watch('door.relay_output_id', function (nv) {
                if (nv != undefined && nv.relay_index > -1) {
//                    $scope.selectedRelay = MessageProperties.get('door.relayList', nv.relay_index, $scope.DEVICE_ROLE[nv.device_index]);
                    $scope.selectedRelay = MessageProperties.get('door.relayList', nv.relay_index, $scope.familyDevicesMaps[nv.device_id.id].name);
                } else {
                    $scope.selectedRelay = MessageProperties.get('common.none');
                }
            }, true);
            $scope.$watch('door.exit_button_input_id', function (nv) {
                if (nv != undefined && nv.input_index > -1) {
                    $scope.exitButtinInput = MessageProperties.get('door.portList', nv.input_index, $scope.familyDevicesMaps[nv.device_id.id].name);
                } else {
                    $scope.exitButtinInput = MessageProperties.get('common.none');
                }
            }, true);
            $scope.$watch('door.sensor_input_id', function (nv) {
                if (nv != undefined && nv.input_index > -1) {
                    $scope.sensorInput = MessageProperties.get('door.portList', nv.input_index, $scope.familyDevicesMaps[nv.device_id.id].name);
                    if ($scope.door.open_timeout == undefined) {
                        $scope.door.open_timeout = Const.DEFAULT_DOOR_HELD_OPEN_DURATION;
                    }
                } else {
                    $scope.sensorInput = MessageProperties.get('common.none');
                }
            }, true);
//            $scope.$watch('door.relay_output_id.relay_index', function(nv) {
//
//            });
//            $scope.$watch('door.relay_output_id.relay_index', function(nv) {
//
//            });

            angular.element('.doorSensorComboBox').on('focusin', function () {
                angular.element('.doorSensorComboBox').addClass('open');
            });
            angular.element('.doorSensorComboBox').on('focusout', function () {
                angular.element('.doorSensorComboBox').removeClass('open');
            });

            angular.element('.exitButtonComboBox').on('focusin', function () {
                angular.element('.exitButtonComboBox').addClass('open');
            });
            angular.element('.exitButtonComboBox').on('focusout', function () {
                angular.element('.exitButtonComboBox').removeClass('open');
            });

            angular.element('.doorRelayComboBox').on('focusin', function () {
                angular.element('.doorRelayComboBox').addClass('open');
            });
            angular.element('.doorRelayComboBox').on('focusout', function () {
                angular.element('.doorRelayComboBox').removeClass('open');
            });

            $scope.$watch('door.dual_authentication.device', function (nv, ov) {
                if (nv != ov && nv == Const.DUAL_AUTHENTICATION_EXIT_DEVICE_ID) {
                    if($scope.door.exit_device_id == {} || $scope.door.exit_device_id == undefined || $scope.door.exit_device_id.id == undefined) {
                        MessageService.displayWarningMessage(MessageProperties.get('validate.notExistsExitDevices'));
                        door.dual_authentication.device = ov;
                    }
                }
            });


        }]);
    });
