'use strict';

define([
        'modules/common/services/door-agent',
        'modules/accessControl/doorGroupTreeController',
    'modules/accessControl/doorInfoController',
        'modules/accessControl/dialog/door-info-batch-edit-dlg',
        'modules/common/datagrid/columnSettingController',
        'modules/common/datagrid/columnSettingService',
        'modules/common/print/printOptionController',
        'modules/common/print/printOptionService'
    ],
    function () {

        var app = angular.module('biostarWebApp');

        app.register.controller('DoorsController', ['$scope', '$rootScope', '$compile', '$document', '$window', '$q', '$location', 'Const', 'DoorAgent', 'DeviceAgent', 'MessageProperties', 'Util', 'ColumnSettingService', 'PrintOptionService', 'MessageService', 'PrivilegeService', function ($scope, $rootScope, $compile, $document, $window, $q, $location, Const, DoorAgent, DeviceAgent, MessageProperties, Util, ColumnSettingService, PrintOptionService, MessageService, PrivilegeService) {


            if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.ACCESS_CONTROL)) {
                $scope.readOnly = true;
            }

            $scope.gridId = "doorGrid";

            $scope.clearCheckItemArray = function () {
                $scope.checkItemArray = [];
            };

            var
                doorInfo = undefined,
                httpTimeOut = undefined,
                canceller = $q.defer(),
                doorListQueryStrings = {
                    query: undefined,
                    offset: 0,
                    limit: 15,
                    group_id: 1,
                    total: undefined,
                    timeout: undefined
                },
                doorListDataGridOptions = {
                    scrollbarSize: 0,
                    view: $.extend({}, $.fn.datagrid.defaults.view, {
                        onBeforeRender: function (target, rows) {
                            angular.element(target).datagrid('resize');
                        },
                        onAfterRender: function (target) {
                            $.fn.datagrid.defaults.view.onAfterRender.call(this, target);
                            var grdPanel = angular.element(target).datagrid('getPanel');
                            var opts = angular.element(target).datagrid('options');
                            var vc = grdPanel.children('div.datagrid-view');
                            grdPanel.find(".datagrid-empty").remove();

                            if (angular.element(target).datagrid('getRows').length === 0) {
                                var html = '<div style="width:100%; height:360px; font-size:1.333em; line-height: 360px; ' +
                                    'vertical-align: middle; text-align: center;">' + MessageProperties.get('validate.notFound') +
                                    '</div>';
                                grdPanel.find('div.datagrid-view2 div.datagrid-body').html(html);
                            }

                            //헤더에 전체선택 버튼이 없다면 추가
                            if (grdPanel.find('.datagrid-header-check.compiled').length <= 0) {
                                grdPanel.find('.datagrid-header-check').attr({
                                    'select-all-yn': 'SelectAllYN',
                                    'datagrid-id': '"' + target.id + '"',
                                    'clear-function': 'clearCheckItemArray'
                                });
                                $compile(grdPanel.find('.datagrid-header-row'))($scope);
                                grdPanel.find('.datagrid-header-check').addClass('compiled');
                            }
                        }
                    }),
                    fitColumns: true,
                    resize: true,
                    singleSelect: true,
                    selectOnCheck: false,
                    checkOnSelect: false,
                    checkbox: !$scope.readOnly,
                    columns: [
                        [
                            {field: ($scope.readOnly ? 'none' : Const.FIELD_NAME.CHECKBOX), checkbox: !$scope.readOnly, hidden: $scope.readOnly},
                            {field: 'name', title: MessageProperties.get('common.name'), width: 80, align: 'left', sortable: true},
                            {field: 'door_group_id', title: MessageProperties.get('common.group'), width: 80, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    if (value) {
                                        return value.name;
                                    }
                                    return '-';

                                }},
                            {field: 'entry_device_id', title: MessageProperties.get('door.entryDevice'), width: 80, align: 'left', sortable: true,
                                formatter: function (value, row, index) {
                                    if (value) {
                                        return value.name;
                                    }
                                    return '-';
                                }},
                            {field: 'exit_device_id', title: MessageProperties.get('door.exitDevice'), width: 80, align: 'left', sortable: true,
                                formatter: function (value, row, index) {
                                    if (value) {
                                        return value.name;
                                    }
                                    return '-';
                                }},
                            {field: 'status', title: MessageProperties.get('common.status'), width: 80, align: 'left',
                                formatter: function (value, row, index) {
                                    var returnValue = '';
                                    if (row.status != undefined) {
                                        var inStatus = parseInt(row.status);
                                        if (inStatus & Const.DOOR_STATUS.APB_FAILED) {
                                            returnValue += MessageProperties.get("common.apbFailed") + '<br/>';
                                        }
                                        if (inStatus & Const.DOOR_STATUS.DISCONNECTED) {
                                            returnValue += MessageProperties.get("common.disconnected") + '<br/>';
                                        }
                                        if (inStatus & Const.DOOR_STATUS.FORCED_OPEN) {
                                            returnValue += MessageProperties.get("common.forcedOpen") + '<br/>';
                                        }
                                        if (inStatus & Const.DOOR_STATUS.HELD_OPENED) {
                                            returnValue += MessageProperties.get("common.heldOpen") + '<br/>';
                                        }
                                        if (inStatus & Const.DOOR_STATUS.LOCKED) {
                                            returnValue += MessageProperties.get("common.locked") + '<br/>';
                                        }
                                        if (inStatus & Const.DOOR_STATUS.UNLOCKED) {
                                            returnValue += MessageProperties.get("common.unlocked") + '<br/>';
                                        }
                                        if (inStatus == Const.DOOR_STATUS.NORMAL) {
//                                            returnValue += MessageProperties.get("common.invalid");
                                            returnValue += MessageProperties.get("common.normal") + '<br/>';
                                        }
                                        if (returnValue.length < 1) {
                                            returnValue = row.status;
                                        }
                                    }
                                    return returnValue;
                                }}
                        ]
                    ],
                    onCheck: function (rowIndex, rowData) {
                        var existsYn = $scope.checkItemArray.indexOf(rowData.id);
                        if (existsYn == -1) {
                            if ($scope.checkItemArray.length >= Const.SELECT_LIST_ITEM_LIMIT) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.canNotSelectMoreItem', Const.SELECT_LIST_ITEM_LIMIT));
                                angular.element('#doorGrid').datagrid('uncheckRow', rowIndex);
                            } else {
                                $scope.checkItemArray.push(rowData.id);
                                if (!$scope.$root.$$phase) $scope.$apply();
                            }
                        }
                    },
                    onUncheck: function (rowIndex, rowData) {
                        if ($scope.SelectAllYN == true) {
                            $scope.checkItemArray = [];
                            var data = angular.element('#doorGrid').datagrid('getChecked');
                            $scope.SelectAllYN = false;
                            if (!$scope.$root.$$phase) $scope.$apply();
                            for (var i = 0; i < data.length; i++) {
                                var index = angular.element('#doorGrid').datagrid('getRowIndex', data[i]);
                                angular.element('#doorGrid').datagrid('checkRow', index);
                            }
                        } else {
                            var idx = $scope.checkItemArray.indexOf(rowData.id);
                            if (idx > -1) {
                                $scope.checkItemArray.splice(idx, 1);

                                if (!$scope.$root.$$phase) $scope.$apply();
                            }
                        }
                    },
                    onCheckAll: function (rows) {
                        var i, uncheckList = [];
                        for (i = 0; i < rows.length; i++) {
                            var item = rows[i];
                            var idx = $scope.checkItemArray.indexOf(item.id);

                            if (idx == -1) {
                                if ($scope.checkItemArray.length < Const.SELECT_LIST_ITEM_LIMIT) {
                                    $scope.checkItemArray.push(item.id);
                                } else {
                                    uncheckList.push(item);
                                }
                            }
                        }

						if (uncheckList.length > 0 && $scope.SelectAllYN == false) {
                            MessageService.displayWarningMessage(MessageProperties.get('validate.canNotSelectMoreItem', Const.SELECT_LIST_ITEM_LIMIT));
                            for (i = uncheckList.length; i--;) {
                                console.log(uncheckList[i].user_id);
                                angular.element('#doorGrid').datagrid('uncheckRow', angular.element('#doorGrid').datagrid('getRowIndex', uncheckList[i]));
                            }
                        }

                        if (!$scope.$root.$$phase) $scope.$apply();
                    },
                    onUncheckAll: function (rows) {
                        if ($scope.SelectAllYN == true) {
                            $scope.SelectAllYN = false;
                            $scope.checkItemArray = [];
                        }
                        for (var i = rows.length; i--;) {
                            var item = rows[i];
                            var idx = $scope.checkItemArray.indexOf(item.id);
                            if (idx > -1) {
                                $scope.checkItemArray.splice(idx, 1);
                            }
                        }
                        if (!$scope.$root.$$phase) $scope.$apply();

                    },
                    onSelect: function (rowIndex, rowData) {
                        angular.element('#doorGrid').datagrid('unselectAll');
                    },
                    onClickCell: function (rowIndex, field) {
                        $rootScope.clickCellOnGrid($scope.gridId, rowIndex, field, "id", function (rowData) {
                            DoorAgent.getOneDoor(rowData.id).then(function success(data) {
                                window.sessionStorage.setItem('doorCurrPos', parseInt($scope.doorListQueryStrings.offset) + rowIndex + 1);
                                setDoorInfoUrl()
                            });
                        });
                    },
                    onSortColumn: function (sort, order) {
//                        console.log('%cOnSortColumn', 'background:crimson;color:white;')
                        var i,
                            orderItemArray,
                            descending = order == 'desc',
                            orderItemObject = {};

                        $scope.doorListQueryStrings.order_by = sort + ":" + descending;

                        beSearchDoorList();
                    }
                };

            function setDoorInfoUrl() {
                $scope.selectedDoorInfo = 'modules/accessControl/doorInfo.html';
            }

            function CheckUnCheckOnGrid(rowData, CheckAllYN) {
                var i, idx, item;
                if (rowData.constructor == Object) {
                    idx = $scope.checkItemArray.indexOf(rowData.id);
                    if (idx > -1) {
                        $scope.checkItemArray.splice(idx, 1);
                    } else {
                        $scope.checkItemArray.push(rowData.id);
                    }

                } else {
                    if (CheckAllYN) {
                        for (i = rowData.length; i--;) {
                            item = rowData[i];
                            idx = $scope.checkItemArray.indexOf(item.id);
                            if (idx == -1) {
                                $scope.checkItemArray.push(item.id);
                            }
                        }
                    } else {
                        for (i = rowData.length; i--;) {
                            item = rowData[i];
                            idx = $scope.checkItemArray.indexOf(item.id);
                            if (idx > -1) {
                                $scope.checkItemArray.splice(idx, 1);
                            }
                        }
                    }
                }
                if (!$scope.$root.$$phase) $scope.$apply();

            }

//            function SelectDoorGroupAfterAction(door_group) {
//                $scope.door_group = door_group
//            }
//
            $scope.deleteTheSelectedDoor = function () {
                MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                    if (confirmYn) {
                        var param = $scope.checkItemArray;
                        if ($scope.SelectAllYN) {
                            param = 'ALL';
                        }
                        DoorAgent.deleteDoors(param, function () {
                            $scope.$emit('ReloadDoorsDataGrid', {reload: true});
                        });
                    }
                });
            };

            $scope.checkItemArray = [];
            $scope.door_group = {};

            $scope.$on('CurrentSelectedDoorGroupName', function (event, data) {
                $scope.door_group.id = data.id;
                $scope.door_group.name = data.name;
            });

            $scope.doorListQueryStrings = doorListQueryStrings;
            $scope.DoorListData = [];

            angular.element('#doorGrid').datagrid(doorListDataGridOptions);

            //헤더에 전체선택 버튼 추가
            $scope.SelectAllYN = false;

            $scope.$watch('SelectAllYN', function (nv) {
                if (nv) {
                    angular.element('#doorGrid').datagrid('checkAll');
                } else {
                    angular.element('#doorGrid').datagrid('uncheckAll');
                    $scope.checkItemArray = [];
                }
            });

            $scope.cancelSelectAllDoors = function () {
                angular.element('#doorGrid').datagrid('uncheckAll');
            };


            function beSearchDoorList(query) {

                var i, rowData, allDoorIdList, startIdx, limit;

                if (query) {
                    doorListQueryStrings.query = query;
                }

                DoorAgent.getDoors($scope.doorListQueryStrings, (function success(data) {
                    var index = 0;
                    if (data.DoorCollection.rows != undefined) {
                        for (index = 0; index < data.DoorCollection.rows.length; index++) {
                            if (data.DoorCollection != undefined && data.DoorCollection.rows[index] != undefined &&
                                data.DoorCollection.rows[index].door_group_id != undefined && data.DoorCollection.rows[index].door_group_id.id == Const.DEFAULT_ALL_GROUP_ID) {
                                data.DoorCollection.rows[index].door_group_id.name = MessageProperties.get("menu.door.allDoorGroups");
                            }
                        }
                    }

                    $scope.DoorListData = data.DoorCollection.rows || [];
                    angular.element('#doorGrid').datagrid('loadData', $scope.DoorListData);

                    window.sessionStorage.setItem('DoorListData', angular.toJson($scope.DoorListData));

                    allDoorIdList = new Array(parseInt(data.DoorCollection.total));

                    startIdx = parseInt($scope.doorListQueryStrings.offset);
                    limit = parseInt($scope.doorListQueryStrings.limit);

                    if (parseInt($scope.doorListQueryStrings.offset) + parseInt($scope.doorListQueryStrings.limit) > data.DoorCollection.total) {
                        limit = parseInt(data.DoorCollection.total) - $scope.doorListQueryStrings.offset;
                    }

                    for (i = 0; i < limit; i++) {
                        allDoorIdList[startIdx + i] = data.DoorCollection.rows[i].id;
                    }

                    window.sessionStorage.setItem('getAllDoorIdList', angular.toJson(allDoorIdList));


                    $scope.doorListQueryStrings.total = data.DoorCollection.total || 0;

                    if ($scope.SelectAllYN) {
                        for (i = $scope.DoorListData.length; i--;) {
                            rowData = $scope.DoorListData[i];
                            if ($scope.checkItemArray.indexOf(rowData.id) < 0) {
                                $scope.checkItemArray.push(rowData.id);
                            }
                        }
                    }

                    if (!$scope.SelectAllYN) {
                        angular.element('.datagrid-header-check').find('input').attr('checked', false);
                    } else {

                    }

                    for (i = $scope.DoorListData.length; i--;) {
                        rowData = $scope.DoorListData[i];
                        if ($scope.checkItemArray.indexOf(rowData.id) > -1) {
                            angular.element('#doorGrid').datagrid('checkRow', i);
                        }
                    }
                }));
            }


            $scope.SearchCallback = {
                fn: beSearchDoorList
            };

            $scope.$on('ReloadDoorsDataGrid', function (event, data) {
                console.log('%cReloadDoorsDataGrid', 'background:gray;color:yellow');
                angular.element('#doorGrid').datagrid('uncheckAll');

                if (data && data.id) {
                    doorListQueryStrings.group_id = data.id;
                } else {
                    doorListQueryStrings.group_id = 1;
                }

                doorListQueryStrings.offset = 0;

                if (data && data.offset) {
                    doorListQueryStrings.offset = data.offset;
                }

                beSearchDoorList();
            });

            $scope.$on('SelectedDoorsInDoorGroupsTree', function (event, node) {
                DoorAgent.getOneDoor(node.id).then(function success(data) {
                    setDoorInfoUrl();
                });
            });

            $scope.$watch(function () {
                return  angular.element('.panel.datagrid').width();
            }, function (nv, ov) {
                if (nv !== ov) {
                    angular.element('#doorGrid').datagrid('resize');
                }
            }, true);

            angular.element($window).on('resize', function ($event) {
                var event = $event | window.event;
                angular.element('#doorGrid').datagrid('resize');
                Util.debug('div#doorGrid resize');
            });

//            $scope.disableSelectedDoorInfo = function () {
//               MessageService.displayConfirmMessage(MessageProperties.get('SelectedDoorsDisabled'), function (confirmYn) {
//                    if (confirmYn) {
//                        DoorAgent.putDoors({"doorIDs": $scope.checkItemArray, "door": {"disabled": "true"}}).then(function success(data) {
//                            beSearchDoorList();
//                        }, function error(reason, status) {
//
//                        })
//                    }
//                })
//            }

            $scope.dropDownMenu = [
                {
                    menu: MessageProperties.get('menu.print'),
                    func: function () {
                        PrintOptionService.openPrintOptionPopupDlg();
                    }
                },
                {
                    menu: MessageProperties.get('menu.columnSetting'),
                    func: function () {
                        ColumnSettingService.openColumnSettingPopupDlg();
                    }
                }
            ];


            function closeDoorInfo(event, data) {
                window.sessionStorage.removeItem('doorInfo');
                window.sessionStorage.removeItem('DoorListData');
                window.sessionStorage.removeItem('doorCurrPos');
                window.sessionStorage.removeItem('DoorAlarmTypeInfo');
                $scope.$broadcast('ReloadDoorGroupData', {reload: true});
                $scope.selectedDoorInfo = undefined;
            }

            var clickMenuInitUnbind = $rootScope.$on('ClickMenuInit', closeDoorInfo);
            $scope.$on('$locationChangeStart', clickMenuInitUnbind);

            $scope.$on('CloseDoorInfo', closeDoorInfo);

            $scope.$on('OpenDoorInfo', function (event, data) {
                window.sessionStorage.setItem('doorInfo', '{}');
                setDoorInfoUrl();
            });


            doorInfo = window.sessionStorage.getItem('doorInfo');
            if (doorInfo && doorInfo != 'undefined') {
                doorInfo = angular.fromJson(doorInfo);
                setDoorInfoUrl()
            }

            $scope.doorInfoBatchEditDlg = {};
            $scope.doOpenDoorInfoBatchEditDlg = function () {
                $scope.doorInfoBatchEditDlg.url = 'modules/accessControl/dialog/door-info-batch-edit-dlg.html';
            };

            $scope.$on('InitDoorInfoBatchEditDlgUrl', function () {
                $scope.doorInfoBatchEditDlg = {};
                angular.element('#doorInfoBatchEditDlg').dialog('destroy');

                if (!$scope.$root.$$phase) $scope.$apply();

            });

            $scope.doAfterOpenDoorInfoBatchEditDlg = function () {
                var options = {};
                options.iconCls = 'icon-save';
                options.modal = true;
                options.shadow = false;
                options.border = false;
                options.title = MessageProperties.get('user.header.batchEdit');
                options.width = 550;
                options.height = 'auto';
                options.top = 200;

                options.onClose = function () {
                    $scope.$emit('InitDoorInfoBatchEditDlgUrl');
                };

                $("#doorInfoBatchEditDlg").dialog(options);
            };

            $scope.$watch('doorListQueryStrings', function (nv, ov) {
                if (nv !== ov) {
                    PrintOptionService.buildGridQueryFunction(Const.REPORT_COLLECTION_NAME.DOOR, nv);
                }
            }, true);

            PrintOptionService.getReportColumnTypeFunction = function (gridColumnName) {
                if (gridColumnName == "status") {
                    return "mask"
                } else {
                    return "string";
                }
            };

            PrintOptionService.getReportColumnNameFunction = function (gridColumnName) {
                var reportColumnName = gridColumnName;
                if (gridColumnName == "door_group_id") {
                    return "door_group_id.name"
                } else if (gridColumnName == "entry_device_id") {
                    return "entry_device_id.name"
                } else if (gridColumnName == "exit_device_id") {
                    return "exit_device_id.name"
                }
                return reportColumnName;
            };
            $scope.reportUrl = Const.API.BASE_URL + 'doors/report';
            $scope.openPrintOptionPopupDlg = PrintOptionService.openPrintOptionPopupDlg;
            $scope.openColumnSettingPopupDlg = ColumnSettingService.openColumnSettingPopupDlg;

            ColumnSettingService.setGridId($scope.gridId);
            ColumnSettingService.setCallbackFunction($scope.SearchCallback.fn);
            var customColumnSetting = ColumnSettingService.getColumnSettingFromLocalStroage();
            if (customColumnSetting) {
                ColumnSettingService.applyChangedColumnsToGrid(customColumnSetting, false);
            }

//                    beSearchDoorList();
            DeviceAgent.getDeviceSpec();

        }]);
    });



