'use strict';

define([
    'modules/common/services/monitoringAgent',
    'modules/common/alert/alertListService',
    'modules/common/alert/alertService'
], function () {

    var app = angular.module('biostarWebApp');

    app.controller('AlertListController', [
        '$scope', '$rootScope', '$timeout', '$compile', '$location', '$filter', 'Const', 'API', 'Util', 'AlertListService', 'AlertService', 'MonitoringAgent', 'MessageProperties', 'PrivilegeService', '$window',
        function ($scope, $rootScope, $timeout, $compile, $location, $filter, Const, API, Util, AlertListService, AlertService, MonitoringAgent, MessageProperties, PrivilegeService, $window) {

            $scope.gridId = "AlertListGrid";
            $scope.alertQuery = {
//                query: undefined,
                offset: 0,
                limit: 15,
                total: undefined,
                timeout: undefined
            };

            function dataRetrieved(dataList, query) {
                var i, rowData;
                $scope.alertListData = dataList;
                $scope.alertQuery = query;
                AlertService.setPendingCount(query.total);

                for (i = $scope.alertListData.length; i--;) {
                    rowData = $scope.alertListData[i];
                    if ($scope.checkItemArray.indexOf(rowData) > -1) {
                        angular.element('#' + $scope.gridId).datagrid('checkRow', i);
//                        $scope.checkItemArray.push(rowData);
                    }
                }
            }

            $scope.searchCallback = {
                fn: function (ackAlert) {
                    var chkTotal = (ackAlert)? $scope.alertQuery.total - ackAlert.AlertCollection.total : $scope.alertQuery.total;
                    AlertListService.getGridQueryFunction(function () {
                        var query = {};
                        query.offset = $scope.alertQuery.offset;
                        query.limit = $scope.alertQuery.limit;
                        query.total = chkTotal;
                        var conditionsObject = new Array();
                        conditionsObject.push({
                            column: 'status',
                            operator: Const.QUERY_OPERATOR.EQUAL,
                            values: [Const.ALERT_STATUS.PENDING]
                        });
                        query.conditions = conditionsObject;
                        query.orders = [];
                        query.orders.push({column: 'event_id.datetime', descending: true});
                        return query;
                    });
                    AlertListService.search(dataRetrieved, $scope.gridId);
                }
            };


            $scope.initPopup = function (gridId, isCheckField) {
                $scope.gridId = gridId || $scope.gridId;
//            console.log("alert list init popup");
                $scope.alertListInput = {};

                var gridColumns = [
                    [
                        {
                            field: 'event_id.datetime', title: MessageProperties.get('common.date'),
                            width: 80, align: 'center', sortable: false,
                            formatter: function (value, row, index) {
                                return $filter('BssDatetime')(new Date(row.event_id.datetime));
                            }
                        },
                        {
                            field: 'event_id.door_id.name', title: MessageProperties.get('common.door'),
                            width: 100, align: 'center', sortable: false,
                            formatter: function (value, row, index) {
                                return Util.formatIdAndNameSpan(row.event_id.door_id);
                            }
                        },
                        {
                            field: 'event_id.device_id.name', title: MessageProperties.get('common.device'),
                            width: 100, align: 'left', sortable: false,
                            formatter: function (value, row, index) {
                                return Util.formatIdAndNameSpan(row.event_id.device_id);
                            }
                        },
                        {
                            field: 'event_id.user_id.name', title: MessageProperties.get('common.user'),
                            width: 100, align: 'center', sortable: false,
                            formatter: function (value, row, index) {
                                if (angular.isDefined(row.event_id.user_id)) {
                                    row.event_id.user_id.id = row.event_id.user_id.user_id;
                                }
                                return Util.formatIdAndNameSpan(row.event_id.user_id);
                            }
                        },
                        {
                            field: 'event_id.event_type_id.name', title: MessageProperties.get('common.alarm'),
                            width: 100, align: 'center', sortable: false,
                            formatter: function (value, row, index) {
                                return Util.formatIdAndNameSpan(row.event_id.event_type_id);
                            }
                        }
                    ]
                ];
                if (isCheckField !== undefined) {
                    gridColumns[0].unshift({field: ($scope.readOnly ? 'none' : Const.FIELD_NAME.CHECKBOX), checkbox: !$scope.readOnly, hidden: $scope.readOnly});
                }
                var gridOptions = {
                    scrollbarSize: 0,
                    fitColumns: true,
//                resize: true,
                    singleSelect: true,
                    height: '363',
                    selectOnCheck: false,
                    checkOnSelect: false,
                    columns: gridColumns,
                    view: $.extend({}, $.fn.datagrid.defaults.view, {
                        onBeforeRender: function (target, rows) {
                            $('#' + $scope.gridId).datagrid('resize');
                        },
                        onAfterRender: function (target) {
                            var dataGridBodyElement = angular.element("#" + $scope.gridId).parent(".datagrid-view").find(".datagrid-view2").find(".datagrid-body"),
                                grdPanel = angular.element(target).datagrid('getPanel');
                            if (angular.element("#" + $scope.gridId).datagrid("getRows").length === 0) {
                                var html = '<div class="listNotFound">' +
                                    $scope.gridInformation + '</div>';
                                dataGridBodyElement.html(html);
                            }
                        }
                    }),
                    onCheck: function (rowIndex, rowData) {
                        CheckUnCheckOnGrid(rowData);

                    },
                    onUncheck: function (rowIndex, rowData) {
                        if ($scope.SelectAllYN == true) {
                            $scope.checkItemArray = [];
                            var data = angular.element('#' + $scope.gridId).datagrid('getChecked');
                            $scope.SelectAllYN = false;
                            if (!$scope.$root.$$phase) $scope.$apply();
                            for (var i = 0; i < data.length; i++) {
                                var index = angular.element('#' + $scope.gridId).datagrid('getRowIndex', data[i]);
                                angular.element('#' + $scope.gridId).datagrid('checkRow', index);
                            }
                        } else {
                            CheckUnCheckOnGrid(rowData);
                        }
                    },
                    onCheckAll: function (rows) {
                        CheckUnCheckOnGrid(rows, true);
                    },
                    onUncheckAll: function (rows) {
                        if ($scope.SelectAllYN == true) {
                            $scope.SelectAllYN = false;
                            $scope.checkItemArray = [];
                        }
                        CheckUnCheckOnGrid(rows, false);

                    },
                    onClickRow: function (rowIndex, rowData) {
//                        if (PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.MONITORING)) {
//                            if (rowData.status == Const.ALERT_STATUS.ACKNOWLEDGE) {
//                            } else if (rowData.status == Const.ALERT_STATUS.PENDING) {
//                                $scope.toggleAckInputSection(rowData);
//                            } else if (rowData.status == Const.ALERT_STATUS.SNOOZING) {
//                            }
//                        }
                    }
                };

                angular.element('#' + $scope.gridId).datagrid(gridOptions);

                $scope.searchCallback.fn();

                //헤더에 전체선택 버튼 추가
//                angular.element('#alertListPopupDiv').find('.datagrid-header-check').attr({'select-all-yn': 'SelectAllYN', 'datagrid-id': '"' + $scope.gridId + '"'});
//                $compile(angular.element('#alertListPopupDiv').find('.datagrid-header-row'))($scope);
            };
            $scope.gridInformation = MessageProperties.get('validate.notFound');

//            $scope.cancelSelectAllUser = function () {
//                angular.element('#' + $scope.gridId).datagrid('uncheckAll');
//            };

            $scope.checkItemArray = [];
            $scope.$watch('checkItemArray', function(nv, ov){
                if (PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.MONITORING)) {
//                    if (rowData.status == Const.ALERT_STATUS.ACKNOWLEDGE) {
//                    } else if (rowData.status == Const.ALERT_STATUS.PENDING) {
//                    } else if (rowData.status == Const.ALERT_STATUS.SNOOZING) {
//                    }
                    $scope.toggleAckInputSection(nv);
                }
            }, true);

            function CheckUnCheckOnGrid(rowData, CheckAllYN) {
                var i, idx, item;
                if (rowData.constructor == Object) {
                    idx = $scope.checkItemArray.indexOf(rowData);
                    if (idx > -1) {
                        $scope.checkItemArray.splice(idx, 1);
                    } else {
                        $scope.checkItemArray.push(rowData);
                    }

                } else {
//                $scope.checkItemArray = [];
                    if (CheckAllYN) {
                        for (i = rowData.length; i--;) {
                            item = rowData[i];
                            idx = $scope.checkItemArray.indexOf(item);
                            if (idx == -1) {
                                $scope.checkItemArray.push(item);
                            }
                        }
                    } else {
                        for (i = rowData.length; i--;) {
                            item = rowData[i];
                            idx = $scope.checkItemArray.indexOf(item);
                            if (idx > -1) {
                                $scope.checkItemArray.splice(idx, 1);
                            }

                        }
                    }
                }

                if (!$scope.$root.$$phase) $scope.$apply();
            }

            angular.element($window).on('resize', function ($event) {
                var event = $event | window.event;
                angular.element('#' + $scope.gridId).datagrid('resize');
            });

            $scope.doClose = function () {
                angular.element("#alert_list_comment").hide();
                angular.element("#alert_list_ack_button").hide();
                AlertListService.closeDialog();
            };

            $scope.toggleAckInputSection = function (checkItemArray) {
                if (checkItemArray.length > 0) {
                    angular.element("#alert_list_comment").show();
                    angular.element("#alert_list_ack_button").show();
//                    var description = '';
//                    if (alertObject.event_id != undefined && alertObject.event_id.event_type_id != undefined && alertObject.event_id.event_type_id.name != undefined) {
//                        description = ' for ' + alertObject.event_id.event_type_id.name;
//                    }
//                    angular.element("#alert_list_comment").attr("placeholder", "Please enter comment " + description + ".");
                } else {
                    angular.element("#alert_list_comment").hide();
                    angular.element("#alert_list_ack_button").hide();
                }
//                    angular.element("#alert_list_comment").val('');
            };

            $scope.viewHistory = function () {
                MonitoringAgent.setInitialViewId(Const.MONITORING_VIEW.ALERT_HISTORY);
                var currentPath = $location.path();
                if (currentPath != "/monitoring") {
                    $location.path('/monitoring').replace();
                }
                $rootScope.$broadcast("MonitoringViewChanged", {
                        viewId: Const.MONITORING_VIEW.ALERT_HISTORY
                    }
                );
                AlertListService.closeDialog();
            };

            function ackSuccessful() {
                angular.element('#' + $scope.gridId).datagrid('uncheckAll');
                angular.element("#alert_list_comment").hide();
                angular.element("#alert_list_ack_button").hide();
            }

            $scope.doAck = function () {
//            console.log("alertObject=" + $scope.alertObject.event_id.id);
//                AlertService.ack(ackSuccessful, $scope.alertObject.event_id, $scope.alertListInput.comment);
                var params = $scope.checkItemArray;
                AlertService.ackCollection(ackSuccessful, params, angular.element('#alert_list_comment').val());
            };


            $scope.$on('AlertReceived', function (event, alertObject) {
                if (alertObject != undefined) {
                    $scope.searchCallback.fn();
                }
            });

            $scope.$on('Acknowledged', function (event, ackAlert) {
                if (angular.element('#alert_list_comment').length) {
                    angular.element('#alert_list_comment').val('');
                }
                if (ackAlert != undefined) {
                    $scope.searchCallback.fn(ackAlert);
                }
            });
        }]);
})
;


