'use strict';

define([
    'modules/common/commonModule'
], function () {

    var app = angular.module('commonModule');

    app.service('AlertService', [
        '$rootScope', 'Const', 'API', 'Util', '$filter', 'DialogService', 'MessageProperties',
        function ($rootScope, Const, API, Util, $filter, DialogService, MessageProperties) {

        var callbackFunction;
        var alertObject;
        var alertPendingCount = 0;

        var alertService = {
            setAlert: function (paramAlertObject) {
                alertObject = paramAlertObject;
            },
            getAlert: function () {
                return alertObject;
            },
            updatePendingCountOnHtml: function (paramCount) {
                angular.element("#alarm_ticker_red_icon").html(paramCount);
            },
            increasePendingCount: function () {
                alertPendingCount++;
                this.updatePendingCountOnHtml(alertPendingCount);
            },
            decreasePendingCount: function (count) {
                alertPendingCount = alertPendingCount - (count | 1);
                this.updatePendingCountOnHtml(alertPendingCount);
            },
            setPendingCount: function (paramPendingCount) {
                alertPendingCount = paramPendingCount;
                this.updatePendingCountOnHtml(alertPendingCount);
            },
            getPendingCount: function () {
                return alertPendingCount;
            },
            setCallbackFunction: function (paramCallbackFunction) {
                callbackFunction = paramCallbackFunction;
            },
            openDialog: function (targetScope) {
                var alertObject = alertService.getAlert(),
                    descriptionHtml = '',
                    dialogOptions;
                targetScope.dialogComment = '';
                alertObject.datetimeFormatted = $filter('BssDatetimeLongTerm')(new Date(alertObject.event_id.datetime));
                descriptionHtml += '<div class="alertDetail">';
                descriptionHtml += '<dl>';
                var alertTitle = '';
                if (alertObject.event_id.event_type_id.alert_name != undefined && alertObject.event_id.event_type_id.alert_name.length > 0) {
                    alertTitle = alertObject.event_id.event_type_id.alert_name;
                } else {
                    alertTitle = alertObject.event_id.event_type_id.name;
                }
                var alertMessage = '';
                if (alertObject.event_id.event_type_id.alert_message != undefined && alertObject.event_id.event_type_id.alert_message.length > 0) {
                    alertMessage = alertObject.event_id.event_type_id.alert_message;
                } else {
                    alertMessage = alertObject.event_id.event_type_id.description;
                }
                if (alertMessage == undefined || alertMessage == null || alertMessage.length < 1) {
                    alertMessage = alertTitle;
                }
                if (alertObject.event_id != undefined) {
                    if (alertObject.event_id.user_id != undefined) {
                        descriptionHtml += '<dt>' + MessageProperties.get('common.user') + '</dt>';
                        if (alertObject.event_id.user_id.name !== undefined && alertObject.event_id.user_id.name.trim() !== '') {
                            descriptionHtml += '<dd>' + alertObject.event_id.user_id.name + '</dd>';
                        } else {
                            descriptionHtml += '<dd>' + alertObject.event_id.user_id.user_id + '</dd>';
                        }
                    }
                    if (alertObject.event_id.device_id != undefined) {
                        descriptionHtml += '<dt>' + MessageProperties.get('common.device') + '</dt>';
                        descriptionHtml += '<dd>' + alertObject.event_id.device_id.name + '</dd>';
                    }
                    if (alertObject.event_id.door_id != undefined) {
                        descriptionHtml += '<dt>' + MessageProperties.get('common.door') + '</dt>';
                        descriptionHtml += '<dd>' + alertObject.event_id.door_id.name + '</dd>';
                    }
                    descriptionHtml += '<dt>' + MessageProperties.get('common.alarm') + '</dt>';
                    descriptionHtml += '<dd>' + alertMessage + '</dd>';
                    descriptionHtml += '<dt>' + MessageProperties.get('common.date') + '</dt>';
                    descriptionHtml += '<dd>' + alertObject.datetimeFormatted + '</dd>';
                    descriptionHtml += '</dl>';
                    descriptionHtml += '<textarea ng-privilege="MONITORING" hide-when-not-allowed="true" id="alert_comment" name="alert_comment" data-ng-model="dialogComment" rows="5" cols="40" placeholder="Comments"/>';
                    descriptionHtml += '</div>';
                    dialogOptions = {
                        divId: 'alertPopupDialogDiv',
                        title: alertTitle,
                        targetScope: targetScope,
                        okButtonCallback: function () {
                            alertService.ack(function () {
                                DialogService.closeDialog(dialogOptions);
                            }, alertObject.event_id, targetScope.dialogComment);
                        },
                        cancelButtonCallback: function () {
                            DialogService.closeDialog(dialogOptions);
                        },
                        width: 400,
                        okButtonLabel: 'button.acknowledge',
                        cancelButtonLabel: 'button.ignore',
                        okButtonPrivilege: 'MONITORING',
                        type: Const.DIALOG_TYPE.SERVICE,
                        content: descriptionHtml
                    };
                    DialogService.openDialog(dialogOptions);
                }
//                var options = {};
//                options.iconCls = 'icon-save';
//                options.modal = true;
//                options.shadow = false;
//                options.border = false;
//                options.title = 'Alert';
//                options.width = 400;
//                options.height = 'auto';
//                options.top = 150;
//                options.onOpen = function () {
//                    $("#alertPopupDialogDiv").parent().addClass('bssServiceDlg')
//                }
//
//                options.onClose = function () {
//                    $rootScope.$apply(function () {
//                        $rootScope.alertPopupDialog.url = '';
//                        angular.element('#alertPopupDialogDiv').dialog('destroy');
//                    });
//                };
//
//                angular.element("#alertPopupDialogDiv").dialog(options);
//
//                $rootScope.alertPopupDialog = {};
//                $rootScope.alertPopupDialog.url = 'modules/common/alert/alertPopup.html';

            },
            closeDialog: function () {
                $rootScope.alertPopupDialog.url = '';
                angular.element('#alertPopupDialogDiv').dialog('destroy');
            },
            ack: function (callbackFunction, eventObject, comment) {
                var user = Util.getSessionObject(Const.SESSION_STORAGE_KEY.LOGGED_IN_USER);
                var userId = '';
                if (angular.isDefined(user)) {
                    userId = user.user_id;
                }
                var now = new Date();
                var paramObject = {
                    "Alert": {
                        event_id: {
                            id: eventObject.id,
                            datetime: eventObject.datetime
                        },
                        status: Const.ALERT_STATUS.ACKNOWLEDGE,
                        ack_user_id: {
                            user_id: userId
                        },
                        ack_datetime: now.toISOString(),
                        ack_message: comment
                    }
                };
                API.put('/alerts/' + eventObject.id, undefined, paramObject, undefined, undefined, undefined, false).then(function (data) {
//                    console.log('acked' + alertService.getPendingCount());
                    alertService.decreasePendingCount();
//                    console.log('acked' + alertService.getPendingCount());
                    $rootScope.$broadcast('Acknowledged', eventObject);
                    if (angular.isDefined(callbackFunction)) {
                        callbackFunction();
                    }
                });
            },
            ackCollection: function (callbackFunc, data, comment) {
                var i,
                    item,
                    params;

                for (i = data.length; i--;) {
                    item = data[i];
                    item.ack_message = comment;
                    item.status = Const.ALERT_STATUS.ACKNOWLEDGE;//ack status
                    //TODO [bjh] 차후 서버에서 server_datetime 값에 '이 빠지면 아래 코드 삭제
                    var j, k, targetObj;
                    for (j in item) {
                        if (j === 'event_id') {
                            targetObj = item[j];
                            for (k in item[j]) {
                                if (k === 'server_datetime') {
                                    targetObj[k] = targetObj[k].replace(/\'/gi, '');
                                }
                            }
                        }
                    }
                }
                params = {
                    AlertCollection: {
                        rows: data
                    }
                };
//                    user = Util.getSessionObject(Const.SESSION_STORAGE_KEY.LOGGED_IN_USER),
//                    now = new Date();
//                if (angular.isDefined(user)) {
//                    userId = user.user_id;
//                }
//                alertCollection.ack_user_id = {
//                    user_id: userId
//                };
                API.put('alerts', undefined, params).then(function (data) {
                    alertService.decreasePendingCount();
                    $rootScope.$broadcast('Acknowledged', data);
                    if (angular.isDefined(callbackFunc)) {
                        callbackFunc();
                    }
                });
            }

        };

        return alertService;
    }]);
});
