'use strict';

define(['modules/common/datagrid/columnSettingService'], function () {

    var app = angular.module('biostarWebApp');
    app.register.controller('ColumnSettingController', [
        '$scope', '$rootScope', '$timeout', 'ColumnSettingService', 'MessageProperties',
        function ($scope, $rootScope, $timeout, ColumnSettingService, MessageProperties) {

        $scope.initPopup = function () {
            $scope.checkedItemCount = 0;
            $scope.defaultColumnsClicked = false;
            var gridId = $scope.gridId;

            ColumnSettingService.setGridId(gridId);
            ColumnSettingService.setColumnSettingGridId('ColumnSettingGrid');

            var customColumnSetting = ColumnSettingService.getColumnSettingFromLocalStroage();
            var _dataRows = new Array();
            if (customColumnSetting) {
                _dataRows = customColumnSetting;
            } else {
                var defaultColumnSetting = ColumnSettingService.getDefaultColumnSettingFromLocalStroage();
                if (defaultColumnSetting == undefined) {
                    _dataRows = ColumnSettingService.buildAndSaveDefaultColumnSetting();
                } else {
                    _dataRows = defaultColumnSetting;
                }
            }

//            var sourceIndex = -1;
            angular.element("#ColumnSettingGrid").datagrid({
                fitColumns: true,
                height: "auto",
                iconCls: 'icon-edit',
                singleSelect: true,
                data: _dataRows,
                selectOnCheck: false,
                checkOnSelect: false,
                checkbox: true,
                columns: [
                    [
                        {field: 'column_name', checkbox: true, showAllCheckOption: false, width: 55},
                        {field: 'display_name', title: MessageProperties.get("columnSetting.header.column"), width: 250},
                        {field: 'check_value', hidden: true, width: 0},
                        {field: 'width', hidden: true, width: 0}
                    ]
                ],
                onLoadSuccess: function (data) {
                    $(this).datagrid('enableDnd');
                    for (var i = 0; i < data.rows.length; ++i) {
                        if (data.rows[i]['check_value'] == "true") {
                            $(this).datagrid('checkRow', i);
                        }
                    }
                },
                onCheck: function (rowIndex, rowData) {
                    rowData.check_value = "true";
                    $scope.checkedItemCount++;
                    changeApplyButtonStatus();
                },
                onUncheck: function (rowIndex, rowData) {
                    rowData.check_value = "false";
                    $scope.checkedItemCount--;
                    changeApplyButtonStatus();
                },
                onCheckAll: function (rows) {
                    $scope.checkedItemCount = rows.length;
                    for (var index=0; index < rows.length; index++) {
                        rows[index].check_value = "true";
                    }
                    changeApplyButtonStatus();
                },
                onUncheckAll: function (rows) {
                    $scope.checkedItemCount = 0;
                    for (var index=0; index < rows.length; index++) {
                        rows[index].check_value = "false";
                    }
                    changeApplyButtonStatus();
                }
            });

        };

        function changeApplyButtonStatus() {
            if ($scope.checkedItemCount > 0) {
                angular.element("#button_column_setting_apply").removeClass("disabled");
            } else {
                angular.element("#button_column_setting_apply").addClass("disabled");
            }
        };

        $scope.doApply = function () {
            if ($scope.checkedItemCount > 0) {
                var rows = angular.element("#ColumnSettingGrid").datagrid('getRows');
                var applyWidth = false;
                if ($scope.defaultColumnsClicked == true) {
                    applyWidth = true;
                }
                ColumnSettingService.applyChangedColumnsToGrid(rows, true, applyWidth);

                ColumnSettingService.closeDialog();
                $scope.defaultColumnsClicked = false;
            }
        };

        $scope.doCancel = function () {
            ColumnSettingService.closeDialog();
            $scope.defaultColumnsClicked = false;
        };

        $scope.doSetDefaultColumn = function () {
            var gridId = $scope.gridId;
            var defaultColumnSetting = ColumnSettingService.getDefaultColumnSettingFromLocalStroage();
            var _dataRows = new Array();
            if (defaultColumnSetting) {
                _dataRows = defaultColumnSetting;
            } else {
                _dataRows = ColumnSettingService.buildAndSaveDefaultColumnSetting();
            }
            angular.element("#ColumnSettingGrid").datagrid("loadData", _dataRows);
            $scope.defaultColumnsClicked = true;
        };


    }]);
});


