'use strict';

define(function () {

    var app = angular.module('biostarWebApp');

    app.register.service('ColumnSettingService', ['$rootScope', 'Const', 'MessageProperties', function ($rootScope, Const, MessageProperties) {

        var gridId = '';
//        var gridRows = {};
        var columnSettingGridId = '';
        var callbackFunction;
        var includeAllColumnsForPrinting = false;

        return {
            setGridId: function (paramGridId) {
                gridId = paramGridId;
            },
            setColumnSettingGridId: function (paramColumnSettingGridId) {
                columnSettingGridId = paramColumnSettingGridId;
            },
            setIncludeAllColumnsForPrinting: function (paramIncludeAllColumnsForPrinting) {
                includeAllColumnsForPrinting = paramIncludeAllColumnsForPrinting;
            },
            getIncludeAllColumnsForPrinting: function () {
                return includeAllColumnsForPrinting;
            },
            setCallbackFunction: function (paramCallbackFunction) {
                callbackFunction = paramCallbackFunction;
            },
//            setGridRows: function (paramGridRows) {
//                gridRows = paramGridRows;
//            },
            buildColumnSettingRows: function () {
                var columns = angular.element("#" + gridId).datagrid('getColumnFields');
                var rows = [];
                for (var index = 0; index < columns.length; index++) {
                    var options = angular.element("#" + gridId).datagrid('getColumnOption', columns[index]);
//                    console.log(columns[index]);
                    if (options.field == "event_image") {
                        continue;
                    }
                    if (options.title) {

                        var title = options.title;
//                        console.log(title);
                        if (title.indexOf("ibtnFilter") > -1) {
                            title = title.substr(0, title.indexOf("<"));
                        } else if (title.indexOf("ngbss-equal-filter") > -1) {
                            title = title.substr(0, title.indexOf("<"));
                        } else if (title.indexOf("ngbss-datetime-range-filter") > -1) {
                            title = title.substr(0, title.indexOf("<"));
                        } else {
                            var charIndex = title.indexOf('<');
                            if (charIndex > -1) {
                                charIndex = title.indexOf('>');
                                title = title.substr(charIndex + 1);
                                charIndex = title.indexOf('<');
                                title = title.substr(0, charIndex);
                            }
                        }
                        var checked = "true";
                        if (options.hidden !== undefined && options.hidden == true) {
                            checked = "false";
                        }
                        var printable = false;
                        if (includeAllColumnsForPrinting) {
                            printable = true;
                        } else if (options.sortable != undefined) {
                            printable = options.sortable;
                        }
                        var columnSettingObject = {
                            display_name: title.trim(),
                            column_name: columns[index],
                            check_value: checked,
                            width: options.width,
                            printable: printable
                        };
                        rows.push(columnSettingObject);
//                        console.log(columnSettingObject);
                    }
                }
                return rows;
            },
            buildLocalStroageKey: function () {
                return Const.LOCAL_STORAGE_KEY_PREFIX + '_grid_column_' + gridId;
            },
            saveColumnSettingToLocalStroage: function (gridRows) {
                localStorage.setItem(this.buildLocalStroageKey(gridId), angular.toJson(gridRows));
            },
            getColumnSettingFromLocalStroage: function () {
                var data = localStorage.getItem(this.buildLocalStroageKey(gridId));
                if (data) {
                    return angular.fromJson(data);
                } else {
                    return;
                }
            },
            buildLocalStroageKeyForDefaultColumns: function () {
                return Const.LOCAL_STORAGE_KEY_PREFIX + '_grid_default_column_' + gridId;
            },
            saveDefaultColumnSettingToLocalStroage: function (gridRows) {
                localStorage.setItem(this.buildLocalStroageKeyForDefaultColumns(gridId), angular.toJson(gridRows));
            },
            getDefaultColumnSettingFromLocalStroage: function () {
                var data = localStorage.getItem(this.buildLocalStroageKeyForDefaultColumns(gridId));
                if (data) {
                    return angular.fromJson(data);
                } else {
                    return;
                }
            },
            openColumnSettingPopupDlg: function () {
                var options = {};
                options.iconCls = 'icon-save';
                options.modal = true;
                options.shadow = false;
                options.border = false;
                options.title = MessageProperties.get("columnSetting.dialog.header");
                options.width = 400;
                options.height = 'auto';
//                options.top = 150;

                options.onClose = function () {
                    $rootScope.$apply(function () {
                        $rootScope.columnSettingDialog.url = '';
                        angular.element('#columnSettingPopupDlg').dialog('destroy');
                    });
                };

                angular.element("#columnSettingPopupDlg").dialog(options);

                $rootScope.columnSettingDialog = {};
                $rootScope.columnSettingDialog.url = 'modules/common/datagrid/columnSetting.html';

            },
            closeDialog: function () {
                $rootScope.columnSettingDialog.url = '';
                angular.element('#columnSettingPopupDlg').dialog('destroy');
            },
            buildAndSaveDefaultColumnSetting: function () {
                var defaultColumnRows = this.buildColumnSettingRows();
                this.saveDefaultColumnSettingToLocalStroage(defaultColumnRows);
                return defaultColumnRows;
            },
            applyChangedColumnsToGrid: function (gridRows, saveToLocalStroage, applyWidth) {
                var newColumnOptionsArray = new Array();

                var originalColumns = angular.element("#" + gridId).datagrid('getColumnFields');

                var startRestructuring = false;
                for (var originalIndex = 0; originalIndex < originalColumns.length; originalIndex++) {
//                    console.log(originalColumns[originalIndex]);
                    var originalColumnOptions = angular.element("#" + gridId).datagrid('getColumnOption', originalColumns[originalIndex]);
                    if (originalColumnOptions.title) {
                        if (!startRestructuring) {
                            for (var index = 0; index < gridRows.length; index++) {
                                var movedColumnOptions = angular.element("#" + gridId).datagrid('getColumnOption', gridRows[index].column_name);
                                if (movedColumnOptions != undefined) {
                                    if (gridRows[index].check_value == "false") {
                                        movedColumnOptions.hidden = true;
                                    } else if (gridRows[index].check_value == "true") {
                                        movedColumnOptions.hidden = false;
//                                        BSS-399 처리중 applyWidth를 사용하는 곳이 default로 돌릴경우만 쓰는데 applyWidth값 체크 않해도 문제가 없어 주석처리함
//                                        if (applyWidth != undefined && applyWidth == true) {
                                        movedColumnOptions.width = parseInt(gridRows[index].width);
//                                        }
                                    }
//                                console.log('movedColumnOptions:'+angular.toJson(movedColumnOptions));
                                    newColumnOptionsArray.push(movedColumnOptions);
                                }
                            }
                            startRestructuring = true;
                        }
                    } else {
                        startRestructuring = false;
                        newColumnOptionsArray.push(originalColumnOptions);
                    }
                }

                var newColumnDoubleArray = new Array();
                newColumnDoubleArray.push(newColumnOptionsArray);
                angular.element("#" + gridId).datagrid({
                    columns: newColumnDoubleArray
                });
                if (callbackFunction) {
                    callbackFunction(undefined, gridId + "_column_setting_api");
                }
                if (saveToLocalStroage != undefined && saveToLocalStroage == true) {
                    this.saveColumnSettingToLocalStroage(gridRows);
                }
            }

        };
    }]);
});
