'use strict';

define(['modules/common/services/auth'], function () {
    var app = angular.module('commonModule');
    app.directive('ngHeader', ['$rootScope', function ($rootScope) {
        return {
            restrict: 'A',
            replace: true,
            controller: ['$scope', '$location', '$timeout', 'API', 'Auth', 'Util', 'Const', 'DialogService', 'MessageProperties', function MenuCtrl($scope, $location, $timeout, API, Auth, Util, Const, DialogService, MessageProperties) {

                $scope.loginUser = Auth.getUser();


                $rootScope.$on('LoginToSuccess', function () {
                    $scope.loginUser = Auth.getUser();
                });

                $scope.$on('UserInfoUpdated', function (event, userData) {
                    if (Auth.getUser().user_id == userData.user_id) {
                        $scope.loginUser = Auth.getUser();
                        $scope.loginUser.name = userData.name;
                        Util.setSessionObject(Const.SESSION_STORAGE_KEY.LOGGED_IN_USER, $scope.loginUser);
                    }
                });

                $scope.doLogout = function () {
                    Auth.doLogout();
                };

                $scope.doClickSetting = function () {
                    $location.path('/setting');
                };


                $scope.doClickHelp = function () {
                    window.open('/help/' + Util.getLanguageFromLocalStorage() + '/index.html', 'biostar_help');
                };


                function openAboutDialog(biostarSetting) {
                    var dialogOptions = {
                        divId: 'AboutDialog',
                        title: MessageProperties.get("common.about"),
                        width: 520,
                        hideCancelButton: true,
                        okButtonCallback: function () {
                            DialogService.closeDialog(dialogOptions);
                        },
                        cancelButtonCallback: function () {
                            DialogService.closeDialog(dialogOptions);
                        },
                        content: '<div class="formList" >\
                        <h1 class="logo"><img src="/images/contents/login_logo.png"/></h1><br/><br/>\
                    <ul class="spaceS">\
                    <li class="col"><h5>' + MessageProperties.get("about.bioStarVersion") + '</h5><div style="display: inline-block; margin-left: 40px; margin-top: 10px;">V' + Const.BUILD_MAJOR_VERSION + '.' + Const.BUILD_MINOR_VERSION + '.' + Const.BUILD_VERSION + '</div></li>\
                    </ul>\
                    </div>\
                    <div><br/>' + MessageProperties.get("about.copyright") + '</div>\
                    <div style="color: white; position:absolute; left:10px; top: 10px; display:none;">\
                        <span>' + MessageProperties.get("about.webAppVersion") + '<br/>V' + Const.BUILD_MAJOR_VERSION + '.' + Const.BUILD_MINOR_VERSION + '.' + Const.BUILD_VERSION + '<br/>' + Const.BUILD_GIT_HASH + '<br/>' + MessageProperties.get("about.serverVersion") + '<br/>V' + biostarSetting.version + '<br/>' + Const.BUILD_SERVER_GIT_HASH + '</span>\
                    </div>'
                    };
                    DialogService.openDialog(dialogOptions);
                }

                $scope.doClickAbout = function () {

                    var biostarSetting = Util.getSessionObject(Const.SESSION_STORAGE_KEY.SERVER_SETTING);
                    if (biostarSetting == undefined) {
                        API.get("setting/biostar").then(function (data) {
                            Util.setSessionObject(Const.SESSION_STORAGE_KEY.SERVER_SETTING, data.BioStar);
                            openAboutDialog(data.BioStar);
                        });
                    } else {
                        openAboutDialog(biostarSetting);
                    }

                };
            }]
        };
    }]);
});