'use strict';
/**
 * <div ngbss-list-pagination data-ng-model="Parameters" data-search-function="SearchCallback" ng-show="DevicesData.length">
 *
 *   written by mphong 2014.07.17
 *
 *   directive name : ngbss-list-pagination
 *   type : attribute
 *   ng-model object : { query(String, optional), offset(Integer, optional), limit(Integer, optional) }
 *
 *   data-search-function object : { fn(Function, required) }
 *
 *   ng-show condition : list data length
 *
 */
define(function () {
    var app = angular.module('commonModule');
    app.directive('ngbssListPagination', ['$parse', '$compile', '$window', function ($parse, $compile, $window) {
        return {
            restrict: 'A',
            ngModel: true,
            scope: {
                Parameters: '=ngModel',
                SearchCallback: '=searchFunction'
            },
            replace: true,
            template: '<div class="paging">' +
                '<p tabindex="-1" class="pagingInfo">{{paginationDescription}}</p>' +
                '<a href="javascript:void(null);" ng-click="doPrev()" class="prev" ng-class="{\'disabled\':!Parameters.previousEnabled}">Previous Page</a>' +
                '<a href="javascript:void(null);" ng-click="doNext()" class="next" ng-class="{\'disabled\':!Parameters.nextEnabled}">Next Page</a>' +
                '<ul class="pagingScopePop" ng-click="doClickLimitBox($event)" tabindex="-1">' +
                '<li ng-repeat="item in [15, 25, 50, 75, 100] track by $index" ng-class="{\'on\':Parameters.limit == item}" >' +
                '<a href="javascript:void(null);">{{ item }}</a>' +
                '</li>' +
                '</ul>' +
                '</div>',
            controller: ['$scope', '$element', '$attrs', function ngbssListPagination($scope, $element, $attrs) {

                $scope.$watch('Parameters', function (nv) {
                    if (parseInt(nv.total) > 0 || (nv.displayOverallTotal == false && nv.retrievedRecordCount)) {
                        $element.show();
                    } else {
                        $element.hide();
                    }
                }, true);

                function isMaxReached() {

                    if($scope.Parameters.moreDataYn == false) {
                        return true;
                    }

                    return $scope.Parameters.retrievedRecordCount < $scope.Parameters.limit;
                }

                function calculateLastCount() {
                    var lastCount = 0;
                    if ($scope.Parameters.limit + $scope.Parameters.offset > $scope.Parameters.total) {
                        lastCount = $scope.Parameters.total;
                    } else {
                        lastCount = $scope.Parameters.limit + $scope.Parameters.offset;
                    }
                    return lastCount;
                }

                function setPaginationDescription() {
                    if ($scope.Parameters.displayOverallTotal != undefined && $scope.Parameters.displayOverallTotal == false) {
                        if (isMaxReached()) {
                            $scope.paginationDescription = ($scope.Parameters.offset + 1) + '-' + ($scope.Parameters.offset + $scope.Parameters.retrievedRecordCount);
                        } else {
                            $scope.paginationDescription = ($scope.Parameters.offset + 1) + '-' + ($scope.Parameters.offset + $scope.Parameters.limit ) + '+';
                        }
                    } else {
                        $scope.paginationDescription = ($scope.Parameters.offset + 1) + '-' + calculateLastCount() + '/' + $scope.Parameters.total;
                    }
                }

                function isPreviousEnabled() {
                    var enabled = false;
                    if ($scope.Parameters.offset == 0) {
                        enabled = false;
                    } else if ($scope.Parameters.displayOverallTotal != undefined && $scope.Parameters.displayOverallTotal == false) {
                        enabled = true;
                    } else {
                        enabled = true;
                    }
                    return enabled;
                }

                function isNextEnabled() {
                    var enabled = false;
                    if ($scope.Parameters.displayOverallTotal != undefined && $scope.Parameters.displayOverallTotal == false) {
                        if (isMaxReached()) {
                            enabled = false;
                        } else {
                            enabled = true;
                        }
                    } else if (($scope.Parameters.limit + $scope.Parameters.offset) >= $scope.Parameters.total) {
                        enabled = false;
                    } else {
                        enabled = true;
                    }
                    return enabled;
                }

                function setPreviousButtonStatus() {
                    if (isPreviousEnabled()) {
                        $scope.Parameters.previousEnabled = true;
                    } else {
                        $scope.Parameters.previousEnabled = false;
                    }
                }

                function setNextButtonStatus() {
                    if (isNextEnabled()) {
                        $scope.Parameters.nextEnabled = true;
                    } else {
                        $scope.Parameters.nextEnabled = false;
                    }
                }

                setPaginationDescription();
                setPreviousButtonStatus();
                setNextButtonStatus();

                $scope.$watch('Parameters', function (nv, ov) {
                    if (nv === ov) return false;

                    setPaginationDescription();
                    setPreviousButtonStatus();
                    setNextButtonStatus();
//                    console.log(nv.offset + "::" + ov.offset);
                    if (nv.displayOverallTotal != undefined && nv.displayOverallTotal == false) {
                    } else {
                        if (nv.offset < 0 || nv.offset > nv.total) {
                            nv.offset = ov.offset;
                            return false;
                        }
                    }
                }, true);

                $scope.doPrev = function () {
                    if ($scope.Parameters.offset === 0) {
                        return false;
                    }
                    $scope.Parameters.offset -= $scope.Parameters.limit;
                    $scope.SearchCallback.fn()
                };
                $scope.doNext = function () {
                    if ($scope.Parameters.displayOverallTotal != undefined && $scope.Parameters.displayOverallTotal == false) {
                        if (isMaxReached()) {
//                            if ($scope.Parameters.retrievedRecordCount == 0) {
//                                $scope.Parameters.offset -= ($scope.Parameters.limit * 2);
//                            } else {
                            return false;
//                            }
                        }
                    } else {
                        if ((parseInt($scope.Parameters.limit) + parseInt($scope.Parameters.offset)) >= parseInt($scope.Parameters.total)) {
                            return false;
                        }
                    }
                    $scope.Parameters.offset += $scope.Parameters.limit;
                    $scope.SearchCallback.fn();
                };

//                $element.find('ul').bind('focusout', function ($event) {
//                    var event = $event || $window.event,
//                        target = event.target || (event.srcElement || document);
//                    $element.find('ul').removeClass('open');
//                });
//                $element.find('ul').bind('focusin', function ($event) {
//                    var event = $event || $window.event,
//                        target = event.target || (event.srcElement || document);
//                    $element.find('ul').addClass('open');
//                });

                $element.find('p').bind('focusin', function ($event) {
                    var event = $event || $window.event,
                        target = event.target || (event.srcElement || document);
//                    $element.find('ul').addClass('open');
                    $element.find('ul').focusin();
                });

                $scope.doClickLimitBox = function doClickLimitBox($event) {
                    var event = $event || $window.event,
                        target = event.target || (event.srcElement || document);

                    if (target.text == undefined) {
                        return false;
                    }
                    $scope.Parameters.limit = +target.text;
                    $scope.Parameters.offset = 0;
                    if (event.preventDefault) event.preventDefault();
                    if (event.returnValue) event.returnValue = false;

                    $element.blur();
                    $element.find('ul').removeClass('open');

                    $scope.SearchCallback.fn();

                    return false;
                };
                $element.find('.pagingInfo').on('click', function () {
                    $element.find('.pagingScopePop').toggleClass('open');
                });
                $element.on('focusout', function (evt) {
                    if (!$element.find('.pagingScopePop').is(':hover')) {
                        $element.find('.pagingScopePop').removeClass('open');
                    }
                });
            }]
        };
    }]);
});