'use strict';

define(function () {

    var app = angular.module('commonModule');

    app.directive('ngbssSlider', function () {
        return {
            templateUrl: 'modules/common/directives/ngbss-slider.html',
            restrict: 'C',
            require: 'ngModel',
            scope: {
                ngModel: '=',
                min: '=',
                max: '=',
                step: '=',
                unit: '=',
                ngValue: '=',
                enabledModel: '=',
                disabledModel: '=',
                disabledYn: '='
            },
            replace: true,
            controller: ['$scope', function SliderCtrl($scope) {
                if (!$scope.ngModel) {
                    $scope.ngModel = 0;
                }

                if (angular.isUndefined($scope.max)) {
                    $scope.max = 10;
                }

                if (angular.isUndefined($scope.min)) {
                    $scope.min = 0;
                }

                if (angular.isDefined($scope.unit) && typeof $scope.unit == 'object') {
                    $scope.dataList = $scope.unit;
                } else {
                    $scope.dataList = {};
                    var unit = angular.isDefined($scope.unit) ? $scope.unit : 'sec';
                    for (var i = $scope.min; i <= $scope.max; i++) {
                        $scope.dataList[i + ''] = i + ' ' + unit;
                    }
                }

                if (angular.isUndefined($scope.step)) {
                    $scope.step = 1;
                }

                var maxRuler = 25;
                var getRuler = function (gap) {
                    if (gap > maxRuler) {
                        return maxRuler;
                    }
                    return gap;
                };

                $scope.ruler = getRuler($scope.max - $scope.min);

                $scope.getDataList = function (cnt) {
                    return new Array(cnt);
                };

                $scope.isDisabled = function (disabledModel, enabledModel) {
                    var disabled = false;
                    if (angular.isDefined(disabledModel)) {
                        disabled = disabledModel;
                    } else if (angular.isDefined(enabledModel) && angular.isUndefined(disabledModel)) {
                        disabled = !enabledModel;
                    }
                    return disabled;
                }


            }],
            link: function postLink(scope, element, attrs) {
                function disabled() {
                    if (scope.disabledYn == true) {
                        return false;
                    }
                }

                element.on('mousedown', disabled);
                element.on('mousepress', disabled);
                element.on('click', disabled);

                element.on('change', function () {
                    if (angular.isDefined(scope.ngValue)) {
                        scope.$apply(scope['ngValue'] = scope.dataList[element.val()].replace(' sec', ''));
                    }
                })
            }
        };
    });

    app.directive('ngbssSliderObjectList', function () {
        return {
            templateUrl: 'modules/common/directives/ngbss-slider-object-list.html',
            restrict: 'AC',
            require: 'ngModel',
            scope: {
                ngModel: '=',
                min: '=',
                max: '=',
                step: '=',
                unit: '=',
                dataList: '=ngDataList',
                valueField: '=ngValueField',
                textField: '=ngTextField',
                ngValue: '=',
                enabledModel: '=',
                disabledModel: '=',
                disabledYn: '='
            },
            replace: true,
            controller: ['$scope', function SliderCtrl($scope) {
                $scope.bindModel = -1;

                if (angular.isUndefined($scope.max)) {
                    $scope.max = 10;
                }

                if (angular.isUndefined($scope.min)) {
                    $scope.min = 0;
                }

                if (angular.isUndefined($scope.valueField)) {
                    $scope.valueField = 'value';
                }

                if (angular.isUndefined($scope.textField)) {
                    $scope.textField = 'text';
                }

                if (angular.isUndefined($scope.step)) {
                    $scope.step = 1;
                }

                if (angular.isDefined($scope.ngModel)) {

//                        console.log('ngModel bind:', nv);
                    var index = -1;
                    for (var propKey in $scope.dataList) {
                        var data = $scope.dataList[propKey];
                        if (data[$scope.valueField] == $scope.ngModel) {
                            index = propKey;
                            break;
                        }
                    }
                    $scope.bindModel = index;
                }

                var maxRuler = 25;
                var getRuler = function (gap) {
                    if (gap > maxRuler) {
                        return maxRuler;
                    }
                    return gap;
                };

                if (angular.isUndefined($scope.dataList)) {
                    $scope.ruler = getRuler($scope.max - $scope.min);

                    if (angular.isDefined($scope.unit) && typeof $scope.unit == 'object') {
                        $scope.dataList = $scope.unit;
                    } else {
                        $scope.dataList = {};
                        var unit = angular.isDefined($scope.unit) ? $scope.unit : 'sec';
                        for (var i = $scope.min; i <= $scope.max; i++) {
                            $scope.dataList[i + ''] = i + ' ' + unit;
                        }
                    }
                } else {
                    var cnt = -1;
                    for (var key in $scope.dataList) {
                        cnt++;
                    }
                    $scope.ruler = getRuler(cnt);
                }

                $scope.getDataList = function (cnt) {
                    return new Array(cnt);
                };

                $scope.isDisabled = function (disabledModel, enabledModel) {
                    var disabled = false;
                    if (angular.isDefined(disabledModel)) {
                        disabled = disabledModel;
                    } else if (angular.isDefined(enabledModel) && angular.isUndefined(disabledModel)) {
                        disabled = !enabledModel;
                    }
                    return disabled;
                };

                $scope.$watch('bindModel', function (nv, ov) {
                    if (nv == undefined || nv == ov) {
                        return;
                    }

//                    console.log('bindModel[' + $scope.bindModel + ']:', $scope.dataList[$scope.bindModel]);
                    $scope.ngModel = $scope.dataList[$scope.bindModel][$scope.valueField];

                }, true);

                if (angular.isDefined($scope.ngModel)) {
                    var index = -1;
                    for (var propKey in $scope.dataList) {
                        var data = $scope.dataList[propKey];
                        if (data[$scope.valueField] == $scope.ngModel) {
                            index = propKey;
                            break;
                        }
                    }
                    $scope.bindModel = index;
                }

                $scope.$watch('ngModel', function (nv, ov) {
					console.log('ngModel', nv, ov, $scope.bindModel);
                    if (nv == undefined || nv == ov) {
                        return;
                    }
//                    console.log('bindModel:', $scope.bindModel, ' nv:', nv);
                    if (ov == undefined && nv != undefined && $scope.bindModel == -1) {
//                        console.log('ngModel bind:', nv);
                        var index = -1;
                        for (var propKey in $scope.dataList) {
                            var data = $scope.dataList[propKey];
                            if (data[$scope.valueField] == $scope.ngModel) {
                                index = propKey;
                                break;
                            }
                        }
                        $scope.bindModel = index;
                    } else if($scope.bindModel > -1 && ov != nv) {
						var index = -1;
						for (var propKey in $scope.dataList) {
							var data = $scope.dataList[propKey];
							if (data[$scope.valueField] == $scope.ngModel) {
								index = propKey;
								break;
							}
						}
						$scope.bindModel = index;
					}

                }, true);

            }],
            link: function postLink(scope, element, attrs) {
                function disabled() {
                    if (scope.disabledYn == true) {
                        return false;
                    }
                }

                element.on('mousedown', disabled);
                element.on('mousepress', disabled);
                element.on('click', disabled);

                element.on('change', function () {
                    if (angular.isDefined(scope.ngValue)) {
                        scope.$apply(scope['ngValue'] = scope.dataList[element.val()][scope.valueField].replace(' sec', ''));
                    }
                })
            }
        };
    });
});
