'use strict';
define(function () {
	var app = angular.module('biostarWebApp');
	app.register.directive('ngbssUserAuthModeList', function () {
		return {
			templateUrl: 'modules/common/directives/ngbss-user-auth-mode-list.html',
			restrict: 'A',
			require: '?ngModel',
			scope: {
				select: '&',
				ngModel: '='
			},
			replace: false,
			controller: ['$rootScope', '$q', '$scope', '$log', 'API', 'Const', 'Util', 'MessageProperties', 'MessageService', function ngbssUserAuthModeList($rootScope, $q, $scope, $log, API, Const, Util, MessageProperties, MessageService) {

				var currUserId;
				$scope.USER_AUTH_MODE_LIMIT = Const.USER_AUTH_MODE_LIMIT;

				function getCurrUserId(userId) {
					currUserId = userId;
				}

				$scope.$emit('getUserId', getCurrUserId);

				if (angular.isUndefined($scope.ngModel)) {
					$scope.ngModel = [];
				}

				$scope.hasMultiAuth = function (auth_mode) {
					if ((auth_mode.first && auth_mode.first.length == 2)
						|| (auth_mode.second && auth_mode.second.length == 2)
						|| (auth_mode.third && auth_mode.third.length == 2)) {
						return 'A';
					}
					return '';
				};
				$scope.isSingle = function (arr_auth_mode) {
					return arr_auth_mode && arr_auth_mode.length == 1;
				};
				$scope.hasSecond = function (auth_mode) {
					return auth_mode.second && auth_mode.second.length > 0;
				};
				$scope.hasThird = function (auth_mode) {
					return auth_mode.third && auth_mode.third.length > 0;
				};

				$scope.selectedAuthModeIndex = undefined;
				$scope.editAuthMode = function ($index) {
					$scope.selectedAuthModeIndex = $index;
					$scope.doOpenAddAuthModeDlg();
				};
				$scope.delAuthMode = function ($index) {
					var defer = $q.defer();
					MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
						if (confirmYn) {
							$scope.ngModel.splice($index, 1);
							if (!$scope.$root.$$phase) $scope.$apply();
							defer.resolve(true);
						} else {
							defer.resolve(false);
							return false;
						}
					});

					return defer.promise;
				};
				/**
				 * @param schedule_name
				 * @param auth_mode
				 */
				$scope.getText = function (schedule_name, auth_mode) {
					if (schedule_name) {
						return schedule_name;
					} else {
						var str = auth_mode.first.join('/');
						str += (auth_mode.second ? ' + ' + auth_mode.second.join('/') : '');
						str += (auth_mode.third ? ' + ' + auth_mode.third.join('/') : '');
						return str;
					}
				};

				var authModePreset = angular.copy(Const.OPERATION_MODE);

				authModePreset.isHasAuthItem = function (authItemContainer) {
					var authItemContainerHasOR = authItemContainer.indexOf("OR".toUpperCase()) > -1;

					for (var i = 0; i < this.length; i++) {
						var auth_item_cnt = 0;
						if (this[i].indexOf('OR'.toUpperCase()) > -1 && !authItemContainerHasOR) {
							continue;
						}
						for (var j = authItemContainer.length; j--;) {
							if (this[i].indexOf(authItemContainer[j].toUpperCase()) > -1) {
								auth_item_cnt++;
							} else {
								auth_item_cnt--;
							}
						}
						if (auth_item_cnt == authItemContainer.length && authItemContainer.length > 1) {
							return i;
						}
					}
					return 255;
				};

				$scope.makeAuthModeList = function (item) {
					var mode = item.operation_mode,
						regExp = /([A-Z_]+)/g;
					regExp.exec();

					if (angular.isUndefined(mode)) {
						mode = item.operation_mode;
					}

					authModePreset[mode].match(regExp);

					var result = RegExp.$1.split(Const.OPERATION_MODE_SPLITTER);

					var _html = [];
					_html.push('<div class="authModeType');
					if (result.length > 2) {
						_html.push('A');
					}
					_html.push('">');

					for (var i = 0, len = result.length; i < len; i++) {
						var _auth_item = result[i].toLowerCase();
						if (_auth_item == 'only') {
							continue;
						} else if (_auth_item == 'or') {
							continue;
						} else if (_auth_item == 'fingerprint') {
							_auth_item = 'finger';
						} else if (_auth_item == 'pin') {
							_auth_item = 'password';
						}
						if (i == 1) {
							_html.push('<span class="plusA"></span>');
							if (result.indexOf('OR') > -1) {
								_html.push('<div>');
							}
						} else if (i == 2) {
							_html.push('<span class="plusB"></span>');
						}

						_html.push('<span class="' + _auth_item + '"></span>');

						if (i == 3) {
							_html.push('</div>');
						}
					}
					_html.push('</div>');
					return _html.join('');
				};


				/*Add New Auth Mode Dialog Logic*/

				$scope.auth_item_list = {};
				$scope.auth_item_list.part_one = [];
				$scope.auth_item_list.part_two = [];
				$scope.auth_item_list.part_three = [];
				$scope.auth_item = [];

				var authEditMode = 'create';
				$scope.doOpenAddAuthModeDlg = function ($event) {
					if ($scope.settingMode == 'user' && $scope.ngModel.length >= 3) {
						MessageService.displayWarningMessage(MessageProperties.get('validate.onlyThreeItemsAreAllowedForAuthenticationMode'));
						return false;
					}

					var addAuthModeDlg = {};
					addAuthModeDlg.url = 'addAuthModeTemplate';
					$scope.addAuthModeDlg = addAuthModeDlg;

					$scope.auth_item_list = {};
					$scope.auth_item_list.part_one = [];
					$scope.auth_item_list.part_two = [];
					$scope.auth_item_list.part_three = [];
					$scope.auth_item = [];
				};

				var loadAuthModeInfo = function (auth_mode_string) {
					var auth_mode_arr = auth_mode_string.replace('auth_mode_', '').split(Const.OPERATION_MODE_SPLITTER);

					for (var i = 0, len = auth_mode_arr.length; i < len; i++) {
						var item = auth_mode_arr[i].replace('biometric', 'finger'), part_el, containerHtml,
							elImg = angular.element('<img/>')
//                                .attr('src', 'images/contents/icon_auth_' + item + '_drag.png')
								.attr('src', 'images/contents/icon_auth_' + item + '.png')
								.attr('alt', item)
								.on('mousedown', function (event) {
									return false;
								}),
							delete_div = angular.element('<div/>').addClass('delete').on('click', deleteAuthItem),
							deleteImg = angular.element('<img/>', {
								'data-item-type': item
							}).addClass('delete').on('click', deleteAuthItem).attr('src', 'images/contents/btn_close_select_door.png');

						if (i == 0) {
							$scope.auth_item_list.part_one.push(item);
							part_el = angular.element('#part_one_0');
						} else if (i == 1 && item != 'only') {
							$scope.auth_item_list.part_two.push(item);
							part_el = angular.element('#part_two_0');
						} else if (i == 2 && item != 'or') {
							$scope.auth_item_list.part_three.push(item);
							part_el = angular.element('#part_three_0');
						} else if (i == 3) {
							$scope.auth_item_list.part_two.push(item);
							part_el = angular.element('#part_two_1');
						}
						if (item == 'only' || item == 'or') {
							continue;
						}
//                        part_el.prepend(elImg).append(delete_div);
						containerHtml = angular.element('<div/>', {
								class: 'authModeDragPannel'
							}
						).append(elImg).append('<span ng-label="user.authMode.' + item + '">' + MessageProperties.get('user.authMode.' + item) + '</span>');
						part_el.prepend(containerHtml).append(deleteImg);
						$scope.auth_item.push(item);
					}
				};

				var deleteAuthItem = function (event) {

					var target = event.target || (event.srcElement || document);

					var itemType = angular.element(target).attr('data-item-type');

					var auth_mode_string = $scope.AuthModeCombinationResult.toLowerCase().replace('_only', ''),
						auth_mode_array = auth_mode_string.split(Const.OPERATION_MODE_SPLITTER);

					var orIdx = auth_mode_array.indexOf('or');

					if (orIdx > -1) {
						auth_mode_array.splice(orIdx, 1);
					}

					var idx = auth_mode_array.indexOf(itemType);


					auth_mode_array.splice(idx, 1);
					console.log(auth_mode_array);
					auth_mode_string = auth_mode_array.join(Const.OPERATION_MODE_SPLITTER);
					console.log(auth_mode_string);
					var nestedIdx = -1;
					for (var i = 0; i < Const.OPERATION_MODE.length; i++) {
						if (Const.OPERATION_MODE[i].indexOf(auth_mode_string.toUpperCase()) == 0) {
							nestedIdx = i;
						}
					}

					$scope.auth_item = [];
					$scope.auth_item_list.part_one = [];
					$scope.auth_item_list.part_two = [];
					$scope.auth_item_list.part_three = [];
					angular.element('.part_one > div.item').html('');
					angular.element('.part_two > div.item').html('');
					angular.element('.part_three > div.item').html('');

					if (auth_mode_array.length > 0 && nestedIdx > -1) {

						if (auth_mode_array.length == 1) {
							auth_mode_array.push('only');
						}

						auth_mode_string = auth_mode_array.join(Const.OPERATION_MODE_SPLITTER);

						loadAuthModeInfo(auth_mode_string);
					} else {

					}

					if (!$scope.$root.$$phase) $scope.$apply();
				};

				$scope.doAfterOpenAddAuthModeDlg = function () {
					var options = {};
					options.modal = true;
					options.shadow = false;
					options.title = MessageProperties.get('device.addNewAuthMode');
					options.width = 624;
					options.height = 530;

					options.onClose = function () {
						$scope.addAuthModeDlg = {};
						$scope.selectedAuthModeIndex = undefined;

						angular.element('#addAuthModeDlg').dialog('destroy');
						if (!$scope.$root.$$phase) $scope.$apply();
					};
					angular.element('#addAuthModeDlg').dialog(options);

					$scope.selectedScheduleItem = undefined;

					if (angular.isDefined($scope.selectedAuthModeIndex)) {
						var auth_mode = $scope.ngModel[$scope.selectedAuthModeIndex],
							auth_mode_string = authModePreset[auth_mode.operation_mode].toLowerCase();

						if (angular.isUndefined(auth_mode.operation_mode)) {
							auth_mode_string = authModePreset[auth_mode.operation_mode].toLowerCase();
						}
						$scope.selectedScheduleItem = auth_mode.schedule_id;

						loadAuthModeInfo(auth_mode_string);

						$scope.AuthModeCombinationResult = auth_mode_string.toUpperCase();

						authEditMode = 'update';
					} else {
						$scope.AuthModeCombinationResult = '';
						authEditMode = 'create';
					}

					var dragSrcEl = null;

					angular.element('.drag > li').on('dragstart', function ($event) {
						$event.originalEvent.dataTransfer.effectAllowed = 'move';
						dragSrcEl = this.innerHTML;
					});

					angular.element('div.drop div.item')
						.on('dragstart', function ($event) {
							return false;
						}).on('dragover', function ($event) {
							if ($event.originalEvent.preventDefault) {
								$event.originalEvent.preventDefault(); // Necessary. Allows us to drop.
							}
							$event.originalEvent.dataTransfer.dropEffect = 'move';  // See the section on the DataTransfer object.
							return false;
						}).on('drop', function ($event) {
							if ($event.originalEvent.stopPropagation) {
								$event.originalEvent.stopPropagation(); // Stops some browsers from redirecting.
							}

							var _this = this
								, thisClassName = _this.parentElement.className
								, class_regExp = /(part_[a-z]+)/g
								, class_name = thisClassName.match(class_regExp)
								, part_name = RegExp.$1
								, innerHTML = dragSrcEl
								, item_regExp = /alt="([a-z]+)"/g
								, item = innerHTML.match(item_regExp)
								, item_name = RegExp.$1;

							var html_reqexp = /(src="[a-z_ /]+)/g
								, _html = innerHTML.replace(html_reqexp, '$1').replace('draggable="true"', 'onmousedown="return false;"');

							var delete_div = angular.element('<div/>').addClass('delete').bind('click', deleteAuthItem),
								deleteImg = angular.element('<img/>', {
									'data-item-type': item.toString().replace(item_regExp, '$1')
								}).addClass('delete').on('click', deleteAuthItem).attr('src', 'images/contents/btn_close_select_door.png');

							if (part_name.indexOf('one') > -1 && $scope.auth_item_list[part_name].length >= 1) {
								return false;
							} else if (_this.id.indexOf('part_two_0') > -1 && $scope.auth_item_list[part_name].length >= 1) {
								return false;
							} else if ($scope.auth_item_list[part_name].length >= 2) {
								return false;
							}

//                                angular.element(_this).prepend(_html).append(delete_div);
							angular.element(_this).prepend(_html).append(deleteImg);
							$scope.auth_item_list[part_name].push(item_name);
							$scope.auth_item.push(item_name);

							if (!$scope.$root.$$phase) $scope.$apply();

							return false;
						});

					$scope.$watch('auth_item_list', function (nv, ov) {
						if (nv !== ov) {
							var ret = [],
								regExp = /([A-Z]+)/g;

							if (nv.part_one.length > 0) {
								ret.push(nv.part_one.join('_or_'));
							}
							if (nv.part_two.length > 0) {
								ret.push(nv.part_two.join('_or_'));
							}
							if (nv.part_three.length > 0) {
								ret.push(nv.part_three.join('_or_'));
							}

							if (ret.length == 1) {
								ret.push('only');
							}

							$scope.AuthModeCombinationResult = ret.join(Const.OPERATION_MODE_SPLITTER).toUpperCase();
							console.log($scope.AuthModeCombinationResult);

						}
					}, true);

				};

				$scope.doSaveAuthModeCombination = function () {
					var operationMethod = {"ID": 0, "CARD": 1, "BIOMETRIC": 2, "FINGER": 2, "FACE": 2},
						AuthItem = authModePreset.isHasAuthItem($scope.AuthModeCombinationResult.split(Const.OPERATION_MODE_SPLITTER)),
						options = angular.element('#addAuthModeDlg').dialog('options'),
						auth_data = {
							'operation_mode': AuthItem,
							'user_id': currUserId
						};

					var correctCombineYn = Const.OPERATION_MODE.indexOf($scope.AuthModeCombinationResult) > -1;

					if ($scope.AuthModeCombinationResult == undefined || $scope.AuthModeCombinationResult.length == 0) {
						$scope.delAuthMode($scope.selectedAuthModeIndex).then(function success(data) {
							if (data) {
								options.onClose();
							}
						});
						return false;
					}

					if (correctCombineYn) {
						auth_data.operation_method = operationMethod[$scope.AuthModeCombinationResult.split(Const.OPERATION_MODE_SPLITTER)[0]];
					} else {
						MessageService.displayWarningMessage(MessageProperties.get('validate.InvalidAuthentication'));
						return false;
					}


					if ($scope.AuthModeCombinationResult.length <= 0) {
						MessageService.displayConfirmMessage(MessageProperties.get('validate.cancelEdit'), function (r) {
							if (r) {
								options.onClose();
							}
							return false;
						});
						return false;
					}
					var checkExistAuthModeIdx = -1,
						likeAuthType = false,
						duplicatedAuthTypeMsg = MessageProperties.get('validate.alreadyItemAndOverwrite');

					if (angular.isUndefined($scope.ngModel)) {
						$scope.ngModel = [];
					}

					for (var i = $scope.ngModel.length; i--;) {
						var item = $scope.ngModel[i];
						var forRegisteringPrefix = Const.OPERATION_MODE[AuthItem].split(Const.OPERATION_MODE_SPLITTER)[0],
							registeredPrefix = Const.OPERATION_MODE[item.operation_mode].split(Const.OPERATION_MODE_SPLITTER)[0];
						if (item.operation_mode == AuthItem && i != $scope.selectedAuthModeIndex) {
							checkExistAuthModeIdx = i;
							break;
						} else if (forRegisteringPrefix == registeredPrefix) {
							likeAuthType = true;
							checkExistAuthModeIdx = i;
							break;
						}
					}

					if (checkExistAuthModeIdx == -1) {
						if (authEditMode == 'update') {
							auth_data.index = $scope.ngModel[$scope.selectedAuthModeIndex].index;
							$scope.ngModel[$scope.selectedAuthModeIndex] = angular.copy(auth_data);
						} else {
							auth_data.index = $scope.ngModel.length;
							$scope.ngModel.push(angular.copy(auth_data));
						}
						options.onClose();
					} else {
						auth_data.index = $scope.ngModel[checkExistAuthModeIdx].index;
						$scope.ngModel[checkExistAuthModeIdx] = auth_data;

						if ($scope.selectedAuthModeIndex != undefined
							&& checkExistAuthModeIdx != $scope.selectedAuthModeIndex) {
							$scope.ngModel.splice($scope.selectedAuthModeIndex, 1);
						}

						options.onClose();

						return false;
					}
				};

				$scope.doCloseAddAuthModeDlg = function () {
					var options = angular.element('#addAuthModeDlg').dialog('options');
					options.onClose();
				};


			}],
			link: function postLink(scope, element, attrs, ngModel) {
				if (!ngModel) {
					return;
				}

				ngModel.$render = function () {

				};


			}
		};
	});
});
