'use strict';

define(function () {
    var app = angular.module('biostarWebApp');
    app.register.directive('ngbssDeviceOperatorList', ['$rootScope', 'UserAgent', '$timeout', 'Const', 'MessageService', 'MessageProperties', function ($rootScope, UserAgent, $timeout, Const, MessageService, MessageProperties) {
        return {
            restrict: 'EA',
            replace: true,
            scope: {
                deviceOperators: '=ngModel',
                isBatchEdit: '=isBatchEdit',
                invalidUsers: '=',
                disabledYn: '='
            },
            templateUrl: 'modules/common/directives/ngbssDeviceOperatorList.html',
            controller: ['$scope', function ($scope) {
                function DeviceOperator(level) {
                    this.user_id = undefined;
                    this.level = level;
                }

//                level (integer, optional) = ['0 = Admin' or '1 = Config' or '2 = User']

                $scope.LEVELS = {};
                $scope.LEVELS.ADMIN = '1';
                $scope.LEVELS.CONFIG = '2';
                $scope.LEVELS.USER = '3';

                $scope.LEVEL_TEXT = [
                    {level: '1', name: MessageProperties.get('device.operatorLevel.all')},
                    {level: '3', name: MessageProperties.get('device.operatorLevel.user')},
                    {level: '2', name: MessageProperties.get('device.operatorLevel.configuration')}
                ];

                $scope.LevelsOperators = {};
                $scope.LevelsOperators[$scope.LEVELS.ADMIN] = [];
                $scope.LevelsOperators[$scope.LEVELS.CONFIG] = [];
                $scope.LevelsOperators[$scope.LEVELS.USER] = [];

                $scope.usersList = [];
                $scope.filtered = [];


                $scope.addOperator = function (level) {
                    if ($scope.disabledYn) {
                        return false;
                    }
                    $scope.LevelsOperators[level].push({});
                };

                $scope.deleteOperators = function (level, operator) {
                    if ($scope.disabledYn) {
                        return false;
                    }

                    MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                        if (confirmYn) {
                            var idx = $scope.LevelsOperators[level].indexOf(operator);
                            $scope.LevelsOperators[level].splice(idx, 1);
                            var idxDeviceOperators = -1;
                            for (var i = 0; i < $scope.deviceOperators.length; i++) {
                                if (operator.user_id && operator.user_id.user_id &&
                                    operator.user_id.user_id == $scope.deviceOperators[i].user_id.user_id) {
                                    idxDeviceOperators = i;
                                    break;
                                }
                            }
                            $scope.deviceOperators.splice(idxDeviceOperators, 1);
                        }
                    });
                };

                $scope.checkValidUser = function () {
                    var userIDs = [];
                    $scope.invalidUsers = [];
                    for (var i = 0; i < $scope.deviceOperators.length; i++) {
                        if ($scope.deviceOperators[i].user_id == undefined) {
                            continue;
                        }
                        var crrUserID = $scope.deviceOperators[i].user_id.user_id;
                        if (userIDs.contains(crrUserID)) {
                            if (!$scope.invalidUsers.contains(crrUserID)) {
                                $scope.invalidUsers.push(crrUserID);
                            }
                        } else {
                            userIDs.push(crrUserID);
                        }
                    }
                };

                $scope.$watch('LevelsOperators', function (nv, ov) {
                    if (nv != ov) {
                        $scope.deviceOperators = [];
                        for (var level in nv) {
                            for (var i = nv[level].length; i--;) {
                                var operator = nv[level][i];
                                $scope.deviceOperators.push({'user_id': operator, 'level': level});
                            }
                        }
                        console.log($scope.deviceOperators);
                        $scope.checkValidUser();
                    }
                }, true);

                $scope.markInvalidUser = function (user) {
                    if (user && $scope.invalidUsers && $scope.invalidUsers.length > 0) {
                        return $scope.invalidUsers.contains(user.user_id);
                    } else {
                        return false;
                    }
                };

                $scope.init = function () {
                    $scope.$on(Const.UI_EVENT_TYPE.DEVICE_INFO_LOADED, function () {
                        console.log(Const.UI_EVENT_TYPE.DEVICE_INFO_LOADED + " received");
                        if ($scope.deviceOperators) {
                            for (var i = 0; i < $scope.deviceOperators.length; i++) {
                                var operator = $scope.deviceOperators[i];
                                $scope.LevelsOperators[operator.level].push(operator.user_id);
                            }
                        } else {
                            $scope.deviceOperators = [];
                        }
                    });
                };
            }],
            link: function postLink($scope, $element, $attrs, $timeout) {

            }
        };
    }]);
});