'use strict';

define(function () {
    var app = angular.module('biostarWebApp');
    app.register.directive('ngbssOutputSignalModule', ['Const', 'Util', 'MessageProperties', function (Const, Util, MessageProperties) {
        return {
            restrict: 'EA',
            replace: true,
            scope: {
                outputSignals: '=ngModel',
                disabledYn: '='
            },
            templateUrl: 'modules/common/directives/ngbssOutputSignalModule.html',
            controller: ['$scope', '$filter', '$rootScope', function ngbssOutputSignalModule($scope, $filter, $rootScope) {
                $scope.number = /^[-0-9]+$/;
                $scope.selectedRow = 0;

                $scope.safeApply = function (fn) {
                    var phase = this.$root.$$phase;
                    if (phase == '$apply' || phase == '$digest') {
                        if (fn && (typeof(fn) === 'function')) {
                            fn();
                        }
                    } else {
                        this.$apply(fn);
                    }
                };

                var BuzzerSignal = {
                        tone: '0',
                        fadeout: 'false',
                        duration: 0,
                        delay: 0
                    },
                    BuzzerOutputSignal = {
                        count: 1,
                        signals: [BuzzerSignal, BuzzerSignal, BuzzerSignal]
                    },
                    LEDSignal = {
                        color: '0',
                        duration: 0,
                        delay: 0
                    },
                    LEDOutputSignal = {
                        count: 1,
                        signals: [LEDSignal, LEDSignal, LEDSignal]
                    },
                    DeviceOutputSignal = {
                        status: undefined,
                        led: LEDOutputSignal,
                        buzzer: BuzzerOutputSignal
                    };

                $scope.colorSignalCount = 0;

                $scope.toneSignalCount = 0;

                $scope.infiniteLED = undefined;

                $scope.infiniteBuzzer = undefined;

                $scope.STATUS_TEXT = {
                    '0': MessageProperties.get('device.displaySound.event.normal'),
                    '1': MessageProperties.get('device.displaySound.event.locked'),
                    '2': MessageProperties.get('device.displaySound.event.rtcError'),
                    '3': MessageProperties.get('device.displaySound.event.waitingInput'),
                    '4': MessageProperties.get('device.displaySound.event.waitingDhcp'),
                    '5': MessageProperties.get('device.displaySound.event.scanFinger'),
                    '6': MessageProperties.get('device.displaySound.event.scanCard'),
                    '7': MessageProperties.get('device.displaySound.event.authSuccess'),
                    '8': MessageProperties.get('device.displaySound.event.authFail'),
                    '9': MessageProperties.get('device.displaySound.event.authDuress')
                };


                $scope.statusList = [];
                for (var status in $scope.STATUS_TEXT) {
                    if ($scope.STATUS_TEXT.hasOwnProperty(status)) {
                        $scope.statusList.push({'status': status, 'statusName': $scope.STATUS_TEXT[status]});
                    }
                }

                $scope.COLORS = {
                    '0': MessageProperties.get("device.info.ledBuzzer.color.off"),
                    '1': MessageProperties.get("device.info.ledBuzzer.color.red"),
                    '2': MessageProperties.get("device.info.ledBuzzer.color.yellow"),
                    '3': MessageProperties.get("device.info.ledBuzzer.color.green"),
                    '4': MessageProperties.get("device.info.ledBuzzer.color.cyan"),
                    '5': MessageProperties.get("device.info.ledBuzzer.color.blue"),
                    '6': MessageProperties.get("device.info.ledBuzzer.color.magenta"),
                    '7': MessageProperties.get("device.info.ledBuzzer.color.white")
                };

                $scope.TONES = {
                    '0': MessageProperties.get("device.info.ledBuzzer.tone.off"),
                    '1': MessageProperties.get("device.info.ledBuzzer.tone.low"),
                    '2': MessageProperties.get("device.info.ledBuzzer.tone.middle"),
                    '3': MessageProperties.get("device.info.ledBuzzer.tone.high")
                };

                $scope.getBuzzerInfo = function (buzzer) {
                    return buzzer;
                };

                function initOutputSignal(index, status) {
                    if (!$scope.outputSignals) {
                        $scope.outputSignals = [];
                    }
                    if (!$scope.outputSignals[index]) {
                        var outputSignal = angular.copy(DeviceOutputSignal);
//                        outputSignal.statusText = $scope.STATUS_TEXT[index];
//                        outputSignal.index = index;
                        outputSignal.status = status;
                        $scope.outputSignals[index] = outputSignal;
                    }
                }

                $scope.doAddOutputSignal = function (index) {
                    initOutputSignal(index);

                    $scope.outputSignals = $filter('orderObjectBy')($scope.outputSignals, 'status');

                    $scope.selectedRow = index;
                    $scope.deviceOutputSignal = $scope.outputSignals[index];
                };

                $scope.msecBlur = function (bindModel, key) {
                    var elem = angular.element(event.target || event.srcElement);
                    var val = elem.val();
                    var min = elem.attr('min');
                    var max = elem.attr('max');

                    if (val == undefined || val == '') {
                        val = 0;
                    } else if (parseInt(val) < parseInt(min)) {
                        val = parseInt(min);
                    } else if (parseInt(val) > parseInt(max)) {
                        val = parseInt(max);
                    } else {
                        val = parseInt(val);
                    }

                    $scope.safeApply(function () {
                        bindModel[key] = val;
                    });
                    return val;
                };

                $scope.infiniteBuzzerCountBlur = function (bindModel) {
                    if ($scope.msecBlur(bindModel) == -1) {
                        $scope.infiniteBuzzer = true;
                    }
                };

                $scope.infiniteLEDBlur = function (bindModel, key) {
                    if ($scope.msecBlur(bindModel, key) == -1) {
                        $scope.infiniteLED = true;
                    }
                };
//                $scope.$watch('outputSignals', function (nv, ov) {
//                    // this is used because outputSignal data from device is loaded later
//                    // it needs to watch if data gets set for the first time and initialize display sound data for user
//                        console.log('$watch(outputSignals)');
//                        console.log(ov);
//                        console.log(nv);
//                    if(ov == null && nv != null) {
//                        $scope.deviceOutputSignal = $scope.outputSignals[0];
//                    }
//                }, true);

                $scope.$watch('infiniteLED', function (nv, ov) {
                    if (ov == undefined || nv == ov) {
                        return false;
                    }
                    if (nv == true && $scope.deviceOutputSignal.led.count > -1) {
                        $scope.deviceOutputSignal.led.count = -1;
                    } else if (ov == true && nv == false && $scope.deviceOutputSignal.led.count == -1) {
                        $scope.deviceOutputSignal.led.count = 0;
                    }
                }, true);

                $scope.$watch('infiniteBuzzer', function (nv, ov) {
                    if (ov == undefined || nv == ov) {
                        return false;
                    }
                    if (nv == true && $scope.deviceOutputSignal.buzzer.count > -1) {
                        $scope.deviceOutputSignal.buzzer.count = -1;
                    } else if (ov == true && nv == false && $scope.deviceOutputSignal.buzzer.count == -1) {
                        $scope.deviceOutputSignal.buzzer.count = 0;
                    }
                }, true);


                $scope.$watch('deviceOutputSignal.led', function (nv, ov) {
                    if (nv !== ov) {
                        var i,
                            cnt = 0;
                        if (nv == undefined) {
                            return;
                        }
                        if (nv.signals != undefined) {
                            for (i = 0; i < nv.signals.length; i++) {
                                if ($scope.COLORS[nv.signals[i].color]) {
                                    cnt++;
                                }
                            }
                        }
                        console.log('deviceOutputSignal.led=' + $scope.selectedRow);
//                        if(cnt == 2 && $scope.deviceOutputSignal.led.signals.length == 2) {
//                            $scope.deviceOutputSignal.led.signals.push(angular.copy(LEDSignal));
//                        }
                        $scope.colorSignalCount = cnt - 1;
                    }
                }, true);

                $scope.$watch('deviceOutputSignal.buzzer', function (nv, ov) {
                    if (nv !== ov) {
                        var i,
                            cnt = 0;
                        if (nv == undefined) {
                            return;
                        }
                        if (nv.signals != undefined) {
                            for (i = 0; i < nv.signals.length; i++) {
                                if ($scope.TONES[nv.signals[i].tone]) {
                                    cnt++;
                                }
                            }
                        }
                        console.log('deviceOutputSignal.buzzer=' + $scope.selectedRow);
//                        if(cnt == 2 && $scope.deviceOutputSignal.buzzer.signals.length == 2) {
//                            $scope.deviceOutputSignal.buzzer.signals.push(angular.copy(BuzzerSignal));
//                        }
                        $scope.toneSignalCount = cnt - 1;
                    }
                }, true);

                $scope.doEditOutputSignalItem = function ($index, status) {
                    $scope.deviceOutputSignal = undefined;
                    $scope.infiniteLED = undefined;
                    $scope.infiniteBuzzer = undefined;
                    console.log('doEditOutputSignalItem');
//                    console.log($scope.outputSignals);
                    initOutputSignal($index, status);

                    for (var i = 0; i < $scope.outputSignals[$index].led.signals.length; i++) {
                        var tempSignal = $scope.outputSignals[$index].led.signals[i];
                        tempSignal.delay = parseInt(tempSignal.delay);
                        tempSignal.duration = parseInt(tempSignal.duration);
                    }
                    for (var i = 0; i < $scope.outputSignals[$index].buzzer.signals.length; i++) {
                        var tempSignal = $scope.outputSignals[$index].buzzer.signals[i];
                        tempSignal.delay = parseInt(tempSignal.delay);
                        tempSignal.duration = parseInt(tempSignal.duration);
                    }

                    $scope.selectedRow = $index;
                    $scope.deviceOutputSignal = $scope.outputSignals[$index];

                    if ($scope.deviceOutputSignal == undefined) {
                        $scope.doAddOutputSignal($index);
                    }

                    $scope.deviceOutputSignal.led.count = parseInt($scope.deviceOutputSignal.led.count);
                    $scope.deviceOutputSignal.buzzer.count = parseInt($scope.deviceOutputSignal.buzzer.count);

                    if ($scope.deviceOutputSignal.led != undefined && parseInt($scope.deviceOutputSignal.led.count) == -1) {
                        $scope.infiniteLED = true;
                    } else {
                        $scope.infiniteLED = false;
                    }

                    if ($scope.deviceOutputSignal.buzzer != undefined && parseInt($scope.deviceOutputSignal.buzzer.count) == -1) {
                        $scope.infiniteBuzzer = true;
                    } else {
                        $scope.infiniteBuzzer = false;
                    }
                };

                $scope.init = function () {
//                    $rootScope.$on('bindOutputSignals', function(event) {
//
//                    });
                    //if($scope.outputSignals != null && $scope.outputSignals.length > 0) {

                    //}

                    $scope.$on(Const.UI_EVENT_TYPE.DEVICE_INFO_LOADED, function () {
                        Util.log(Const.UI_EVENT_TYPE.DEVICE_INFO_LOADED + ' received');
                        $scope.doEditOutputSignalItem(0, '0');
                        $rootScope.$broadcast(Const.UI_EVENT_TYPE.DEVICE_INFO_INIT_COMPLETED);
                    });

                    $scope.$on(Const.UI_EVENT_TYPE.EVENT_TYPE_CLICKED, function (statusIndex, status) {
                        Util.log(Const.UI_EVENT_TYPE.EVENT_TYPE_CLICKED + ' received', 'statusIndex:' + statusIndex, 'status:' + status);
                        $scope.doEditOutputSignalItem(statusIndex, status);
                    });
                };

            }],
            link: function postLink($scope, $element, $attrs, $timeout) {

            }
        };
    }]);
});