'use strict';

define([
    "modules/accessControl/dialog/door-output-signal-add-dlg",
    'modules/accessControl/add-schedule'
], function () {
    var app = angular.module('biostarWebApp');
    app.register.directive('ngbssTriggerActionList', function () {
        return {
            templateUrl: 'modules/common/directives/ngbssTriggerActionList.html',
            restrict: 'A',
            require: '?ngModel',
            scope: {
                ngModel: '=',
                cbFunction: '=applyTriggerInfo',
                disabledYn: '='
            },
            replace: false,
            controller: ['$scope', '$rootScope', '$window', '$parse', '$attrs', 'API', 'Const', 'Util', 'DeviceAgent', 'MessageProperties', 'MessageService', 'ScheduleAgent',
                function ngbssTriggerActionList($scope, $rootScope, $window, $parse, $attrs, API, Const, Util, DeviceAgent, MessageProperties, MessageService, ScheduleAgent) {

                    $scope.hasSlave = false;

                    $scope.predefined_alarm_name_list = {}; // Pre-Defined Alarm name list(이름 조회용)
                    $scope.trigger_name_list = { // Trigger name list
                        '0': MessageProperties.get('triggerAction.input'),
                        '1': MessageProperties.get('triggerAction.preDefinedAlarm')
                    };
                    $scope.action_name_list = {
                        //'2': 'Sound',
                        '1': MessageProperties.get('triggerAction.output'),
                        '0': {
//                        '1': 'Generic input',
//                        '2': 'Emergency open',
                            '4': MessageProperties.get('triggerAction.releaseAllAlarm'),
                            '8': MessageProperties.get('triggerAction.rebootDevice'),
                            '16': MessageProperties.get('triggerAction.disableDevice')
                        }
                    };

                    $scope.safeApply = function (fn) {
                        var phase = this.$root.$$phase;
                        if (phase == '$apply' || phase == '$digest') {
                            if (fn && (typeof(fn) === 'function')) {
                                fn();
                            }
                        } else {
                            this.$apply(fn);
                        }
                    };

                    var TriggerAction = function () {
                        this.index = undefined;
                        this.input_device_id = undefined;
                        this.input_type = undefined;
                        this.input_port = undefined;
                        this.input_switch = 0;
                        this.input_duration = undefined;
                        this.input_schedule_id = undefined;
                        this.input_event_id = undefined;
                        this.output_device_id = undefined;
                        this.output_type = undefined;
                        this.output_function_mask = undefined;
                        this.output_port = undefined;
                        this.output_signal_id = 1;
                        this.output_priority = undefined;
                    };
                    $scope.TriggerAction = undefined;

                    $scope.page_mode = 'create';

                    $scope.$watch('TriggerAction', function (nv, ov) {
                        if (nv === ov) return false;
//                    console.log(nv);
                    }, true);

                    $scope.doOpenAddTriggerActionDlg = function () {
                        if ($scope.disabledYn) {
                            return false;
                        }
                        var addTriggerActionDlg = {};
                        addTriggerActionDlg.url = 'addTriggerActionTemplate';
                        $scope.addTriggerActionDlg = addTriggerActionDlg;
                    };

                    var addTriggerActionDlgOptions = {};
                    addTriggerActionDlgOptions.modal = true;
                    addTriggerActionDlgOptions.shadow = false;
                    addTriggerActionDlgOptions.title = MessageProperties.get('triggerAction.addTriggerNAction');
                    addTriggerActionDlgOptions.width = 1100;
                    addTriggerActionDlgOptions.height = 677;

                    addTriggerActionDlgOptions.onClose = function () {
                        $scope.addTriggerActionDlg = {};

                        $scope.safeApply(function () {
                            $scope.TriggerAction = undefined;
                            angular.element('#addTriggerActionDlg').dialog('destroy');
                        });
                    };
                    addTriggerActionDlgOptions.onOpen = function () {
                        angular.element('.relayComboBox').on('focusin', function () {
                            console.log('relayComboBox: focusin');
                            angular.element(event.target || event.srcElement).parents('.select').addClass('open');
                        });

                        angular.element('#addTriggerActionDlg, .window-mask').on('click', function (event) {
                            console.log('#addTriggerActionDlg: click');
                            if (angular.element(event.target || event.srcElement).parents('.select').length <= 0) {
                                angular.element('.select.open').blur();
                                angular.element('.select.open').removeClass('open');
                            }
                        });
                    };

                    $scope.predefined_alarm_list = [];
                    var loadPredefinedAlarmList = function () {
                        API.get('event_types').then(function (data) {
                            for (var i = data.EventTypeCollection.rows.length; i--;) {
                                var item = data.EventTypeCollection.rows[i];
                                if (item.alertable == "true" && item.category && item.category == Const.EVENT_TYPE_CATEGORY.DEVICE) {
                                    $scope.predefined_alarm_list.push(item);
                                    item.code = item.code;
                                    $scope.predefined_alarm_name_list[item.code] = item;
                                } else if (item.alertable == "true" &&
                                    (parseInt(item.code) >= 13312 && parseInt(item.code) <= 16640 )) {
                                    // 12288 System Reset은 제거 됨.
                                    item.description = item.description || '';
                                    $scope.predefined_alarm_list.push(item);
                                    $scope.predefined_alarm_name_list[item.code] = item;
                                }
                            }
                        });
                    };
                    loadPredefinedAlarmList();

                    $scope.doAfterOpenAddTriggerActionDlg = function () {

                        //loadPredefinedAlarmList();

                        angular.element('#addTriggerActionDlg').dialog(addTriggerActionDlgOptions);

                        if (!$scope.TriggerAction) {
                            $scope.page_mode = 'create';
                            $scope.TriggerAction = new TriggerAction();

                            $scope.TriggerAction.input_type = 1;
                            $scope.TriggerAction.output_type = 1;

                            $scope.TriggerAction.input_device_id = {
                                'id': $scope.ngModel.id,
                                'text': $scope.ngModel.name,
                                'isDevice': true,
                                'device_type_id': $scope.ngModel.device_type_id,
                                'deviceSpec': angular.copy($scope.deviceSpecData[$scope.ngModel.device_type_id.id])
                            };
                            $scope.TriggerAction.input_port = 0;
                            $scope.TriggerAction.output_device_id = {
                                'id': $scope.ngModel.id,
                                'text': $scope.ngModel.name,
                                'isDevice': true,
                                'device_type_id': $scope.ngModel.device_type_id,
                                'deviceSpec': angular.copy($scope.deviceSpecData[$scope.ngModel.device_type_id.id])
                            };
                            //$scope.TriggerAction.output_port = 0;
                        }
                    };

                    $scope.getNumArray = function (len) {
//                    console.log('getNumArray:'+len);
                        var ret_arr = [];
                        for (var i = 0; i < len; i++) {
                            ret_arr.push(i);
                        }
                        return ret_arr;
                    };

                    var onlyDigit = function (nv, ov) {
                        if (nv === ov) return false;

                        if (nv == null) {
                            var model = $parse(this.exp);
                            model.assign($scope, ov);
                            if (ov < 10) {
                                model.assign($scope, 0);
                            }
                        }
                    };

//                $scope.$watch('TriggerAction.input_duration', onlyDigit);

                    $scope.doSelectTriggerType = function ($event) {

                        var event = $event || $window.event;
                        var target = event.target || event.srcElement;


                        if (target.nodeName !== 'INPUT') {
                            return;
                        }

                        $scope.TriggerAction.input_port = undefined;

                        if ($scope.TriggerAction.input_type == 1) {
                            //loadPredefinedAlarmList();
                            $scope.TriggerAction.input_switch = undefined;
                            $scope.TriggerAction.input_duration = undefined;
                            $scope.TriggerAction.input_schedule_id = undefined;

                        } else if ($scope.TriggerAction.input_type == 0) {
                            $scope.TriggerAction.input_event_id = undefined;
                            $scope.TriggerAction.input_switch = 0;
                        }
                    };

                    $scope.doSelectActionType = function ($event) {
                        var event = $event || $window.event;
                        var target = event.target || event.srcElement;

                        if (target.nodeName !== 'INPUT') {
                            return;
                        }

                        if (angular.element(target).attr('ng-model').indexOf('output_type') > -1) {
                            $scope.TriggerAction.output_function_mask = undefined;
                        } else if (angular.element(target).attr('ng-model').indexOf('output_function_mask') > -1) {
                            $scope.TriggerAction.output_type = 0;
                        }

                        $scope.TriggerAction.output_port = undefined;
                        if ($scope.TriggerAction.output_type !== 1) {
                            //Todo: undefined로 필히 바꿔야 함. (추후)
                            $scope.TriggerAction.output_signal_id = 1;
                        } else {
                            $scope.TriggerAction.output_signal_id = 1;
                        }
                    };

                    $scope.doApplyTriggerAction = function () {
                        if ($scope.disabledYn) {
                            return false;
                        }
                        if ($scope.TriggerAction.input_type == 1) {
                            if ($scope.TriggerAction.input_event_id == undefined) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseSelectEvent'));
                                return false;
                            }
                        } else if ($scope.TriggerAction.input_type == 0) {
                            if ($scope.TriggerAction.input_duration == undefined) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseSelectDuration'));
                                return false;
                            }
                            if ($scope.TriggerAction.input_schedule_id == undefined) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.selectSchedule'));
                                return false;
                            }
                            if ($scope.TriggerAction.input_port == undefined) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseSelectInputPort'));
                                return false;
                            }

                        }

                        if ($scope.TriggerAction.output_type == 1) {
                            if ($scope.TriggerAction.output_signal_id == 1) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseSelectSignal'));
                                return false;
                            }
                            if ($scope.TriggerAction.output_port == undefined && $scope.TriggerAction.output_relay == undefined) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseSelectOutputPort'));
                                return false;
                            }
                        } else if ($scope.TriggerAction.output_type != 1 && ($scope.TriggerAction.output_function_mask == undefined)) {
                            MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseSelectAction'));
                            return false;
                        }

                        var trigger = {};
                        console.log($scope.page_mode);
                        if ($scope.page_mode == 'create') {
                            if (!$scope.ngModel.trigger_actions) {
                                $scope.ngModel.trigger_actions = [];
                            }
                            trigger = angular.copy($scope.TriggerAction);
                            $scope.ngModel.trigger_actions.push(trigger);
                        } else {
                            $scope.ngModel.trigger_actions[$scope.doSelectedEditTriggerActionRowIndex] = {};
                            $scope.ngModel.trigger_actions[$scope.doSelectedEditTriggerActionRowIndex] = angular.copy($scope.TriggerAction);
                        }
                        addTriggerActionDlgOptions.onClose();
//                  console.log($scope.TriggerAction);
//                  $scope.cbFunction($scope.TriggerAction);
                        $scope.TriggerAction = undefined;
                        $scope.doSelectedEditTriggerActionRowIndex = undefined;
                    };

                    $scope.doSelectedEditTriggerActionRowIndex = undefined;
                    $scope.doEditTriggerActionRow = function ($index) {
                        if ($scope.disabledYn) {
                            return false;
                        }
                        $scope.page_mode = 'update';

                        $scope.TriggerAction = undefined;
                        $scope.doSelectedEditTriggerActionRowIndex = $index;

                        $scope.TriggerAction = angular.copy($scope.ngModel.trigger_actions[$index]);
                        $scope.selectedInputPort = MessageProperties.get('triggerAction.inputPort', $scope.TriggerAction.input_port);

                        $scope.doOpenAddTriggerActionDlg();
                    };

                    $scope.doDeleteTriggerActionRow = function ($index) {
                        if ($scope.disabledYn) {
                            return false;
                        }
                        MessageService.displayConfirmMessage(MessageProperties.get("confirm.delete"), function (r) {
                            if (r) {
                                $scope.safeApply(function () {
                                    $scope.ngModel.trigger_actions.splice($index, 1);
                                });
                            } else {
                                return false;
                            }
                        });
                    };

                    $scope.doCancelTriggerAction = function () {
                        addTriggerActionDlgOptions.onClose();
                    };

                    $scope.doChangeInputSwitch = function () {
                        $scope.TriggerAction.input_switch = $scope.TriggerAction.input_switch == true ? 0 : 1;
                    };

//                $scope.fnSelectBoxBlur = function (event) {
//                    if (!event.target) {
//                        event.target = event.srcElement || document;
//                    }
//                    angular.element(event.target).removeClass('open');
//                };
//
//                $scope.fnSelectBoxFocus = function (event) {
//                    if (!event.target) {
//                        event.target = event.srcElement || document;
//                    }
//                    angular.element(event.target).addClass('open')
//                };

                    $scope.doSelectInputPort = function (port) {
//                    $scope.safeApply(function() {
                        $scope.TriggerAction.input_port = port;
                        $scope.selectedInputPort = MessageProperties.get('triggerAction.inputPort', port);
                        angular.element('.select.open').blur();
                        angular.element(event.target || event.srcElement).parents('.select').removeClass('open');
//                    });
                    };

                    $scope.doSelectOutputPort = function (port) {
                        $scope.TriggerAction.output_port = port;
                        angular.element('.select.open').blur();
                        angular.element(event.target || event.srcElement).parents('.select').removeClass('open');
                    };
                    $scope.doSelectOutputRelay = function (relay) {
                        $scope.TriggerAction.output_relay = relay;
                        angular.element('.select.open').blur();
                        angular.element(event.target || event.srcElement).parents('.select').removeClass('open');
                    };

                    $scope.doLoadSchedule = function ($event) {
                        /**
                         * 스케쥴 목록 정보 조회
                         */
                        ScheduleAgent.getScheduleList().then(function (data) {
                            $scope.scheduleListData = data.ScheduleCollection.rows;
                            angular.element($event.target.parentElement).addClass('open');
                        })
                    };

                    var addNewOutputSIgnalDlgOptions = {};
                    addNewOutputSIgnalDlgOptions.modal = true;
                    addNewOutputSIgnalDlgOptions.shadow = false;
                    addNewOutputSIgnalDlgOptions.title = 'New Output Signal';
                    addNewOutputSIgnalDlgOptions.width = 450;
                    addNewOutputSIgnalDlgOptions.height = 432;

                    addNewOutputSIgnalDlgOptions.onClose = function () {
                        $scope.addNewOutputSignalDlg = {};

                        $scope.safeApply(function () {
                            $scope.doSelectedEditTriggerActionRowIndex = {};
                            angular.element('#addNewOutputSignalDlg').dialog('destroy');
                        });
                    };

                    $scope.DeviceData = [];
                    $scope.deviceSpecData = undefined;

                    $scope.setDropDownData = function () {

                        $scope.DeviceData = [];
                        $scope.DeviceDataSpec = {};
                        var thisDevice = {
                            id: $scope.ngModel.id,
                            name: $scope.ngModel.name
//                        text: $scope.ngModel.name,
//                        deviceSpec: $scope.deviceSpecData[$scope.ngModel.device_type_id.id],
//                        device_type_id: $scope.ngModel.device_type_id
                        };
                        $scope.DeviceData.push(thisDevice);
                        $scope.DeviceDataSpec[thisDevice.id] = {
                            spec: $scope.deviceSpecData[$scope.ngModel.device_type_id.id]
                        };

                        if ($scope.ngModel.rs485 && $scope.ngModel.rs485.channels &&
                            $scope.ngModel.rs485.channels[0].slave_devices &&
                            $scope.ngModel.rs485.channels[0].slave_devices.length > 0) {

                            $scope.hasSlave = true;

                            for (var i = $scope.ngModel.rs485.channels[0].slave_devices.length; i--;) {
                                var slave = $scope.ngModel.rs485.channels[0].slave_devices[i];
                                var name = slave.id;
                                if (DeviceAgent.getAllDevicesMap()[slave.id]) {
                                    name = DeviceAgent.getAllDevicesMap()[slave.id].name;
                                }
                                var child = {
                                    id: slave.id,
                                    name: name
//                                text: slave.name,
//                                deviceSpec: $scope.deviceSpecData[slave.device_type_id.id],
//                                device_type_id: slave.device_type_id
                                };
                                $scope.DeviceData.push(child);
                                $scope.DeviceDataSpec[child.id] = {
                                    spec: $scope.deviceSpecData[slave.device_type_id.id]
                                };
                            }
                        } else {
                            $scope.hasSlave = false;
                        }
                    };

                    $scope.init = function () {

                        $scope.$on(Const.UI_EVENT_TYPE.DEVICE_INFO_LOADED, function () {
                            console.log(Const.UI_EVENT_TYPE.DEVICE_INFO_LOADED + ' received');

                            DeviceAgent.getDeviceSpec().then(function success(data) {
                                $scope.deviceSpecData = data;
                                $scope.setDropDownData();
                            });
                        });
                    };
                }]
        };
    });
});