'use strict';

angular.module('commonModule')
    .directive('smSlider', ['$compile', '$filter', '$document', '$window', function ($compile, $filter, $document, $window) {
        return {
            restrict: 'C',
            require: 'ngModel',
            scope: {
                ngModel: '=',
                timeRange: '=',
                disabled: '='
            },
            template: '<div class="sliderBg">' +
                '<a class="sliderHandle" href="javascript:void(null);" style="left:0%;"></a>' +
                '<span class="sliderRange" style="width:0%;"></span>' +
                '<ul class="unit length10">' +
                '<li ng-repeat="item in getRangeArray(10) track by $index"></li>' +
                '</ul>' +
                '</div>' +
                '<div class="sliderResult">{{ngModel}} sec</div>',
            link: function (scope, element, attrs) {

            },
            controller: ['$scope', '$element', function ($scope, $element) {

                if (angular.isDefined($scope.ngModel)) {
                    var curr_left = $scope.ngModel / $scope.timeRange * 100,
                        curr_width = $scope.ngModel / $scope.timeRange * 100;
                    $element.find('.sliderHandle').css('left', curr_left + '%');
                    $element.find('.sliderRange').css('width', curr_width + '%');
                }

                var unbindWatcher = $scope.$watch('ngModel', function (nv, ov) {
                    var curr_left = $scope.ngModel / $scope.timeRange * 100,
                        curr_width = $scope.ngModel / $scope.timeRange * 100;
                    $element.find('.sliderHandle').css('left', curr_left + '%');
                    $element.find('.sliderRange').css('width', curr_width + '%');
                });


                if (angular.isUndefined($scope.timeRange)) {
                    $scope.timeRange = 10;
                }

                if (angular.isUndefined($scope.disabled)) {
                    $scope.disabled = false;
                }

                $scope.safeApply = function (fn) {
                    var phase = this.$root.$$phase;
                    if (phase == '$apply' || phase == '$digest') {
                        if (fn && (typeof(fn) === 'function')) {
                            fn();
                        }
                    } else {
                        this.$apply(fn);
                    }
                };


                $scope.getRangeArray = function (range) {
                    return new Array(range);
                };

                var parseProp = function (clas, prop) {
                    var style = $element.find('.' + clas).attr('style'),
                        prop_arr = style.split(';'),
                        prop_map = {};

                    for (var i = 0; i < prop_arr.length; i++) {
                        var item = prop_arr[i].split(':');
                        prop_map[item[0]] = item[1];
                    }
                    return parseFloat(prop_map[prop]);
                };

                var handleObject = null,
                    last_position = {};

                $element.on('mousemove', function ($event) {
                    if (typeof(last_position.x) != 'undefined' && !$scope.disabled) {
                        var deltaX = last_position.x - $event.clientX,
                            deltaY = last_position.y - $event.clientY,
                            curr_left = parseFloat($element.find('.sliderHandle').css('left')) - deltaX,
                            curr_width = parseFloat($element.find('.sliderRange').css('width')) - deltaX,
                            elem_width = parseFloat($element.find('.sliderBg').css('width'));

                        if (handleObject != null) {
                            if ((curr_left / elem_width) * 100 >= 0 && (curr_left / elem_width) * 100 <= 100) {
                                $element.find('.sliderHandle').css('left', curr_left + 'px');
                                $element.find('.sliderRange').css('width', curr_width + 'px');
                                onChangeSlider();
                            }

//              if (Math.abs(deltaX) > Math.abs(deltaY) && deltaX > 0) {
//                //left
//                console.info('left');
//              } else if (Math.abs(deltaX) > Math.abs(deltaY) && deltaX < 0) {
//                //right
//                console.info('right');
//              } else if (Math.abs(deltaY) > Math.abs(deltaX) && deltaY > 0) {
//                //up
//                console.info('up');
//              } else if (Math.abs(deltaY) > Math.abs(deltaX) && deltaY < 0) {
//                //down
//                console.info('down');
//              }
                        }
                    }

                    last_position = {
                        x: $event.clientX,
                        y: $event.clientY
                    };
                });

                $element.on('mousedown', function ($event) {
                    if ($event.target.className == 'sliderHandle' && !$scope.disabled) {
                        unbindWatcher();
                        handleObject = {};
                    }
                });

                var setPx2Percentage = function () {
                    var elem_width = parseFloat($element.find('.sliderBg').css('width')),
                        curr_left = parseFloat($element.find('.sliderHandle').css('left')) / elem_width * 100,
                        curr_width = parseFloat($element.find('.sliderRange').css('width')) / elem_width * 100;
                    $element.find('.sliderHandle').css('left', curr_left + '%');
                    $element.find('.sliderRange').css('width', curr_width + '%');
                };

                var onChangeSlider = function () {
                    var curr_left = parseFloat($element.find('.sliderHandle').css('left')) / parseFloat($element.find('.sliderBg').css('width')) * 100;
                    $scope.safeApply(function () {
                        $scope.ngModel = Math.round(curr_left / (100 / $scope.timeRange));
                    });
                };
                onChangeSlider();

                $element.on('mouseup', function ($event) {
                    if (handleObject != null) {
                        setPx2Percentage();
                        handleObject = null;
                    }
                });

                $element.on('mouseleave', function ($event) {
                    if (handleObject != null) {
                        setPx2Percentage();
                        handleObject = null;
                    }
                });
            }]
        }
    }]);