define([
    'modules/common/services/preferenceService'
], function () {

    var app = angular.module('filterModule', []);

    // name field filter, 검색결과가 없을 경우 noData 를 array에 담아 return,
    app.filter('listFilter', ['Util', function (Util) {
        return function (rowData, searchWord, field) {
            var i,
                item,
                resultArray = [];

            if (!searchWord && rowData) {
                return rowData;
            } else if (!searchWord && !rowData) {
                return resultArray;
            }

            if(!rowData) {
                return resultArray;
            }

            if (rowData.constructor == Array) {
                for (i = 0; i < rowData.length; i++) {
                    item = rowData[i];

                    if (angular.uppercase(item[field]).indexOf(angular.uppercase(searchWord)) > -1) {
                        resultArray.push(item);
                    }
                }
            } else if (rowData.constructor == Object) {
                for (i in rowData) {
                    item = rowData[i];

                    if (angular.uppercase(item[field]).indexOf(angular.uppercase(searchWord)) > -1) {
                        resultArray.push(item);
                    }
                }
            }

            resultArray.sort(Util.dynamicSort('name'));

            return resultArray;

        }
    }]);

    // name field filter, 검색결과가 없을 경우 noData 를 array에 담아 return,
    app.filter('userListFilter', ['Util', function (Util) {
        return function (rowData, searchWord, field) {
            var i,
                item,
                resultArray = [];

            if (!searchWord && rowData) {
                return rowData;
            } else if (!searchWord && !rowData) {
                return resultArray;
            }

            if(!rowData) {
                return resultArray;
            }

            if (rowData.constructor == Array) {
                for (i = 0; i < rowData.length; i++) {
                    item = rowData[i];

                    if (angular.uppercase(item[field]).indexOf(angular.uppercase(searchWord)) > -1) {
                        resultArray.push(item);
                    } else if (angular.isDefined(item['user_id']) && angular.uppercase(item['user_id']).indexOf(angular.uppercase(searchWord)) > -1) {
                        resultArray.push(item);
                    }
                }
            } else if (rowData.constructor == Object) {
                for (i in rowData) {
                    item = rowData[i];

                    if (angular.uppercase(item[field]).indexOf(angular.uppercase(searchWord)) > -1) {
                        resultArray.push(item);
                    } else if (angular.uppercase(item['user_id']).indexOf(angular.uppercase(searchWord)) > -1) {
                        resultArray.push(item);
                    }
                }
            }

            resultArray.sort(Util.dynamicSort('name'));

            return resultArray;

        }
    }]);

    /**
     * deviceTypeFilter
     */
    app.filter('devicesTypeFilter', ['$filter', 'Util', function ($filter, Util) {
        return function (rowData, searchTypes) {
            var i,
                item,
                resultArray = [];

            if (!searchTypes && rowData) {
                return rowData;
            } else if (!searchTypes && !rowData) {
                return resultArray;
            }

            if (rowData.length > 0) {
                if (angular.isDefined(rowData[0].device_type)) {

                    for (i = 0; i < rowData.length; i++) {
                        item = rowData[i];

                        if (searchTypes.indexOf(item.device_type) >= 0) {
                            resultArray.push(item);
                        }
                    }
                } else if (angular.isDefined(rowData[0].device_type_id)) {

                    for (i = 0; i < rowData.length; i++) {
                        item = rowData[i];

                        if (searchTypes.indexOf(item.device_type_id.id) >= 0) {
                            resultArray.push(item);
                        }
                    }
                }
            }


            return $filter('orderObjectBy')(resultArray, 'device_type');

        }
    }]);

    app.filter('range', function () {
        return function (rowData, range) {
            var i;
            range = parseInt(range);
            for (i = range; i--;) {
                rowData.push(i);
            }

            return rowData;

        }
    });

    app.filter('fromTo', function () {
        return function (rowData, from, to) {
            if (angular.isUndefined(from) || angular.isUndefined(to)) {
                return rowData;
            } else if (!rowData) {
                return [];
            }

            if (rowData.length < to) {
                to = rowData.length;
            }

            return rowData.slice(from, to);

        }
    });

    app.filter('orderObjectBy', function () {
        return function (items, field, reverse) {
            var filtered = [];
            angular.forEach(items, function (item) {
                filtered.push(item);
            });
            var fields = [];
            if (field.indexOf('.') > 0) {
                fields = field.split('.');
            }
            if (fields.length == 2) {
                filtered.sort(function (a, b) {
                    return (a[fields[0]][fields[1]] > b[fields[0]][fields[1]] ? 1 : -1);
                });
            } else {
                filtered.sort(function (a, b) {
                    return (a[field] > b[field] ? 1 : -1);
                });
            }
            if (reverse) filtered.reverse();
            return filtered;
        };
    });

//    app.filter('BssDate', function ($filter, PreferenceService, Util) {
//        return function (dateObject, formatString, timeZoneOffset){
//            var value = '';
//            if (formatString == undefined){
//                formatString = PreferenceService.getDateFormat();
//            }
//            if (timeZoneOffset == undefined){
//                timeZoneOffset = PreferenceService.getTimeZoneOffset();
//            }
//            value = $filter('date')(dateObject.convertDateUsingRelativeTimeZoneOffset(parseInt(timeZoneOffset)), formatString);
//            return value;
//        };
//    });

    app.filter('BssDate', ['$filter', 'PreferenceService', 'Util', function ($filter, PreferenceService, Util) {
        return function (value, formatString, timeZoneOffset, doNotUseTimeZoneAndTreatAsLocalTime) {
            var date;
            if (formatString == undefined) {
                formatString = PreferenceService.getDateFormat();
            }

            if (doNotUseTimeZoneAndTreatAsLocalTime == undefined) {
                doNotUseTimeZoneAndTreatAsLocalTime = false;
            }
            if (doNotUseTimeZoneAndTreatAsLocalTime == true) {
//                Util.debug(value);
                date = new Date();
                date.setFullYear(value.getFullYear());
                date.setMonth(value.getMonth());
                date.setDate(value.getDate());
                date.setHours(0);
                date.setMinutes(0);
                date.setSeconds(0);
                date.setMilliseconds(0);
                if(PreferenceService.isUseSystemDatetimeFormat() == true) {
                    return date.toLocaleDateString();
                }
                return $filter('date')(date, formatString);
            } else {
                if (value && value.constructor == Date) {
                    date = new Date(value.getTime());
                } else {
                    date = new Date(Date.parse(value) + (new Date().getTimezoneOffset() * 60 * 1000));
                }

                if (timeZoneOffset == undefined) {
                    timeZoneOffset = PreferenceService.getTimeZoneOffset();
                }
                if(PreferenceService.isUseSystemDatetimeFormat() == true) {
                    return date.toLocaleDateString();
                }
                return $filter('date')(Date.parse(date) + (timeZoneOffset * 60 * 1000), formatString);
            }

        };
    }]);


//    app.filter('BssTime', function ($filter, PreferenceService, Util) {
//        return function (dateObject, formatString, timeZoneOffset){
//            var value = '';
//            if (formatString == undefined){
//                formatString = PreferenceService.getTimeFormat();
//            }
//            if (timeZoneOffset == undefined){
//                timeZoneOffset = PreferenceService.getTimeZoneOffset();
//            }
//            value = $filter('date')(dateObject.convertDateUsingRelativeTimeZoneOffset(parseInt(timeZoneOffset)), formatString);
//            return value;
//        };
//    });

    app.filter('BssTime', ['$filter', 'PreferenceService', 'Util', function ($filter, PreferenceService, Util) {
        return function (value, formatString, timeZoneOffset, dependOnOS) {
            var date;
            if (formatString == undefined) {
                formatString = PreferenceService.getTimeFormat();
            }


            if (value && value.constructor == Date) {
                date = new Date(value.getTime() + parseInt(new Date().getTimezoneOffset() * 60 * 1000));
            } else {
                date = new Date(Date.parse(value) + (new Date().getTimezoneOffset() * 60 * 1000));
            }

            if (timeZoneOffset == undefined) {
                timeZoneOffset = PreferenceService.getTimeZoneOffset();
            }

            if(PreferenceService.isUseSystemDatetimeFormat() == true && dependOnOS != false) {
                return date.toLocaleTimeString();
            }

            return $filter('date')(Date.parse(date) + (timeZoneOffset * 60 * 1000), formatString);
        };
    }]);

    app.filter('BssTimeLongTerm', ['$filter', 'PreferenceService', 'Util', function ($filter, PreferenceService, Util) {
        return function (value, formatString, timeZoneOffset) {
            var date, len;
            if (formatString == undefined) {
                formatString = PreferenceService.getTimeLongTermFormat();
            }


            if (value && value.constructor == Date) {
                date = new Date(value.getTime() + parseInt(new Date().getTimezoneOffset() * 60 * 1000));
            } else {
                date = new Date(Date.parse(value) + (new Date().getTimezoneOffset() * 60 * 1000));
            }

            if (timeZoneOffset == undefined) {
                timeZoneOffset = PreferenceService.getTimeZoneOffset();
            }

            if(PreferenceService.isUseSystemDatetimeFormat() == true) {
                return date.toLocaleTimeString();
            }

            return $filter('date')(Date.parse(date) + (timeZoneOffset * 60 * 1000), formatString);
        };
    }]);


//    app.filter('BssDatetime', function ($filter, PreferenceService, Util) {
//        return function (dateObject, formatString, timeZoneOffset){
//            var value = '';
//            if (formatString == undefined){
//                formatString = PreferenceService.getDatetimeFormat();
//            }
//            if (timeZoneOffset == undefined){
//                timeZoneOffset = PreferenceService.getTimeZoneOffset();
//            }
//            value = $filter('date')(dateObject.convertDateUsingRelativeTimeZoneOffset(parseInt(timeZoneOffset)), formatString);
//            return value;
//        };
//    });

    app.filter('BssDatetime', ['$filter', 'PreferenceService', 'Util', function ($filter, PreferenceService, Util) {
        return function (value, formatString, timeZoneOffset) {
            var date;
            if (formatString == undefined) {
                formatString = PreferenceService.getDatetimeFormat();
            }


            if (value && value.constructor == Date) {
                date = new Date(value.getTime() + parseInt(new Date().getTimezoneOffset() * 60 * 1000));
            } else {
                date = new Date(Date.parse(value) + (new Date().getTimezoneOffset() * 60 * 1000));
            }

            if (timeZoneOffset == undefined) {
                timeZoneOffset = PreferenceService.getTimeZoneOffset();
            }

            if(PreferenceService.isUseSystemDatetimeFormat() == true) {
                return date.toLocaleString();
            }

            return $filter('date')(Date.parse(date) + (timeZoneOffset * 60 * 1000), formatString);
        };
    }]);

    app.filter('BssDatetimeLongTerm', ['$filter', 'PreferenceService', 'Util', function ($filter, PreferenceService, Util) {
        return function (value, formatString, timeZoneOffset) {
            var date;
            if (formatString == undefined) {
                formatString = PreferenceService.getDatetimeLongTermFormat();
            }

            if (value && value.constructor == Date) {
                date = new Date(value.getTime() + parseInt(new Date().getTimezoneOffset() * 60 * 1000));
            } else {
                date = new Date(Date.parse(value) + (new Date().getTimezoneOffset() * 60 * 1000));
            }

            if (timeZoneOffset == undefined) {
                timeZoneOffset = PreferenceService.getTimeZoneOffset();
            }

            if(PreferenceService.isUseSystemDatetimeFormat() == true) {
                return date.toLocaleString();
            }

            return $filter('date')(Date.parse(date) + (timeZoneOffset * 60 * 1000), formatString);
        };
    }]);

//    app.filter('invalidIP', function() {
//        return function(value) {
//            var valid = true;
//            console.log('invalidIP');
//            if (value == undefined && event && (event.target || event.srcElement)) {
//                value = angular.element(event.target || event.srcElement).val();
//            }
//            if (value != undefined) {
//                var bytes = value.split('.');
//                var number = /^[0-9]+$/;
//                if (bytes.length != 4) {
//                    valid = false;
//                } else {
//                    for (var i = 0; i < bytes.length; i++) {
//                        var byte = bytes[i];
//                        if (!number.test(byte)) {
//                            valid = false;
//                            break;
//                        } else if (byte < 0 || byte > 255) {
//                            valid = false;
//                            break;
//                        }
//                    }
//                }
//            }
//            return !valid;
//        };
//    });

    return app;
});