'use strict';

define(function () {

    // following constants are moved from const.js to prevent circular dependencies
    var PROPERTY_KEY_PREFIX_REGEX = /^[a-z]/;
    var SERVER_RESPONSE_CODE = {
        NOT_DEFINED: "-1",

        // STATUS: "OK
        SUCCESS: "0",		///< Success (no error)
        PARTIAL_SUCCESS: "1",		///< Request is successful", but partially
        ALREADY_LOGGED_IN: "2",		///< User has already logged in
        DEVICE_HAS_INVALID_CONFIG: '3', ///< Device has confugration which server can not be parsed.
        WEB_REQUEST_TIMEOUT: "4",		///< Synced Web Request is not respond in timeout period

        // STATUS: "UNAUTHORIZED
        LOGIN_REQUIRED: "10",		///< Login is required
        SESSION_TIMEOUT: "11",		///< Session is invalid for timeout

        // STATUS: "FORBIDDEN
        PERMISSION_DENIED: "20",		///< Request is forbidden because have no permission

        INVALID_PARAMETERS: "30",       ///< Parameter(s) is(are) not valid on URI.

        // STATUS: "BAD_REQUEST
        // general
        LOGIN_FAILED: "101",		///< Failed to login for invalid username or password
        METHOD_NOT_ALLOWED: "102",		///< Request method is not allowed
        NOT_SUPPORTED_REQUEST: "103",		///< Request is not supported
        INVALID_JSON_FORMAT: "104",		///< Request parameter is invalid
        INVALID_QUERY_PARAMETER: "105",		///< Request query parameter is invalid
        COULD_NOT_UPDATE_OR_DELETE_PREDEFINED_ITEMS: "106", ///< Impossible to update or delete predefined items.
        // user
        USER_NOT_FOUND: "201",		///< User can not be found with id
        DUPLICATED_USER: "202",		///< User who has same id alreay exists
        DUPLICATED_FINGERPRINT: "203",		///< Fingerprint template duplicated with other user
        DUPLICATED_USERGROUP: "204",		///< User group that has same name already exists
        ADMIN_COULD_NOT_BE_DELETED: "205",      ///< Admin could be deleted.
        ADMIN_COULD_NOT_HAVE_OTHER_PERMISSION: "206",
        ID_COULD_NOT_BE_CHANGED: "207",
        DUPLICATED_LOGIN_ID: "208",		///< User who has same id alreay exists
        FAILED_TO_ENROLL_USER: "209",		///< Failed to enroll user
        DEVICE_USERS_FULL: "210",		///< users maxed out on device
        INVALID_FILE_FORMAT: "211",

        // device
        DUPLICATED_DEVICE: "300",		///< Device that has same name already exists
        DEVICE_NOT_FOUND: "301",		///< Device can not be found with id
        DEVICE_COULD_NOT_ENABLE_FULL_ACCESS: "302", ///< Device what is included in an Access Group can not enable full access.
        TOO_MANY_DEVICE_OPERATOR: "303",     ///< The number of device operators can be up to 10.
        FAILED_TO_ADD_DEVICE: "304",     ///< Failed to add device.
        DUPLICATED_DEVICE_GROUP: "65645",		///< Device Group 명이 중복됩니다.
        TOO_MANY_ACCESS_GROUP_ASSIGNED: "65637",		///< Too many access groups are assigned for given user ( max 16 access groups)
        // card
        CARD_NOT_FOUND: "400",		///< Card can not be found with id
        DUPLICATED_CARD: "401",		///< Card that has same id already exists
        // device group
        DEVICE_GROUP_NOT_FOUND: "500",      ///< Device Group doesn't exist.
        // User Group
        USER_GROUP_NOT_FOUND: "600",      ///< User Group doesn't exist.
        // Door
        DOOR_NOT_FOUND: "601",      ///< Door doesn't exist.
        HOLIDAYGROUP_NOT_FOUND: "602",      ///< Holiday Group doesn't exist.
        ACCESSGROUP_NOT_FOUND: "603",      ///< Access Group doesn't exist.
        SCHEDULE_NOT_FOUND: "604",      ///< Schedule doesn't exist.
        ACCESSLEVEL_NOT_FOUND: "605",      ///< Access Level doesn't exist.
        PERMISSION_NOT_FOUND: "606",
        DOOR_GROUP_NOT_FOUND: "607",      ///< Door Group doesn't exist.
        PREFERENCE_NOT_FOUND: "608",
        //Access Group
        COULD_HAVE_FULL_ACCESS_DEVICES: "650", ///< Access Group can not have Device(s) what is(are) set Full Access enabled.
        TOO_MANY_ACCESSGROUP: "651",
        ACCESSGROUPLEVEL_DUPLICATED_NAME: "652",   ///< The name is duplicated.
        NO_ACCESSGROUP_NAME_EXISTS: "653",
        INVALID_DOOR: "65644",

        // Json
        FAILED_TO_PARSE_JSON: "700",      ///< Failed to parse json data.
        FAILED_TO_PARSE_JSON_INTERNAL: "701", ///< Failed to parse json data by internal server error

        FAILED_TO_EXECUTE_DB_QUERY: "800",    ////< Failed to execute Database query
        // STATUS: "INTERNAL
        // General
        SERVER_ERROR: "1000",		///< Something wrong with server
        DEVICE_IS_NOT_CONNECTED: "1001",		///< Device is not connected to biostar
        DEVICE_IS_NOT_READY: "1002",		///< Device is connected but not accpeted
        DEVICE_REQUEST_TIMEOUT: "1003",		///< Device does not respond within the timeout period
        // Network
        NET_INVALID_ADDRESS: "1004",		///< Network address is invalid
        NET_CONNECTION_FAILED: "1005",		///< Failed to connect device
        NET_WRONG_CHECKSUM: "1010",		///< Packet checksum is wrong
        NET_MALFORMED_HEADER: "1011",		///< Header format is not valid
        NET_MALFORMED_PAYLOAD: "1012",		///< Payload format is not valid
        DEVICE_NOT_SUPPORTED: "1013",
        // fingerprint scan
        FINGERPRINT_QUALITY_TOO_LOW: "1014",
        FAILED_TO_VERIFY_FINGERPRINT: "1015",
        FAILED_TO_UPGRADE_FIRMWARE: "1016",
        DEVICE_IS_BUSY: "1017",
        FAILED_TO_SCAN_FINGERPRINT: "1018",
        //biostar update
        BIOSTAR_UPDATE_SERVER_BUSY: "1020",  ///< biostar launcher is busy... try later.
        BIOSTAR_UPDATE_NOT_EXIST: "1021",  ///< biostar update version is not exist
        LAUNCHER_REQUEST_TIMEOUT: "1022",	///< Launcher does not respond within the timeout period

        //wet socket
        WEB_SOCKET_NOT_FOUND: "1100", ///< web socket not found by session id
        WEB_SOCKET_INVALID_SESSION: "1101", ///< invalid session

        // user
        INVALID_LENGTH_OF_USERID: "131072",
        INVALID_USERID: "131073",
        INVALID_USER_NAME: "131074",
        INVALID_LENGTH_OF_TITLE: "131075",
        INVALID_LENGTH_OF_PHONE_NUM: "131076",
        INVALID_PHONE: "131077",
        INVALID_LENGTH_OF_EMAIL: "131078",
        INVALID_EMAIL: "131079",
        INVALID_LENGTH_OF_PIN: "131080",
        INVALID_PIN: "131081",
        INVALID_LENGTH_OF_LOGINID: "131082",
        INVALID_LOGINID: "131083",
        INVALID_LENGTH_OF_PASSWORD: "131084",
        INVALID_EXPIRY_DATE: "131085",
        EXPIRY_DATE_IS_LT_START: "131086",
        INVALID_LENGTH_OF_MESSAGE: "131087",
        INVALID_COUNT_OF_FINGERPRINT: "131088",
        INVALID_COUNT_OF_FACETEMPLATE: "131089",
        OVER_MAX_ACCESS_GROUPS: "131090",
        INVALID_SECURITY_LEVEL: "131091",
        INVALID_DEVICE_STATUS: "131092",
        EXCEED_DESCRIPTION_MAX_LENGTH: "131093",
        EXCEED_NAME_MAX_LENGTH: "131094",
        CARD_ID_IS_REQUIRED: "131095",
        PARENT_DOOR_GROUP_IS_REQUIRED: "131096",
        DUPLICATE_USER_GROUP: "65646",
        PARENT_USER_GROUP_NOT_FOUND: "65647",
        PARENT_USER_GROUP_NOT_SET: "65648",
        USER_NOT_EXIST: "65649",
        ERR_NUM_CARD_ID_ALREADY_EXISTS: "65651",
        ERR_NUM_DOOR_NAME_ALREADY_EXISTS: "65652",
        ERR_NUM_DOOR_GROUP_NAME_ALREADY_EXISTS: "65653",
        ERR_NUM_DOOR_GROUP_NOT_FOUND: "65654",
        ERR_NUM_DEVICE_ALREADY_USED: "65655",
        ERR_NUM_RELAY_ALREADY_USED: "65656",
        ERR_NUM_DEVICE_NOT_IN_SAME_RS485: "65657"

        // database
//        DB_CONNECTION_FAILED: "66636",		///< Failed to database server
//        DB_CONFIGURATION_FAILED: "66637",		///< Database configuration does not exist or invalid
//        DB_INTERNAL_ERROR: "66638",     ///< Database internal error
//        DB_INVALID_VALUE_ERROR: "66639"     ///< Failed to check validation of value

    };
    var MSG_SKIP_RESPONSE_CODE = [
        SERVER_RESPONSE_CODE.LOGIN_REQUIRED
    ];
    var MESSAGE_SERVER_RESPONSE_PREFIX = "serverResponse";

    var app = angular.module('messageModule', []);

    app.service('MessageProperties', ['$rootScope', '$q', function ($rootScope, $q) {
        // AngularJS will instantiate a singleton by calling "new" on this function

        var language = 'en',
            MESSAGE_PROPERTIES = {};

        function init(lang) {
            console.log('MessageProperties.init::' + lang);
            var defer = $q.defer();
            if (lang == undefined) {
                defer.reject();
            } else {
                language = lang;

                jQuery.ajax({
                    url: '/resources/messages_' + language + '.properties',
                    success: function (data) {
                        var array = data.split("\n");
                        var jsonObject = {};
                        for (var i = 0; i < array.length; i++) {
                            try {
                                var keyValueArray = [];
                                var tempIndex = array[i].indexOf("=");
                                keyValueArray[0] = array[i].substr(0, tempIndex);
                                keyValueArray[1] = array[i].substr(tempIndex + 1);

                                if (keyValueArray[1] != undefined && keyValueArray[1].length > 0) {
                                    var valueJsonObject = JSON.parse('"' + keyValueArray[1].trim() + '"');
                                    jsonObject[keyValueArray[0].trim()] = unescape(valueJsonObject);
                                } else {
                                    if (keyValueArray[0] != undefined && keyValueArray[0].length > 0) {
                                        jsonObject[keyValueArray[0].trim()] = '';
                                    }
                                }
                            } catch (err) {
                                console.log('Failed to parse resource property::' + array[i]);
                            }
                        }
                        MESSAGE_PROPERTIES = jsonObject;
                        defer.resolve(jsonObject);

                        $rootScope.$broadcast('ChangeLanguageSet');
                    },
                    async: false
                });
            }
            return defer.promise;
        }

        init('en');

        var messageProperties = {
            init: init,
            get: function () {
                console.log()
                if (arguments == undefined || arguments.length < 1) {
                    return;
                }
                var messageId = arguments[0];
                var message = MESSAGE_PROPERTIES[messageId];
                var responseCode = undefined;
                if (message == undefined && messageId.indexOf(MESSAGE_SERVER_RESPONSE_PREFIX) == 0) {
                    responseCode = messageId.substr(MESSAGE_SERVER_RESPONSE_PREFIX.length + 1);
                    for (var key in SERVER_RESPONSE_CODE) {
                        if (SERVER_RESPONSE_CODE.hasOwnProperty(key)) {
                            var tempValue = SERVER_RESPONSE_CODE[key];
                            if (tempValue == responseCode) {
                                console.log(messageId + " has been converted to " + MESSAGE_SERVER_RESPONSE_PREFIX + "." + key);
                                messageId = MESSAGE_SERVER_RESPONSE_PREFIX + "." + key;
                                message = MESSAGE_PROPERTIES[messageId];
                                break;
                            }
                        }
                    }
                }
                if (message == undefined) {
                    return "$" + messageId + "$";
                } else {
                    for (var i = 1; i < arguments.length; i++) {
                        var argMessageKey = arguments[i];
                        if (argMessageKey != undefined && argMessageKey.length > 0) {
//                            console.log(argMessageKey);
                            if (argMessageKey.match(PROPERTY_KEY_PREFIX_REGEX) && argMessageKey.indexOf(".") > -1) {
                                // when argument is message property key
                                var argMessage = messageProperties.get(argMessageKey);
                                message = message.replace("\{" + (i - 1) + "\}", argMessage);
                            } else {
                                // when argument is value, not message property key
                                message = message.replace("\{" + (i - 1) + "\}", argMessageKey);
                            }
                        } else if (argMessageKey != undefined) {
                            // when argument is value, not message property key
                            message = message.replace("\{" + (i - 1) + "\}", argMessageKey);
                        }
                    }
                    return message;
                }
            }
        }
        return messageProperties;
    }]);

    app.directive('ngLabel', ['$rootScope', 'MessageProperties', function ($rootScope, MessageProperties) {
        return {
            restrict: 'A',

            link: function ($scope, $element, $attrs) {
                assignMsg();

                function assignMsg() {
                    var msg = MessageProperties.get($attrs.ngLabel, $attrs.arg0, $attrs.arg1, $attrs.arg2, $attrs.arg3, $attrs.arg4, $attrs.arg5, $attrs.arg6, $attrs.arg7, $attrs.arg8);

                    if (!msg) {
                        $element.html('$' + $attrs.ngLabel + '$');
                    } else {
                        $element.html(msg);
                    }
                }

                $scope.$on('ChangeLanguageSet', function (event, data) {
                    assignMsg();
                });
            }
        };
    }]);


    app.service('DialogService', ['$rootScope', '$compile', 'Const', function ($rootScope, $compile, Const) {

        /* List of options in DialogService  :
         var dialogOptions = {
         divId : 'serverSettingDialog',
         title : MessageProperties.get("common.biostar"),
         modal : true,
         width: 'auto',
         height: 'auto',,
         type: 'bssServiceDlg',
         targetScope: '$scope',
         top: 100,
         position : 'center',
         hideCancelButton : false,
         okButtonLabel : "button.upgrade",
         cancelButtonLabel : "button.cancel",
         okButtonPrivilege : "MONITORING",
         iconClass : 'icon-save',
         okButtonCallback : function(){
         console.log("upgrade ok clicked");
         DialogService.closeDialog(dialogOptions);
         },
         cancelButtonCallback : function(){
         console.log("upgrade cancel clicked");
         DialogService.closeDialog(dialogOptions);
         },
         content : '<div class="tableTop" style="text-align: center;">\
         <h4 class="popCntTitleM" ng-label="setting.server.upgradeNow"></h4>\
         </div>'
         };
         */
        var blockESCKey = undefined;
        var dialogService = {
            generateUuid: function () {
                function _p8(s) {
                    var p = (Math.random().toString(16) + "000000000").substr(2, 8);
                    return s ? "-" + p.substr(0, 4) + "-" + p.substr(4, 4) : p;
                }

                return _p8() + _p8(true) + _p8(true) + _p8();
            },
            openDialog: function (serviceOptions) {

                serviceOptions.type = Const.DIALOG_TYPE.DIALOG;
                if (serviceOptions.okButtonLabel == undefined) {
                    serviceOptions.okButtonLabel = "button.ok";
                }
                if (serviceOptions.cancelButtonLabel == undefined) {
                    serviceOptions.cancelButtonLabel = "button.cancel";
                }
                if (serviceOptions.title == undefined) {
                    serviceOptions.title = '';
                }
                if (serviceOptions.modal == undefined) {
                    serviceOptions.modal = true;
                }
                if (serviceOptions.width == undefined) {
                    serviceOptions.width = 'auto';
                }
                if (serviceOptions.height == undefined) {
                    serviceOptions.height = 'auto';
                }
                if (serviceOptions.top == undefined) {
//                    serviceOptions.top = 100;
                }
                if (serviceOptions.position == undefined || serviceOptions.position == 'center') {
                    serviceOptions.position = {
                        my: "center top",
                        at: "center top",
                        of: window
                    };
                }
                if (serviceOptions.icon !== undefined) {
                    serviceOptions.type = Const.DIALOG_TYPE.MESSAGER;
                }
                if (serviceOptions.targetScope === undefined) {
                    serviceOptions.targetScope = $rootScope;
                }

                if (serviceOptions.hideCancelButton == undefined) {
                    serviceOptions.hideCancelButton = false;
                }
                serviceOptions.okFunctionName = "okClicked" + serviceOptions.divId.replace(/-/g, "");
                serviceOptions.cancelFunctionName = "cancelClicked" + serviceOptions.divId.replace(/-/g, "");

                var dialogOptions = {};
//                dialogOptions.iconCls = serviceOptions.iconClass;
                dialogOptions.iconCls = serviceOptions.icon;
                dialogOptions.modal = serviceOptions.modal;
                dialogOptions.shadow = false;
                dialogOptions.border = false;
                dialogOptions.title = serviceOptions.title;
                dialogOptions.width = serviceOptions.width;
                dialogOptions.height = serviceOptions.height;
                dialogOptions.top = serviceOptions.top;
                dialogOptions.position = serviceOptions.position;
                if (serviceOptions.type === Const.DIALOG_TYPE.SERVICE) {
                    dialogOptions.onOpen = function () {
                        $('#' + serviceOptions.divId).parent().addClass(Const.DIALOG_TYPE.SERVICE);
                    };
                }
//                check minimum width
                if (!dialogOptions.width || dialogOptions.width === 'auto' || dialogOptions.width < 350) {
                    serviceOptions.width = dialogOptions.width = 350;
                }
                dialogOptions.onClose = function () {
                    if (!serviceOptions.targetScope.$$phase) {
                        serviceOptions.targetScope.$apply(function () {
                            serviceOptions.targetScope[serviceOptions.cancelFunctionName]();
                        });
                    }
                };

                serviceOptions.targetScope[serviceOptions.okFunctionName] = function () {
                    if (serviceOptions.okButtonCallback != undefined) {
                        serviceOptions.okButtonCallback();
                    }
                };

                serviceOptions.targetScope[serviceOptions.cancelFunctionName] = function () {
                    if (serviceOptions.cancelButtonCallback != undefined) {
                        serviceOptions.cancelButtonCallback();
                    }
                };

                var contents = '<div id="' + serviceOptions.divId + '" class="' + serviceOptions.type + '">';
                if (serviceOptions.icon) {
                    contents += '<div class="' + serviceOptions.type + 'Icon ' + serviceOptions.icon + '"></div>';
                }
                contents += '<div class="popCnt">' + serviceOptions.content;
                contents += '</div>\
                    <!-- footer -->\
                    <div class="popFooter">\
                        <div class="btnC">\
                            <a href="javascript:void(null);" ng-click="' + serviceOptions.okFunctionName + '()"\
                            class="btnCntS btnColorPoint" ng-label="' + serviceOptions.okButtonLabel + '"'
                if (serviceOptions.okButtonPrivilege) {
                    contents += '" ng-privilege="' + serviceOptions.okButtonPrivilege + '"';
                }
                contents += '></a>';
                if (serviceOptions.hideCancelButton == false) {
                    contents += '<a href="javascript:void(null);" ng-click="' + serviceOptions.cancelFunctionName + '()"\
                            class="btnCntS btnColorGray" style="margin-left:5px;" ng-label="' + serviceOptions.cancelButtonLabel + '"></a>';
                }
                contents += '</div>\
                    </div>\
                </div>';
                try {
                    angular.element('#' + serviceOptions.divId).dialog('destroy');
                    console.log('#' + serviceOptions.divId + " destroyed");
                } catch (error) {
                    console.log("closing dialog " + serviceOptions.divId + " failed", error);
                }
                angular.element(document.body).append(contents);
                $compile(angular.element('#' + serviceOptions.divId))(serviceOptions.targetScope);

                angular.element('#' + serviceOptions.divId).dialog(dialogOptions);
//                focused element focus out
                if (document.activeElement && document.activeElement.nodeName && document.activeElement.nodeName.toLowerCase() != 'body') {
                    $(document.activeElement).blur();
                }
//                esc, enter keypress binding
                if (serviceOptions.type === Const.DIALOG_TYPE.SERVICE || serviceOptions.type === Const.DIALOG_TYPE.MESSAGER) {
                    blockESCKey = function (evt) {
                        console.log(evt.keyCode);
                        switch (evt.keyCode) {
                            case 13://enter
//                                angular.element('#' + serviceOptions.divId).parent().find('.panel-tool-close').trigger('click');
                                break;
                            case 27://esc
                                angular.element('#' + serviceOptions.divId).parent().find('.panel-tool-close').trigger('click');
                                //angular.element(document).off('keydown');
                                break;
                        }
                    };
                    angular.element(document).on('keydown', blockESCKey);
                }

                if (serviceOptions.title == '') {
                    angular.element('#' + serviceOptions.divId).css(
                        {
                            'background': 'none',
                            'box-shadow': 'none'
                        }
                    );
                    angular.element('#' + serviceOptions.divId).find(".panel").css(
                        {
                            'border-radius': '10px',
                            'padding': '10px',
                            'box-shadow': 'none',
                            'border': '0px'
                        }
                    );
                }
            },
            closeDialog: function (serviceOptions) {
                if (blockESCKey != undefined) {
                    angular.element(document).off('keydown', blockESCKey);
                    blockESCKey = undefined;
                }
                angular.element('#' + serviceOptions.divId).dialog('destroy');
            }
        };

        return dialogService;
    }]);


    app.service('MessageService', ['$q', '$rootScope', 'MessageProperties', 'DialogService', 'Const', 'Util', function ($q, $rootScope, MessageProperties, DialogService, Const, Util) {
        var allDeviceMapWithStatus = {};
        var messageService = {
            getAllDeviceMapWithStatus: function () {
                return allDeviceMapWithStatus;
            },
            setAllDeviceMapWithStatus: function (paramAllDeviceMapWithStatus) {
                allDeviceMapWithStatus = paramAllDeviceMapWithStatus;
            },
            displayPartialSuccessfulMessage: function (data, displaySuccessPopoup, divId, paramCallback) {
                if (divId == undefined) {
                    divId = DialogService.generateUuid();
                }
                if (data != undefined && (data.DeviceResponse != undefined || displaySuccessPopoup == true)) {
//                    var headerMessage = '';
                    var messageMap = {};
                    var detailMessage = '';
                    var count = 0;
//                    if (data.Response.code != undefined) {
//                        if (data.DeviceResponse == undefined || data.DeviceResponse.result == Const.DEVICE_RESPONSE_RESULT.SUCCESS) {
//                            headerMessage = MessageProperties.get(MESSAGE_SERVER_RESPONSE_PREFIX + "." + data.Response.code) + "<br><br>";
////                            count++;
//                        } else {
//                            message += "<br>";
//                        }
//                    }
                    if (data.DeviceResponse != undefined && data.DeviceResponse.rows != undefined) {
                        for (var i = 0; i < data.DeviceResponse.rows.length; i++) {
                            var deviceResponseObject = data.DeviceResponse.rows[i];
                            if (deviceResponseObject.code != undefined && deviceResponseObject.code != SERVER_RESPONSE_CODE.SUCCESS && deviceResponseObject.id != undefined) {
                                var existingDeviceResponseObject = messageMap[deviceResponseObject.id];
                                if (existingDeviceResponseObject == undefined) {
                                    messageMap[deviceResponseObject.id] = [deviceResponseObject];
                                    if (count > 0) {
                                        detailMessage += "<br>";
                                    }
                                    var deviceObject = allDeviceMapWithStatus[deviceResponseObject.id];
                                    var responseMessage = MessageProperties.get(MESSAGE_SERVER_RESPONSE_PREFIX + "." + deviceResponseObject.code);
                                    if (deviceObject != undefined) {
                                        Util.debug(deviceObject.name + " : " + responseMessage);
                                        detailMessage += deviceObject.name;
                                        count++;
                                    }
                                } else {
                                    messageMap[deviceResponseObject.id].push(deviceResponseObject);
                                }
                            }
                        }
                    }
                    Util.info('parial success message', data.Response.code, data.Response, detailMessage, messageMap);
                    if (data.Response.code == Const.SERVER_RESPONSE_CODE.DEVICE_HAS_INVALID_CONFIG) {
                        var deviceId = undefined;
                        if (data.Device != undefined) {
                            deviceId = data.Device.id;
                        } else if (data.DeviceResponse != undefined && data.DeviceResponse.rows != undefined && data.DeviceResponse.rows.length > 0) {
                            deviceId = data.DeviceResponse.rows[0].id;
                        }
                        var message = MessageProperties.get(MESSAGE_SERVER_RESPONSE_PREFIX + "." + data.Response.code) + "<br><br>";
                        messageService.displayDeviceResetMessage(deviceId, message, function callback(isResetClicked, inputDeviceId) {
                            if (isResetClicked == true) {
                                if (paramCallback != undefined) {
                                    paramCallback(data.Response, inputDeviceId);
                                }
                            } else {
                                return false;
                            }
                        }, divId);
                        return false;
                    } else if (displaySuccessPopoup == true) {
                        if (data.Response.code == Const.SERVER_RESPONSE_CODE.SUCCESSFUL &&
                            data.DeviceResponse == undefined) {
                            var headerMessage = MessageProperties.get(MESSAGE_SERVER_RESPONSE_PREFIX + "." + data.Response.code) + "<br><br>";
                            var message = headerMessage;
                            messageService.displayInformationMessage(message, undefined, divId);
                            return true;
                        } else if (data.DeviceResponse != undefined && data.DeviceResponse.result == Const.DEVICE_RESPONSE_RESULT.SUCCESS) {
                            var headerMessage = MessageProperties.get(MESSAGE_SERVER_RESPONSE_PREFIX + "." + Const.SERVER_RESPONSE_CODE.SUCCESSFUL) + "<br><br>";
//                            var message = headerMessage + "<br>" + detailMessage;
                            var message = headerMessage;
                            messageService.displayInformationMessage(message, undefined, divId);
                            return true;
                        } else if (data.DeviceResponse != undefined && data.DeviceResponse.result == Const.DEVICE_RESPONSE_RESULT.FAIL) {
                            messageService.displayErrorMessage({
                                Response: {
                                    code: SERVER_RESPONSE_CODE.PARTIAL_SUCCESS,
                                    message: detailMessage
                                }
                            }, true, 570, divId);
                            return false;
                        } else {
                            var headerMessage = MessageProperties.get(MESSAGE_SERVER_RESPONSE_PREFIX + "." + data.Response.code) + "<br><br>";
                            var message = headerMessage + "<br>" + detailMessage;
                            messageService.displayErrorMessage({
                                Response: {
                                    code: SERVER_RESPONSE_CODE.SERVER_ERROR,
                                    message: message
                                }
                            }, true, 570, divId, true);
                            return false;
                        }
                    } else {
                        console.log("Partial successful message not display::" + message);
                    }
                }
                // when all of the partial response is successful, return return true
                return true;
            },
            displayErrorMessage: function (data, displayErrorPopup, width, divId, displayDetailMessage) {
                if (divId == undefined) {
                    divId = DialogService.generateUuid();
                }
                if (displayDetailMessage == undefined) {
                    displayDetailMessage = false;
                }
                var dialogOptions;
                Util.info("displayErrorMessage", data);
                if (data != undefined && data.Response != undefined) {
                    Util.debug('responseError::' + data.Response.code);
                    var message = MessageProperties.get(MESSAGE_SERVER_RESPONSE_PREFIX + "." + data.Response.code);
                    if (data.Response.message != undefined && data.Response.code == SERVER_RESPONSE_CODE.SERVER_ERROR) {
                        if (displayDetailMessage == true) {
                            message += " :<br><br>" + data.Response.message + " ( " + data.Response.code + " )";
                        } else {
                            message += " :<br><br>" + MessageProperties.get("serverResponse.SERVER_ERROR_DETAIL_MESSAGE") + " ( " + data.Response.code + " )";
                            ;
                        }
                    } else {
                        message += " ( " + data.Response.code + " )";
                        ;
                    }
                    if (data.Response.message != undefined &&
                        (
                            data.Response.code == SERVER_RESPONSE_CODE.PARTIAL_SUCCESS ||
                            data.Response.status == Const.HTTP_STATUS_CODE.NOT_FOUND
                            )) {
                        message += " :<br><br>" + data.Response.message;
                    }
                    if (displayErrorPopup == true && MSG_SKIP_RESPONSE_CODE.indexOf(data.Response.code) < 0) {
                        dialogOptions = {
                            divId: divId,
                            title: MessageProperties.get('common.errorDlgTitle'),
                            icon: Const.ALERT_MESSAGE_ICON.ERROR,
                            hideCancelButton: true,
                            okButtonCallback: function () {
                                DialogService.closeDialog(dialogOptions);
                            },
                            cancelButtonCallback: function () {
                                DialogService.closeDialog(dialogOptions);
                            },
                            content: message
                        };
                        if (width != undefined) {
                            dialogOptions.width = width;
                        }
                        DialogService.openDialog(dialogOptions);
                    } else {
                        console.log("Error message not display::" + message);
                    }
                }
            },
            displayInformationMessage: function (message, callback, divId) {
                if (divId == undefined) {
                    divId = DialogService.generateUuid();
                }
                var dialogOptions = {
                    divId: divId,
                    title: MessageProperties.get('common.infoDlgTitle'),
                    icon: Const.ALERT_MESSAGE_ICON.INFO,
                    hideCancelButton: true,
                    okButtonCallback: function () {
                        if (callback !== undefined) callback(true);
                        DialogService.closeDialog(dialogOptions);
                    },
                    cancelButtonCallback: function () {
                        if (callback !== undefined) callback(false);
                        DialogService.closeDialog(dialogOptions);
                    },
                    content: message
                };
                DialogService.openDialog(dialogOptions);
            },
            displayDeviceResetMessage: function (deviceId, message, callback, divId) {
                if (divId == undefined) {
                    divId = DialogService.generateUuid();
                }
                var dialogOptions = {
                    divId: divId,
                    title: MessageProperties.get('common.warningDlgTitle'),
                    icon: Const.ALERT_MESSAGE_ICON.WARNING,
                    okButtonLabel: 'device.info.reset',
                    okButtonCallback: function () {
                        if (callback !== undefined) callback(true, deviceId);
                        DialogService.closeDialog(dialogOptions);
                    },
                    cancelButtonCallback: function () {
                        if (callback !== undefined) callback(false);
                        DialogService.closeDialog(dialogOptions);
                    },
                    content: message
                };
                DialogService.openDialog(dialogOptions);
            },
            displayWarningMessage: function (message, callback, divId) {
                if (divId == undefined) {
                    divId = DialogService.generateUuid();
                }
                var dialogOptions = {
                    divId: divId,
                    title: MessageProperties.get('common.warningDlgTitle'),
                    icon: Const.ALERT_MESSAGE_ICON.WARNING,
                    hideCancelButton: true,
                    okButtonCallback: function () {
                        if (callback !== undefined) callback(true);
                        DialogService.closeDialog(dialogOptions);
                    },
                    cancelButtonCallback: function () {
                        if (callback !== undefined) callback(false);
                        DialogService.closeDialog(dialogOptions);
                    },
                    content: message
                };
                DialogService.openDialog(dialogOptions);
            },
            displayConfirmMessage: function (message, callback, divId) {
                if (divId == undefined) {
                    divId = DialogService.generateUuid();
                }
                var dialogOptions = {
                    divId: divId,
                    title: MessageProperties.get('common.confirmDlgTitle'),
                    icon: Const.ALERT_MESSAGE_ICON.CONFIRM,
                    okButtonLabel: 'button.yes',
                    okButtonCallback: function () {
                        if (callback !== undefined) callback(true);
                        DialogService.closeDialog(dialogOptions);
                    },
                    cancelButtonLabel: 'button.no',
                    cancelButtonCallback: function () {
                        if (callback !== undefined) callback(false);
                        DialogService.closeDialog(dialogOptions);
                    },
                    content: message
                };
                DialogService.openDialog(dialogOptions);
            },
            displaySimpleMessage: function (message, elem, title, callback) {
                var htmlStr = '',
                    elemBoundingRect = elem.getBoundingClientRect(),
                    positionOptions,
                    parentEle = angular.element('#biostarWebAppDiv > .wrap > .sm-container .contents'),
                    scrollTop = parentEle.scrollTop(),
                    divId = DialogService.generateUuid(),
                    offsetTop = angular.element('#biostarWebAppDiv > .wrap > header').outerHeight(true),
                    offsetLeft = angular.element('#biostarWebAppDiv > .wrap .sm-container > .leftMainMenu').outerWidth(true) + angular.element('#biostarWebAppDiv > .wrap .sm-container > .main-ng-view > .panelArea').outerWidth(true),
                    targetTop = 0,
                    targetLeft = 0;
//                    offsetTop = parseInt(parentEle.css('padding-top')) + parseInt(parentEle.css('padding-bottom')),
//                    offsetLeft = parseInt(parentEle.css('padding-left')) + parseInt(parentEle.css('padding-right'));
                htmlStr += '<section id="' + divId + '" class="simpleMessager" tabindex="-1">';
                htmlStr += '    <div class="inner">';
                if (title) {
                    htmlStr += '   <h1>' + title + '</h1>';
                }
                htmlStr += '       <p>';
                htmlStr += message;
                htmlStr += '       </p>';
                htmlStr += '    </div>';
                htmlStr += '</section>';
                angular.element('.simpleMessager').remove();
                parentEle.append(htmlStr);
                angular.element('.simpleMessager').on('focusout', function () {
                    $(this).off();
                    $(this).remove();
                });
                targetTop = elemBoundingRect.top + angular.element(elem).outerHeight(true) - offsetTop - 2 + scrollTop;
                targetLeft = elemBoundingRect.left - angular.element('#' + divId).outerWidth(true) - offsetLeft + 2;
                if (targetTop < 5) {
                    targetTop = 5;
                }
                if (targetLeft < 5) {
                    targetLeft = 5;
                }
                positionOptions = {
                    top: targetTop,
                    left: targetLeft
                }
                angular.element('#' + divId).css(positionOptions);
                angular.element('.simpleMessager').focus();
                angular.element('#' + divId).on('click', function () {
                    if (callback !== undefined) {
                        callback();
                    }
                    angular.element(this).off();
                    angular.element(this).remove();
                });
            }
        };
        return messageService;
    }]);
    return app;
});
