'use strict';

define([
    'modules/common/print/printOptionService',
    'modules/common/datagrid/columnSettingService',
    'modules/common/services/preferenceService'
], function () {

    var app = angular.module('biostarWebApp');
    app.register.controller('PrintOptionController', [
        '$scope', '$rootScope', '$timeout', '$filter', 'ColumnSettingService', 'Const', 'API', 'Util', 'Auth', 'PrintOptionService', 'PreferenceService', 'MessageService', 'MessageProperties',
        function ($scope, $rootScope, $timeout, $filter, ColumnSettingService, Const, API, Util, Auth, PrintOptionService, PreferenceService, MessageService, MessageProperties) {

            function getColumnSettings() {
                var customColumnSettings = ColumnSettingService.getColumnSettingFromLocalStroage();
                if (!customColumnSettings) {
                    customColumnSettings = ColumnSettingService.buildColumnSettingRows();
                    ColumnSettingService.saveDefaultColumnSettingToLocalStroage(customColumnSettings);
                }
                return customColumnSettings;
            }

            function getDatetimeFormat(gridId) {
                if (Const.GRID_ID.ALERT_HISTORY == gridId || Const.GRID_ID.EVENT_LOG == gridId) {
                    return PreferenceService.getDatetimeLongTermFormat();
                }
                return PreferenceService.getDatetimeFormat();
            }
            $scope.dateTimeFormatReg;

            $scope.initPopup = function () {

                var gridId = $scope.gridId;

                ColumnSettingService.setGridId(gridId);

                var _dataRows = getColumnSettings();
                var dateTimeFormatIdx;

                dateTimeFormatIdx =  Const.DATE_TIME_FORMAT.indexOf(getDatetimeFormat($scope.gridId));
                $scope.dateTimeFormatReg = Const.REGULAR_EXPRESSION.DATE_TIME_FORMAT[dateTimeFormatIdx];

//            var groupBy = {};
                $scope.groupBySelectBox = new Array();
                $scope.groupBySelectBox.push({});

                for (var index = 0; index < _dataRows.length; index++) {
                    var columnSetting = _dataRows[index];
                    if (columnSetting.check_value == "true" && columnSetting.printable == true) {
                        var groupByItem = {};
                        groupByItem.std_id = columnSetting.column_name;
                        groupByItem.std_name = columnSetting.display_name;
//                    if (groupBy.std_id == undefined || groupBy.std_id.length < 1) {
//                        groupBy = groupByItem;
//                    }
                        $scope.groupBySelectBox.push(groupByItem);
                    }
                }

//            $scope.doGroupBySelectItem(groupBy);

                var title = angular.element(".cntTitle").find("b").html();
//            console.log("title="+ title);
                $scope.printOption.title = title;
                PreferenceService.getPreference().then(function (pref) {
                    Util.debug("doPrint preference retrieved", pref);
                });

                $scope.printOption.reportingDateCheckbox = true;
                $scope.printOption.commentCheckbox = true;
                $scope.printOption.pageNumberCheckbox = true;

                $scope.printOption.comment = MessageProperties.get("printOption.createdBy", Auth.getUser().name);

                $scope.$watch("printOption.reportingDateCheckbox", function (nv, ov) {
//                Util.debug("reporting date format checkbox changed", nv, ov);
                    if (nv == false) {
//                        $scope.printOption.reportingDate = '';
                        $scope.printOption.reportingDate = Const.DATE_TIME_FORMAT[0];
                    } else {
                        $scope.printOption.reportingDate = getDatetimeFormat($scope.gridId);
                    }
                });

                $scope.$watch("printOption.commentCheckbox", function (nv, ov) {
//                console.log(nv);
//                console.log(ov);
                    if (nv == false) {
                        $scope.printOption.comment = '';
                    } else {
                        $scope.printOption.comment = MessageProperties.get("printOption.createdBy", Auth.getUser().name);
                    }
                });

            };

            $scope.doCancel = function () {
                PrintOptionService.closeDialog();
            };

            $scope.printOption = {};
            $scope.datetimeRegex = Const.REGULAR_EXPRESSION.DATE_TIME_FORMAT;


            API.getWithOptions({url: '/modules/common/print/page-size.json', encryptData: false}).then(function (data) {
                var i,
                    item;
                $scope.pageSizeSelectBox = {};
                for (i = 0; i < data.length; i++) {
                    item = data[i];
                    $scope.pageSizeSelectBox[item.std_id] = item.std_name;
                    if (item.std_id === Const.REPORT_PAGE_SIZE.A4) {
                        $scope.printOption.pageSize = item.std_id;
                    }
                }
            });

            API.getWithOptions({url: '/modules/common/print/file-format.json', encryptData: false}).then(function (data) {
                var i,
                    item;
                $scope.fileFormatSelectBox = {};
                for (i = 0; i < data.length; i++) {
                    item = data[i];
                    $scope.fileFormatSelectBox[item.std_id] = item.std_name;
                    if (item.std_id === Const.REPORT_FILE_FORMAT.SVG) {
                        $scope.printOption.fileFormat = item.std_id;
                    }
                }
            });

//            $scope.doPageSizeSelectItem = function (item) {
//                $scope.pageSize = item;
//                $scope.pageSizeSelectView = false;
//                $scope.printOption.pageSize = item.std_id;
//            };

//            $scope.doGroupBySelectItem = function (item) {
//                $scope.groupBy = item;
//                $scope.groupBySelectView = false;
//                $scope.printOption.groupBy = item.std_id;
////            console.log($scope.printOption);
//            };
//
//            $scope.doFileFormatSelectItem = function (item) {
//                $scope.fileFormat = item;
//                $scope.fileFormatSelectView = false;
//                $scope.printOption.fileFormat = item.std_id;
//            };

            function buildDisplayFields() {
                var displayFields = new Array();
                var columnSettings = getColumnSettings();
                var width = Math.floor(100 / columnSettings.length);
                for (var index = 0; index < columnSettings.length; index++) {
                    if (columnSettings[index].check_value == "true") {
                        var display_type = PrintOptionService.getReportColumnTypeFunction(columnSettings[index].column_name);
                        var field = {
                            name: PrintOptionService.getReportColumnNameFunction(columnSettings[index].column_name),
                            display_name: columnSettings[index].display_name,
                            display_type: display_type,
                            width: width
                        };
                        if (display_type == Const.REPORT_DISPLAY_TYPE.ENUM) {
                            if ($scope.gridId == Const.GRID_ID.DEVICE_LIST) {
                                field.enum = [
                                        Const.DEVICE_STATUS.DISCONNECTED + " = " + Const.DEVICE_STATUS[Const.DEVICE_STATUS.DISCONNECTED],
                                        "0x0" + Const.DEVICE_STATUS.NORMAL + " = " + Const.DEVICE_STATUS[Const.DEVICE_STATUS.NORMAL],
                                        "0x0" + Const.DEVICE_STATUS.SYNC_ERROR + " = " + Const.DEVICE_STATUS[Const.DEVICE_STATUS.SYNC_ERROR]
                                ];
                            } else if ($scope.gridId == Const.GRID_ID.USER_LIST) {
                                field.enum = [
                                    "false=",
                                        "true=" + MessageProperties.get("user.status.inactive")
                                ];
                            }
                        } else if (display_type == Const.REPORT_DISPLAY_TYPE.MASK) {
                            if ($scope.gridId == Const.GRID_ID.DOOR_LIST) {
                                field.enum = [
                                        Const.DOOR_STATUS.NORMAL + " = " + Const.DOOR_STATUS[Const.DOOR_STATUS.NORMAL],
                                        "0x0" + Const.DOOR_STATUS.LOCKED.toString(16) + " = " + Const.DOOR_STATUS[Const.DOOR_STATUS.LOCKED],
                                        "0x0" + Const.DOOR_STATUS.UNLOCKED.toString(16) + " = " + Const.DOOR_STATUS[Const.DOOR_STATUS.UNLOCKED],
                                        "0x0" + Const.DOOR_STATUS.FORCED_OPEN.toString(16) + " = " + Const.DOOR_STATUS[Const.DOOR_STATUS.FORCED_OPEN],
                                        "0x0" + Const.DOOR_STATUS.HELD_OPENED.toString(16) + " = " + Const.DOOR_STATUS[Const.DOOR_STATUS.HELD_OPENED],
                                        "0x" + Const.DOOR_STATUS.APB_FAILED.toString(16) + " = " + Const.DOOR_STATUS[Const.DOOR_STATUS.APB_FAILED],
                                        "0x" + Const.DOOR_STATUS.DISCONNECTED.toString(16) + " = " + Const.DOOR_STATUS[Const.DOOR_STATUS.DISCONNECTED]
                                ];
                            }
                        } else if (display_type == Const.REPORT_DISPLAY_TYPE.DATETIME) {
                            if ($scope.printOption.reportingDate != undefined) {
                                field.time_format = $scope.printOption.reportingDate;
                            } else {
                                field.time_format = getDatetimeFormat($scope.gridId);
                            }
                            field.time_zone = PreferenceService.getTimeZoneOffset() * 60;
                        }
                        displayFields.push(field);
                    }
                }
                return displayFields;
            }

            function populateViewObjectWithInputValues(paramObject) {
                if ($scope.printOption.title !== undefined) {
                    paramObject.name = $scope.printOption.title;
                }
                if ($scope.printOption.showTitleEveryPage !== undefined && $scope.printOption.showTitleEveryPage == true) {
                    paramObject.show_title_every_page = "true";
                } else {
                    paramObject.show_title_every_page = "false";
                }
                if ($scope.printOption.groupBy !== undefined) {
                    paramObject.group_field = PrintOptionService.getReportColumnNameFunction($scope.printOption.groupBy);
                    if (paramObject.group_field.length > 4 && paramObject.group_field.indexOf(".name") == (paramObject.group_field.length - 5)) {
                        paramObject.group_field = paramObject.group_field.substr(0, paramObject.group_field.length - 5) + ".id";
                    }
                    console.log(paramObject.group_field);
                    if (paramObject.Query.orders == undefined) {
                        paramObject.Query.orders = [];
                        paramObject.Query.orders.push({
                            column: paramObject.group_field, descending: false
                        });
                    }
                }
                if ($scope.printOption.groupByPageBreak !== undefined && $scope.printOption.groupByPageBreak == true) {
                    paramObject.group_by_page_break = "true";
                } else {
                    paramObject.group_by_page_break = "false";
                }
                if ($scope.printOption.reportingDateCheckbox !== undefined && $scope.printOption.reportingDateCheckbox == true) {
                    paramObject.footer_report_date = true;
                } else {
                    paramObject.footer_report_date = false;
                }
                if ($scope.printOption.commentCheckbox !== undefined && $scope.printOption.commentCheckbox == true) {
                    if ($scope.printOption.comment !== undefined) {
                        paramObject.footer_comment = $scope.printOption.comment;
                    }
                }
                if ($scope.printOption.pageNumberCheckbox !== undefined && $scope.printOption.pageNumberCheckbox == true) {
                    paramObject.footer_display_page_number = "true";
                } else {
                    paramObject.footer_display_page_number = "false";
                }
                if ($scope.printOption.fileFormat !== undefined) {
                    paramObject.content_type = $scope.printOption.fileFormat;
                }
                if ($scope.printOption.pageSize !== undefined) {
                    paramObject.page_size = $scope.printOption.pageSize;
                }
                paramObject.page_portrait = "false";
                if ($scope.printOption.page_orientation !== undefined) {
                    if ($scope.printOption.page_orientation == Const.REPORT_PAGE_ORIENTATION.PORTRAIT) {
                        paramObject.page_portrait = "true";
                    }
                }
                return paramObject;
            }

            $scope.doPrint = function () {
                Util.debug("doPrint begin");
                var paramObject = {};
                var queryObject = PrintOptionService.getQueryObject();
                paramObject.Query = queryObject;
                if (paramObject.Query == undefined) {
                    paramObject.Query = {};
                }
                paramObject.display_fields = buildDisplayFields();
                paramObject = populateViewObjectWithInputValues(paramObject);

                API.post($scope.reportUrl, undefined, {View: paramObject}).then(function (data) {
                    if (data && data.FileCollection.rows) {
                        var scrollbars = '';
                        if ($scope.printOption.fileFormat == Const.REPORT_FILE_FORMAT.SVG) {
                            scrollbars = 'scrollbars=1';
                        } else {
                            scrollbars = 'scrollbars=0';
                        }
                        var reportWindow = window.open('', '_report', 'width=1160,height=600,directories=0,titlebar=0,toolbar=0,location=0,status=0,menubar=0,' + scrollbars + ',resizable=yes');
                        var contents = "<html>\
                        <style>\
                    @page {\
                    size: " + $scope.printOption.pageSize + ";\
                    margin: 0;\
                }\
                </style><body>";
//                        page-break-after: always;\
                        for (var index = 0; index < data.FileCollection.rows.length; index++) {
                            var reportFile = data.FileCollection.rows[index];
//                        console.log(reportFile.uri);
//                        console.log(reportFile.type);
                            if ($scope.printOption.fileFormat == Const.REPORT_FILE_FORMAT.SVG) {
                                contents += "<img src='" + reportFile.uri + "' type='" + reportFile.type + "'>\n";
                            } else {
                                contents += "<object id='pdf" + index + "' type='application/pdf' data='" + reportFile.uri + "' style='width : 100vw; height : 100vh;'></object>\n";
//                                contents += "<embed src='" + reportFile.uri + "' style='width : 100vw; height : 100vh;'>\n";
                            }
                        }
                        contents += "</body></html>";
//                    console.log(contents);
//                        reportWindow.document.write(contents);
                        if(reportWindow) {
                            reportWindow.document.write(contents);
                        } else {
                            MessageService.displayWarningMessage(MessageProperties.get('common.pleaseDisablePopupBlocker'));
                            return false;
                        }
//                        $timeout(function () {
//                            reportWindow.print();
//                        }, 600);

                    }
                    $scope.doCancel();
                });
            }

        }]);
});


