define(function () {

    var app = angular.module('privilegeModule', []);

    app.service('PrivilegeService', ['$parse', 'Auth', 'Const', '$location', '$timeout', function ($parse, Auth, Const, $location, $timeout) {
        var idx,
            loginUserInfo,
            privilege,
            permissionType = Const.PERMISSION_TYPE;

        var privilegeService = {
            getModuleId: function (moduleName) {
                var moduleId = '';
                for (var key in Const.PERMISSION_MODULE_ID) {
                    if (Const.PERMISSION_MODULE_ID.hasOwnProperty(key)) {
                        if (key == moduleName) {
                            moduleId = Const.PERMISSION_MODULE_ID[key];
                            break;
                        }
                    }
                }
                return moduleId;
            },
            getModuleNameByModuleId: function (moduleId) {
                var moduleName = '';
                for (var key in Const.PERMISSION_MODULE_ID) {
                    if (Const.PERMISSION_MODULE_ID.hasOwnProperty(key)) {
                        if (Const.PERMISSION_MODULE_ID[key] == moduleId) {
                            moduleName = key;
                            break;
                        }
                    }
                }
                return moduleName;
            },
            getPermissionItem: function (moduleId, permissionItemList) {
                var foundPermissionItem = undefined;
                for (var i = 0; i < permissionItemList.length; i++) {
                    var permissionItem = permissionItemList[i];
                    if (permissionItem.module_id != undefined && permissionItem.module_id == moduleId) {
                        foundPermissionItem = permissionItem;
                        break;
                    }
                }
                return foundPermissionItem;
            },
            isUIAuthorizedForUpdate: function (moduleId) {

                loginUserInfo = Auth.getUser();

                if (loginUserInfo) {
                    privilege = privilegeService.getPermissionItem(moduleId, loginUserInfo.permission.permission_items);
                }

                if (Const.PERMISSION_MODULE_ID.SETTING == moduleId) {
                    return true;
                } else if (privilege && privilege.permission & permissionType.CanRead) {
                    return false;
                } else if (privilege && privilege.permission & permissionType.CanUpdate) {
                    return true;
                }
                return false;
            },
            isUrlAuthorized: function (url) {
                if (url.indexOf(Const.MODULE_URL.USER) > -1 ||
                    url.indexOf(Const.MODULE_URL.DEVICE) > -1 ||
                    url.indexOf(Const.MODULE_URL.ACCESS_CONTROL) > -1 ||
                    url.indexOf(Const.MODULE_URL.MONITORING) > -1 ||
                    url.indexOf(Const.MODULE_URL.SETTING) > -1) {
                    var loginUserInfo = Auth.getUser();
                    if (loginUserInfo == undefined) {
                        return false;
                    }
                    var isAdmin = false;
                    if (loginUserInfo.permission.id == Const.PERMISSION.ADMIN) {
                        isAdmin = true;
                    }
                    if (url.indexOf(Const.MODULE_URL.SETTING) > -1) {
                        return true;
                    }
                    for (var i = 0; i < loginUserInfo.permission.permission_items.length; i++) {
                        var item = loginUserInfo.permission.permission_items[i];
                        var moduleUrl = '';
                        if (item.module_id == Const.PERMISSION_MODULE_ID.ACCESS_CONTROL) {
                            moduleUrl = Const.MODULE_URL.ACCESS;
//                        } else if (item.module_id == Const.PERMISSION_MODULE_ID.DASHBOARD) {
//                            moduleUrl = Const.MODULE_URL.DASHBOARD;
                        } else if (item.module_id == Const.PERMISSION_MODULE_ID.DEVICE) {
                            moduleUrl = Const.MODULE_URL.DEVICE;
                        } else if (item.module_id == Const.PERMISSION_MODULE_ID.MONITORING) {
                            moduleUrl = Const.MODULE_URL.MONITORING;
                        } else if (item.module_id == Const.PERMISSION_MODULE_ID.SETTING) {
                            moduleUrl = Const.MODULE_URL.SETTING;
                        } else if (item.module_id == Const.PERMISSION_MODULE_ID.USER) {
                            moduleUrl = Const.MODULE_URL.USER;
                        }
                        if (url.indexOf(moduleUrl) > -1) {
                            return true;
                        }
                    }
                    return false;
                }
                return true;
            },
            getModuleNameByUrl: function (url) {
                if (url.indexOf(Const.MODULE_URL.USER) > -1) {
                    return privilegeService.getModuleNameByModuleId(Const.PERMISSION_MODULE_ID.USER);
                } else if (url.indexOf(Const.MODULE_URL.DEVICE) > -1) {
                    return privilegeService.getModuleNameByModuleId(Const.PERMISSION_MODULE_ID.DEVICE);
                } else if (url.indexOf(Const.MODULE_URL.ACCESS) > -1) {
                    return privilegeService.getModuleNameByModuleId(Const.PERMISSION_MODULE_ID.ACCESS_CONTROL);
                } else if (url.indexOf(Const.MODULE_URL.MONITORING) > -1) {
                    return privilegeService.getModuleNameByModuleId(Const.PERMISSION_MODULE_ID.MONITORING);
                } else if (url.indexOf(Const.MODULE_URL.SETTING) > -1) {
                    return privilegeService.getModuleNameByModuleId(Const.PERMISSION_MODULE_ID.SETTING);
                } else if (url.indexOf(Const.MODULE_URL.DASHBOARD) > -1) {
                    return privilegeService.getModuleNameByModuleId(Const.PERMISSION_MODULE_ID.DASHBOARD);
                }
            }
        };
        return privilegeService;
    }]);
    // name field filter, 검색결과가 없을 경우 noData 를 array에 담아 return,

    app.directive('ngPrivilege', ['$parse', 'Auth', 'Const', '$location', '$timeout', 'PrivilegeService', function ($parse, Auth, Const, $location, $timeout, PrivilegeService) {
        return {
            restrict: 'A',
            templateUrl: '',
//            scope: {
//                ngPrivilege: '@',
//                adminOnly: '@',
//                hideWhenNotAllowed: '@'
//            },
            link: function ($scope, $element, $attrs) {
                var i,
                    moduleName = $attrs.ngPrivilege,
                    permissionType = Const.PERMISSION_TYPE,
                    loginUserInfo = Auth.getUser(),
                    hideWhenNotAllowed = $attrs.hideWhenNotAllowed,
                    privilege;
//                console.log("ngPrivilege::"+ $attrs.ngPrivilege);

//                CanRead: 1,
//                CanUpdate: 2,
//                CanCreate: 4,
//                CanDelete: 8,
//                CanHandleAlarm: 16

                if (moduleName == undefined || moduleName.length == 0) {
                    moduleName = PrivilegeService.getModuleNameByUrl($location.$$url);
                }
                if (hideWhenNotAllowed == undefined || hideWhenNotAllowed.length == 0) {
                    hideWhenNotAllowed = false;
                } else if (hideWhenNotAllowed == "true" || hideWhenNotAllowed == true) {
                    hideWhenNotAllowed = true;
                }

                function preventEvent($event) {
                    var event = $event || window.event;

                    if (event.preventDefault) event.preventDefault();
                    if (event.returnValue) event.returnValue = false;

                    return false;
                }

                function CanRead() {
                    console.log("read only")
                    $timeout(function () {
                        $element.addClass('disabled');

                        $element.attr('disabled', true);

                        $element.unbind('click');
                        $element.on('click', preventEvent);
                        $element.find('a').addClass('disabled');
                        $element.find('a').unbind('click');

                        angular.element('input[type=\'range\']').attr('disabled', true)
                    });

                    $timeout(function () {
                        $element.find('div').addClass('disabled');
                        $element.find('div').on('focusin', function () {
                            angular.element(this).addClass('disabled');
                            angular.element(this).removeClass('open');
                        });
                    }, 200);
                }


                if (loginUserInfo) {
                    var moduleId = PrivilegeService.getModuleId(moduleName);
                    privilege = PrivilegeService.getPermissionItem(moduleId, loginUserInfo.permission.permission_items);
                }

                var getter = $parse($attrs.adminOnly);
                var adminOnly = getter($scope);

                if (adminOnly == true) {
                    if (loginUserInfo.permission.id != Const.PERMISSION.ADMIN) {
                        if (hideWhenNotAllowed == true) {
                            $element.hide();
                        } else {
                            CanRead();
                        }
                    }
                } else {
                    if (privilege && privilege.permission & permissionType.CanRead) {
                        if (hideWhenNotAllowed == true) {
                            $element.hide();
                        } else {
                            CanRead();
                        }
                    } else if (privilege && privilege.permission & permissionType.CanUpdate) {

                    } else if (privilege && privilege.permission & permissionType.CanHandleAlarm) {

                    } else {
                        if (hideWhenNotAllowed == true) {
                            $element.hide();
                        } else {
                            if (moduleId != Const.PERMISSION_MODULE_ID.DASHBOARD && moduleId != Const.PERMISSION_MODULE_ID.SETTING) {
                                CanRead();
                                console.log("ngPrivilege::can read::" + moduleName);
                            } else {
                                console.log("ngPrivilege::can update");
                            }
                        }
                    }
                }


            },
            controller: function ($scope, $element, $attrs) {

            }
        };
    }]);

    return app;
});