'use strict';

define(['modules/common/services/auth'], function () {

    var app = angular.module('routeModule', [
        'ngResource',
        'ngRoute',
        'ngCookies',
        'ngSanitize',
        'routeResolverModule',
        'privilegeModule',
        'messageModule'
    ]);

    app.config([ '$routeProvider', 'routeResolverProvider', '$controllerProvider', '$compileProvider', '$filterProvider', '$provide', '$httpProvider', function ($routeProvider, routeResolverProvider, $controllerProvider, $compileProvider, $filterProvider, $provide, $httpProvider) {

        app.register =
        {
            controller: $controllerProvider.register,
            directive: $compileProvider.directive,
            filter: $filterProvider.register,
            factory: $provide.factory,
            service: $provide.service
        };

        routeResolverProvider.routeConfig.setBaseDirectory('modules/');

        var route = routeResolverProvider.route;

        $routeProvider
            .when('/', route.resolve('Dashboard', 'dashboard/'))
            .when('/dashboard', route.resolve('Dashboard', 'dashboard/'))
            .when('/main', {
                templateUrl: '../views/main.html',
                controller: 'MainCtrl'

            })
            .when('/login', route.resolve('Login', 'login/'))
            .when('/user', route.resolve('User', 'user/'))
//            .when('/user/add-user/:user_group_id?', route.resolve('UserInfo', 'user/'))
//            .when('/user/user-info/:user_id', route.resolve('UserInfo', 'user/'))

            .when('/device', route.resolve('Device', 'device/'))
            .when('/device/edit/:device_id?', route.resolve('DeviceInfo', 'device/'))

            .when('/setting', route.resolve('Setting', 'setting/'))
            .when('/setting/schedule', route.resolve('Schedule', 'setting/'))
            .when('/setting/holiday', route.resolve('Holiday', 'setting/'))
            .when('/setting/card', route.resolve('Card', 'setting/'))
//            .when('/setting/card-batch-enroll', route.resolve('CardBatchEnroll', 'setting/'))

            .when('/access-control/ac-doors', route.resolve('Doors', 'accessControl/'))
            .when('/access-control/ac-groups', route.resolve('AccessGroups', 'accessControl/'))
            .when('/access-control/ac-levels', route.resolve('AccessLevels', 'accessControl/'))
            .when('/access-control/ac-zones', route.resolve('Zones', 'accessControl/'))

            .when('/monitoring', route.resolve('Monitoring', 'monitoring/'))

            .when('/setting/accounts', route.resolve('Accounts', 'setting/'))
            .when('/setting/preference', route.resolve('Preference', 'setting/'))
            .when('/setting/cardFormat', route.resolve('CardFormat', 'setting/'))
            .when('/setting/sia26bit', route.resolve('Sia26bit', 'setting/'))
            .when('/setting/mifare', route.resolve('Mifare', 'setting/'))
            .when('/setting/server', route.resolve('Server', 'setting/'))
            .when('/setting/triggerActions', route.resolve('TriggerActions', 'setting/'))
            .when('/setting/usbdevices', route.resolve('UsbDevices', 'setting/'))
            .when('/setting/badging', route.resolve('Badging', 'setting/'))
            .when('/setting/alert', route.resolve('Alert', 'setting/'))
            .when('/setting/https', route.resolve('Https', 'setting/'))
//            .when('/setting/addAlarm', route.resolve('AddAlarm', 'setting/'))

            .when('/monitoring/add-gridmonitor', {
                templateUrl: 'views/monitoring/add-gridmonitor.html',
                controller: 'MonitoringAddGridmonitorCtrl'
            })
            .when('/monitoring/tview-monitoring', {
                templateUrl: 'views/monitoring/tableView.html',
                controller: 'MonitoringTviewMonitoringCtrl'
            })

            .when('/monitoring/grid-monitoring', {
                templateUrl: 'views/monitoring/grid-monitoring.html',
                controller: 'MonitoringGridMonitoringCtrl'
            })

            .when('/main', {
                templateUrl: 'views/main.html',
                controller: 'MainCtrl'
            })
            .when('/msgDlgSample', {
                templateUrl: 'views/msgDlgSample.html',
                controller: 'MsgdlgsampleCtrl'
            })
            .when('/popupDlgSample', {
                templateUrl: 'views/popupDlgSample.html',
                controller: 'PopupdlgsampleCtrl'
            })
            .when('/progressbarSample', {
                templateUrl: 'views/progressbarSample.html',
                controller: 'ProgressbarsampleCtrl'
            })
            .when('/accordionPanel', {
                templateUrl: 'views/accordionPanel.html',
                controller: 'AccordionpanelCtrl'
            })
            .when('/404', {
                templateUrl: '404.html',
                access_level: 1
            })
            .when('/fingerprint', {
                templateUrl: 'views/fingerprint.html',
                controller: 'FingerprintCtrl'
            })
            .when('/sample', {
                templateUrl: 'views/sample.html',
                controller: 'SampleCtrl'
            })
            .when('/sample/select-range', {
                templateUrl: 'views/sample/select-range.html',
                controller: 'SampleCtrl'
            })
            .when('/sample/datatable', {
                templateUrl: 'views/sample/datatable.html',
                controller: 'SampleCtrl'
            })
            .when('/sample/reordertable', {
                templateUrl: 'views/sample/reordertable.html',
                controller: 'SampleCtrl'
            })
            .when('/logout', {
                controller: 'LogoutCtrl'
            })
            .when('/sample/add-user-page', {
                templateUrl: 'views/sample/add-user-page.html',
                controller: 'SampleAddUserPageCtrl'
            })
            .when('/users', {
                templateUrl: 'views/users.html',
                controller: 'UsersCtrl'
            })
            .when('/user/users-report', {
                templateUrl: 'views/report/viewer-dlg.html',
                controller: 'ReportViewerDlgCtrl'
            })
            .when('/user/card-registration-dlg', {
                templateUrl: 'views/user/card-registration-dlg.html',
                controller: 'UserCardRegistrationDlgCtrl'
            })
            .when('/sample/user-list-print-preview', {
                templateUrl: 'views/user/user-list-print-preview.html',
                controller: 'UserListPrintPreviewCtrl'
            })
            .when('/report', {
                templateUrl: 'views/report/report.html',
                controller: 'ReportReportCtrl'
            })
            .when('/report/report', {
                templateUrl: 'views/report/report.html',
                controller: 'ReportReportCtrl'
            })
            .when('/time/times', {
                templateUrl: 'views/time/times.html',
                controller: 'TimeTimesCtrl'
            })
            .when('/access/ac-level', {
                templateUrl: 'views/access/accessLevels.html',
                controller: 'AccessAcLevelCtrl'
            })
            .when('/access/add-door/:door_group_info?', {
                templateUrl: 'views/access/doorInfo.html',
                controller: 'AccessDoorInfoCtrl'
            })
            .when('/access/door-info/:id?', {
                templateUrl: 'views/access/doorInfo.html',
                controller: 'AccessDoorInfoCtrl'
            })
            .when('/access/add-zone', {
                templateUrl: 'views/access/add-zone.html',
                controller: 'AccessAddZoneCtrl'
            })
            .when('/access/add-level', {
                templateUrl: 'views/access/acLevelInfo.html',
                controller: 'AccessLevelInfoCtrl'
            })
            .when('/access/level-info/:id?', {
                templateUrl: 'views/access/acLevelInfo.html',
                controller: 'AccessLevelInfoCtrl'
            })
            .when('/access/add-group', {
                templateUrl: 'views/access/acGroupInfo.html',
                controller: 'AccessGroupInfoCtrl'
            })
            .when('/access/group-info/:id?', {
                templateUrl: 'views/access/acGroupInfo.html',
                controller: 'AccessGroupInfoCtrl'
            })
            .when('/access/add-schedule', {
                templateUrl: 'views/access/add-schedule.html',
                controller: 'AccessAddScheduleCtrl'
            })
            .when('/time/add-tareport', {
                templateUrl: 'views/time/add-tareport.html',
                controller: 'TimeAddTareportCtrl'
            })
            .when('/time/paycode-list', {
                templateUrl: 'views/time/paycode-list.html',
                controller: 'TimePaycodeListCtrl'
            })
            .when('/time/shift-list', {
                templateUrl: 'views/time/shift-list.html',
                controller: 'TimeShiftListCtrl'
            })
            .when('/time/shifttemp-list', {
                templateUrl: 'views/time/shifttemp-list.html',
                controller: 'TimeShifttempListCtrl'
            })
            .when('/time/rule-list', {
                templateUrl: 'views/time/rule-list.html',
                controller: 'TimeRuleListCtrl'
            })
            .when('/time/shiftgroup-list', {
                templateUrl: 'views/time/shiftgroup-list.html',
                controller: 'TimeShiftgroupListCtrl'
            })
            .when('/time/timecard-list', {
                templateUrl: 'views/time/timecard-list.html',
                controller: 'TimeTimecardListCtrl'
            })
            .when('/time/add-paycode', {
                templateUrl: 'views/time/add-paycode.html',
                controller: 'TimeAddPaycodeCtrl'
            })
            .when('/time/add-shift', {
                templateUrl: 'views/time/add-shift.html',
                controller: 'TimeAddShiftCtrl'
            })
            .when('/time/add-shifttemp', {
                templateUrl: 'views/time/add-shifttemp.html',
                controller: 'TimeAddShifttempCtrl'
            })
            .when('/time/add-rule', {
                templateUrl: 'views/time/add-rule.html',
                controller: 'TimeAddRuleCtrl'
            })
            .when('/time/add-shiftgroup', {
                templateUrl: 'views/time/add-shiftgroup.html',
                controller: 'TimeAddShiftgroupCtrl'
            })
            .when('/time/add-timecard', {
                templateUrl: 'views/time/add-timecard.html',
                controller: 'TimeAddTimecardCtrl'
            })
            .when('/etc/help', {
                templateUrl: 'views/etc/help.html',
                controller: 'EtcHelpCtrl'
            })
            .when('/etc/help-device', {
                templateUrl: 'views/etc/help-device.html',
                controller: 'EtcHelpDeviceCtrl'
            })
            .when('/setting/smartcard', {
                templateUrl: 'views/setting/smartcard.html',
                controller: 'SettingSmartcardCtrl'
            })
            .when('/report/print-option', {
                templateUrl: 'views/report/print-option.html',
                controller: 'ReportPrintOptionCtrl'
            })
            .when('/access/add-multipledoor', {
                templateUrl: 'views/access/add-multipledoor.html',
                controller: 'AccessAddMultipledoorCtrl'
            })
            .when('/time/add-tasetting', {
                templateUrl: 'views/time/add-tasetting.html',
                controller: 'TimeAddTasettingCtrl'
            })
            .when('/sample/slider', {
                templateUrl: 'views/sample/slider.html',
                controller: 'SampleSliderCtrl'
            })
            .when('/sample/db-sample', {
                templateUrl: 'views/sample/db-sample.html',
                controller: 'SampleDbSampleCtrl'
            })
            .otherwise({
                redirectTo: '/'
            });

        $provide.factory('myHttpInterceptor', [ '$q', '$rootScope', '$location', 'Util', function ($q, $rootScope, $location, Util) {
            return {
                'request': function (config) {
                    if ($location.url().indexOf('/login') === -1) {
                        $rootScope.$broadcast('checkAuthState', $location.url());
                    } else {
//                        $rootScope.$broadcast('menuUrlChange');
                    }
                    return config || $q.when(config);
                },
                'requestError': function (rejection) {

                    return $q.reject(rejection);
                },


                'response': function (response) {
//				console.debug('response::'+response);
                    return response || $q.when(response);

                },

                'responseError': function (rejection) {
                    switch (rejection.status) {
                        case 401 :
                            if (rejection.config.url.indexOf('/logout') == -1) {
                                $rootScope.$broadcast('auth:loginRequired', rejection.config.url);
                            }
                            break;
                        case 403 :
                            $rootScope.$broadcast('auth:forbidden');
                            break;

                        case 404 :
                            $rootScope.$broadcast('page:notFound');
                            break;

                        case 500 :
                            $rootScope.$broadcast('server:error');
                            break;
                    }

                    return $q.reject(rejection);
                }
            };

        }]);

        $httpProvider.interceptors.push('myHttpInterceptor');
    }]);

    app.run([ '$rootScope', '$location', 'Auth', '$cookieStore', 'API', 'Const', 'Util', 'PrivilegeService', 'MessageProperties', 'MessageService', function ($rootScope, $location, Auth, $cookieStore, API, Const, Util, PrivilegeService, MessageProperties, MessageService) {

            $rootScope.$on('page:notFound', function (a) {
                //$location.path('/404');
            });

            $rootScope.checkFullscreen = function () {
                if (Const.FULLSCREEN_PAGES.indexOf($location.path()) > 0) {
                    $('.panelAreaOpen>header, footer, .sm-container>nav').hide();
                    $('.panelAreaOpen').addClass('fullscreen');
                } else {
                    $('.panelAreaOpen>header, footer, .sm-container>nav').show();
                    $('.panelAreaOpen').removeClass('fullscreen');
                }
            };
            $rootScope.$on('$routeChangeStart', function (evt, next, curr) {
//            console.debug('$location.url() :' + $location.url()); // 과도한 로그로 주석처리함
                $rootScope.checkFullscreen();
                if (PrivilegeService.isUrlAuthorized($location.$$url) == false) {
                    MessageService.displayErrorMessage(MessageProperties.get("error.accessDeniedDescription"));
                    $location.path("/");
                }

                angular.element('#contextMenu').remove();
            });

            $rootScope.$on('$routeChangeSuccess', function (evt, next, curr) {
                var notSelectMenuPages = [
                    '/setting'
                ];
//            if (next.originalPath && notSelectMenuPages.indexOf(next.originalPath) >= 0) {
//                $rootScope.$broadcast('menuUrlChange');
//            }

                if (angular.element('.panelArea').is(":visible")) {
                    angular.element('.wrap').addClass('tree-bg').removeClass('menu-bg');
                } else {
                    angular.element('.wrap').addClass('menu-bg').removeClass('tree-bg');
                }
            });

            $rootScope.$on('checkAuthState', function (evt, callingRoute) {
//            console.log("checkAuthState");
                if (Auth.isLoggedIn() !== true) {
                    console.log("checkAuthState: not logged in calling from " + callingRoute);
                    if (callingRoute.toString().substr(0, 6) != '/login') {
                        Auth.setBeforeUrl(callingRoute);
                        Auth.doLogout();
                    }
                }
            });

            if (Auth.isLoggedIn() === true) {
//            console.log("checkAuthState: logged in");
                var _user = Auth.getUser();
                API.setSessionID(_user.sessionId);

                $rootScope.$broadcast('LoginToSuccess');
            }

            $rootScope.$on('auth:loginRequired', function (evt, callingRoute) {
                Util.info("auth:loginRequired: calling from " + callingRoute);
                Auth.setBeforeUrl(callingRoute);
                Auth.doLogout();
            });

//        $rootScope.$on('LoginToSuccess', function () {
//            console.log('LoginToSuccess => redirect to dashboard');
//            $location.path('/dashboard');
//        });
            //publish Control Code area;
            $rootScope.$on('$viewContentLoaded', function () {
                $rootScope.checkFullscreen();

//            $("div.select > a").click(function (){
//                if ( $(this).parent().hasClass("disabled") ){
//
//                }else{
//                    if ( $(this).parent().hasClass("open") ){
//                        $(this).parent().removeClass("open");
//                    }else{
//                        $(this).parent().addClass("open");
//                    }
//                }
//                return false;
//            });
//            $("div.select").mouseleave(function (){
//                $(this).closest(".select").removeClass("open");
//            });


//            $(".boardSearch input").focusin(function() {
//                $(this).parent().addClass("focus");
//                $(this).parent().addClass("open");
//            }).focusout(function() {
//                $(this).parent().removeClass("open");
//                $(this).parent().removeClass("focus");
//            });

                $('.btnFuncDark:has("img")').each(function () {
                    if (!$(this).hasClass("disabled")) {
                        var img = $(this).find("img").attr('src');
                        $(this).find("img").attr('src', img.replace("_disabled.", "."));
                    } else {
                        var img = $(this).find("img").attr('src');
                        $(this).find("img").attr('src', img.replace(".", "_disabled."));
                    }
                });

                $(".filterCnt .title > li").each(function () {
                    var filterwidth = $(this).width() + 2;
                    $(this).find(".filter").css("min-width", filterwidth);
                });

                $(".selectUserList .scrollArea").each(function () {
                    if ($(this).height() > 432) {
                        $(this).addClass("scrollY");
                    } else {
                        $(this).removeClass("scrollY");
                    }
                });

                $(".invisibleAct").click(function () {
                    if ($(this).parent().hasClass("disabled")) {

                    } else {
                        if ($(this).parent().hasClass("open")) {
                            $(this).parent().removeClass("open");
                        } else {
                            $(this).parent().addClass("open");
                        }
                    }
                    return false;
                });

                //tree icon
                for (var i = 0; i < $(".tree li a, .treeTypeGrid li a").length; i++) {
                    if ($(".tree li a, .treeTypeGrid li a").eq(i).parent().find("> ul").length == 0) {
                        $(".tree li a, .treeTypeGrid li a").eq(i).parent().addClass("noDepth");
                    }
                }
                for (i = 0; i < $(".accessRule .cntTypeB li a").length; i++) {
                    if ($(".accessRule .cntTypeB li a").eq(i).parent().find("> ul").length == 0) {
                        $(".accessRule .cntTypeB li a").eq(i).parent().addClass("noDepth");
                    }
                }
            });
//
//            angular.element(window).on('contextmenu', function($event) {
//                var event = $event || window.event;
//
//                if(event.preventDefault) event.preventDefault();
//                if(event.returnValue) event.returnValue = false;
//
//                event.cancelBubble = true;
//            });

        }]
    );

    return app;
});
