'use strict';

define([
    'modules/common/commonModule'
], function () {
    var app = angular.module('commonModule');
    app.service('AccessControlAgent', ['$log', '$q', '$rootScope', 'API', 'Const', function ($log, $q, $rootScope, API, Const) {
        var
            accessGroupParam,
            ACCESS_GROUP_DATA,
            ACCESS_LEVEL_DATA,
            httpTimeOut = {},
            canceller = {};

        $rootScope.$on('LoginToSuccess', function () {
//            console.log('Login to Success, AccessControlAgentStart!.');
            API.get('api/' + 'access_groups').then(function (data) {
                ACCESS_GROUP_DATA = data.AccessGroupCollection.rows;
            });
        });

        function getAccessGroups(param, deferFunctionName) {
            var defer = $q.defer();
            API.get('access_groups', deferFunctionName, param).then(function (data) {
                window.sessionStorage.setItem('getAccessGroups', angular.toJson(data));
                defer.resolve(data);
            });
            return defer.promise;
        }

        function getOneAcGroup(acGroupId) {

            var defer = $q.defer();

            API.get('access_groups' + '/' + acGroupId).then(function (data) {
                window.sessionStorage.setItem('acGroupInfo', angular.toJson(data.AccessGroup));
                defer.resolve(data.AccessGroup);
            });

            return defer.promise;
        }

        function postOneAcGroup(acGroup) {
            var defer = $q.defer();

            API.post('access_groups', 'postOneAcGroup', {AccessGroup: acGroup}).then(function success(data) {
                defer.resolve(data);
            }, function error(data) {
                defer.reject(data);
            });

            return defer.promise;
        }

        function putOneAcGroup(param) {
            var defer = $q.defer();

            API.put('access_groups/' + param.id, 'putOneAcGroup', {AccessGroup: param.acGroupInfo}).then(function success(data) {
                defer.resolve(data);
            }, function error(data) {
                defer.reject(data);
            });

            return defer.promise;
        }

        function deleteAcGroups(acGroupIds) {
            var defer = $q.defer();

            if (acGroupIds.constructor == Array) {
                API.delete('access_groups?id=' + (acGroupIds.join ? acGroupIds.join('+') : acGroupIds), 'deleteAcGroups').then(function success(data) {
                    $rootScope.$broadcast('ReloadAcGroupsDataGrid');
                    defer.resolve(data);
                });

            } else if (acGroupIds.constructor == String) {
                API.delete('access_groups?id=*', 'deleteAcGroups').then(function success(data) {
                    $rootScope.$broadcast('ReloadAcGroupsDataGrid');
                    defer.resolve(data);
                });
            }

            return defer.promise;
        }

        function getAccessLevels(param, deferFunctionName) {

            var defer = $q.defer();

            if (ACCESS_LEVEL_DATA && !param) {
                defer.resolve(ACCESS_LEVEL_DATA);
            } else {
                API.get('access_levels', deferFunctionName, param).then(function (data) {
                    ACCESS_LEVEL_DATA = data.AccessLevelCollection.rows;
                    defer.resolve(data);
                });
            }

            return defer.promise;
        }

        function getOneAcLevel(acLevelId) {

            var defer = $q.defer();

            API.get('access_levels' + '/' + acLevelId, 'getOneAcLevel').then(function (data) {
                window.sessionStorage.setItem('acLevelInfo', angular.toJson(data.AccessLevel));
                defer.resolve(data.AccessLevel);
            });

            return defer.promise;
        }

        function postOneAcLevel(acLevel) {
            var defer = $q.defer();

            API.post('access_levels', 'postOneAcLevel', {AccessLevel: acLevel}).then(function success(data) {
                // Access Level List Reload( in accessLevelController)
                $rootScope.$broadcast('ReloadAcLevelsDataGrid');
                defer.resolve(data);
            });

            return defer.promise;
        }

        function putOneAcLevel(param) {
            var defer = $q.defer();

            API.put('access_levels/' + param.id, 'putOneAcLevel', {AccessLevel: param.acLevelInfo}).then(function success(data) {
                // Access Level List Reload( in accessLevelController)
                $rootScope.$broadcast('ReloadAcLevelsDataGrid');
                defer.resolve(data);
            });

            return defer.promise;
        }

        function deleteAcLevels(acLevelIds) {
            var defer = $q.defer();

            if (acLevelIds.constructor == Array) {
                API.delete('access_levels?id=' + (acLevelIds.join ? acLevelIds.join('+') : acLevelIds), 'deleteAcLevels').then(function success(data) {
                    $rootScope.$broadcast('ReloadAcLevelsDataGrid');
                    defer.resolve(data);
                });

            } else if (acLevelIds.constructor == String) {
                API.delete('access_levels?id=*', 'deleteAcLevels').then(function success(data) {
                    $rootScope.$broadcast('ReloadAcLevelsDataGrid');
                    defer.resolve(data);
                });
            }

            return defer.promise;
        }


        return {
            getAccessGroups: getAccessGroups,
            getOneAcGroup: getOneAcGroup,
            postOneAcGroup: postOneAcGroup,
            putOneAcGroup: putOneAcGroup,
            deleteAcGroups: deleteAcGroups,
            getAccessLevels: function (param, deferFunctionName) {
                return getAccessLevels(param, deferFunctionName);
            },
            getOneAcLevel: getOneAcLevel,
            postOneAcLevel: postOneAcLevel,
            putOneAcLevel: putOneAcLevel,
            deleteAcLevels: deleteAcLevels
        }
    }]);
})