'use strict';

define([
    'modules/common/commonModule',
    'modules/common/services/util',
    'modules/common/services/const'
], function () {

    var app = angular.module('commonModule');
    app.service('API', ['$http', '$q', '$location', '$timeout', 'Util', 'Const', 'MessageService', 'MessageProperties', function API($http, $q, $location, $timeout, Util, Const, MessageService, MessageProperties) {

        var httpTimeOut = {},
            canceller = {};
        var sessionId = undefined;

        var taskResponseList = {};

        // AngularJS will instantiate a singleton by calling "new" on this function
        var _requestManager = {
            requests: [],
            checkSameRequests: function (APIurl) {
                for (var i in requests) {
                    var req = requests[i];
                    if (req.pathname == APIurl.pathname) {
                        return true;
                    }
                }
                return false;
            }
        };

        function buildHttpResponseObject(data, status, headers, config) {
            if (data == undefined || data == "") {
                data = {};
            }
            data.httpResponseStatus = status;
            data.httpResponseHeaders = headers;
            return data;
        }

        function checkTaskProgress(taskId, options, defer, data, status, headers, config) {
            Util.debug("checkTaskProgress begin for task " + taskId + " " + options.url);

            var taskResponseObject = api.getTaskResponse(taskId);
            if (taskResponseObject == undefined) {
                Util.debug("task " + taskId + " " + options.url + " new task response object created");
                api.addTaskResponse({
                    lookupCount: 1,
                    Response: {
                        task_id: taskId
                    }
                });
                $timeout(function () {
                    checkTaskProgress(taskId, options, defer, data, status, headers, config);
                }, 1000);
            } else {
                if (taskResponseObject.Response.code != undefined) {
//                    Util.debug("task " + taskId + " " + options.url, taskResponseObject);
                    if ((taskResponseObject.Response.code == Const.SERVER_RESPONSE_CODE.SUCCESSFUL ||
                        taskResponseObject.Response.code == Const.SERVER_RESPONSE_CODE.PARTIAL_SUCCESS)
                        && taskResponseObject.Response.task_id == taskId) {
                        Util.info("task " + taskId + " " + options.url + " successful response received", taskResponseObject);
                        displayPartialSuccess(options, defer, taskResponseObject, status, headers, config);
                        api.removeTaskResponse(taskId);
                    } else {
                        Util.info("task " + taskId + " " + options.url + " failed response received", taskResponseObject);
                        displayError(options, defer, taskResponseObject, status, headers, config);
                        api.removeTaskResponse(taskId);
                    }
                } else {
                    if (taskResponseObject.lookupCount > Const.MAX_TASK_RESPONSE_LOOKUP_COUNT) {
                        taskResponseObject.lookupCount = 1;
                        Util.info("task " + taskId + " " + options.url + " lookup max limit " + Const.MAX_TASK_RESPONSE_LOOKUP_COUNT + " reached checking to see if task is still running in server side");
                        api.getWithOptions({
                            url: 'pending_request/' + taskId,
                            displaySuccessPopoup: false,
                            displayErrorPopup: false,
                            displayLoading: false,
                            encryptData: options.encryptData
                        }).then(function (data) {
                            Util.info("task " + taskId + " still running " + options.url);
                            api.incrementTaskResponseLookupCount(taskId);
                            $timeout(function () {
                                checkTaskProgress(taskId, options, defer, data, status, headers, config);
                            }, 1000);
                        }, function error(data) {
                            Util.error("task " + taskId + " " + options.url + " not found and response not received", data);
                            displayTaskProgressError(taskId, options, defer, data);
                        });
                    } else {
                        Util.debug("task " + taskId + " " + options.url + " lookup count " + taskResponseObject.lookupCount + " incremented");
                        api.incrementTaskResponseLookupCount(taskId);
                        $timeout(function () {
                            checkTaskProgress(taskId, options, defer, data, status, headers, config);
                        }, 1000);
                    }
                }
            }
//            api.getWithOptions({
//                url: 'pending_request/' + taskId,
//                displaySuccessPopoup: false,
//                displayErrorPopup: false,
//                displayLoading: false,
//                encryptData: options.encryptData
//            }).then(function (data) {
//                if (data != undefined && data.Response != undefined && data.Response.code != undefined) {
//                    if (data.Response.code == Const.SERVER_RESPONSE_CODE.TASK_NOT_FOUND) {
//                        displayPartialSuccess(options, defer, data, status, headers, config);
//                    } else if (data.Response.code == Const.SERVER_RESPONSE_CODE.SUCCESSFUL) {
//                        $timeout(function () {
//                            checkTaskProgress(taskId, options, defer, data, status, headers, config);
//                        }, 1000);
//                    } else {
//                        Util.error("pending_request/" + taskId + " response invalid", data);
//                        displayTaskProgressError(taskId, options, defer, data);
//                    }
//                } else {
//                    Util.error("pending_request/" + taskId + " response invalid", data);
//                    displayTaskProgressError(taskId, options, defer, data);
//                };
//
//            }, function error(data) {
//                if (data != undefined && data.Response != undefined && data.Response.code != undefined &&
//                    data.Response.code == Const.SERVER_RESPONSE_CODE.TASK_NOT_FOUND) {
//                    Util.info("task " + taskId + " completed " + options.url);
//                    displayPartialSuccess(options, defer, data, status, headers, config);
//                } else {
//                    Util.error("pending_request/" + taskId + " failed", data);
//                    displayTaskProgressError(taskId, options, defer, data);
//                }
//            });
        }

        function displayPartialSuccess(options, defer, data, status, headers, config, successCallback) {

            api.closeLoadingPopup(options.displayLoading);

            var allSuccessful = MessageService.displayPartialSuccessfulMessage(data, options.displaySuccessPopoup, options.messagePopupDivId, successCallback);
            Util.debug("displayPartialSuccess", allSuccessful);
            if (options.deferFunctionName != undefined) {
                httpTimeOut[options.deferFunctionName] = undefined;
                data = buildHttpResponseObject(data, status, headers, config);
                if (allSuccessful) {
                    defer.resolve(data);
                } else {
                    defer.reject(data);
                }
            }
        }

        function displayError(options, defer, data, status, headers, config) {

            api.closeLoadingPopup(options.displayLoading);

            MessageService.displayErrorMessage(data, options.displayErrorPopup, undefined, options.messagePopupDivId);
            data = buildHttpResponseObject(data, status, headers, config);
            defer.reject(data);
        }

        function displayTaskProgressError(taskId, options, defer, data) {

            api.closeLoadingPopup(options.displayLoading);

            MessageService.displayErrorMessage(MessageProperties.get("serverResponse.1000"));
            if (options.deferFunctionName != undefined) {
                httpTimeOut[options.deferFunctionName] = undefined;
                defer.resolve(data);
            }
        }

        var api = {
            addTaskResponse: function (taskResponseObject) {
                var existingObject = undefined;
                if (taskResponseObject.Response != undefined && taskResponseObject.Response.task_id != undefined) {
                    existingObject = api.getTaskResponse(taskResponseObject.Response.task_id);
                }
                if (existingObject != undefined) {
                    taskResponseObject.lookupCount = existingObject.lookupCount;
                }
                taskResponseList[taskResponseObject.Response.task_id] = taskResponseObject;
            },
            incrementTaskResponseLookupCount: function (taskId) {
                var taskResponseObject = api.getTaskResponse(taskId);
                if (taskResponseObject == undefined) {
                    api.addTaskResponse({
                        lookupCount: 1
                    });
                } else {
                    taskResponseObject.lookupCount++;
                }
            },
            getTaskResponse: function (taskId) {
                return taskResponseList[taskId];
            },
            removeTaskResponse: function (taskId) {
                taskResponseList[taskId] = undefined;
            },
            buildEncryptOptions: function (keyInHexString) {
                var ivString = keyInHexString.slice(0, 32);
                var encOptions = {
                    mode: CryptoJS.mode.CBC,
                    padding: CryptoJS.pad.Pkcs7,
                    iv: CryptoJS.enc.Hex.parse(ivString)
                };
                Util.debug("buildEncryptOptions", keyInHexString, ivString);
                return encOptions;
            },
            encryptJson: function (url, method, encryptData, bsSessionKey, data) {
                if (Util.getLocalStorageObject(Const.LOCAL_STORAGE_DISABLE_ENCRYPTION) != undefined) {
                    return data;
                }
                if (encryptData == false) {
                    return data;
                }
                if (data == undefined || data.length < 1) {
                    return;
                }
                var protocol = $location.protocol();
                Util.debug(protocol);
                if (protocol == Const.WEB_SERVER_PROTOCOL.HTTPS) {
                    return data;
                }
                var stringData = angular.toJson(data);
                stringData = stringData.trim();
                var key = api.getSessionID();

                if (key == undefined || key == null || key.length < 1) {
                    if (bsSessionKey != undefined) {
                        key = bsSessionKey;
                    }
                }

                Util.debug("encrypt key::" + key);
                var base64DecodedKey = atob(key);
                var keyInHexString = Util.ascii2hex(base64DecodedKey);
                var keyWordArray = CryptoJS.enc.Hex.parse(keyInHexString);
                var encOptions = api.buildEncryptOptions(keyInHexString)

                var encrypted = CryptoJS.AES.encrypt(stringData, keyWordArray, encOptions);
                var encryptedHexText = encrypted.ciphertext.toString();
                var encryptedAsciiText = Util.hex2ascii(encryptedHexText);
                var base64Text = btoa(encryptedAsciiText);
                Util.debug("encrypt::" + method + "::" + url + "::", data);
//                Util.debug(stringData, stringData.length, encryptedHexText, base64Text);

                return base64Text;
            },
            decryptJson: function (url, method, encryptData, bsSessionId, data) {
                if (Util.getLocalStorageObject(Const.LOCAL_STORAGE_DISABLE_ENCRYPTION) != undefined) {
                    return data;
                }
                if (encryptData == false) {
                    return data;
                }
                if (bsSessionId == undefined || bsSessionId.length < 1) {
                    if (angular.equals(url, Const.LOGIN_URL) == false) {
                        return;
                    }
                }
                if (data == undefined || data.length < 1) {
                    return;
                }
                var protocol = $location.protocol();
                Util.debug(protocol);
                if (protocol == Const.WEB_SERVER_PROTOCOL.HTTPS) {
                    return data;
                }
                Util.debug("decrypt key::" + bsSessionId);
                var base64DecodedKey = atob(bsSessionId);
                var keyInHexString = Util.ascii2hex(base64DecodedKey);
                var keyWordArray = CryptoJS.enc.Hex.parse(keyInHexString);
                var encOptions = api.buildEncryptOptions(keyInHexString);

                var cipherParams = CryptoJS.lib.CipherParams.create({
                    ciphertext: CryptoJS.enc.Base64.parse(data)
                });
                var decrypted = CryptoJS.AES.decrypt(
                    cipherParams,
                    keyWordArray,
                    encOptions);

                var decryptedText = undefined;
                var jsonObject = undefined;
                try {
                    decryptedText = decrypted.toString(CryptoJS.enc.Utf8);

                    jsonObject = angular.fromJson(decryptedText);

                    if (jsonObject == undefined) {
                        Util.debug("decrypt::" + method + "::" + url + "::", decryptedText);
                    } else {
                        Util.debug("decrypt::" + method + "::" + url + "::", jsonObject);
                    }
                    return jsonObject;
                } catch (ex) {
                    Util.error("Failed to parse decrypted text", jsonObject, data, decryptedText, ex);
                }
            },
            isLoggedIn: function () {
                var sessionUserObject = Util.getSessionObject(Const.SESSION_STORAGE_KEY.LOGGED_IN_USER);
                return  sessionUserObject != undefined ? true : false;
            },
            openLoadingPopup: function (displayLoading) {
//                console.log("loading image opened");
                if (displayLoading == true) {
                    angular.element("#div_loading_img_dialog").dialog({ title: '', modal: true });
                } else {
                    console.log("loading popup ignored, so popup not opened");
                }
            },
            closeLoadingPopup: function (displayLoading) {
//                console.log("loading image closed");
                if (displayLoading == true) {
                    angular.element("#div_loading_img_dialog").dialog("close");
                } else {
                    console.log("loading popup ignored, so nothing to close");
                }
            },
            /**
             * Options :
             *
             * method,  // required, used for internal use
             * url,     // required
             * deferFunctionName,
             * params,
             * headers,
             * displaySuccessPopoup,
             * displayErrorPopup,
             * displayLoading,
             * messagePopupDivId,
             * encryptData
             * successCallback
             * errorCallback
             * transformRequest
             *
             */
            httpRequest: function (options) {

                var defer = $q.defer();

                if (options.displayLoading == undefined) {
                    options.displayLoading = true;
                }

                if (options.encryptData == undefined) {
                    options.encryptData = true;
                }

                api.openLoadingPopup(options.displayLoading);

                try {
                    if (options.displaySuccessPopoup == undefined) {
                        options.displaySuccessPopoup = false;
                    }
                    if (options.displayErrorPopup == undefined) {
                        options.displayErrorPopup = true;
                    }
                    var config = {};
                    if (options.headers != undefined) {
                        config.headers = options.headers;
                    }
                    if (options.transformRequest != undefined) {
                        config.transformRequest = options.transformRequest;
                    }


                    config.method = options.method;

                    if (options.deferFunctionName == undefined) {
                        if (options.url.indexOf("?") > -1) {
                            options.deferFunctionName = options.url.substr(0, options.url.indexOf("?"));
                        } else {
                            options.deferFunctionName = options.url;
                        }
                        if (options.url.indexOf("/") > -1) {
                            options.deferFunctionName = options.deferFunctionName.replace(/\//g, '_');
                        }
                        options.deferFunctionName = options.method + "_" + options.deferFunctionName;
                    }
                    Util.debug("deferFunctionName", options.deferFunctionName);

                    if (options.deferFunctionName != undefined) {


                        if (httpTimeOut[options.deferFunctionName]) {
                            canceller[options.deferFunctionName].resolve("Cancelled duplicate requests");
                            httpTimeOut[options.deferFunctionName] = undefined;
                            canceller[options.deferFunctionName] = $q.defer();
                        }

                        if (!canceller[options.deferFunctionName]) {
                            canceller[options.deferFunctionName] = $q.defer()
                        }

                        httpTimeOut[options.deferFunctionName] = canceller[options.deferFunctionName].promise;
                        config.timeout = httpTimeOut[options.deferFunctionName];
                    }

                    $('#dimAPIRequest').remove();
                    $('body').append('<div id="dimAPIRequest" class="window-mask transparent" style="display: block; z-index: 9008;"></div>').css('overflow', 'hidden');

                    var APIurl = Util.parseAPIUrl(options.url, Const.API.BASE_URL);

                    var bsSessionKey = undefined;
                    if (options.headers != undefined) {
                        bsSessionKey = options.headers[Const.HTTP_HEADER.BS_SESSION_KEY];
                    }
                    var requestData = api.encryptJson(APIurl.href, options.method, options.encryptData, bsSessionKey, options.params);
                    var xhr;
                    if (Const.HTTP_METHOD.GET == options.method) {
                        if (options.params) {
                            config.params = options.params;
                        }

                        if (!config.params) {
                            config.params = {};
                        }
                        config.params.timestamp = new Date().getTime();

                        xhr = $http.get(APIurl.href, config);
                    } else if (Const.HTTP_METHOD.POST == options.method) {
                        xhr = $http.post(APIurl.href, requestData, config)
                    } else if (Const.HTTP_METHOD.PUT == options.method) {
                        xhr = $http.put(APIurl.href, requestData, config)
                    } else if (Const.HTTP_METHOD.DELETE == options.method) {
                        xhr = $http.delete(APIurl.href, config)
                    }
                    xhr.success(function (responseData, status, headers, config) {
                        var key = api.getSessionID();
                        if (key == undefined) {
                            key = headers(Const.HTTP_HEADER.BS_SESSION_ID);
                        }
                        var data = undefined;
                        try {
                            data = api.decryptJson(config.url, config.method, options.encryptData, key, responseData);
                        } catch (error) {
                            Util.error(error);
                        }
                        $('#dimAPIRequest').remove();
                        $('body').css('overflow', 'auto');
                        if (data != undefined && data.Response != undefined && data.Response.code != undefined &&
                            data.Response.code == Const.SERVER_RESPONSE_CODE.USER_ALREADY_LOGGED_IN) {

                            api.closeLoadingPopup(options.displayLoading);

                            MessageService.displayWarningMessage(MessageProperties.get(
                                    Const.SERVER_RESPONSE_MESSAGE_PREFIX + "." + Const.SERVER_RESPONSE_CODE.USER_ALREADY_LOGGED_IN));
                            data = buildHttpResponseObject(data, status, headers, config);
                            defer.resolve(data);
                        } else {

                            if (data != undefined && data.Response != undefined && data.Response.code != undefined &&
                                data.Response.code == Const.SERVER_RESPONSE_CODE.REQUEST_TIMEOUT) {
                                checkTaskProgress(data.Response.task_id, options, defer, data, status, headers, config);
                            } else {
                                displayPartialSuccess(options, defer, data, status, headers, config, options.successCallback);
                            }
                        }
                    });
                    xhr.error(function (responseData, status, headers, config) {
                        Util.debug("http response::" + status);
                        var data = undefined;
                        if (status == Const.HTTP_STATUS_CODE.NOT_FOUND) {
                            data = {
                                Response: {
                                    code: Const.SERVER_FAIL_CODE.SERVER_ERROR,
                                    status: status,
                                    message: MessageProperties.get("error.resourceNotFound")
                                }
                            };
                        } else {
                            var key = api.getSessionID();
                            if (key == undefined) {
                                key = headers(Const.HTTP_HEADER.BS_SESSION_ID);
                            }
                            if (key == undefined && options != undefined && options.headers != undefined) {
                                Util.info("bs-session-id does not exists in the header, so bs-session-key is used");
                                key = options.headers[Const.HTTP_HEADER.BS_SESSION_KEY];
                            }
                            try {
                                data = api.decryptJson(config.url, config.method, options.encryptData, key, responseData);
                            } catch (error) {
                                Util.error(error);
                            }
                        }
                        $('#dimAPIRequest').remove();
                        $('body').css('overflow', 'auto');

                        displayError(options, defer, data, status, headers, config);

                    });
                    if (options.deferFunctionName != undefined) {
                        return defer.promise;
                    } else {
                        return xhr;
                    }
                } catch (error) {
                    Util.error(APIurl, error);
                    api.closeLoadingPopup(options.displayLoading);
                    defer.reject(error);
                }
                return defer.promise;
            },
            post: function (url, deferFunctionName, params, headers, displaySuccessPopoup, displayErrorPopup, displayLoading, divId) {
                return api.httpRequest({
                    method: Const.HTTP_METHOD.POST,
                    url: url,
                    deferFunctionName: deferFunctionName,
                    params: params,
                    headers: headers,
                    displaySuccessPopoup: displaySuccessPopoup,
                    displayErrorPopup: displayErrorPopup,
                    displayLoading: displayLoading,
                    divId: divId
                });
            },
            postWithOptions: function (options) {
                options.method = Const.HTTP_METHOD.POST;
                return api.httpRequest(options);
            },
            get: function (url, deferFunctionName, params, headers, displaySuccessPopoup, displayErrorPopup, displayLoading, divId) {
                return api.httpRequest({
                    method: Const.HTTP_METHOD.GET,
                    url: url,
                    deferFunctionName: deferFunctionName,
                    params: params,
                    headers: headers,
                    displaySuccessPopoup: displaySuccessPopoup,
                    displayErrorPopup: displayErrorPopup,
                    displayLoading: displayLoading,
                    divId: divId
                });
            },
            getWithOptions: function (options) {
                options.method = Const.HTTP_METHOD.GET;
                return api.httpRequest(options);
            },
            put: function (url, deferFunctionName, params, headers, displaySuccessPopoup, displayErrorPopup, displayLoading, divId) {
                return api.httpRequest({
                    method: Const.HTTP_METHOD.PUT,
                    url: url,
                    deferFunctionName: deferFunctionName,
                    params: params,
                    headers: headers,
                    displaySuccessPopoup: displaySuccessPopoup,
                    displayErrorPopup: displayErrorPopup,
                    displayLoading: displayLoading,
                    divId: divId
                });
            },
            putWithOptions: function (options) {
                options.method = Const.HTTP_METHOD.PUT;
                return api.httpRequest(options);
            },
            del: function (url, deferFunctionName, headers, displaySuccessPopoup, displayErrorPopup, displayLoading, divId) {
                return api.delete(url, deferFunctionName, headers, displaySuccessPopoup, displayErrorPopup, displayLoading, divId);
            },
            delete: function (url, deferFunctionName, headers, displaySuccessPopoup, displayErrorPopup, displayLoading, divId) {
                return api.httpRequest({
                    method: Const.HTTP_METHOD.DELETE,
                    url: url,
                    deferFunctionName: deferFunctionName,
                    headers: headers,
                    displaySuccessPopoup: displaySuccessPopoup,
                    displayErrorPopup: displayErrorPopup,
                    displayLoading: displayLoading,
                    divId: divId
                });
            },
            deleteWithOptions: function (options) {
                options.method = Const.HTTP_METHOD.DELETE;
                return api.httpRequest(options);
            },
            pdf: function (url, deferFunctionName, params, displaySuccessPopoup, displayErrorPopup, displayLoading, divId) {
                return api.httpRequest({
                    method: Const.HTTP_METHOD.POST,
                    url: url,
                    deferFunctionName: deferFunctionName,
                    params: params,
                    headers: {'Content-Type': 'application/pdf'},
                    displaySuccessPopoup: displaySuccessPopoup,
                    displayErrorPopup: displayErrorPopup,
                    displayLoading: displayLoading,
                    divId: divId
                });
            },
            pdfWithOptions: function (options) {
                options.method = Const.HTTP_METHOD.POST;
                options.headers = {'Content-Type': 'application/pdf'};
                return api.httpRequest(options);
            },
            setSessionID: function (paramSessionId) {
                sessionId = paramSessionId;
                $http.defaults.headers.common[Const.HTTP_HEADER.BS_SESSION_ID] = paramSessionId;
            },
            getSessionID: function () {
                return sessionId;
            },
            rejectRequest: function (deferFunctionName) {
                if (httpTimeOut[deferFunctionName]) {
                    canceller[deferFunctionName].resolve("Cancelled requests");
                    httpTimeOut[deferFunctionName] = undefined;
                    canceller[deferFunctionName] = $q.defer();
                }
            }
        };
        return api;
    }]);
    app.service('UUID', function UUID() {
        var uuid = function () {
            function _p8(s) {
                var p = (Math.random().toString(16) + "000000000").substr(2, 8);
                return s ? "-" + p.substr(0, 4) + "-" + p.substr(4, 4) : p;
            }

            return _p8() + _p8(true) + _p8(true) + _p8();
        };

        return uuid;
    });
    app.service('ModifiedFieldExtraction', function fnModifiedFieldExtraction() {
        var fnModifiedFieldExtraction = function (src, dest) {
            var item, ext_user = {};
            for (item in dest) {
                if (angular.isDefined(src[item]) && dest[item] !== src[item]) {
                    ext_user[item] = dest[item];
                } else if (angular.isUndefined(src[item])) {
                    ext_user[item] = dest[item];
                }
            }

            return ext_user;
        };

        return {
            get: fnModifiedFieldExtraction
        };
    });
    app.service('MakeHashString', function fnMakeHashString() {
        var fnGetHashString = function (pagination) {
            var hashString = '';
            var item = '';
            for (item in pagination) {
                if (item == 'toString')
                    continue;
                hashString += item + ',' + pagination[item] + ','
            }
            if (hashString.length > 2) {
                hashString = hashString.substr(0, hashString.length - 1);
            }
            return hashString;
        };

        return {
            get: fnGetHashString
//      , set : fnSetHashString
        };
    });
    app.service('MiniAgent', ['$http', 'Const', 'Util', function MiniAgent($http, Const, Util) {
        // AngularJS will instantiate a singleton by calling "new" on this function
        var agent = {};
        var config = {};
        agent = {
            post: function (url, param) {
                var USBAgentUrl = Util.parseAPIUrl(url, Const.USBAGENT.BASE_URL);
                return $http.post(USBAgentUrl.href, param, config);
            },
            get: function (url) {
                var USBAgentUrl = Util.parseAPIUrl(url, Const.USBAGENT.BASE_URL);
                return $http.get(USBAgentUrl.href, config);
            },
            put: function (url, param) {
                var USBAgentUrl = Util.parseAPIUrl(url, Const.USBAGENT.BASE_URL);
                return $http.put(USBAgentUrl.href, param, config);
            },
            del: function (url) {
                var USBAgentUrl = Util.parseAPIUrl(url, Const.USBAGENT.BASE_URL);
                return $http["delete"](USBAgentUrl.href, config);
            }
        };
        return agent;
    }]);

});