'use strict';

define([
    'modules/common/commonModule',
    'modules/common/services/api',
    'modules/common/services/const',
    'modules/common/alert/alertService'
], function () {

    var app = angular.module('commonModule');

    app.service('Auth', [
        '$q', '$rootScope', '$cookieStore', '$location', 'API', 'Const', 'AlertService', 'WebSocketService', 'Util',
        function ($q, $rootScope, $cookieStore, $location, API, Const, AlertService, WebSocketService, Util) {
        var _user = Util.getSessionObject(Const.SESSION_STORAGE_KEY.LOGGED_IN_USER),
            httpTimeOut = {},
            canceller = {};

        if (!_user) {
            _user = {};
        }

        var setUser = function (user) {
            _user = user;
//            var last_visit = $cookieStore.get('last_visit');
//            _user.last_visit = new Date(last_visit);
//            $cookieStore.put('last_visit', new Date().toISOString());
//            sessionStorage.setItem('photo', _user.photo);
//            delete _user.photo;
//            $cookieStore.put('user', _user);
            Util.setSessionObject(Const.SESSION_STORAGE_KEY.LOGGED_IN_USER, _user);
            $location.path(this.getBeforeUrl());
        };

        var _beforeUrl = '/';

        function resetLocalStorage() {
            var removeStorageKeyList = new Array();
            for (var i = 0, len = localStorage.length; i < len; ++i) {
                var storageKey = localStorage.key(i);
                if (storageKey.indexOf(Const.LOCAL_STORAGE_KEY_PREFIX) > -1) {
                    removeStorageKeyList.push(storageKey);
                }
            }
            for (var i = 0; i < removeStorageKeyList.length; i++) {
                localStorage.removeItem(removeStorageKeyList[i]);
            }
        }

        function checkAndResetLocalStorage() {
            var localStorageVersion = localStorage.getItem(Const.LOCAL_STORAGE_VERSION_KEY);
            if (localStorageVersion) {
                if (Const.LOCAL_STORAGE_VERSION != localStorageVersion) {
                    resetLocalStorage();
                    localStorage.setItem(Const.LOCAL_STORAGE_VERSION_KEY, Const.LOCAL_STORAGE_VERSION);
                }
            } else {
                resetLocalStorage();
                localStorage.setItem(Const.LOCAL_STORAGE_VERSION_KEY, Const.LOCAL_STORAGE_VERSION);
            }
        }

        var authService = {
            clearSessionData: function () {
                Util.info("clearSessionData");
                API.setSessionID(undefined);
                window.sessionStorage.clear();
                WebSocketService.close();
                _user = null;
            },
            isAuthorized: function (lvl) {
                return _user.role >= lvl;
            },
            setUser: setUser,
            isLoggedIn: function () {
                return API.isLoggedIn()
            },
            getUser: function () {
                return Util.getSessionObject(Const.SESSION_STORAGE_KEY.LOGGED_IN_USER);
            },
            getId: function () {
                return _user ? _user._id : null;
            },
            getToken: function () {
                return _user ? _user.token : '';
            },
            doLogin: function (user) {

                //            var tempBsSessionKey = (new Date()).getTime();
//            Util.debug("bsSessionKey", tempBsSessionKey, tempBsSessionKey.length);
//            tempBsSessionKey = '' + tempBsSessionKey;
//            tempBsSessionKey = Util.leftPad(tempBsSessionKey, 32, '0');

//            var salt = CryptoJS.lib.WordArray.random(128/8);
//            var key256Bits = CryptoJS.PBKDF2(tempBsSessionKey, salt, { keySize: 256/32 });

                var wordArray = CryptoJS.lib.WordArray.random(32);
//            var bsSessionKey = key256Bits
                var hexString = CryptoJS.enc.Hex.stringify(wordArray);
                var asciiString = Util.hex2ascii(hexString);
//            var tempString = Util.ascii2hex(asciiString);
//            Util.debug(hexString, asciiString, tempString);
//            var bsSessionKey = hexString;

                // below line is test code
//            bsSessionKey = 'd7ee10502a028bc6cc4d666e6c9917c261736f5f98e9bf1c4ef9a8ce9189e920';

                var bsSessionKey = btoa(asciiString);

                Util.debug("bsSessionKey", hexString, bsSessionKey, bsSessionKey.length);

                var _this = this;
                var httpHeaders = {};
                httpHeaders[Const.HTTP_HEADER.BS_SESSION_KEY] = bsSessionKey;
                window.sessionStorage.clear();
                API.post('login', 'login', {'User': user}, httpHeaders, false, true, true, 'div_login_message').then(function (data) {
//                            console.log(headers(Const.HTTP_HEADER.BS_SESSION_ID));
                    API.setSessionID(data.httpResponseHeaders(Const.HTTP_HEADER.BS_SESSION_ID));
                    Util.info("login successful", API.getSessionID());
                    var loginUserData = data.User;
                    loginUserData.sessionId = data.httpResponseHeaders(Const.HTTP_HEADER.BS_SESSION_ID);
                    _this.setUser(loginUserData);

                    checkAndResetLocalStorage();

                    $rootScope.$broadcast('LoginToSuccess');
                });
            },
            doLogout: function () {
                if (_user == null) {
                    Util.info("logout called, but user object does not exist");
                    authService.clearSessionData();
                    $location.path('/login');
                } else {
                    if (this.isLoggedIn() === false) {
                        Util.info("logout called, but not logged in");
                        authService.clearSessionData();
                        $location.path('/login');
                    } else {
                        var sessionId = API.getSessionID();
                        Util.info("logout called and " + sessionId + " session still logged in");
                        var options = {
                            url: 'logout',
                            displaySuccessPopoup: false,
                            displayErrorPopup: false,
                            displayLoading: false
                        };
                        API.postWithOptions(options).then(function success(data) {
                            Util.info(sessionId + " session logged out");
                            authService.clearSessionData();
                            $location.path('/login');
                        }, function error(data) {
                            Util.error("logout failed", data);
                            authService.clearSessionData();
                            $location.path('/login');
                        });
                    }
                }
            },
            setBeforeUrl: function (url) {
                _beforeUrl = url;
            },
            getBeforeUrl: function () {
                return _beforeUrl;
            }
        };

        return authService;

    }]);
});