'use strict';

define([
    'modules/common/commonModule'
], function () {

    var app = angular.module('commonModule');

    app.service('Const', [
        '$location', 'MessageProperties',
        function Const($location, MessageProperties) {

            function leftPad(n, width, z) {
                z = z || '0';
                n = n + '';
//                console.log(n);
                return n.length >= width ? n : new Array(width - n.length + 1).join(z) + n;
            }

            function deepFreeze(obj) {
                var prop, propKey;
                Object.freeze(obj);
                for (propKey in obj) {
                    prop = obj[propKey];
                    if (!obj.hasOwnProperty(propKey) || !(typeof prop === 'object') || Object.isFrozen(prop)) {
                        // 객체가 프로토타입이거나, object가 아니거나, 이미 Freeze 되었으면 넘어간다.
                        // Freeze 된 객체의 하위에 일부만 Freeze 되지 않은 경우 외부에서 이 Freeze 되지 않은 일부를 수정할 수 있다.
                        continue;
                    }

                    deepFreeze(prop); // deepFreeze은 재귀호출.
                }
            }

            // AngularJS will instantiate a singleton by calling "new" on this function
            var constant = {};
            constant = {
                PERMISSION_IDS: {
                    ADMINISTRATOR: '1',
                    USER: '2',
                    MONITORING: '3'
                },
                DEFAULT_ALL_GROUP_ID: '1',
                SCHEDULE_ALWAYS_ID: '1',
                ADMINISTRATOR_DEFAULT_ID: '1',
                SESSION_STORAGE_KEY: {
                    CARD_FORMAT_INFO: "biostar_session_card_format_info",
                    SERVER_SETTING: "biostar_session_server_setting",
                    LOGGED_IN_USER: 'biostar_session_loggedInUser',
                    DEVICE_SPEC: 'biostar_session_device_spec',
                    DEVICE_SPEC_LAST_DATE: 'biostar_session_device_spec_last_date',
                    DEVICE_GROUP_TREE_STATE: 'biostar_session_device_group_tree_state'
                },
                DEVICE_SPEC_UPDATE_INTERVAL: 10800000,      // 60 * 60 * 1000 = 3 hours
                BUILD_MAJOR_VERSION: '$MAJOR_VERSION$',
                BUILD_MINOR_VERSION: '$MINOR_VERSION$',
                BUILD_VERSION: '$BUILD_VERSION$',
                BUILD_GIT_HASH: '$GIT_HASH$',
                BUILD_SERVER_GIT_HASH: '$SERVER_GIT_HASH$',
                USER_AUTH_MODE_LIMIT: 3,
                LOCAL_STORAGE_VERSION: '9',
                LOCAL_STORAGE_VERSION_KEY: 'biostar_local_storage_version',
                LOCAL_STORAGE_KEY_PREFIX: 'biostar',
                LOCAL_STORAGE_DEV_ADMIN_KEY: 'biostar_dev_admin',
                LOCAL_STORAGE_DISABLE_ENCRYPTION: 'biostar_disable_encryption',
                LOCAL_STORAGE: {
                    DISPLAY_LANGUAGE: 'biostar_display_language'
                },
                FILENAME_PREFIX: 'BIOSTAR_',
                WEBDAV_PREFIX: '/webdav/',
                DEFAULT_ENCODING: 'UTF-8',
                UTF8_ENCODING: 'UTF-8',
                DEFAULT_LANGUAGE_CODE: "en",
                DEFAULT_DATE_FORMAT: "yyyy/MM/dd",
                DEFAULT_TIME_FORMAT: "HH:mm",
                DEFAULT_TIME_LONG_TERM_FORMAT: "HH:mm:ss",
                DEFAULT_DATETIME_FORMAT: "yyyy/MM/dd HH:mm",
                DEFAULT_DATETIME_LONG_TERM_FORMAT: "yyyy/MM/dd HH:mm:ss",
                TIME_SPLITTER: ':',
                DEFAULT_SHORT_DATE_FORMAT: "MM/dd",
                DEFAULT_SHORT_TIME_FORMAT: "HH:mm",
                DEFAULT_SHORT_DATETIME_FORMAT: "MM/dd HH:mm",
                API: {
                    BASE_URL: 'api/'
                },
                LOGIN_URL: 'api/login',
                WEBSOCKET: {
                    URL: $location.host() + ':' + $location.port() + '/wsapi',
                    PROTOCOL: 'ws',
                    SECURE_PROTOCOL: 'wss'
                },
                USBAGENT: {
                    HTTPS_BASE_URL: 'https://localhost:8081/api/',
                    HTTP_BASE_URL: 'http://localhost:8082/api/'
                },
                MOCK: {
                    BASE_URL: 'http://' + $location.host() + ':3412/api/'
                },
                MAX_TASK_RESPONSE_LOOKUP_COUNT: 60,
                FULLSCREEN_PAGES: [
                    '/404',
                    '/login',
                    '/user/users-report'
                ],
                REAL_TIME_BUTTON_IMG_URL: {
                    PLAY: 'images/common/funcdark_play.png',
                    PAUSE: 'images/common/funcdark_pause.png'
                },
                MENU: {
                    USER: '/user',
                    DEVICE: '/device',
                    AC: '/access',
                    MONITORING: '/monitoring',
                    TA: '/time',
                    REPORT: '/report'
                },
                SELECTED_ALL: '*',
                RS485_BAUDRATE: [
                    '9600', '19200', '38400', '57600', '115200'// , '230400', '460800', '921600'
                ],
                WIEGAND_FAIL_CODE: {
                    '0': '0x00',
                    '255': '0xFF'
                },
                DEVICE_TYPE: {
                    'BIO_ENTRY_PLUS': '1',  //1
                    'BIO_ENTRY_W': '2',     //11
                    'BIO_LITE_NET': '3',    //2
                    'X_PASS': '4',          //3
                    'X_PASS_S2': '5',
                    'SECURE_IO_2': '6',
                    'DOOR_MODULE_20': '7',
                    'BIO_STATION_2': '8',   //0
                    'BIO_STATION_T2': '9',  //6
                    'FACE_STATION_2': '10', //10
                    '_default': 'unknown'
                },
                DISPLAY_DEVICES: [
                    '3',
                    '8',
                    '9',
                    '10'
                ],
                HTTP_HEADER: {
                    BS_SESSION_ID: 'bs-session-id',
                    BS_SESSION_KEY: 'bs-session-key'
                },
                BLACKFIN_DEVICES: [
                    '1',
                    '2',
                    '3',
                    '4',
                    '5'
                ],
                FINGERPRINT_DEVICES: [
                    '8',
                    '1',
                    '2',
                    '3',
                    '9',
                    '10'
                ],
                CARD_DEVICES: [
                    '1',
                    '2',
                    '3',
                    '4',
                    '5',
                    '8',
                    '9',
                    '10'
                ],
                CARD_ONLY_DEVICE: [
                    '4',
                    '5'
                ],
                KEYPAD_DEVICES: [
                    '3',
                    '8',
                    '9',
                    '10'
                ],
                VOLUME_DEVICES: [
                    '8',
                    '9',
                    '10'
                ],
                ONLY_SLAVE_DEVICE: [
                    '6',
                    '7'
                ],
                CONNECTION_MODE: {
                    '0': false,
                    '1': true,
                    'true': '1',
                    'false': '0',
                    SERVER_TO_DEVICE: '0',
                    DEVICE_TO_SERVER: '1',
                    '_default': true
                },
                DEVICE_STATUS_ICON_CLASS: {
                    DISCONNECTED: 'disconnect',
                    SYNC_ERROR: 'sync_error',
                    NORMAL: 'normal',
                    WAITING: 'waiting',
                    '0': 'disconnect',
                    '1': 'normal',
                    '2': 'sync_error'
                },
                WIEGAND_FORMAT_MAX_SIZE: 256,
                WIEGAND_FORMAT_DISPLAY_SPLIT_SIZE: 8,
                REGULAR_EXPRESSION: {
                    ONLY_NUMBER: /^([0-9]+)$/,
                    IP_ADDRESS: /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/,
                    DATE_TIME_FORMAT: [
                        /^[y]{2,4}[\/-][M]{2}[\/-][d]{2}([ ][Hh]{2}:[m]{2}(:[s]{2})?)?$/,
                        /^[y]{2,4}[\/-][M]{2}[\/-][d]{2}([ ][a][ ][Hh]{2}:[m]{2}(:[s]{2})?)?$/,
                        /^[y]{2,4}[\/-][M]{2}[\/-][d]{2}([ ][Hh]{2}:[m]{2}(:[s]{2})?)[ ][a][ ]?$/,
                        /^[y]{2,4}[\/-][M]{2}[\/-][d]{2}([ ][a][ ][Hh]{2}:[m]{2}(:[s]{2})?)?$/,
                        /^[y]{2,4}[\/-][M]{2}[\/-][d]{2}([ ][Hh]{2}:[m]{2}(:[s]{2})?)[ ][a][ ]?$/,
                        /^[y]{2,4}[\/-][M]{2}[\/-][d]{2}([ ][Hh]{2}:[m]{2}(:[s]{2})?)?$/,
                        /^[M]{2}[\/-][d]{2}[\/-][y]{2,4}([ ][Hh]{2}:[m]{2}(:[s]{2})?)?$/,
                        /^[M]{2}[\/-][d]{2}[\/-][y]{2,4}([ ][a][ ][Hh]{2}:[m]{2}(:[s]{2})?)?$/,
                        /^[M]{2}[\/-][d]{2}[\/-][y]{2,4}([ ][Hh]{2}:[m]{2}(:[s]{2})?)[ ][a][ ]?$/,
                        /^[M]{2}[\/-][d]{2}[\/-][y]{2,4}([ ][a][ ][Hh]{2}:[m]{2}(:[s]{2})?)?$/,
                        /^[M]{2}[\/-][d]{2}[\/-][y]{2,4}([ ][Hh]{2}:[m]{2}(:[s]{2})?)[ ][a][ ]?$/,
                        /^[M]{2}[\/-][d]{2}[\/-][y]{2,4}([ ][Hh]{2}:[m]{2}(:[s]{2})?)?$/
                    ]
                },
                DEVICE_TYPE_ICON: {
                    '1': 'dvBioEntryPlus',
                    '2': 'dvBioEntryW',
                    '3': 'dvBioLiteNet',
                    '4': 'dvXpass',
                    '5': 'dvXpassSlim',     //XPassS2
                    '6': 'dvSecureIO2',      //SecureIO2
                    '7': 'dvSecureIO',      //DoorModule20
                    '8': 'dvBioStation2',    //BioStation 2
                    '9': 'dvBioStationT2',
                    '10': 'dvFaceStation'   //FaceStation 2
                },
                CONFIG: {
                    USE_SSL: "1",
                    USE_HTTP: "0"
                },
                FIELD_NAME: {
                    CHECKBOX: "ck"
                },
                DEVICE_TYPE_ID: {
                    'BIO_ENTRY_PLUS': '1',
                    'BIO_ENTRY_W': '2',
                    'BIO_LITE_NET': '3',
                    'X_PASS': '4',
                    'X_PASS_SLIM': '5',     //XPassS2
                    'SECURE_IO2': '6',      //SecureIO2
                    'SECURE_IO': '7',      //DoorModule20
                    'BIO_STATION_2': '8',    //BioStation 2
                    'BIO_STATION_T2': '9',
                    'FACE_STATION': '10'   //FaceStation 2
                },
                FINGERPRINT_ENROLL_QUALITY: {
                    '0': '20',
                    '1': '40',
                    '2': '60',
                    '3': '80',
                    '4': '100'
                },
                BOOLEAN: {
                    TRUE: "true",
                    FALSE: "false"
                },
                HTTP_METHOD: {
                    GET: "GET",
                    POST: "POST",
                    PUT: "PUT",
                    DELETE: "DELETE"
                },
                LOG_UPLOAD_TYPE: {
                    AUTO: "0",
                    MANUAL: "1"
                },
                QUERY_OPERATOR: {
                    'EQUAL': 0,
                    'NOT_EQUAL': 1,
                    'IN': 2,
                    'BETWEEN': 3
                },
                REPORT_PAGE_ORIENTATION: {
                    'PORTRAIT': 'PORTRAIT',
                    'LANDSCAPE': 'LANDSCAPE'
                },
                REPORT_FILE_FORMAT: {
                    'PDF': 'PDF',
                    'SVG': 'SVG'
                },
                REPORT_PAGE_SIZE: {
                    'A4': 'A4',
                    'Letter': 'Letter'
                },
                REPORT_COLLECTION_NAME: {
                    DEVICE: "DeviceCollection",
                    USER: "UserCollection",
                    DOOR: "DoorCollection",
                    EVENT: "EventCollection",
                    ALERT: "AlertCollection"
                },
                REPORT_DISPLAY_TYPE: {
                    DATETIME: "datetime",
                    ENUM: "enum",
                    MASK: "mask",
                    OBJECT: "object",
                    STRING: "string"
                },
                BIOSTAR_SERVER_PARAM_ALL: "ALL",
                GRID_ID: {
                    DEVICE_LIST: "devicegrid",
                    USER_LIST: "userGrid",
                    DOOR_LIST: "doorGrid",
                    EVENT_LOG: "grdPastEvent",
                    ALERT_HISTORY: "grdAlertHistory"
                },
                ALERT_STATUS: {
                    ACKNOWLEDGE: "1",
                    PENDING: "0",
                    SNOOZING: "2"
                },
                DEVICE_RESPONSE_RESULT: {
                    SUCCESS: "true",
                    FAIL: "false"
                },
                SERVER_RESPONSE_MESSAGE_PREFIX: "serverResponse",
                SERVER_RESPONSE_CODE: {
                    USER_ALREADY_LOGGED_IN: '2',
                    DEVICE_HAS_INVALID_CONFIG: '3',
                    REQUEST_TIMEOUT: '4',
                    SUCCESSFUL: '0',
                    PARTIAL_SUCCESS: '1',
                    TASK_NOT_FOUND: '1024',
                    INVALID_QUERY_PARAMETER: '105',
                    ADMIN_COULD_NOT_BE_DELETED: "205"
                },
                UI_EVENT_TYPE: {
                    MONITORING_FILTER_CLICKED: "MonitoringFilterClicked",
                    MONITORING_TREE_ITEM_CLICKED: "MonitoringTreeItemClicked",
                    ADD_FILTER_CLICKED: "AddFilterClicked",
                    SAVE_FILTER_CLICKED: "SaveFilterClicked",
                    EVENT_TYPE_CLICKED: "EventTypeClicked",
                    INPUT_FOCUS: "InputFocus",
                    DEVICE_INFO_LOADED: "DeviceInfoLoaded",
                    DEVICE_INFO_INIT_COMPLETED: "DeviceInfoInitCompleted",
                    DEVICE_INFO_UPDATED: "DeviceInfoUpdated"
                },
                EVENT_TYPE_CODE: {
                    DOOR_OPENED: '20992',
                    DOOR_CLOSED: '21248',
                    DOOR_UNLOCKED: '20480',  // door is unlocked
                    DOOR_LOCKED: '20736',  // door is locked
                    DOOR_FORCE_OPENED: '21504',
                    DOOR_HELD_OPENED: '21760'
                },
                CLEAR_ALARM_EVENT_TYPE_CODES: [
                    "22272",
                    "22784",
                    "25088",
                    "25856",
                    "26624"
                ],
                EVENT_TYPE_CATEGORY: {
                    DEVICE: '0',
                    DOOR: '1'
                },
                MONITORING_VIEW: {
                    PAST_EVENT: "MV1",
                    REAL_TIME_EVENT: "MV2",
                    DEVICE: "MV3",
                    DOOR: "MV4",
                    AUDIT: "MV5",
                    ALERT_HISTORY: "MV6"
                },
                MAX_REAL_TIME_DATA_SIZE: 200,
                DATA_FILTER: {
                    DOOR_NAME: "1",
                    DOOR_ID: "2",
                    DEVICE_ID: "3",
                    DEVICE_NAME: "4",
                    USER_GROUP_ID: "5",
                    USER_GROUP_NAME: "6",
                    USER_ID: "7",
                    USER_NAME: "8",
                    EVENT_ID: "9",
                    EVENT_NAME: "10",
                    EVENT_DATE: "11"
                },
                MODULE_URL: {
                    USER: "user",
                    DEVICE: "device",
                    ACCESS_CONTROL: "access-control",
                    ACCESS: "access",
                    MONITORING: "monitoring",
                    SETTING: "setting"
                },
                PERMISSION: {
                    ADMIN: "1",
                    USER: "2",
                    MONITORING: "3"
                },
                WEBSOCKET_READY_STATE: {
                    CONNECTING: 0,
                    OPEN: 1,
                    CLOSING: 2,
                    CLOSED: 3
                },
                PERMISSION_TYPE: {
                    1: "CanRead",
                    2: "CanUpdate",
                    4: "CanCreate",
                    8: "CanDelete",
                    16: "CanHandleAlarm",
                    CanRead: 1,
                    CanUpdate: 2,
                    CanCreate: 4,
                    CanDelete: 8,
                    CanHandleAlarm: 16
                },
                SETTING_ALARM_SMTP_SETUP: {
                    0: "None",
                    1: "SSL",
                    2: "TLS"
                },
                WIEGAND_USE_YN: {
                    '0': 'Normal',
                    '1': 'Wiegand'
                },
                CARD_BYTE_ORDER: {
                    '0': 'MSB',
                    '1': 'LSB'
                },
                DEVICE_DISPLAY_LANGUAGE: {
                    '0': MessageProperties.get('preference.language.option.ko'),
                    '1': MessageProperties.get('preference.language.option.en'),
                    '2': MessageProperties.get('device.language.option.custom')
                },
                OPERATION_MODE: [
                    "FINGER_ONLY",
                    "FINGER_PIN",
                    "CARD_ONLY",
                    "CARD_FINGER",
                    "CARD_PIN",
                    "CARD_FINGER_OR_PIN",
                    "CARD_FINGER_PIN",
                    "ID_FINGER",
                    "ID_PIN",
                    "ID_FINGER_OR_PIN",
                    "ID_FINGER_PIN"
                ],
                OTHER_OPERATION_MODE: {
                    "CARD_PIN_OR_FINGER": "CARD_FINGER_OR_PIN",
                    "ID_PIN_OR_FINGER": "ID_FINGER_OR_PIN"
                },
                OPERATION_MODE_SPLITTER: '_',
                TIMEZONE_OFFSET: {
                    GREENWICH: '0',
                    ASIA_SEOUL: '26',
                    US_EASTERN: '7'
                },
                // timezone options name property will be populated by localizationConstantVariables
                TIMEZONE_OPTIONS: [
                    {"id": "0", "offset": "-720"},
                    {"id": "1", "offset": "-660"},
                    {"id": "2", "offset": "-600"},
                    {"id": "3", "offset": "-540"},
                    {"id": "4", "offset": "-480"},
                    {"id": "5", "offset": "-420"},
                    {"id": "6", "offset": "-360"},
                    {"id": "7", "offset": "-300"},
                    {"id": "8", "offset": "-240"},
                    {"id": "9", "offset": "-210"},
                    {"id": "10", "offset": "-180"},
                    {"id": "11", "offset": "-120"},
                    {"id": "12", "offset": "-60"},
                    {"id": "13", "offset": "0"},
                    {"id": "14", "offset": "+60"},
                    {"id": "15", "offset": "+120"},
                    {"id": "16", "offset": "+180"},
                    {"id": "17", "offset": "+210"},
                    {"id": "18", "offset": "+240"},
                    {"id": "19", "offset": "+270"},
                    {"id": "20", "offset": "+300"},
                    {"id": "21", "offset": "+330"},
                    {"id": "22", "offset": "+345"},
                    {"id": "23", "offset": "+360"},
                    {"id": "24", "offset": "+420"},
                    {"id": "25", "offset": "+480"},
                    {"id": "26", "offset": "+540"},
                    {"id": "27", "offset": "+570"},
                    {"id": "28", "offset": "+600"},
                    {"id": "29", "offset": "+660"},
                    {"id": "30", "offset": "+720"}
                ],
                DATE_FORMAT_OPTIONS: {
                    "MM/dd/yyyy": "mm/dd/yyyy",
//                    "MMM/dd/yyyy": "Jun/16/2014",
                    "yyyy/MM/dd": "yyyy/mm/dd"
                },
                TIME_FORMAT_OPTIONS: {
                    "HH:mm": "hh:mm",
                    "hh:mm a": "hh:mm a",
                    "a hh:mm": "a hh:mm"
                },
                ALERT_MESSAGE_ICON: {
                    WARNING: "warning",
                    ERROR: "error",
                    QUESTION: "question",
                    INFO: "info",
                    CONFIRM: "confirm"
                },
                OUTPUT_TYPES: {
                    'OUTPUT': 1,
                    'SOUND': 2,
                    'FUNCTION': 0
                },

                DOOR_ALARM: {
                    HELD_OPEN: '22528',
                    FORCED_OPEN: '22016',
                    ANTI_PASS_BACK: '24832'
                },
                DOOR_ALARM_LIST: [
                    '22528', '22016', '24832'
                ],
                CARD_STATUS: {
                    UNASSIGNED: 0,
                    ASSIGNED: 1,
                    BLOCKED: 2
                },
                HTTP_STATUS_CODE: {
                    OK: 200,
                    CREATED: 201,
                    NO_CONTENT: 204,
                    BAD_REQUEST: 400,
                    NOT_FOUND: 404,
                    REQUEST_TIMEOUT: 408
                },
                DEFAULT_START_DATE: '2001-01-01T00:00:00.00Z',
                DEFAULT_START_FULL_YEAR: 2000,
                DEFAULT_START_MONTH: 0,
                DEFAULT_START_DAY: 1,
                DEFAULT_START_TIME: '00:00',
                DEFAULT_EXPIRY_DATE: '2030-12-31T23:59:59.00Z',
                DEFAULT_EXPIRY_TIME: '23:59',
                PASSWORD_INITSTRING: '################',
                PARAM_ALL_STRING: '*',
                BIO_MINI_ID: '-1',
                DEFAULT_SCHEDULE: {
                    'NO_TIME': "0",
                    'ALWAYS': "1"
                },
                FINGERPRINT_USB_SCANNER_TEMPLATE_TYPE_FOR_SWITCH: {},
                SWITCH_TYPE: {
                    NORMALLY_OPEN: "0",
                    NORMALLY_CLOSE: "1"
                },
                OPEN_ONCE: {
                    TRUE_VALUE: 'true',
                    FALSE_VALUE: 'false'
                },
                WIEGAND_DEFAULT_FORMAT_ID: 1,
                CARD_TYPE_DEFAULT_INDEX: 0,
                CARD_TYPE_LIST: {
                    "CSN": "1",
                    "CSN_WIEGAND": "10"
                },
                DEFAULT_WIEGAND_FORMAT_ID: {id: 0},
                REPEAT_INTERVAL_VALUE: {
                    "ONCE": 0,
                    "EVERY_YEAR": 1,
                    "EVERY_MONTH": 2,
                    "EVERY_WEEK": 3
                },
                DEFAULT_REPEAT_INTERVAL: 0, // Once
                DURATION_LIST: {
                    1: "1 Day",
                    2: "2 Days",
                    3: "3 Days",
                    4: "4 Days",
                    5: "5 Days",
                    6: "6 Days",
                    7: "7 Days"
                },

                ON_SEARCH_USERS_IN_DEVICE: 'searchUsersInDevice',
                ON_SEARCH_DEVICE: 'UDPSearchDevice',
                ON_OPEN_FW_UPGRADE_POPUP: 'openFWUpgrade',
                PAGE_MODE: {
                    'CREATE': 0,
                    'UPDATE': 1
                },

                CARD_FORMAT_ID: {
                    H10301: '0',
                    H10302: '1',
                    H10304: '2',
                    HCORP1000: '3'
                },
                SERVER_AUTO_UPGRADE: {
                    AUTO: '1',
                    MANUAL: '0'
                },
                SERVER_FAIL_CODE: {
                    DUPLICATED_USER_ID: '202',
                    DUPLICATED_LOGIN_ID: '2021',
                    SERVER_ERROR: '1000'
                },
                PORT_UNSELECT: -1,
                TIME_SEGMENT_MAX_COUNT: 5,

                DIALOG_TYPE: {
                    DIALOG: 'bssDialog',
                    MESSAGER: 'bssMessager',
                    SERVICE: 'bssServiceDialog'
                },
                HOLIDAY_LIMIT: 128,
                SCHEDULE_LIMIT: 128,
                DEFAULT_DOOR_OPEN_DURATION: 3,
                DEFAULT_DOOR_HELD_OPEN_DURATION: 3,
                OPEN_TIME_LIST: {
                    '0': {'value': '1', 'text': '1 sec'},
                    '1': {'value': '3', 'text': '3 sec'},
                    '2': {'value': '5', 'text': '5 sec'},
                    '3': {'value': '10', 'text': '10 sec'},
                    '4': {'value': '15', 'text': '15 sec'},
                    '5': {'value': '30', 'text': '30 sec'},
                    '6': {'value': '60', 'text': '1 min'},
                    '7': {'value': '180', 'text': '3 min'},
                    '8': {'value': '300', 'text': '5 min'},
                    '9': {'value': '600', 'text': '10 min'},
                    '10': {'value': '900', 'text': '15 min'}
                },
                DEFAULT_DOOR_APB_RESET_TIME: '1440',
                DEFAULT_DOOR_HELD_OPEN_TIME: '1440',

                TRIGGER_ACTION_TYPE: {
                    0: 'biostar',
                    1: 'device',
                    2: 'door',
                    biostar: 0,
                    device: 1,
                    door: 2
                },

                FINGERPRINT_QUALITY_LIST: {
                    '0': {'value': '20', 'text': '20'},
                    '1': {'value': '40', 'text': '40'},
                    '2': {'value': '60', 'text': '60'},
                    '3': {'value': '80', 'text': '80'},
                    '4': {'value': '100', 'text': '100'}
                },
                USER_ID_MAX_VALUE: 4294967294,
                SERVER_TRUE_STRING: 'true',
                SERVER_FALSE_STRING: 'false',
                SORTING_DIRECTION: {
                    ASCEND: 'ASCEND',
                    DESCEND: 'DESCEND',
                    NONE: 'NONE'
                },
                DEVICE_BATCH_EDIT_SELECTED_COUNT: 'DEVICE_BATCH_EDIT_SELECTED_COUNT',
                LAST_FINGERPRINT_SCAN_DEVICE: 'LAST_FINGERPRINT_SCAN_DEVICE',
                LAST_CARD_SCAN_DEVICE: 'LAST_CARD_SCAN_DEVICE',
                ALERT_TYPE: {
                    DEVICE: 0,
                    DOOR: 1,
                    ZONE: 2,
                    USER: 3,
                    AUTH: 4
                },

                DUAL_AUTHENTICATION_EXIT_DEVICE_ID: 2,
                DUAL_AUTHENTICATION_DEVICE_DEFAULT_VALUE: '0',

                DUAL_AUTHENTICATION_APPROVAL_TYPE_DEFAULT_VALUE: '0',
                DUAL_AUTHENTICATION_TIMEOUT_LIST: {
                    '0': {'value': '1', 'text': '1 sec'},
                    '1': {'value': '3', 'text': '3 sec'},
                    '2': {'value': '5', 'text': '5 sec'},
                    '3': {'value': '10', 'text': '10 sec'},
                    '4': {'value': '15', 'text': '15 sec'},
                    '5': {'value': '30', 'text': '30 sec'},
                    '6': {'value': '60', 'text': '1 min'},
                    '7': {'value': '180', 'text': '3 min'},
                    '8': {'value': '300', 'text': '5 min'},
                    '9': {'value': '600', 'text': '10 min'},
                    '10': {'value': '900', 'text': '15 min'}
                },
                DUAL_AUTHENTICATION_TIMEOUT_DEFAULT_VALUE: 15,

                SERVER_WEB_SERVER_PROTOCOL: {
                    HTTP: 0,
                    HTTPS: 1
                },
                WEB_SERVER_PROTOCOL: {
                    HTTP: "http",
                    HTTPS: "https"
                },
                CARD_FORMAT_SELECT_STATUS: {
                    ALL: -1,
                    WIEGAND: 0
                },
                SIGNAL: {
                    MAX_VALUE_DURATION: 65535,
                    MAX_VALUE_DELAY: 65535,
                    MAX_VALUE_COUNTS: 65535,
                    MIN_VALUE_DURATION: 0,
                    MIN_VALUE_DELAY: 0,
                    MIN_VALUE_COUNTS: -1
                },
                DEVICE_LOCK_STATUS: {
                    'true': 'Locked',
                    'false': 'Unlocked'
                },
                TIME_LINE_BASE_WIDTH: 720,
                TIME_LINE_BASE_MINUTE: 1440,
                INTEGER_MAX: 65535,
                SELECT_LIST_ITEM_LIMIT: 100,
                localizationConstantVariables: localizationConstantVariables,
                DATE_TIME_FORMAT: [
                    'yyyy/MM/dd HH:mm',
                    'yyyy/MM/dd a hh:mm',
                    'yyyy/MM/dd hh:mm a',
                    'yyyy/MM/dd a hh:mm:ss',
                    'yyyy/MM/dd hh:mm:ss a',
                    'yyyy/MM/dd HH:mm:ss',
                    'MM/dd/yyyy HH:mm',
                    'MM/dd/yyyy a hh:mm',
                    'MM/dd/yyyy hh:mm a',
                    'MM/dd/yyyy a hh:mm:ss',
                    'MM/dd/yyyy hh:mm:ss a',
                    'MM/dd/yyyy HH:mm:ss'
                ],
                BIOSTATION_2_MESSAGE_TIMEOUT_OPTIONS: {
                    '5': {'value': '500', 'text': '0.5 sec'},
                    '10': {'value': '1000', 'text': '1.0 sec'},
                    '15': {'value': '2000', 'text': '2.0 sec'},
                    '20': {'value': '3000', 'text': '3.0 sec'},
                    '25': {'value': '4000', 'text': '4.0 sec'},
                    '30': {'value': '5000', 'text': '5.0 sec'}
                },
                DEVICE_MESSAGE_TIMEOUT_OPTIONS: {
                    '5': {'value': '500', 'text': '0.5 sec'},
                    '10': {'value': '1000', 'text': '1.0 sec'},
                    '15': {'value': '1500', 'text': '1.5 sec'},
                    '20': {'value': '2000', 'text': '2.0 sec'},
                    '25': {'value': '2500', 'text': '2.5 sec'},
                    '30': {'value': '3000', 'text': '3.0 sec'},
                    '35': {'value': '3500', 'text': '3.5 sec'},
                    '40': {'value': '4000', 'text': '4.0 sec'},
                    '45': {'value': '4500', 'text': '4.5 sec'},
                    '50': {'value': '5000', 'text': '5.0 sec'}
                }
            };

            constant.FINGERPRINT_TEMPLATE_FORMAT_OPTION = {
                "0": 'Suprema',
                "1": 'ISO',
                "2": 'ANSI378'
            };

            constant.FINGERPRINT_USB_SCANNER_TEMPLATE_TYPE_FOR_SWITCH = {
                'ANSI378': '2003',
                'ISO': '2002',
                'Suprema': '2001'
            };


            constant.CARD_REGISTRATION_TYPE_ID = {
                'READ_CARD': '0',
                'ASSIGN_CARD': '1',
                'MANUAL_CARD': '2'
            };

            constant.FINGERPRINT_TEMPLATE_FORMAT = constant.FINGERPRINT_TEMPLATE_FORMAT_OPTION;

            function localizationConstantVariables(language) {
                MessageProperties.init(language).then(function (data) {

                    constant.GNB_MENU_ITEMS = {
                        'DASHBOARD': {id: 0, property: 'dashboard', url: '/dashboard'},
                        'USER': {id: 1, property: 'user', url: '/user'},
                        'DEVICE': {id: 2, property: 'device', url: '/device'},
                        'DOOR': {id: 3, property: 'door', url: '/access-control/ac-doors'},
                        'ACCESS_CONTROL': {
                            id: 4,
                            property: 'access_control',
                            url: '/access-control/ac-groups',
                            url2: '/access-control/ac-levels'
                        },
                        'MONITORING': {id: 5, property: 'monitoring', url: '/monitoring'}
//                'TIME_ATTENDANCE' : {id:5, property : 'ta', url:'/time/paycode-list'},
//                'REPORT' : {id:6, property : 'report', url:'/report'}
                    };
                    constant.GNB_MENU_ITEMS_NAME = {
                        DASHBOARD: MessageProperties.get('common.gnb.dashboard'),
                        USER: MessageProperties.get('common.gnb.user'),
                        DEVICE: MessageProperties.get('common.gnb.device'),
                        DOOR: MessageProperties.get('common.gnb.door'),
                        ACCESS_CONTROL: MessageProperties.get('common.gnb.access_control'),
                        MONITORING: MessageProperties.get('common.gnb.monitoring')
                    };

                    constant.PERMISSION_MODULE_ID = {
                        USER: "1",
                        DEVICE: "2",
                        ACCESS_CONTROL: "3",
                        MONITORING: "4",
                        SETTING: "5",
                        DASHBOARD: "6"
                    };

                    constant.PERMISSION_MODULE_NAME = {
                        '1': MessageProperties.get('common.permission.module.user'),
                        '2': MessageProperties.get('common.permission.module.device'),
                        '3': MessageProperties.get('common.permission.module.ac'),
                        '4': MessageProperties.get('common.permission.module.monitoring'),
                        '5': MessageProperties.get('common.permission.module.setting'),
                        '6': MessageProperties.get('common.permission.module.dashboard')
                    };


                    constant.BIOSTAR_IP_ADDRESS_LIST_DEFAULT_VALUE = MessageProperties.get("setting.server.ipAddress.any");
                    constant.BIOSTAR_IP_ADDRESS_LIST_DEFAULT_OBJECT = {};
                    constant.BIOSTAR_IP_ADDRESS_LIST_DEFAULT_OBJECT[MessageProperties.get("setting.server.ipAddress.any")] = MessageProperties.get("setting.server.ipAddress.any");
                    constant.CARD_REGISTRATION_TYPE_TEXT = {
                        "0": MessageProperties.get('common.registerByCardReader'),
                        "1": MessageProperties.get('common.assignCard'),
                        "2": MessageProperties.get('common.enterManually')
                    };
                    constant.RS485_MODE = {
//                '0': 'Disabled',
                        '1': MessageProperties.get('device.rs485.master'),
                        '2': MessageProperties.get('device.rs485.slave'),
                        '3': MessageProperties.get('device.rs485.default'),
                        'true': MessageProperties.get('device.rs485.master'),
                        'false': MessageProperties.get('device.rs485.slave'),
                        'MASTER': '1',
                        'SLAVE': '2',
                        'AUTO': '3',
//                'DISABLED': '0',
                        '_default': MessageProperties.get('device.rs485.default')
                    };

                    constant.RS485_MODE_FOR_SELECT_BOX = {
                        '1': constant.RS485_MODE['1'],
                        '2': constant.RS485_MODE['2'],
                        '3': constant.RS485_MODE['3']
                    };

                    constant.RS485_MODE_HTML = {
//                '0': '',            // Disabled
                        '1': '<span class="iconMaster">M</span>',
                        '2': '<span class="iconSlave">S</span>',
                        '3': '',
                        'true': '<span class="iconMaster">M</span>',
                        'false': '<span class="iconSlave">S</span>',
                        '_default': ''      // Disabled
                    };

                    constant.AUTOMATIC_USER_SYNC_OPTION = {
                        1: MessageProperties.get("setting.server.syncOption.allDevices"),
                        0: MessageProperties.get("setting.server.syncOption.notUsed")
                    };
                    constant.WIEGAND_PARITY_TYPE = {
                        2: MessageProperties.get("setting.cardFormat.parityType.even"),
                        1: MessageProperties.get("setting.cardFormat.parityType.odd")
//                    0: MessageProperties.get("setting.cardFormat.parityType.none")
//                    NONE: "0"
                    };

                    constant.DEVICE_STATUS = {
                        '0': MessageProperties.get("device.status.disconnected"),
                        '1': MessageProperties.get("device.status.normal"),
                        '2': MessageProperties.get("device.status.syncError"),
                        NORMAL: '1',
                        SYNC_ERROR: '2',
                        DISCONNECTED: '0'
                    };

                    constant.DOOR_STATUS = {
//                INVALID: "0",
                        NORMAL: "0",
                        LOCKED: 0x01,
                        UNLOCKED: 0x02,
                        FORCED_OPEN: 0x04,
                        HELD_OPENED: 0x08,
                        APB_FAILED: 0x10,
                        DISCONNECTED: 0x20,
                        "0": MessageProperties.get("door.status.normal"),
                        "1": MessageProperties.get("door.status.locked"),
                        "2": MessageProperties.get("door.status.unlocked"),
                        "4": MessageProperties.get("door.status.forcedOpen"),
                        "8": MessageProperties.get("door.status.heldOpen"),
                        "16": MessageProperties.get("door.status.apbFailed"),
                        "32": MessageProperties.get("door.status.disconnected")
                    };
                    constant.MONITORING_DOOR_STATUS = {
                        OPENED: MessageProperties.get("common.opened"),
                        CLOSED: MessageProperties.get("common.closed"),
                        true: MessageProperties.get("common.opened"),
                        false: MessageProperties.get("common.closed")
                    };
                    constant.MONITORING_DOOR_RELAY_STATUS = {
                        LOCKED: MessageProperties.get("common.locked"),
                        UNLOCKED: MessageProperties.get("common.unlocked"),
                        false: MessageProperties.get("common.locked"),
                        true: MessageProperties.get("common.unlocked")
                    };

                    constant.DOOR_ALARM_STATUS = {
                        '0': '',
                        '1': MessageProperties.get("door.alarm.status.heldOpen"),
                        '2': MessageProperties.get("door.alarm.status.forcedOpen"),
                        'NoAlarm': '0',
                        'HeldOpened': '1',
                        'ForcedOpened': '2'
                    };
                    constant.FINGERPRINT_FAST_MODE = {
                        '0': MessageProperties.get("user.fingerprint.mode.auto"),
                        '1': MessageProperties.get("user.fingerprint.mode.normal"),
                        '2': MessageProperties.get("user.fingerprint.mode.fast"),
                        '3': MessageProperties.get("user.fingerprint.mode.fastest"),
                        'Auto': '0',
                        'Normal': '1',
                        'Fast': '2',
                        'Fastest': '3',
                        '_default': 'Auto'
                    };
                    constant.FINGERPRINT_FAST_MODE_FOR_SELECT_BOX = {};
                    constant.FINGERPRINT_FAST_MODE_FOR_SELECT_BOX['0'] = constant.FINGERPRINT_FAST_MODE['0'];
                    constant.FINGERPRINT_FAST_MODE_FOR_SELECT_BOX['1'] = constant.FINGERPRINT_FAST_MODE['1'];
                    constant.FINGERPRINT_FAST_MODE_FOR_SELECT_BOX['2'] = constant.FINGERPRINT_FAST_MODE['2'];
                    constant.FINGERPRINT_FAST_MODE_FOR_SELECT_BOX['3'] = constant.FINGERPRINT_FAST_MODE['3'];

                    constant.FINGERPRINT_SECURITY_LEVEL = {
                        '0': MessageProperties.get("securityLevel.deviceDefault"),
                        '1': MessageProperties.get("securityLevel.lowest"),
                        '2': MessageProperties.get("securityLevel.low"),
                        '3': MessageProperties.get("securityLevel.normal"),
                        '4': MessageProperties.get("securityLevel.high"),
                        '5': MessageProperties.get("securityLevel.highest")
                    };

                    constant.DEVICE_FINGERPRINT_SECURITY_LEVEL = {
                        '0': MessageProperties.get("device.securityLevel.normal"),
                        '1': MessageProperties.get("device.securityLevel.secure"),
                        '2': MessageProperties.get("device.securityLevel.mostSecure")
                    };

                    constant.FINGERPRINT_SENSOR_MODE = {
                        '0': MessageProperties.get("device.sensorMode.alwaysOn"),
                        '1': MessageProperties.get("device.sensorMode.autoOn"),
                        DEFAULT: '1'
                    };

                    constant.FINGERPRINT_SENSOR_MODE_OPTIONS = {
                        '0': constant.FINGERPRINT_SENSOR_MODE['0'],
                        '1': constant.FINGERPRINT_SENSOR_MODE['1']
                    };

//                    constant.RELAY_OPEN_CONDITION = {
//                        "0": "All Event",
//                        "1": "TNA + Auth",
//                        "2": "TNA Only",
//                        "3": "Auth Only",
//                        "4": "Disabled"
//                    };
                    constant.APB_TYPE = {
                        '0': MessageProperties.get("door.apbType.none"),
                        '1': MessageProperties.get("door.apbType.soft"),
                        '2': MessageProperties.get("door.apbType.hard")
                    };
                    constant.APB_TYPE_TEXT = {
                        NONE: '0',
                        SOFT_APB: '1',
                        HARD_APB: '2'
                    };
                    constant.ACTION_WHEN_FAILED = {
                        0: MessageProperties.get("door.actionWhenFailed.failSafe"),
                        1: MessageProperties.get("door.actionWhenFailed.failSecure")
                    };

                    constant.USER_SYNC_STATUS = {
                        'SAME': '0',
                        'NEW': '1',
                        'DIFF': '2',
                        'DIFF_NEW': '3'
                    };
                    constant.USER_SYNC_STATUS_LABEL = {
                        '0': MessageProperties.get("user.syncStatus.same"),
                        '1': MessageProperties.get("user.syncStatus.newUser"),
                        '2': MessageProperties.get("user.syncStatus.different"),
                        '3': MessageProperties.get("user.syncStatus.differentOrNewUser")
                    };

                    constant.BLACKFIN_BASE_BAND_LABEL = {
                        'true': MessageProperties.get("device.blackfin.baseband.use"),
                        'false': MessageProperties.get("device.blackfin.baseband.notUse")
                    };
                    constant.BLACKFIN_BASE_BAND_VALUE = {
                        'true': '1',
                        'false': '0'
                    };
                    constant.BLACKFIN_BASE_BAND_TYPE = {
                        '0': false,
                        '1': true
                    };

                    constant.DEVICE_CONTEXT_MENU = {
                        ADD_ALL_WAITING_DEVICES: {
                            clickFunctionName: 'addWaitingDevice',
                            menuName: MessageProperties.get("device.menu.addAllWaitingDevices")
                        },
                        ADD_WAITING_DEVICE: {
                            clickFunctionName: 'addWaitingDevice',
                            menuName: MessageProperties.get("device.menu.addWaitingDevice")
                        },
                        ADD_DEVICE_GROUP: {
                            clickFunctionName: 'addDeviceGroup',
                            menuName: MessageProperties.get("device.menu.addDeviceGroup")
                        },
                        DELETE_DEVICE: {
                            clickFunctionName: 'deleteDevice',
                            menuName: MessageProperties.get("device.menu.deleteDevice")
                        },
                        DELETE_DEVICE_GROUP: {
                            clickFunctionName: 'deleteDeviceGroup',
                            menuName: MessageProperties.get("device.menu.deleteDeviceGroup")
                        },
                        MANAGE_USERS_IN_DEVICE: {
                            clickFunctionName: 'manageUsersinDevice',
                            menuName: MessageProperties.get("device.menu.manageUsersInDevice")
                        },
                        RECONNECT: {
                            clickFunctionName: 'reconnectDevice',
                            menuName: MessageProperties.get("device.menu.reconnect")
                        },
                        REBOOT: {
                            clickFunctionName: 'rebootDevice',
                            menuName: MessageProperties.get("device.menu.reboot")
                        },
                        RENAME_DEVICE_GROUP: {
                            clickFunctionName: 'renameDeviceGroup',
                            menuName: MessageProperties.get("device.menu.renameDeviceGroup")
                        },
                        SEARCH_SLAVE_DEVICE: {
                            clickFunctionName: 'searchSlaveDevice',
                            menuName: MessageProperties.get("device.menu.searchSlaveDevice")
                        },
                        SEARCH_USB_DEVICE: {
                            clickFunctionName: 'searchUSBDevice',
                            menuName: MessageProperties.get("device.menu.searchUsbDevice")
                        },
                        SYNC_DEVICE: {
                            clickFunctionName: 'syncDevice',
                            menuName: MessageProperties.get("device.menu.syncDevice")
                        },
                        FULL_SYNC_DEVICE: {
                            clickFunctionName: 'fullSyncDevice',
                            menuName: MessageProperties.get("device.menu.fullSyncDevice")
                        }
                    };

                    constant.CARD_STATUS_LIST = {
                        'BLOCKED': MessageProperties.get("setting.card.blocked"),
                        'UNBLOCKED': MessageProperties.get("setting.card.unblocked"),
                        'ASSIGNED': MessageProperties.get("setting.card.assigned"),
                        'UNASSIGNED': MessageProperties.get("setting.card.unassigned")
                    };
                    constant.DEFAULT_NOT_USE = MessageProperties.get('common.notUse');
                    constant.RESET_TIME_LIST = {
                        '0': {'value': '0', 'text': MessageProperties.get('common.none')},
                        '1': {'value': '5', 'text': '5 min'},
                        '2': {'value': '10', 'text': '10 min'},
                        '3': {'value': '30', 'text': '30 min'},
                        '4': {'value': '60', 'text': '1 hours'},
                        '5': {'value': '180', 'text': '3 hours'},
                        '6': {'value': '600', 'text': '10 hours'},
                        '7': {'value': '1440', 'text': '24 hours'},
                        '8': {'value': '4320', 'text': '3 days'},
                        '9': {'value': '10080', 'text': '7 days'}
                    };
                    constant.TRIGGER_ACTION_TYPE_LABEL = {
                        biostar: MessageProperties.get('common.biostar'),
                        device: MessageProperties.get('common.device'),
                        door: MessageProperties.get('common.door')
                    };
                    constant.DUAL_AUTHENTICATION_DEVICE_OPTION_LIST = {
                        '0': MessageProperties.get('door.dualAuthentication.device.noDevice'),
                        '1': MessageProperties.get('door.dualAuthentication.device.entryDeviceOnly'),
                        '2': MessageProperties.get('door.dualAuthentication.device.exitDeviceOnly')
//                    '3': MessageProperties.get('door.dualAuthentication.device.bothDevice')
                    };
                    constant.DUAL_AUTHENTICATION_APPROVAL_TYPE_OPTION_LIST = {
                        '0': MessageProperties.get('door.dualAuthentication.approvalType.none'),
                        '1': MessageProperties.get('door.dualAuthentication.approvalType.last')
//                    '2': MessageProperties.get('door.dualAuthentication.approvalType.any')
                    };
                    constant.ACTION_LIST = [
                        {
                            id: 1,
                            name: MessageProperties.get('door.action.output'),
                            output_type: 1,
                            output_function_mask: 0
                        },
                        {
                            id: 2,
                            name: MessageProperties.get('door.action.releaseAllAlarm'),
                            output_type: 0,
                            output_function_mask: 4
                        },
                        {
                            id: 3,
                            name: MessageProperties.get('door.action.rebootDevice'),
                            output_type: 0,
                            output_function_mask: 8
                        },
                        {
                            id: 4,
                            name: MessageProperties.get('door.action.disableDevice'),
                            output_type: 0,
                            output_function_mask: 16
                        }
                    ];

                    constant.SIMPLE_DAYS_TEXT = [
                        MessageProperties.get('common.dayOfTheWeek.short.sun'),
                        MessageProperties.get('common.dayOfTheWeek.short.mon'),
                        MessageProperties.get('common.dayOfTheWeek.short.tue'),
                        MessageProperties.get('common.dayOfTheWeek.short.wed'),
                        MessageProperties.get('common.dayOfTheWeek.short.thu'),
                        MessageProperties.get('common.dayOfTheWeek.short.fri'),
                        MessageProperties.get('common.dayOfTheWeek.short.sat')
                    ];
                    constant.FULL_DAYS_TEXT = [
                        MessageProperties.get('common.dayOfTheWeek.long.sun'),
                        MessageProperties.get('common.dayOfTheWeek.long.mon'),
                        MessageProperties.get('common.dayOfTheWeek.long.tue'),
                        MessageProperties.get('common.dayOfTheWeek.long.wed'),
                        MessageProperties.get('common.dayOfTheWeek.long.thu'),
                        MessageProperties.get('common.dayOfTheWeek.long.fri'),
                        MessageProperties.get('common.dayOfTheWeek.long.sat')
                    ];
                    constant.FULL_MONTH_TEXT = [
                        MessageProperties.get('common.month.long.jan'),
                        MessageProperties.get('common.month.long.feb'),
                        MessageProperties.get('common.month.long.mar'),
                        MessageProperties.get('common.month.long.apr'),
                        MessageProperties.get('common.month.long.may'),
                        MessageProperties.get('common.month.long.jun'),
                        MessageProperties.get('common.month.long.jul'),
                        MessageProperties.get('common.month.long.aug'),
                        MessageProperties.get('common.month.long.sep'),
                        MessageProperties.get('common.month.long.oct'),
                        MessageProperties.get('common.month.long.nov'),
                        MessageProperties.get('common.month.long.dec')
                    ];
                    constant.SIMPLE_MONTH_TEXT = [
                        MessageProperties.get('common.month.short.jan'),
                        MessageProperties.get('common.month.short.feb'),
                        MessageProperties.get('common.month.short.mar'),
                        MessageProperties.get('common.month.short.apr'),
                        MessageProperties.get('common.month.short.may'),
                        MessageProperties.get('common.month.short.jun'),
                        MessageProperties.get('common.month.short.jul'),
                        MessageProperties.get('common.month.short.aug'),
                        MessageProperties.get('common.month.short.sep'),
                        MessageProperties.get('common.month.short.oct'),
                        MessageProperties.get('common.month.short.nov'),
                        MessageProperties.get('common.month.short.dec')
                    ];
                    constant.REPEAT_INTERVAL_LIST = {
                        0: MessageProperties.get('schedule.repeatInterval.once'),
                        1: MessageProperties.get('schedule.repeatInterval.everyYear')
//                    2: "Every Month",
//                    3: "Every Week",
                    };

                    constant.DEFAULT_OUTPUT_FUNCTION_MASK = 0;
                    constant.DEFAULT_OUTPUT_TYPE = 1;

                    constant.DEFAULT_SCHEDULE_ID = {
                        id: 1,
                        name: MessageProperties.get('schedule.always')
                    };
                    for (var i = 0; i < constant.TIMEZONE_OPTIONS.length; i++) {
                        var offset = parseInt(constant.TIMEZONE_OPTIONS[i].offset);
                        if (offset > 0) {
                            var messageKeyPostFix = '' + offset;
                            messageKeyPostFix = "+" + leftPad(messageKeyPostFix, 3, 0);
                        } else if (offset == 0) {
                            var messageKeyPostFix = '' + offset;
                            messageKeyPostFix = leftPad(messageKeyPostFix, 3, 0);
                        } else {
                            var messageKeyPostFix = '' + (offset * -1);
                            messageKeyPostFix = "-" + leftPad(messageKeyPostFix, 3, 0);
                        }
                        constant.TIMEZONE_OPTIONS[i].name = MessageProperties.get("common.timezone.offset." + messageKeyPostFix);
                    }
                    constant.DEVICE_WLAN_OPERATION_MODES = {
                        '0': MessageProperties.get("device.wlan.operationMode.managed"),
                        '1': MessageProperties.get("device.wlan.operationMode.adhoc")
                    };
                    constant.DEVICE_WLAN_AUTHORIZATION_TYPES = {
                        '0': MessageProperties.get("device.wlan.authType.open"),
                        '1': MessageProperties.get("device.wlan.authType.shared"),
                        '2': MessageProperties.get("device.wlan.authType.wpaPsk"),
                        '3': MessageProperties.get("device.wlan.authType.wpa2Psk")
                    };
                    constant.DEVICE_WLAN_ENCRYPTION_TYPES = {
                        '0': MessageProperties.get("device.wlan.encryptionType.none"),
                        '1': MessageProperties.get("device.wlan.encryptionType.wep"),
                        '2': MessageProperties.get("device.wlan.encryptionType.tkipAes"),
                        '3': MessageProperties.get("device.wlan.encryptionType.aes"),
                        '4': MessageProperties.get("device.wlan.encryptionType.tkip")
                    };
                    constant.DEVICE_INTERPHONE_TYPES = {
                        DISABLED: '0',
                        ENABLED: '1'
                    };
                    constant.DEVICE_TNA_MODES = {
                        'UNUSED': '0',
                        'BY_USER': '1',
                        'BY_SCHEDULE': '2',
                        'LAST_CHOICE': '3',
                        'FIXED': '4'
                    };
                    constant.DEVICE_TNA_MODES_OPTIONS = {
                        '0': MessageProperties.get("device.tna.mode.unused"),
                        '2': MessageProperties.get("device.tna.mode.bySchedule"),
                        '4': MessageProperties.get("device.tna.mode.fixed")
                    };
                    constant.DEVICE_KEYPAD_TNA_MODES_OPTIONS = {
                        '0': MessageProperties.get("device.tna.mode.unused"),
                        '1': MessageProperties.get("device.tna.mode.byUser"),
                        '2': MessageProperties.get("device.tna.mode.bySchedule"),
                        '3': MessageProperties.get("device.tna.mode.lastChoice"),
                        '4': MessageProperties.get("device.tna.mode.fixed")
                    };
                    constant.DEVICE_TNA_MODE_DEFAULT = '1';
                    constant.DEVICE_TNA_EVENT_KEYS_OBJECT_LIST = [
                        {key: 0, 'name': MessageProperties.get("device.tna.eventKey.f1"), 'order': 1},
                        {key: 1, 'name': MessageProperties.get("device.tna.eventKey.f2"), 'order': 2},
                        {key: 2, 'name': MessageProperties.get("device.tna.eventKey.f3"), 'order': 3},
                        {key: 3, 'name': MessageProperties.get("device.tna.eventKey.f4"), 'order': 4},
                        {key: 4, 'name': MessageProperties.get("device.tna.eventKey.ext1"), 'order': 5},
                        {key: 5, 'name': MessageProperties.get("device.tna.eventKey.ext2"), 'order': 6},
                        {key: 6, 'name': MessageProperties.get("device.tna.eventKey.ext3"), 'order': 7},
                        {key: 7, 'name': MessageProperties.get("device.tna.eventKey.ext4"), 'order': 8},
                        {key: 8, 'name': MessageProperties.get("device.tna.eventKey.ext5"), 'order': 9},
                        {key: 9, 'name': MessageProperties.get("device.tna.eventKey.ext6"), 'order': 10},
                        {key: 10, 'name': MessageProperties.get("device.tna.eventKey.ext7"), 'order': 11},
                        {key: 11, 'name': MessageProperties.get("device.tna.eventKey.ext8"), 'order': 12},
                        {key: 12, 'name': MessageProperties.get("device.tna.eventKey.ext9"), 'order': 13},
                        {key: 13, 'name': MessageProperties.get("device.tna.eventKey.ext10"), 'order': 14},
                        {key: 14, 'name': MessageProperties.get("device.tna.eventKey.ext11"), 'order': 15},
                        {key: 15, 'name': MessageProperties.get("device.tna.eventKey.ext12"), 'order': 16}
                    ];
                    constant.DEVICE_TNA_FIXED_CODE_OPTIONS = [];
                    constant.DEVICE_TNA_FIXED_CODE_OPTIONS['0'] = MessageProperties.get("common.none");
                    for (var keyIndex = 0; keyIndex < constant.DEVICE_TNA_EVENT_KEYS_OBJECT_LIST.length; keyIndex++) {
                        var oneItem = constant.DEVICE_TNA_EVENT_KEYS_OBJECT_LIST[keyIndex];
                        constant.DEVICE_TNA_FIXED_CODE_OPTIONS['' + (oneItem.key + 1)] = oneItem.name;
                    }

                    constant.DEVICE_DISPLAY_THEME_OPTIONS = [];
                    constant.DEVICE_DISPLAY_THEME_OPTIONS['0'] = MessageProperties.get("device.info.theme") + " 1";
                    constant.DEVICE_DISPLAY_THEME_OPTIONS['1'] = MessageProperties.get("device.info.theme") + " 2";
                    constant.DEVICE_DISPLAY_THEME_OPTIONS['2'] = MessageProperties.get("device.info.theme") + " 3";
                    constant.DEVICE_DISPLAY_THEME_OPTIONS['3'] = MessageProperties.get("device.info.theme") + " 4";
                    constant.DEVICE_DISPLAY_THEME_OPTION_DEFAULT = '0';

                    constant.DEVICE_DISPLAY_BACKGROUND_OPTIONS = {
                        '0': MessageProperties.get("device.display.background.logo"),
                        '1': MessageProperties.get("device.display.background.notice"),
                        '2': MessageProperties.get("device.display.background.slideShow")
                    };
                    constant.DEVICE_DISPLAY_BACKGROUND = {
                        LOGO: '0',
                        NOTICE: '1',
                        SLIDE_SHOW: '2'
                    };
                    constant.BIOSTAR_RESOURCE_DATA_TYPE = {
                        UI: '0',
                        NOTICE: '1',
                        LOGO: '2',
                        SLIDE_SHOW: '3',
                        SOUND: '4'
                    };
                    constant.DEVICE_DISPLAY_BACKGROUND_OPTION_DEFAULT = '0';
                    constant.DEVICE_DISPLAY_SOUNDS = {
                        '0': MessageProperties.get("device.display.sound.start"),
                        '1': MessageProperties.get("device.display.sound.verifySuccessful"),
                        '2': MessageProperties.get("device.display.sound.verifyFailed")
                    };
                    constant.PREFERENCE_LANGUAGE_OPTIONS = {
                        'en': 'English',
                        'ko': '한국어'
                    };
                    constant.KEYPAD_DEVICE_TYPES = {};
                    constant.KEYPAD_DEVICE_TYPES[constant.DEVICE_TYPE.BIO_LITE_NET] = constant.DEVICE_TYPE.BIO_LITE_NET;
                    constant.KEYPAD_DEVICE_TYPES[constant.DEVICE_TYPE.BIO_STATION_2] = constant.DEVICE_TYPE.BIO_STATION_2;
                    constant.KEYPAD_DEVICE_TYPES[constant.DEVICE_TYPE.BIO_STATION_T2] = constant.DEVICE_TYPE.BIO_STATION_T2;
                    constant.KEYPAD_DEVICE_TYPES[constant.DEVICE_TYPE.FACE_STATION_2] = constant.DEVICE_TYPE.FACE_STATION_2;
                });
            }

            localizationConstantVariables();
            console.log("const built");
//            deepFreeze(constant);
            return constant;
        }]);
});
