'use strict';

define([
    'modules/common/commonModule',
    'modules/common/services/device-agent',
    'modules/common/services/user-agent',
    'modules/common/services/door-agent'
], function () {

    var app = angular.module('commonModule');
    app.service('DatagridFilterService', [
        '$q', '$rootScope', 'API', 'Const', 'Util', 'DeviceAgent', 'UserAgent', 'DoorAgent',
        function ($q, $rootScope, API, Const, Util, DeviceAgent, UserAgent, DoorAgent) {

        var filterObject = {}
        var originalDataList = [];
        var callbackFunction = undefined;
        var viewId;
        var filterOptionItems = {
            doorList: [],
            deviceList: [],
            userGroupList: [],
            userList: [],
            eventList: []
        };

        var filterService = {
            getViewId: function () {
                return viewId;
            },
            setViewId: function (paramViewId) {
                viewId = paramViewId;
            },
            getViewFilterObject: function (filterObjectList, paramViewId) {
                if (filterObjectList != undefined) {
                    var viewFilterObject = filterObjectList[paramViewId];
                    if (viewFilterObject == undefined) {
                        viewFilterObject = {
                            filterList: []
                        };
                        filterObjectList[paramViewId] = viewFilterObject;
                        this.saveFilterObjectListToStorage(filterObjectList);
                    }
                    return viewFilterObject;
                }
                return;
            },
            getOriginalDataList: function () {
                return originalDataList;
            },
            setOriginalDataList: function (paramOriginalDataList) {
                originalDataList = paramOriginalDataList;
            },
            getFilterObjectListStorageKey: function () {
                return Const.LOCAL_STORAGE_KEY_PREFIX + "_monitoring_filters";
            },
            saveFilterObjectListToStorage: function (filterList) {
                localStorage.setItem(this.getFilterObjectListStorageKey(), angular.toJson(filterList));
            },
            getFilterObjectListFromStorage: function () {
                return angular.fromJson(localStorage.getItem(this.getFilterObjectListStorageKey()));
            },
            findFilterObjectByIdFromLocalStorage: function (id) {
                var filterObjectList = this.getFilterObjectListFromStorage();
                if (filterObjectList != undefined) {
                    var viewFilterObject = this.getViewFilterObject(filterObjectList, viewId);
                    if (viewFilterObject != undefined) {
                        return this.findFilterObjectById(viewFilterObject, id);
                    }
                }
                return;
            },
            findFilterObjectById: function (viewFilterObject, id) {
                for (var i = 0; i < viewFilterObject.filterList.length; i++) {
                    var filterObject = viewFilterObject.filterList[i];
                    if (filterObject.id == id) {
                        return filterObject;
                    }
                }
                return;
            },
            saveFilter: function (filterId) {
                console.log("saveFilter:" + filterId);
                var filterObjectList = this.getFilterObjectListFromStorage();
                if (filterObjectList == undefined) {
                    filterObjectList = {};
                }
                var viewFilterObject = this.getViewFilterObject(filterObjectList, viewId);
                if (filterId == undefined) {
                    filterObject.id = this.getNextFilterId(viewId);
                    viewFilterObject.filterList.push(filterObject);
                } else {
                    filterObject.id = filterId;
                    var found = false;
                    for (var i = 0; i < viewFilterObject.filterList.length; i++) {
                        if (viewFilterObject.filterList[i].id == filterId) {
                            filterObject.name = viewFilterObject.filterList[i].name;
                            viewFilterObject.filterList[i] = filterObject;
                            found = true;
                            break;
                        }
                    }
                    if (found == false) {
                        viewFilterObject.filterList.push(filterObject);
                    }
                }
                this.saveFilterObjectListToStorage(filterObjectList);
                return filterObject;
            },
            getNextFilterId: function (viewId) {
                return '' + (new Date()).getTime();
            },
            updateFilterName: function (viewId, filterId, newName) {
                var filterObjectList = this.getFilterObjectListFromStorage();
                if (filterObjectList == undefined) {
                    filterObjectList = {};
                }
                var viewFilterObject = this.getViewFilterObject(filterObjectList, viewId);
                var filterObject = this.findFilterObjectById(viewFilterObject, filterId);
                filterObject.name = newName;
                this.saveFilterObjectListToStorage(filterObjectList);
            },
            deleteFilter: function (viewId, filterId) {
                var filterObjectList = this.getFilterObjectListFromStorage();
                if (filterObjectList == undefined) {
                    filterObjectList = {};
                }
                var viewFilterObject = this.getViewFilterObject(filterObjectList, viewId);
                for (var i = 0; i < viewFilterObject.filterList.length; i++) {
                    var filterObject = viewFilterObject.filterList[i];
                    if (filterObject.id == filterId) {
                        viewFilterObject.filterList.splice(i, 1);
                    }
                }
                this.saveFilterObjectListToStorage(filterObjectList);
            },
            getFilterObject: function () {
                return filterObject;
            },
            setFilterObject: function (paramFilterObject) {
                filterObject = paramFilterObject;
            },
            getFilterOptionItems: function () {
                return filterOptionItems;
            },
            setFilterOptionItems: function (paramFilterOptionItems) {
                filterOptionItems = paramFilterOptionItems;
            },
            setCallbackFunction: function (paramCallbackFunction) {
                callbackFunction = paramCallbackFunction;
            },
            getCallbackFunction: function () {
                return callbackFunction;
            },
            init: function () {
                API.get('event_types').then(function (data) {
                    for (var i = data.EventTypeCollection.rows.length; i--;) {
                        var item = data.EventTypeCollection.rows[i];
                        var newItem = {
                            id: item.code,
                            name: item.name
                        };
                        if (!filterOptionItems.eventList.contains(newItem)) {
                            filterOptionItems.eventList.push(newItem);
                        }
                    }
                });
                if (DoorAgent.getAllDoorsWithoutStatus().then) {
                    DoorAgent.getAllDoorsWithoutStatus().then(function success(data) {
                        filterOptionItems.doorList = data;
                    });
                } else {
                    filterOptionItems.doorList = DoorAgent.getAllDoorsWithoutStatus();
                }
                if (DeviceAgent.getAllDevicesWithoutStatus().then) {
                    DeviceAgent.getAllDevicesWithoutStatus().then(function success(data) {
                        filterOptionItems.deviceList = data;
                    });
                } else {
                    filterOptionItems.deviceList = DeviceAgent.getAllDevicesWithoutStatus();
                }
                UserAgent.getUserGroups().then(function success(data) {
                    filterOptionItems.userGroupList = data.UserGroupCollection.rows;
                    filterOptionItems.userGroupList.sort(Util.dynamicSort('name'));
                });
            },
            isFilterObjectEmpty: function () {
                if (filterObject.dateRange.datetime != undefined && filterObject.dateRange.datetime.length > 0) {
                    return false;
                } else if (filterObject.doorName != undefined && filterObject.doorName.length > 0) {
                    return false;
                } else if (filterObject.deviceId != undefined && filterObject.deviceId.length > 0) {
                    return false;
                } else if (filterObject.deviceName != undefined && filterObject.deviceName.length > 0) {
                    return false;
                } else if (filterObject.userGroupName != undefined && filterObject.userGroupName.length > 0) {
                    return false;
                } else if (filterObject.userName != undefined && filterObject.userName.length > 0) {
                    return false;
                } else if (filterObject.eventName != undefined && filterObject.eventName.length > 0) {
                    return false;
                }
                return true;
            },
            clearFilterObject: function () {
                filterObject = {
                    dateRange: {
                        datetime: [],
                        fromDatetime: '',//today.toISOString(),
                        toDatetime: ''//tomorrow.toISOString()
                    },
                    doorName: [],
                    deviceId: [],
                    deviceName: [],
                    userGroupName: [],
                    userName: [],
                    eventName: []
                };
            },
            addFilterItems: function (filterSelectedInput) {
                if (filterSelectedInput.filterType != undefined) {
                    if (Const.DATA_FILTER.DEVICE_ID == filterSelectedInput.filterType) {
                        filterObject.deviceId = filterSelectedInput.filteredItems.deviceList;
                    } else if (Const.DATA_FILTER.DEVICE_NAME == filterSelectedInput.filterType) {
                        filterObject.deviceName = filterSelectedInput.filteredItems.deviceNameList;
                    } else if (Const.DATA_FILTER.DOOR_NAME == filterSelectedInput.filterType) {
                        filterObject.doorName = filterSelectedInput.filteredItems.doorList;
                    } else if (Const.DATA_FILTER.EVENT_NAME == filterSelectedInput.filterType) {
                        filterObject.eventName = filterSelectedInput.filteredItems.eventList;
                    } else if (Const.DATA_FILTER.USER_GROUP_NAME == filterSelectedInput.filterType) {
                        filterObject.userGroupName = filterSelectedInput.filteredItems.userGroupList;
                    } else if (Const.DATA_FILTER.USER_NAME == filterSelectedInput.filterType) {
                        filterObject.userName = filterSelectedInput.filteredItems.userList;
                    } else if (Const.DATA_FILTER.EVENT_DATE == filterSelectedInput.filterType) {
                        filterObject.dateRange.datetime = [];
                        filterObject.dateRange.fromDatetime = new Date(filterSelectedInput.fromDatetime);
                        filterObject.dateRange.toDatetime = new Date(filterSelectedInput.toDatetime);
                        filterObject.dateRange.datetime.push(filterObject.dateRange.fromDatetime);
                        filterObject.dateRange.datetime.push(filterObject.dateRange.toDatetime);
                    }
//                    var savedFilter = this.saveFilter(filterObject.id);
                    if (callbackFunction != undefined) {
                        callbackFunction();
                    }
//                    return savedFilter;
                }
            },
            removeFilterItem: function ($event, filterType, filterItem) {
                if (Const.DATA_FILTER.DEVICE_ID == filterType) {
                    Util.removeItemByIdFromList(filterObject.deviceId, filterItem);
                } else if (Const.DATA_FILTER.DEVICE_NAME == filterType) {
                    Util.removeItemByIdFromList(filterObject.deviceName, filterItem);
                } else if (Const.DATA_FILTER.DOOR_NAME == filterType) {
                    Util.removeItemByIdFromList(filterObject.doorName, filterItem);
                } else if (Const.DATA_FILTER.EVENT_NAME == filterType) {
                    Util.removeItemByIdFromList(filterObject.eventName, filterItem);
                } else if (Const.DATA_FILTER.USER_GROUP_NAME == filterType) {
                    Util.removeItemByIdFromList(filterObject.userGroupName, filterItem);
                } else if (Const.DATA_FILTER.USER_NAME == filterType) {
                    Util.removeItemByIdFromList(filterObject.userName, filterItem);
                } else if (Const.DATA_FILTER.EVENT_DATE == filterType) {
                    var foundIndex = -1;
                    for (var i = 0; i < filterObject.dateRange.datetime.length; i++) {
                        var dataDate = (filterObject.dateRange.datetime[i] instanceof Date) ? filterObject.dateRange.datetime[i] : new Date(filterObject.dateRange.datetime[i]);
                        var filterItemDate = (filterItem instanceof Date) ? filterItem : new Date(filterItem);
                        if (dataDate.getTime() == filterItemDate.getTime()) {
                            foundIndex = i;
                            break;
                        }
                    }
                    if (foundIndex > -1) {
                        // remove range
                        filterObject.dateRange.datetime = [];
                        filterObject.dateRange.fromDatetime = '';
                        filterObject.dateRange.toDatetime = '';
                    }
                }
                angular.element($event.target).parent("li").remove();

//                this.saveFilter(filterObject.id);

                if (callbackFunction != undefined) {
                    callbackFunction();
                }
            },
            filterData: function (gridId) {
                var newList = new Array();
                newList = newList.concat(originalDataList);
                console.log("original data size=" + newList.length);
                if (originalDataList.length > 0) {

                    for (var i = 0; i < newList.length; i++) {
                        var resultFound = true;
                        if (resultFound == true && filterObject.deviceId != undefined && filterObject.deviceId.length > 0) {
                            resultFound = false;
                            for (var index2 = 0; index2 < filterObject.deviceId.length; index2++) {
                                var dataValue = '';
                                if (newList[i].device_id != undefined) {
                                    dataValue = newList[i].device_id.id;
                                } else if (newList[i].id != undefined) {
                                    dataValue = newList[i].id;
                                }
                                if (dataValue == filterObject.deviceId[index2].id) {
                                    resultFound = true;
                                    break;
                                }
                            }
                            if (resultFound == false) {
                                newList.splice(i, 1);
                                i--;
                            }
                        }
                        if (resultFound == true && filterObject.deviceName != undefined && filterObject.deviceName.length > 0) {
                            resultFound = false;
                            for (var index2 = 0; index2 < filterObject.deviceName.length; index2++) {
                                var dataValue = '';
                                if (newList[i].device_id != undefined) {
                                    dataValue = newList[i].device_id.id;
                                } else if (newList[i].id != undefined) {
                                    dataValue = newList[i].id;
                                }
                                if (dataValue == filterObject.deviceName[index2].id) {
                                    resultFound = true;
                                    break;
                                }
                            }
                            if (resultFound == false) {
                                newList.splice(i, 1);
                                i--;
                            }
                        }
                        if (resultFound == true && filterObject.doorName != undefined && filterObject.doorName.length > 0) {
                            resultFound = false;
                            for (var index2 = 0; index2 < filterObject.doorName.length; index2++) {
                                var dataValue = '';
                                if (newList[i].door_id != undefined) {
                                    dataValue = newList[i].door_id.id;
                                } else if (newList[i].id != undefined) {
                                    dataValue = newList[i].id;
                                }
                                if (dataValue == filterObject.doorName[index2].id) {
                                    resultFound = true;
                                    break;
                                }
                            }
                            if (resultFound == false) {
                                newList.splice(i, 1);
                                i--;
                            }
                        }
                        if (resultFound == true && filterObject.userName != undefined && filterObject.userName.length > 0) {
                            resultFound = false;
                            for (var index2 = 0; index2 < filterObject.userName.length; index2++) {
                                var dataValue = '';
                                if (newList[i].user_id != undefined) {
                                    dataValue = newList[i].user_id.user_id;
                                } else if (newList[i].id != undefined) {
                                    dataValue = newList[i].id;
                                }
                                if (dataValue == filterObject.userName[index2].id) {
                                    resultFound = true;
                                    break;
                                }
                            }
                            if (resultFound == false) {
                                newList.splice(i, 1);
                                i--;
                            }
                        }
                        if (resultFound == true && filterObject.userGroupName != undefined && filterObject.userGroupName.length > 0) {
                            resultFound = false;
                            for (var index2 = 0; index2 < filterObject.userGroupName.length; index2++) {
                                var dataValue = '';
                                if (newList[i].userGroupName != undefined) {
                                    dataValue = newList[i].userGroupName.id;
                                } else if (newList[i].id != undefined) {
                                    dataValue = newList[i].id;
                                }
                                if (dataValue == filterObject.userGroupName[index2].id) {
                                    resultFound = true;
                                    break;
                                }
                            }
                            if (resultFound == false) {
                                newList.splice(i, 1);
                                i--;
                            }
                        }
                        if (resultFound == true && filterObject.eventName != undefined && filterObject.eventName.length > 0) {
                            resultFound = false;
                            for (var index2 = 0; index2 < filterObject.eventName.length; index2++) {
                                var dataValue = '';
                                if (newList[i].event_type_id != undefined) {
                                    dataValue = newList[i].event_type_id.code;
                                } else if (newList[i].code != undefined) {
                                    dataValue = newList[i].code;
                                } else if (newList[i].id != undefined) {
                                    dataValue = newList[i].id;
                                }
                                if (dataValue == filterObject.eventName[index2].id) {
                                    resultFound = true;
                                    break;
                                }
                            }
                            if (resultFound == false) {
                                newList.splice(i, 1);
                                i--;
                            }
                        }
                        if (resultFound == true && filterObject.dateRange.datetime != undefined && filterObject.dateRange.datetime.length > 0) {
                            resultFound = false;
                            var beginDate = (filterObject.dateRange.fromDatetime instanceof Date) ? filterObject.dateRange.fromDatetime : new Date(filterObject.dateRange.fromDatetime);
                            var endDate = (filterObject.dateRange.toDatetime instanceof Date) ? filterObject.dateRange.toDatetime : new Date(filterObject.dateRange.toDatetime);
                            var dataDate = '';
                            if (angular.isDefined(newList[i].device_datetime)) {
                                dataDate = (newList[i].device_datetime instanceof Date) ? newList[i].device_datetime : new Date(newList[i].device_datetime);
                            } else if (angular.isDefined(newList[i].datetime)) {
                                dataDate = (newList[i].datetime instanceof Date) ? newList[i].datetime : new Date(newList[i].datetime);
                            } else if (angular.isDefined(newList[i].event_id.datetime)) {
                                dataDate = (newList[i].event_id.datetime instanceof Date) ? newList[i].event_id.datetime : new Date(newList[i].event_id.datetime);
                            }
                            if (beginDate.getTime() > dataDate.getTime() || endDate.getTime() <= dataDate.getTime()) {
                                newList.splice(i, 1);
                                i--;
                            }
                        }
                    }
                }

                console.log("data size reduced to " + newList.length);
                if (newList.length > Const.MAX_REAL_TIME_DATA_SIZE) {
                    console.log("data size reduced from " + newList.length + " to " + Const.MAX_REAL_TIME_DATA_SIZE);
                    newList = newList.slice(0, Const.MAX_REAL_TIME_DATA_SIZE);
                }

                var data = {
                    total: newList.length,
                    rows: newList
                };
                angular.element('#' + gridId).datagrid('loadData', data);
                return newList;
            }
        };

        $rootScope.$on('LoginToSuccess', function (event) {
            console.log('DatagridFilterService: LoginToSuccess');
            filterService.init();
        });

        return filterService;
    }]);

});