'use strict';

define([
    'modules/common/commonModule'
], function () {
    var app = angular.module('commonModule');
//    app.service('DeviceAgent', function DeviceAgent($q, $rootScope, API, Const, LocalDB) {
    app.service('DeviceAgent', ['$timeout',
        '$q', '$rootScope', 'API', 'Const', 'Util', 'MessageService', 'MessageProperties',
        function DeviceAgent($timeout, $q, $rootScope, API, Const, Util, MessageService, MessageProperties) {

            var DEVICE_GROUP,
                DEVICE_GROUP_WITH_DEVICES,
                treeState,
                DEVICE_GROUP_WITHOUT_DEVICES,
                DEVICE_GROUP_PATH_DATA = {},
                httpTimeOut = {},
                canceller = {},
                clearAllDevicesMap = true;

            var allDeviceListWithoutStatus = [];
            var allDeviceMapWithStatus = {};

            // ---------------------- tree 상태 저장 관련 s ----------------------------------------------
            var extractTreeState = function (node, treeState) {
                if (node.children && node.children.length > 0) {
                    if (angular.isDefined(node.isDevice) && node.isDevice) {
                        treeState[node.id + node.isDevice.toString()] = node.state;
                    } else {
                        treeState[node.id] = node.state;
                    }
                    for (var i = 0; i < node.children.length; i++) {
                        extractTreeState(node.children[i], treeState);
                    }
                }
            };

            var saveTreeState = function () {
                var rootNode = angular.element('#devicegrouptree').tree('getRoot');
                treeState = {};
                extractTreeState(rootNode, treeState);
                Util.setSessionObject(Const.SESSION_STORAGE_KEY.DEVICE_GROUP_TREE_STATE, treeState);
            };

            var loadTreeState = function () {
                treeState = Util.getSessionObject(Const.SESSION_STORAGE_KEY.DEVICE_GROUP_TREE_STATE) || {};
            };
            // ---------------------- tree 상태 저장 관련 e ----------------------------------------------


            function setTreeNode(node, deviceObject) {
                node.id = deviceObject.id;
                node.name = deviceObject.name;
                node.text = deviceObject.name;
                if (deviceObject.device_type_id != undefined) {
                    node.device_type_id = deviceObject.device_type_id;
                }
                if (deviceObject.parent_node_id != undefined) {
                    node.parent_node_id = deviceObject.parent_node_id;
                }
                //node.connection_mode = (deviceObject.lan != undefined && deviceObject.lan.connection_mode != undefined) ? deviceObject.lan.connection_mode : Const.CONNECTION_MODE.SERVER_TO_DEVICE;
                node.isDevice = deviceObject.isDevice || false;
                node.rs485 = deviceObject.rs485 || {mode: Const.RS485_MODE.SLAVE};
                if (node.isDevice) {
                    node.state = treeState[node.id + node.isDevice.toString()] || 'open';
                } else {
                    node.state = treeState[node.id] || 'open';
                }
                node.status = deviceObject.status;
                node.iconCls = deviceObject.iconCls || '';

                if (node.status != undefined) {
                    if (node.device_type_id != undefined) {
                        node.iconCls = Const.DEVICE_TYPE_ICON[node.device_type_id.id] + ' ' + Const.DEVICE_STATUS_ICON_CLASS[node.status];
                    }
                }
//            console.log("device tree node updated", node);
            }

            var setTreeValue = function (data, rebuildAllDeviceList) {
                var node = {};
                setTreeNode(node, data);
                node.children = [];

                if (data.devices && data.devices.length > 0) {
                    if (rebuildAllDeviceList == true) {
                        Util.rebuildListWithoutStatusAndMapWithStatus(data.devices, allDeviceListWithoutStatus, allDeviceMapWithStatus);
                        MessageService.setAllDeviceMapWithStatus(allDeviceMapWithStatus);
                    }
                    for (var i = 0; i < data.devices.length; i++) {
                        data.devices[i].isDevice = true;
                        data.devices[i].iconCls = Const.DEVICE_TYPE_ICON[data.devices[i].device_type_id.id] + ' ' + Const.DEVICE_STATUS_ICON_CLASS[data.devices[i].status];
                        data.devices[i]['deviceData'] = angular.copy(data.devices[i]);
                        node.children.push(setTreeValue(data.devices[i], rebuildAllDeviceList));
                    }
                }
                if (angular.isDefined(data.rs485) && angular.isDefined(data.rs485.channels) &&
                    angular.isDefined(data.rs485.channels[0].slave_devices) && data.rs485.channels[0].slave_devices.length > 0) {
                    if (rebuildAllDeviceList == true) {
                        Util.rebuildListWithoutStatusAndMapWithStatus(data.rs485.channels[0].slave_devices,
                            allDeviceListWithoutStatus, allDeviceMapWithStatus, null, {'id': data.id, 'name': data.name});
                        MessageService.setAllDeviceMapWithStatus(allDeviceMapWithStatus);
                    }
                    for (var j = 0; j < data.rs485.channels[0].slave_devices.length; j++) {
                        data.rs485.channels[0].slave_devices[j].isDevice = true;
//                  data.rs485.channels[0].slave_devices[i].deviceData = data.rs485.channels[0].slave_devices[i];

                        if (angular.isDefined(data.rs485.channels[0].slave_devices[j].device_type_id)) {
                            data.rs485.channels[0].slave_devices[j].iconCls = Const.DEVICE_TYPE_ICON[data.rs485.channels[0].slave_devices[j].device_type_id.id] + ' ' + Const.DEVICE_STATUS_ICON_CLASS[data.rs485.channels[0].slave_devices[j].status];
                        } else {
                            data.rs485.channels[0].slave_devices[j].iconCls = Const.DEVICE_TYPE_ICON[6] + ' ' + Const.DEVICE_STATUS_ICON_CLASS[data.rs485.channels[0].slave_devices[j].status];
                        }
                        data.rs485.channels[0].slave_devices[j].parent_node_id = node.id;
                        node.children.push(setTreeValue(data.rs485.channels[0].slave_devices[j], rebuildAllDeviceList));
                    }
                }
                if (data.device_groups && data.device_groups.length > 0) {
                    for (var i = 0; i < data.device_groups.length; i++) {
                        data.device_groups[i].isDevice = false;
                        node.children.push(setTreeValue(data.device_groups[i], rebuildAllDeviceList));
                    }
                }
                return node;
            };

            var setTreeValueOnlyGroup = function (data) {
                var node = {
                    'id': data.id,
                    'text': data.name,
                    'name': data.name,
                    'children': []
                };
                var prefix = '';
                if (data.id == '1') {
                    DEVICE_GROUP_PATH_DATA[data.id] = data.name;
                } else {
                    prefix = '… > ';
                }
                if (data.device_groups && data.device_groups.length > 0 && data.isDevice != true) {
                    for (var i = 0; i < data.device_groups.length; i++) {
                        if (data.device_groups[i].isDevice != true) {
                            node.children.push(setTreeValueOnlyGroup(data.device_groups[i]));
                            DEVICE_GROUP_PATH_DATA[data.device_groups[i].id] = prefix + data.name + ' > ' + data.device_groups[i].name;
                        }
                    }
                }
                return node;
            };

            function setTreeDatatoLocalVariable(data) {
                console.log("deviceAgent setTreeDatatoLocalVariable");
                allDeviceListWithoutStatus = [];
                allDeviceMapWithStatus = {};
                if (data.DeviceGroup != undefined && data.DeviceGroup.id == Const.DEFAULT_ALL_GROUP_ID) {
                    data.DeviceGroup.name = MessageProperties.get("menu.device.allDeviceGroups");
                }
                loadTreeState();
                DEVICE_GROUP_WITH_DEVICES = setTreeValue(data.DeviceGroup, true);
                DEVICE_GROUP = setTreeValueOnlyGroup(data.DeviceGroup);
                allDeviceListWithoutStatus.sort(Util.dynamicSort("name"));
            }

            function updateStatusOnTree(treeNode, deviceObject) {
                var found = false;
                if (treeNode != undefined) {
                    if (treeNode.id == deviceObject.id) {
                        found = true;
                        treeNode.status = deviceObject.status;
                        if (deviceObject.status != undefined) {
                            if (treeNode.device_type_id != undefined) {
                                treeNode.iconCls = Const.DEVICE_TYPE_ICON[treeNode.device_type_id.id] + ' ' + Const.DEVICE_STATUS_ICON_CLASS[deviceObject.status];
                            }
                        }
                    }
                    if (found == false) {
                        for (var i = 0; i < treeNode.children.length; i++) {
                            if (updateStatusOnTree(treeNode.children[i], deviceObject) == true) {
                                break;
                            }
                        }
                    }
                }
                return found;
            }

            function updateDeviceOnTree(treeNode, deviceObject) {
                var found = false;
                if (treeNode != undefined) {
                    if (treeNode.id == deviceObject.id) {
                        found = true;
                        setTreeNode(treeNode, deviceObject);
                        treeNode.iconCls = deviceObject.iconCls || '';
                    }
                    if (found == false) {
                        for (var i = 0; i < treeNode.children.length; i++) {
                            if (updateDeviceOnTree(treeNode.children[i], deviceObject) == true) {
                                break;
                            }
                        }
                    }
                }
                return found;
            }

            var deviceSpecDefer = undefined;
            var deviceAgent = {
                init: function () {
                    API.get('device_groups/1?recursive=true').then(function (data) {
                        console.log("deviceAgent init");
                        setTreeDatatoLocalVariable(data);
                        $rootScope.$emit('ReceiveDeviceTreeDataSuccess', DEVICE_GROUP_WITH_DEVICES);
                    });
                },
                getDeviceGroupWithoutDevice: function () {
                    var defer = $q.defer();
                    if (DEVICE_GROUP_WITHOUT_DEVICES && DEVICE_GROUP_WITHOUT_DEVICES.length > 0) {
                        defer.resolve(DEVICE_GROUP_WITHOUT_DEVICES);
                    } else {
                        API.get('device_groups', undefined, undefined, undefined, undefined, undefined, false).then(function success(data) {
                            DEVICE_GROUP_WITHOUT_DEVICES = data.DeviceGroupCollection.rows;
                            defer.resolve(DEVICE_GROUP_WITHOUT_DEVICES);
                        });
                    }

                    return defer.promise;
                },
                setSelectedDevice: function (device) {
                    sessionStorage.setItem('selectedDevice', angular.toJson(device));
                },
                getSelectedDevice: function () {
                    var item = sessionStorage.getItem('selectedDevice');
                    if (item != undefined) {
                        return angular.fromJson(item);
                    }
                    return undefined;
                },
                /**
                 * INFO: http request Reject 예제
                 * @param deviceID
                 * @returns {defer.promise|*}
                 */
                getDevice: function (deviceID) {
                    var options = {
                        url: 'devices/' + deviceID,
                        displaySuccessPopoup: false,
                        displayErrorPopup: true,
                        displayLoading: true,
                        successCallback: function (responseObject, deviceId) {
                            if (responseObject.code == Const.SERVER_RESPONSE_CODE.DEVICE_HAS_INVALID_CONFIG) {
                                deviceAgent.resetDevice(deviceId);
                            }
                        }
                    };

                    return API.getWithOptions(options);
                },
                /**
                 * INFO: http request Reject 예제
                 */
                rejectGetDevice: function () {
                    API.rejectRequest('getDevice');
                },
                getDeviceList: function (params) {
                    return API.get('devices', undefined, params);
                },
                resetDevice: function (deviceId) {
                    return deviceAgent.resetDevices([
                        {
                            id: deviceId
                        }
                    ]);
                },
                resetDevices: function (deviceObjectList) {
                    var defer = $q.defer();
                    API.post('devices/factory_reset', undefined, {
                        'DeviceCollection': {
                            'total': deviceObjectList.length,
                            'rows': deviceObjectList
                        }
                    }).then(function (data) {
                        defer.resolve(data);
                    }, function error(data) {
                        defer.reject(data);
                    });
                    return defer.promise;
                },
                /**
                 * INFO: http request Reject 예제
                 */
                rejectGetDeviceList: function () {
                    if (httpTimeOut.getDeviceList) {
                        console.log('device httpTimeOut');
                        canceller.getDeviceList.resolve('cancelled get Device');
                        httpTimeOut.getDeviceList = undefined;
                        canceller.getDeviceList = $q.defer();
                    }
                },
                removeDevice: function () {
                    sessionStorage.removeItem('selectedDevice');
                },
                syncLocalDB: function () {


//                API.get('devices').then(function (data) {
//                    if (data && data.DeviceCollection && data.DeviceCollection.rows && data.DeviceCollection.rows.length > 0) {
//                        LocalDB.doWorkTest('testData').then(function () {
//                            for (var index in data.DeviceCollection.rows) {
//                                LocalDB.addDevice(data.DeviceCollection.rows[index]);
//                            }
//                        });
//                    }
//                });
                },
                getDeviceGroupTreeOnlyGroup: function () {
                    if (DEVICE_GROUP) {
                        return DEVICE_GROUP;
                    } else {
                        var defer = $q.defer();
                        var displayLoading = false;
                        API.get('device_groups/1?recursive=true', 'getDeviceGroupTreeOnlyGroup', undefined, undefined, undefined, undefined, displayLoading).then(function (data) {
                            console.log("deviceAgent getDeviceGroupTreeOnlyGroup");
                            setTreeDatatoLocalVariable(data);
                            defer.resolve(DEVICE_GROUP);
                        });
                        return defer.promise;
                    }
                },
                getDeviceGroupRecursiveData: function () {
                    if (DEVICE_GROUP_WITH_DEVICES) {
						Util.log(DEVICE_GROUP_WITH_DEVICES);
						if (DEVICE_GROUP_WITH_DEVICES != undefined && DEVICE_GROUP_WITH_DEVICES.id == Const.DEFAULT_ALL_GROUP_ID) {
							DEVICE_GROUP_WITH_DEVICES.name = MessageProperties.get("menu.device.allDeviceGroups");
							DEVICE_GROUP_WITH_DEVICES.text = MessageProperties.get("menu.device.allDeviceGroups");
						}
                        return DEVICE_GROUP_WITH_DEVICES;
                    } else {
                        var defer = $q.defer();
                        var displayLoading = false;
                        API.get('device_groups/1?recursive=true', 'getDeviceGroupRecursiveData', undefined, undefined, undefined, undefined, displayLoading).then(function (data) {
                            console.log("deviceAgent getDeviceGroupRecursiveData");
                            setTreeDatatoLocalVariable(data);
                            defer.resolve(DEVICE_GROUP_WITH_DEVICES);
                        });
                        return defer.promise;
                    }
                },
                getDeviceGroupRecursiveDataDefer: function () {
                    var defer = $q.defer();
                    if (DEVICE_GROUP_WITH_DEVICES) {
                        defer.resolve(DEVICE_GROUP_WITH_DEVICES);
                    } else {
                        var displayLoading = false;
                        API.get('device_groups/1?recursive=true', 'getDeviceGroupRecursiveData', undefined, undefined, undefined, undefined, displayLoading).then(function (data) {
                            console.log("deviceAgent getDeviceGroupRecursiveData");
                            setTreeDatatoLocalVariable(data);
                            defer.resolve(DEVICE_GROUP_WITH_DEVICES);
                        });
                    }
                    return defer.promise;
                },
                clearDeviceGroupWithDevice: function () {
                    saveTreeState();
                    DEVICE_GROUP_WITH_DEVICES = undefined;
                    DEVICE_GROUP_WITHOUT_DEVICES = undefined;
                },
                getAllDevicesWithoutStatus: function () {
                    if (allDeviceListWithoutStatus != undefined && allDeviceListWithoutStatus.length > 0) {
                        return allDeviceListWithoutStatus;
                    } else {
                        var defer = $q.defer();
                        var displayLoading = false;
                        API.get('device_groups/1?recursive=true', 'getAllDevicesWithoutStatus', undefined, undefined, undefined, undefined, displayLoading).then(function (data) {
                            console.log("deviceAgent getAllDevicesWithoutStatus");
                            setTreeDatatoLocalVariable(data);

                            defer.resolve(allDeviceListWithoutStatus);
                        });
                        return defer.promise;
                    }
                },
                getAllDevicesMapWithStatus: function () {
                    if (allDeviceMapWithStatus != undefined && !Util.isEmpty(allDeviceMapWithStatus)) {
                        return allDeviceMapWithStatus;
                    } else {
                        var defer = $q.defer();
                        var displayLoading = false;
                        API.get('device_groups/1?recursive=true', 'getAllDevicesMapWithStatus', undefined, undefined, undefined, undefined, displayLoading).then(function (data) {
                            console.log("deviceAgent getAllDevicesMapWithStatus");
                            setTreeDatatoLocalVariable(data);

                            defer.resolve(allDeviceMapWithStatus);
                        });
                        return defer.promise;
                    }
                },
                refreshAllDevicesMap: function () {
                    clearAllDevicesMap = true;

                    var defer = $q.defer();
                    if (allDeviceMapWithStatus != undefined && !Util.isEmpty(allDeviceMapWithStatus) && !clearAllDevicesMap) {
                        defer.resolve(allDeviceMapWithStatus);
                    } else {
                        var displayLoading = false;
                        var params = {
                            offset: 0
                        };
                        API.get('devices', 'getAllDevicesMap', params, undefined, undefined, undefined, displayLoading).then(function (data) {
                            console.log("deviceAgent getAllDevicesMap");

                            Util.rebuildListWithoutStatusAndMapWithStatus(data.DeviceCollection.rows, allDeviceListWithoutStatus, allDeviceMapWithStatus);
                            clearAllDevicesMap = false;

                            defer.resolve(allDeviceMapWithStatus);
                        });
                    }
                    return defer.promise;
                },
                getAllDevicesMap: function () {
                    return allDeviceMapWithStatus;
                },
                updateStatus: function (deviceObject) {
                    if (deviceObject != undefined) {
                        if (deviceObject.status != undefined) {
                            var existingDevice = allDeviceMapWithStatus[deviceObject.id];
                            if (existingDevice != undefined) {
                                allDeviceMapWithStatus[deviceObject.id].status = deviceObject.status;
                            }
                            updateStatusOnTree(DEVICE_GROUP_WITH_DEVICES, deviceObject);
                        }
                    }
                },
                updateDevice: function (deviceObject) {
                    if (deviceObject != undefined) {
                        var existingDevice = allDeviceMapWithStatus[deviceObject.id];
                        if (existingDevice != undefined) {
                            existingDevice.name = deviceObject.name;
                            existingDevice.rs485 = deviceObject.rs485;
                            existingDevice.status = deviceObject.status;
                            updateDeviceOnTree(DEVICE_GROUP_WITH_DEVICES, existingDevice);
                            $rootScope.$broadcast(Const.UI_EVENT_TYPE.DEVICE_INFO_UPDATED, existingDevice);
                        }
                    }
                },
                updateAlarm: function (alertObject) {
                    var eventObject = alertObject.event_id;
                    if (eventObject != undefined) {
                        var deviceObject = eventObject.device_id;
                        if (deviceObject != undefined) {
                            var existingDevice = allDeviceMapWithStatus[deviceObject.id];
                            if (existingDevice != undefined) {
                                allDeviceMapWithStatus[deviceObject.id].alert = alertObject;
                            }
                        }
                    }
                },
                updateLastEvent: function (eventObject) {
                    var deviceObject = eventObject.device_id;
                    if (deviceObject != undefined) {
                        var existingDevice = allDeviceMapWithStatus[deviceObject.id];
                        if (existingDevice != undefined) {
                            allDeviceMapWithStatus[deviceObject.id].last_event = eventObject;
                        }
                    }
                },
                syncDevices: function (deviceObjectList) {
                    var params = {
                        DeviceCollection: {
                            total: 1,
                            rows: [
                            ]
                        }
                    };

                    for (var i = 0; i < deviceObjectList.length; i++) {
                        params.DeviceCollection.rows.push({id: deviceObjectList[i].id});
                    }
                    if (deviceObjectList == undefined || deviceObjectList.length < 1) {
                        params = undefined;
                    }
//                console.log("sync device param :: ");
//                console.log(params);
                    API.post('devices/sync', undefined, params, undefined, true).then(function (data) {
//               if (data.DeviceResponse.rows != undefined){
//                   if (data.DeviceResponse.rows[0].code == "0"){
//
//                   }
//               }
                        // web socket will be sent by server when device status is updated
                    });
                },
                fullSyncDevices: function (deviceObjectList) {
                    var params = {
                        DeviceCollection: {
                            total: 1,
                            rows: [
                            ]
                        }
                    };

                    for (var i = 0; i < deviceObjectList.length; i++) {
                        if (deviceObjectList[i].rs485 && deviceObjectList[i].rs485.mode != Const.RS485_MODE.SLAVE) {
                            params.DeviceCollection.rows.push({id: deviceObjectList[i].id});
                        }
                    }
                    API.post('devices/sync?clean=true', undefined, params, undefined, true);
                },
                DeviceSpec: undefined,
                getDeviceSpec: function (deviceTypeID) {
                    if (deviceTypeID != undefined && !angular.isString(deviceTypeID) &&
                        angular.isDefined(deviceTypeID.device_type_id)) {
                        deviceTypeID = deviceTypeID.device_type_id.id.toString();
                    }

                    if (deviceSpecDefer == undefined) {
                        deviceSpecDefer = $q.defer();
                    } else {
                        return deviceSpecDefer.promise;
                    }

                    var deviceSpec = Util.getSessionObject(Const.SESSION_STORAGE_KEY.DEVICE_SPEC);
                    var deviceSpecLastDate = sessionStorage.getItem(Const.SESSION_STORAGE_KEY.DEVICE_SPEC_LAST_DATE);
                    if (deviceSpec != undefined && deviceSpecLastDate != undefined) {
                        // DeviceSpec 이 이미 존재한다 하더라도 너무 오래된 데이터라면 다시 받아올 있도록 갱신한다.
                        //  갱신 주기는 Const.DEVICE_SPEC_UPDATE_INTERVAL 값으로 조절할 수 있다.
                        if ((Date.now() - (new Date(deviceSpecLastDate))) > Const.DEVICE_SPEC_UPDATE_INTERVAL) {
                            deviceAgent.DeviceSpec = undefined;
                        } else if (Util.isEmpty(deviceAgent.DeviceSpec)) {
                            deviceAgent.DeviceSpec = angular.copy(deviceSpec);
                        }
                    }

                    if (deviceAgent.DeviceSpec) {
                        console.log('getDeviceSpec: Direct Return');
                        if (deviceTypeID != undefined) {
                            deviceSpecDefer.resolve(angular.copy(deviceAgent.DeviceSpec[deviceTypeID]));
                        } else {
                            deviceSpecDefer.resolve(angular.copy(deviceAgent.DeviceSpec));
                        }
                        deviceSpecDefer.promise.finally(function () {
                            console.log('deviceSpecDefer.promise:Direct finally');
                            deviceSpecDefer = undefined;
                        });
                    } else {
                        var displayLoading = false;
                        API.get('device_types', undefined, undefined, undefined, undefined, undefined, displayLoading).then(
                            function (data) {
                                console.log('getDeviceSpec:API Callback');
                                deviceAgent.DeviceSpec = {};
                                for (var i = 0; i < data.DeviceTypeCollection.rows.length; i++) {
                                    var spec = {};
                                    spec[data.DeviceTypeCollection.rows[i].id] = data.DeviceTypeCollection.rows[i];
                                    angular.extend(deviceAgent.DeviceSpec, spec);
                                }
                                Util.setSessionObject(Const.SESSION_STORAGE_KEY.DEVICE_SPEC, deviceAgent.DeviceSpec);
                                sessionStorage.setItem(Const.SESSION_STORAGE_KEY.DEVICE_SPEC_LAST_DATE, new Date());

                                if (deviceTypeID != undefined) {
                                    deviceSpecDefer.resolve(angular.copy(deviceAgent.DeviceSpec[deviceTypeID]));
                                } else {
                                    deviceSpecDefer.resolve(angular.copy(deviceAgent.DeviceSpec));
                                }
                            }
                        ).finally(
                            function () {
                                console.log('API.get:finally');
                                deviceSpecDefer.promise.finally(function () {
                                    console.log('deviceSpecDefer.promise:finally');
                                    deviceSpecDefer = undefined;
                                });
                            }
                        );
                    }
                    return deviceSpecDefer.promise;
                },
                getDeviceGroupPath: function (deivceGroupID) {
                    if (deivceGroupID) {
                        return DEVICE_GROUP_PATH_DATA[deivceGroupID];
                    } else {
                        return DEVICE_GROUP_PATH_DATA;
                    }
                }
            };

            $rootScope.$on('getDeviceGroupTreeData', function (event, cbFunction) {
                cbFunction(DEVICE_GROUP_WITH_DEVICES);
            });

            $rootScope.$on('LoginToSuccess', function (event) {
                console.log('DeviceAgent: LoginToSuccess');
                deviceAgent.init();
                deviceAgent.getDeviceSpec().then(function (data) {
                    console.log('DeviceAgent: LoginToSuccess and getDeviceSpec Success');
                });
            });

            /**
             * 장치로부터 WebSocket을 통해 받은 장치의 최신 상태를 업데이트 한다.
             *  TODO: 중복등록 방지코드의 개선 필요. 해당 Controller 에서 등록한 이벤트만 재 등록을 방지
             *  DeviceStatusReceived 이벤트는 DeviceAgent, DeviceController 등 필요에 따라
             *  여러군데 등록되어 실행되므로 중복등록 방지코드에서 선별작업이 필요하다.
             *  현재 중복등록 방지코드는 같은 이름을 무조건 삭제하도록 되어 있는데
             *  DeviceStatusReceived 의 경우 해당 Controller 에서 등록한 이벤트만 재 등록을 방지할 수 있어야 하므로
             *  조금 더 고민해 봐야 한다.
             */
            $rootScope.$on('DeviceStatusReceived', function (event, messageObj) {
                if (deviceAgent.getAllDevicesWithoutStatus().then) {
                    deviceAgent.getAllDevicesWithoutStatus().then(function success(data) {
                        deviceAgent.updateStatus(messageObj);
                    });
                } else {
                    deviceAgent.updateStatus(messageObj);
                }
            });

            $rootScope.$on('DeviceEventReceived', function (event, messageObj) {
                if (deviceAgent.getAllDevicesWithoutStatus().then) {
                    deviceAgent.getAllDevicesWithoutStatus().then(function success(data) {
                        deviceAgent.updateLastEvent(messageObj);
                    });
                } else {
                    deviceAgent.updateLastEvent(messageObj);
                }
            });

            $rootScope.$on('DeviceAlertReceived', function (event, messageObj) {
                if (deviceAgent.getAllDevicesWithoutStatus().then) {
                    deviceAgent.getAllDevicesWithoutStatus().then(function success(data) {
                        deviceAgent.updateAlarm(messageObj);
                    });
                } else {
                    deviceAgent.updateAlarm(messageObj);
                }
            });

            if (API.isLoggedIn() == true) {
                deviceAgent.init();
            }
            return deviceAgent;
        }]);

});