'use strict';

define([
    'modules/common/commonModule'
], function () {
    var app = angular.module('commonModule');
    app.service('DoorAgent', [
        '$log', '$q', '$rootScope', 'Const', 'API', 'Util', 'MessageProperties',
        function ($log, $q, $rootScope, Const, API, Util, MessageProperties) {
            var
                DOOR_DATA,
                DOOR_GROUP_DATA,
                DOOR_GROUP_DATA_WITH_DOORS,
                DOOR_GROUP_PATH_DATA = {},
                httpTimeOut = {},
                canceller = {};

            var allDoorListWithoutStatus = [];
            var allDoorMapWithStatus = {};

            Util.removeSessionObject('getDoorGroupRecursiveData');
            Util.removeSessionObject('getDoorGroupRecursiveDataWithDoors');

            function setTreeData(groupList, includeDoors, rebuildAllDoorList) {
                var i,
                    item,
                    parentObj,
                    expandedState = {},
                    doorGroupIdIndex = {};

                expandedState = window.sessionStorage.getItem('DoorGroupTreeExpandedState');
                if (expandedState) {
                    expandedState = angular.fromJson(expandedState);
                } else {
                    expandedState = {};
                }

                if (rebuildAllDoorList == undefined) {
                    rebuildAllDoorList = false;
                }

                if (rebuildAllDoorList == true) {
                    allDoorListWithoutStatus = [];
                    allDoorMapWithStatus = {};
                }

                if (groupList == undefined) {
                    return;
                }
                //도어 그룹 아이디 인덱스 생성 및 하위 도어그룹 초기화
                for (i = groupList.length; i--;) {
                    doorGroupIdIndex[groupList[i].id] = i;
                    groupList[i].door_groups = [];
                }

                for (i = groupList.length; i--;) {
                    item = groupList[i];
                    if (rebuildAllDoorList == true) {
                        Util.rebuildListWithoutStatusAndMapWithStatus(item.doors, allDoorListWithoutStatus, allDoorMapWithStatus);
                    }
                    if (item && item.parent_id) {
                        parentObj = groupList[doorGroupIdIndex[item.parent_id.id]];
                        if (parentObj) {
                            if (parentObj && !parentObj.door_groups) {
                                parentObj.door_groups = [];
                            }
                            parentObj.door_groups.push(item);

                            // 도어 그룹 경로 툴팁 생성
                            item.path = parentObj.name + ' > ' + item.name;
                            if (item.depth > 1)
                                item.path = '… > ' + item.path;

                            // 유저 목록에서 그룹 경로 표시를 위하여 오브젝트 데이터 생성
                            DOOR_GROUP_PATH_DATA[item.id] = item.path;
                        }
                    }

                    if (item.doors && item.doors.length) {
                        item.doors = item.doors.sort(Util.dynamicSort('name'));
                    }

                    if (includeDoors == true) {
                        item.children = item.doors || [];
                    }

                    if (!item.children) {
                        item.children = [];
                    }

                    item.door_groups = item.door_groups.sort(Util.dynamicSort('name'));

                    item.children = item.children.concat(item.door_groups || []);

//                item.children = item.door_groups || []; // JQuery Easy UI Tree 를 위한 필드, 없으면 하위 노드 생성 안됨
//                if (includeDoors == true) {
//                    if (item.doors && item.doors.length) {
//                        item.children = item.children.concat(item.doors);
//                    }
//                } else {
//                    item.doors = [];
//                }

                    if (item.children && item.children.length > 0) {
                        item.state = expandedState[item.id] || 'closed';
                        if (!expandedState[item.id]) {
                            expandedState[item.id] = 'closed';
                        }
                    }

                    item.isDoorGroups = true;
                    if (item.isDoorGroups) {
                        item.iconCls = 'doorGroupIcon';
                    } else {
                        item.iconCls = 'doorIcon';
                    }
                }

                if (rebuildAllDoorList == true) {
                    allDoorListWithoutStatus.sort(Util.dynamicSort("name"));
                }

                groupList[0].state = 'open';
                expandedState[groupList[0].id] = 'open';

                return [groupList[0]]; // 최상위 그룹 하나만 사용하며, JQuery Easy UI Tree 생성을 위해 배열형으로 리턴
            }

            function getDoors(param, successCallbackFunction) {
                window.sessionStorage.removeItem('getCachedDoors');
                Util.debug('%cgetDoors', 'background:gray;color:yellow');
                API.get('doors', 'getDoors', param).then(function success(data) {
                    if (successCallbackFunction)
                        successCallbackFunction(data);
                });
            }

            function putDoors(param) {
                var defer = $q.defer(), query;
                if (param.doorIDs.constructor == Array) {
                    API.put('doors?id=' + param.doorIDs.join('+'), 'putDoors', {"Door": param.door}).then(function success(data) {
                        reloadDoorGroupRecursiveData();
                        defer.resolve(data);
                    }, function error(data) {
                        defer.reject(data);
                        reloadDoorGroupRecursiveData();
                    });
                } else if (param.doorIDs.constructor == String) {
                    if (param.query) {
                        if (param.query.group_id) {
                            query += 'group_id=' + param.query.group_id + '&';
                        }
                        if (param.query.query) {
                            query += 'query=' + param.query.query + '&';
                        }
                    }

                    API.put('doors?' + query + 'id=*', 'putDoors', {"Door": param.door}).then(function success(data) {
                        reloadDoorGroupRecursiveData();
                        defer.resolve(data);
                    }, function error(data) {
                        defer.reject(data);
                        reloadDoorGroupRecursiveData();
                    });
                }
                return defer.promise;
            }

            function getOneDoor(doorId) {

                var defer = $q.defer();

                API.get('doors' + '/' + doorId).then(function (data) {
                    window.sessionStorage.setItem('doorInfo', angular.toJson(data.Door));
                    defer.resolve(data.Door);
                });

                return defer.promise;
            }

            function deleteDoors(doorIds, callbackFunction) {
                if (doorIds.constructor == Array) {
                    API.delete('doors?id=' + (doorIds.join ? doorIds.join('+') : doorIds)).then(function success(data) {
                        reloadDoorGroupRecursiveData();
                        callbackFunction();
                    }, function error(data) {
                        reloadDoorGroupRecursiveData();
                        callbackFunction();
                    });

                } else if (doorIds.constructor == String) {
                    API.delete('doors?id=*').then(function success(data) {
                        reloadDoorGroupRecursiveData();
                        callbackFunction();
                    }, function error(data) {
                        callbackFunction();
                        reloadDoorGroupRecursiveData();
                    });
                }
            }

            function postOneDoor(door) {

                var defer = $q.defer();

                API.post('doors', 'postOneDoor', {"Door": door}).then(function success(data) {
                    reloadDoorGroupRecursiveData();
                    defer.resolve(data);
                }, function error(data) {
                    defer.reject(data);
                    reloadDoorGroupRecursiveData();
                });

                return defer.promise;
            }

            function putOneDoor(param) {

                var defer = $q.defer();

                API.put('doors/' + param.id, 'putOneDoor', {"Door": param.door}).then(function success(data) {
                    defer.resolve(data);
                    reloadDoorGroupRecursiveData();
                }, function error(data) {
                    defer.reject(data);
                    reloadDoorGroupRecursiveData();
                });

                return defer.promise;

            }

            function postOneDoorGroup(param) {
                var defer = $q.defer();

                API.post('door_groups', 'postOneDoorGroup', {"DoorGroup": param}).then(function success(data) {
//                    reloadDoorGroupRecursiveData();
                    defer.resolve(data);
                }, function error(data) {
//                    reloadDoorGroupRecursiveData();
                    defer.reject(data);
                });

                return defer.promise;
            }

            function putOneDoorGroup(param) {
                var defer = $q.defer(), group_id = param.id;

                API.put('door_groups/' + group_id, 'putOneDoorGroup', {"DoorGroup": param}).then(function success(data) {
                    reloadDoorGroupRecursiveData();
                    defer.resolve(data);
                }, function error(data) {
                    reloadDoorGroupRecursiveData();
                    defer.reject(data);
                });

                return defer.promise;
            }

            function getDoorGroups(param, deferFunctionName) {
                var defer = $q.defer();
                API.get('door_groups', deferFunctionName, param).then(function success(data) {
                    if (data != undefined && data.DoorGroupCollection != undefined && data.DoorGroupCollection.rows != undefined
                        && data.DoorGroupCollection.rows.length > 0) {
                        if (data.DoorGroupCollection.rows[0].id == Const.DEFAULT_ALL_GROUP_ID) {
                            data.DoorGroupCollection.rows[0].name = MessageProperties.get("menu.door.allDoorGroups");
                        }
                    }
                    defer.resolve(data);
                }, function error(data) {
                    defer.reject(data);
                });
                return defer.promise;
            }

            function deleteDoorGroups(groupIds) {
                var defer = $q.defer();
                API.delete('door_groups?id=' + (groupIds.join ? groupIds.join('+') : groupIds), 'deleteDoorGroups').then(function success(data) {
                    reloadDoorGroupRecursiveData();
                    defer.resolve(data);
                }, function error(data) {
                    reloadDoorGroupRecursiveData();
                    defer.reject(data);
                });

                return defer.promise;
            }

            function reloadDoorGroupRecursiveData() {
                Util.debug('%cCall reloadDoorGroupRecursiveData()', 'background:red');
                $rootScope.$broadcast('ReloadDoorGroupData', {reload: true});
            }

            function setTreeDatatoLocalVariable(data) {
                Util.debug("%cdoorAgent setTreeDatatoLocalVariable", 'background:blue;color:white;');

                DOOR_GROUP_DATA_WITH_DOORS = setTreeData(angular.copy(data.DoorGroupCollection.rows), true, true);
                window.sessionStorage.setItem('getDoorGroupRecursiveDataWithDoors', angular.toJson(DOOR_GROUP_DATA_WITH_DOORS));
                DOOR_GROUP_DATA = setTreeData(angular.copy(data.DoorGroupCollection.rows), false, false);
                window.sessionStorage.setItem('getDoorGroupRecursiveData', angular.toJson(DOOR_GROUP_DATA));
            }

            var doorAgent = {
                getDoorGroupRecursiveData: function (reload) {
                    var defer = $q.defer();

                    var doorData = Util.getSessionObject('getDoorGroupRecursiveData');

                    if (typeof doorData != 'object') {
                        reload = true;
                    }

                    if (doorData && reload != true) {
                        Util.debug("%cDoorAgent getDoorGroupRecursiveData >> direct return", 'color:blue');
                        defer.resolve(angular.fromJson(doorData));
                    } else {
                        getDoorGroups().then(function success(data) {
                            Util.debug("%cDoorAgent getDoorGroupRecursiveData", 'background:yellow; color:blue');
                            setTreeDatatoLocalVariable(angular.copy(data));
//                        window.sessionStorage.setItem('getDoorGroupRecursiveData',angular.toJson(DOOR_GROUP_DATA));
                            defer.resolve(DOOR_GROUP_DATA);
                        }, function error(data) {
                            if (DOOR_GROUP_DATA) {
                                defer.resolve(DOOR_GROUP_DATA);
                            } else {
                                defer.reject(data);
                            }
                        });
                    }
                    return defer.promise;
                },
                getDoorGroupRecursiveDataWithDoors: function (reload) {
                    var defer = $q.defer();
                    var doorData = Util.getSessionObject('getDoorGroupRecursiveDataWithDoors');

                    if (typeof doorData != 'object') {
                        reload = true;
                    }

                    if (doorData && reload != true) {
                        Util.debug("%cDoorAgent getDoorGroupRecursiveDataWithDoors >> direct return", 'color:pink');
						var doorDataJson = angular.fromJson(doorData);
						if (doorDataJson != undefined && doorDataJson.length > 0 && doorDataJson[0].id == Const.DEFAULT_ALL_GROUP_ID) {
							doorDataJson[0].name = MessageProperties.get("menu.door.allDoorGroups");
							doorDataJson[0].text = MessageProperties.get("menu.door.allDoorGroups");
						}

                        defer.resolve(doorDataJson);
                        Util.debug(doorData);
                    } else {
                        getDoorGroups().then(function (data) {
                            Util.debug("%cDoorAgent getDoorGroupRecursiveDataWithDoors", 'background:yellow; color:pink');
                            setTreeDatatoLocalVariable(angular.copy(data));
//                        window.sessionStorage.setItem('getDoorGroupRecursiveDataWithDoors', angular.toJson(DOOR_GROUP_DATA_WITH_DOORS));
                            defer.resolve(DOOR_GROUP_DATA_WITH_DOORS);
//                        Util.debug(DOOR_GROUP_DATA_WITH_DOORS);
                        }, function error(data) {
                            if (DOOR_GROUP_DATA_WITH_DOORS) {
                                defer.resolve(DOOR_GROUP_DATA_WITH_DOORS);
                            } else {
                                defer.reject(data);
                            }
                        });
                    }
                    return defer.promise;
                },
                getAllDoorsWithoutStatus: function () {
                    if (allDoorListWithoutStatus != undefined && allDoorListWithoutStatus.length > 0) {
                        return allDoorListWithoutStatus;
                    } else {
                        var defer = $q.defer();
                        getDoorGroups().then(function (data) {
                            Util.debug("doorAgent getAllDoorsWithoutStatus");
                            setTreeDatatoLocalVariable(data);
                            defer.resolve(allDoorListWithoutStatus);
                        });
                        return defer.promise;
                    }
                },
                getAllDoorMapWithStatus: function () {
                    if (allDoorMapWithStatus != undefined && !Util.isEmpty(allDoorMapWithStatus)) {
                        return allDoorMapWithStatus;
                    } else {
                        var defer = $q.defer();
                        getDoorGroups().then(function (data) {
                            Util.debug("doorAgent getAllDoorMapWithStatus");
                            setTreeDatatoLocalVariable(data);
                            defer.resolve(allDoorMapWithStatus);
                        });
                        return defer.promise;
                    }
                },
                buildStatusObjectByEvent: function (eventObject, doorStatus) {
                    // when door event is arrived door status must be updated by client using event
                    if (eventObject.event_type_id.code == Const.EVENT_TYPE_CODE.DOOR_CLOSED) {
                        doorStatus.opened = false;
                    } else if (eventObject.event_type_id.code == Const.EVENT_TYPE_CODE.DOOR_OPENED) {
                        doorStatus.opened = true;
                        // TODO update last open time
                        //doorStatus.last_open_time = eventObject.datetime;
                    } else if (eventObject.event_type_id.code == Const.EVENT_TYPE_CODE.DOOR_LOCKED) {
                        doorStatus.unlocked = false;
                    } else if (eventObject.event_type_id.code == Const.EVENT_TYPE_CODE.DOOR_UNLOCKED) {
                        doorStatus.unlocked = true;
                    } else if (eventObject.event_type_id.code == Const.EVENT_TYPE_CODE.DOOR_FORCE_OPENED) {
                        doorStatus.alarm_status = Const.DOOR_ALARM_STATUS.ForcedOpened;
                        doorStatus.alarm_status_time = eventObject.datetime;
                    } else if (eventObject.event_type_id.code == Const.EVENT_TYPE_CODE.DOOR_HELD_OPENED) {
                        doorStatus.alarm_status = Const.DOOR_ALARM_STATUS.HeldOpened;
                        doorStatus.alarm_status_time = eventObject.datetime;
                    }
                    return doorStatus;
                },
                updateStatus: function (eventObject) {
                    var doorObject = eventObject.door_id;
                    if (doorObject != undefined) {
                        var existingDoor = allDoorMapWithStatus[doorObject.id];
                        if (existingDoor != undefined) {
                            if (allDoorMapWithStatus[doorObject.id].door_status == undefined) {
                                allDoorMapWithStatus[doorObject.id].door_status = {};
                            }
                            doorAgent.buildStatusObjectByEvent(eventObject, allDoorMapWithStatus[doorObject.id].door_status);
//                        Util.debug(allDoorMapWithStatus[doorObject.id]);
                        }
                    }
                },
                updateAlarm: function (alertObject) {
                    var eventObject = alertObject.event_id;
                    if (eventObject != undefined) {
                        var doorObject = eventObject.door_id;
                        if (doorObject != undefined) {
                            var existingDoor = allDoorMapWithStatus[doorObject.id];
                            if (existingDoor != undefined) {
                                allDoorMapWithStatus[doorObject.id].alert = alertObject;
                            }
                        }
                    }
                },
                updateLastEvent: function (eventObject) {
                    var doorObject = eventObject.door_id;
                    if (doorObject != undefined) {
                        var existingDoor = allDoorMapWithStatus[doorObject.id];
                        if (existingDoor != undefined) {
                            allDoorMapWithStatus[doorObject.id].last_event = eventObject;
                        }
                    }
                },
                getCachedDoors: function () {
                    var defer = $q.defer();


                    var DOOR_DATA = Util.getSessionObject('getCachedDoors');

                    if (typeof doorData == 'object') {
                        defer.resolve(angular.fromJson(DOOR_DATA));
                    } else {
                        API.get('doors', 'getCachedDoors').then(function success(data) {
                            Util.setSessionObject('getCachedDoors', data.DoorCollection.rows);
                            defer.resolve(data.DoorCollection.rows);
                        })
                    }

                    return defer.promise;
                },
                getDoors: getDoors,
                putDoors: putDoors,
                getOneDoor: getOneDoor,
                postOneDoor: postOneDoor,
                putOneDoor: putOneDoor,
                deleteDoors: deleteDoors,
                getDoorGroups: getDoorGroups,
                postOneDoorGroup: postOneDoorGroup,
                deleteDoorGroups: deleteDoorGroups,
                putOneDoorGroup: putOneDoorGroup

            };


            $rootScope.$on('LoginToSuccess', function () {
                Util.debug('Login to Success, DoorAgentStart!.');
                getDoorGroups().then(function (data) {
//                DOOR_GROUP_DATA = setTreeData(response.data.DoorGroupCollection.rows, false, true);
//                DOOR_GROUP_DATA_WITH_DOORS = setTreeData(response.data.DoorGroupCollection.rows, true, true);
                    setTreeDatatoLocalVariable(data);
                });
            });

            $rootScope.$on('DoorEventReceived', function (event, messageObj) {
                if (doorAgent.getAllDoorsWithoutStatus().then) {
                    doorAgent.getAllDoorsWithoutStatus().then(function success(data) {
                        doorAgent.updateStatus(messageObj);
                        doorAgent.updateLastEvent(messageObj);
                    });
                } else {
                    doorAgent.updateStatus(messageObj);
                    doorAgent.updateLastEvent(messageObj);
                }
            });

            $rootScope.$on('DoorAlertReceived', function (event, messageObj) {
                if (doorAgent.getAllDoorsWithoutStatus().then) {
                    doorAgent.getAllDoorsWithoutStatus().then(function success(data) {
                        doorAgent.updateAlarm(messageObj);
                    });
                } else {
                    doorAgent.updateAlarm(messageObj);
                }
            });

            return doorAgent;
        }]);
});