'use strict';

define([
    'modules/common/commonModule'
], function () {

    var app = angular.module('commonModule');

    app.service('FileService', [
        '$location', '$filter', '$q', 'Const', 'API', 'Util', 'MessageProperties', 'MessageService',
        function ($location, $filter, $q, Const, API, Util, MessageProperties, MessageService) {

            var fileService = {
                imageFileReader: function ($event, canvasElementId) {
                    var defer = $q.defer();
                    var event = $event || window.event,
                        target = event.target || (event.srcElement || document),
                        files = target.files,
                        type = "default",
                        reader = new FileReader(),
                        url = URL.createObjectURL(files[0]),
                        image = new Image(),
                        canvas = document.getElementById(canvasElementId),
                        ctx = canvas.getContext("2d");

                    if (/image/.test(files[0].type)) {
                        image.src = url;
                        reader.readAsDataURL(files[0]);
                    } else {
                        MessageService.displayWarningMessage(MessageProperties.get('validate.onlyImageFilesAreSupported'));
                        defer.reject();
                    }
                    image.onload = function () {
                        ctx.drawImage(image, 0, 0, canvas.width, canvas.height);

                        var imageData = canvas.toDataURL("image/png").replace(/(^data:[a-z/;]+base64,)/g, '');
                        defer.resolve(imageData);
                    };
                    return defer.promise;
                },
                soundFileReader: function ($event) {
                    var defer = $q.defer();
                    var event = $event || window.event,
                        target = event.target || (event.srcElement || document),
                        files = target.files,
                        type = "default",
                        reader = new FileReader(),
                        url = URL.createObjectURL(files[0]);

                    Util.debug(files[0].type, files[0].size);
                    if (/audio/.test(files[0].type)) {
                        if (files[0].size < 500000) {
                            reader.readAsDataURL(files[0]);
                        } else {
                            MessageService.displayWarningMessage(MessageProperties.get('validate.fileSizeExceedsLimit', '500KB'));
                            defer.reject();
                        }
                    } else {
                        MessageService.displayWarningMessage(MessageProperties.get('validate.onlyAudioFilesAreSupported'));
                        defer.reject();
                    }

                    reader.onload = function () {
                        var binaryData = reader.result.replace(/(^data:[a-z/;]+base64,)/g, '');
                        defer.resolve(binaryData);
                    };
                    return defer.promise;
                },
                csvFileReader: function ($event) {
                    var defer = $q.defer();
                    var event = $event || window.event,
                        target = event.target || (event.srcElement || document),
                        files = target.files,
                        type = "default",
                        reader = new FileReader(),
                        url = URL.createObjectURL(files[0]);

                    Util.debug(files[0].type, files[0].size);

                    reader.readAsText(files[0], Const.UTF8_ENCODING);

                    reader.onload = function () {
                        defer.resolve('\ufeff' + reader.result);
                    };
                    return defer.promise;
                },
                blobFileReader: function ($event) {
                    var defer = $q.defer();
                    var event = $event || window.event,
                        target = event.target || (event.srcElement || document),
                        files = target.files,
                        type = "default",
                        reader = new FileReader(),
                        url = URL.createObjectURL(files[0]);

                    Util.debug(files[0].type, files[0].size);

                    reader.readAsArrayBuffer(files[0]);

                    reader.onload = function () {
                        defer.resolve(new Uint8Array(reader.result));
                    };
                    return defer.promise;
                }
            };
            return fileService;
        }]);
});
