'use strict';

define([
    'modules/common/commonModule'
], function() {

    var app = angular.module('commonModule');
    app.service('LocalDB', ['$q', function ($q) {

        var DBAgent = {
            db: null,
            storeNames: ['device', 'deviceGroup', 'searchHistory'],
            open: function (option) {
                try {
                    // INFO: Indexed Database의 version은 2.3 등의 소수점은 사용할 수 없다. 반드시 정수로 증가되도록 할 것.
                    var version = 7;
                    var dbName = 'biostar';
                    var iDB = indexedDB || webkitIndexedDB || mozIndexedDB || msIndexedDB;
                    self.IDBTransaction = IDBTransaction || webkitIDBTransaction || mozIDBTransaction || msIDBTransaction || {};
                    var request = iDB.open(dbName, version);
                    console.log(dbName + ' version:' + version);
                    request.onerror = option.error;
                    request.onsuccess = function (evt) {
                        console.log('onsuccess/evt.target.result');
                        console.log(JSON.stringify(evt.target.result));
                        console.log(JSON.stringify(request.result));
                        DBAgent.db = evt.target.result;

                        if (option.success) {
                            option.success(evt);
                        }
                    };

                    request.onupgradeneeded = function (evt) {
                        /*
                         * TODO: version 이 변경되면 기존의 table들읗 삭제하고 새로 만든다.
                         */
                        console.log('onupgradeneeded/evt.target.result');
                        console.log(JSON.stringify(evt.target.result));
                        console.log(JSON.stringify(request.result));
                        DBAgent.db = evt.target.result;

                        evt.target.transaction.onerror = option.error;

                        //------------------ Store 생성 :S----------------//
                        for (var i = 0; i < DBAgent.storeNames.length; i++) {
                            if (DBAgent.db.objectStoreNames.contains(DBAgent.storeNames[i])) {
                                DBAgent.db.deleteObjectStore(DBAgent.storeNames[i]);
                            }
                        }

                        DBAgent.db.createObjectStore('device');
                        DBAgent.db.createObjectStore('deviceGroup');
                        DBAgent.db.createObjectStore('searchHistory');
                        //------------------ Store 생성 :E----------------//


                        if (option.upgradeneeded) {
                            option.upgradeneeded(evt);
                        }
                    };
                }
                catch (ex) {
                    option.error(ex);
                }
            },
            addDevice: function (option) {
                try {
                    var tx = DBAgent.db.transaction(['device'], 'readwrite');
                    console.log('SYNC: 1');
                    tx.oncomplete = function (event) {
                        console.log('SYNC: 7');
                        console.log(JSON.stringify(event));
                        option.complete(event);
                    };
                    tx.onerror = function (event) {
                        console.log('SYNC: 8');
                        console.log(JSON.stringify(event));
                        option.error(event);
                    };
                    console.log(JSON.stringify(tx));
                    console.log('SYNC: 2');
                    var deviceStore = tx.objectStore('device');
                    console.log('SYNC: 3');
                    var device = option.devices;
                    console.log('SYNC: 4');
                    console.log(JSON.stringify(device));
                    var request = deviceStore.add(device);
                    console.log('SYNC: 5');
                    request.onsuccess = function (evt) {
                        console.log('SYNC: 6');
                        console.log(JSON.stringify(evt));
//                    if (device === option.devices[option.devices.length - 1]) {
//                        console.log('SYNC: 7');
//                    }
                    };
                    request.onerror = function (evt) {
                        console.log('SYNC: 6.1');
                        console.log(JSON.stringify(evt));
//                    if (device === option.devices[option.devices.length - 1]) {
//                        console.log('SYNC: 7.1');
//                        option.complete(rtnData);
//                    }
                    };
                }
                catch (ex) {
                    console.log(JSON.stringify(ex));
                    option.error(ex);
                }
            },
            syncAllDevice: function (option) {
                try {
                    var rtnData = {
                        successDeviceIDs: [],
                        errorDeviceIDs: []
                    };
                    for (var i in option.devices) {
                        var tx = DBAgent.db.transaction(['device'], 'readwrite');
                        console.log('SYNC: 1');
                        tx.oncomplete = function (event) {
                            console.log('SYNC: 7');
                            console.log(JSON.stringify(event));
                        };
                        tx.onerror = function (event) {
                            console.log('SYNC: 8');
                            console.log(JSON.stringify(event));
                        };
                        console.log(JSON.stringify(tx));
                        console.log('SYNC: 2');
                        var deviceStore = tx.objectStore('device');
                        console.log('SYNC: 3');

                        console.log('SYNC: 4');
                        console.log(JSON.stringify(option.devices[i]));
                        var request = deviceStore.add(option.devices[i]);
                        console.log('SYNC: 5');
                        request.onsuccess = function (evt) {
                            console.log('SYNC: 6');
                            console.log(JSON.stringify(evt));
                            rtnData.successDeviceIDs.put(option.devices[i].id);
                            if (option.devices[i] === option.devices[option.devices.length - 1]) {
                                console.log('SYNC: 7');
                            }
                        };
                        request.onerror = function (evt) {
                            console.log('SYNC: 6.1');
                            console.log(JSON.stringify(evt));
                            rtnData.errorDeviceIDs.put(option.devices[i].id);
                            if (option.devices[i] === option.devices[option.devices.length - 1]) {
                                console.log('SYNC: 7.1');
                                option.complete(rtnData);
                            }
                        };
                        console.log(JSON.stringify(request));
                    }
                }
                catch (ex) {
                    console.log(JSON.stringify(ex));
                    option.error(ex);
                }
            }
        };

        var idbWorker = new Worker('/modules/common/services/workers/worker-indexed-db.js');
        // TODO: defer 는 배열로 관리되어야 한다. 예약어와 index를 가지고 수신된 메시지에 맞는 defer를 찾아 resolve 한 후 반드시 배열에서 제거되어야 한다.
        var defer, localDB;
        idbWorker.addEventListener('message', function (evt) {
            console.log('Worker :');
            console.log(evt.data);
            defer.resolve(evt.data);
        }, false);

        localDB = {
            doWorkTest: function (myData) {
                defer = $q.defer();
                //idbWorker.postMessage(myData);
                DBAgent.open({
                    success: function (evt) {
                        defer.resolve('successfully opened db');
                    },
                    upgradeneeded: function (evt) {
                        defer.resolve('successfully upgraded db');
                    },
                    error: function (evt) {
                        defer.resolve('db open error');
                    }
                });
                return defer.promise;
            },
            addDevice: function (data) {
//                defer = $q.defer();
//                idbWorker.postMessage({command: 'addDevice', devices: data});
//                return defer.promise;
                DBAgent.addDevice({
                    devices: {'Device': data},
                    success: function (result) {
                        console.log(result);
                        console.log('syncAllDevice: success');
                    },
                    error: function (result) {
                        console.log(result);
                        console.log('syncAllDevice: error');
                    },
                    complete: function (result) {
                        console.log(result);
                        console.log('syncAllDevice: complete');
                    }
                });
            },
            syncAllDevice: function (data) {
                defer = $q.defer();
                idbWorker.postMessage({command: 'syncAllDevice', devices: data});
                return defer.promise;
//                DBAgent.syncAllDevice({
//                    devices: data,
//                    success: function (result) {
//                        console.log(result);
//                        console.log('syncAllDevice: success');
//                    },
//                    error: function (result) {
//                        console.log(result);
//                        console.log('syncAllDevice: error');
//                    },
//                    complete: function (result) {
//                        console.log(result);
//                        console.log('syncAllDevice: complete');
//                    }
//                });
            }
        };
        return localDB;
    }]);
});