'use strict';

define(function () {

    var app = angular.module('biostarWebApp');
    app.register.service('MakeGroupTree', [
        '$q', '$http', '$log', 'API', 'Const',
        function MakeGroupTree($q, $http, $log, API, Const) {
        // AngularJS will instantiate a singleton by calling "new" on this function

        var rows = [];
        var tree_data = [];

        var path = [];
        var _treeId = '';

        function fnMakeGroupPath(node) {
            path.push(node.name);
            var parent = $('#' + _treeId).tree('getParent', node.target);
            if (parent !== null) {
                fnMakeGroupPath(parent);
            } else {
                return;
            }
        }

        var getTreePath = function (treeId, node) {
            path = [];
            _treeId = treeId;

            fnMakeGroupPath(node);

            var group_path = path.slice(0, path.length - 1).reverse().join('>');
            if (group_path.length <= 0) {
                group_path = path[0];
            }
            return group_path;
        }

        var getUserGroupTree = function () {
            var deferred = $q.defer();
            API.get('user_groups').then(function (data) {
                var group = data.UserGroupCollection.rows;
                var group_idx = {};
                for (var i = group.length; i--;) {
                    group_idx[group[i].id] = i;
                }

                for (var i = group.length; i--;) {
                    if (angular.isDefined(group[i].users)) {
                        if (angular.isDefined(group[i].parent_id)) {
                            if (angular.isDefined(group[group_idx[group[i].parent_id.id]].users)) {
                                group[group_idx[group[i].parent_id.id]].users =
                                    group[group_idx[group[i].parent_id.id]].users.concat(group[i].users);
                            } else {
                                group[group_idx[group[i].parent_id.id]].users = [];
                                angular.copy(group[i].users, group[group_idx[group[i].parent_id.id]].users);
                            }

                            var user_groups = group[group_idx[group[i].parent_id.id]].user_groups;
                            for (var j = user_groups.length; j--;) {
                                if (group[i].id === user_groups[j].id) {
                                    user_groups[j] = group[i];
                                }
                            }
                        } else {

                        }
                    }
                }
                deferred.resolve({'UserGroup': group[0]});
            });
            return deferred.promise;
        };

        var getDeviceGroupTree = function () {
            return API.get('device_groups/1?recursive=true');
        };

        var getDoorGroupTree = function () {
            return API.get('door_groups/1?recursive=true');
        };

        var getAccessGroupTree = function () {
            var deferred = $q.defer();
//      API.get(Const.API.BASE_URL + 'devices/1?recursive=true/1?recursive=true')
            //$http.get('http://127.0.0.1:3412/api/devices/1?recursive=true')
            $http.get("/scripts/controllers/sample/access-group.json")
                .success(function (data) {
                    deferred.resolve(data);
                })
                .error(function (reason) {
                    deferred.reject(reason);
                });

            return deferred.promise;
        };

        var getLevelGroupTree = function () {
            var deferred = $q.defer();
//      API.get(Const.API.BASE_URL + 'devices/1?recursive=true/1?recursive=true')
            //$http.get('http://127.0.0.1:3412/api/devices/1?recursive=true')
            $http.get("/scripts/controllers/sample/level-group.json")
                .success(function (data) {
                    deferred.resolve(data);
                })
                .error(function (reason) {
                    deferred.reject(reason);
                });

            return deferred.promise;
        };

        var getZoneGroupTree = function () {
            var deferred = $q.defer();
//      API.get(Const.API.BASE_URL + 'devices/1?recursive=true/1?recursive=true')
            //$http.get('http://127.0.0.1:3412/api/devices/1?recursive=true')
            $http.get("/scripts/controllers/sample/zone-group.json")
                .success(function (data) {
                    deferred.resolve(data);
                })
                .error(function (reason) {
                    deferred.reject(reason);
                });

            return deferred.promise;
        };

        function fnFillTreeData(tree_data) {
            var item = tree_data;
            if (angular.isDefined(item.user_groups) && item.user_groups.length > 0) {
                for (var i = 0; i < item.user_groups.length; i++) {
                    var retData = fnFindTreeData(item.user_groups[i], rows);
                    item.user_groups[i] = retData;
                    fnFillTreeData(retData);
                }
            }
            return;
        }

        function fnFindTreeData(item, rows) {
            for (var i = 0; i < rows.length; i++) {
                if (item.id == rows[i].id) {
                    return rows[i];
                }
            }
            return undefined;
        }

        var group_only = false;

        var fnSetEasyTreeValue = function (data) {
            for (var i = 0, len = data.length; i < len; i++) {
                data[i].text = data[i].name;
                data[i].id = data[i].id;
                if (data[i]['user_groups'] !== undefined && typeof data[i]['user_groups'] === 'object') {
                    data[i].children = data[i]['user_groups'];
                    fnSetEasyTreeValue(data[i]['user_groups']);
                } else if (data[i]['device_groups'] !== undefined && typeof data[i]['device_groups'] === 'object') {
                    data[i].children = data[i]['device_groups'];

                    fnSetEasyTreeValue(data[i]['device_groups']);
                } else if (data[i]['door_groups'] !== undefined && typeof data[i]['door_groups'] === 'object') {
                    data[i].children = data[i]['door_groups'];

                    fnSetEasyTreeValue(data[i]['door_groups']);
                } else if (data[i]['access_groups'] !== undefined && typeof data[i]['access_groups'] === 'object') {
                    data[i].children = data[i]['access_groups'];

                    fnSetEasyTreeValue(data[i]['access_groups']);
                } else if (data[i]['level_groups'] !== undefined && typeof data[i]['level_groups'] === 'object') {
                    data[i].children = data[i]['level_groups'];

                    fnSetEasyTreeValue(data[i]['level_groups']);
                } else if (data[i]['zone_groups'] !== undefined && typeof data[i]['zone_groups'] === 'object') {
                    data[i].children = data[i]['zone_groups'];

                    fnSetEasyTreeValue(data[i]['zone_groups']);
                }


                if (angular.isDefined(data[i].devices) && data[i].devices.length > 0) {
                    if (angular.isUndefined(data[i].children)) {
                        data[i].children = [];
                    }
                    for (var j = 0; j < data[i].devices.length; j++) {
                        var item = {};
                        item.name = data[i].devices[j].name;
                        item.isDevices = true;
                        item.id = data[i].devices[j].id;
                        data[i].children.push(item);
                    }
                }

                if (group_only != true) {
                    if (angular.isDefined(data[i].doors) && data[i].doors.length > 0) {
                        if (angular.isUndefined(data[i].children)) {
                            data[i].children = [];
                        }
                        for (var j = 0; j < data[i].doors.length; j++) {
                            var item = {};
                            item.name = data[i].doors[j].name;
                            item.isDoors = true;
                            item.id = data[i].doors[j].id;
                            data[i].children.push(item);
                        }
                    }
                }
            }
        };

        return {

            /**
             * 그룹 데이터를 EasyUI 트리 구조에 맞게 가공한다.
             * @param data
             * @param _group_only
             * @returns {Array}
             */
            getTreeData: function (data, _group_only) {
                tree_data = [];
                tree_data.push(data);
//        console.log('do fnSetEasyTreeValue', data);
                fnSetEasyTreeValue(tree_data);
                group_only = _group_only;
                return tree_data;
            },
            getUserGroupTree: getUserGroupTree,
            getDeviceGroupTree: getDeviceGroupTree,
            getDoorGroupTree: getDoorGroupTree,
            getAccessGroupTree: getAccessGroupTree,
            getLevelGroupTree: getLevelGroupTree,
            getZoneGroupTree: getZoneGroupTree,
            getTreePath: getTreePath
        };

    }]);
});