'use strict';

define([
    'modules/common/commonModule'
], function () {

    var app = angular.module('commonModule');
    app.service('PreferenceService', [
        '$q', '$rootScope', '$filter', 'API', 'Const', 'Util', 'Auth', 'MessageProperties',
        function ($q, $rootScope, $filter, API, Const, Util, Auth, MessageProperties) {


            Date.prototype.convertDateUsingAbsoluteTimeZoneOffset = function (timeZoneOffsetInMinutes) {
                if (timeZoneOffsetInMinutes == undefined) {
                    timeZoneOffsetInMinutes = prefService.getTimeZoneOffset();
                }
                var offsetTime = new Date(this.getTime() + timeZoneOffsetInMinutes * 60 * 1000);
//            console.log("convertDateUsingAbsoluteTimeZoneOffset::" + this.toISOString() + "::" + timeZoneOffsetInMinutes + "::" + offsetTime.toISOString());
                return offsetTime;
            };

            // 변환하려는 Date Value가 GMT라면 timeZoneOffsetInMinutes은 undefined로 설정하여야 합니다.
            Date.prototype.convertLocaleDateToISOString = function (timeZoneOffsetInMinutes) {
                if (timeZoneOffsetInMinutes == undefined) {
                    timeZoneOffsetInMinutes = prefService.getTimeZoneOffset();
                }
                var offsetTime = new Date(this.getTime() - this.getTimezoneOffset() * 60 * 1000 - timeZoneOffsetInMinutes * 60 * 1000);
//            console.log("convertDateUsingAbsoluteTimeZoneOffset::" + this.toISOString() + "::" + timeZoneOffsetInMinutes + "::" + offsetTime.toISOString());
                return offsetTime.toISOString();
            };

            Date.prototype.convertDateUsingRelativeTimeZoneOffset = function (timeZoneOffsetInMinutes) {
                if (timeZoneOffsetInMinutes == undefined) {
                    timeZoneOffsetInMinutes = prefService.getTimeZoneOffset();
                }
                var tzDifference = timeZoneOffsetInMinutes + this.getTimezoneOffset();
//            console.log( tzDifference +"="+ timeZoneOffsetInMinutes +"+"+ this.getTimezoneOffset());
                //convert the offset to milliseconds, add to targetTime, and make a new Date
                var offsetTime = new Date(this.getTime() + tzDifference * 60 * 1000);
//            console.log("convertDateUsingRelativeTimeZoneOffset::" + this.toISOString() + "::" + timeZoneOffsetInMinutes + "::" + offsetTime.toISOString());
                return offsetTime;
            };

            Date.prototype.convertDateBackToCurrentTimeUsingAbsoluteTimeZoneOffset = function (timeZoneOffsetInMinutes) {
                if (timeZoneOffsetInMinutes == undefined) {
                    timeZoneOffsetInMinutes = prefService.getTimeZoneOffset();
                }
                var tzDifference = (this.getTimezoneOffset() * -1) - timeZoneOffsetInMinutes;
//            console.log( tzDifference +"=-"+ timeZoneOffsetInMinutes +"-"+ this.getTimezoneOffset());
                //convert the offset to milliseconds, add to targetTime, and make a new Date
                var offsetTime = new Date(this.getTime() + tzDifference * 60 * 1000);
//            console.log("convertDateBackToCurrentTimeUsingAbsoluteTimeZoneOffset::" + this.toISOString() + "::" + timeZoneOffsetInMinutes + "::" + offsetTime.toISOString());
                return offsetTime;
            };

            var timeZoneList = {};

            var prefService = {
                setPreference: function (paramPreferenceObject) {
                    if (paramPreferenceObject != undefined) {
                        if (paramPreferenceObject.depend_on_os == Const.BOOLEAN.TRUE) {

                        } else if (paramPreferenceObject.date_format != undefined && paramPreferenceObject.time_format != undefined) {
                            paramPreferenceObject.datetime_format = paramPreferenceObject.date_format + ' ' + paramPreferenceObject.time_format;
                            paramPreferenceObject.datetime_long_term_format = paramPreferenceObject.date_format + ' ' + paramPreferenceObject.time_long_term_format;
                        } else {
                            paramPreferenceObject.datetime_format = Const.DEFAULT_DATETIME_FORMAT;
                            paramPreferenceObject.datetime_long_term_format = Const.DEFAULT_DATETIME_LONG_TERM_FORMAT;
                        }
                        if (Util.getLanguageFromLocalStorage() != paramPreferenceObject.language) {
                            Util.resetColumnSetting();
                        }
                        prefService.saveToStorage(paramPreferenceObject);
                    }
                },
                getPreference: function () {
                    var preferenceObject = prefService.getFromStroage();
                    var defer = $q.defer();
                    if (preferenceObject != undefined) {
                        Util.debug("PreferenceService::getPreference::readFromSession");
                        defer.resolve(preferenceObject);
                    } else {
                        Util.debug("PreferenceService::getPreference::readFromServer");
                        API.get('preferences/' + Auth.getUser().user_id).then(function (data) {
                            Util.debug("PreferenceService::getPreference::received response");
                            if (data.Preference != undefined) {
                                if (data.Preference.default == Const.BOOLEAN.TRUE) {
                                    var tempDate = new Date();
                                    var localTimeZoneOffset = -1 * tempDate.getTimezoneOffset();
                                    for (var i = 0; i < timeZoneList.length; i++) {
                                        var currentTimeZoneOffset = parseInt(timeZoneList[i].offset);
                                        if (localTimeZoneOffset == currentTimeZoneOffset) {
                                            data.Preference.timeZoneObject = timeZoneList[i];
                                            Util.debug("found local time zone::%o", timeZoneList[i]);
                                            break;
                                        }
                                    }
                                } else {
                                    data.Preference.timeZoneObject = timeZoneList[data.Preference.time_zone];
                                }

                                var timeFormat = data.Preference.time_format || Const.DEFAULT_TIME_FORMAT, len = timeFormat.indexOf('mm') + 'mm'.length;
                                data.Preference.time_long_term_format = timeFormat.slice(0, len) + ':ss' + timeFormat.slice(len, timeFormat.length);
                                prefService.setPreference(data.Preference);
                                defer.resolve(data.Preference);
                            } else {
                                defer.reject();
                            }
                        }, function error(data) {

                        });
                    }
                    return defer.promise;
                },
                doesDataExistOnServer: function () {
                    var preferenceObject = prefService.getFromStroage();
                    // prefereceObject should exists here, if it is undefined, there is a bug in retrieving logic
                    if (preferenceObject.default == Const.BOOLEAN.TRUE) {
                        return false;
                    }
                    return true;
                },
                isUseSystemDatetimeFormat: function () {
                    var preferenceObject = prefService.getFromStroage();
                    // prefereceObject should exists here, if it is undefined, there is a bug in retrieving logic
                    return preferenceObject.depend_on_os == "true";
                },
//                getDefaultDateFormat: function() {
//                    return Const.DEFAULT_DATE_FORMAT;
//                },
//                getDefaultTimeFormat: function() {
//                    return Const.DEFAULT_TIME_FORMAT;
//                },
//                getDefaultTimeLongTermFormat: function() {
//                    return Const.DEFAULT_TIME_LONG_TERM_FORMAT;
//                },
//                getDefaultDatetimeFormat: function() {
//                    return Const.DEFAULT_DATETIME_FORMAT;
//                },
//                getDefaultDatetimeLongTermFormat: function() {
//                    return Const.DEFAULT_DATETIME_LONG_TERM_FORMAT;
//                },
                getDateFormat: function () {
                    var preferenceObject = prefService.getFromStroage();
                    // prefereceObject should exists here, if it is undefined, there is a bug in retrieving logic
                    return preferenceObject.date_format;
                },
                getTimeFormat: function () {
                    var preferenceObject = prefService.getFromStroage();
                    // prefereceObject should exists here, if it is undefined, there is a bug in retrieving logic
                    return preferenceObject.time_format;
                },
                getTimeLongTermFormat: function () {
                    var preferenceObject = prefService.getFromStroage();
                    // prefereceObject should exists here, if it is undefined, there is a bug in retrieving logic
                    return preferenceObject.time_format_long_term;
                },
                getDatetimeFormat: function () {
                    var preferenceObject = prefService.getFromStroage();
//                console.log(preferenceObject);
                    // prefereceObject should exists here, if it is undefined, there is a bug in retrieving logic
                    return preferenceObject.datetime_format;
                },
                getDatetimeLongTermFormat: function () {
                    var preferenceObject = prefService.getFromStroage();
//                console.log(preferenceObject);
                    // prefereceObject should exists here, if it is undefined, there is a bug in retrieving logic
//                console.info(preferenceObject.datetime_long_term_format);
                    return preferenceObject.datetime_long_term_format || Const.DEFAULT_DATETIME_LONG_TERM_FORMAT;
                },
                getTimeZoneOffset: function () {
                    var preferenceObject = prefService.getFromStroage();
                    // prefereceObject should exists here, if it is undefined, there is a bug in retrieving logic
                    return preferenceObject.timeZoneObject != undefined ? parseInt(preferenceObject.timeZoneObject.offset) : undefined;
                },
                buildStroageKey: function () {
                    return Const.LOCAL_STORAGE_KEY_PREFIX + '_pref';
                },
                saveToStorage: function (preferenceObject) {
                    Util.setSessionObject(prefService.buildStroageKey(), preferenceObject);
                    Util.saveLanguageToLocalStorage(preferenceObject.language);
                },
                getFromStroage: function () {
                    return Util.getSessionObject(prefService.buildStroageKey());
                },
                init: function () {
                    Const.localizationConstantVariables(Util.getLanguageFromLocalStorage());
                    if (API.isLoggedIn() == true) {
                        prefService.getPreference();
                    }
                },
                getTodayUsingPreferenceTimeZone: function () {
                    return prefService.getDateUsingPreferenceTimeZone(new Date());
                },
                getDateUsingPreferenceTimeZone: function (dateObject) {
                    var newDate = dateObject.convertDateUsingRelativeTimeZoneOffset(prefService.getTimeZoneOffset());
                    return newDate;
                }

            };

            timeZoneList = Const.TIMEZONE_OPTIONS;
            prefService.init();

            $rootScope.$on('LoginToSuccess', function (event) {
                console.log('PreferenceService: LoginToSuccess');
                prefService.init();
            });

            return prefService;
        }]);

});