'use strict';

define([
    'modules/common/commonModule'
], function () {
    var app = angular.module('commonModule');
    app.service('ScheduleAgent', ['$log', '$q', 'API', 'Util', 'MessageProperties', 'Const',
        function ($log, $q, API, Util, MessageProperties, Const) {

            function getSchedules(param, deferFunctionName) {
                var defer = $q.defer();
                var options = {
                    url: 'schedules',
                    params: param,
                    deferFunctionName: deferFunctionName,
                    displaySuccessPopoup: false,
                    displayErrorPopup: false,
                    displayLoading: false
                };
                API.getWithOptions(options).then(function (data) {
                    Util.debug('load server data :: getScheduleList');
                    var scheduleMap = {};
                    for (var i = data.ScheduleCollection.rows.length; i--;) {
                        if (data.ScheduleCollection.rows[i].id == Const.SCHEDULE_ALWAYS_ID) {
                            data.ScheduleCollection.rows[i].name = MessageProperties.get("schedule.always");
                        }
                        scheduleMap[data.ScheduleCollection.rows[i].id] = data.ScheduleCollection.rows[i];
                    }
                    Util.setSessionObject('getScheduleList', data.ScheduleCollection.rows);
                    Util.setSessionObject('getScheduleMap', scheduleMap);
                    console.log(data);
                    defer.resolve(data);
                }, function (error) {
                    Util.debug('load server data :: getScheduleList :: fail', error);
                    defer.reject(error);
                });
                return defer.promise;
            }

            var api = {
                getScheduleList: getSchedules,
                getScheduleMap: function (reloadYn) {
                    var defer = $q.defer();

                    var scheduleMap = Util.getSessionObject('getScheduleMap');

                    if (scheduleMap && reloadYn != true) {
                        Util.debug('load session data :: getScheduleMap');
                        defer.resolve(scheduleMap);
                    } else {
                        return getSchedules();
                    }

                    return defer.promise;
                }
            };

            function init() {
                getSchedules();
            }

            init();

            return api;
        }]);
})