'use strict';

define([
    'modules/common/commonModule'
], function () {

    var app = angular.module('commonModule');

    app.service('Util', [
        '$location', '$filter', 'Const', '$q', 'MessageProperties',
        function Util($location, $filter, Const, $q, MessageProperties) {
            // AngularJS will instantiate a singleton by calling "new" on this function
            var beforeUrl = '';
            var util = {};

            var tagFilterList = {
                '<': '&lt;'
            };
            /**
             * 숫자를 문자로 변환할 때 원하는 길이에 맞춰 앞에 '0' 문자를 채운다.
             * Date.prototype.toFormatString 함수 등에서 사용한다.
             * 김재훈
             *  Number.zeroPadding */
            Number.prototype.zeroPadding = function (len) {
                var result = this.toString();
                if (result.length < len) {
                    var addCount = len - result.length;
                    for (var i = 0; i < addCount; i++) {
                        result = "0" + result;
                    }
                } else if (result.length > len) {
                    result = result.substr(len);
                }
                return result;
            };
            var monthName = {
                '1': 'Jan',
                '2': 'Feb',
                '3': 'Mar',
                '4': 'Apr',
                '5': 'May',
                '6': 'Jun',
                '7': 'Jul',
                '8': 'Aug',
                '9': 'Sep',
                '10': 'Oct',
                '11': 'Nov',
                '12': 'Dec',
                '_default': 'unknown'
            };
            var monthFullName = {
                '1': 'January',
                '2': 'February',
                '3': 'March',
                '4': 'April',
                '5': 'May',
                '6': 'June',
                '7': 'July',
                '8': 'August',
                '9': 'September',
                '10': 'October',
                '11': 'November',
                '12': 'December ',
                '_default': 'unknown'
            };
            var dayOfWeek = {
                '0': 'Sun',
                '1': 'Mon',
                '2': 'Tue',
                '3': 'Wed',
                '4': 'Thu',
                '5': 'Fri',
                '6': 'Sat',
                '_default': 'unknown'
            };
            Date.prototype.getMonthName = function (month) {
                var m = this;
                if (month) {
                    return monthName[month] || monthName._default;
                } else {
                    return monthName[(m.getMonth() + 1).toString()] || monthName._default;
                }
            };

            Date.prototype.getMonthFullName = function (month) {
                var m = this;
                if (month) {
                    return monthFullName[month] || monthFullName._default;
                } else {
                    return monthFullName[(m.getMonth() + 1).toString()] || monthFullName._default;
                }
            };
            Date.prototype.getDayOfWeek = function (day) {
                var w = this;
                if (day) {
                    return dayOfWeek[day] || dayOfWeek._default;
                } else {
                    return dayOfWeek[w.getDay()] || dayOfWeek._default;
                }
            };
            /**
             * 날짜 시간 형식의 형식 문자열을 인자로 받아서 원하는 형식으로 날짜 시간을 표시한다.
             * 김재훈
             *  Date.toFormatString
             *  @param format: (optional) 형식 문자열
             *  @param timeZone: (optional) ex.) '9.0', '-8.5', '12.0'...
             *  */
            Date.prototype.toFormatString = function (format, timeZone) {
                var d = this;
                // TODO: format, timeZone를 인자로 받는 것이 아니라 공통영역에 사용자가 선택한 format, timeZone을
                // 가지고 있고 이를 기본값으로 사용해야 한다.
                if (timeZone != null) {
                    d = this.convertTimeZoneDateTime(timeZone);
                }
                var dateFormat = {
                    'YYYY': d.getFullYear().toString(),
                    'yyyy': d.getFullYear().toString(),
                    'YY': d.getFullYear().zeroPadding(2),
                    'yy': d.getFullYear().zeroPadding(2),
                    'mmmm': d.getMonthFullName(),
                    'mmm': d.getMonthName(),
                    'MM': (d.getMonth() + 1).zeroPadding(2),
                    'dddd': d.getDayOfWeek(d.getDay()),
                    'ddd': d.getDayOfWeek(d.getDay()),
                    'day': d.getDayOfWeek(d.getDay()),
                    'DD': d.getDate().zeroPadding(2),
                    'dd': d.getDate().zeroPadding(2),
                    'HH': d.getHours().zeroPadding(2),
                    'H': d.getHours().toString(),
                    'hh': ((d.getHours() % 12) ? (d.getHours() % 12) : 12).zeroPadding(2),
                    'h': ((d.getHours() % 12) ? (d.getHours() % 12) : 12).toString(),
                    'mm': (d.getMinutes()).zeroPadding(2),
                    'MIN': d.getMinutes().zeroPadding(2),
                    'min': d.getMinutes().toString(),
                    'SS': d.getSeconds().zeroPadding(2),
                    'ss': d.getSeconds().zeroPadding(2),
                    'A': d.getHours() < 12 ? 'AM' : 'PM',
                    'a': d.getHours() < 12 ? 'am' : 'pm',
                    'P': d.getHours() < 12 ? 'AM' : 'PM',
                    'p': d.getHours() < 12 ? 'am' : 'pm',
                    '_default': d.toLocaleString()
                };
                var index = format.indexOf("mm:ss");
                if (index > -1) {
                    format = format.substr(0, index) + "MIN:ss" + format.substr(index + 5);
                }
//            console.log(format);
                format = format || Const.DEFAULT_DATETIME_FORMAT;
                return format.replace(/(YYYY|yyyy|YY|yy|mmmm|mmm|MM|dddd|ddd|day|DD|dd|HH|H|hh|h|mm|MIN|min|SS|ss|A|a|P|p)/gi, function ($1) {
                    return dateFormat[$1] || dateFormat._default;
                });
            };

            Date.prototype.setDateAsLocalTimeFromUTCString = function (utcString) {
//            util.logDebug(utcString);
                if (utcString == undefined || utcString.indexOf("NaN") > -1 || utcString.indexOf("not-a-date-time") > -1) {
                    return;
                }
                this.setFullYear(parseInt(utcString.substr(0, 4)));
                this.setMonth(parseInt(utcString.substr(5, 7)) - 1);
                this.setDate(parseInt(utcString.substr(8, 10)));
                this.setHours(parseInt(utcString.substr(11.13)));
                this.setMinutes(parseInt(utcString.substr(14.16)));
                this.setSeconds(parseInt(utcString.substr(17.19)));
                this.setMilliseconds(parseInt(utcString.substr(20.22)));
            };

            Date.prototype.buildCurrentTimeWithNoTimeZoneInUTCString = function () {
                var utcString = '';
                utcString = this.getFullYear() + "-" + util.leftPad((this.getMonth() + 1), 2, '0') + "-" + util.leftPad((this.getDate()), 2, '0') +
                    "T" + util.leftPad(this.getHours(), 2, '0') + ":" + util.leftPad(this.getMinutes(), 2, '0') + ":" + util.leftPad(this.getSeconds(), 2, '0') + ":" + util.leftPad(this.getMilliseconds(), 2, '0') + "Z";
                return utcString;
            };

            Array.prototype.contains = function (obj) {
                var i = this.length;
                while (i--) {
                    if (this[i].id != undefined) {
                        if (this[i].id === obj.id) {
                            return true;
                        }
                    } else {
                        if (this[i] === obj) {
                            return true;
                        }
                    }
                }
                return false;
            };

            /**
             * Object로 구성된 Array일 경우 넘겨받은 objId 값을 가지고 목록에서 해당 Object의 Index를 찾아서 반환한다.
             * @param objId
             * @returns {Number}
             */
            Array.prototype.indexById = function (objId) {
                var i = this.length;
                while (i--) {
                    if (this[i].id != undefined) {
                        if (this[i].id === objId) {
                            return i;
                        }
                    } else {
                        if (this[i] === objId) {
                            return i;
                        }
                    }
                }
                return i;
            };

//        Array.prototype.remove = function (item) {
//            var arr = this;
//            return arr.slice(0, arr.indexOf(item)).concat(arr.slice(arr.indexOf(item) + 1, arr.length));
//        };

            /* Replace */
            String.prototype.replaceAll = function (oldValue, newValue) {

                var retValue = this;

                while (retValue.indexOf(oldValue) >= 0) {
                    retValue = retValue.replace(oldValue, newValue);
                }

                return retValue;
            };

            Array.prototype.keySort = function (keys) {

                keys = keys || {};

                var obLen = function (obj) {
                    var size = 0, key;
                    for (key in obj) {
                        if (obj.hasOwnProperty(key))
                            size++;
                    }
                    return size;
                };

                var obIx = function (obj, ix) {
                    var size = 0, key;
                    for (key in obj) {
                        if (obj.hasOwnProperty(key)) {
                            if (size == ix)
                                return key;
                            size++;
                        }
                    }
                    return false;
                };

                var keySort = function (a, b, d) {
                    d = d !== null ? d : 1;
                    // a = a.toLowerCase(); // this breaks numbers
                    // b = b.toLowerCase();
                    if (a == b)
                        return 0;
                    return a > b ? 1 * d : -1 * d;
                };

                var KL = obLen(keys);

                if (!KL)
                    return this.sort(keySort);

                for (var k in keys) {
                    // asc unless desc or skip
                    keys[k] =
                            keys[k] == 'desc' || keys[k] == -1 ? -1
                        : (keys[k] == 'skip' || keys[k] === 0 ? 0
                        : 1);
                }

                this.sort(function (a, b) {
                    var sorted = 0, ix = 0;

                    while (sorted === 0 && ix < KL) {
                        var k = obIx(keys, ix);
                        if (k) {
                            var dir = keys[k];
                            sorted = keySort(a[k], b[k], dir);
                            ix++;
                        }
                    }
                    return sorted;
                });
                return this;
            };

            function isString(s) {
                return typeof(s) === 'string' || s instanceof String;
            }

            function isEmpty(map) {
                for (var key in map) {
                    if (map.hasOwnProperty(key)) {
                        return false;
                    }
                }
                return true;
            }

            function dispatchFunction(fn, args) {
                fn = (typeof fn == "function") ? fn : window[fn];  // Allow fn to be a function object or the name of a global function
                return fn.apply(this, args || []);  // args is optional, use an empty array by default
            }

            var log = function () {

                var enableDebug = true;
//                    var separator = "<<";

                function bind(f, thisArg, ctx) {
                    if (f.bind !== 'undefined') { // IE < 10
                        return Function.prototype.bind.call(f, thisArg, ctx);
                    }
                    else {
                        return f.bind(thisArg, ctx);
                    }
                }

                function newConsole(context, parentConsole) {
                    var log;
                    var debug;
                    var info;
                    var warn;
                    var error;

                    if (!parentConsole) {
                        parentConsole = console;
                    }
//                        context = context + separator;

                    if (enableDebug) {
//                            debug = bind(console.debug, console, context + "DEBUG" + separator);
                        if (console.debug == undefined) {
                            debug = bind(console.log, console, context);
                        } else {
                            debug = bind(console.debug, console, context);
                        }
                    } else {
                        debug = function () {
                            // suppress all debug messages
                        };
                    }

                    log = bind(console.log, console, context);

                    if (console.info == undefined) {
                        info = bind(console.log, console, context);
                    } else {
                        info = bind(console.info, console, context);
                    }

                    if (console.warn == undefined) {
                        warn = bind(console.log, console, context);
                    } else {
                        warn = bind(console.warn, console, context);
                    }

                    error = bind(console.error, console, context);

                    return {
                        debug: debug,
                        info: info,
                        log: log,
                        warn: warn,
                        error: error,
                        /* access console context information */
                        context: context,
                        /* create a new console with nested context */
                        nest: function (subContext) {
                            return newConsole(context + subContext, this);
                        },
                        parent: parentConsole
                    };
                }

                return newConsole('');
            };

            util = {
                isEmpty: isEmpty,
                rebuildListWithoutStatusAndMapWithStatus: function (dataList, list, map, copyToIdField, parentObject) {
                    if (dataList != undefined && dataList.length > 0) {
                        for (var index = 0; index < dataList.length; index++) {
                            if (copyToIdField != undefined) {
                                dataList[index].id = dataList[index][copyToIdField];
                            }
                            if (map[dataList[index].id] == undefined) {
                                if (parentObject != null) {
                                    list.push({
                                        id: dataList[index].id,
                                        name: dataList[index].name,
                                        user_id: dataList[index].user_id,
                                        parent_device_id: parentObject,
                                        parent_object_id: parentObject
                                    });
                                    dataList[index]['parent_device_id'] = parentObject;
                                    dataList[index]['parent_object_id'] = parentObject;
                                    map[dataList[index].id] = dataList[index];
                                } else {
                                    list.push({
                                        id: dataList[index].id,
                                        user_id: dataList[index].user_id,
                                        name: dataList[index].name
                                    });
                                    map[dataList[index].id] = dataList[index];
                                }
                            } else {
                                angular.extend(map[dataList[index].id], dataList[index])
                            }
                        }
                    }
                },
                formatIdAndNameSpan: function (dataObject) {
                    var idValue = '';
                    if (angular.isDefined(dataObject) && dataObject.id) {
                        idValue = dataObject.id;
                    }

                    if (angular.isDefined(dataObject) && dataObject.name) {
                        return '<span title="' + idValue + '">' + dataObject.name + '</span>';
                    } else {
                        if (idValue != undefined && idValue.length > 0) {
                            return '<span title="' + idValue + '">' + idValue + '</span>';
                        } else {
                            return '';
                        }
                    }
                },
                formatEventSpan: function (dataObject) {
                    var idValue = '';
                    if (angular.isDefined(dataObject) && angular.isDefined(dataObject.event_type_id) && dataObject.event_type_id.code) {
                        idValue = dataObject.event_type_id.code;
                    }

                    var returnValue = '';
                    if (angular.isDefined(dataObject)) {
                        if (dataObject.datetime) {
                            var tempDate = new Date(dataObject.datetime);
                            returnValue += $filter('BssTime')(tempDate, Const.DEFAULT_TIME_FORMAT);
                        }
                        if (angular.isDefined(dataObject.event_type_id) && dataObject.event_type_id.name) {
                            returnValue += ' ' + dataObject.event_type_id.name;
                        }
                    }
                    return '<span id="' + idValue + '">' + returnValue + '</span>';
                    ;
                },
                dynamicSort: function (property) {
                    var sortOrder = 1;
                    if (property[0] === "-") {
                        sortOrder = -1;
                        property = property.substr(1);
                    }
                    return function (a, b) {
                        var result = 0;
                        if (isString(a[property]) && isString(b[property])) {
                            return a[property].localeCompare(b[property]);
                        } else {
                            result = (a[property] < b[property]) ? -1 : (a[property] > b[property]) ? 1 : 0;
                        }
                        return result * sortOrder;
                    }
                },
                replaceTagFilter: function (text) {
                    text = text.replaceAll('<', tagFilterList['<']);
                    return text;
                },
                deepFreeze: function (obj) {
                    var prop, propKey;
                    Object.freeze(obj);
                    for (propKey in obj) {
                        prop = obj[propKey];
                        if (!obj.hasOwnProperty(propKey) || !(typeof prop === 'object') || Object.isFrozen(prop)) {
                            // 객체가 프로토타입이거나, object가 아니거나, 이미 Freeze 되었으면 넘어간다.
                            // Freeze 된 객체의 하위에 일부만 Freeze 되지 않은 경우 외부에서 이 Freeze 되지 않은 일부를 수정할 수 있다.
                            continue;
                        }

                        util.deepFreeze(prop); // deepFreeze은 재귀호출.
                    }
                },
                /**
                 * 수정 모드에서 원본에서 변경된 내용만 추출하기 위해 사용한다. 읽어온 원본 데이터에서 수정된 내용만 추출
                 * @param origin
                 * @param modified
                 * @returns {*}
                 */
                extractModifiedFields: function (origin, modified) {
                    var prop, propKey, extrObj;
                    for (propKey in modified) {
                        prop = modified[propKey];
                        var extrSubObj;
                        if (typeof prop == 'boolean') {
                            modified[propKey] = prop.toString();
                        }
                        if (angular.isArray(prop) && !angular.equals(origin[propKey], modified[propKey])) {
                            /**
                             * 원본에는 존재하지 않았던 Array의 수정된 내용의 길이가 0이면 동일한 것이므로 패스한다.
                             */
                            if (origin[propKey] == null && modified[propKey].length <= 0) {
                                continue;
                            }
                            if (extrObj == undefined) {
                                extrObj = {};
                            }
                            /**
                             * INFO: Array 는 요소 중 하나만이라도 변경되었다면 Array 전체를 수정된 것으로 간주하고 함께 보내야 한다.
                             * 수정된 일부만 보내면 나머지 요소는 삭제로 간주된다.
                             */
                            extrObj[propKey] = modified[propKey];
                        } else if (typeof prop == 'object') {
                            if (origin == undefined) {
                                extrSubObj = util.extractModifiedFields({}, modified[propKey]);
                            } else {
                                extrSubObj = util.extractModifiedFields(origin[propKey], modified[propKey]);
                            }
                            if (extrSubObj != undefined) {
                                if (extrObj == undefined) {
                                    extrObj = {};
                                }
                                extrObj[propKey] = extrSubObj;
                            }
                        } else if (angular.isDefined(origin) && angular.isDefined(modified[propKey]) && modified[propKey] !== origin[propKey]) {
                            if (extrObj == undefined) {
                                extrObj = {};
                            }
                            extrObj[propKey] = modified[propKey];
                        }
                    }
                    return extrObj;
                },
                interceptionAPIs: [
//                'api/device_groups/1',
//                'api/device_groups',
//                'api/devices/:id/discover',
//                'api/settings/device-types'
//                'setting/preference',
//                'api/alerts/search',
//                'api/doors/status',
//                'api/devices/firmware_upgrade'
                ],
                parseAPIUrl: function (url, base_url) {
                    //base_url = 'http://localhost:80/' + base_url;
//                console.log('base_url: ' + base_url + '  url:' + url); // 과도한 로그로 주석처리함
                    var _url = url;
                    if (url.indexOf(base_url) < 0 && url.indexOf($location.protocol()) < 0 && url.indexOf('.json') < 0) {
                        _url = (base_url + url).replace('//', '/');
                        if (_url.indexOf('http://') < 0 && _url.indexOf('http:/') == 0) {
                            _url = _url.replace('http:/', 'http://');
                        }

                    }
                    var rexUrlParse = /^\s*(((([^:\/#\?]+:)?(?:(\/\/)((?:(([^:@\/#\?]+)(?:\:([^:@\/#\?]+))?)@)?(([^:\/#\?\]\[]+|\[[^\/\]@#?]+\])(?:\:([0-9]+))?))?)?)?((\/?(?:[^\/\?#]+\/+)*)([^\?#]*)))?(\?[^#]+)?)(#.*)?/;
                    var matches = rexUrlParse.exec(_url || "") || [];

                    var urlObj = {
                        href: matches[  0 ] || "",
                        hrefNoHash: matches[  1 ] || "",
                        hrefNoSearch: matches[  2 ] || "",
                        domain: matches[  3 ] || "",
                        protocol: matches[  4 ] || "",
                        doubleSlash: matches[  5 ] || "",
                        authority: matches[  6 ] || "",
                        username: matches[  8 ] || "",
                        password: matches[  9 ] || "",
                        host: matches[ 10 ] || "",
                        hostname: matches[ 11 ] || "",
                        port: matches[ 12 ] || "",
                        pathname: matches[ 13 ] || "",
                        directory: matches[ 14 ] || "",
                        filename: matches[ 15 ] || "",
                        search: matches[ 16 ] || "",
                        hash: matches[ 17 ] || ""
                    };
//                console.log(urlObj); // 과도한 로그로 주석처리함
                    /**
                     * 주의!
                     * 다음 if 문의 첫번째 조건 urlObj.port !== '3412' 은 반드시 있어야 무한 재귀에 빠지지 않는다.
                     */
//                Array.prototype.indexOf = function(obj) {
//                  for(var i = 0; i < this.length; i++) {
//                    console.log(this[i]);
//                  }
//                  return 1;
//                };

                    var interceptorOK = false;
                    for (var i = 0; i < util.interceptionAPIs.length; i++) {
                        var item = util.interceptionAPIs[i];
                        if (urlObj.pathname.indexOf(item) > -1) {
                            interceptorOK = true;
                        }
                    }
//                console.log(urlObj.pathname); // 과도한 로그로 주석처리함
                    if (urlObj.port !== '3412' && interceptorOK) {
//                    console.log('interceptionAPIs: ' + urlObj.pathname); // 과도한 로그로 주석처리함
                        if (urlObj.authority == '') {
                            //$location.host();
                            urlObj = util.parseAPIUrl($location.protocol() + '://' + $location.host() + ':3412/' + urlObj.href, base_url);
                        } else {
                            urlObj = util.parseAPIUrl(urlObj.protocol + urlObj.doubleSlash + urlObj.hostname + ':3412/' + urlObj.pathname + urlObj.search, base_url);
                        }
                    }

                    return urlObj;
                },
                beforeUrl: function (url) {
                    if (url) {
                        beforeUrl = url;

                    } else {
                        return beforeUrl;
                    }
                },
                /**
                 * 지정된 범위의 정수를 랜덤하게 반환하는 함수
                 * 김재훈
                 * @param n1 하한값
                 * @param n2 상한값
                 * @returns {number|Number} 정수
                 */
                randomRange: function (n1, n2) {
                    return Math.floor((Math.random() * (n2 - n1 + 1)) + n1);
                },
                isNullOrUndefined: function (obj) {
                    if (obj === undefined || obj === null) {
                        return true;
                    }
                    return false;
                },
                buildUniqueListById: function (objList, obj) {
                    if (objList === undefined || objList === null) {
                        objList = new Array();
                    }
                    if (obj != undefined && obj != null) {
                        var found = false;
                        for (var i = 0; i < objList.length; i++) {
                            if (objList[i].id == obj.id) {
                                found = true;
                                break;
                            }
                        }
                        if (found == false) {
                            objList.push(obj);
                        }
                    }
                    return objList;
                },
                resizeLeftTreeWidth: function () {
                    var width = angular.element(".panelArea").width();
                    if (width >= 400) {
                        angular.element(".splitterLeft").addClass("splitterRight").removeClass("splitterLeft");
                        angular.element(".panelArea").width(0).addClass('off');
                        angular.element('.panelArea').find('.leftTreeMenu').hide();
                        var tabAct = angular.element(".panelArea").find(".tabAct");
                        if (tabAct != undefined) {
                            tabAct.hide();
                        }
                        var lnb = angular.element(".panelArea").find(".lnb");
                        if (lnb != undefined) {
                            lnb.hide();
                        }
                        angular.element(".panelAreaBtn").hide();
                    } else if (width > 0 && width <= 250) {
                        angular.element(".panelArea").width(400);
                        angular.element(".splitterRight").addClass("splitterLeft").removeClass("splitterRight");
                    } else if (width == 0) {
                        angular.element(".panelAreaBtn").show();
                        var tabAct = angular.element(".panelArea").find(".tabAct");
                        if (tabAct != undefined) {
                            tabAct.show();
                        }
                        var lnb = angular.element(".panelArea").find(".lnb");
                        if (lnb != undefined) {
                            lnb.show();
                        }
                        angular.element(".panelArea").width(240).removeClass('off');
                        angular.element('.panelArea').find('.leftTreeMenu').show();
                    }
                },
                removeItemByIdFromList: function (arrayObject, object) {
                    var foundIndex = -1;
                    for (var i = 0; i < arrayObject.length; i++) {
                        if (object.id == arrayObject[i].id) {
                            foundIndex = i;
                            break;
                        }
                    }
                    if (foundIndex > -1) {
                        arrayObject.splice(foundIndex, 1);
                    }
                },
                buildIdArrayFromObjectArray: function (arrayObject) {
                    var newArray = new Array();
                    for (var i = 0; i < arrayObject.length; i++) {
                        newArray.push(arrayObject[i].id);
                    }
                    return newArray;
                },
                buildNameArrayFromObjectArray: function (arrayObject) {
                    var newArray = new Array();
                    for (var i = 0; i < arrayObject.length; i++) {
                        newArray.push(arrayObject[i].name);
                    }
                    return newArray;
                },
                /**
                 *
                 * @param jqDataGrid: DataGrid의 ID 또는 selector
                 * @param selectorQuery: 하위에 찾고자 하는 selectorQuery
                 * @returns {gridPanelElement}
                 */
                datagridFind: function (jqDataGrid, selectorQuery) {
                    var grdPanel = angular.element(jqDataGrid).datagrid('getPanel');
                    if (selectorQuery) {
                        return grdPanel.find(selectorQuery);
                    }
                    return grdPanel;
                },
                setSessionObject: function (key, object) {
                    if (typeof object == 'string') {
                        window.sessionStorage.setItem(key, object);
                    } else {
                        window.sessionStorage.setItem(key, angular.toJson(object));
                    }

                },
                getSessionObject: function (key) {
                    var data = window.sessionStorage.getItem(key);
                    if (data) {
                        if (data.slice(0, 1) == '{' || data.slice(0, 1) == '[') {
                            return angular.fromJson(data);
                        } else {
                            return data;
                        }
                    } else {
                        return;
                    }
                },
                removeSessionObject: function (key) {
                    window.sessionStorage.removeItem(key);
                },
                removeLocalStorageObject: function (key) {
                    localStorage.removeItem(key);
                },
                setLocalStorageObject: function (key, object) {
                    if (typeof object == 'string') {
                        localStorage.setItem(key, object);
                    } else {
                        localStorage.setItem(key, angular.toJson(object));
                    }
                },
                getLocalStorageObject: function (key) {
                    var data = localStorage.getItem(key);
                    if (data) {
                        if (data.slice(0, 1) == '{' || data.slice(0, 1) == '[') {
                            return angular.fromJson(data);
                        } else {
                            return data;
                        }
                    } else {
                        return;
                    }
                },
                leftPad: function (n, width, z) {
                    z = z || '0';
                    n = n + '';
//                console.log(n);
                    return n.length >= width ? n : new Array(width - n.length + 1).join(z) + n;
                },
                hex2ascii: function (hexx) {
                    var hex = hexx.toString();//force conversion
                    var str = '';
                    for (var i = 0; i < hex.length; i += 2)
                        str += String.fromCharCode(parseInt(hex.substr(i, 2), 16));
                    return str;
                },
                ascii2hex: function (str) {
                    var arr = [];
                    for (var i = 0, l = str.length; i < l; i++) {
                        var hex = Number(str.charCodeAt(i)).toString(16);
                        arr.push(util.leftPad(hex, 2, '0'));
                    }
                    return arr.join('');
                },
                convert256BinaryArrayStringToString: function (binaryString) {
                    var returnValue = '';
                    var length = binaryString.length;
                    var loopCount = Math.floor(length / 8);
                    var currentIndex = 0;
                    var currentCount = 0;
                    for (var i = 0; i < loopCount; i++) {
                        var tempIntValue = parseInt(binaryString.substr(currentIndex, 8), 2);
                        returnValue += String.fromCharCode(tempIntValue);
                        currentIndex += 8;
                    }
//            if (currentIndex < length -1){
//                var tempIntValue = parseInt(binaryString.substr(currentIndex), 2);
//                returnValue += String.fromCharCode(tempIntValue);
//                currentIndex = length;
//            }
                    return returnValue;
                },
                buildBase64EncodedWiegandFormat: function (startBitString, endBitString, totalBitsString) {
                    var bitArrayString = util.buildWiegandFormatString(startBitString, endBitString, totalBitsString);
//                console.log(bitArrayString);
                    var stringValue = util.convert256BinaryArrayStringToString(bitArrayString);
//                console.log(stringValue);
                    return stringValue;
                },
                buildWiegandFormatString: function (startBitString, endBitString, totalBitsString) {
                    var startBit = parseInt(startBitString);
                    var endBit = parseInt(endBitString);
                    var totalBits = parseInt(totalBitsString);
//                console.log(startBit);
//                console.log(endBit);
//                console.log(totalBits);
                    var leftEmptyDataLength = Const.WIEGAND_FORMAT_MAX_SIZE - totalBits + startBit;
                    var bitArrayString = util.leftPad('', leftEmptyDataLength);
                    bitArrayString += util.leftPad('', endBit - startBit + 1, '1');
                    if (endBit < (totalBits - 1)) {
                        bitArrayString += util.leftPad('', totalBits - endBit);
                    }
//                console.log(bitArrayString);
                    return bitArrayString;
                },
                saveLanguageToLocalStorage: function (paramLanguage) {
                    util.setLocalStorageObject(Const.LOCAL_STORAGE.DISPLAY_LANGUAGE, {language: paramLanguage});
                },
                getLanguageFromLocalStorage: function () {
                    var storageObject = util.getLocalStorageObject(Const.LOCAL_STORAGE.DISPLAY_LANGUAGE);
                    if (storageObject != undefined) {
                        return storageObject.language;
                    }
                    return Const.DEFAULT_LANGUAGE_CODE;
                },
                debug: log().debug,
                info: log().info,
                warn: log().warn,
                error: log().error,
                log: log().log,
                resetColumnSetting: function () {
                    var removeStorageKeyList = new Array();
                    for (var i = 0, len = localStorage.length; i < len; ++i) {
                        var storageKey = localStorage.key(i);
                        var columnSettingStorageKeyPrefix = Const.LOCAL_STORAGE_KEY_PREFIX + '_grid_column_';
                        var columnSettingStorageKeyPrefix2 = Const.LOCAL_STORAGE_KEY_PREFIX + '_grid_default_column_';
                        if (storageKey.indexOf(columnSettingStorageKeyPrefix) > -1 || storageKey.indexOf(columnSettingStorageKeyPrefix2) > -1) {
                            removeStorageKeyList.push(storageKey);
                        }
                    }
                    for (var i = 0; i < removeStorageKeyList.length; i++) {
                        localStorage.removeItem(removeStorageKeyList[i]);
                    }
                },
                buildRandomString: function (len, bits) {
                    bits = bits || 36;
                    var outStr = "", newStr;
                    while (outStr.length < len) {
                        newStr = Math.random().toString(bits).slice(2);
                        outStr += newStr.slice(0, Math.min(newStr.length, (len - outStr.length)));
                    }
                    return outStr.toUpperCase();
                }
            };
            return util;
        }]);
});
