'use strict';

define([
    'modules/common/commonModule'
], function () {

    var app = angular.module('commonModule');

    app.provider('CustomWebSocket', function () {
        // when forwarding events, prefix the event name
        var _prefix = 'websocket:';
        var _WebSocket;
        var _uri;
        var _protocols;
        var _definedEvents = [];

        this.prefix = function (newPrefix) {
            _prefix = newPrefix;
            return this;
        };

        this.uri = function (uri, protocols) {
            protocols = Array.prototype.slice.call(arguments, 1);
            _uri = uri;
            _protocols = protocols;
            _WebSocket = new WebSocket(uri, protocols);
            console.log('uri: new web socket created');
            return this;
        };

        // expose to provider
        this.$get = ['$rootScope', '$timeout', 'Const', 'Util', function ($rootScope, $timeout, Const, Util) {

            _uri = Const.WEBSOCKET.URL;
            var ws = _WebSocket;

            var asyncAngularify = function (callback) {
                return function (args) {
                    args = Array.prototype.slice.call(arguments);
                    $timeout(function () {
                        callback.apply(ws, args);
                    });
                };
            };

            var addListener = function (event) {
                event = event && 'on' + event || 'onmessage';
                return function (callback) {
                    ws[event] = asyncAngularify(callback);
                    _definedEvents.push(event);
                    return this;
                };
            };

            var wrappedWebSocket = {
                states: ['CONNECTING', 'OPEN', 'CLOSING', 'CLOSED'],
                on: function (event, callback) {
                    return addListener(event)(callback);
                },
                onmessage: addListener('message'),
                onclose: addListener('close'),
                onopen: addListener('open'),
                onerror: addListener('error'),
                new: function (paramUri, paramProtocols) {
                    var oldws = ws;
                    if (paramUri != undefined) {
                        _uri = paramUri;
                    }
                    if (paramProtocols != undefined) {
                        _protocols = paramProtocols;
                    }
                    if (_protocols != undefined) {
                        ws = new WebSocket(_uri, _protocols);
                    } else {
                        ws = new WebSocket(_uri);
                    }
                    Util.info('new web socket created with uri:' + _uri);
//                    ws = new WebSocket(_uri, _protocols);
                    //assign the old events to the new websocket
                    var _len;
                    for (var i = 0, _len = _definedEvents.length; i < _len; i++) {
                        ws[_definedEvents[i]] = oldws[_definedEvents[i]];
                    }
                    return this;
                },
                close: function () {
                    Util.info('wrappedWebSocket close called', ws);
                    if (ws != undefined) {
                        ws.close();
                    }
                    return this
                },
                readyState: function () {
                    return ws.readyState
                },
                currentState: function () {
                    return this.states[ws.readyState];
                },
                send: function (message) {
                    message = Array.prototype.slice.call(arguments);
//                     Util.debug('WebSocket send. msg=' + message);
                    ws.send.apply(ws, message);
                    return this;
                },

                removeListener: function (args) {
                    args = Array.prototype.slice.call(arguments);
                    ws.removeEventListener.apply(ws, args);
                    return this;
                },

                // when ws.on('someEvent', fn (data) { ... }),
                // call scope.$broadcast('someEvent', data)
                forward: function (events, scope) {

                    if (events instanceof Array === false) {
                        events = [events];
                    }

                    if (!scope) {
                        scope = $rootScope;
                    }

                    events.forEach(function (eventName) {
                        var prefixedEvent = _prefix + eventName;
                        var forwardEvent = asyncAngularify(function (data) {
                            scope.$broadcast(prefixedEvent, data);
                        });
                        scope.$on('$destroy', function () {
                            ws.removeEventListener(eventName, forwardEvent);
                        });
                        ws.onmessage(eventName, forwardEvent);
                    });
                    return this;

                }
            };
            return wrappedWebSocket;
        }];
    });
    app.config(['CustomWebSocketProvider', function (CustomWebSocketProvider) {
        CustomWebSocketProvider
            .prefix('');
//            .uri('ws://localhost:8080/wsapi');
    }]);

    app.service('WebSocketService', [
        'Const', 'Util', 'API', '$rootScope', '$document', '$cookies', '$location', 'CustomWebSocket', '$timeout',
        function (Const, Util, API, $rootScope, $document, $cookies, $location, CustomWebSocket, $timeout) {

            var realtimeEventStarted = false;
            var webSocketService = {
                send: function (message) {
                    try {
                        Util.debug("WebSocketService send", message);
                        if (typeof WebSocket == "undefined") {
                            Util.info("WebSocket is not supported.");
                            return;
                        }
                        if (CustomWebSocket.readyState() === 1) {
                            Util.info("WebSocketService: Connection is made")
                        } else {
                            Util.info("WebSocketService: wait for connection...")
                        }
                        if (CustomWebSocket.readyState() != undefined && CustomWebSocket.readyState() == 1) {
                            CustomWebSocket.send(message);
                        }
                    } catch (e) {
                        Util.error("Error: " + e.message + ": " + e.toString(), e);
                    }
                },
                openConnection: function () {
                    var protocol = $location.protocol();
                    if (Const.WEB_SERVER_PROTOCOL.HTTPS == protocol) {
                        CustomWebSocket.new(Const.WEBSOCKET.SECURE_PROTOCOL + "://" + Const.WEBSOCKET.URL);
                    } else {
                        CustomWebSocket.new(Const.WEBSOCKET.PROTOCOL + "://" + Const.WEBSOCKET.URL);
                    }
                },
                init: function (options) {
                    Util.info('WebSocketService init');
                    console.log('WebSocketService init');
                    if (typeof WebSocket == 'undefined') {
                        Util.info('WebSocket is not supported.');
                        return;
                    }
                    webSocketService.openConnection();

//                CustomWebSocket.new('ws://' + $location.host() + ':8888/wsapi');
                    CustomWebSocket.onmessage(function (event) {
//                     Util.debug('WebSocketService message received: ' + event.data);
                        if (API.isLoggedIn() == false) {
                            Util.info('WebSocketService message ignored because not logged in: ' + event.data);
                            return;
                        }
                        var messageObj = JSON.parse(event.data);
                        if (angular.isDefined(messageObj.Alert)) {
                            $rootScope.$broadcast('AlertReceived', messageObj.Alert);
                            if (angular.isDefined(messageObj.Alert.event_id.door_id)) {
//                                $rootScope.$broadcast('DoorAlertReceived', messageObj.Alert);
                            }
                            if (angular.isDefined(messageObj.Alert.event_id.device_id)) {
//                                $rootScope.$broadcast('DeviceAlertReceived', messageObj.Alert);
                            }
                        } else if (angular.isDefined(messageObj.Event)) {
                            $rootScope.$broadcast('EventReceived', messageObj);
                            if (angular.isDefined(messageObj.Event.door_id)) {
                                $rootScope.$broadcast('DoorEventReceived', messageObj.Event);
                                if (Const.DOOR_ALARM_LIST.indexOf(messageObj.Event.event_type_id.code) > -1) {
                                    $rootScope.$broadcast('DoorAlarmReceived', messageObj.Event);
                                }
                            }
                            if (angular.isDefined(messageObj.Event.device_id)) {
                                $rootScope.$broadcast('DeviceEventReceived', messageObj.Event);

                            }
                        } else if (angular.isDefined(messageObj.Device)) {
                            $rootScope.$broadcast('DeviceStatusReceived', messageObj.Device);
//                    } else if (angular.isDefined(messageObj.Event.user_id)) {
//                        $rootScope.$broadcast('UserStatusReceived', messageObj.Event);
                        } else if (messageObj.Response != undefined) {
                            API.addTaskResponse(messageObj);
                        }
                    });
                    CustomWebSocket.onclose(function (event) {
                        Util.info('WebSocketService connection closed', event);
                        if (API.isLoggedIn() == true) {
                            $timeout(function () {
                                webSocketService.openConnection();
                            }, 5000);
                        }
                    });
                    CustomWebSocket.onopen(function (event) {
                        Util.info('WebSocketService connection open');
                        webSocketService.send(Const.HTTP_HEADER.BS_SESSION_ID + "=" + options[Const.HTTP_HEADER.BS_SESSION_ID]);
                    });
                },
                getState: function () {
                    return CustomWebSocket.readyState();
                },
                close: function () {
                    Util.info('WebSocketService close called');
                    realtimeEventStarted = false;
                    return CustomWebSocket.close();
                },
                isRealtimeEventStarted: function () {
                    return realtimeEventStarted;
                },
                startRealtimeEvent: function (successCallbackFunction, failedCallbackFunction) {
                    Util.info("startRealtimeEvent called", realtimeEventStarted);
                    if (realtimeEventStarted == false) {
                        var options = {
                            url: 'events/start',
                            params: {'Query': {}},
                            displaySuccessPopoup: false,
                            displayErrorPopup: false,
                            displayLoading: false
                        };
                        API.postWithOptions(options).then(function success(data) {
                            realtimeEventStarted = true;
                            Util.debug("realtime event started");
                            if (angular.isDefined(successCallbackFunction)) {
                                successCallbackFunction();
                            }
                        }, function error(data) {
                            realtimeEventStarted = false;
                            Util.debug("realtime event failed to start");
                            if (angular.isDefined(failedCallbackFunction)) {
                                failedCallbackFunction();
                            }
                        });
                    }
                },
                stopRealtimeEvent: function (successCallbackFunction, failedCallbackFunction) {
                    Util.info("stopRealtimeEvent called", realtimeEventStarted);
                    if (realtimeEventStarted == true) {
                        var options = {
                            url: 'events/stop',
                            params: {'Query': {}},
                            displaySuccessPopoup: false,
                            displayErrorPopup: false,
                            displayLoading: false
                        };
                        API.postWithOptions(options).then(function success(data) {
                            realtimeEventStarted = false;
                            Util.debug("realtime event stopped");
                            if (angular.isDefined(successCallbackFunction)) {
                                successCallbackFunction();
                            }
                        }, function error(data) {
                            Util.debug("realtime event failed to stop");
                            realtimeEventStarted = true;
                            if (angular.isDefined(failedCallbackFunction)) {
                                failedCallbackFunction();
                            }
                        });
                    }
                }
            };

            $rootScope.$on('LoginToSuccess', function (event) {
                console.log('WebSocketService: LoginToSuccess');
                var tempObject = {};
                tempObject[Const.HTTP_HEADER.BS_SESSION_ID] = API.getSessionID();
                webSocketService.init(tempObject);
                API.get("setting/biostar").then(function (data) {
                    Util.setSessionObject(Const.SESSION_STORAGE_KEY.SERVER_SETTING, data.BioStar);
                });
            });

            return webSocketService;
        }]);
});
