'use strict';

define([
    'modules/common/services/device-agent',
    'modules/device/deviceTreeController',
    'modules/common/directives/ngbssTriggerActionList',
    'modules/common/directives/ngbssOutputSignalModule',
    'modules/device/deviceInfoController',
    'modules/device/dialog/batchEditController',
    'modules/device/dialog/firmwareUpgradeController',
    'modules/device/dialog/languageResourceController',
    'modules/common/datagrid/columnSettingController',
    'modules/common/datagrid/columnSettingService',
    'modules/common/print/printOptionController',
    'modules/common/print/printOptionService'
], function () {
    var app = angular.module('biostarWebApp');
    app.register.controller('DeviceController', ['$scope', '$rootScope', '$q', '$window', '$compile', '$timeout', 'API', 'Const', '$location', 'DeviceAgent', 'Util', 'ColumnSettingService', 'PrintOptionService', 'MessageProperties', 'MessageService', 'PrivilegeService', function ($scope, $rootScope, $q, $window, $compile, $timeout, API, Const, $location, DeviceAgent, Util, ColumnSettingService, PrintOptionService, MessageProperties, MessageService, PrivilegeService) {

        $scope.gridId = "devicegrid";
        $scope.groupName = MessageProperties.get("menu.device.allDeviceGroups");
        $scope.groupId = "1";
        $scope.readOnly = false;
        if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.DEVICE)) {
            $scope.readOnly = true;
        }

        $scope.Parameters = {
            query: undefined,
            offset: 0,
            limit: 15,
            group_id: undefined,
            order_by: undefined,
            total: 0
        };

        $scope.SORT_COLUMN = {
            'id': 'id',
            'name': 'name',
            'device_group_id': 'device_group_id.id',
            'device_type_id': 'device_type_id.id',
            'lan': 'lan.ip',
            'status': 'status'
        };

        $scope.selectedDeviceIDs = [];
        $scope.selectedDevices = [];
        $scope.selectedDevicesNotSlave = [];
        $scope.selectedServerToDeviceModeDeviceID = undefined;
        // ---------------------- + Shift key 복수 선택 기능 ----------------------
        var lastSelectedRowIndex = 0;
        // ---------------------- + Shift key 복수 선택 기능 ----------------------

        $scope.$watch('Parameters', function (nv, ov) {
            if (nv !== ov) {
                sessionStorage.setItem('deviceQueryParam', angular.toJson($scope.Parameters));
            }
        }, true);

        $scope.safeApply = function (fn) {
            var phase = this.$root.$$phase;
            if (phase == '$apply' || phase == '$digest') {
                if (fn && (typeof(fn) === 'function')) {
                    fn();
                }
            } else {
                this.$apply(fn);
            }
        };

        $rootScope.$on('ChangeGroupName', function (event, data) {
            $scope.groupName = data.groupName;
            $scope.groupId = data.groupId;
        });
        $scope.oneMoreChecked = false;
        $scope.onlyOneChecked = false;
        $scope.onlyOneNotSlaveChecked = false;
        $scope.existInvalidDevices = false;
        $scope.checkOneMore = function () {
            $scope.safeApply(function () {
                var checkedDevice = $scope.selectedDevices;
                var checkedCount = checkedDevice.length;
                if (checkedCount <= 0) {
                    $scope.oneMoreChecked = false;
                    $scope.onlyOneChecked = false;
                    $scope.onlyOneNotSlaveChecked = false;
                } else if (checkedCount == 1) {
                    $scope.oneMoreChecked = true;
                    $scope.onlyOneChecked = true;
                    if (angular.isDefined(checkedDevice[0].parent_device_id) && angular.isDefined(checkedDevice[0].parent_device_id.id)) {
                        $scope.onlyOneNotSlaveChecked = false;
                    } else {
                        $scope.onlyOneNotSlaveChecked = true;
                    }

                    // reconnect 명령을 위한 부분
                    // reconnect 는 단일 장치에만 적용되는 명령이며 connection_mode 가 Server to Device 일 때에만 실행한다.
                    if (angular.isDefined($scope.selectedDevices[0].lan) &&
                        $scope.selectedDevices[0].lan.connection_mode == Const.CONNECTION_MODE.SERVER_TO_DEVICE) {
                        $scope.selectedServerToDeviceModeDeviceID = $scope.selectedDevices[0].id;
                    } else {
                        $scope.selectedServerToDeviceModeDeviceID = undefined;
                    }
                } else if (checkedCount > 1) {
                    $scope.oneMoreChecked = true;
                    $scope.onlyOneChecked = false;
                    $scope.onlyOneNotSlaveChecked = false;

                    $scope.existInvalidDevices = false;
                    for (var i = 0; i < $scope.selectedDevices.length; i++) {
                        var rowDate = $scope.selectedDevices[i];

                        // SECUREIO2는 bach Edit 할 수 없도록 s
                        if (angular.isDefined(rowDate.device_type_id) && rowDate.device_type_id.id == Const.DEVICE_TYPE.SECURE_IO_2) {
                            $scope.existInvalidDevices = true;
                        }
                        // SECUREIO2는 bach Edit 할 수 없도록 e
                    }
                }
            });
        };

        $('#devicegrid').datagrid({
            view: $.extend({}, $.fn.datagrid.defaults.view, {
                onBeforeRender: function (target, rows) {
                    angular.element(target).datagrid('resize');
                },
                onAfterRender: function (target) {
                    $.fn.datagrid.defaults.view.onAfterRender.call(this, target);
                    var grdPanel = angular.element(target).datagrid('getPanel');
                    var opts = angular.element(target).datagrid('options');
                    var vc = grdPanel.children('div.datagrid-view');
                    grdPanel.find(".datagrid-empty").remove();

                    if (angular.element(target).datagrid('getRows').length === 0) {
                        var html = '<div style="width:100%; height:360px; font-size:1.333em; line-height: 360px; ' +
                            'vertical-align: middle; text-align: center;">' + MessageProperties.get('validate.notFound') +
                            '</div>';
                        grdPanel.find('div.datagrid-view2 div.datagrid-body').html(html);
                    }

                    //헤더에 전체선택 버튼이 없다면 추가
                    if (grdPanel.find('.datagrid-header-check.compiled').length <= 0) {
                        grdPanel.find('.datagrid-header-check').attr({
                            'select-all-yn': 'SelectAllYN',
                            'datagrid-id': '"' + target.id + '"'
                        });
                        $compile(grdPanel.find('.datagrid-header-row'))($scope);
                        grdPanel.find('.datagrid-header-check').addClass('compiled');
                    }
                }
            }),
            fitColumns: true,
            scrollbarSize: 0,
            singleSelect: true,
            ctrlSelect: false,
            selectOnCheck: false,
            resize: true,
            checkOnSelect: false,
            autoRowHeight: true,
            checkbox: !$scope.readOnly,
            columns: [
                [
                    {field: ($scope.readOnly ? 'none' : Const.FIELD_NAME.CHECKBOX), checkbox: !$scope.readOnly, hidden: $scope.readOnly},
                    {field: 'id', title: MessageProperties.get('device.info.deviceId'), width: 80, sortable: true},
                    {field: 'name', title: MessageProperties.get('device.info.name'), width: 150, sortable: true},
                    {field: 'device_group_id', title: MessageProperties.get('device.info.group'), width: 150, sortable: false,
                        formatter: function (value, row) {
                            if (row.device_group_id === undefined || row.device_group_id === '') {
                                return '';
                            }
                            return '<span title="' + DeviceAgent.getDeviceGroupPath(row.device_group_id.id) + '">' +
                                row.device_group_id.name + '</span>';
                        }
                    },
                    {field: 'device_type_id', title: '<span>' + MessageProperties.get('device.info.deviceType') +
                        '</span><br/>(' + MessageProperties.get('device.rs485.master') + '/' +
                        MessageProperties.get('device.rs485.slave') + ')', width: 100, sortable: true,
                        formatter: function (value, row) {
                            if (angular.isUndefined(row.rs485)) {
                                return $scope.deviceSpec[value.id].name;
                            } else {
                                var rs485_mode = Const.RS485_MODE_HTML[row.rs485.mode] || Const.RS485_MODE_HTML._default;
                                if (rs485_mode != undefined) {
                                    return $scope.deviceSpec[value.id].name + rs485_mode;
                                }
                            }
                        }
                    },
                    {field: 'lan', title: MessageProperties.get('device.info.ipAddress'), width: 80, sortable: true, formatter: function (value, row, index) {
                        return angular.isDefined(value) && angular.isDefined(value.ip) ? value.ip : '';
                    }},
                    {field: 'status', title: MessageProperties.get('monitoring.deviceStatus'), width: 100, sortable: true, formatter: function (value, row, index) {
                        var statusText = Const.DEVICE_STATUS[Const.DEVICE_STATUS.DISCONNECTED];
                        if (DeviceAgent.getAllDevicesMapWithStatus()[row.id] != undefined && angular.isDefined(DeviceAgent.getAllDevicesMapWithStatus()[row.id].status)) {
                            statusText = Const.DEVICE_STATUS[(DeviceAgent.getAllDevicesMapWithStatus()[row.id].status) || Const.DEVICE_STATUS.DISCONNECTED];
                        }
                        return '<span id="dvcStatus' + row.id + '">' + statusText + '</span>';
                    }}
                ]
            ],
            onSortColumn: function (sort, order) {
                var descending = order == 'desc';
                $scope.Parameters.offset = 0;
                $scope.Parameters.order_by = $scope.SORT_COLUMN[sort] + ":" + descending;

                console.log('doSearch:');
                $scope.doSearch();
            },
            onCheck: function (rowIndex, rowData) {
                var idxID = $scope.selectedDeviceIDs.indexOf(rowData.id);
                if (idxID < 0) {

                    if ($scope.selectedDeviceIDs.length >= Const.SELECT_LIST_ITEM_LIMIT) {
                        MessageService.displayWarningMessage(MessageProperties.get('validate.canNotSelectMoreItem', Const.SELECT_LIST_ITEM_LIMIT));
                        angular.element('#devicegrid').datagrid('uncheckRow', rowIndex);
                    } else {
                        $scope.selectedDeviceIDs.push(rowData.id);
                        $scope.selectedDevices.push(rowData);
                    }
                }
                // parent 를 가지고 있는 Slave 장치는 syncDevice 명령을 실행할 수 없다.
                if (!(angular.isDefined(rowData.parent_device_id) && angular.isDefined(rowData.parent_device_id.id)) && !$scope.selectedDevicesNotSlave.contains(rowData)) {
                    $scope.selectedDevicesNotSlave.push(rowData);
                }
                $scope.checkOneMore();

                // ---------------------- + Shift key 복수 선택 기능 ----------------------
//                angular.element('#devicegrid').datagrid('getPanel').find('[datagrid-row-index=' + rowIndex + ']')
//                    .addClass('datagrid-row-selected');

                if (event.shiftKey && lastSelectedRowIndex != undefined && lastSelectedRowIndex >= 0 && lastSelectedRowIndex != rowIndex) {
                    if (lastSelectedRowIndex < rowIndex) {
                        angular.element('#devicegrid').datagrid('checkRow', rowIndex - 1);
                    } else {
                        angular.element('#devicegrid').datagrid('checkRow', rowIndex + 1);
                    }
                }
                lastSelectedRowIndex = rowIndex;
                // ---------------------- + Shift key 복수 선택 기능 ----------------------
            },
            onUncheck: function (rowIndex, rowData) {
                if ($scope.SelectAllYN) {
                    $scope.SelectAllYN = false;

                    $scope.selectedDeviceIDs = [];
                    $scope.selectedDevices = [];
                    $scope.selectedDevicesNotSlave = [];
                    var checkedRows = angular.element('#devicegrid').datagrid('getChecked');
                    for (var i = 0; i < checkedRows.length; i++) {
                        $scope.selectedDeviceIDs.push(checkedRows[i].id);
                        $scope.selectedDevices.push(checkedRows[i]);

                        // parent 를 가지고 있는 Slave 장치는 syncDevice 명령을 실행할 수 없다.
                        if (!(angular.isDefined(checkedRows[i].parent_device_id) && angular.isDefined(checkedRows[i].parent_device_id.id)) && !$scope.selectedDevicesNotSlave.contains(checkedRows[i])) {
                            $scope.selectedDevicesNotSlave.push(checkedRows[i]);
                        }
                    }
                } else {
                    var idxID = $scope.selectedDeviceIDs.indexOf(rowData.id);
                    if (idxID >= 0) {
                        $scope.selectedDeviceIDs.splice(idxID, 1);
                        $scope.selectedDevices.splice(idxID, 1);
                    }
                    // parent 를 가지고 있는 Slave 장치는 syncDevice 명령을 실행할 수 없다.
                    var idxNotSlaveID = $scope.selectedDevicesNotSlave.indexById(rowData.id);
                    if (idxNotSlaveID >= 0) {
                        $scope.selectedDevicesNotSlave.splice(idxNotSlaveID, 1);
                    }
                }
                $scope.checkOneMore();

                // ---------------------- + Shift key 복수 선택 기능 ----------------------
//                angular.element('#devicegrid').datagrid('getPanel').find('[datagrid-row-index=' + rowIndex + ']')
//                    .removeClass('datagrid-row-selected');

                if (event.shiftKey && lastSelectedRowIndex != undefined && lastSelectedRowIndex >= 0 && lastSelectedRowIndex != rowIndex) {
                    if (lastSelectedRowIndex < rowIndex) {
                        angular.element('#devicegrid').datagrid('uncheckRow', rowIndex - 1);
                    } else {
                        angular.element('#devicegrid').datagrid('uncheckRow', rowIndex + 1);
                    }
                }
                lastSelectedRowIndex = rowIndex;
                // ---------------------- + Shift key 복수 선택 기능 ----------------------
            },
            onCheckAll: function (rows) {
                //$("#gridDropDownMenuDiv").css({"visibility": "visible"});
//                console.log(rows.length);
                var uncheckList = [];
                for (var i = 0; i < rows.length; i++) {
                    var rowData = rows[i];
                    var idxID = $scope.selectedDeviceIDs.indexOf(rowData.id);
                    if (idxID < 0) {
                        if ($scope.selectedDeviceIDs.length >= Const.SELECT_LIST_ITEM_LIMIT) {
                            uncheckList.push(rowData);
                        } else {
                            $scope.selectedDeviceIDs.push(rowData.id);
                            $scope.selectedDevices.push(rowData);
                        }
                    }
                    // parent 를 가지고 있는 Slave 장치는 syncDevice 명령을 실행할 수 없다.
                    if (!(angular.isDefined(rowData.parent_device_id) && angular.isDefined(rowData.parent_device_id.id)) && !$scope.selectedDevicesNotSlave.contains(rowData)) {
                        $scope.selectedDevicesNotSlave.push(rowData);
                    }
                }

                if (uncheckList.length) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.canNotSelectMoreItem', Const.SELECT_LIST_ITEM_LIMIT));
                    for (i = 0; i < uncheckList.length; i++) {
                        angular.element('#devicegrid').datagrid('uncheckRow', angular.element('#devicegrid').datagrid('getRowIndex', uncheckList[i]));
                    }

                }

                $scope.checkOneMore();
                // ---------------------- + Shift key 복수 선택 기능 ----------------------
                lastSelectedRowIndex = 0;
                // ---------------------- + Shift key 복수 선택 기능 ----------------------

            },
            onUncheckAll: function (rows) {
                $scope.SelectAllYN = false;
                for (var i = 0; i < rows.length; i++) {
                    var rowData = rows[i];
                    var idxID = $scope.selectedDeviceIDs.indexOf(rowData.id);
                    if (idxID >= 0) {
                        $scope.selectedDeviceIDs.splice(idxID, 1);
                        $scope.selectedDevices.splice(idxID, 1);
                    }
                    // parent 를 가지고 있는 Slave 장치는 syncDevice 명령을 실행할 수 없다.
                    var idxNotSlaveID = $scope.selectedDevicesNotSlave.indexById(rowData.id);
                    if (idxNotSlaveID >= 0) {
                        $scope.selectedDevicesNotSlave.splice(idxNotSlaveID, 1);
                    }
                }
                $scope.checkOneMore();
                // ---------------------- + Shift key 복수 선택 기능 ----------------------
                lastSelectedRowIndex = 0;
                // ---------------------- + Shift key 복수 선택 기능 ----------------------
            },
            onClickCell: function (rowIndex, field) {
                $rootScope.clickCellOnGrid($scope.gridId, rowIndex, field, "id", function (rowData) {
                    sessionStorage.setItem('deviceSelectedIndex', $scope.Parameters.offset + rowIndex);
                    DeviceAgent.setSelectedDevice(rowData);
                    $scope.editDevice();
                });
            },
            onLoadSuccess: function (data) {
//                console.log("DeviceController: onLoadSuccess");
                if (!$scope.readOnly) {

                    if (data.rows.length <= 0) {
                        angular.element('#devicegrid').datagrid('uncheckAll');
                    } else if ($scope.SelectAllYN) {
                        angular.element('#devicegrid').datagrid('checkAll');
                    } else {
                        for (var i = 0; i < data.rows.length; i++) {
                            var idxID = $scope.selectedDeviceIDs.indexOf(data.rows[i].id);
                            if (idxID >= 0) {
                                angular.element('#devicegrid').datagrid('checkRow', i);
                            } else {
                                angular.element('#devicegrid').datagrid('uncheckRow', i);
                            }
                        }
                    }
                }
                // ---------------------- + Shift key 복수 선택 기능 ----------------------
                lastSelectedRowIndex = 0;
                // ---------------------- + Shift key 복수 선택 기능 ----------------------
            }
        });

        $scope.SelectAllYN = false;
        //헤더에 전체선택 버튼 추가
//        angular.element('.datagrid-header-check').attr({'select-all-yn' : 'SelectAllYN', 'datagrid-id': '"devicegrid"'});
//        $compile(angular.element('.datagrid-header-row'))($scope);

        $scope.$watch('SelectAllYN', function (nv) {
            if (nv) {
                angular.element('#devicegrid').datagrid('checkAll');
            } else {
                if (angular.element('#devicegrid').datagrid('getChecked').length == $scope.Parameters.limit) {
                    $scope.selectedDeviceIDs = [];
                    $scope.selectedDevices = [];
                    angular.element('#devicegrid').datagrid('uncheckAll');
                }
            }
        });

        $scope.$watch(function () {
            return  angular.element('.panel.datagrid').width();
        }, function (nv, ov) {
            if (nv !== ov) {
                angular.element('#devicegrid').datagrid('resize');
            }
        }, true);

        angular.element($window).on('resize', function ($event) {
            var event = $event | window.event;
            angular.element('#devicegrid').datagrid('resize');
        });

        // -------------------------- shift 키 조합으로 복수 선택을 위한 코드 s -------------------------------------
        angular.element(document).on('selectstart', 'td[field=ck]', function () {
            return false;
        });
        // -------------------------- shift 키 조합으로 복수 선택을 위한 코드 e -------------------------------------

        $scope.doSearch = function (query, functionName) {// search

            if (query) {
                $scope.Parameters.query = query;
            }

            if (functionName == null) {
                functionName = 'getDevicesInController';
            }

            return API.get('devices', functionName, $scope.Parameters).then(function (data) {

                var index = 0;
                if (data.DeviceCollection.rows != undefined) {
                    for (index = 0; index < data.DeviceCollection.rows.length; index++) {
                        if (data.DeviceCollection != undefined && data.DeviceCollection.rows[index] != undefined &&
                            data.DeviceCollection.rows[index].device_group_id != undefined && data.DeviceCollection.rows[index].device_group_id.id == Const.DEFAULT_ALL_GROUP_ID) {
                            data.DeviceCollection.rows[index].device_group_id.name = MessageProperties.get("menu.device.allDeviceGroups");
                        }
                    }
                }
                // TODO: sessionStorage 에 저장하지 않고 Indexed DB에 저장하도록 변경되어야 함.
                var jsData = angular.toJson(data.DeviceCollection.rows);
                window.sessionStorage.setItem("deviceData", jsData);// 세션저장

                $("#devicegrid").datagrid("loadData", data.DeviceCollection.rows || []);
                $scope.DevicesData = data.DeviceCollection.rows;
                $scope.Parameters.total = data.DeviceCollection.total;

            });
        };


        $scope.SearchCallback = {
            fn: $scope.doSearch
        };

        if ($rootScope.$$listeners['onRefreshDeviceList'] != undefined) {
            $rootScope.$$listeners['onRefreshDeviceList'] = undefined;
        }
        $rootScope.$on('onRefreshDeviceList', function (event, data) {
            $scope.Parameters.offset = 0;
            angular.element('#devicegrid').datagrid('uncheckAll');
            $scope.doSearch();
        });

        if ($rootScope.$$listeners['clickDeviceGroupItemOnTree'] != undefined) {
            $rootScope.$$listeners['clickDeviceGroupItemOnTree'] = undefined;
        }
        $rootScope.$on('clickDeviceGroupItemOnTree', function (event, group_id) {
            $scope.Parameters.group_id = group_id;
            $scope.Parameters.offset = 0;
            angular.element('#devicegrid').datagrid('uncheckAll');
            console.log('clickDeviceGroupItemOnTree:');
            $scope.doSearch();
        });

        if ($rootScope.$$listeners['AlarmDeviceAddMessage'] != undefined) {
            $rootScope.$$listeners['AlarmDeviceAddMessage'] = undefined;
        }
        $rootScope.$on('AlarmDeviceAddMessage', function (event, addedDeviceCount) { //child ---> parent
            console.log('AlarmDeviceAddMessage:');
            $scope.doSearch();
//            if (angular.isNumber(addedDeviceCount) && addedDeviceCount > 0) {
//                $scope.messCnt = addedDeviceCount;
//                var width = angular.element('#gridTopMenu').width();
//                var left = angular.element('#gridTopMenu').position().left;
//                angular.element('#topAddMessage').css({'left': left});
//                angular.element('#topAddMessage').width(width - 2);
//                angular.element('#topAddMessage').stop().fadeIn(100).delay(5000).fadeOut(100);//상단 메시지 추가
//            }
        });

        /**
         * Search Device 및 Search Slave Device, 그리고 Advanced Search Device 를 통해 추가된 장치의 수를 토스트 팝업으로 표시한다.
         */
        $scope.$on('AlarmDeviceMultiAddMessage', function (event, addedDeviceInfo) { //child ---> parent
            console.log('AlarmDeviceMultiAddMessage', addedDeviceInfo);

            if (addedDeviceInfo && addedDeviceInfo.rows && addedDeviceInfo.rows.length) {
                var parentNode = $('#devicegrouptree').tree('find', addedDeviceInfo.parent_device_id.id);
                var childrenNodes = $('#devicegrouptree').tree('getChildren', parentNode.target);
                var node;
                for (var i = (childrenNodes.length - 1); i >= 0; i--) {
                    if (childrenNodes[i].isDevice) {
                        node = childrenNodes[i];
                        break;
                    }
                }
                for (var index in addedDeviceInfo.rows) {
                    var device = addedDeviceInfo.rows[index];

                    $('#devicegrouptree').tree('insert', {
                        before: node.target,
                        data: {
                            id: device.id,
                            text: device.name,
                            isDevice: true,
                            deviceData: device,
                            iconCls: (angular.isDefined(device.device_type_id) ? Const.DEVICE_TYPE_ICON[device.device_type_id.id] : '')
                        }
                    });
                }

                $scope.messCnt = addedDeviceInfo.rows.length;
                $('#topAddMessage').stop().fadeIn(100).delay(5000).fadeOut(100);//상단 메시지 추가
            }

            console.log('doSearch:');
            $scope.doSearch();
        });

        function updateDeviceStatus(deviceObject) {
            var deviceStatusHtmlObject = angular.element('#dvcStatus' + deviceObject.id);
            if (deviceStatusHtmlObject != null && deviceObject.status) {
                deviceStatusHtmlObject.text(Const.DEVICE_STATUS[deviceObject.status]);
            }
            // ----------------------------- rs485 처리 s -----------------------------
            var rowIndex = angular.element('#dvcStatus' + deviceObject.id).parents('[datagrid-row-index]').attr('datagrid-row-index');
            if (angular.isDefined(deviceObject.rs485) && rowIndex != undefined) {
                var rows = angular.element('#' + $scope.gridId).datagrid('getRows');
                var rs485;
                if (angular.isDefined(rows[rowIndex].rs485)) {
                    rs485 = rows[rowIndex].rs485;
                } else {
                    rs485 = {};
                }
                angular.extend(rs485, deviceObject.rs485);
                angular.element('#' + $scope.gridId).datagrid('updateRow', {
                    index: rowIndex,
                    row: {
                        rs485: rs485
                    }
                });
            }
            // ----------------------------- rs485 처리 e -----------------------------
        }

        /**
         * 장치로부터 WebSocket을 통해 받은 장치의 최신 상태를 업데이트 한다.
         *  TODO: 중복등록 방지코드의 개선 필요. 해당 Controller 에서 등록한 이벤트만 재 등록을 방지
         *  DeviceStatusReceived 이벤트는 DeviceAgent, DeviceController 등 필요에 따라
         *  여러군데 등록되어 실행되므로 중복등록 방지코드에서 선별작업이 필요하다.
         *  현재 중복등록 방지코드는 같은 이름을 무조건 삭제하도록 되어 있는데
         *  DeviceStatusReceived 의 경우 해당 Controller 에서 등록한 이벤트만 재 등록을 방지할 수 있어야 하므로
         *  조금 더 고민해 봐야 한다.
         */
        $scope.$on('DeviceStatusReceived', function (event, message) {
            if (message == null || message == undefined) {
                return;
            }
            $timeout(function () {
                updateDeviceStatus(message);
            }, 400);
        });

        $scope.$on('ReceiveDeviceTreeDataSuccess', function (event, message) {
            for (var key in DeviceAgent.getAllDevicesMapWithStatus()) {
                var deviceObject = DeviceAgent.getAllDevicesMapWithStatus()[key];
                if (deviceObject != undefined) {
                    updateDeviceStatus(deviceObject);
                }
            }
        });

        $scope.$on(Const.UI_EVENT_TYPE.DEVICE_INFO_UPDATED, function (event, deviceObject) {
            console.log(Const.UI_EVENT_TYPE.DEVICE_INFO_UPDATED, deviceObject);
            var rows = angular.element('#' + $scope.gridId).datagrid('getRows');
            for (var i = 0; i < rows.length; i++) {
                if (rows[i].id == deviceObject.id) {
                    var rowIndex = angular.element('#' + $scope.gridId).datagrid('getRowIndex', rows[i]);
                    rows[i].name = deviceObject.name;
                    rows[i].rs485 = deviceObject.rs485;
                    rows[i].status = deviceObject.status;
                    angular.element('#' + $scope.gridId).datagrid('refreshRow', rowIndex);
                    break;
                }
            }
        });


        $scope.doDeleteMessage = function (id) {// 메시지 삭제
            $(id).stop().hide();
        };

//        sessionStorage.removeItem('deviceData');//session storage 삭제


        $scope.doDeleteDevice = function () {
            MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (dialogResult) {
                if (dialogResult) {
                    var deviceIDs = [];
                    if ($scope.SelectAllYN) {
                        deviceIDs.push(Const.SELECTED_ALL);
                    } else {
                        deviceIDs = $scope.selectedDeviceIDs;
//                        var rows = angular.element('#devicegrid').datagrid('getChecked');//선택된 데이터 배열
//                        for (var i = 0; i < rows.length; i++) {
//                            deviceIDs.push(rows[i].id);
//                        }
                    }
                    if ($scope.SelectAllYN == false && deviceIDs.length == 1) {
                        API.del('devices/' + deviceIDs[0]).then(function (data) {
//                            console.log('delete idx');
                            console.log('doSearch:');
                            $scope.Parameters.offset = 0;
                            $scope.selectedDeviceIDs = [];
                            $scope.selectedDevices = [];
                            $scope.SelectAllYN = false;
                            $scope.doSearch();
                            $scope.safeApply(function () {
                                $rootScope.$emit('reloadDeviceTree');
                            });
                            //$.messager.alert('Information', MessageProperties.get('common.deleted'), 'info');
                        });
                    } else if (deviceIDs.length > 0) {
                        var queryParam = '';
                        if ($scope.SelectAllYN && angular.isDefined($scope.Parameters)) {
                            if (angular.isDefined($scope.Parameters.group_id) && $scope.Parameters.group_id != '') {
                                queryParam += '&group_id=' + $scope.Parameters.group_id;
                            }
                            if (angular.isDefined($scope.Parameters.query) && $scope.Parameters.query != '') {
                                queryParam += '&query=' + $scope.Parameters.query;
                            }
                        }
                        API.del('devices?id=' + deviceIDs.join('+') + queryParam).then(function (data) {
//                            console.log('delete idx');
                            console.log('doSearch:');
                            $scope.Parameters.offset = 0;
                            $scope.selectedDeviceIDs = [];
                            $scope.selectedDevices = [];
                            $scope.SelectAllYN = false;
                            $scope.doSearch();
                            $scope.safeApply(function () {
                                $rootScope.$emit('reloadDeviceTree');
                            });
                            //$.messager.alert('Information', MessageProperties.get('common.deleted'), 'info');
                        });
                    }
//                    var def = $.Deferred();
//                    def.done(function () {
//                        for (var i = arrCheckedDevice.length - 1; i >= 0; i--) {
//                            var idx = angular.element('#devicegrid').datagrid('getRowIndex', arrCheckedDevice[i]);
//                            Device["delete"]({id: arrCheckedDevice[i].id}, (function (data) {
//                                console.log('delete idx:' + idx);
//                                angular.element('#devicegrid').datagrid('deleteRow', idx);
//                            })(idx));
//                        }
//                    }).done(function () {
//                         MessageService.displayWarningMessage(MessageProperties.get('common.deleted'));
//                    });
//
//                    def.resolve();
                }
            });
        };

        $scope.doReconnectDevice = function () {

//            var rows = angular.element('#devicegrid').datagrid('getChecked');//선택된 데이터 배열
            if (angular.isDefined($scope.selectedServerToDeviceModeDeviceID)) {
                $rootScope.$emit(Const.DEVICE_CONTEXT_MENU.RECONNECT, {id: $scope.selectedServerToDeviceModeDeviceID});
            }
        };


        $scope.doSyncDevice = function () {
            if ($scope.selectedDevicesNotSlave.length == $scope.selectedDevices.length && $scope.selectedDevices.length > 0) {
                DeviceAgent.fullSyncDevices($scope.selectedDevices);
            } else if ($scope.SelectAllYN || $scope.selectedDevicesNotSlave.length != $scope.selectedDevices.length) {
                MessageService.displayInformationMessage(MessageProperties.get('device.notAllowSlaveDevice'), function () {
                    if ($scope.SelectAllYN) {
                        DeviceAgent.fullSyncDevices([]);
                    } else {
                        if ($scope.selectedDevicesNotSlave.length > 0) {
                            DeviceAgent.fullSyncDevices($scope.selectedDevicesNotSlave);
                        }
                    }
                });
            }
        };

        $scope.manageUsers = function () {

//            var rows = angular.element('#devicegrid').datagrid('getChecked');//선택된 데이터 배열
            if ($scope.selectedDevices.length > 0) {
                $rootScope.$emit('manageUsersinDevice', {id: $scope.selectedDevices[0].id, name: $scope.selectedDevices[0].name});
            }
        };

        // -------------------------- batchEdit 관련 s --------------------------

//        $scope.batchEdit = function () {// 페이지 이동
//
//            if ($scope.selectedDevices.length > 0) {
//                var jsData = angular.toJson($scope.selectedDevices);
//                window.sessionStorage.setItem('deviceBatchEditData', jsData);// 세션저장
//
//                if ($scope.SelectAllYN) {
//                    window.sessionStorage.setItem('deviceBatchEditSelectAll', Const.SELECTED_ALL);// 세션저장
//                }
//
//                $scope.editDevice();
//
//            } else {
//                MessageService.displayWarningMessage(MessageProperties.get('common.pleaseSelectAtLeastOne'));
//                return false;
//            }
//        };
        $scope.batchEdit = function () {

            var selectedDeviceCount = 0;
            if ($scope.SelectAllYN) {
                selectedDeviceCount = $scope.Parameters.total;
            } else {
                selectedDeviceCount = $scope.selectedDevices.length;
            }

            Util.setSessionObject(Const.DEVICE_BATCH_EDIT_SELECTED_COUNT, selectedDeviceCount + '');

            $scope.batchEditPopup();
        };

        $scope.batchEditDialog = '';
        $scope.batchEditPopup = function () {
            $scope.safeApply(function () {
                $scope.batchEditDialog = 'modules/device/dialog/batchEdit.html';
            });
        };
        $scope.afterOpenBatchEditPopup = function () {

            var options = {};
            options.modal = true;
            options.shadow = false;
            options.border = false;
            options.resizable = false;
            options.cache = false;
            options.title = MessageProperties.get('device.batchEdit');
            options.width = 495;
            options.height = 'auto';
            options.selectedDevices = $scope.selectedDevices;
            options.Parameters = $scope.Parameters;
            options.SelectAllYN = $scope.SelectAllYN;

            options.onClose = function () {
                $scope.safeApply(function () {
                    $scope.batchEditDialog = '';
                    angular.element('#dlgBatchEdit').dialog('destroy');
                });
            };

            angular.element('#dlgBatchEdit').dialog(options);
        };
        // -------------------------- batchEdit 관련 e --------------------------

        // -------------------------- firmwareUpgrade 관련 s -------------------------- 
        $scope.firmwareUpgrade = function () {
            var rows = $scope.selectedDevices;//선택된 데이터 배열
            if ($scope.SelectAllYN && parseInt($scope.Parameters.total) > rows.length) {
                MessageService.displayWarningMessage(MessageProperties.get('device.selectAllDeviceFWUpgradeWarning'));
                return false;
            }

            if ($scope.selectedDevicesNotSlave.length != $scope.selectedDevices.length) {
                MessageService.displayInformationMessage(MessageProperties.get('device.notAllowSlaveDevice') + '<br />' +
                    MessageProperties.get('device.infoFWUpgradeSlaveDevice'), function () {
                    $scope.firmwareUpgradePopup();
                });
            } else {
                $scope.firmwareUpgradePopup();
            }
        };

        $scope.firmwareUpgradeDialog = '';
        $scope.firmwareUpgradePopup = function () {
            $scope.safeApply(function () {
                $scope.firmwareUpgradeDialog = 'modules/device/dialog/firmwareUpgrade.html';
            });
        };
        $scope.afterOpenFirmwareUpgradePopup = function () {
            var deviceSessionData = DeviceAgent.getSelectedDevice();
            var selectedDevices = undefined;
            if (deviceSessionData != undefined && deviceSessionData != null) {
                selectedDevices = [deviceSessionData];
            } else {
                selectedDevices = $scope.selectedDevicesNotSlave;
            }

            var options = {};
            options.modal = true;
            options.shadow = false;
            options.border = false;
            options.cache = false;
            options.title = MessageProperties.get('device.firmwareUpgrade');
            options.width = 400;
            options.height = 'auto';
            options.selectedDevices = selectedDevices;

            options.onOpen = function () {
                $rootScope.$emit(Const.ON_OPEN_FW_UPGRADE_POPUP);
            };

            options.onClose = function () {
                $scope.safeApply(function () {
                    $scope.firmwareUpgradeDialog = '';
                    angular.element('#dlgFirmwareUpgrade').dialog('destroy');
                });
            };

            angular.element('#dlgFirmwareUpgrade').dialog(options);
        };
        // -------------------------- firmwareUpgrade 관련 e --------------------------

        // -------------------------- languageResource 관련 s -------------------------- 
        $scope.languageResource = function () {
            var rows = $scope.selectedDevices;//선택된 데이터 배열
            if ($scope.SelectAllYN && parseInt($scope.Parameters.total) > rows.length) {
                MessageService.displayWarningMessage(MessageProperties.get('device.selectAllDeviceFWUpgradeWarning'));
                return false;
            }

//            for (var i = 1; i < rows.length; i++) {
//                console.log(rows[i].device_type_id.id);
//                if (rows[i - 1].device_type_id.id != rows[i].device_type_id.id) {
//                    MessageService.displayWarningMessage(MessageProperties.get('device.selectDifferentDeviceFWUpgradeWarning'));
//                    return false;
//                }
//            }
//            MessageService.displayWarningMessage('Firmware Upgrade');
            $scope.languageResourcePopup();
        };

        $scope.languageResourceDialog = '';
        $scope.languageResourcePopup = function () {
            $scope.safeApply(function () {
                $scope.languageResourceDialog = 'modules/device/dialog/languageResource.html';
            });
        };
        $scope.afterOpenlanguageResourcePopup = function () {
            var deviceSessionData = DeviceAgent.getSelectedDevice();
            var selectedDevices = undefined;
            if (deviceSessionData != undefined && deviceSessionData != null) {
                selectedDevices = [deviceSessionData];
            } else {
                selectedDevices = $scope.selectedDevices;
            }

            var options = {};
            options.modal = true;
            options.shadow = false;
            options.border = false;
            options.cache = false;
            options.title = MessageProperties.get('device.languageResource');
            options.width = 400;
            options.height = 'auto';
            options.selectedDevices = selectedDevices;

            options.onClose = function () {
                $scope.safeApply(function () {
                    $scope.languageResourceDialog = '';
                    angular.element('#dlgLanguageResource').dialog('destroy');
                });
            };

            angular.element('#dlgLanguageResource').dialog(options);
        };
        // -------------------------- languageResource 관련 e --------------------------

        $scope.selectedDeviceInfo = '';
        $scope.editDevice = function () {
            $("#devicegrid").datagrid('uncheckAll');
            $scope.safeApply(function () {
                $scope.selectedDeviceInfo = 'modules/device/deviceInfo.html';
            });
        };

        if ($rootScope.$$listeners['EditDeviceInfo'] != undefined) {
            $rootScope.$$listeners['EditDeviceInfo'] = undefined;
        }
        $rootScope.$on('EditDeviceInfo', function (event, data) {
//            var index = angular.element('#devicegrid').datagrid('getRowIndexByID', {id: DeviceAgent.getSelectedDevice().id});
//            if(index > 0) {
//                sessionStorage.setItem('deviceSelectedIndex', $scope.Parameters.offset + index);
//            } else {
//                sessionStorage.setItem('deviceSelectedIndex', index);
//            }
            sessionStorage.removeItem('deviceSelectedIndex');
            $scope.selectedDeviceInfo = undefined;
            $scope.editDevice();
        });

        $scope.$on('CloseDeviceInfo', function (event, data) {
//            console.log("deviceController::CloseDeviceInfo");
            DeviceAgent.rejectGetDevice();
            DeviceAgent.rejectGetDeviceList();
            window.sessionStorage.removeItem('deviceSelectedIndex');
            $scope.selectedDeviceInfo = undefined;
            DeviceAgent.removeDevice();
        });

        if ($rootScope.$$listeners['ClickMenuInit'] != undefined) {
            $rootScope.$$listeners['ClickMenuInit'] = undefined;
        }
        $rootScope.$on('ClickMenuInit', function (event, data) {
            window.sessionStorage.removeItem('deviceSelectedIndex');
            $scope.selectedDeviceInfo = undefined;
            DeviceAgent.removeDevice();
        });

        if ($rootScope.$$listeners['firmwareUpgradePopup'] != undefined) {
            $rootScope.$$listeners['firmwareUpgradePopup'] = undefined;
        }
        $rootScope.$on('firmwareUpgradePopup', function () {
            $scope.firmwareUpgradePopup();
        });

        if ($rootScope.$$listeners['languageResourcePopup'] != undefined) {
            $rootScope.$$listeners['languageResourcePopup'] = undefined;
        }
        $rootScope.$on('languageResourcePopup', function () {
            $scope.languageResourcePopup();
        });

        $scope.dropDownMenu = [
//            {
//                menu: 'Print Preview',
//                func: function () {
//                    Util.beforeUrl($location.url());
//                    $location.path('/user/users-report');
//                }
//            },
            {
                menu: MessageProperties.get('menu.print'),
                func: function () {
                    $scope.openPrintOptionPopupDlg();
                }
            },
            {
                menu: MessageProperties.get('menu.columnSetting'),
                func: function () {
                    $scope.openColumnSettingPopupDlg();
                }
            }
        ];

        $scope.$watch('Parameters', function (nv, ov) {
            if (nv !== ov) {
                PrintOptionService.buildGridQueryFunction(Const.REPORT_COLLECTION_NAME.DEVICE, nv);
            }
        }, true);

        PrintOptionService.getReportColumnTypeFunction = function (gridColumnName) {
            if (gridColumnName == "status") {
                return "enum"
            } else {
                return "string";
            }
        };

        PrintOptionService.getReportColumnNameFunction = function (gridColumnName) {
            var reportColumnName = gridColumnName;
            if (gridColumnName == "device_group_id") {
                return "device_group_id.name"
            } else if (gridColumnName == "device_type_id") {
                return "device_type_id.name"
            } else if (gridColumnName == "lan") {
                return "lan.ip"
            }
            return reportColumnName;
        };

        $scope.reportUrl = Const.API.BASE_URL + 'devices/report';
        $scope.openPrintOptionPopupDlg = PrintOptionService.openPrintOptionPopupDlg;
        $scope.openColumnSettingPopupDlg = ColumnSettingService.openColumnSettingPopupDlg;

        ColumnSettingService.setGridId($scope.gridId);
        ColumnSettingService.setCallbackFunction($scope.SearchCallback.fn);
        var customColumnSetting = ColumnSettingService.getColumnSettingFromLocalStroage();

        var deviceSessionData = DeviceAgent.getSelectedDevice();

        /**
         * Device 목록 조회의 4가지 경우
         * 1. 컬럼 셋팅 수정이 있을 경우
         * 2. 장치 상세보기의 선택된 장치가 있을 경우
         * 3. 1과 2가 모두 있는 경우
         * 4. 1과 2가 모두 없는 경우
         *
         * 동작
         * 1. 컬럼셋팅정보를 반영하여 Device 목록을 조회한다.
         * 2. Device 목록을 조회한 후에 장치 상세 페이지를 띄운다.
         * 3. 컬럼셋팅정보를 반영하여 Device 목록을 조회한 후에 장치 상세 페이지를 띄워야 하지만
         *      ColumnSettingService 등의 다른 코드의 변경이 필요하여 side effect 이 우려된다.
         *      따라서
         *          Device 목록을 조회한 후에
         *          컬럼셋팅정보를 반영하여 Device 목록을 조회하면서
         *          장치 상세 페이지를 띄우도록 하였다.
         *      이 때 동일 API 호출을 막기 위한 canceler 가 동작하지 않도록 functionName 을 다르게 부여한다.
         *      TODO: 가급적 한 번만 목록 조회를 하면서도 1과 2가 모두 반영될 수 있도록 해야한다.
         * 4. 그냥 Device 목록을 조회한다.
         */
        $scope.deviceSpec = undefined;
        $scope.init = function () {
            DeviceAgent.getDeviceSpec().then(function (data) {
                $scope.deviceSpec = data;
            }).finally(function () {
                if (deviceSessionData != undefined && deviceSessionData != null) {
                    console.log("deviceController device session data exist:" + deviceSessionData);
                    console.log('doSearch:');
                    if (customColumnSetting) {
                        $scope.doSearch(undefined, 'editDevice').finally(function () {
                            ColumnSettingService.applyChangedColumnsToGrid(customColumnSetting, false);
                            $scope.editDevice();
                        });
                    } else {
                        console.log('doSearch:');
                        $scope.doSearch(undefined, 'editDevice').finally(function () {
                            $scope.editDevice();
                        });
                    }
                } else {
                    if (customColumnSetting) {
                        ColumnSettingService.applyChangedColumnsToGrid(customColumnSetting, false);
                    } else {
                        console.log('doSearch:');
                        $scope.doSearch();
                    }
                }
            });
        };

        $scope.init();
    }]);
});