'use strict';

define([
    'modules/common/services/device-agent',
    'modules/device/dialog/searchDeviceDlgController',
    'modules/device/dialog/advanceDeviceController',
    'modules/device/dialog/searchSlaveDeviceController',
    'modules/device/dialog/manageUsersInDeviceController',
    'modules/device/deviceInfoController',
    'modules/user/fingerprintScanService'
], function () {
    var app = angular.module('biostarWebApp');

    app.register.controller('DeviceTreeController', ['$scope', '$rootScope', '$q', 'API', 'Const', '$location', 'Util', 'DeviceAgent', '$timeout', '$compile', 'MessageProperties', 'MiniAgent', 'FingerprintScanService', 'MessageService', 'PrivilegeService', function ($scope, $rootScope, $q, API, Const, $location, Util, DeviceAgent, $timeout, $compile, MessageProperties, MiniAgent, FingerprintScanService, MessageService, PrivilegeService) {

//        console.log('GO!! LNBDeviceTree!!');
        $scope.groupId = 1;
        $scope.deviceMap = {};
        $scope.safeApply = function (fn) {
            var phase = this.$root.$$phase;
            if (phase == '$apply' || phase == '$digest') {
                if (fn && (typeof(fn) === 'function')) {
                    fn();
                }
            } else {
                this.$apply(fn);
            }
        };

        $scope.isDeviceMenu = function () {
            return ($location.path() == Const.MENU.DEVICE);
        };

        function clearMenuContext(divId) {
//            angular.element(divId).attr("class", "");
//            angular.element(divId).attr("style", "");
            angular.element(divId).find(".menu-item").remove();
        }

        function buildContextMenuDiv(contextMenuObject, ngIfCondition) {
            var html = '<div';
            if (ngIfCondition != undefined) {
                html += ' ng-if="' + ngIfCondition + '" ';
            }
            html += ' class="menu-item" style="height:31px;" ng-click="' + contextMenuObject.clickFunctionName + '()" ng-privilege><div class="menu-text">' + contextMenuObject.menuName + '</div></div>';
            return html;
        }

        // -------------------------- Tree 셋팅 관련 s --------------------------
        var isAndroid = (/android/gi).test(navigator.appVersion);
        $scope.initDeviceGroupTree = function () {

            angular.element('#devicegrouptree').tree({
                lines: false,
                animate: true,
                dnd: !isAndroid,
                onClick: function (node) {// 선택한 node값 search로 치환
//                    console.log('onClick');
                },
                onBeforeSelect: function (node) {
//                    console.log('onBeforeSelect');
                },
                onSelect: function (node) {
//                    console.log('onSelect');

                    if ($scope.isDeviceMenu()) {
                        var group_id = node.id;
                        if (node.isDevice) {
//                        Util.beforeUrl($location.url());
//                        $scope.$apply($location.url('/device/edit').search({id: node.id, index: 0}));//?id=' + rowData.id));
                            DeviceAgent.setSelectedDevice({'id': node.id, 'name': node.text, 'device_type_id': node.device_type_id});
                            $rootScope.$emit('EditDeviceInfo');
                        } else if ($scope.groupId != node.id) {
                            $scope.groupName = node.text;
                            $scope.groupId = node.id;

                            if (1 === +group_id) {
                                group_id = undefined;
                            }
                            $rootScope.$emit('ChangeGroupName', {'groupName': node.text, 'groupId': node.id});
                            $rootScope.$emit('clickDeviceGroupItemOnTree', group_id);
                        }
                    }
                },
                onDblClick: function (node) {
                    if (node.id == '1' && !node.isDevice) {
                        return false;
                    }
                    angular.element('#' + node.domId).parents('.tree').tree('beginEdit', node.target);
                    angular.element('#' + node.domId).find('input.tree-editor').focus()[0].setSelectionRange(0, node.text.length);
                },
                onBeforeLoad: function (node) {
//                    console.log('onBeforeLoad');
//                    console.log(node);
                },
                onLoadSuccess: function (node, data) {
//                    console.log('onLoadSuccess');
//                    console.log(node);
//                    console.log(data);
                },
                onExpand: function (node) {
//                    console.log('onExpand');

                },
                onBeforeEdit: function (node) {
//                    console.log('onBeforeEdit');

                },
                onAfterEdit: function (node) {
//                    console.log('onAfterEdit');
                    if (String.prototype.trim) {
                        node.text = node.text.trim();
                    }
                    if (node.text == undefined || node.text == '') {
                        MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseDeviceGroupName'));
                        DeviceAgent.init();
                        $rootScope.$broadcast('onRefreshDeviceList');
                        if (node.id === $scope.groupId) {
                            $rootScope.$emit('ChangeGroupName', {'groupName': node.name, 'groupId': node.id});
                        }
                        angular.element('#' + node.domId).parents('.tree').tree('update', {
                            target: node.target,
                            text: node.name,
                            name: node.name
                        });
                        return false;
                    }
                    API.put('/device_groups/' + node.id, undefined, {
                        'DeviceGroup': {'name': node.text}  //TODO: '<' 문자를 변환, Util.replaceTagFilter(node.text)}
                    }).then(function success(data) {
                        DeviceAgent.init();
                        $rootScope.$broadcast('onRefreshDeviceList');
                        if (node.id === $scope.groupId) {
                            $rootScope.$emit('ChangeGroupName', {'groupName': node.text, 'groupId': node.id});
                        }
                        angular.element('#' + node.domId).parents('.tree').tree('update', {
                            target: node.target,
                            text: node.text,
                            name: node.text
                        });
                    }, function error(data) {
                        console.log('/device_groups/' + node.id + '::error');
                        DeviceAgent.init();
                        $rootScope.$broadcast('onRefreshDeviceList');
                        if (node.id === $scope.groupId) {
                            $rootScope.$emit('ChangeGroupName', {'groupName': node.name, 'groupId': node.id});
                        }

                        angular.element('#' + node.domId).parents('.tree').tree('update', {
                            target: node.target,
                            text: node.name,
                            name: node.name
                        });
                    }).finally(function (data) {
                        angular.element('.leftTreeMenu').scrollLeft(0);
                    });
                },
                onBeforeDrag: function (node) {
//                    console.log('onBeforeDrag');
                    if (!$scope.isDeviceMenu()) {
                        return false;
                    }
                    var p_node = angular.element(node.target).parents('.tree').tree('getParent', node.target);
                    if ((p_node && p_node.isDevice)) {
                        return false;
                    }
                    if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.DEVICE)) {
                        return false;
                    }
                },
                onStopDrag: function (node) {
//                    console.log('onStopDrag');
                    angular.element(node.target).removeAttr('style');
                },
                onDragOver: function (target, source) {
                    var node = angular.element(target).parents('.tree').tree('getNode', target);
                    // INFO: Device 에 자식 추가(RS485 Slave) 는 DnD 으로 추가할 수 없다. 우클릭 Search Slave Device 메뉴 사용
                    if (node.isDevice) {
                        return false;
                    }
                },
                onBeforeDrop: function (target, source, point) {
//                    console.log('onBeforeDrop');
                    if (source.isWaiting != undefined && source.isWaiting == true &&
                        angular.element(source.target).parents('.tree').attr('id') != angular.element(target).parents('.tree').attr('id')) {
                        /**
                         * Device->Server 장치 승인 Waiting 에서 추가된 장치
                         *
                         */
                        var target_node = angular.element(target).parents('.tree').tree('getNode', target);
                        if (target_node.isDevice) {
                            // INFO: Device 에 자식 추가( RS485 Slave) 는 DnD 으로 추가할 수 없다. 우클릭 Search Slave Device 메뉴 사용
                            return false;
//                            target_node = angular.element(target).parents('.tree').tree('getParent', target);
                        }
                        var sourceRootNode = angular.element(source.target).parents('.tree').tree('getRoot');
                        if (sourceRootNode.id == source.id) {
                            /**
                             * Waiting Device 전체를 등록한다.
                             */
                            var addNodes = angular.element(source.target).parents('.tree').tree('getChildren', source.target);
                            var addDevices = [];
                            for (var i = 0; i < addNodes.length; i++) {
                                addNodes[i].deviceData['device_group_id'] = {id: target_node.id};
                                addDevices.push(addNodes[i].deviceData);
                            }
                            API.post('devices', undefined, {'DeviceCollection': {'total': addDevices.length, 'rows': addDevices}}).then(function (data) {
//                                console.log('Add Device Success');
                                $rootScope.$broadcast('onRefreshDeviceList');
                                $scope.refreshDeviceTreeData();

                                // 등록이 성공된 장치는 Waiting Tree에서 삭제
                                if (angular.isDefined(data.DeviceCollection) && angular.isDefined(data.DeviceCollection.rows)) {
                                    for (var i = data.DeviceCollection.rows.length - 1; i >= 0; i--) {
                                        var addedNode = angular.element(source.target).parents('.tree').tree('find', data.DeviceCollection.rows[i].id);
                                        angular.element(source.target).parents('.tree').tree('remove', addedNode.target);
                                    }
                                }
                                // 부분만 등록 성공한 경우 안내창 띄움
                                if (data.Response && data.Response.code == '1') {
                                    MessageService.displayWarningMessage(data.Response.message);
                                }
                            });
                            // [중요] Tree Node 자체를 옮기는 것은 아니므로 DnD에 대하여 false 를 리턴한다.
                            return false;
                        } else {
                            /**
                             * Waiting Device 단건을 등록한다.
                             */
                            source.deviceData.device_group_id = {id: target_node.id};
                            API.post('devices', undefined, {'Device': source.deviceData}).then(function (data) {
//                                console.log('Add Device Success');
                                var beforeNode;
                                for (var i = 0; i < target_node.children.length; i++) {
                                    if (target_node.children[i].isDevice == false) {
                                        beforeNode = angular.element('#devicegrouptree').tree('find', target_node.children[i].id);
                                        break;
                                    }
                                }
                                var findDevice = angular.element('#treeWaitingDevice').tree('find', source.id);
                                var p_node = angular.element('#treeWaitingDevice').tree('pop', findDevice.target);
                                p_node.iconCls = p_node.iconCls.replace('waiting', 'disconnect');
                                if (beforeNode) {
                                    angular.element('#devicegrouptree').tree('insert', {
                                        before: beforeNode.target,
                                        data: p_node
                                    });
                                } else {
                                    angular.element('#devicegrouptree').tree('append', {
                                        parent: target_node.target,
                                        data: [p_node]
                                    });
                                }
                                angular.element('#devicegrouptree').tree('expand', target_node.target);
                                $rootScope.$broadcast('onRefreshDeviceList');
                                $scope.refreshDeviceTreeData();
                            });
                            return false;
                        }
                    } else if (angular.element(source.target).parents('.tree').attr('id') == angular.element(target).parents('.tree').attr('id')) {
                        /**
                         * 장치를 다른 위치로 옮기거나 그룹을 옮긴 경우
                         */
                        var target_node = angular.element(target).parents('.tree').tree('getNode', target);

                        if (source.isDevice) {
                            if (target_node.isDevice) {
                                // INFO: Device 에 자식 추가( RS485 Slave) 는 DnD 으로 추가할 수 없다. 우클릭 Search Slave Device 메뉴 사용
                                return false;
                            }

                            if (target_node == angular.element(source.target).parents('.tree').tree('getParent', source.target)) {
                                return false;
                            }

                            API.put('devices/' + source.id, undefined, {
                                'Device': {'device_group_id': {id: target_node.id}}
                            }).then(function (data) {
//                                console.log('Saved');
                                var beforeNode;
                                for (var i = 0; i < target_node.children.length; i++) {
                                    if (target_node.children[i].isDevice == false) {
                                        beforeNode = angular.element('#devicegrouptree').tree('find', target_node.children[i].id);
                                        break;
                                    }
                                }
                                var p_node = angular.element('#devicegrouptree').tree('pop', source.target);
                                if (beforeNode) {
                                    angular.element('#devicegrouptree').tree('insert', {
                                        before: beforeNode.target,
                                        data: p_node
                                    });
                                } else {
                                    angular.element('#devicegrouptree').tree('append', {
                                        parent: target_node.target,
                                        data: [p_node]
                                    });
                                }
                                angular.element('#devicegrouptree').tree('expand', target_node.target);
                                $rootScope.$broadcast('onRefreshDeviceList');
                                DeviceAgent.init();
                            });
                            return false;
                        } else {
                            if (target_node.isDevice) {
                                target_node = angular.element(target).parents('.tree').tree('getParent', target);
                            }

                            /**
                             // TODO: 그룹의 단계가 8단계를 넘어가는지 확인해야 함
                             * 선택된 source 의 자식 그룹의 단계와
                             * target 의 부모 그룹의 단계를 모두 합친 값으로 비교해야 함
                             * TODO: 도착 그룹의 형제들 중 같은 이름을 가진 그룹이 있는지 확인해야 함
                             */
                            var p_depth = angular.element(target).parents('ul:not(.tree)').length;
                            var c_depth = angular.element(source.target).parentsUntil('ul').find('ul').length;
                            if (p_depth + c_depth >= 8) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.deviceGroupCanNotExceedSteps'));
                                return false;
                            }

                            for (var i = 0; i < target_node.children.length; i++) {
                                if (source.name == target_node.children[i].name) {
                                    MessageService.displayWarningMessage(MessageProperties.get('validate.duplicateDeviceGroup'));
                                    return false;
                                }
                            }

                            API.put('/device_groups/' + source.id, undefined, {
                                'DeviceGroup': {'parent_id': {id: target_node.id}}
                            }).then(function (data) {
//                                console.log(data);
                                DeviceAgent.init();
                            }, function (data) {
                                $rootScope.$broadcast('onRefreshDeviceList');
                                $scope.refreshDeviceTreeData();
                            });
                            //                    return false;
                        }
                    }
                },
//          loadFilter: function (rows) {
//              console.log('loadFilter');
//              console.log(rows);
//              return rows;
//          },
                onDrop: function (target, source, point) {
//                    console.log('onDrop');
                },
                onContextMenu: function (event, node) {
//                    console.log('onContextMenu');
                    if (event.preventDefault) {
                        event.preventDefault();
                    }

                    if (!$scope.isDeviceMenu()) {
                        return false;
                    }

                    if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.DEVICE)) {
                        return false;
                    }

                    angular.element("#deviceContextMenu").menu("hide");
                    var rootNode = angular.element(node.target).parents('.tree').tree('getRoot');
                    var parentNode = angular.element(node.target).parents('.tree').tree('getParent', node.target);

                    if (node.isDevice && node.rs485 == undefined) {
                        var deviceData = $scope.deviceMap[node.id];
                        if (deviceData == undefined) {
                            deviceData = DeviceAgent.getAllDevicesMap()[node.id];
                        }
                        if (deviceData && deviceData.rs485) {
                            node.rs485 = deviceData.rs485;
                        }
                    }
                    if (node.isDevice && (node.lan == undefined || node.lan.connection_mode == undefined)) {
                        var deviceData = $scope.deviceMap[node.id];
                        if (deviceData == undefined) {
                            deviceData = DeviceAgent.getAllDevicesMap()[node.id];
                        }
                        if (deviceData && deviceData.lan && deviceData.lan.connection_mode) {
                            node.lan = deviceData.lan;
                        } else if (node.deviceData != undefined) {
                            deviceData = node.deviceData;
                            if (deviceData.lan && deviceData.lan.connection_mode) {
                                node.lan = deviceData.lan;
                            }
                        }
                    }
                    var menuID = '#deviceContextMenu';
                    clearMenuContext(menuID);
//                    console.log(node);

                    if (node.id == rootNode.id) {
                        // Root Node 인 경우
                        angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.ADD_DEVICE_GROUP));
                    } else if (node.isDevice) {
                        if (node.parent_node_id != undefined) {
                            if (node.device_type_id.id != Const.DEVICE_TYPE.SECURE_IO_2) {
                                angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.REBOOT));
                            }
                            angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.DELETE_DEVICE));
                        } else {
                            if (node.rs485.mode == Const.RS485_MODE.MASTER) {
                                angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.SEARCH_SLAVE_DEVICE));
                            }
                            if (node.lan.connection_mode != Const.CONNECTION_MODE.DEVICE_TO_SERVER) {
                                angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.RECONNECT));
                            }
                            angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.MANAGE_USERS_IN_DEVICE));
                            if (node.rs485.mode != Const.RS485_MODE.SLAVE) {
                                angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.SYNC_DEVICE));
                            }
                            if (Util.getLocalStorageObject(Const.LOCAL_STORAGE_DEV_ADMIN_KEY) != undefined) {
                                angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.FULL_SYNC_DEVICE));
                            }
                            if (node.device_type_id.id != Const.DEVICE_TYPE.SECURE_IO_2) {
                                angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.REBOOT));
                            }
                            angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.DELETE_DEVICE));
                        }
                    } else {
                        angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.ADD_DEVICE_GROUP));
                        angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.RENAME_DEVICE_GROUP));
                        angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.DELETE_DEVICE_GROUP));

                    }

                    angular.element(menuID).css("height", "auto");
                    angular.element(menuID).css("width", "auto");

                    $compile(angular.element(menuID))($scope);

                    angular.element(menuID).menu('options').ownerTreeNode = node;
                    angular.element(menuID).menu('show', {
                        left: event.pageX,
                        top: event.pageY
                    });
                },
                formatter: function (node) {
                    if (node.id == Const.DEFAULT_ALL_GROUP_ID) {
                        node.name = MessageProperties.get("menu.device.allDeviceGroups");
                    }
                    return '<span title="' + node.name + '">' + node.name + '</span>';
                }
            });

            angular.element('#devicegrouptree').on('keydown', 'input.tree-editor', function (event) {
                //console.log('keyCode:' + event.keyCode);
//                if (event.keyCode == 53) {
//                    // INFO: KeyCode: 53 = '%'
//                    return false;
//                } else if (event.keyCode == 188) {
//                    // INFO: KeyCode: 188 = '<'
//                    return false;
//                }
            });
        };

        $scope.initWaitingDeviceTree = function () {
            var isAndroid = (/android/gi).test(navigator.appVersion);
            angular.element('#treeWaitingDevice').tree({
                data: [
                    {
                        'id': '1',
                        'name': MessageProperties.get("menu.device.waitingDevice"),
                        'text': MessageProperties.get("menu.device.waitingDevice"),
                        'description': '',
                        'isWaiting': true,
                        'depth': '0',
                        'devices': [],
                        'device_groups': []
                    }
                ],
                lines: false,
                animate: true,
                dnd: !isAndroid,
                onLoadSuccess: function (node) {

                },
                onBeforeSelect: function (node) {
                    return false;
                },
                onBeforeDrag: function (node) {
//                    console.log('Waiting: onBeforeDrag');
                    var rootNode = angular.element(node.target).parents('.tree').tree('getRoot');
                    if (rootNode.id == node.id && angular.element(node.target).parents('.tree').tree('isLeaf', node.target)) {
//                        console.log('Waiting: onBeforeDrag:return false');
                        return false;
                    }
                    if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.DEVICE)) {
                        return false;
                    }
                },
                onStartDrag: function (node) {
//                    console.log('Waiting: onStartDrag');
                },
                onStopDrag: function (node) {
//                    console.log('Waiting: onStopDrag');
                    if (node == undefined) {
                        return;
                    }
                    var rootNode = angular.element(node.target).parents('.tree').tree('getRoot');
                    if (rootNode.id == node.id) {
                        angular.element(node.target).removeAttr('style');
                    }
                },
                onDragEnter: function (target, source) {
//                    console.log('Waiting: onDragEnter');
                    return false;
                },
                onDragOver: function (target, source) {
//                    console.log('Waiting: onDragOver');
                },
                onDragLeave: function (target, source) {
//                    console.log('Waiting: onDragLeave');
                },
                onBeforeDrop: function (target, source, point) {
//                    console.log('Waiting: onBeforeDrop');
                },
                onDrop: function (target, source, point) {
//                    console.log('Waiting: onDrop');
                },
                onClick: function (node) {
                    if (angular.element('#treeWaitingDevice').tree('isLeaf', node.target)) {

                    } else {

                    }
                },
                onContextMenu: function (event, node) {
//                    console.log('onContextMenu Waiting');
                    if (event.preventDefault) {
                        event.preventDefault();
                    }
                    if (!$scope.isDeviceMenu()) {
                        return false;
                    }
                    if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.DEVICE)) {
                        return false;
                    }
                    var menuID = '#deviceContextMenu';
                    var rootNode = angular.element('#treeWaitingDevice').tree('getRoot');
                    clearMenuContext(menuID);

                    if (rootNode.id == node.id) {
                        if (angular.element('#treeWaitingDevice').tree('isLeaf', node.target)) {
                            console.log('Waiting Device 없으므로 메뉴 표시 안함.');
                            return false;
                        } else {
                            angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.ADD_ALL_WAITING_DEVICES));
                        }
                    } else {
                        angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.ADD_WAITING_DEVICE));
                    }

                    angular.element(menuID).css("height", "auto");
                    angular.element(menuID).css("width", "auto");

                    $compile(angular.element(menuID))($scope);

                    angular.element(menuID).menu('options').ownerTreeNode = node;
                    angular.element(menuID).menu('show', {
                        left: event.pageX,
                        top: event.pageY
                    });
                }
            });
        };

        $scope.initUSBDeviceTree = function () {
            angular.element('#treeUSBDevice').tree({
                data: [
                    {
                        'id': '0',
                        'name': MessageProperties.get("menu.device.usbDevice"),
                        'text': MessageProperties.get("menu.device.usbDevice"),
                        'children': []
                    }
                ],
                lines: false,
                animate: true,
                onLoadSuccess: function (node, data) {
                },
                onBeforeSelect: function (node) {
                    return false;
                },
                onExpand: function () {
                },
                onClick: function (node) {// 선택한 node값 search로 치환
                    if (angular.element('#treeUSBDevice').tree('isLeaf', node.target)) {

                    } else {

                    }
                },
                onContextMenu: function (event, node) {
//                    console.log('onContextMenu Waiting');
                    if (event.preventDefault) {
                        event.preventDefault();
                    }
                    if (!$scope.isDeviceMenu()) {
                        return false;
                    }
                    if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.DEVICE)) {
                        return false;
                    }
                    var menuID = '#deviceContextMenu';
                    clearMenuContext(menuID);
                    var rootNode = angular.element('#treeUSBDevice').tree('getRoot');

                    angular.element(menuID).append(buildContextMenuDiv(Const.DEVICE_CONTEXT_MENU.SEARCH_USB_DEVICE));

                    angular.element(menuID).css("height", "auto");
                    angular.element(menuID).css("width", "auto");

                    $compile(angular.element(menuID))($scope);

                    if (rootNode.id == node.id) {
                        angular.element(menuID).menu('options').ownerTreeNode = node;
                        angular.element(menuID).menu('show', {
                            left: event.pageX,
                            top: event.pageY
                        });
                    }
                }
            });
        };
        // -------------------------- Tree 셋팅 관련 e --------------------------

        // -------------------------- Tree Context Menu 관련 s --------------------------
        $scope.searchSlaveDevice = function (node) {
            angular.element('#deviceContextMenu').menu('hide');
            if (node == null) {
                var target = event.target || (event.srcElement || document);
                var menuID = angular.element(target).parents('.menu-top').attr('id');
                node = angular.element('#' + menuID).menu('options').ownerTreeNode;
                if (node == null) {
                    console.log('장치를 찾을 수 없습니다.');
                    return false;
                }
            }

            console.log('Search Slave Device: ' + node.id);
            $rootScope.$emit('searchSlaveDevice', {id: node.id, name: node.text});
        };

        $scope.deleteDevice = function (node) {
            angular.element('#deviceContextMenu').menu('hide');
            if (node == null) {
                var target = event.target || (event.srcElement || document);
                var menuID = angular.element(target).parents('.menu-top').attr('id');
                node = angular.element('#' + menuID).menu('options').ownerTreeNode;
                if (node == null) {
                    console.log('장치를 찾을 수 없습니다.');
                    return false;
                }
            }

//            console.log('DeleteDevice');
            MessageService.displayConfirmMessage(MessageProperties.get("confirm.delete"), function (dialogResult) {
                if (dialogResult) {
                    API.del('devices/' + node.id).then(function (data) {
                        angular.element('#devicegrouptree').tree('remove', node.target);
                        $rootScope.$emit('onRefreshDeviceList');
                        DeviceAgent.init();
                    });
                }
            });
        };

        $scope.reconnectDevice = function (node) {
            angular.element('#deviceContextMenu').menu('hide');
            if (node == null) {
                var target = event.target || (event.srcElement || document);
                var menuID = angular.element(target).parents('.menu-top').attr('id');
                node = angular.element('#' + menuID).menu('options').ownerTreeNode;
                if (node == null) {
                    console.log('장치를 찾을 수 없습니다.');
                    return false;
                }
            }

            console.log('ReconnectDevice');
            if (node.iconCls.indexOf('disconnect') < 0) {
                API.post('devices/' + node.id + '/disconnect', undefined, {id: node.id}).then(function (data) {

                    $timeout(function () {
                        API.post('devices/' + node.id + '/connect', undefined, {id: node.id}).then(function (data) {
                            console.log('connect');
                        });
                    }, 5000);
                });
            } else {
                API.post('devices/' + node.id + '/connect', undefined, {id: node.id}).then(function (data) {
                    console.log('connect');
                });
            }
        };

        $scope.rebootDevice = function (node) {
            angular.element('#deviceContextMenu').menu('hide');
            if (node == null) {
                var target = event.target || (event.srcElement || document);
                var menuID = angular.element(target).parents('.menu-top').attr('id');
                node = angular.element('#' + menuID).menu('options').ownerTreeNode;
                if (node == null) {
                    console.log('장치를 찾을 수 없습니다.');
                    return false;
                }
            }

            console.log('rebootDevice');
            var param = {
                DeviceCollection: {
                    rows: [
                        {
                            id: node.id
                        }
                    ],
                    total: 1
                }
            };
            API.post('devices/reset', undefined, param).then(function (data) {
                // reboot sucessful
            });
        };

        $scope.manageUsersinDevice = function (node) {
            angular.element('#deviceContextMenu').menu('hide');
            if (node == null) {
                var target = event.target || (event.srcElement || document);
                var menuID = angular.element(target).parents('.menu-top').attr('id');
                node = angular.element('#' + menuID).menu('options').ownerTreeNode;
                if (node == null) {
                    console.log('장치를 찾을 수 없습니다.');
                    return false;
                }
            }

//            console.log('Manage Users in Device');
            $rootScope.$emit('manageUsersinDevice', {id: node.id, name: node.text});
        };

        $scope.manageUsersinSlaveDevice = function (node) {
            angular.element('#deviceContextMenu').menu('hide');
            if (node == null) {
                var target = event.target || (event.srcElement || document);
                var menuID = angular.element(target).parents('.menu-top').attr('id');
                node = angular.element('#' + menuID).menu('options').ownerTreeNode;
                var parentNode = angular.element(node.target).parents('.tree').tree('getParent', node.target);
                if (parentNode == null) {
                    console.log('장치를 찾을 수 없습니다.');
                    return false;
                }
            }

//            console.log('Manage Users in Slave Device');
            $rootScope.$emit('manageUsersinDevice', {id: parentNode.id, name: parentNode.text});
        };

        $scope.addDeviceGroup = function (node) {
            angular.element('#deviceContextMenu').menu('hide');
            if (node == null) {
                var target = event.target || (event.srcElement || document);
                var menuID = angular.element(target).parents('.menu-top').attr('id');
                node = angular.element('#' + menuID).menu('options').ownerTreeNode;
                if (node == null) {
                    console.log('장치를 찾을 수 없습니다.');
                    return false;
                }
            }
//            console.log('addDeviceGroup');

            var depth = angular.element(node.target).parents('ul:not(.tree)').length;
            if (depth > 7) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.deviceGroupCanNotExceedSteps'));
                return;
            }
            var defaultGroupName = MessageProperties.get("menu.device.newDeviceGroup");
            var arrContainDefaultName = angular.element('#' + node.domId).parents('.tree').find('.tree-title').contents().filter(function () {
                var self = this;
                return (self.textContent.indexOf(defaultGroupName) > -1);
            });

            var maxValue = 1;
            var rexDigit = /[\d]+/;
            angular.forEach(arrContainDefaultName, function (defaultName) {
                var matches = rexDigit.exec(defaultName.textContent || '') || [];
//                console.log('maxValue: ' + maxValue);
                if (matches && (matches[0] - '') >= maxValue) {
                    maxValue = ((matches[0] - '') + 1);
                }
//                console.log('maxValue: ' + maxValue);
            });


            API.post('device_groups', undefined, {
                'DeviceGroup': {
                    'name': defaultGroupName + ' ' + maxValue,
                    'description ': 'description',
                    'parent_id': {id: node.id}
                }
            }).then(function (data) {
                DeviceAgent.clearDeviceGroupWithDevice();
                var tree = angular.element('#' + node.domId).parents('.tree');
                var n_node = {
                    'parent': node.target,
                    'data': [
                        {
                            text: data.DeviceGroup.name,
                            name: data.DeviceGroup.name,
                            id: data.DeviceGroup.id,
                            isDevice: false,
                            state: 'open',
                            iconCls: '',
                            children: []
                        }
                    ]
                };
                tree.tree('append', n_node);
                n_node = tree.tree('find', data.DeviceGroup.id);
                if (node.state == 'closed') {
                    tree.tree('expand', node.target);
                    tree.tree('scrollTo', n_node.target);
                }
                tree.tree('beginEdit', n_node.target);
                angular.element(n_node.target).find('input.tree-editor').focus()[0].setSelectionRange(0, data.DeviceGroup.name.length);
            });
        };

        $scope.renameDeviceGroup = function (node) {
            angular.element('#deviceContextMenu').menu('hide');
            if (node == null) {
                var target = event.target || (event.srcElement || document);
                var menuID = angular.element(target).parents('.menu-top').attr('id');
                node = angular.element('#' + menuID).menu('options').ownerTreeNode;
                if (node == null) {
                    console.log('장치를 찾을 수 없습니다.');
                    return false;
                }
            }
//            console.log('renameDeviceGroup');

            angular.element('#' + node.domId).parents('.tree').tree('beginEdit', node.target);
            angular.element('#' + node.domId).find('input.tree-editor').focus()[0].setSelectionRange(0, node.text.length);
        };

        $scope.deleteDeviceGroup = function (node) {
            angular.element('#deviceContextMenu').menu('hide');
            if (node == null) {
                var target = event.target || (event.srcElement || document);
                var menuID = angular.element(target).parents('.menu-top').attr('id');
                node = angular.element('#' + menuID).menu('options').ownerTreeNode;
                if (node == null) {
                    console.log('장치를 찾을 수 없습니다.');
                    return false;
                }
            }
//            console.log('deleteDeviceGroup');
            MessageService.displayConfirmMessage(MessageProperties.get("confirm.delete"), function (dialogResult) {
                if (dialogResult) {
                    API.delete('device_groups/' + node.id).then(function (data) {
                        var selectedNode = angular.element('#devicegrouptree').tree('getSelected');
                        if (selectedNode == undefined) {
                            $rootScope.$emit('clickDeviceGroupItemOnTree');
                        } else if (selectedNode.id == node.id) {
                            var parentNode = angular.element('#devicegrouptree').tree('getParent', node.target);
                            angular.element('#devicegrouptree').tree('select', parentNode.target);
                        } else {
                            var group_id = selectedNode.id;
                            $scope.groupName = selectedNode.text;
                            $scope.groupId = selectedNode.id;

                            if (1 === +group_id) {
                                group_id = undefined;
                            }
                            $rootScope.$emit('ChangeGroupName', {'groupName': $scope.groupName, 'groupId': $scope.groupId});
                            $rootScope.$emit('clickDeviceGroupItemOnTree', group_id);
                        }
                        angular.element('#' + node.domId).parents('.tree').tree('remove', node.target);
                        DeviceAgent.init();

                    });
                }
            });
        };

        $scope.addWaitingDevice = function (node) {
            angular.element('#deviceContextMenu').menu('hide');
            if (node == null) {
                var target = event.target || (event.srcElement || document);
                var menuID = angular.element(target).parents('.menu-top').attr('id');
                node = angular.element('#' + menuID).menu('options').ownerTreeNode;
                if (node == null) {
                    console.log('장치를 찾을 수 없습니다.');
                    return false;
                }
            }
//            console.log('addWaitingDevice');

            var rootNode = angular.element(node.target).parents('.tree').tree('getRoot');
            if (rootNode.id == node.id) {
                var addNodes = angular.element(node.target).parents('.tree').tree('getChildren', node.target);
                var addDevices = [];
                for (var i = 0; i < addNodes.length; i++) {
                    addNodes[i].deviceData['device_group_id'] = {id: '1'};
                    addDevices.push(addNodes[i].deviceData);
                }
                API.post('devices', undefined, {'DeviceCollection': {'total': addDevices.length, 'rows': addDevices}}).then(function (data) {
//                    console.log('Add Device Success');
                    $rootScope.$emit('onRefreshDeviceList');
                    $scope.refreshDeviceTreeData();

                    if (angular.isDefined(data.DeviceCollection) && angular.isDefined(data.DeviceCollection.rows)) {
                        for (var i = data.DeviceCollection.rows.length - 1; i >= 0; i--) {
                            var addedNode = angular.element(node.target).parents('.tree').tree('find', data.DeviceCollection.rows[i].id);
                            angular.element(node.target).parents('.tree').tree('remove', addedNode.target);
                        }
                    }
                });
            } else {
                node.deviceData.device_group_id = {id: '1'};
                API.post('devices', undefined, {'Device': node.deviceData}).then(function (data) {
//                    console.log('Add Device Success');
                    $rootScope.$broadcast('onRefreshDeviceList');
                    $scope.refreshDeviceTreeData();
                    var beforeNode;
                    var target_node = angular.element('#devicegrouptree').tree('getRoot');
                    for (var i = 0; i < target_node.children.length; i++) {
                        if (target_node.children[i].isDevice == false) {
                            beforeNode = angular.element('#devicegrouptree').tree('find', target_node.children[i].id);
                            break;
                        }
                    }
//                    var findDevice = angular.element('#devicegrouptree').tree('find', source.id);
                    var p_node = angular.element(node.target).parents('.tree').tree('pop', node.target);
                    p_node.iconCls = p_node.iconCls.replace('waiting', 'disconnect');
                    if (beforeNode) {
                        angular.element('#devicegrouptree').tree('insert', {
                            before: beforeNode.target,
                            data: p_node
                        });
                    } else {
                        angular.element('#devicegrouptree').tree('append', {
                            parent: target_node.target,
                            data: [p_node]
                        });
                    }
                    angular.element('#devicegrouptree').tree('expand', target_node.target);
                });
            }
        };

        $scope.searchUSBDevice = function (node) {
            angular.element('#deviceContextMenu').menu('hide');
            if (node == null) {
                var target = event.target || (event.srcElement || document);
                var menuID = angular.element(target).parents('.menu-top').attr('id');
                node = angular.element('#' + menuID).menu('options').ownerTreeNode;
                if (node == null) {
                    console.log('장치를 찾을 수 없습니다.');
                    return false;
                }
            }
//            console.log('addWaitingDevice');

            var rootNode = angular.element(node.target).parents('.tree').tree('getRoot');
            if (rootNode.id == node.id) {
                var silent = false;
                $scope.bindUSBDeviceTreeData(silent);
            }
        };


        $scope.syncDevice = function (node) {
            angular.element('#deviceContextMenu').menu('hide');
            if (node == null) {
                var target = event.target || (event.srcElement || document);
                var menuID = angular.element(target).parents('.menu-top').attr('id');
                node = angular.element('#' + menuID).menu('options').ownerTreeNode;
                if (node == null) {
                    console.log('장치를 찾을 수 없습니다.');
                    return false;
                }
            }
//            DeviceAgent.syncDevices([
//                {
//                    id: node.id
//                }
//            ]);
            // BSS-774
            DeviceAgent.fullSyncDevices([
                {
                    id: node.id,
                    rs485: node.rs485
                }
            ]);
        };

        $scope.fullSyncDevice = function (node) {
            angular.element('#deviceContextMenu').menu('hide');
            if (node == null) {
                var target = event.target || (event.srcElement || document);
                var menuID = angular.element(target).parents('.menu-top').attr('id');
                node = angular.element('#' + menuID).menu('options').ownerTreeNode;
                if (node == null) {
                    console.log('장치를 찾을 수 없습니다.');
                    return false;
                }
            }
            DeviceAgent.fullSyncDevices([
                {
                    id: node.id,
                    rs485: node.rs485
                }
            ]);
        };

        $scope.initContextMenu = function () {
            $scope.isDisconnected = false;
            angular.element('#deviceContextMenu').menu({});

        };
        // -------------------------- Tree Context Menu 관련 e --------------------------


        // -------------------------- 상태정보 업데이트 관련 s --------------------------
        $scope.deviceStatusWaiting = function (device) {
            /**
             // TODO: Waiting Device Tree 에서 해당 ID의 Device 를 찾아 상태가 변경된 것인지 확인
             * 못찾았다면 Waiting Device Group 바로 아래에 추가
             * 찾았는데 상태가 변경되었다면 상태 아이콘 변경
             */
//            console.log('deviceStatusWaiting');

            if (angular.element('#treeWaitingDevice').length <= 0) {
                return;
            }
            var findDevice = angular.element('#treeWaitingDevice').tree('find', device.id);
            if (findDevice == undefined || findDevice == []) {
                var rootNode = angular.element('#treeWaitingDevice').tree('getRoot');
                if (angular.isUndefined(device.name)) {
                    device.name = '';
                    if ($scope.deviceSpec[device.device_type_id.id] != undefined) {
                        device.name += $scope.deviceSpec[device.device_type_id.id].name;
                    }
                    if (angular.isDefined(device.id)) {
                        device.name += ' ' + device.id;
                    }
                    if (angular.isDefined(device.lan) && angular.isDefined(device.lan.ip)) {
                        device.name += ' (' + device.lan.ip + ')';
                    }
                }
                var deviceNode = {
                    text: device.name,
                    name: device.name,
                    id: device.id,
                    isDevice: true,
                    isWaiting: true,
                    status: Const.DEVICE_STATUS.NORMAL,
                    deviceData: device,
                    iconCls: (angular.isDefined(device.device_type_id) ? Const.DEVICE_TYPE_ICON[device.device_type_id.id] + ' ' + Const.DEVICE_STATUS_ICON_CLASS.WAITING : Const.DEVICE_STATUS_ICON_CLASS.WAITING)
                };
                /**
                 *  ajax 호출은 비동기적으로 처리되므로 이미 트리에 장치가 추가되었을 가능성이 있다.(실제 발생됨)
                 *  따라서 노드에 붙이기 전에 다시 한번 확인한다.
                 */
                var refindDevice = angular.element('#treeWaitingDevice').tree('find', device.id) || angular.element('#devicegrouptree').tree('find', device.id);
                if (refindDevice == undefined || refindDevice == []) {
                    angular.element('#treeWaitingDevice').tree('append', {parent: rootNode.target, data: [deviceNode]});
                    //angular.element('#treeWaitingDevice').parents('.scrollX').show();
                }
            } else {
                angular.element(findDevice.target).find('span[class*="dv"]').removeClass(Const.DEVICE_STATUS_ICON_CLASS.DISCONNECTED).
                    removeClass(Const.DEVICE_STATUS_ICON_CLASS.SYNC_ERROR).removeClass(Const.DEVICE_STATUS_ICON_CLASS.NORMAL).addClass(Const.DEVICE_STATUS_ICON_CLASS.WAITING);
            }
        };
        var updateDeviceStatus = function (event, deviceObject) {
            if (deviceObject == null || deviceObject == undefined) {
                return;
            }
            if (angular.element('#devicegrouptree').length) {
                var node = angular.element('#devicegrouptree').tree('find', deviceObject.id);
                if (node) {
                    if (deviceObject.status != undefined) {

                        // ----------------------------- status 처리 s -----------------------------
                        $(node.target).find('.tree-icon').removeClass('disconnect').removeClass('sync_error');
                        if (angular.isDefined(node.device_type_id)) {
                            node.iconCls = Const.DEVICE_TYPE_ICON[node.device_type_id.id] + ' ' + Const.DEVICE_STATUS_ICON_CLASS[deviceObject.status];
                        } else if (node.deviceData && node.deviceData.device_type_id) {
                            // Waiting 장치 추가할 때 사용됨.
                            node.iconCls = Const.DEVICE_TYPE_ICON[node.deviceData.device_type_id.id] + ' ' + Const.DEVICE_STATUS_ICON_CLASS[deviceObject.status];
                        } else {
                            node.iconCls = Const.DEVICE_TYPE_ICON[6] + ' ' + Const.DEVICE_STATUS_ICON_CLASS[deviceObject.status];
                        }
                        // ----------------------------- status 처리 e -----------------------------

                    }
                    // ----------------------------- rs485 처리 s -----------------------------
                    if (angular.isDefined(deviceObject.rs485)) {
                        node.rs485 = deviceObject.rs485;

                        if (angular.isDefined(node.deviceData)) {
                            node.deviceData.rs485 = deviceObject.rs485;
                        }
                    }
                    // ----------------------------- rs485 처리 e -----------------------------

                    // ----------------------------- connection_mode 처리 s -----------------------------
                    if (angular.isDefined(deviceObject.connection_mode) ||
                        (angular.isDefined(deviceObject.lan) && angular.isDefined(deviceObject.lan.connection_mode))) {
                        var connection_mode = deviceObject.connection_mode || deviceObject.lan.connection_mode;
                        if (node.lan == undefined) {
                            node.lan = {};
                        }
                        node.lan.connection_mode = connection_mode;

                        if (angular.isDefined(node.deviceData)) {
                            if (node.deviceData.lan == undefined) {
                                node.deviceData.lan = {};
                            }
                            node.deviceData.lan.connection_mode = connection_mode;
                        }
                    }
                    // ----------------------------- connection_mode 처리 e -----------------------------

                    angular.element('#devicegrouptree').tree('update', {target: node.target, iconCls: node.iconCls});

                }
            }
        };

        // -------------------------- 상태정보 업데이트 관련 e --------------------------


        // -------------------------- Popup 관련 s --------------------------
        $scope.searchSlaveDevicePopupDlg = {url: undefined};
        $scope.afterOpenSearchSlaveDevicePopupDlg = function () {
            var options = {};
            options.modal = true;
            options.shadow = false;
            options.border = false;
            options.cache = false;
            options.title = MessageProperties.get('device.menu.searchSlaveDevice');
            options.width = 600;
            options.height = 'auto';
            options.addCount = 0;

            options.onClose = function () {
                $scope.safeApply(function () {
                    var opt = angular.element('#searchSlaveDevicePopupDlg').dialog('options');
                    if (opt.addCount != null && opt.addCount > 0) {
                        $scope.refreshDeviceTreeData();
                        $rootScope.$emit('AlarmDeviceAddMessage', opt.addCount);
                    }
                    $scope.searchSlaveDevicePopupDlg = {url: ''};
                    angular.element('#searchSlaveDevicePopupDlg').dialog('destroy');
                });
            };

            angular.element('#searchSlaveDevicePopupDlg').dialog(options);
        };

        $scope.manageUsersInDevicePopupDlg = {url: undefined};
        $scope.doOpenManageUsersInDevicePopup = function () {
            $scope.safeApply(function () {
                $scope.manageUsersInDevicePopupDlg = {url: undefined};
                $scope.manageUsersInDevicePopupDlg.url = 'modules/device/dialog/manageUsersInDevice.html?' + ((new Date).getTime());
            });
        };

        $scope.afterOpenManageUsersInDevicePopupDlg = function () {
            var options = {};
            options.modal = true;
            options.shadow = false;
            options.border = false;
            options.cache = false;
            options.title = MessageProperties.get('button.manageUsersInDevice');
            options.width = 760;
            options.height = 'auto';

            options.onOpen = function () {
                $rootScope.$emit(Const.ON_SEARCH_USERS_IN_DEVICE);
            };

            options.onClose = function () {
                $scope.safeApply(function () {
                    $scope.manageUsersInDevicePopupDlg = {url: ''};
                    angular.element('#manageUsersInDevicePopupDlg').dialog('destroy');
                });
            };

            angular.element('#manageUsersInDevicePopupDlg').dialog(options);
        };

        $scope.searchDevicePopupDlg = {url: undefined};
        $scope.doOpenSearchDevicePopup = function () {
            $scope.searchDevicePopupDlg = {url: undefined};
            $scope.searchDevicePopupDlg.url = 'modules/device/dialog/searchDeviceDlg.html?' + ((new Date).getTime());
        };

        $scope.afterOpenSearchDevicePopupDlg = function () {
            var options = {};
            options.iconCls = 'icon-save';
            options.modal = true;
            options.shadow = false;
            options.border = false;
            options.title = MessageProperties.get('device.searchDevice');
            options.width = 1000;
            options.height = 'auto';
            options.top = 150;
            options.addCount = 0;

            options.onClose = function () {
                $scope.safeApply(function () {
                    var opt = angular.element('#searchDevicePopupDlg').dialog('options');
                    if (opt.addCount != null && opt.addCount > 0) {
                        $scope.refreshDeviceTreeData();
                        $rootScope.$emit('AlarmDeviceAddMessage', opt.addCount);
                    }
                    $scope.searchDevicePopupDlg = {url: ''};
                    angular.element('#searchDevicePopupDlg').dialog('destroy');
                });
            };

            angular.element('#searchDevicePopupDlg').dialog(options);

        };


        $scope.advancedSearchDevicePopupDlg = {url: undefined};
        $scope.doOpenAdvancedSearchDevicePopup = function () {
            $scope.advancedSearchDevicePopupDlg = {url: undefined};
            $scope.advancedSearchDevicePopupDlg.url = 'modules/device/dialog/advanceDevice.html?' + ((new Date).getTime());
        };

        $scope.afterOpenAdvancedSearchDevicePopupDlg = function () {
            var options = {};
            options.iconCls = 'icon-save';
            options.closable = true;
            options.modal = true;
            options.shadow = false;
            options.border = false;
            options.title = MessageProperties.get('device.advancedSearch');
            options.width = 560;
            options.height = 'auto';
            options.addCount = 0;

            options.onClose = function () {
                $scope.safeApply(function () {
                    var opt = angular.element('#advancedSearchDevicePopupDlg').dialog('options');
                    if (opt.addCount != null && opt.addCount > 0) {
                        $scope.refreshDeviceTreeData();
                        $rootScope.$emit('AlarmDeviceAddMessage', opt.addCount);
                    }
                    $scope.advancedSearchDevicePopupDlg = {url: ''};
                    angular.element('#advancedSearchDevicePopupDlg').dialog('destroy');
                });
            };

            angular.element("#advancedSearchDevicePopupDlg").dialog(options);
        };
        // -------------------------- Popup 관련 e --------------------------

        // -------------------------- Tree Data 조회 및 바인딩 s --------------------------
        $scope.refreshDeviceTreeData = function () {
            DeviceAgent.clearDeviceGroupWithDevice();
            DeviceAgent.getDeviceGroupRecursiveDataDefer().then(function success(data) {
                $scope.deviceGroupTree = data;
                angular.element('#devicegrouptree').tree('loadData', [$scope.deviceGroupTree]);
            }).finally(function () {
                DeviceAgent.refreshAllDevicesMap().then(function (data) {
                    $scope.deviceMap = data;
                });
            });
        };
        $scope.bindDeviceTreeData = function () {
            DeviceAgent.getDeviceGroupRecursiveDataDefer().then(function success(data) {
                $scope.deviceGroupTree = data;
                angular.element('#devicegrouptree').tree('loadData', [$scope.deviceGroupTree]);
            });
        };
        $scope.bindWaitingDeviceTreeData = function () {
            API.get('devices/waiting').then(function (data) {
                if (data && data.DeviceCollection && data.DeviceCollection.rows && data.DeviceCollection.rows.length) {
                    var deviceTreeData = [];
                    for (var i = 0; i < data.DeviceCollection.rows.length; i++) {
                        var device = data.DeviceCollection.rows[i];
                        if (angular.isUndefined(device.name)) {
                            device.name = '';
                            if ($scope.deviceSpec[device.device_type_id.id] != undefined) {
                                device.name += $scope.deviceSpec[device.device_type_id.id].name;
                            }
                            if (angular.isDefined(device.id)) {
                                device.name += ' ' + device.id;
                            }
                            if (angular.isDefined(device.lan) && angular.isDefined(device.lan.ip)) {
                                device.name += ' (' + device.lan.ip + ')';
                            }
                        }
                        var iconClass = Const.DEVICE_STATUS_ICON_CLASS.WAITING;
                        if (angular.isDefined(device.device_type_id)) {
                            iconClass = Const.DEVICE_TYPE_ICON[device.device_type_id.id] + ' ' + iconClass;
                        }
                        var deviceNode = {
                            text: device.name,
                            name: device.name,
                            id: device.id,
                            isDevice: true,
                            isWaiting: true,
                            status: Const.DEVICE_STATUS.NORMAL,
                            deviceData: device,
                            lan: device.lan || {connection_mode: "1"},
                            iconCls: iconClass
                        };
                        deviceTreeData.push(deviceNode);
                    }

                    var rootNode = angular.element('#treeWaitingDevice').tree('getRoot');
                    angular.element('#treeWaitingDevice').tree('append', {parent: rootNode.target, data: deviceTreeData});
                }
            });
        };

        $scope.bindUSBDeviceTreeData = function (silent) {
            FingerprintScanService.connectCheckUSBDevice().then(function (data) {
                var responseData = angular.fromJson(data);
                if (responseData && responseData.retcode && responseData.retcode != -2) {
                    var rootNode = angular.element('#treeUSBDevice').tree('getRoot');
                    var USBDeviceNode = angular.element('#treeUSBDevice').tree('getChildren', rootNode.target);
                    if (USBDeviceNode.length <= 0) {
                        if (!silent) {
                            MessageService.displayWarningMessage(MessageProperties.get("device.usbDeviceFound"));
                        }
                        angular.element('#treeUSBDevice').tree('append', {
                            parent: rootNode.target,
                            data: [
                                {
                                    'id': '1',
                                    'name': 'USB Scanner',
                                    'text': 'USB Scanner',
                                    'iconCls': 'dvBiomini'
                                }
                            ]
                        });
                    } else {
                        if (!silent) {
                            MessageService.displayWarningMessage(MessageProperties.get("device.sameUsbDeviceFound"));
                        }
                    }
                } else {
                    // USB Agent 가 성공으로 응답했으나 responseData.retcode == -2 이면 USB 장치가 연결되지 않은 것이다.
                    if (!silent) {
                        MessageService.displayWarningMessage(MessageProperties.get("device.noUsbDeviceFound"));
                        $scope.initUSBDeviceTree();
                    }
                }
            }, function (error) {
                if (!silent) {
                    $scope.initUSBDeviceTree();
                }
            });

//
//            $scope.bs_session_id = $http.defaults.headers.common['bs-session-id'];
//            delete $http.defaults.headers.common['bs-session-id'];
//            $.ajax({
//                type: "POST",
//                url: Const.USBAGENT.BASE_URL + "Verify",
//                success: function (data) {
//                    $http.defaults.headers.common['bs-session-id'] = $scope.bs_session_id;
//                    if (data && data.retcode && data.retcode != '-1') {
//                        $.messager.alert('Information', 'USB 장치가 검색되었습니다.', 'information');
//                        var rootNode = angular.element('#treeUSBDevice').tree('getRoot');
//                        angular.element('#treeUSBDevice').tree('append', {
//                            parent: rootNode.target,
//                            data: [
//                                {
//                                    'id': '1',
//                                    'name': 'Bio Mini',
//                                    'text': 'Bio Mini',
//                                    'iconCls': 'dvBiomini'
//                                }
//                            ]
//                        });
//                    } else {
//                        $.messager.alert('Information', '검색된 USB 장치가 없습니다.', 'information');
//                        $scope.initUSBDeviceTree();
//                    }
//                },
//                error: function (xhr, status, error) {
//                    $http.defaults.headers.common['bs-session-id'] = $scope.bs_session_id;
//                    $.messager.alert('Information', '검색된 USB 장치가 없습니다.', 'information');
//                    $scope.initUSBDeviceTree();
//                }
//            });
//            MiniAgent.post('Verify').success(function (data) {
//                if (data && data.retcode && data.retcode != '-1') {
//                    $.messager.alert('Information', 'USB 장치가 검색되었습니다.', 'information');
//                    var rootNode = angular.element('#treeUSBDevice').tree('getRoot');
//                    angular.element('#treeUSBDevice').tree('append', {
//                        parent: rootNode.target,
//                        data: [
//                            {
//                                'id': '1',
//                                'name': 'Bio Mini',
//                                'text': 'Bio Mini',
//                                'iconCls': 'dvBiomini'
//                            }
//                        ]
//                    });
//                } else {
//                    $.messager.alert('Information', '검색된 USB 장치가 없습니다.', 'information');
//                    $scope.initUSBDeviceTree();
//                }
//            }).error(function () {
//                $.messager.alert('Information', '검색된 USB 장치가 없습니다.', 'information');
//                $scope.initUSBDeviceTree();
//            });
        };
        // -------------------------- Tree Data 조회 및 바인딩 e --------------------------

        $scope.deviceSpec = undefined;
        $scope.init = function () {

            $scope.initContextMenu();
            $scope.initDeviceGroupTree();
            DeviceAgent.getDeviceGroupRecursiveDataDefer().then(function success(data) {
                $scope.deviceGroupTree = data;
                angular.element('#devicegrouptree').tree('loadData', [$scope.deviceGroupTree]);
            }).finally(function () {
                if (Util.isEmpty($scope.deviceMap)) {
                    DeviceAgent.refreshAllDevicesMap().then(function (data) {
                        $scope.deviceMap = data;
                    });
                }
            });


            if (angular.element('#treeWaitingDevice').length > 0) {
                $scope.initWaitingDeviceTree();
                DeviceAgent.getDeviceSpec().then(function success(data) {
                    $scope.deviceSpec = data;
                }).finally(function () {
                    $scope.bindWaitingDeviceTreeData();
                });
            }

            if (angular.element('#treeUSBDevice').length > 0) {
                $scope.initUSBDeviceTree();
                var silent = true;
                $scope.bindUSBDeviceTreeData(silent);
            }

            if ($rootScope.$$listeners[Const.DEVICE_CONTEXT_MENU.RECONNECT] != undefined) {
                $rootScope.$$listeners[Const.DEVICE_CONTEXT_MENU.RECONNECT] = undefined;
            }
            $rootScope.$on(Const.DEVICE_CONTEXT_MENU.RECONNECT, function (event, message) {
                if (message == null || angular.isUndefined(message.id)) {
                    return;
                }
                var treeNode = angular.element('#devicegrouptree').tree('find', message.id);

                $scope.reconnectDevice(treeNode);
            });

            if ($rootScope.$$listeners['searchSlaveDevice'] != undefined) {
                $rootScope.$$listeners['searchSlaveDevice'] = undefined;
            }
            $rootScope.$on('searchSlaveDevice', function (event, message) {
                if (message == null || angular.isUndefined(message.id)) {
                    return;
                }
                sessionStorage.setItem('selectedDeviceNode', angular.toJson({id: message.id, name: message.name}));
                $scope.safeApply(function () {
                    $scope.searchSlaveDevicePopupDlg.url = 'modules/device/dialog/searchSlaveDevice.html?' + ((new Date).getTime());
                });
            });

            if ($rootScope.$$listeners['manageUsersinDevice'] != undefined) {
                $rootScope.$$listeners['manageUsersinDevice'] = undefined;
            }
            $rootScope.$on('manageUsersinDevice', function (event, message) {
                if (message == null || angular.isUndefined(message.id)) {
                    return;
                }
                sessionStorage.setItem('selectedDeviceNode', angular.toJson({id: message.id, name: message.name}));
                $scope.doOpenManageUsersInDevicePopup();
            });

            if ($rootScope.$$listeners['SelectedDeviceTreeItem'] != undefined) {
                $rootScope.$$listeners['SelectedDeviceTreeItem'] = undefined;
            }
            $rootScope.$on('SelectedDeviceTreeItem', function (event, cbFunction) {
                var selectedTreeItem = angular.element('#devicegrouptree').tree('getSelected');
                cbFunction(selectedTreeItem);
            });

            if ($rootScope.$$listeners['reloadDeviceTree'] != undefined) {
                $rootScope.$$listeners['reloadDeviceTree'] = undefined;
            }
            $rootScope.$on('reloadDeviceTree', function (event, deviceTreeData) {
                console.log('reloadDeviceTree');
                $scope.refreshDeviceTreeData();
            });

            /**
             * 장치로부터 WebSocket을 통해 받은 장치의 최신 상태를 업데이트 한다.
             *  TODO: 중복등록 방지코드의 개선 필요. 해당 Controller 에서 등록한 이벤트만 재 등록을 방지
             *  DeviceStatusReceived 이벤트는 DeviceAgent, DeviceController 등 필요에 따라
             *  여러군데 등록되어 실행되므로 중복등록 방지코드에서 선별작업이 필요하다.
             *  현재 중복등록 방지코드는 같은 이름을 무조건 삭제하도록 되어 있는데
             *  DeviceStatusReceived 의 경우 해당 Controller 에서 등록한 이벤트만 재 등록을 방지할 수 있어야 하므로
             *  조금 더 고민해 봐야 한다.
             *
             *  -- $rootScope.$on을 $scope.$on 으로 변경해야 중복 등록이 발생되지 않는다.
             */
            $scope.$on('DeviceStatusReceived', updateDeviceStatus);

            $scope.$on(Const.UI_EVENT_TYPE.DEVICE_INFO_UPDATED, function (event, deviceObject) {
                if (deviceObject == null || deviceObject == undefined) {
                    return;
                }
                if (angular.element('#devicegrouptree').length) {
                    var node = angular.element('#devicegrouptree').tree('find', deviceObject.id);
                    if (node) {
//                        console.log(node);
                        $scope.bindDeviceTreeData();
                    }
                }
            });

            angular.element('#deviceContextMenu').on('contextmenu', function ($event) {
                var event = $event || window.event;

                if (event.preventDefault) event.preventDefault();
                if (event.returnValue) event.returnValue = false;

                event.cancelBubble = true;
            });

        };


        // -------------------------- LNB 영역 resize s --------------------------
        // -------------------------- LNB 영역 resize e --------------------------
    }]);
});