'use strict';

define([], function () {
    var app = angular.module('biostarWebApp');

    app.register.controller('AddDeviceController', ['$scope', '$rootScope', 'Const', 'API', 'Util', 'MessageService', 'MessageProperties', function ($scope, $rootScope, Const, API, Util, MessageService, MessageProperties) {
        $scope.init = function () {
        };
        var products = [
            {productid: 'FI-SW-01', name: 'Koi'},
            {productid: 'K9-DL-01', name: 'Dalmation'},
            {productid: 'RP-SN-01', name: 'Rattlesnake'},
            {productid: 'RP-LI-02', name: 'Iguana'},
            {productid: 'FL-DSH-01', name: 'Manx'},
            {productid: 'FL-DLH-02', name: 'Persian'},
            {productid: 'AV-CB-01', name: 'Amazon Parrot'}
        ];


        $('#addgrid').datagrid({
            fitColumns: true,
            scrollbarSize: 0,
            iconCls: 'icon-edit',
            //singleSelect:true,
            selectOnCheck: true,
            checkOnSelect: true,
            checkbox: true,
            onClickCell: onClickCell,
            columns: [
                [
                    {field: 'ck', checkbox: true},
                    {field: 'name', title: 'Name', width: 130,
                        formatter: function (value, row) {
                            if (row.name && row.name.length > 0) {
                                return row.name;
                            } else {
                                row.name = row.device_type_id.name + ' (' + row.lan.ip + ')';
                                return row.name;
                            }
                        },
                        editor: 'text'
                    },
                    {field: 'id', title: 'ID', width: 50},
                    {field: 'device_type_id', title: 'Device Type', width: 80,
                        formatter: function (value, row) {
                            var rs485_mode = Const.RS485_MODE[row.rs485.rs485_mode] || Const.RS485_MODE._default;
                            if (rs485_mode == Const.RS485_MODE._default) {
                                return value.name;
                            } else {
                                return value.name + "(" + rs485_mode + ")";
                            }
                        }
                    },
                    {field: 'lan', title: 'IP Address', width: 80, formatter: function (value, row, index) {
                        return value.ip;
                    }},
                    {field: 'group', title: 'Group', width: 100,
                        formatter: function (value) {
                            for (var i = 0; i < products.length; i++) {
                                if (products[i].productid == value) return products[i].name;
                            }
                            return value;
                        },
                        editor: {
                            type: 'combotree',
                            options: {
                                valueField: 'productid',
                                textField: 'name',
                                data: products,
                                required: true
                            }
                        }

                    }
                ]
            ],
            onLoadSuccess: function (data) {
                $('#tooltipCont').parents('.panel').find('.panel-tool-close').click(function () {
                    $scope.doDlgCancel();
                });
            },
            onAfterEdit: function (rowIndex, rowData, changes) {
                console.log('onAfterEdit')
            },
            onEndEdit: function (rowIndex, rowData, changes) {
                console.log('onEndEdit')
            },
            onCancelEdit: function (rowIndex, rowData) {
                console.log('onCancelEdit')
            }
        }).datagrid('loadData', $scope.dlg.result);

        $.extend($.fn.datagrid.methods, {
            editCell: function (jq, param) {
                return jq.each(function () {
                    var opts = $(this).datagrid('options');
                    var fields = $(this).datagrid('getColumnFields', true).concat($(this).datagrid('getColumnFields'));
                    for (var i = 0; i < fields.length; i++) {
                        console.log("fie:" + fields[i]);
                        var col = $(this).datagrid('getColumnOption', fields[i]);
                        col.editor1 = col.editor;
                        if (fields[i] != param.field) {
                            col.editor = null;
                        }
                    }
                    $(this).datagrid('beginEdit', param.index);
                    for (var i = 0; i < fields.length; i++) {
                        var col = $(this).datagrid('getColumnOption', fields[i]);
                        col.editor = col.editor1;
                    }
                });
            }
        });

        $(document).on('focusout', '.datagrid-body .datagrid-editable .datagrid-editable-input', function (evt) {
            $('#addgrid').datagrid('endEdit', 0)
        });

        var editIndex = undefined;

        function endEditing() {
            if (editIndex == undefined) {
                return true
            }
            if ($('#addgrid').datagrid('validateRow', editIndex)) {
                $('#addgrid').datagrid('endEdit', editIndex);
                editIndex = undefined;
                return true;
            } else {
                return false;
            }
        }

        function onClickCell(index, field) {
            console.log(index + "=" + field);

            if (endEditing()) {
                $('#addgrid').datagrid('selectRow', index)
                    .datagrid('editCell', {index: index, field: field});
                editIndex = index;
            }
        }


        $scope.addCount = 0;
        $scope.doDlgApply = function (name) {

            if ($('#btnAddDevice').hasClass('disabled')) {
                return;
            }
            var selectedData = $('#addgrid').datagrid('getSelections');//선택된 데이터 배열

            console.log(selectedData);
//            $('#addCont').text('0');
            angular.forEach(selectedData, function (deviceData) {
                //angular.extend(deviceData, {name:''});
                //TODO: device_type_id.id를 '1'로 고정하여 값을 넣는 것은 임시 조치이다. 반드시 서버가 준비된 이후 제거해야 한다.
                deviceData.device_type_id = {id: '1'};
                deviceData.device_group_id = {id: '1'};

            });
            if (selectedData.length == 1) {
                API.post('devices', undefined, {'Device': selectedData[0]}).then(function (data) {
                    console.log('저장됨');
                    $('#addgrid').datagrid('deleteRow', $('#addgrid').datagrid('getRowIndex', selectedData[0]));

                    $('#popdevicegrid').datagrid('deleteRow', $('#popdevicegrid').datagrid('getRowIndex', selectedData[0]));
                    $scope.addCount++;
                    MessageService.displayInformationMessage(MessageProperties.get("device.deviceAdded"), function (result) {
                        var crrData = $('#addgrid').datagrid('getData');
                        if (crrData.rows.length == 0) {
                            $scope.doDlgCancel();
                        }
                    });
                }, function error(data) {
                    Util.debug(data.httpResponseStatus);
                    if (data.httpResponseStatus == 500) {
                        MessageService.displayWarningMessage(MessageProperties.get("device.deviceNotResponding"), function (result) {

                        });
                    }
                });
            } else if (selectedData.length > 1) {
                API.post('devices', undefined, {'DeviceCollection': {'total': selectedData.length, 'rows': selectedData}}).then(function (data) {
                    console.log('저장됨');
                    MessageService.displayInformationMessage(MessageProperties.get("device.deviceAdded"), function (result) {
                        var crrData = $('#addgrid').datagrid('getData');
                        if (crrData.rows.length == 0) {
                            $scope.doDlgCancel();
                        }
                    });
                    $('#addgrid').datagrid('deleteRow', $('#addgrid').datagrid('getRowIndex', deviceData));

                    $('#popdevicegrid').datagrid('deleteRow', $('#popdevicegrid').datagrid('getRowIndex', deviceData));
                    var crrData = $('#addgrid').datagrid('getData');
                    $scope.addCount++;
                    if (crrData.rows.length == 0) {
                        $scope.doDlgCancel();
                    }
                });
            }

//            $scope.$emit('addDevices', data);
        };


        $scope.doDlgCancel = function () {
            $scope.tooltippage = {url: ''};
            $('#tooltipCont').dialog('destroy');

            /**
             * toasrMessage 띄우는 부분
             */
                //$('#addCont').text($scope.addCount + '');
            $scope.$emit('dlgClose', {addCount: $scope.addCount});
        };

    }]);
});