'use strict';

define([], function () {
    var app = angular.module('biostarWebApp');

    app.register.controller('AdvanceDeviceController', ['$scope', '$timeout', 'API', 'DeviceAgent', 'Const', 'Util', 'MessageService', 'MessageProperties', function ($scope, $timeout, API, DeviceAgent, Const, Util, MessageService, MessageProperties) {

        $scope.ipAddress = Const.REGULAR_EXPRESSION.IP_ADDRESS;// /(((\d{1,2})|(1\d{2})|(2[0-4]\d)|(25[0-5]))\.){3}(((\d{1,2})|(1\d{2})|(2[0-4]\d)|(25[0-5])))/;
        $scope.number = /^[0-9]+$/;
        $scope.addCount = 0;
        $scope.selectedTab = 0;
        $scope.$watch('selectedTab', function (nv, ov) {
            if (nv === ov) return false;
            if (nv == 0) {
                $("#advancedSearchDevicePopupDlg").dialog({width: 560});
            } else if (nv == 1) {
                $("#advancedSearchDevicePopupDlg").dialog({width: 800});
            }
            console.log('selectedTab', nv, ov);
        });
        $scope.selected = false;

        $scope.searchOption = {
            Device: {
                lan: {
                    ip: '',
                    device_port: '51211'
                }
            }
        };

        $scope.deviceType = {
            data: [
                {name: 'BST/DST/XST/BST2/FST', port: '30001'},
                {name: 'BEPL/BEW/BLN/Xpass/Xpass Slim', port: '51211'},
                {name: 'Custom', port: '51211'}
            ],
            selectedIndex: 2
        };

        $scope.$watch('deviceType.selectedIndex', function (nv, ov) {
            if (nv === ov) return false;

            if (nv == 2) {
                var target = event.target || (event.srcElement || document);
                if (target.id !== 'txtPort') {
                    $('#txtPort').focus()[0].setSelectionRange(0, $('#txtPort').val().length);
                }
            } else {
                $scope.searchOption.Device.lan.device_port = $scope.deviceType.data[nv].port;
            }
            console.log('deviceType.selectedIndex', nv, ov);
        });

        var deviceGroupTreeInit = function () {
            if (DeviceAgent.getDeviceGroupTreeOnlyGroup().then) {
                DeviceAgent.getDeviceGroupTreeOnlyGroup().then(function success(data) {
                    $scope.deviceGroupTree = [data];
                });
            } else {
                $scope.deviceGroupTree = [DeviceAgent.getDeviceGroupTreeOnlyGroup()];
            }
        };

        $scope.selectClick = function (id) {
            if ($(id).hasClass('disabled')) {
                return false;
            }
            $(id).toggleClass('open');
        };

        $scope.changePort = function () {
            $scope.deviceType.selectedIndex = 2;
        };

        $scope.cancel = function () {
            $("#advancedSearchDevicePopupDlg").dialog('close');
        };

        $scope.tcpSearch = function (invalid) {
            if (invalid || $('#btnSearch').hasClass('disabled')) {
                return false;
            }
            $('#btnSearch').attr({'disabled': 'disabled'}).addClass('disabled');

            deviceGroupTreeInit();

            DeviceAgent.getDeviceSpec().then(function success(data) {
                $scope.deviceSpec = data;
            }).finally(function () {
                var postOptions = {
                    url : 'devices/tcp_search',
                    params: $scope.searchOption,
                    displaySuccessPopoup : false,
                    displayErrorPopup :false
                };
                API.postWithOptions(postOptions).then(function (data) {

                    $('#btnSearch').removeAttr('disabled').removeClass('disabled');
                    if (data.DeviceCollection == undefined || data.DeviceCollection.rows.length <= 0) {
                        $scope.selectedTab = 0;
                        MessageService.displayInformationMessage(MessageProperties.get("device.deviceNotFound"));
                    } else {
                        $scope.selectedTab = 1;
                        $timeout(function () {
                            $scope.initGrid(data)
                        }, 10);
                    }

                }, function error(data) {
                    $('#btnSearch').removeAttr('disabled').removeClass('disabled');
                    $scope.selectedTab = 0;
                    MessageService.displayInformationMessage(MessageProperties.get("device.deviceNotFound"));
                });
            });
        };

        $scope.addDevice = function () {
            var target = event.target || (event.srcElement || document);
            if ($(target).hasClass('disabled')) {
                return;
            }
            var devices = $('#grdSearchedDevice').datagrid('getData');

            if (angular.isUndefined(devices.rows) || devices.rows.length <= 0) {
                return;
            }
            angular.forEach(devices.rows, function (deviceData) {
                //angular.extend(deviceData, {name:''});
                if (angular.isUndefined(deviceData.device_group_id) || angular.isUndefined(deviceData.device_group_id.id) || deviceData.device_group_id.id == '') {
                    deviceData.device_group_id = {id: '1'};
                }
            });


            if (devices.rows.length == 1) {
                API.post('devices', undefined, {'Device': devices.rows[0]}, undefined, false, true, true).then(function (data) {
                    console.log('저장됨');

                    $('#grdSearchedDevice').datagrid('deleteRow', $('#grdSearchedDevice').datagrid('getRowIndex', devices.rows[0]));
                    $scope.addCount++;
                    angular.element('#advancedSearchDevicePopupDlg').dialog('options').addCount = $scope.addCount;
                    MessageService.displayInformationMessage(MessageProperties.get("device.deviceAdded"), function (result) {
                        var crrData = $('#grdSearchedDevice').datagrid('getData');
                        if (crrData.rows.length == 0) {
                            $scope.cancel();
                        }
                    });
                }, function error(data) {
                    Util.debug(data.httpResponseStatus);
                    if (data.httpResponseStatus == 500) {
                        MessageService.displayWarningMessage(MessageProperties.get("device.deviceNotResponding"), function (result) {

                        });
                    }
                });
            } else if (devices.rows.length > 1) {
                API.post('devices', undefined, {'DeviceCollection': {'total': devices.rows.length, 'rows': devices.rows}}, undefined, false, true, true).then(function (data) {
                    console.log('저장됨');
                    if (angular.isDefined(data.DeviceCollection) && angular.isDefined(data.DeviceCollection.rows)) {
                        var addedCount = data.DeviceCollection.rows.length;
                        angular.forEach(data.DeviceCollection.rows, function (deviceData) {
                            var index = $('#grdSearchedDevice').datagrid('getRowIndexByID', {id: deviceData.id});
                            $("#grdSearchedDevice").datagrid('deleteRow', index);
                        });

                        MessageService.displayInformationMessage(MessageProperties.get("device.deviceAdded"), function (result) {
                            var crrData = $('#grdSearchedDevice').datagrid('getData');
                            if (crrData.rows.length == 0) {
                                $scope.cancel();
                            }
                        });
                    }
                });
            }
        };

        $scope.deviceSpec = undefined;
        $scope.initGrid = function (data) {

            var scrollbarSize = 11;

            if (data.DeviceCollection == undefined || data.DeviceCollection.rows.length < 8) {
                scrollbarSize = 0;
            }

            $("#grdSearchedDevice").datagrid({
                fitColumns: true,
                scrollbarSize: scrollbarSize,
                iconCls: 'icon-edit',
                singleSelect: true,
                selectOnCheck: false,
                checkOnSelect: false,
                //editOnSelect: true, // TODO: editOnSelect는 없는 속성이다. 구현해야 한다. 편집중일 때 선택이 풀리는 문제
//                checkbox: true,
                columns: [
                    [
//                        {field: 'ck', checkbox: true, showAllCheckOption: false, width: 55},
                        {field: 'id', title: MessageProperties.get('device.info.deviceId'), width: 110},
                        {field: 'name', title: MessageProperties.get('device.info.name'), width: 250,
                            formatter: function (value, row) {
                                if (row.name && row.name.length > 0) {
                                    return row.name;
                                } else {
                                    row.name = $scope.deviceSpec[row.device_type_id.id].name + ' ' + row.id + ' (' + row.lan.ip + ')';
                                    return row.name;
                                }
                            },
                            editor: 'text'
                        },
                        {field: 'device_group', title: MessageProperties.get('device.info.group'), width: 180,
                            formatter: function (value, row, index) {
                                if (angular.isUndefined(value)) {
                                    value = '1';
                                    row['device_group'] = value;
                                }
                                row['device_group_id'] = {id: value};
                                var data = $('#devicegrouptree').tree('find', value);
                                if (data != null) {
                                    return data.text;
                                } else {
                                    return '';
                                }
                            },
                            editor: {
                                type: 'combotree',
                                options: {
                                    animate: true,
                                    valueField: 'id',
                                    textField: 'text',
                                    data: $scope.deviceGroupTree,
                                    panelWidth: 'auto',
                                    panelHeight: '300',
                                    onHidePanel: function () {
                                        console.log('onHidePanel');
                                        var target = event.target || (event.srcElement || document);
                                        if ($(target).hasClass('tree-title') != true && $(target).hasClass('tree-node') != true) {
                                            $scope.editingIndex = $(this).parents('tr[datagrid-row-index]').attr('datagrid-row-index') - '';
                                            endEditing();
//                                            $('#grdSearchedDevice').datagrid('endEdit', $(this).parents('tr[datagrid-row-index]').attr('datagrid-row-index'));
                                        } else if ($(this).parents('td').find('.combo-value') && $(this).parents('td').find('.combo-value').val) {
                                            var oldValue = $('#devicegrouptree').tree('find', $(this).parents('td').find('.combo-value').val()).text;
                                            var newValue = $(target).text();
                                            if (oldValue == newValue) {
                                                $scope.editingIndex = $(this).parents('tr[datagrid-row-index]').attr('datagrid-row-index') - '';
                                                endEditing();
                                            }
                                        }
                                    },
                                    onChange: function (newValue, oldValue) {
                                        console.log('onChange');
                                        var target = event.target || (event.srcElement || document);
                                        if (oldValue != '' || $(this).css('display') !== 'none' || ($(target).hasClass('datagrid-editable') != true && target.tagName != 'SPAN')) {
                                            $scope.editingIndex = $(this).parents('tr[datagrid-row-index]').attr('datagrid-row-index') - '';
                                            endEditing();
//                                            $('#grdSearchedDevice').datagrid('endEdit', $(this).parents('tr[datagrid-row-index]').attr('datagrid-row-index'));
                                        }
                                        console.log('newValue: ' + newValue + ' oldValue: ' + oldValue);
                                    },
//                                    formatter: function(row){
//                                        var opts = $(this).tree('options');
//                                        return row[opts.textField];
//                                    },
                                    onLoadSuccess: function () {
                                        console.log('onLoadSuccess');
                                        var selectedNode = $(this).tree('getSelected');
                                        if (selectedNode == null) {
                                            $(this).tree('select', $(this).tree('getRoot').target);
                                        }
                                    }
                                }
                            }
                        },
                        {field: 'device_group_id', title: 'Group', hidden: true},
                        {field: 'device_type_id', title: '<span>' + MessageProperties.get('device.info.deviceType') +
                            '</span><br/>(' + MessageProperties.get('device.rs485.master') + '/' +
                            MessageProperties.get('device.rs485.slave') + ')', width: 147,
                            formatter: function (value, row) {
                                if (angular.isUndefined(row.rs485)) {
                                    return $scope.deviceSpec[value.id].name;
                                } else {
                                    var rs485_mode = Const.RS485_MODE_HTML[row.rs485.mode] || Const.RS485_MODE_HTML._default;
                                    if (rs485_mode == Const.RS485_MODE_HTML._default) {
                                        return $scope.deviceSpec[value.id].name;
                                    } else {
                                        return $scope.deviceSpec[value.id].name + rs485_mode;
                                    }
                                }
                            }
                        },
                        {field: 'lan', title: MessageProperties.get('device.info.ipAddress'), width: 134, formatter: function (value, row, index) {
                            return angular.isDefined(value) && angular.isDefined(value.ip) ? value.ip : '';
                        }}
                    ]
                ],
                onBeforeLoad: function (param) {
                    console.log('onBeforeLoad');
                    console.log(param);
                },
                onLoadSuccess: function (data) {
                    console.log('onLoadSuccess');
                    $scope.validCheckforAdd()
                },
                onBeginEdit: function (rowIndex, rowData) {
                    console.log('onBeginEdit');
                    $('div.panel.combo-p > div.combo-panel.panel-body.panel-body-noheader').addClass('scrollY').attr('style', 'border: 1px solid #b4b4b4');
                    $('div.panel.combo-p > div.combo-panel.panel-body.panel-body-noheader > ul.tree').addClass('treeTypeDevice');
                },
                onCancelEdit: function (rowIndex, rowData) {
                    console.log('onCancelEdit');
                },
                onEndEdit: function (rowIndex, rowData, changes) {
                    console.log('onEndEdit');
                },
                onAfterEdit: function (rowIndex, rowData, changes) {
                    console.log('onAfterEdit');
                    $('#grdSearchedDevice').datagrid('checkRow', rowIndex);
                },
                onClickCell: function (index, field) {
                    console.log('onClickCell');
                    if (endEditing()) {
                        var data = $(this).datagrid('getData');
                        if (data.rows[index].lan.ip.substring(0, 7) === '169.254' || data.rows[index].lan.ip === '') {
                            return false;
                        } else {
                            $(this).datagrid('editCell', {index: index, field: field});
                            $scope.editingIndex = index;
                        }
                    }
                },
                onSelect: function (rowIndex, rowData) {
                    console.log('onSelect');
                },
                onUnselect: function (rowIndex, rowData) {
                    console.log('onUnselect');
                },
                onCheck: function (rowIndex, rowData) {
                    console.log('onCheck');
                    $scope.validCheckforAdd();
                },
                onUncheck: function (rowIndex, rowData) {
                    console.log('onUncheck');
                    $scope.validCheckforAdd();
                },
                onCheckAll: function (rows) {
                    console.log('onCheckAll');
                    $scope.validCheckforAdd()

                },
                onUncheckAll: function (rows) {
                    console.log('onUncheckAll');
                    $scope.validCheckforAdd()
                },
                data: data.DeviceCollection
            });
        };

        $.extend($.fn.datagrid.methods, {
            editCell: function (jq, param) {
                return jq.each(function () {
                    console.log(this);
                    console.log(jq);
                    console.log(param);
                    //var opts = $(this).datagrid('options');
                    var fields = $(this).datagrid('getColumnFields', true).concat($(this).datagrid('getColumnFields'));
                    for (var i = 1; i < fields.length; i++) {
                        //console.log("fie:" + fields[i]);
                        if (fields[i] == 'ck') {
                            continue;
                        }
                        var col = $(this).datagrid('getColumnOption', fields[i]);
                        if (angular.isUndefined(col.editor)) {
                            continue;
                        }
                        if (typeof col.editor === 'string') {
                            col.editor1 = col.editor.toString();
                        } else {
                            col.editor1 = {};
                            angular.copy(col.editor, col.editor1);
                        }
                        if (fields[i] != param.field) {
                            col.editor = null;
                        }
                    }
                    $(this).datagrid('beginEdit', param.index);
                    for (var i = 1; i < fields.length; i++) {
                        if (fields[i] == 'ck') {
                            continue;
                        }
                        var col = $(this).datagrid('getColumnOption', fields[i]);
                        if (angular.isUndefined(col.editor)) {
                            continue;
                        }
                        if (typeof col.editor1 === 'string') {
                            col.editor = col.editor1.toString();
                        } else {
                            col.editor = {};
                            angular.copy(col.editor1, col.editor);
                        }
                    }
                });
            },
            getRowIndexByID: function (jq, param) {
                var index = -1;
                jq.each(function () {
                    try {
                        var id = (angular.isDefined(param.id) ? param.id : param);
                        var data = $(this).datagrid('getData');
                        for (var i = 0; i < data.rows.length; i++) {
                            if (id == data.rows[i].id) {
                                index = i;
                                return index;
                            }
                        }
                    } catch (ex) {
                        console.error(ex);
                    }
                });
                return index;
            }
        });

        $(document).on('focusout', '.datagrid-body .datagrid-editable .datagrid-editable-input', function (evt) {
            $scope.editingIndex = $(evt.target).parents('tr[datagrid-row-index]').attr('datagrid-row-index') - '';
            endEditing();
            //$('#grdSearchedDevice').datagrid('endEdit', $(evt.target).parents('tr[datagrid-row-index]').attr('datagrid-row-index'));
        });

        $(document).on('keypress', '.datagrid-body .datagrid-editable .datagrid-editable-input', function (evt) {
            console.log('keypress:' + evt.keyCode);
            if (evt.keyCode == 13) {
                $scope.editingIndex = $(evt.target).parents('tr[datagrid-row-index]').attr('datagrid-row-index') - '';
                endEditing();
            }
        });

        $scope.editingIndex = undefined;

        function endEditing() {
            console.log('$scope.editingIndex:' + $scope.editingIndex);
            if ($scope.editingIndex == undefined) {
                return true
            }
            if ($('#grdSearchedDevice').datagrid('validateRow', $scope.editingIndex)) {
                $('#grdSearchedDevice').datagrid('endEdit', $scope.editingIndex);
                $scope.editingIndex = undefined;
                return true;
            } else {
                return false;
            }
        }

        $scope.validCheckforAdd = function () {
            var devices = $('#grdSearchedDevice').datagrid('getData');
            $scope.selected = devices.rows.length > 0;

            angular.forEach(devices.rows, function (device) {
                if (angular.isUndefined(device.lan) || device.lan.ip === '' || device.lan.ip.substring(0, 7) === '169.254') {
                    $scope.selected = $scope.selected && false;
                }
            });
            $scope.$apply($scope.selected);
            console.log('validCheckforAdd');
            return $scope.selected;
        };
    }]);
});