'use strict';

define([], function () {
    var app = angular.module('biostarWebApp');

    app.register.controller('BatchEditController', ['$scope', '$rootScope', 'API', 'Const', 'Util', 'MessageProperties', 'MessageService', 'DeviceAgent', function ($scope, $rootScope, API, Const, Util, MessageProperties, MessageService, DeviceAgent) {
        $scope.safeApply = function (fn) {
            var phase = this.$root.$$phase;
            if (phase == '$apply' || phase == '$digest') {
                if (fn && (typeof(fn) === 'function')) {
                    fn();
                }
            } else {
                this.$apply(fn);
            }
        };

        $scope.selectedDeviceCount = '0';
        $scope.ipAddress = Const.REGULAR_EXPRESSION.IP_ADDRESS;
        $scope.number = /^[0-9]+$/;
        $scope.RS485_MODE = Const.RS485_MODE_FOR_SELECT_BOX;
        $scope.RS485_BAUDRATE = Const.RS485_BAUDRATE;
        $scope.invalidSubnetMask = false;
        $scope.invalidGateway = false;
        $scope.invalidServerIP = false;

        $scope.editDhcp = false;
        $scope.useDHCP = 'true';
        $scope.editSubnetMask = false;
        $scope.subnetMask = '';
        $scope.editGateway = false;
        $scope.gateway = '';
        $scope.editDevicePort = false;
        $scope.devicePort = '51211';
        $scope.editConnectionMode = false;
        $scope.connectionMode = 'false';
        $scope.editServerIP = false;
        $scope.serverIP = '';
        $scope.editServerPort = false;
        $scope.serverPort = '51212';
        $scope.editRs485Mode = false;
        $scope.rs485Mode = '';
        $scope.editBaudRate = false;
        $scope.baudRate = '115200';

        $scope.initPopup = function () {
            $scope.selectedDeviceCount = Util.getSessionObject(Const.DEVICE_BATCH_EDIT_SELECTED_COUNT);
            Util.removeSessionObject(Const.DEVICE_BATCH_EDIT_SELECTED_COUNT);
        };
        var getDeviceInfo = function () {
            var deviceInfo = {};
            if ($scope.editDhcp) {
                if (angular.isUndefined(deviceInfo.lan)) {
                    deviceInfo['lan'] = {};
                }
                deviceInfo.lan['enable_dhcp'] = $scope.useDHCP;
            }
            if ($scope.editSubnetMask) {
                if (angular.isUndefined(deviceInfo.lan)) {
                    deviceInfo['lan'] = {};
                }
                deviceInfo.lan['subnet_mask'] = $scope.subnetMask;
            }
            if ($scope.editGateway) {
                if (angular.isUndefined(deviceInfo.lan)) {
                    deviceInfo['lan'] = {};
                }
                deviceInfo.lan['gateway'] = $scope.gateway;
            }
            if ($scope.editDevicePort) {
                if (angular.isUndefined(deviceInfo.lan)) {
                    deviceInfo['lan'] = {};
                }
                deviceInfo.lan['device_port'] = $scope.devicePort;
            }
            if ($scope.editConnectionMode) {
                if (angular.isUndefined(deviceInfo.lan)) {
                    deviceInfo['lan'] = {};
                }
                deviceInfo.lan['connection_mode'] = $scope.connectionMode;
            }
            if ($scope.editServerIP) {
                if (angular.isUndefined(deviceInfo.lan)) {
                    deviceInfo['lan'] = {};
                }
                deviceInfo.lan['server_ip'] = $scope.serverIP;
            }
            if ($scope.editServerPort) {
                if (angular.isUndefined(deviceInfo.lan)) {
                    deviceInfo['lan'] = {};
                }
                deviceInfo.lan['server_port'] = $scope.serverPort;
            }
            if ($scope.editRs485Mode) {
                if (angular.isUndefined(deviceInfo.rs485)) {
                    deviceInfo['rs485'] = {};
                }
                deviceInfo.rs485['mode'] = $scope.rs485Mode;
            }
            if ($scope.editRs485Mode) {
                if (angular.isUndefined(deviceInfo.rs485)) {
                    deviceInfo['rs485'] = {};
                }
                if (angular.isUndefined(deviceInfo.rs485.channels)) {
                    deviceInfo.rs485['channels'] = [];
                }
                if (deviceInfo.rs485.channels.length < 0) {
                    deviceInfo.rs485.channels.push({
                        'baudrate': $scope.baudRate
                    });
                } else {
                    deviceInfo.rs485.channels[0] = {
                        'baudrate': $scope.baudRate
                    };
                }
            }
            return deviceInfo;
        };

        var getDeviceIDs = function () {
            var deviceID = '';
            if ($scope.SelectAllYN) {
                deviceID = Const.SELECTED_ALL;
            } else {
                var batchDeviceIDs = [];
                for (var i = 0; i < $scope.selectedDevices.length; i++) {
                    batchDeviceIDs.push($scope.selectedDevices[i].id);
                }

                deviceID = batchDeviceIDs.join('+');
            }

            if (angular.isDefined($scope.Parameters)) {
                if (angular.isDefined($scope.Parameters.group_id) && $scope.Parameters.group_id != '') {
                    deviceID += '&group_id=' + $scope.Parameters.group_id;
                }
                if (angular.isDefined($scope.Parameters.query) && $scope.Parameters.query != '') {
                    deviceID += '&query=' + $scope.Parameters.query;
                }
            }
            return deviceID;
        };

        $scope.applyDeviceInfo = function() {
            var options = angular.element('#dlgBatchEdit').dialog('options');
            $scope.selectedDevices = options.selectedDevices || [];
            $scope.Parameters = options.Parameters || {total: 0};
            $scope.SelectAllYN = options.SelectAllYN;

            var deviceID = getDeviceIDs();

            var deviceInfo = getDeviceInfo();

            API.put('devices?id=' + deviceID, 'batchEditDevices', {'Device': deviceInfo}, undefined, true, true, true,
                'batchEditDevices').then(function (data) {
                $rootScope.$emit('reloadDeviceTree');
                $rootScope.$emit('onRefreshDeviceList');
                $scope.closePopup();
            });
        };

        $scope.closePopup = function() {
            angular.element('#dlgBatchEdit').dialog('close');
        };
    }]);
});