'use strict';

define([], function () {
    var app = angular.module('biostarWebApp');

    app.register.controller('FirmwareUpgradeController', ['$scope', '$rootScope', '$timeout', 'API', 'Const', 'Util', 'MessageProperties', 'MessageService', '$filter', 'DeviceAgent', function ($scope, $rootScope, $timeout, API, Const, Util, MessageProperties, MessageService, $filter, DeviceAgent) {

        $scope.safeApply = function (fn) {
            var phase = this.$root.$$phase;
            if (phase == '$apply' || phase == '$digest') {
                if (fn && (typeof(fn) === 'function')) {
                    fn();
                }
            } else {
                this.$apply(fn);
            }
        };

        $scope.firmwareFilesData = [
            {
                version: 'V1.01',
                device_type: '1',
                file:{
                    id: 1,
                    path: ''
                }
            },
            {
                version: 'V1.02',
                device_type: '1',
                file:{
                    id: 2,
                    path: ''
                }
            },
            {
                version: 'V1.03',
                device_type: '1',
                file:{
                    id: 3,
                    path: ''
                }
            },
            {
                version: 'V1.02',
                device_type: '2',
                file:{
                    id: 2,
                    path: ''
                }
            },
            {
                version: 'V1.03',
                device_type: '2',
                file:{
                    id: 3,
                    path: ''
                }
            },
            {
                version: 'V1.04',
                device_type: '1',
                file:{
                    id: 4,
                    path: ''
                }
            },
            {
                version: 'V1.05',
                device_type: '1',
                file:{
                    id: 5,
                    path: ''
                }
            },
            {
                version: 'V1.03',
                device_type: '3',
                file:{
                    id: 6,
                    path: ''
                }
            },
            {
                version: 'V1.06',
                device_type: '1',
                file:{
                    id: 6,
                    path: ''
                }
            }
        ];

        $scope.deviceSpec = undefined;
        $scope.initPopup = function () {
            if ($rootScope.$$listeners[Const.ON_OPEN_FW_UPGRADE_POPUP] != undefined) {
                $rootScope.$$listeners[Const.ON_OPEN_FW_UPGRADE_POPUP] = undefined;
            }
            $rootScope.$on(Const.ON_OPEN_FW_UPGRADE_POPUP, function (event, data) {
                $scope.searchFirmwares();
            });
        };

        $scope.searchFirmwares = function() {
            var options = angular.element('#dlgFirmwareUpgrade').dialog('options');
            $scope.selectedDevices = options.selectedDevices;

            DeviceAgent.getDeviceSpec().then(function success(data) {
                $scope.deviceSpec = data;
            }).finally(function () {
                $scope.firmwareFiles = [];
                API.get('firmwares').then(function(data) {
                    if(data && data.FirmwareFileCollection) {
                        $scope.firmwareFilesData = data.FirmwareFileCollection.rows;
                        var selectedDeviceTypes = [];
                        for (var i = 0; i < $scope.selectedDevices.length; i++) {
                            var deviceTypeID = $scope.selectedDevices[i].device_type_id.id;
                            if (selectedDeviceTypes.indexOf(deviceTypeID) < 0) {
                                selectedDeviceTypes.push(deviceTypeID);
                            }
                        }
                        $scope.safeApply(function () {
                            $scope.firmwareFiles = $filter('devicesTypeFilter')($scope.firmwareFilesData, selectedDeviceTypes);
                        });
                    }
                });
            });
        };

        $scope.applyUpgrade = function(file) {
            console.log(file);
            MessageService.displayConfirmMessage(MessageProperties.get('device.firmwareUpgradeConfirm'), function (dialogResult) {
                if (dialogResult) {
                    var selectedDeviceType = $filter('devicesTypeFilter')($scope.selectedDevices, [file.device_type]);
                    API.post('firmwares/update', undefined, {
                        'FirmwareFile': file,
                        'DeviceCollection': {total: selectedDeviceType.length, rows: selectedDeviceType}
                    }).then(function (data) {
                        MessageService.displayInformationMessage(MessageProperties.get('device.firmwareUpgradeSuccess'));
                    });
                } else {
                }
            });
        };
        $scope.closePopup = function() {
            angular.element('#dlgFirmwareUpgrade').dialog('close');
        };
    }]);
});