'use strict';

define([
    'modules/device/dialog/setipDeviceController'
], function () {
    var app = angular.module('biostarWebApp');

    app.register.controller('SearchDeviceDlgController', [
        '$scope', '$rootScope', '$timeout', '$compile', 'API', 'Const', 'Util', 'DeviceAgent', 'MessageProperties', 'MessageService',
        function ($scope, $rootScope, $timeout, $compile, API, Const, Util, DeviceAgent, MessageProperties, MessageService) {

            var options = {};
            var ipRegEx = Const.REGULAR_EXPRESSION.IP_ADDRESS;// /(((\d{1,2})|(1\d{2})|(2[0-4]\d)|(25[0-5]))\.){3}(((\d{1,2})|(1\d{2})|(2[0-4]\d)|(25[0-5])))/;

            $scope.addCount = 0;
            $scope.maxCount = 600;
            $scope.selectedRow = -1;
            $scope.isLoading = true;
            $scope.isAllChecked = false;
            $scope.sortingOption = {
                direction: Const.SORTING_DIRECTION.NONE,
                field: ''
            };

            $scope.UdpSearchOption = {
                'UdpSearchOption': {
                    'timeout': 3,
                    'with_rs485': false,
                    'hide_known_devices': true
                }
            };

            $scope.rowStatus = {};
            $scope.DeviceCollection = [];
            $scope.invalidIPList = [];
            $scope.invalidSelectedList = [];
            $scope.validSelectedList = [];
            $scope.deletedList = [];

            var deviceGroupTreeInit = function () {
                if (DeviceAgent.getDeviceGroupTreeOnlyGroup().then) {
                    DeviceAgent.getDeviceGroupTreeOnlyGroup().then(function success(data) {
                        $scope.deviceGroupTree = [data];
                    });
                } else {
                    $scope.deviceGroupTree = [DeviceAgent.getDeviceGroupTreeOnlyGroup()];
                }
            };

            $scope.deviceSpec = undefined;
            $scope.initPopup = function () {

                deviceGroupTreeInit();

                DeviceAgent.getDeviceSpec().then(function success(data) {
                    $scope.deviceSpec = data;
                }).finally(function () {
                    $scope.doSearch();
                });

                angular.element('#deviceList').on('keydown', '.inputEdit[ng-model="device.name"]', function (event) {
                    //console.log('keyCode:' + event.keyCode);
//                    if (event.keyCode == 53) {
//                        // INFO: KeyCode: 53 = '%'
//                        return false;
//                    } else if (event.keyCode == 188) {
//                        // INFO: KeyCode: 188 = '<'
//                        return false;
//                    }
                });
            };

            $scope.doSearch = function () {
                $rootScope.$emit('onCloseDeviceGroupTree');
                $scope.isLoading = true;
                $scope.selected = false;
                if (parseInt($scope.UdpSearchOption.UdpSearchOption.timeout) >= 60) {
                    $scope.UdpSearchOption.UdpSearchOption.timeout = 59;
                }
                $scope.DeviceCollection = [];
                $scope.maxCount = 600;
                API.post('devices/udp_search', undefined, $scope.UdpSearchOption).then(function (data) {

                    $scope.rowStatus = {};
                    $scope.invalidIPList = [];
                    $scope.invalidSelectedList = [];
                    $scope.validSelectedList = [];
                    if (data.DeviceCollection != undefined) {

                        for (var i = 0; i < data.DeviceCollection.rows.length; i++) {
                            var row = data.DeviceCollection.rows[i];
                            var ipStatus = 'OK';
                            var invalidIP = false;

                            if (row.name == undefined || row.name.length <= 0) {
                                var deviceType = $scope.deviceSpec[row.device_type_id.id] || $scope.deviceSpec["255"];
                                row.name = deviceType.name + ' ' + row.id + ' (' + row.lan.ip + ')';

//                                if (row.device_type_id.name == undefined) {
                                    row.device_type_id.name = deviceType.name;
//                                }
                            }
                            if (row.device_group_id == undefined) {
                                row.device_group_id = {
                                    id: $scope.deviceGroupTree[0].id,
                                    name: $scope.deviceGroupTree[0].name || $scope.deviceGroupTree[0].text
                                };
                            }

                            if (row.lan == undefined || row.lan.ip === '') {
                                invalidIP = true;
                                ipStatus = MessageProperties.get('device.invalidIPStatus');
                            } else if (row.lan.ip.substring(0, 7) === '169.254') {
                                invalidIP = true;
                                ipStatus = MessageProperties.get('device.dhcpFailStatus');
                            } else if (!row.lan.ip.match(ipRegEx)) {
                                invalidIP = true;
                                ipStatus = MessageProperties.get('device.invalidIPStatus');
                            } else {
                                row.ip = row.lan.ip.split('.');
                                for (var j = 0; j < row.ip.length; j++) {
                                    row.ip[j] = row.ip[j] - 0;
                                }
                            }
                            row.id = row.id - 0;

                            // used for sorting paddedId
//                        row.paddedId = Util.leftPad(row.id, 20, '0');
                            if (row.lan != undefined && row.lan.ip != undefined) {
                                var ipSectionList = row.lan.ip.split(".");
//                        console.log(ipSectionList);
                                row.ipForSorting = '';
                                for (var tempIndex = 0; tempIndex < ipSectionList.length; tempIndex++) {
                                    var oneIpSection = ipSectionList[tempIndex];
                                    row.ipForSorting += Util.leftPad(oneIpSection, 5, '0');
                                }
//                        console.log(row.ipForSorting);
                            }

                            if (invalidIP) {
                                $scope.invalidIPList.push(row);
                            } else {
                                $scope.validSelectedList.push(row);
                            }
                            $scope.rowStatus[row.id] = {
                                checked: !invalidIP,
                                deleted: false,
                                valid: !invalidIP,
                                status: ipStatus,
                                device_id: row.id
                            };
                            row.status = ipStatus;

                        }

                        if ($scope.invalidIPList.length > 0) {
//                            var sortList = {
//                                'ipForSorting': "asc", 'device_type_id.name': "asc"
//                            };
//                            $scope.invalidIPList.keySort(sortList);
//                        data.DeviceCollection.rows.sort(Util.dynamicSort('device_type_id.name'));
                        }
                        if ($scope.validSelectedList.length > 0) {
//                            var sortList = {
//                                'ipForSorting': "asc", 'device_type_id.name': "asc"
//                            };
//
//                            $scope.validSelectedList.keySort(sortList);
//                        data.DeviceCollection.rows.sort(Util.dynamicSort('device_type_id.name'));
                        }
                    }
                    $scope.DeviceCollection = $scope.validSelectedList.concat($scope.invalidIPList);
//                console.log($scope.validSelectedList, $scope.invalidIPList);
//                console.log("list", $scope.DeviceCollection);

                  $scope.isAllChecked = (($scope.validSelectedList.length + $scope.invalidSelectedList.length - $scope.deletedList.length)
                        == $scope.DeviceCollection.length);

                }, function error(data) {

                }).finally(function () {
                    $scope.isLoading = false;

                });
            };

            $scope.onSelect = function ($event, $index) {
                $scope.selectedRow = $index;
                var field = '';
                if ($event.target.tagName == 'TD') {
                    field = angular.element($event.target).attr('field');
                } else {
                    field = angular.element($event.target).parents('td').attr('field');
                }
                if (field != 'device_group_id') {
                    angular.element('#deviceList').find('.selectTypeB.open').removeClass('open');
                    $rootScope.$emit('onCloseDeviceGroupTree');
                }
            };

            angular.element('#checkboxAll').on('change', function () {
                console.log('allChangeCheck: ' + $scope.isAllChecked);

                if ($scope.isAllChecked) {
                    $scope.validSelectedList = [];
                    $scope.invalidSelectedList = [];
                    for (var key in $scope.rowStatus) {
                        if ($scope.rowStatus.hasOwnProperty(key)) {
                            var status = $scope.rowStatus[key];
                            if (!status.deleted) {
                                status.checked = false;
                            }
                        }
                    }
                } else {
                    $scope.validSelectedList = [];
                    $scope.invalidSelectedList = [];

                    for (var i = 0; i < $scope.DeviceCollection.length; i++) {
                        var device = $scope.DeviceCollection[i];
                        var status = $scope.rowStatus[device.id];
                        if (!status.deleted) {
                            status.checked = true;
                            if (status.valid) {
                                $scope.validSelectedList.push($scope.DeviceCollection[i]);
                            } else {
                                $scope.invalidSelectedList.push($scope.DeviceCollection[i]);
                            }
                        }
                    }
                }
                $scope.isAllChecked = !$scope.isAllChecked;
            });

            $scope.changeCheck = function ($index, device) {
                console.log('changeCheck: ' + $index + ' device: ' + device.id);
                if ($scope.rowStatus[device.id].valid) {
                    if ($scope.rowStatus[device.id].checked) {
                        if ($scope.validSelectedList.indexOf(device) < 0) {
                            $scope.validSelectedList.push(device);
                        }
                    } else {
                        var idx = $scope.validSelectedList.indexOf(device);
                        if (idx >= 0) {
                            $scope.validSelectedList.splice(idx, 1);
                        }
                    }
                } else {
                    if ($scope.rowStatus[device.id].checked) {
                        if ($scope.invalidSelectedList.indexOf(device) < 0) {
                            $scope.invalidSelectedList.push(device);
                        }
                    } else {
                        var idx = $scope.invalidSelectedList.indexOf(device);
                        if (idx >= 0) {
                            $scope.invalidSelectedList.splice(idx, 1);
                        }
                    }
                }
                $scope.isAllChecked = (($scope.validSelectedList.length + $scope.invalidSelectedList.length - $scope.deletedList.length)
                    == $scope.DeviceCollection.length);
            };

            $scope.setIPPopupDlg = {};
            $scope.doOpenSetIPPopupDlg = function () {//popup load
                $rootScope.$emit('onCloseDeviceGroupTree');
                if ($scope.validSelectedList.length <= 0 && $scope.invalidSelectedList.length <= 0) {
                    return;
                }
                $scope.setIPPopupDlg.url = 'modules/device/dialog/setipDevice.html';
            };

            $scope.doAfterOpenSetIPPopupDlg = function () {
                var options = {};

                options.modal = true;
                options.shadow = false;
                options.border = false;
                options.title = MessageProperties.get('device.setIp');
                options.width = 682;
                options.setIpdata = $scope.invalidSelectedList.concat($scope.validSelectedList);
                options.setIPDeletedList = [];

                options.onClose = function () {
                    $scope.safeApply(function () {

                        var dlgOptions = angular.element('#setIPPopupDlg').dialog('options');
                        if (dlgOptions.setIPDeletedList && dlgOptions.setIPDeletedList.length > 0) {
                            angular.element('#grdSetIP').datagrid('loadData', dlgOptions.setIpdata);
                            var setIPDeletedList = dlgOptions.setIPDeletedList;
                            for (var i = 0; i < setIPDeletedList.length; i++) {

                                $scope.rowStatus[setIPDeletedList[i].id].checked = false;
                                $scope.rowStatus[setIPDeletedList[i].id].deleted = true;
                                for (var j = 0; j < $scope.DeviceCollection.length; j++) {
                                    var device = $scope.DeviceCollection[j];
                                    if (device != undefined && device.id == setIPDeletedList[i].id) {
                                        $scope.deletedList.push(device);
                                        break;
                                    }
                                }
                            }
                            $scope.validSelectedList = [];
                            $scope.invalidSelectedList = [];
                            for (var j = 0; j < $scope.DeviceCollection.length; j++) {
                                var device = $scope.DeviceCollection[j];
                                var status = $scope.rowStatus[device.id];
                                if (status.valid && status.checked && !status.deleted) {
                                    $scope.validSelectedList.push($scope.DeviceCollection[j])
                                } else if (!status.valid && status.checked && !status.deleted) {
                                    $scope.invalidSelectedList.push($scope.DeviceCollection[j])
                                }
                            }

                            $scope.isAllChecked = (($scope.validSelectedList.length + $scope.invalidSelectedList.length - $scope.deletedList.length)
                                == $scope.DeviceCollection.length);
                        }
                        $scope.setIPPopupDlg.url = '';
                        $("#setIPPopupDlg").dialog('destroy');
                        if ($scope.deletedList.length == $scope.DeviceCollection.length) {
                            $scope.doCloseSearchDeviceDlg();
                        }
                    });
                };
                options.onOpen = function () {
                    var dlgOptions = angular.element("#setIPPopupDlg").dialog('options');
                    if (dlgOptions.setIpdata && dlgOptions.setIpdata.length > 0) {
                        angular.element('#grdSetIP').datagrid('loadData', dlgOptions.setIpdata);
                    }
                };

                angular.element('#setIPPopupDlg').dialog(options);
            };

            $scope.doCloseSearchDeviceDlg = function () {
                angular.element('#searchDevicePopupDlg').dialog('close');
            };

            angular.element('#btnSearchOption').on('focusin', function () {
                angular.element('#btnSearchOption').addClass('open');
            }).on('focusout', function () {
                angular.element('#btnSearchOption').removeClass('open');
            });

            angular.element('#deviceList').on('scroll', function () {
                var target = event.target || (event.srcElement || document);
//            console.log('top: ' + angular.element(target).find('table').position().top);
                var top = angular.element(target).find('table').position().top;
                var intervalGap = -(($scope.maxCount * 40) - 6000);//($scope.maxCount > 20 ? 1200 : 700));
                if ($scope.DeviceCollection.length > $scope.maxCount && intervalGap > top) {
                    $scope.safeApply(function () {
                        $scope.maxCount += 600;
                        if ($scope.DeviceCollection.length < $scope.maxCount) {
                            $scope.maxCount = $scope.DeviceCollection.length;
                        }
                    });
                    console.log('onScroll / maxCount: ' + $scope.maxCount, ' top: ' + top, ' intervalGap: ' + intervalGap);
                }
            });

            $scope.inputEnter = function () {
                angular.element('#btnSearchOption').blur();
            };

            $scope.editName = function () {
                $(event.target || (event.srcElement || document)).prev('.inputEdit').focus();
            };

            $scope.addDevice = function () {
                $rootScope.$emit('onCloseDeviceGroupTree');
                if (angular.element(event.target || event.srcElement).hasClass('disabled')) {
                    return;
                }

                if ($scope.validSelectedList.length <= 0) {
                    return false;
                }

                if ($scope.validSelectedList.length == 1) {
                    API.post('devices', undefined, {'Device': $scope.validSelectedList[0]}, undefined, false, true, true).then(function (data) {
                        console.log('저장됨');

                        $scope.rowStatus[$scope.validSelectedList[0].id].checked = false;
                        $scope.rowStatus[$scope.validSelectedList[0].id].deleted = true;
                        $scope.deletedList.push($scope.validSelectedList[0]);
                        $scope.validSelectedList = [];
                        $scope.isAllChecked = (($scope.validSelectedList.length + $scope.invalidSelectedList.length - $scope.deletedList.length)
                            == $scope.DeviceCollection.length);

                        $scope.addCount++;
                        angular.element('#searchDevicePopupDlg').dialog('options').addCount = $scope.addCount;
                        MessageService.displayInformationMessage(
                            MessageProperties.get('device.addDeviceMessage', 1), function (result) {
                                if ($scope.deletedList.length == $scope.DeviceCollection.length) {
                                    $scope.doCloseSearchDeviceDlg();
                                }
                            });
                    });
                } else if ($scope.validSelectedList.length > 1) {
                    API.post('devices', undefined, {'DeviceCollection': {'total': $scope.validSelectedList.length, 'rows': $scope.validSelectedList}}, undefined, false, true, true).then(function (data) {
                        console.log('저장됨');
                        if (angular.isDefined(data.DeviceCollection) && angular.isDefined(data.DeviceCollection.rows)) {
                            var addedCount = data.DeviceCollection.rows.length;
                            if (addedCount == $scope.validSelectedList.length) {
                                for (var key in $scope.rowStatus) {
                                    if ($scope.rowStatus.hasOwnProperty(key)) {
                                        var status = $scope.rowStatus[key];
                                        if (status.checked && status.valid && !status.deleted) {
                                            status.checked = false;
                                            status.deleted = true;
                                        }
                                    }
                                }
                                $scope.deletedList = $scope.deletedList.concat($scope.validSelectedList);
                                $scope.validSelectedList = [];
                            } else {
                                for (var i = data.DeviceCollection.rows.length - 1; i >= 0; i--) {
                                    var deviceData = data.DeviceCollection.rows[i];
                                    $scope.rowStatus[deviceData.id].checked = false;
                                    $scope.rowStatus[deviceData.id].deleted = true;
                                    $scope.deletedList.push(deviceData);
                                }
                                $scope.validSelectedList = [];
                                for (var i = 0; i < $scope.DeviceCollection.length; i++) {
                                    var device = $scope.DeviceCollection[i];
                                    var status = $scope.rowStatus[device.id];
                                    if (status.valid && status.checked && !status.deleted) {
                                        $scope.validSelectedList.push(device)
                                    }
                                }
                            }
                            $scope.isAllChecked = (($scope.validSelectedList.length + $scope.invalidSelectedList.length - $scope.deletedList.length)
                                == $scope.DeviceCollection.length);

                            $scope.addCount += addedCount;
                            angular.element('#searchDevicePopupDlg').dialog('options').addCount = $scope.addCount;

                            MessageService.displayInformationMessage(
                                MessageProperties.get('device.addDeviceMessage', addedCount), function (result) {
                                    if ($scope.deletedList.length == $scope.DeviceCollection.length) {
                                        $scope.doCloseSearchDeviceDlg();
                                    }
                                });
                        }
                    });
                }
            };
        }]);
});