'use strict';

define([
    'modules/common/services/device-agent'
], function () {
    var app = angular.module('biostarWebApp');

    app.register.controller('SearchSlaveDeviceController', ['$scope', '$rootScope', 'API', 'Const', 'Util', 'DeviceAgent', 'MessageProperties', 'MessageService', function ($scope, $rootScope, API, Const, Util, DeviceAgent, MessageProperties, MessageService) {

        $scope.safeApply = function (fn) {
            var phase = this.$root.$$phase;
            if (phase == '$apply' || phase == '$digest') {
                if (fn && (typeof(fn) === 'function')) {
                    fn();
                }
            } else {
                this.$apply(fn);
            }
        };

        $scope.addCount = 0;

        var options = {
            view: $.extend({}, $.fn.datagrid.defaults.view, {
                onBeforeRender: function (target, rows) {
                    angular.element(target).datagrid('resize');
                },
                onAfterRender: function (target) {
                    $.fn.datagrid.defaults.view.onAfterRender.call(this, target);
                    var opts = angular.element(target).datagrid('options');
                    var grdPanel = angular.element(target).datagrid('getPanel');
                    grdPanel.find(".datagrid-empty").remove();
                    var vc = grdPanel.children('div.datagrid-view');
                    var resultBoxHeight = 360;
                    if (opts.height) {
                        resultBoxHeight = opts.height - 39;
                    }
                    if ($(target).datagrid('getRows').length === 0) {
                        var html = '<div style="width:100%; height:' + resultBoxHeight +
                            'px; font-size:1.333em; line-height: ' + resultBoxHeight +
                            'px; vertical-align: middle; text-align: center;">' + MessageProperties.get('validate.notFound') + '</div>';
                        grdPanel.find('div.datagrid-view2 div.datagrid-body').html(html);
                    }
                }
            }),
            fitColumns: true,
            scrollbarSize: 0,
            iconCls: 'icon-edit',
            //singleSelect:true,
            selectOnCheck: false,
            checkOnSelect: false,
            checkbox: true,
            height: 109,
            columns: [
                [
                    {field: 'ck', checkbox: true},
                    {field: 'id', title: MessageProperties.get('device.info.deviceId'), width: 80},
                    {field: 'name', title: MessageProperties.get('device.info.name'), width: 130,
                        formatter: function (value, row) {
                            if (row.name && row.name.length > 0) {
                                return row.name;
                            } else {
                                row.name = $scope.deviceSpec[row.device_type_id.id].name + ' ' + row.id;
                                row.rs485 = {
                                    mode: Const.RS485_MODE.SLAVE
                                };
                                return row.name;
                            }
                        },
                        editor: 'text'
                    },
                    {field: 'device_type_id', title: MessageProperties.get('device.info.deviceType'), width: 100,
                        formatter: function (value, row) {
                            row['parent_device_id'] = {'id': $scope.selectedDevice.id};
                            return $scope.deviceSpec[value.id].name;
                        }
                    }
                ]
            ],
            onLoadSuccess: function (data) {

                $('#btnSearchSlave').removeAttr('disabled').removeClass('disabled');
            },
            onCheck: function (rowIndex, rowData) {
                $scope.validCheckforAdd();
            },
            onUncheck: function (rowIndex, rowData) {
                $scope.validCheckforAdd();
            },
            onCheckAll: function (rows) {
                $scope.validCheckforAdd();
            },
            onUncheckAll: function (rows) {
                $scope.validCheckforAdd();
            },
            onClickCell: onClickCell
        };
        $.extend($.fn.datagrid.methods, {
            editCell: function (jq, param) {
                return jq.each(function () {
                    var opts = $(this).datagrid('options');
                    var fields = $(this).datagrid('getColumnFields', true).concat($(this).datagrid('getColumnFields'));
                    for (var i = 0; i < fields.length; i++) {
                        console.log("fie:" + fields[i]);
                        var col = $(this).datagrid('getColumnOption', fields[i]);
                        col.editor1 = col.editor;
                        if (fields[i] != param.field) {
                            col.editor = null;
                        }
                    }
                    $(this).datagrid('beginEdit', param.index);
                    for (var i = 0; i < fields.length; i++) {
                        var col = $(this).datagrid('getColumnOption', fields[i]);
                        col.editor = col.editor1;
                    }
                });
            }
        });

        var editIndex = undefined;

        function endEditing() {
            if (editIndex == undefined) {
                return true
            }
            if ($('#searchSlaveDeviceGrid').datagrid('validateRow', editIndex)) {
                $('#searchSlaveDeviceGrid').datagrid('endEdit', editIndex);
                editIndex = undefined;
                return true;
            } else {
                return false;
            }
        }

        function onClickCell(index, field) {
            console.log(index + "=" + field);
            var rows = angular.element('#searchSlaveDeviceGrid').datagrid('getRows');
            var rowData = rows[index];
            if (field == Const.FIELD_NAME.CHECKBOX) {
                var checkedRows = angular.element('#searchSlaveDeviceGrid').datagrid('getChecked');
                var isChecked = false;
                for (var i = 0; i < checkedRows.length; i++) {
                    if (checkedRows[i]['id'] == rowData['id']) {
                        isChecked = true;
                        angular.element('#searchSlaveDeviceGrid').datagrid('uncheckRow', index);
                    }
                }
                if (isChecked == false) {
                    angular.element('#searchSlaveDeviceGrid').datagrid('checkRow', index);
                }
            }
            if (endEditing()) {
                if (field != Const.FIELD_NAME.CHECKBOX) {
                    $('#searchSlaveDeviceGrid').datagrid('selectRow', index)
                        .datagrid('editCell', {index: index, field: field});
                    editIndex = index;
                }
            }
        }

        $scope.selectedDevice = undefined;

        $scope.searchSlaveDevice = function () {
            $scope.selectedDevice = angular.fromJson(sessionStorage.getItem('selectedDeviceNode'));
            //sessionStorage.removeItem('selectedDeviceNode');
            angular.element('#searchSlaveDeviceGrid').datagrid('uncheckAll');
            if (angular.isDefined($scope.selectedDevice)) {

                $('#btnSearchSlave').attr({'disabled': 'disabled'}).addClass('disabled');
                $scope.selected = false;
                API.post('devices/' + $scope.selectedDevice.id + '/discover').then(function (data) {
                    if (data.DeviceCollection == undefined || data.DeviceCollection.rows <= 0) {
                        //                    MessageService.displayWarningMessage(MessageProperties.get("common.");
                        angular.element('#searchSlaveDeviceGrid').datagrid('loadData', []);
                    } else {
                        options.data = data.DeviceCollection.rows || [];
                        angular.element('#searchSlaveDeviceGrid').datagrid('loadData', options.data);
                        $scope.SlaveDeviceCount = data.DeviceCollection.rows.length;
//                        $scope.foundMessage = $scope.selectedDevice.name + ' have ' + data.DeviceCollection.rows.length + (data.DeviceCollection.rows.length <= 1 ? ' slave device.' : ' slave devices.');
                    }
                }, function error(data) {
                    angular.element('#searchSlaveDeviceGrid').datagrid('loadData', []);
                }).finally(function () {
                    $('#btnSearchSlave').removeAttr('disabled').removeClass('disabled');
                });
            } else {
                angular.element('#searchSlaveDeviceGrid').datagrid('loadData', []);
            }
        };

        $scope.deviceSpec = undefined;
        $scope.init = function() {
            DeviceAgent.getDeviceSpec().then(function success(data) {
                $scope.deviceSpec = data;
            }).finally(function () {
                angular.element('#searchSlaveDeviceGrid').datagrid(options);
                $scope.searchSlaveDevice();
            });
        };

        $scope.validCheckforAdd = function () {
            var checkedItems = angular.element('#searchSlaveDeviceGrid').datagrid('getChecked');
            $scope.safeApply(function () {
                $scope.selected = checkedItems.length > 0;
            });

            return $scope.selected;
        };


        $scope.addedDeviceList = [];
        $scope.doAddSlaveDeviceToMaster = function () {
            if ($scope.selected != true) {
                return false;
            }

            $scope.addedDeviceList = angular.element('#searchSlaveDeviceGrid').datagrid('getChecked');

            API.post('devices', undefined, { 'DeviceCollection': {'rows': $scope.addedDeviceList}}).then(function (data) {
                for (var i = $scope.addedDeviceList.length; i--;) {
                    $scope.addCount++;
                    var idx = angular.element('#searchSlaveDeviceGrid').datagrid('getRowIndex', $scope.addedDeviceList[i]);
                    angular.element('#searchSlaveDeviceGrid').datagrid('deleteRow', idx);
                }
                angular.element('#searchSlaveDevicePopupDlg').dialog('options').addCount = $scope.addCount;

                $scope.validCheckforAdd();

                MessageService.displayInformationMessage(MessageProperties.get("device.deviceAdded"), function (result) {
                    var crrData = $('#searchSlaveDeviceGrid').datagrid('getData');
                    if (crrData.rows.length == 0) {
                        $scope.doCloseSearchDeviceDlg();
                    }
                });
            });
        };


        $scope.doCloseSearchDeviceDlg = function () {
            angular.element('#searchSlaveDevicePopupDlg').dialog('close');
        };
    }]);
});