'use strict';

define(function () {
    var app = angular.module('biostarWebApp');

    app.register.controller('SetipDeviceController', ['$scope', '$rootScope', '$timeout', 'API', 'Const', 'Util', 'MessageProperties', 'MessageService', function ($scope, $rootScope, $timeout, API, Const, Util, MessageProperties, MessageService) {

        $scope.ipAddress = Const.REGULAR_EXPRESSION.IP_ADDRESS;// /(((\d{1,2})|(1\d{2})|(2[0-4]\d)|(25[0-5]))\.){3}(((\d{1,2})|(1\d{2})|(2[0-4]\d)|(25[0-5])))/;
        $scope.number = /^[0-9]+$/;

        $scope.safeApply = function (fn) {
            var phase = this.$root.$$phase;
            if (phase == '$apply' || phase == '$digest') {
                if (fn && (typeof(fn) === 'function')) {
                    fn();
                }
            } else {
                this.$apply(fn);
            }
        };

        $scope.DeviceUpdate = {};
        $scope.setIPDeletedList = [];

        $('#grdSetIP').datagrid({
            fitColumns: true,
            iconCls: 'icon-edit',
            scrollbarSize: 0,
            singleSelect: true,
            selectOnCheck: true,
            checkOnSelect: true,
            autoRowHeight: false,
            //checkbox : true,
            columns: [
                [
//			          {field:'ck', checkbox:true},
                    {field: 'id', title: MessageProperties.get('device.info.deviceId'), width: 110},
                    {field: 'device_type_id', title: MessageProperties.get('device.info.deviceType'), width: 160,
                        formatter: function (v, r, i) {
                            return v.name;
                        }
                    }
                ]
            ],
            onSelect: function (rowIndex, rowData) {
                $scope.selectChanged();
            },
            onUnselect: function (rowIndex, rowData) {
                if ($('#grdSetIP').datagrid('options').singleSelect == true) {
                    return;
                }
                $scope.selectChanged();
            },
            onSelectAll: function (rowIndex, rowData) {
                $scope.selectChanged();
            },
            onUnselectAll: function (rowIndex, rowData) {
                $scope.selectChanged();
            },
            onLoadSuccess: function (data) {
                if (data.total > 0) {
                    $timeout(function () {
                        $('#grdSetIP').datagrid('selectRow', 0)
                    }, 10);
                }
            }

        });

        $scope.initSubPopup = function () {

        };
        $scope.selectChanged = function () {
            var rows = $('#grdSetIP').datagrid('getSelections');    //선택된 데이터 배열
            console.log('getSelections:');
            console.log(rows);

            $scope.enable_dhcp = true;
            $scope.connection_mode = false;
            //$scope.DeviceUpdate = $scope.originDevice = {'Device' : rows[0] || {}};
            if (rows.length <= 0) {
                return;
            }
            $scope.loadedData = false;
            API.post('devices/udp_get', undefined, {'Device': rows[0]}).then(function (data) {
                if (data && angular.isDefined(data.Device)) {
                    $scope.originDevice = data;
                    $scope.safeApply(function () {
                        angular.copy($scope.originDevice, $scope.DeviceUpdate);
                        $scope.enable_dhcp = $scope.DeviceUpdate.Device.lan.enable_dhcp == 'true';
                        $scope.connection_mode = Const.CONNECTION_MODE[$scope.DeviceUpdate.Device.lan.connection_mode || '_default'];
                        $scope.loadedData = true;
                    });
                } else {
                    $scope.enable_dhcp = true;
                    $scope.connection_mode = false;
                    $scope.originDevice = undefined;
                    $scope.DeviceUpdate = undefined;
                }
            }, function error(data) {
                $scope.enable_dhcp = true;
                $scope.connection_mode = false;
                $scope.originDevice = undefined;
                $scope.DeviceUpdate = undefined;
            });
        };

        $scope.invalidIPAddress = false;
        $scope.invalidSubnetMask = false;
        $scope.invalidGateway = false;
        $scope.invalidServerIP = false;
        var focusElement = undefined;
        var validationCheck = function () {
            var valid = true;

            $scope.invalidIPAddress = false;
            $scope.invalidSubnetMask = false;
            $scope.invalidGateway = false;
            $scope.invalidServerIP = false;

            focusElement = undefined;

            if ($scope.DeviceUpdate.Device != undefined && $scope.DeviceUpdate.Device.lan != undefined) {
                if ($scope.DeviceUpdate.Device.lan.enable_dhcp == false){
                    if (!$scope.ipAddress.test($scope.DeviceUpdate.Device.lan.ip) || $scope.DeviceUpdate.Device.lan.ip == '0.0.0.0') {
                        $scope.invalidIPAddress = true;
                        valid = false;
                        focusElement = focusElement || '#txtIPAddress';
                    }
                    if (!$scope.ipAddress.test($scope.DeviceUpdate.Device.lan.subnet_mask) || $scope.DeviceUpdate.Device.lan.subnet_mask == '0.0.0.0') {
                        $scope.invalidSubnetMask = true;
                        valid = false;
                        focusElement = focusElement || '#txtSubnetMask';
                    }
                    if (!$scope.ipAddress.test($scope.DeviceUpdate.Device.lan.gateway) || $scope.DeviceUpdate.Device.lan.gateway == '0.0.0.0') {
                        $scope.invalidGateway = true;
                        valid = false;
                        focusElement = focusElement || '#txtGateway';
                    }
                }
                if ($scope.DeviceUpdate.Device.lan.connection_mode == Const.CONNECTION_MODE.DEVICE_TO_SERVER) {
                    if (!$scope.ipAddress.test($scope.DeviceUpdate.Device.lan.server_ip) || $scope.DeviceUpdate.Device.lan.server_ip == '0.0.0.0') {
                        $scope.invalidServerIP = true;
                        valid = false;
                        focusElement = focusElement || '#txtServerIP';
                    }
                }
            }

            if (focusElement != undefined) {
                angular.element(focusElement).focus();
            }
            return valid;
        };

        $scope.doApplySetIP = function (invalid) {
            if (!$scope.loadedData) {
                MessageService.displayWarningMessage(MessageProperties.get("validate.noNetworkDataForSelectedDevice"));
                return false;
            }
            $scope.DeviceUpdate.Device.lan.enable_dhcp = $scope.enable_dhcp;
            $scope.DeviceUpdate.Device.lan.connection_mode = Const.CONNECTION_MODE[$scope.connection_mode];

            if (!validationCheck() || invalid) {
                MessageService.displayWarningMessage(MessageProperties.get("validate.invalidIP"), function() {
                    if (focusElement != undefined) {
                        angular.element(focusElement).focus();
                    }
                });
                return false;
            }

//            if ($scope.loadedData && $scope.originDevice) {
//                var saveData = Util.extractModifiedFields($scope.originDevice, $scope.DeviceUpdate);
//                if (saveData == undefined) {
//                    MessageService.displayWarningMessage(MessageProperties.get("common."));
//                    return false;
//                }
//            }

           MessageService.displayConfirmMessage(MessageProperties.get('confirm.saveSetIP'), function (dialogResult) {
                if (dialogResult) {

                    API.post('devices/udp_put', undefined, $scope.DeviceUpdate).then(function (data) {
                        console.log(data);
                        var IPData = $('#grdSetIP').datagrid('getData');
                        for (var i = 0; i < IPData.rows.length; i++) {
                            if (IPData.rows[i].id == $scope.DeviceUpdate.Device.id) {
                                $scope.setIPDeletedList.push(IPData.rows[i]);
                                var rowIndexSetIP = $('#grdSetIP').datagrid('getRowIndex', IPData.rows[i]);
                                $("#grdSetIP").datagrid('deleteRow', rowIndexSetIP);
                                break;
                            }
                        }
                        angular.element('#setIPPopupDlg').dialog('options').setIPDeletedList = $scope.setIPDeletedList;
                        if (IPData.total < 1) {
                            $scope.doCloseSetIPDlg();
                        } else {
                            $timeout(function () {
                                $('#grdSetIP').datagrid('selectRow', 0)
                            }, 10);
                        }

                    });
                }
            });
        };


        $scope.doCloseSetIPDlg = function () {
            $("#setIPPopupDlg").dialog('close');
        };

    }]);

});