'use strict';

define([
    'modules/common/services/device-agent',
    'modules/common/services/user-agent',
    'modules/common/services/monitoringAgent',
    'modules/common/services/door-agent',
    'modules/common/services/datagridFilterService',
    'modules/common/datagrid/columnSettingController',
    'modules/common/datagrid/columnSettingService',
    'modules/common/print/printOptionController',
    'modules/common/print/printOptionService'
], function () {

    var app = angular.module('biostarWebApp');

    app.register.controller('AlertHistoryController', [
        '$rootScope', '$scope', '$window', '$location', '$timeout', '$compile', '$filter', 'Util', 'API', 'Const', 'DeviceAgent', 'UserAgent', 'DoorAgent', 'MonitoringAgent', 'DatagridFilterService', 'MessageProperties', 'ColumnSettingService', 'PrintOptionService', 'PreferenceService', 'MessageService',
        function ($rootScope, $scope, $window, $location, $timeout, $compile, $filter, Util, API, Const, DeviceAgent, UserAgent, DoorAgent, MonitoringAgent, DatagridFilterService, MessageProperties, ColumnSettingService, PrintOptionService, PreferenceService, MessageService) {

            $scope.gridId = Const.GRID_ID.ALERT_HISTORY;

            $scope.Parameters = {
                offset: 0,
                limit: 15,
                total: 0,
                query: ''
            };

            $scope.dropDownMenu = [
                {
                    menu: MessageProperties.get('menu.print'),
                    func: function () {
                        $scope.openPrintOptionPopupDlg();
                    }
                },
                {
                    menu: MessageProperties.get('menu.columnSetting'),
                    func: function () {
                        $scope.openColumnSettingPopupDlg();
                    }
                }
            ];

            $scope.searchOption = {
                "Query": {
                    "offset": $scope.Parameters.offset,
                    "limit": $scope.Parameters.limit,
                    "orders": [
                        {
                            "column": "event_id.datetime",
                            "descending": "true"
                        }
                    ]
                }
            };

            function buildQueryObject() {
                var queryObject = {};
                queryObject.offset = $scope.Parameters.offset;
                queryObject.limit = $scope.Parameters.limit;
                queryObject.conditions = [];
                if ($scope.filterObject.dateRange.datetime != undefined && $scope.filterObject.dateRange.datetime.length > 0 &&
                    ($scope.filterObject.dateRange.fromDatetime != '' || $scope.filterObject.dateRange.toDatetime != '')) {
                    queryObject.conditions.push({
                        "column": "event_id.datetime",
                        "operator": Const.QUERY_OPERATOR.BETWEEN,
                        "values": [
                            $scope.filterObject.dateRange.fromDatetime,
                            $scope.filterObject.dateRange.toDatetime
                        ]
                    });
                }

                if ($scope.filterObject.deviceId != undefined && $scope.filterObject.deviceId.length > 0) {
                    queryObject.conditions.push({
                        "column": "event_id.device_id.id",
                        "operator": Const.QUERY_OPERATOR.IN,
                        "values": Util.buildIdArrayFromObjectArray($scope.filterObject.deviceId)
                    });
                }

                if ($scope.filterObject.deviceName != undefined && $scope.filterObject.deviceName.length > 0) {
                    queryObject.conditions.push({
                        "column": "event_id.device_id.id",
                        "operator": Const.QUERY_OPERATOR.IN,
                        "values": Util.buildIdArrayFromObjectArray($scope.filterObject.deviceName)
                    });
                }

                if ($scope.filterObject.doorName != undefined && $scope.filterObject.doorName.length > 0) {
                    queryObject.conditions.push({
                        "column": "event_id.door_id.id",
                        "operator": Const.QUERY_OPERATOR.IN,
                        "values": Util.buildIdArrayFromObjectArray($scope.filterObject.doorName)
                    });
                }

                if ($scope.filterObject.eventName != undefined && $scope.filterObject.eventName.length > 0) {
                    queryObject.conditions.push({
                        "column": "event_id.event_type_id.code",
                        "operator": Const.QUERY_OPERATOR.IN,
                        "values": Util.buildIdArrayFromObjectArray($scope.filterObject.eventName)
                    });
                }

                if ($scope.filterObject.userName != undefined && $scope.filterObject.userName.length > 0) {
                    queryObject.conditions.push({
                        "column": "event_id.user_id.user_id",
                        "operator": Const.QUERY_OPERATOR.IN,
                        "values": Util.buildIdArrayFromObjectArray($scope.filterObject.userName)
                    });
                }

                queryObject.conditions.push({
                    "column": "status",
                    "operator": Const.QUERY_OPERATOR.EQUAL,
                    "values": [Const.ALERT_STATUS.ACKNOWLEDGE]
                });

                queryObject.orders = [];
                queryObject.orders.push({column: 'event_id.datetime', descending: true});

                return queryObject;
            }

            $scope.search = function (paramQueryString, deferFunctionName) {
                $scope.filterObject = DatagridFilterService.getFilterObject();
                $scope.gridInformation = MessageProperties.get('common.searching');

                $scope.searchOption.Query = buildQueryObject();

                API.post('alerts/search', deferFunctionName, $scope.searchOption).then(function (data) {
                    if (angular.isDefined(data.AlertCollection) && data.AlertCollection.rows.length > 0) {
                        $scope.gridInformation = '';
                        $scope.Parameters.total = data.AlertCollection.total;
                        angular.element("#" + $scope.gridId).datagrid('loadData', data.AlertCollection.rows);
                    } else {
                        $scope.gridInformation = MessageProperties.get('common.notFound');
                        $scope.Parameters.total = 0;
                        angular.element("#" + $scope.gridId).datagrid('loadData', []);
                    }
                });
            };

            $scope.SearchCallback = {
                fn: $scope.search,
                reset: function () {
                    $scope.search();
                }
            };

            $scope.$on('Acknowledged', function (event, ackAlert) {
                if (ackAlert != undefined) {
                    $scope.SearchCallback.reset();
                }
            });

            $scope.gridInformation = MessageProperties.get('common.notFound');

            $scope.init = function () {

                $scope.filterOptionItems = DatagridFilterService.getFilterOptionItems();
                DatagridFilterService.clearFilterObject();
                $scope.filterObject = DatagridFilterService.getFilterObject();
                DatagridFilterService.setCallbackFunction($scope.search);

                angular.element("#" + $scope.gridId).datagrid({
                    view: $.extend({}, $.fn.datagrid.defaults.view, {
                        onBeforeRender: function (target, rows) {
                            angular.element("#" + $scope.gridId).datagrid('resize');

                        },
                        onAfterRender: function (target) {
                            var dataGridBodyElement = angular.element("#" + $scope.gridId).parent(".datagrid-view").find(".datagrid-view2").find(".datagrid-body");
                            if (angular.element("#" + $scope.gridId).datagrid("getRows").length === 0) {
                                var html = '<div class="listNotFound">' +
                                    $scope.gridInformation + '</div>';
                                dataGridBodyElement.html(html);
                            }
                            angular.element('.alertHistoryBtn').on('click', function (evt) {
                                MessageService.displaySimpleMessage($(this).data('comment'), this);
                            });
                        }
                    }),
                    scrollbarSize: 0,
                    fitColumns: true,
                    autoRowHeight: false,
                    resize: true,
                    singleSelect: true,
                    columns: [
                        [
                            {field: 'event_image', title: '<p class="item icon"><img src="images/contents/photo_icon_w.png" alt=""></p>', width: 50, align: 'center', hidden: true, sortable: false,
                                formatter: function (value, row, index) {
                                    return value ? '<img src="images/contents/photo_icon_b.png" alt="">' : '';
                                }
                            },
                            {field: 'event_id.datetime', title: MessageProperties.get('common.date') + ' <div ngbss-datetime-range-filter ' +
                                'data-from-datetime="filterObject.dateRange.fromDatetime" data-to-datetime="filterObject.dateRange.toDatetime" ' +
                                'data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)"></div>',
                                filterValues: 'filterObject.dateRange.datetime',
                                filterType: Const.DATA_FILTER.EVENT_DATE,
                                width: 20, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
//                                return $filter('BssDatetime')(new Date(row.event_id.datetime), PreferenceService.getDateFormat() + " " + Const.DEFAULT_TIME_FORMAT);
                                    return $filter('BssDatetimeLongTerm')(new Date(row.event_id.datetime));
                                }
                            },
//                        {field: 'event_id.door_id.name', title: 'Door <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.DOOR_NAME
//                            + '" data-item-list="filterOptionItems.doorList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
//                            filterValues: 'filterObject.doorName',
//                            filterType: Const.DATA_FILTER.DOOR_NAME,
//                            width: 100, align: 'center', sortable: false,
//                            formatter: function (value, row, index) {
//                                return Util.formatIdAndNameSpan(row.event_id.door_id);
//                            }
//                        },
                            {field: 'event_id.device_id.name', title: MessageProperties.get('common.device') + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.DEVICE_NAME
                                + '" data-item-list="filterOptionItems.deviceList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
                                filterValues: 'filterObject.deviceName',
                                filterType: Const.DATA_FILTER.DEVICE_NAME,
                                width: 15, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    return Util.formatIdAndNameSpan(row.event_id.device_id);
                                }
                            },
                            {field: 'event_id.user_id.name', title: MessageProperties.get('common.user') + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.USER_NAME
                                + '" data-item-list="filterOptionItems.userList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
                                filterValues: 'filterObject.userName',
                                filterType: Const.DATA_FILTER.USER_NAME,
                                width: 15, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    if (angular.isDefined(row.event_id.user_id)) {
                                        row.event_id.user_id.id = row.event_id.user_id.user_id;
                                    }
                                    return Util.formatIdAndNameSpan(row.event_id.user_id);
                                }
                            },
                            {field: 'event_id.event_type_id.name', title: MessageProperties.get('common.alarm') + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.EVENT_NAME + '" ' +
                                'data-item-list="filterOptionItems.eventList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
                                filterValues: 'filterObject.eventName',
                                filterType: Const.DATA_FILTER.EVENT_NAME,
                                width: 20, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    return Util.formatIdAndNameSpan(row.event_id.event_type_id);
                                }
                            },
                            {field: 'ack_user_id.name', title: MessageProperties.get('alertHistory.ackBy'),
                                width: 10, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    if (angular.isDefined(row.ack_user_id)) {
                                        row.ack_user_id.id = row.ack_user_id.user_id;
                                    }
                                    return Util.formatIdAndNameSpan(row.ack_user_id);
                                }
                            },
                            {field: 'ack_datetime', title: MessageProperties.get('alertHistory.status'),
                                width: 20, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    var returnValue = '';
                                    if (row.ack_datetime != undefined) {
                                        returnValue += $filter('BssDatetime')(new Date(row.ack_datetime));
                                    }
                                    if (row.ack_message != undefined && row.ack_message.trim().length > 0) {
                                        var tempMessage = row.ack_message;
                                        returnValue += '<button style="margin-left: 10px; vertical-align: middle;" class="ibtnHistory alertHistoryBtn" data-comment="' + tempMessage + '"></button>';
//                                    returnValue += '<a style="margin-left: 10px; vertical-align: middle;" href="javascript:void(null);" class="ibtnHistory easyui-tooltip" title="' + row.ack_message + '" ></a>';
                                    }
                                    return returnValue;
                                }
                            }
                        ]
                    ],
                    data: [],
                    onLoadSuccess: function (data) {
                        angular.element('#' + $scope.gridId).datagrid('setFilterRow', {
                            callback: function () {
                                $compile(angular.element('#' + $scope.gridId).datagrid('getPanel').find('.datagrid-view2 .datagrid-htable'))($scope);
                            }
                        });
                    }
                });
//            $scope.search();

                $scope.$watch(function () {
                    return  angular.element('.panel.datagrid').width();
                }, function (nv, ov) {
                    if (nv !== ov) {
                        angular.element("#" + $scope.gridId).datagrid('resize');
                    }
                }, true);

                angular.element($window).on('resize', function ($event) {
                    var event = $event | window.event;
                    angular.element("#" + $scope.gridId).datagrid('resize');
                });

                $scope.$watch('Parameters', function (nv, ov) {
                    if (nv !== ov) {
                        var queryObject = buildQueryObject();
                        queryObject.collection_name = Const.REPORT_COLLECTION_NAME.ALERT;
                        queryObject.object = "query";
                        PrintOptionService.setQueryObject(queryObject);
                    }
                }, true);

                PrintOptionService.getReportColumnNameFunction = function (gridColumnName) {
                    var reportColumnName = gridColumnName;
                    if (gridColumnName == "event_id.door_id.name") {
                        return "event_id.door_id.name"
                    } else if (gridColumnName == "event_id.device_id.name") {
                        return "event_id.device_id.name"
                    } else if (gridColumnName == "event_id.user_id.name") {
                        return "event_id.user_id.name"
                    } else if (gridColumnName == "event_id.event_type_id.name") {
                        return "event_id.event_type_id.name"
                    } else if (gridColumnName == "ack_user_id.name") {
                        return "ack_user_id.name"
                    }
                    return reportColumnName;
                };
                PrintOptionService.getReportColumnTypeFunction = function (gridColumnName) {
                    if (gridColumnName == "event_id.datetime") {
                        return "datetime"
                    } else if (gridColumnName == "ack_datetime") {
                        return "datetime"
                    } else {
                        return "string";
                    }
                }

                $scope.reportUrl = Const.API.BASE_URL + 'alerts/report';
                $scope.openPrintOptionPopupDlg = PrintOptionService.openPrintOptionPopupDlg;
                $scope.openColumnSettingPopupDlg = ColumnSettingService.openColumnSettingPopupDlg;

                ColumnSettingService.setGridId($scope.gridId);
                ColumnSettingService.setIncludeAllColumnsForPrinting(true);
                ColumnSettingService.setCallbackFunction($scope.SearchCallback.reset);
                var customColumnSetting = ColumnSettingService.getColumnSettingFromLocalStroage();
                if (customColumnSetting) {
                    $timeout(function () {
                        ColumnSettingService.applyChangedColumnsToGrid(customColumnSetting, false);
                    }, 100);
                }

                $scope.$on(Const.UI_EVENT_TYPE.MONITORING_FILTER_CLICKED, function (event, message) {
                    if (message.viewId == Const.MONITORING_VIEW.ALERT_HISTORY) {
//                    console.log("deviceStatus MonitoringFilterClicked");
                        if (message.filterId != undefined) {
                            var filterObject = DatagridFilterService.findFilterObjectByIdFromLocalStorage(message.filterId);
                            DatagridFilterService.setFilterObject(filterObject);
                        } else {
                            DatagridFilterService.clearFilterObject();
                        }
                        $scope.Parameters.offset = 0;
                        $scope.Parameters.total = 0;
                        $scope.search();
                    }
                });
            };

        }]);
});