'use strict';

define([
    'modules/common/services/monitoringAgent',
    'modules/common/services/device-agent',
    'modules/common/services/datagridFilterService',
    'modules/common/datagrid/columnSettingController',
    'modules/common/datagrid/columnSettingService',
    'modules/common/print/printOptionController',
    'modules/common/print/printOptionService'
], function () {

    var app = angular.module('biostarWebApp');

    app.register.controller('DeviceStatusController', [
        '$rootScope', '$scope', '$window', '$location', '$timeout', '$compile', '$filter', 'Util', 'API', 'Const', 'DeviceAgent', 'MonitoringAgent', 'DatagridFilterService', 'MessageProperties', 'ColumnSettingService', 'PrintOptionService', 'PrivilegeService',
        function ($rootScope, $scope, $window, $location, $timeout, $compile, $filter, Util, API, Const, DeviceAgent, MonitoringAgent, DatagridFilterService, MessageProperties, ColumnSettingService, PrintOptionService, PrivilegeService) {

        $scope.gridId = "grdDeviceStatus";
        $scope.clearAlarmAction = false;


        function hideAllButtons() {
            $scope.clearAlarmAction = false;
        }

        function displayAllButtons() {
            $scope.clearAlarmAction = true;
        }

        function toggleAllButtons() {
            if ($scope.checkIdList != undefined && $scope.checkIdList.length > 0) {
                displayAllButtons();
            } else {
                hideAllButtons();
            }
        }

        function buildCheckedCollection() {
            var params = {
                DeviceCollection: {
                    total: $scope.checkIdList.length,
                    rows: []
                }
            };
            for (var i = 0; i < $scope.checkIdList.length; i++) {
                params.DeviceCollection.rows.push({
                    id: $scope.checkIdList[i]
                });
            }
            return params;
        }

        $scope.doClearAlarm = function () {
            console.log("doClearAlarm with " + $scope.checkIdList);
            API.post('devices/clear_alarm', undefined, buildCheckedCollection());
        };

        $scope.dropDownMenu = [
//            {
//                menu: MessageProperties.get('menu.print'),
//                func: function () {
//                    $scope.openPrintOptionPopupDlg();
//                }
//            },
            {
                menu: MessageProperties.get('menu.columnSetting'),
                func: function () {
                    $scope.openColumnSettingPopupDlg();
                }
            }
        ];

        function safeApply(fn) {
            $scope.$root.$$phase ? fn() : $scope.$apply(fn);
        }

        function loadData(deviceListWithoutStatus) {
            var deviceList = [];
            var count = 0;
            for (var i = 0; i < deviceListWithoutStatus.length; i++) {
                var deviceWithStatus = DeviceAgent.getAllDevicesMapWithStatus()[deviceListWithoutStatus[i].id];
                if (deviceWithStatus != undefined && deviceWithStatus.device_type_id != undefined) {
                    deviceList[count] = deviceWithStatus;
                    count++;
                }
            }


            DatagridFilterService.setOriginalDataList(deviceList);
            angular.element("#" + $scope.gridId).datagrid('loadData', deviceList);
            DatagridFilterService.filterData($scope.gridId);

            for (i = deviceListWithoutStatus.length; i--;) {
                var rowData = deviceListWithoutStatus[i];
                if ($scope.checkIdList.indexOf(rowData.id) > -1) {
                    angular.element('#'+$scope.gridId).datagrid('checkRow', i);
                }
            }
        }

        $scope.gridInformation = MessageProperties.get('common.notFound');

            if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.MONITORING)) {
                $scope.readOnly = true;
            }

        $scope.init = function () {

            $scope.checkIdList = [];
            DatagridFilterService.setOriginalDataList([]);

            $scope.filterOptionItems = DatagridFilterService.getFilterOptionItems();
            DatagridFilterService.clearFilterObject();
            $scope.filterObject = DatagridFilterService.getFilterObject();

            $scope.callbackAfterApplyFilter = function () {
                $scope.filterObject = DatagridFilterService.getFilterObject();
                if (DeviceAgent.getAllDevicesWithoutStatus().then) {
                    DeviceAgent.getAllDevicesWithoutStatus().then(function success(deviceList) {
                        loadData(deviceList);
                    });
                } else {
                    loadData(DeviceAgent.getAllDevicesWithoutStatus());
                }
            };

            DatagridFilterService.setCallbackFunction($scope.callbackAfterApplyFilter);

            angular.element("#" + $scope.gridId).datagrid({
                view: $.extend({}, $.fn.datagrid.defaults.view, {
                    onBeforeRender: function (target, rows) {
                        angular.element("#" + $scope.gridId).datagrid('resize');

                    },
                    onAfterRender: function (target) {
                        var dataGridBodyElement = angular.element("#" + $scope.gridId).parent(".datagrid-view").find(".datagrid-view2").find(".datagrid-body");
                        if (angular.element("#" + $scope.gridId).datagrid("getRows").length === 0) {
                            var html = '<div class="listNotFound">' +
                                $scope.gridInformation + '</div>';
                            dataGridBodyElement.html(html);
                        }
                    }
                }),
                scrollbarSize: 0,
                fitColumns: true,
                autoRowHeight: false,
                resize: true,
                singleSelect: true,
                selectOnCheck: false,
                checkOnSelect: false,
                checkbox: !$scope.readOnly,
                columns: [
                    [
                        {field: ($scope.readOnly ? 'none' : Const.FIELD_NAME.CHECKBOX), checkbox: !$scope.readOnly, hidden: $scope.readOnly},
                        {field: 'event_image', title: '<p class="item icon"><img src="images/contents/photo_icon_w.png" alt=""></p>', width: 50, align: 'left', hidden: true, sortable: false,
                            formatter: function (value, row, index) {
                                return value ? '<img src="images/contents/photo_icon_b.png" alt="">' : '';
                            }
                        },
                        {field: 'name', title: MessageProperties.get("common.deviceName") + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.DEVICE_NAME
                            + '" data-item-list="filterOptionItems.deviceList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
                            filterValues: 'filterObject.deviceName',
                            filterType: Const.DATA_FILTER.DEVICE_NAME,
                            width: 100, align: 'left', sortable: false,
                            formatter: function (value, row, index) {
                                return Util.formatIdAndNameSpan(row);
                            }
                        },
                        {field: 'status', title: MessageProperties.get("common.deviceStatus"),
                            width: 100, align: 'left', sortable: false,
                            formatter: function (value, row, index) {
                                var returnValue = '';
//                                if (angular.isDefined(row.door_status) && row.door_status.sensor_status_time) {
//                                    returnValue += row.door_status.sensor_status_time;
//                                }
                                if (angular.isDefined(row.status) && row.status) {
                                    returnValue = Const.DEVICE_STATUS[row.status];
                                }
                                return returnValue;
                            }
                        },
                        {field: 'alarm', title: MessageProperties.get("common.alarm"),
                            width: 100, align: 'left', sortable: false,
                            formatter: function (value, row, index) {
//                                if (angular.isDefined(row.alert) && angular.isDefined(row.alert.event_id)) {
//                                    return Util.formatEventSpan(row.alert.event_id);
//                                }
//                                return '';
                                return Util.formatEventSpan(row.alarm);
                            }
                        },
                        {field: 'last_event', title: MessageProperties.get("common.lastEvent"),
                            width: 160, align: 'left', sortable: false,
                            formatter: function (value, row, index) {
                                return Util.formatEventSpan(row.last_event);
                            }
                        }
                    ]
                ],
                data: [],
                onLoadSuccess: function (data) {
                    angular.element('#' + $scope.gridId).datagrid('setFilterRow', {
                        callback: function () {
                            $compile(angular.element('#' + $scope.gridId).datagrid('getPanel').find('.datagrid-view2 .datagrid-htable'))($scope);
                        }
                    });
                    angular.element(".btnChkSelect").parent().remove();
                },
                onCheck: function (rowIndex, rowData) {
                    safeApply(function () {
                        var idx = $scope.checkIdList.indexOf(rowData.id);
                        if(idx == -1) {
                            $scope.checkIdList.push(rowData.id);
                        }
                        toggleAllButtons();
                    });
                },
                onUncheck: function (rowIndex, rowData) {
                    safeApply(function () {
                        var index = $scope.checkIdList.indexOf(rowData.id);
                        if (index > -1) {
                            $scope.checkIdList.splice(index, 1);
                        }
                        toggleAllButtons();
                    });
                },
                onCheckAll: function (rows) {
                    safeApply(function () {
                        $scope.checkIdList = [];
                        for (var i = 0; i < rows.length; i++) {
                            $scope.checkIdList.push(rows[i].id);
                        }
                        toggleAllButtons();
                    });
                },
                onUncheckAll: function (rows) {
                    safeApply(function () {
                        $scope.checkIdList = [];
                        toggleAllButtons();
                    });
                },
                onClickCell: function (rowIndex, field) {
                    $rootScope.clickCellOnGrid($scope.gridId, rowIndex, field, "id", function (rowData) {
                        // do nothing
                    });
                }
            });

            $scope.callbackAfterApplyFilter();

            $scope.$watch(function () {
                return  angular.element('.panel.datagrid').width();
            }, function (nv, ov) {
                if (nv !== ov) {
                    angular.element("#" + $scope.gridId).datagrid('resize');
                }
            }, true);

            angular.element($window).on('resize', function ($event) {
                var event = $event | window.event;
                angular.element("#" + $scope.gridId).datagrid('resize');
            });

            // TODO BSS-195 BSS-192 print :: server api not implemented
            PrintOptionService.getReportColumnNameFunction = function (gridColumnName) {
                return gridColumnName;
            };
            $scope.reportUrl = Const.API.BASE_URL + 'monitoring/devicestatus/report';
            $scope.openPrintOptionPopupDlg = PrintOptionService.openPrintOptionPopupDlg;
            $scope.openColumnSettingPopupDlg = ColumnSettingService.openColumnSettingPopupDlg;

            ColumnSettingService.setGridId($scope.gridId);
            ColumnSettingService.setCallbackFunction($scope.callbackAfterApplyFilter);
            var customColumnSetting = ColumnSettingService.getColumnSettingFromLocalStroage();
            if (customColumnSetting) {
                $timeout(function () {
                    ColumnSettingService.applyChangedColumnsToGrid(customColumnSetting, false);
                }, 100);
            }

            $scope.$on(Const.UI_EVENT_TYPE.MONITORING_FILTER_CLICKED, function (event, message) {
                if (message.viewId == Const.MONITORING_VIEW.DEVICE) {
//                    console.log("deviceStatus MonitoringFilterClicked");
                    if (message.filterId != undefined) {
                        var filterObject = DatagridFilterService.findFilterObjectByIdFromLocalStorage(message.filterId);
                        DatagridFilterService.setFilterObject(filterObject);
                    } else {
                        DatagridFilterService.clearFilterObject();
                    }
                    $scope.callbackAfterApplyFilter();
                    hideAllButtons();
//                    $scope.$apply();
                }
            });

            $scope.$on('DeviceStatusReceived', function (event, messageObject) {
                var rows = angular.element("#" + $scope.gridId).datagrid('getRows');
                var foundIndex = -1;
                for (var i = 0; i < rows.length; i++) {
                    if (messageObject.id == rows[i].id) {
                        foundIndex = i;
                        break;
                    }
                }
                if (foundIndex > -1) {
                    rows[foundIndex].status = messageObject.status;
//                    console.log(rows[foundIndex]);
                    if (rows[foundIndex].ck == undefined) {
                        rows[foundIndex].ck = false;
                    }
                    angular.element("#" + $scope.gridId).datagrid('refreshRow', foundIndex);
                }
            });

            $scope.$on('DeviceEventReceived', function (event, messageObject) {
                var rows = angular.element("#" + $scope.gridId).datagrid('getRows');
                var foundIndex = -1;
                for (var i = 0; i < rows.length; i++) {
                    if (messageObject.device_id.id == rows[i].id) {
                        foundIndex = i;
                        break;
                    }
                }
                if (foundIndex > -1) {
                    rows[foundIndex].last_event = messageObject;
                    if (Const.CLEAR_ALARM_EVENT_TYPE_CODES.indexOf(messageObject.event_type_id.code) > -1) {
                        rows[foundIndex].alarm = undefined;
                    } else if (messageObject.event_type_id.enable_alert == Const.BOOLEAN.TRUE) {
                        rows[foundIndex].alarm = messageObject;
                    }
//                    console.log(rows[foundIndex]);
                    if (rows[foundIndex].ck == undefined) {
                        rows[foundIndex].ck = false;
                    }
                    angular.element("#" + $scope.gridId).datagrid('refreshRow', foundIndex);
                }
            });

            $scope.$on('DeviceAlertReceived', function (event, messageObject) {
                var rows = angular.element("#" + $scope.gridId).datagrid('getRows');
                var foundIndex = -1;
                for (var i = 0; i < rows.length; i++) {
                    if (messageObject.event_id.device_id.id == rows[i].id) {
                        foundIndex = i;
                        break;
                    }
                }
                if (foundIndex > -1) {
                    rows[foundIndex].alert = messageObject;
//                    console.log(rows[foundIndex]);
                    if (rows[foundIndex].ck == undefined) {
                        rows[foundIndex].ck = false;
                    }
                    angular.element("#" + $scope.gridId).datagrid('refreshRow', foundIndex);
                }
            });
            hideAllButtons();
        };

    }]);
});