'use strict';

define([
    'modules/common/services/monitoringAgent',
    'modules/common/services/door-agent',
    'modules/common/services/datagridFilterService',
    'modules/common/datagrid/columnSettingController',
    'modules/common/datagrid/columnSettingService',
    'modules/common/print/printOptionController',
    'modules/common/print/printOptionService'
], function () {

    var app = angular.module('biostarWebApp');

    app.register.controller('DoorStatusController', [
        '$rootScope', '$scope', '$window', '$location', '$timeout', '$compile', '$filter', 'Util', 'API', 'Const', 'DoorAgent', 'MonitoringAgent', 'DatagridFilterService', 'MessageProperties', 'ColumnSettingService', 'PrintOptionService', 'PrivilegeService',
        function ($rootScope, $scope, $window, $location, $timeout, $compile, $filter, Util, API, Const, DoorAgent, MonitoringAgent, DatagridFilterService, MessageProperties, ColumnSettingService, PrintOptionService, PrivilegeService) {

            $scope.gridId = "grdDoorStatus";
            $scope.lockAction = false;
            $scope.unlockAction = false;
            $scope.openAction = false;
            $scope.clearAlarmAction = false;
            $scope.clearAntiPassBackViolationAction = false;


            function hideAllButtons() {
                $scope.lockAction = false;
                $scope.unlockAction = false;
                $scope.openAction = false;
                $scope.clearAlarmAction = false;
                $scope.clearAntiPassBackViolationAction = false;
            }

            function displayAllButtons() {
                $scope.lockAction = true;
                $scope.unlockAction = true;
                $scope.openAction = true;
                $scope.clearAlarmAction = true;
                $scope.clearAntiPassBackViolationAction = true;
            }

            function toggleAllButtons() {
                if ($scope.checkIdList != undefined && $scope.checkIdList.length > 0) {
                    displayAllButtons();
                } else {
                    hideAllButtons();
                }
            }

            function buildCheckedDoorCollection() {
                var params = {
                    DoorCollection: {
                        total: $scope.checkIdList.length,
                        rows: []
                    }
                };
                for (var i = 0; i < $scope.checkIdList.length; i++) {
                    params.DoorCollection.rows.push({
                        id: $scope.checkIdList[i]
                    });
                }
                return params;
            }

            $scope.doLock = function () {
                console.log("doLock with " + $scope.checkIdList);
                API.post('doors/lock', undefined, buildCheckedDoorCollection());
            };
            $scope.doUnlock = function () {
                console.log("doUnlock with " + $scope.checkIdList);
                API.post('doors/unlock', undefined, buildCheckedDoorCollection());
            };
            $scope.doOpen = function () {
                console.log("doOpen with " + $scope.checkIdList);
                API.post('doors/open', undefined, buildCheckedDoorCollection());
            };
            $scope.doClearAlarm = function () {
                console.log("doClearAlarm with " + $scope.checkIdList);
                API.post('doors/clear_alarm', undefined, buildCheckedDoorCollection());
            };
            $scope.doClearAntiPassBackViolation = function () {
                console.log("doClearAntiPassBackViolation with " + $scope.checkIdList);
                API.post('doors/clear_apb', undefined, buildCheckedDoorCollection());
            };

            $scope.dropDownMenu = [
//            {
//                menu: MessageProperties.get('menu.print'),
//                func: function () {
//                    $scope.openPrintOptionPopupDlg();
//                }
//            },
                {
                    menu: MessageProperties.get('menu.columnSetting'),
                    func: function () {
                        $scope.openColumnSettingPopupDlg();
                    }
                }
            ];

            function safeApply(fn) {
                $scope.$root.$$phase ? fn() : $scope.$apply(fn);
            }

            function loadDataToGrid(doorListWithoutStatus) {
                var doorList = [];
                for (var i = 0; i < doorListWithoutStatus.length; i++) {
                    var doorWithStatus = DoorAgent.getAllDoorMapWithStatus()[doorListWithoutStatus[i].id];
                    doorList[i] = doorWithStatus;
                }
                DatagridFilterService.setOriginalDataList(doorList);
                angular.element("#" + $scope.gridId).datagrid('loadData', doorList);
                DatagridFilterService.filterData($scope.gridId);

                for (i = doorListWithoutStatus.length; i--;) {
                    var rowData = doorListWithoutStatus[i];
                    if ($scope.checkIdList.indexOf(rowData.id) > -1) {
                        angular.element('#'+$scope.gridId).datagrid('checkRow', i);
                    }
                }
            }

            function loadData(doorListWithoutStatus) {
                API.post('doors/status', undefined, undefined, undefined, false, false).then(function (data) {
                    if (data.DoorStatusCollection != undefined && data.DoorStatusCollection.rows) {
                        for (var i = 0; i < data.DoorStatusCollection.rows.length; i++) {
                            var doorStatus = data.DoorStatusCollection.rows[i];
                            var deviceObject = DoorAgent.getAllDoorMapWithStatus()[doorStatus.door_id.id];
//                        console.log(doorStatus, deviceObject);
                            if (deviceObject != undefined) {
                                deviceObject.door_status = doorStatus;
                            }
                        }
                    }
                    loadDataToGrid(doorListWithoutStatus);
                }, function error(data) {
                    loadDataToGrid(doorListWithoutStatus);
                });
            }

            $scope.gridInformation = MessageProperties.get('common.notFound');

            if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.MONITORING)) {
                $scope.readOnly = true;
            }

            $scope.init = function () {

                $scope.checkIdList = [];
                DatagridFilterService.setOriginalDataList([]);

                $scope.filterOptionItems = DatagridFilterService.getFilterOptionItems();
                DatagridFilterService.clearFilterObject();
                $scope.filterObject = DatagridFilterService.getFilterObject();

                $scope.callbackAfterApplyFilter = function () {
                    $scope.filterObject = DatagridFilterService.getFilterObject();
                    if (DoorAgent.getAllDoorsWithoutStatus().then) {
                        DoorAgent.getAllDoorsWithoutStatus().then(function success(doorList) {
                            loadData(doorList);
                        });
                    } else {
                        loadData(DoorAgent.getAllDoorsWithoutStatus());
                    }
                };

                DatagridFilterService.setCallbackFunction($scope.callbackAfterApplyFilter);

                function formatDateTimeForStatusDatetime(datetimeInString) {
                    var returnValue = '';
                    if (angular.isDefined(datetimeInString)) {
                        var dateObject = new Date(datetimeInString);
                        returnValue += $filter('BssTime')(dateObject);
                    }
                    return returnValue;
                }

                angular.element("#" + $scope.gridId).datagrid({
                    view: $.extend({}, $.fn.datagrid.defaults.view, {
                        onBeforeRender: function (target, rows) {
                            angular.element("#" + $scope.gridId).datagrid('resize');

                        },
                        onAfterRender: function (target) {
                            var dataGridBodyElement = angular.element("#" + $scope.gridId).parent(".datagrid-view").find(".datagrid-view2").find(".datagrid-body");
                            if (angular.element("#" + $scope.gridId).datagrid("getRows").length === 0) {
                                var html = '<div class="listNotFound">' +
                                    $scope.gridInformation + '</div>';
                                dataGridBodyElement.html(html);
                            }
                        }
                    }),
                    scrollbarSize: 0,
                    fitColumns: true,
                    autoRowHeight: false,
                    resize: true,
                    singleSelect: true,
                    selectOnCheck: false,
                    checkOnSelect: false,
                    checkbox: !$scope.readOnly,
                    columns: [
                        [
                            {field: ($scope.readOnly ? 'none' : Const.FIELD_NAME.CHECKBOX), checkbox: !$scope.readOnly, hidden: $scope.readOnly},
                            {field: 'event_image', title: '<p class="item icon"><img src="images/contents/photo_icon_w.png" alt=""></p>', width: 50, align: 'left', hidden: true, sortable: false,
                                formatter: function (value, row, index) {
                                    return value ? '<img src="images/contents/photo_icon_b.png" alt="">' : '';
                                }
                            },
                            {field: 'name', title: MessageProperties.get("common.doorName") + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.DOOR_NAME
                                + '" data-item-list="filterOptionItems.doorList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
                                filterValues: 'filterObject.doorName',
                                filterType: Const.DATA_FILTER.DOOR_NAME,
                                width: 100, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    return Util.formatIdAndNameSpan(row);
                                }
                            },
                            {field: 'door_status', title: MessageProperties.get("common.doorStatus"),
                                width: 100, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    var returnValue = '';
//                                console.log(row.door_status, Const.MONITORING_DOOR_STATUS[row.door_status.opened]);
                                    if (angular.isDefined(row.door_status) && row.door_status.opened != undefined) {
                                        returnValue = Const.MONITORING_DOOR_STATUS[row.door_status.opened];
                                    }
                                    return returnValue;
                                }
                            },
                            {field: 'relay_status', title: MessageProperties.get("common.doorRelay"),
                                width: 100, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    var returnValue = '';
                                    if (angular.isDefined(row.door_status) && row.door_status.unlocked != undefined) {
                                        returnValue = Const.MONITORING_DOOR_RELAY_STATUS[row.door_status.unlocked];
                                    }
                                    return returnValue;
                                }
                            },
                            {field: 'alarm', title: MessageProperties.get("common.alarm"),
                                width: 100, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
//                                var returnValue = '';
//                                if (angular.isDefined(row.door_status) && row.door_status.alarm_status_time) {
//                                    returnValue += formatDateTimeForStatusDatetime(row.door_status.alarm_status_time);
//                                }
//                                if (angular.isDefined(row.door_status) && angular.isDefined(row.door_status.alarm_status)) {
//                                    returnValue = Const.DOOR_ALARM_STATUS[row.door_status.alarm_status];
//                                }
//                                return returnValue;
                                    return Util.formatEventSpan(row.alarm);
                                }
                            },
                            {field: 'last_event', title: MessageProperties.get("common.lastEvent"),
                                width: 160, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    return Util.formatEventSpan(row.last_event);
                                }
                            }
                        ]
                    ],
                    data: [],
                    onLoadSuccess: function (data) {
                        angular.element('#' + $scope.gridId).datagrid('setFilterRow', {
                            callback: function () {
                                $compile(angular.element('#' + $scope.gridId).datagrid('getPanel').find('.datagrid-view2 .datagrid-htable'))($scope);
                            }
                        });
                        angular.element(".btnChkSelect").parent().remove();
                    },
                    onCheck: function (rowIndex, rowData) {
                        safeApply(function () {
                            var idx = $scope.checkIdList.indexOf(rowData.id);
                            if(idx == -1) {
                                $scope.checkIdList.push(rowData.id);
                            }
                            toggleAllButtons();
                        });
                    },
                    onUncheck: function (rowIndex, rowData) {
                        safeApply(function () {
                            var index = $scope.checkIdList.indexOf(rowData.id);
                            if (index > -1) {
                                $scope.checkIdList.splice(index, 1);
                            }
                            toggleAllButtons();
                        });
                    },
                    onCheckAll: function (rows) {
                        safeApply(function () {
                            $scope.checkIdList = [];
                            for (var i = 0; i < rows.length; i++) {
                                $scope.checkIdList.push(rows[i].id);
                            }
                            toggleAllButtons();
                        });
                    },
                    onUncheckAll: function (rows) {
                        safeApply(function () {
                            $scope.checkIdList = [];
                            toggleAllButtons();
                        });
                    },
                    onClickCell: function (rowIndex, field) {
                        $rootScope.clickCellOnGrid($scope.gridId, rowIndex, field, "id", function (rowData) {
                            // do nothing
                        });
                    }
                });

                $scope.callbackAfterApplyFilter();

                $scope.$watch(function () {
                    return  angular.element('.panel.datagrid').width();
                }, function (nv, ov) {
                    if (nv !== ov) {
                        angular.element("#" + $scope.gridId).datagrid('resize');
                    }
                }, true);

                angular.element($window).on('resize', function ($event) {
                    var event = $event | window.event;
                    angular.element("#" + $scope.gridId).datagrid('resize');
                });

                // TODO BSS-195 BSS-192 print :: server api not implemented
                PrintOptionService.getReportColumnNameFunction = function (gridColumnName) {
                    return gridColumnName;
                };
                $scope.reportUrl = Const.API.BASE_URL + 'monitoring/doorstatus/report';
                $scope.openPrintOptionPopupDlg = PrintOptionService.openPrintOptionPopupDlg;
                $scope.openColumnSettingPopupDlg = ColumnSettingService.openColumnSettingPopupDlg;

                ColumnSettingService.setGridId($scope.gridId);
                ColumnSettingService.setCallbackFunction($scope.callbackAfterApplyFilter);
                var customColumnSetting = ColumnSettingService.getColumnSettingFromLocalStroage();
                if (customColumnSetting) {
                    $timeout(function () {
                        ColumnSettingService.applyChangedColumnsToGrid(customColumnSetting, false);
                    }, 100);
                }

                $scope.$on(Const.UI_EVENT_TYPE.MONITORING_FILTER_CLICKED, function (event, message) {
                    if (message.viewId == Const.MONITORING_VIEW.DOOR) {
//                    console.log("doorStatus MonitoringFilterClicked");
                        if (message.filterId != undefined) {
                            var filterObject = DatagridFilterService.findFilterObjectByIdFromLocalStorage(message.filterId);
                            DatagridFilterService.setFilterObject(filterObject);
                        } else {
                            DatagridFilterService.clearFilterObject();
                        }
                        $scope.callbackAfterApplyFilter();
                        hideAllButtons();
//                    $scope.$apply();
                    }
                });

                $scope.$on('DoorEventReceived', function (event, messageObject) {
                    var rows = angular.element("#" + $scope.gridId).datagrid('getRows');
                    var foundIndex = -1;
                    for (var i = 0; i < rows.length; i++) {
                        if (messageObject.door_id.id == rows[i].id) {
                            foundIndex = i;
                            break;
                        }
                    }
                    if (foundIndex > -1) {
                        if (rows[foundIndex].door_status == undefined) {
                            rows[foundIndex].door_status = {};
                        }
                        DoorAgent.buildStatusObjectByEvent(messageObject, rows[foundIndex].door_status);
                        rows[foundIndex].last_event = messageObject;
                        if (Const.CLEAR_ALARM_EVENT_TYPE_CODES.indexOf(messageObject.event_type_id.code) > -1) {
                            rows[foundIndex].alarm = undefined;
                        } else if (messageObject.event_type_id.enable_alert == Const.BOOLEAN.TRUE) {
                            rows[foundIndex].alarm = messageObject;
                        }
//                    console.log(rows[foundIndex]);
                        if (rows[foundIndex].ck == undefined) {
                            rows[foundIndex].ck = false;
                        }
                        angular.element("#" + $scope.gridId).datagrid('refreshRow', foundIndex);
                    }
                });

                $scope.$on('DoorAlarmReceived', function (event, messageObject) {
                    var rows = angular.element("#" + $scope.gridId).datagrid('getRows');
                    var foundIndex = -1;
                    for (var i = 0; i < rows.length; i++) {
                        if (messageObject.door_id.id == rows[i].id) {
                            foundIndex = i;
                            break;
                        }
                    }
                    if (foundIndex > -1) {
                        rows[foundIndex].alarm = messageObject;
//                    console.log(rows[foundIndex]);
                        if (rows[foundIndex].ck == undefined) {
                            rows[foundIndex].ck = false;
                        }
                        angular.element("#" + $scope.gridId).datagrid('refreshRow', foundIndex);
                    }
                });

                $scope.$on('DoorAlertReceived', function (event, messageObject) {
                    var rows = angular.element("#" + $scope.gridId).datagrid('getRows');
                    var foundIndex = -1;
                    for (var i = 0; i < rows.length; i++) {
                        if (messageObject.event_id.door_id.id == rows[i].id) {
                            foundIndex = i;
                            break;
                        }
                    }
                    if (foundIndex > -1) {
                        rows[foundIndex].alert = messageObject;
//                    console.log(rows[foundIndex]);
                        if (rows[foundIndex].ck == undefined) {
                            rows[foundIndex].ck = false;
                        }
                        angular.element("#" + $scope.gridId).datagrid('refreshRow', foundIndex);
                    }
                });
                hideAllButtons();
            };

        }]);
});