'use strict';

define([
    'modules/common/services/device-agent',
    'modules/common/services/user-agent',
    'modules/common/services/monitoringAgent',
    'modules/common/services/door-agent',
    'modules/common/services/datagridFilterService',
    'modules/common/datagrid/columnSettingController',
    'modules/common/datagrid/columnSettingService',
    'modules/common/print/printOptionController',
    'modules/common/print/printOptionService',
    'modules/monitoring/monitoringTreeController',
    'modules/monitoring/pastEventController',
    'modules/monitoring/doorStatusController',
    'modules/monitoring/deviceStatusController',
    'modules/monitoring/alertHistoryController',
    'modules/monitoring/realtimeEventController'
], function () {
    var app = angular.module('biostarWebApp');
    app.register.controller('MonitoringController', [
        '$rootScope', '$scope', '$timeout', 'Const', 'Util', 'API', 'DatagridFilterService', 'MonitoringAgent', 'WebSocketService',
        function ($rootScope, $scope, $timeout, Const, Util, API, DatagridFilterService, MonitoringAgent, WebSocketService) {

            $scope.selectedMonitoringView = '';
            $scope.SearchCallback = {fn: function () {
            }};

            $scope.applyFilter = function (filterSelectedInput) {
                var savedFilter = DatagridFilterService.addFilterItems(filterSelectedInput);
                if ($scope.filterId == undefined && savedFilter != undefined && savedFilter.id != undefined) {
                    $scope.filterId = savedFilter.id;
                }
            };

            $scope.removeFilter = function ($event, filterType, filterItem) {
                DatagridFilterService.removeFilterItem($event, filterType, filterItem);
            };

            $scope.displayView = function (viewId, filterId) {
//            console.log("MonitoringController::displayView::" + viewId + "::" + filterId);
                DatagridFilterService.setViewId(viewId);
                $scope.viewId = viewId;

                switch (viewId) {
                    case Const.MONITORING_VIEW.PAST_EVENT :
                        $scope.selectedMonitoringView = 'modules/monitoring/pastEvent.html';
//                    $scope.title = "Past Event";
                        break;
                    case Const.MONITORING_VIEW.REAL_TIME_EVENT :
                        $scope.selectedMonitoringView = 'modules/monitoring/realtimeEvent.html';
//                    $scope.title = "Real-Time Event";
                        break;
                    case Const.MONITORING_VIEW.DEVICE :
//                    $scope.title = "Device Status";
                        $scope.selectedMonitoringView = 'modules/monitoring/deviceStatus.html';
                        break;
                    case Const.MONITORING_VIEW.DOOR :
//                    $scope.title = "Door Status";
                        $scope.selectedMonitoringView = 'modules/monitoring/doorStatus.html';
                        break;
                    case Const.MONITORING_VIEW.AUDIT :
//                    $scope.title = "Audit Trail";
//                    $scope.selectedMonitoringView = 'modules/monitoring/tableView.html';
                        break;
                    case Const.MONITORING_VIEW.ALERT_HISTORY :
//                    $scope.title = "Alert History";
                        $scope.selectedMonitoringView = 'modules/monitoring/alertHistory.html';
                        break;
                }
            };

            $.extend($.fn.datagrid.methods, {
                editCell: function (jq, param) {
                    return jq.each(function () {
//                    console.log(this);
//                    console.log(jq);
//                    console.log(param);
                        //var opts = $(this).datagrid('options');
                        var fields = $(this).datagrid('getColumnFields', true).concat($(this).datagrid('getColumnFields'));
                        for (var i = 1; i < fields.length; i++) {
                            //console.log("fie:" + fields[i]);
                            if (fields[i] == 'ck') {
                                continue;
                            }
                            var col = $(this).datagrid('getColumnOption', fields[i]);
                            if (angular.isUndefined(col.editor)) {
                                continue;
                            }
                            if (typeof col.editor === 'string') {
                                col.editor1 = col.editor.toString();
                            } else {
                                col.editor1 = {};
                                angular.copy(col.editor, col.editor1);
                            }
                            if (fields[i] != param.field) {
                                col.editor = null;
                            }
                        }
                        $(this).datagrid('beginEdit', param.index);
                        for (var i = 1; i < fields.length; i++) {
                            if (fields[i] == 'ck') {
                                continue;
                            }
                            var col = $(this).datagrid('getColumnOption', fields[i]);
                            if (angular.isUndefined(col.editor)) {
                                continue;
                            }
                            if (typeof col.editor1 === 'string') {
                                col.editor = col.editor1.toString();
                            } else {
                                col.editor = {};
                                angular.copy(col.editor1, col.editor);
                            }
                        }
                    });
                },
                getRowIndexByID: function (jq, param) {
                    var index = -1;
                    jq.each(function () {
                        try {
                            var id = (angular.isDefined(param.id) ? param.id : param);
                            var data = $(this).datagrid('getData');
                            for (var i = 0; i < data.rows.length; i++) {
                                if (id == data.rows[i].id) {
                                    index = i;
                                    return index;
                                }
                            }
                        } catch (ex) {
                            console.error(ex);
                        }
                    });
                    return index;
                },
                setFilterRow: function (jq, param) {
                    var index = -1;
                    jq.each(function () {
                        try {
                            var tblHeader = angular.element(this).datagrid('getPanel').find('.datagrid-view2 .datagrid-htable');
                            if (tblHeader.find('.filter-result-row').length == 0) {
                                var fields = $(this).datagrid('getColumnFields');
                                var html = '<tr class="filter-result-row">';
                                for (var i = 0; i < fields.length; i++) {
                                    var columnOption = $(this).datagrid('getColumnOption', fields[i]);
                                    if (columnOption.hidden) {
                                        html += '<td class="hide" field="' + columnOption.field + '">';
                                    } else {
                                        html += '<td field="' + columnOption.field + '">';
                                    }
                                    if (columnOption.filterValues) {
                                        html += '<div class="filterResult scrollY" ng-show="' + columnOption.filterValues + '.length">';
                                        html += '    <ul class="' + columnOption.cellClass + '">';
                                        html += '        <li ng-repeat="filterValue in ' + columnOption.filterValues + '">';
                                        var removeLinkHtml = '<a href="javascript:void(null);" class="btnDel" ng-click="removeFilter($event, \'' + columnOption.filterType + '\', filterValue)"></a>';
                                        if (fields[i].indexOf("datetime") > -1) {
                                            html += '{{filterValue | date:"' + $rootScope.dateFormat + '"}}';
                                            html += '<a ng-show="$index == 0" href="javascript:void(null);" class="btnDel" ng-click="removeFilter($event, \'' + columnOption.filterType + '\', filterValue)"></a>';
                                        } else if (fields[i] == "device_id") {
                                            html += '{{ filterValue.id }} ';
                                            html += removeLinkHtml;
                                        } else if (fields[i] == "user_id") {
                                            html += '{{ filterValue.id + "(" + ( filterValue.name || "-" ) + ")" }} ';
                                            html += removeLinkHtml;
                                        } else {
                                            html += '{{ filterValue.name || filterValue.id }} ';
                                            html += removeLinkHtml;
                                        }
                                        console.log(fields[i]);
                                        html += '       </li>';
                                        html += '    </ul>';
                                        html += '</div>';
                                    }
                                    html += '</td>';
                                }
                                html += '</tr>';
                                tblHeader.append(html);
                                if (param.callback) {
                                    param.callback();
                                }
                            }
                            if (DatagridFilterService.isFilterObjectEmpty()) {
                                angular.element(".filter-result-row").hide();
                            } else {
                                angular.element(".filter-result-row").show();
                            }
                        } catch (ex) {
                            console.error(ex);
                        }
                    });
                    return index;
                }
            });

            DatagridFilterService.init();

            $rootScope.realtimeEventCheckIntervalId = false;

            function clearRealtimeEventCheckInterval() {
                Util.debug("clearRealtimeEventCheckInterval interval=" + $rootScope.realtimeEventCheckIntervalId);
                clearInterval($rootScope.realtimeEventCheckIntervalId);
                $rootScope.realtimeEventCheckIntervalId = false;
            }

            $scope.init = function () {
                WebSocketService.startRealtimeEvent();
                $scope.message = {};
                if (MonitoringAgent.getInitialViewId() == undefined) {
                    MonitoringAgent.setInitialViewId(Const.MONITORING_VIEW.PAST_EVENT);
                }
                $scope.message.viewId = MonitoringAgent.getInitialViewId();

            };

            $scope.$on('$includeContentRequested', function (next, current) {
                Util.debug("$includeContentRequested");
                clearRealtimeEventCheckInterval();
            });

            $scope.$on('$routeChangeStart', function (next, current) {
                if (API.isLoggedIn() == true && WebSocketService.isRealtimeEventStarted() == true) {
                    WebSocketService.stopRealtimeEvent();
                }
                Util.debug("$routeChangeStart");
                clearRealtimeEventCheckInterval();
            });

            $scope.$on(Const.UI_EVENT_TYPE.MONITORING_TREE_ITEM_CLICKED, function (event, message) {
//            console.log('MonitoringTreeItemClicked:' + message.viewId + ":" + message.filterId);
                $scope.displayView(message.viewId, message.filterId);
                var currentViewId = '';
                if ($scope.message != undefined) {
                    currentViewId = $scope.message.viewId;
                }
                $scope.message = message;
                if (currentViewId == message.viewId) {
                    $scope.$broadcast(Const.UI_EVENT_TYPE.MONITORING_FILTER_CLICKED, {
                        viewId: message.viewId,
                        filterId: message.filterId
                    });
                } else {
                    // do nothing $includeContentLoaded event will be fired
                }
                if (!$scope.$$phase) {
                    $scope.$apply();
                }
            });

            $scope.$on("$includeContentLoaded", function (event) {
                if ($scope.message != undefined) {
                    $scope.$broadcast(Const.UI_EVENT_TYPE.MONITORING_FILTER_CLICKED, {
                        viewId: $scope.message.viewId,
                        filterId: $scope.message.filterId
                    });
                }
            });

            $scope.$on('MonitoringViewChanged', function (event, message) {
                $scope.displayView(message.viewId);
            });


        }]);
});