'use strict';

define([
    'modules/common/services/datagridFilterService',
    'modules/common/services/monitoringAgent'
], function () {
    var app = angular.module('biostarWebApp');

    app.register.controller('MonitoringTreeController', [
        '$scope', '$rootScope', '$compile', '$q', 'API', 'Const', '$location', 'Util', '$timeout', 'DatagridFilterService', 'MonitoringAgent', 'MessageProperties', 'MessageService',
        function ($scope, $rootScope, $compile, $q, API, Const, $location, Util, $timeout, DatagridFilterService, MonitoringAgent, MessageProperties, MessageService) {

            $rootScope.$on(Const.UI_EVENT_TYPE.ADD_FILTER_CLICKED, function (event, object) {
                $scope.doAddNewFilter();
            });
            $rootScope.$on(Const.UI_EVENT_TYPE.SAVE_FILTER_CLICKED, function (event, object) {
                $scope.doSaveFilter();
            });


            function addNode(parentNode, id, name) {
                var newNode = {
                    text: name,
                    id: id
                };
                $('#MonitoringTree').tree('insert', {parent: parentNode.target, data: [newNode]});
            };

            function createParentNode(viewId, name, iconClass, state, checked) {
                var node = {
                    id: viewId,
                    "viewId": viewId,
                    iconCls: iconClass,
                    text: name,
                    name: name,
                    state: state,
                    checked: checked
                };
                return node;
            }

            function buildTree() {
                var treeData = new Array();
                treeData.push(createParentNode(Const.MONITORING_VIEW.PAST_EVENT, MessageProperties.get("monitoring.pastEvent"), "tree-item-past-event", 'open', false));
                treeData.push(createParentNode(Const.MONITORING_VIEW.REAL_TIME_EVENT, MessageProperties.get("monitoring.realTimeEvent"), "tree-item-real-time-event", 'open', false));
                treeData.push(createParentNode(Const.MONITORING_VIEW.DEVICE, MessageProperties.get("monitoring.deviceStatus"), "tree-item-device-status", 'open', false));
                treeData.push(createParentNode(Const.MONITORING_VIEW.DOOR, MessageProperties.get("monitoring.doorStatus"), "tree-item-door-status", 'open', false));
//            treeData.push(createParentNode(Const.MONITORING_VIEW.AUDIT, MessageProperties.get("monitoring.auditTrail"), "tree-item-past-event", 'open', false));
                treeData.push(createParentNode(Const.MONITORING_VIEW.ALERT_HISTORY, MessageProperties.get("monitoring.alertHistory"), "tree-item-alert-history", 'open', false));

                var isAndroid = (/android/gi).test(navigator.appVersion);
                angular.element('#monitoringTree').tree({
                    data: treeData,
                    lines: false,
                    animate: false,
                    checkbox: false,
                    dnd: false,
//                onClick: function (node) {
//                    console.log("onClick");
//                    var localViewId = '';
//                    if (node.parent != undefined){
//                        localViewId = node.parent.id;
//                    } else {
//                        localViewId = node.id;
//                    }
//                    $rootScope.$emit(Const.UI_EVENT_TYPE.MONITORING_TREE_ITEM_CLICKED, {
//                        viewId: localViewId,
//                        filterId: node.filterId
//                    });
//                },
                    onSelect: function (node) {
//                    console.log("onSelect");
                        if (node.doNotRefreshPage == true) {
                            node.doNotRefreshPage = false;
                            return;
                        }
                        var localViewId = '';
                        if (node.parent != undefined) {
                            localViewId = node.parent.id;
                        } else {
                            localViewId = node.id;
                        }
                        $scope.$emit(Const.UI_EVENT_TYPE.MONITORING_TREE_ITEM_CLICKED, {
                            viewId: localViewId,
                            filterId: node.filterId
                        });
                    },
                    onAfterEdit: function (node) {
                        var newName = node.text.trim() || MessageProperties.get('common.newFilter');
                        angular.element('#monitoringTree').tree('update', {target: node.target, name: newName });
                        DatagridFilterService.updateFilterName(node.parent.id, node.filterId, newName);
                        node.name = newName;
                    },
                    formatter: function (node) {
                        return '<span title="' + node.name + '">' + node.name + '</span>';
                    }
                });
            };

            function createFilterNode(viewId, filterId, name) {
                var parentNode = angular.element("#monitoringTree").tree("find", viewId);

                if (filterId == undefined) {
                    filterId = DatagridFilterService.getNextFilterId(viewId);
                }
                if (name == undefined) {
                    name = MessageProperties.get('common.newFilter');
                }
                var nodeId = viewId + "_" + filterId;
                var newNode = {
                    "parent": {
                        "id": parentNode.id
                    },
                    id: nodeId,
                    iconCls: "tree-item-filter",
                    viewId: viewId,
                    inherited: true,
                    doNotRefreshPage: false,
                    filterId: filterId,
                    text: name,
                    name: name
                };

                angular.element('#monitoringTree').tree('append', {
                    "parent": parentNode.target,
                    "data": newNode
                });
                var children = angular.element('#monitoringTree').tree('getChildren', parentNode.target);
                var html = '<span class="fr tree-click-ignore treeEditor" >';
                html += '<a href="javascript:void(null)" ng-privilege hide-when-not-allowed="true" ng-click="doEditFilter(\'' + nodeId + '\', \'' + filterId + '\')" class="iconEdit tree-click-ignore"><img src="images/common/icon_edit.png" alt="Edit" class="tree-click-ignore"></a>';
                html += '<a href="javascript:void(null)" ng-privilege hide-when-not-allowed="true" style="padding-left: 5px;" ng-click="doDeleteFilter(\'' + nodeId + '\', \'' + filterId + '\')" class="iconDelete tree-click-ignore"><img src="images/common/icon_delete.png" alt="Delete" class="tree-click-ignore"></a>';
                html += '</span>';
                var newElement = angular.element(children[children.length - 1].target);
                newElement.append(html);
                $compile(newElement)($scope);
                return children[children.length - 1];
            };

            function addNewFilter(viewId) {
                var newNode = createFilterNode(viewId);
                DatagridFilterService.saveFilter(newNode.filterId);
                newNode.doNotRefreshPage = true;
                angular.element('#monitoringTree').tree('select', newNode.target);
                angular.element('#monitoringTree').tree('beginEdit', newNode.target);
            }

            $scope.doAddNewFilter = function () {
                addNewFilter(DatagridFilterService.getViewId());
            };

            $scope.doSaveFilter = function () {
                var selectedNode = angular.element("#monitoringTree").tree("getSelected");
                var newNode = undefined;
                if (selectedNode == undefined) {
                    addNewFilter(Const.MONITORING_VIEW.PAST_EVENT);
                } else {
                    if (selectedNode.filterId == undefined) {
                        addNewFilter(selectedNode.viewId);
                    } else {
                        DatagridFilterService.saveFilter(selectedNode.filterId);
                    }
                }
            };

            $scope.doEditFilter = function (nodeId, filterId) {
                var node = angular.element("#monitoringTree").tree("find", nodeId),
                    inputNode;
                angular.element('#monitoringTree').tree('beginEdit', node.target);
            };
            $scope.doDeleteFilter = function (nodeId, filterId) {
                MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                    if (confirmYn) {
                        var node = angular.element("#monitoringTree").tree("find", nodeId);
                        DatagridFilterService.deleteFilter(node.parent.id, node.filterId);
                        var parentNode = angular.element("#monitoringTree").tree("find", node.parent.id);
                        angular.element('#monitoringTree').tree('remove', node.target);
                        angular.element('#monitoringTree').tree('select', parentNode.target);
                    }
                });
            };

            function selectNode(viewId) {
//            console.log("MonitoringTreeController::selectNode::" + viewId);
                var defaultNode = angular.element("#monitoringTree").tree("find", viewId);
                angular.element('#monitoringTree').tree('select', defaultNode.target);
            }

            $scope.init = function () {
                buildTree();
                var filterObjectList = DatagridFilterService.getFilterObjectListFromStorage();
                var rootNodeList = angular.element("#monitoringTree").tree("getRoots");
                for (var i = 0; i < rootNodeList.length; i++) {
                    var node = rootNodeList[i];
                    if (filterObjectList != undefined) {
                        var viewFilterObject = filterObjectList[node.id];
                        if (viewFilterObject != undefined) {
                            for (var index = 0; index < viewFilterObject.filterList.length; index++) {
                                var filterObject = viewFilterObject.filterList[index];
                                createFilterNode(node.id, filterObject.id, filterObject.name);
                            }
                        }
                    }
                }
                selectNode(MonitoringAgent.getInitialViewId());
            };

            $scope.$on('MonitoringViewChanged', function (event, message) {
//            console.log("MonitoringTreeController::MonitoringViewChanged::"+ message);
                selectNode(message.viewId);
            });

        }]);
});