'use strict';

define([
    'modules/common/services/device-agent',
    'modules/common/services/user-agent',
    'modules/common/services/monitoringAgent',
    'modules/common/services/door-agent',
    'modules/common/services/datagridFilterService',
    'modules/common/datagrid/columnSettingController',
    'modules/common/datagrid/columnSettingService',
    'modules/common/print/printOptionController',
    'modules/common/print/printOptionService'
], function () {

    var app = angular.module('biostarWebApp');

    app.register.controller('PastEventController', [
        '$rootScope', '$scope', '$window', '$location', '$timeout', '$compile', '$filter', 'Util', 'API', 'Const', 'DeviceAgent', 'UserAgent', 'DoorAgent', 'MonitoringAgent', 'DatagridFilterService', 'MessageProperties', 'ColumnSettingService', 'PrintOptionService', 'PreferenceService',
        function ($rootScope, $scope, $window, $location, $timeout, $compile, $filter, Util, API, Const, DeviceAgent, UserAgent, DoorAgent, MonitoringAgent, DatagridFilterService, MessageProperties, ColumnSettingService, PrintOptionService, PreferenceService) {

            $scope.gridId = Const.GRID_ID.EVENT_LOG;
            $scope.displayUpdateLog = false;

            function initPagination(reuseLimit) {
                var limit = 15;
                if (reuseLimit != undefined && reuseLimit == true) {
                    if ($scope.Parameters != undefined && $scope.Parameters.limit != undefined) {
                        limit = $scope.Parameters.limit;
                    }
                }
                $scope.Parameters = {
                    offset: 0,
                    limit: limit,
                    total: 0,
                    query: ''
                };
                $scope.recordsTotal = 0;
            }

            initPagination();
            $scope.dropDownMenu = [
                {
                    menu: MessageProperties.get('menu.print'),
                    func: function () {
                        $scope.openPrintOptionPopupDlg();
                    }
                },
                {
                    menu: MessageProperties.get('menu.columnSetting'),
                    func: function () {
                        $scope.openColumnSettingPopupDlg();
                    }
                }
            ];

            $scope.pastEventSearchOption = {
                "Query": {
                    "offset": $scope.Parameters.offset,
                    "limit": $scope.Parameters.limit,
                    "orders": [
                        {
                            "column": "datetime",
                            "descending": "true"
                        }
                    ],
                    "conditions": [
                        {
                            "column": "datetime",
                            "operator": Const.QUERY_OPERATOR.BETWEEN,
                            "values": [
                                "2014-08-05T05:24:05.00Z",
                                "2014-10-25T05:24:05.00Z"
                            ]
                        }
                    ]
                }
            };

            $scope.doUpdateLog = function () {
                console.log("doUpdateLog = with " + $scope.checkIdList);
                API.post('events/update', undefined, {DeviceCollection: {}}, undefined, false, true, true).then(function (data) {
                    $scope.searchPastEvent(undefined, undefined, false);
                });
            };


            function buildQueryObject(retrieveTotals) {
                var queryObject = {};
                queryObject.offset = $scope.Parameters.offset;
                queryObject.limit = $scope.Parameters.limit;
                queryObject.atLeastOneFilterExists = false;
                queryObject.conditions = [];
                queryObject.orders = [];
                queryObject.orders.push({
                    column: "datetime",
                    descending: true
                });
                if ($scope.filterObject.dateRange.datetime != undefined && $scope.filterObject.dateRange.datetime.length > 0 &&
                    ($scope.filterObject.dateRange.fromDatetime != '' || $scope.filterObject.dateRange.toDatetime != '')) {
                    queryObject.conditions.push({
                        "column": "datetime",
                        "operator": Const.QUERY_OPERATOR.BETWEEN,
                        "values": [
                            $scope.filterObject.dateRange.fromDatetime,
                            $scope.filterObject.dateRange.toDatetime
                        ]
                    });
                    queryObject.atLeastOneFilterExists = true;
//            } else {
//                queryObject.conditions.push({
//                    "column": "datetime",
//                    "operator": Const.QUERY_OPERATOR.BETWEEN,
//                    "values": [
//                        "2014-08-05T05:24:05.00Z",
//                        "2014-10-25T05:24:05.00Z"
//                    ]
//                });
                }

                if ($scope.filterObject.deviceId != undefined && $scope.filterObject.deviceId.length > 0) {
                    queryObject.conditions.push({
                        "column": "device_id.id",
                        "operator": Const.QUERY_OPERATOR.IN,
                        "values": Util.buildIdArrayFromObjectArray($scope.filterObject.deviceId)
                    });
                    queryObject.atLeastOneFilterExists = true;
                }

                if ($scope.filterObject.deviceName != undefined && $scope.filterObject.deviceName.length > 0) {
                    queryObject.conditions.push({
                        "column": "device_id.id",
                        "operator": Const.QUERY_OPERATOR.IN,
                        "values": Util.buildIdArrayFromObjectArray($scope.filterObject.deviceName)
                    });
                    queryObject.atLeastOneFilterExists = true;
                }

                if ($scope.filterObject.doorName != undefined && $scope.filterObject.doorName.length > 0) {
                    queryObject.conditions.push({
                        "column": "door_id.id",
                        "operator": Const.QUERY_OPERATOR.IN,
                        "values": Util.buildIdArrayFromObjectArray($scope.filterObject.doorName)
                    });
                    queryObject.atLeastOneFilterExists = true;
                }

                if ($scope.filterObject.eventName != undefined && $scope.filterObject.eventName.length > 0) {
                    queryObject.conditions.push({
                        "column": "event_type_id.code",
                        "operator": Const.QUERY_OPERATOR.IN,
                        "values": Util.buildIdArrayFromObjectArray($scope.filterObject.eventName)
                    });
                    queryObject.atLeastOneFilterExists = true;
                }

                if ($scope.filterObject.userGroupName != undefined && $scope.filterObject.userGroupName.length > 0) {
                    queryObject.conditions.push({
                        "column": "user_group_id.id",
                        "operator": Const.QUERY_OPERATOR.IN,
                        "values": Util.buildIdArrayFromObjectArray($scope.filterObject.userGroupName)
                    });
                    queryObject.atLeastOneFilterExists = true;
                }

                if ($scope.filterObject.userName != undefined && $scope.filterObject.userName.length > 0) {
                    queryObject.conditions.push({
                        "column": "user_id.user_id",
                        "operator": Const.QUERY_OPERATOR.IN,
                        "values": Util.buildIdArrayFromObjectArray($scope.filterObject.userName)
                    });
                    queryObject.atLeastOneFilterExists = true;
                }

                if (retrieveTotals == false || queryObject.atLeastOneFilterExists == true) {
                    queryObject.total = false;
                } else {
                    queryObject.total = undefined;
                }
                return queryObject;
            }

            $scope.searchPastEvent = function (paramQueryString, deferFunctionName, retrieveTotals) {
                if (retrieveTotals == undefined) {
                    retrieveTotals = false;
                }
                $scope.filterObject = DatagridFilterService.getFilterObject();
                $scope.gridInformation = MessageProperties.get('common.searching');

                $scope.pastEventSearchOption.Query = buildQueryObject(retrieveTotals);
                var atLeastOneFilterExists = $scope.pastEventSearchOption.Query.atLeastOneFilterExists;
                if (atLeastOneFilterExists == true) {
                    $scope.pastEventSearchOption.Query.limit = parseInt($scope.pastEventSearchOption.Query.limit) + 1;
                } else {
                    $scope.pastEventSearchOption.Query.total = true;
                }
                API.post('events/search', deferFunctionName, $scope.pastEventSearchOption).then(function (data) {
                    if (angular.isDefined(data.EventCollection) && data.EventCollection.rows.length > 0) {
//                    if (retrieveTotals == false) {
//                        data.EventCollection.total = $scope.recordsTotal;
//                    }
                        if (atLeastOneFilterExists == true) {
                            $scope.Parameters.displayOverallTotal = false;
                            $scope.Parameters.retrievedLimit = $scope.pastEventSearchOption.Query.limit;

                            $scope.Parameters.moreDataYn = $scope.pastEventSearchOption.Query.limit == data.EventCollection.rows.length;
                            if ($scope.Parameters.moreDataYn) {
                                data.EventCollection.rows.splice($scope.Parameters.limit, data.EventCollection.rows.length);
                            }

                            $scope.Parameters.retrievedRecordCount = data.EventCollection.rows.length;
                        }
                        $scope.gridInformation = '';
                        $scope.Parameters.total = data.EventCollection.total;
                        $scope.recordsTotal = data.EventCollection.total;
                        angular.element("#" + $scope.gridId).datagrid('loadData', data.EventCollection.rows);
                    } else {
                        if (atLeastOneFilterExists == true) {
                            $scope.Parameters.displayOverallTotal = false;
                            $scope.Parameters.retrievedRecordCount = 0;
                        }
                        $scope.gridInformation = MessageProperties.get('common.notFound');
                        $scope.Parameters.total = 0;
                        $scope.recordsTotal = 0;
                        angular.element("#" + $scope.gridId).datagrid('loadData', []);
                    }
                });
            };

            $scope.SearchCallback = {
                fn: $scope.searchPastEvent,
                reset: function () {
                    $scope.searchPastEvent();
                },
                filterCallback: function () {
                    initPagination(true);
                    $scope.searchPastEvent(undefined, undefined, false);
                }
            };

            $scope.gridInformation = MessageProperties.get('common.notFound');

            $scope.init = function () {

                var biostarSetting = Util.getSessionObject(Const.SESSION_STORAGE_KEY.SERVER_SETTING);
                if (Const.LOG_UPLOAD_TYPE.MANUAL == biostarSetting.log_sync_method) {
                    $scope.displayUpdateLog = true;
                }

                $scope.recordsTotal = 0;
                $rootScope.dateFormat = Const.DEFAULT_DATETIME_FORMAT;
                $scope.filterOptionItems = DatagridFilterService.getFilterOptionItems();
                DatagridFilterService.clearFilterObject();
                $scope.filterObject = DatagridFilterService.getFilterObject();
                DatagridFilterService.setCallbackFunction($scope.SearchCallback.filterCallback);

                angular.element("#" + $scope.gridId).datagrid({
                    view: $.extend({}, $.fn.datagrid.defaults.view, {
                        onBeforeRender: function (target, rows) {
                            angular.element("#" + $scope.gridId).datagrid('resize');

                        },
                        onAfterRender: function (target) {
                            var dataGridBodyElement = angular.element("#" + $scope.gridId).parent(".datagrid-view").find(".datagrid-view2").find(".datagrid-body");
                            if (angular.element("#" + $scope.gridId).datagrid("getRows").length === 0) {
                                var html = '<div class="listNotFound">' +
                                    $scope.gridInformation + '</div>';
                                dataGridBodyElement.html(html);
                            }
                        }
                    }),
                    scrollbarSize: 0,
                    fitColumns: true,
                    autoRowHeight: false,
                    resize: true,
                    singleSelect: true,
                    columns: [
                        [
                            {field: 'event_image', title: '<p class="item icon"><img src="images/contents/photo_icon_w.png" alt=""></p>', width: 50, align: 'center', hidden: true, sortable: false,
                                formatter: function (value, row, index) {
                                    return value ? '<img src="images/contents/photo_icon_b.png" alt="">' : '';
                                }
                            },
                            {field: 'datetime', title: MessageProperties.get("common.date") + ' <div ngbss-datetime-range-filter ' +
                                'data-from-datetime="filterObject.dateRange.fromDatetime" data-to-datetime="filterObject.dateRange.toDatetime" ' +
                                'data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" ></div>',
                                filterValues: 'filterObject.dateRange.datetime',
                                filterType: Const.DATA_FILTER.EVENT_DATE,
                                width: 160, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
//                                return $filter('BssDatetime')(new Date(row.datetime), PreferenceService.getDateFormat() + " " + Const.DEFAULT_TIME_FORMAT);
                                    return $filter('BssDatetimeLongTerm')(new Date(row.datetime));
                                }
                            },
//                        {field: 'door_id.name', title: MessageProperties.get("common.door") + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.DOOR_NAME
//                            + '" data-item-list="filterOptionItems.doorList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
//                            filterValues: 'filterObject.doorName',
//                            filterType: Const.DATA_FILTER.DOOR_NAME,
//                            width: 100, align: 'center', sortable: false,
//                            formatter: function (value, row, index) {
//                                return Util.formatIdAndNameSpan(row.door_id);
//                            }
//                        },
                            {field: 'device_id', title: MessageProperties.get("common.deviceId") + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.DEVICE_ID
                                + '" data-item-list="filterOptionItems.deviceList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="id"></div>',
                                filterValues: 'filterObject.deviceId',
                                filterType: Const.DATA_FILTER.DEVICE_ID,
                                width: 90, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    if (angular.isDefined(row.device_id) && row.device_id.id) {
                                        return row.device_id.id;
                                    } else {
                                        return '';
                                    }
                                }
                            },
                            {field: 'device_id.name', title: MessageProperties.get("common.device") + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.DEVICE_NAME
                                + '" data-item-list="filterOptionItems.deviceList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
                                filterValues: 'filterObject.deviceName',
                                filterType: Const.DATA_FILTER.DEVICE_NAME,
                                width: 100, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    return Util.formatIdAndNameSpan(row.device_id);
                                }
                            },
//                        {field: 'user_group_id', title: MessageProperties.get("common.userGroup") + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.USER_GROUP_NAME
//                            + '" data-item-list="filterOptionItems.userGroupList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
//                            filterValues: 'filterObject.userGroupName',
//                            filterType: Const.DATA_FILTER.USER_GROUP_NAME,
//                            width: 100, align: 'center', sortable: false,
//                            formatter: function (value, row, index) {
//                                return Util.formatIdAndNameSpan(row.user_group_id);
//                            }
//                        },
                            {field: 'user_id', title: MessageProperties.get("common.user") + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.USER_NAME
                                + '" data-item-list="filterOptionItems.userList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
                                filterValues: 'filterObject.userName',
                                filterType: Const.DATA_FILTER.USER_NAME,
                                width: 100, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
//                                console.log(value);
//                                if (angular.isDefined(value.user_id)) {
//                                    value.user_id.id = value.user_id.user_id;
//                                }
//                                return Util.formatIdAndNameSpan(value);
                                    var idString = '';
                                    if (angular.isDefined(value)) {
                                        if (value.name) {
                                            idString = value.user_id + '(' + value.name + ')';
                                        } else {
                                            idString = value.user_id + '(-)';
                                        }
                                    } else {
                                        idString = '';
                                    }

                                    return idString;
                                }
                            },
//                        {field: 'tna_key', title: 'T&A Event', width: 60, align: 'center', hidden: true, sortable: false,
//                            formatter: function (value, row, index) {
//                                if (value == null) {
//                                    return '';
//                                } else if (angular.isDefined(value.name)) {
//                                    return value.name;
//                                } else {
//                                    return value;
//                                }
//                            }
//                        },
                            {field: 'event_type_id.name', title: MessageProperties.get("common.event") + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.EVENT_NAME + '" ' +
                                'data-item-list="filterOptionItems.eventList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
                                filterValues: 'filterObject.eventName',
                                filterType: Const.DATA_FILTER.EVENT_NAME,
                                width: 160, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    return Util.formatIdAndNameSpan(row.event_type_id);
                                }
                            }
                        ]
                    ],
                    data: [],
                    onLoadSuccess: function (data) {
                        angular.element('#' + $scope.gridId).datagrid('setFilterRow', {
                            callback: function () {
                                $compile(angular.element('#' + $scope.gridId).datagrid('getPanel').find('.datagrid-view2 .datagrid-htable'))($scope);
                            }
                        });
                    }
                });

//            $scope.searchPastEvent(undefined, undefined, true);

                $scope.$watch(function () {
                    return  angular.element('.panel.datagrid').width();
                }, function (nv, ov) {
                    if (nv !== ov) {
                        angular.element("#" + $scope.gridId).datagrid('resize');
                    }
                }, true);

                angular.element($window).on('resize', function ($event) {
                    var event = $event | window.event;
                    angular.element("#" + $scope.gridId).datagrid('resize');
                });

                $scope.$watch('Parameters', function (nv, ov) {
                    if (nv !== ov) {
                        var queryObject = buildQueryObject();
                        queryObject.collection_name = Const.REPORT_COLLECTION_NAME.EVENT;
                        queryObject.object = "query";
                        PrintOptionService.setQueryObject(queryObject);
                    }
                }, true);

                PrintOptionService.getReportColumnTypeFunction = function (gridColumnName) {
                    if (gridColumnName == "datetime") {
                        return "datetime"
                    } else {
                        return "string";
                    }
                };
                PrintOptionService.getReportColumnNameFunction = function (gridColumnName) {
                    var reportColumnName = gridColumnName;
                    if (gridColumnName == "user_group_id") {
                        return "user_group_id.name"
                    } else if (gridColumnName == "device_id") {
                        return "device_id.id"
                    } else if (gridColumnName == "door_id.name") {
                        return "door_id.name"
                    } else if (gridColumnName == "device_id.name") {
                        return "device_id.name"
                    } else if (gridColumnName == "user_id.name") {
                        return "user_id.name"
                    } else if (gridColumnName == "event_type_id.name") {
                        return "event_type_id.name"
                    }
                    return reportColumnName;
                };
                $scope.reportUrl = Const.API.BASE_URL + 'events/report';
                $scope.openPrintOptionPopupDlg = PrintOptionService.openPrintOptionPopupDlg;
                $scope.openColumnSettingPopupDlg = ColumnSettingService.openColumnSettingPopupDlg;

                ColumnSettingService.setGridId($scope.gridId);
                ColumnSettingService.setIncludeAllColumnsForPrinting(true);
                ColumnSettingService.setCallbackFunction($scope.SearchCallback.reset);
                var customColumnSetting = ColumnSettingService.getColumnSettingFromLocalStroage();
                if (customColumnSetting) {
                    $timeout(function () {
                        ColumnSettingService.applyChangedColumnsToGrid(customColumnSetting, false);
                    }, 100);
                }

                $scope.$on(Const.UI_EVENT_TYPE.MONITORING_FILTER_CLICKED, function (event, message) {
                    if (message.viewId == Const.MONITORING_VIEW.PAST_EVENT) {
//                    console.log("pastEvent MonitoringFilterClicked");
                        if (message.filterId != undefined) {
                            var filterObject = DatagridFilterService.findFilterObjectByIdFromLocalStorage(message.filterId);
                            DatagridFilterService.setFilterObject(filterObject);
                        } else {
                            DatagridFilterService.clearFilterObject();
                        }
                        initPagination(true);
                        $scope.searchPastEvent(undefined, undefined, true);
                        ;
                    }
                });
//            $scope.$on("$destroy", function() {
//                $scope.$on(Const.UI_EVENT_TYPE.MONITORING_FILTER_CLICKED, function () {
//                    $scope.$$listeners[Const.UI_EVENT_TYPE.MONITORING_FILTER_CLICKED] = [];
//                })
//            });
            };

        }]);
});