'use strict';

define([
    'modules/common/services/device-agent',
    'modules/common/services/user-agent',
    'modules/common/services/monitoringAgent',
    'modules/common/services/door-agent',
    'modules/common/services/datagridFilterService',
    'modules/common/datagrid/columnSettingController',
    'modules/common/datagrid/columnSettingService',
    'modules/common/print/printOptionController',
    'modules/common/print/printOptionService'
], function () {

    var app = angular.module('biostarWebApp');
    app.register.controller('RealtimeEventController', [
        '$rootScope', '$scope', '$window', '$location', '$compile', '$timeout', '$filter', 'Util', 'API', 'WebSocketService', 'Const', 'MessageProperties', 'ColumnSettingService', 'PrintOptionService', 'DatagridFilterService', 'MonitoringAgent', 'PreferenceService',
        function ($rootScope, $scope, $window, $location, $compile, $timeout, $filter, Util, API, WebSocketService, Const, MessageProperties, ColumnSettingService, PrintOptionService, DatagridFilterService, MonitoringAgent, PreferenceService) {

            $scope.gridInformation = '';
            $scope.gridId = "grdRealtimeEvent";

            $scope.dropDownMenu = [
//            {
//                menu: MessageProperties.get('menu.print'),
//                func: function () {
//                    $scope.openPrintOptionPopupDlg();
//                }
//            },
                {
                    menu: MessageProperties.get('menu.columnSetting'),
                    func: function () {
                        $scope.openColumnSettingPopupDlg();
                    }
                }
            ];

            function startRealtimeEventCheckInterval() {
                if ($rootScope.realtimeEventCheckIntervalId == false) {
                    $rootScope.realtimeEventCheckIntervalId = setInterval(function () {
                        console.log('event count=' + $scope.receivedEventCount);
                        if ($scope.receivedEventCount > 0) {
                            $scope.receivedEventCount = 0;
                            MonitoringAgent.deleteOldData();
                            DatagridFilterService.setOriginalDataList(MonitoringAgent.getRealtimeEventList());
                            var filteredList = DatagridFilterService.filterData($scope.gridId);
                            DatagridFilterService.setOriginalDataList(filteredList);
                        }
                    }, 1000);

                    Util.debug("startRealtimeEventCheckInterval interval=" + $rootScope.realtimeEventCheckIntervalId);
                } else {
                    Util.debug("startRealtimeEventCheckInterval ignored already running interval=" + $rootScope.realtimeEventCheckIntervalId);
                }
            }

            $scope.init = function () {

                $rootScope.dateFormat = Const.DEFAULT_DATETIME_FORMAT;
                $scope.filterOptionItems = DatagridFilterService.getFilterOptionItems();
                DatagridFilterService.clearFilterObject();
                $scope.filterObject = DatagridFilterService.getFilterObject();

                $scope.callbackAfterApplyFilter = function () {
                    DatagridFilterService.setOriginalDataList(MonitoringAgent.getRealtimeEventList());
                    $scope.filterObject = DatagridFilterService.getFilterObject();
                    DatagridFilterService.filterData($scope.gridId);
                    startRealtimeEventCheckInterval();
                };

                DatagridFilterService.setCallbackFunction($scope.callbackAfterApplyFilter);

                $scope.gridInformation = MessageProperties.get('common.notFound');


                angular.element("#" + $scope.gridId).datagrid({
                    view: $.extend({}, $.fn.datagrid.defaults.view, {
                        onBeforeRender: function (target, rows) {
                            angular.element("#" + $scope.gridId).datagrid('resize');

                        },
                        onAfterRender: function (target) {
                            var dataGridBodyElement = angular.element("#" + $scope.gridId).parent(".datagrid-view").find(".datagrid-view2").find(".datagrid-body");
                            if (angular.element("#" + $scope.gridId).datagrid("getRows").length === 0) {
                                var html = '<div class="listNotFound">' +
                                    $scope.gridInformation + '</div>';
                                dataGridBodyElement.html(html);
                            }
                        }
                    }),
                    scrollbarSize: 0,
//                height: '650',
                    fitColumns: true,
                    autoRowHeight: false,
                    resize: true,
                    singleSelect: true,
                    columns: [
                        [
                            {field: 'event_image', title: '<p class="item icon"><img src="images/contents/photo_icon_w.png" alt=""></p>', width: 50, align: 'center', hidden: true, sortable: false,
                                formatter: function (value, row, index) {
                                    return value ? '<img src="images/contents/photo_icon_b.png" alt="">' : '';
                                }
                            },
                            {field: 'datetime', title: MessageProperties.get("common.date") //+ ' <div ngbss-datetime-range-filter ' +
                                //' data-from-datetime="filterObject.dateRange.fromDatetime" data-to-datetime="filterObject.dateRange.toDatetime" ' +
//                            ' data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)"></div>'
                                ,
                                filterValues: 'filterObject.dateRange.datetime',
                                filterType: Const.DATA_FILTER.EVENT_DATE,
                                width: 160, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
//                                return $filter('BssDatetime')(new Date(row.datetime), PreferenceService.getDateFormat() + " " + Const.DEFAULT_TIME_FORMAT);
                                    return $filter('BssDatetimeLongTerm')(new Date(row.datetime));
                                }
                            },
                            {field: 'door_id.name', title: MessageProperties.get("common.door") + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.DOOR_NAME
                                + '" data-item-list="filterOptionItems.doorList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
                                filterValues: 'filterObject.doorName',
                                filterType: Const.DATA_FILTER.DOOR_NAME,
                                width: 100, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    return Util.formatIdAndNameSpan(row.door_id);
                                }
                            },
                            {field: 'device_id', title: MessageProperties.get("common.deviceId") + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.DEVICE_ID
                                + '" data-item-list="filterOptionItems.deviceList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="id"></div>',
                                filterValues: 'filterObject.deviceId',
                                filterType: Const.DATA_FILTER.DEVICE_ID,
                                width: 90, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    if (angular.isDefined(row.device_id) && row.device_id.id) {
                                        return row.device_id.id;
                                    } else {
                                        return '';
                                    }
                                }
                            },
                            {field: 'device_id.name', title: MessageProperties.get("common.device") + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.DEVICE_NAME
                                + '" data-item-list="filterOptionItems.deviceList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
                                filterValues: 'filterObject.deviceName',
                                filterType: Const.DATA_FILTER.DEVICE_NAME,
                                width: 100, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    return Util.formatIdAndNameSpan(row.device_id);
                                }
                            },
//                        {field: 'user_group_id', title: MessageProperties.get("common.userGroup") + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.USER_GROUP_NAME
//                            + '" data-item-list="filterOptionItems.userGroupList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
//                            filterValues: 'filterObject.userGroupName',
//                            filterType: Const.DATA_FILTER.USER_GROUP_NAME,
//                            width: 100, align: 'center', sortable: false,
//                            formatter: function (value, row, index) {
//                                return Util.formatIdAndNameSpan(row.user_group_id);
//                            }
//                        },
                            {field: 'user_id.name', title: MessageProperties.get("common.user") + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.USER_NAME
                                + '" data-item-list="filterOptionItems.userList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
                                filterValues: 'filterObject.userName',
                                filterType: Const.DATA_FILTER.USER_NAME,
                                width: 100, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    if (angular.isDefined(row.user_id)) {
                                        row.user_id.id = row.user_id.user_id;
                                    }
                                    return Util.formatIdAndNameSpan(row.user_id);
                                }
                            },
                            {field: 'event_type_id.name', title: MessageProperties.get("common.event") + ' <div ngbss-equal-filter  data-filter-type="' + Const.DATA_FILTER.EVENT_NAME + '" ' +
                                'data-item-list="filterOptionItems.eventList" data-filter-object="filterObject" data-apply-filter="applyFilter(filterSelectedInput)" data-label="name"></div>',
                                filterValues: 'filterObject.eventName',
                                filterType: Const.DATA_FILTER.EVENT_NAME,
                                width: 160, align: 'left', sortable: false,
                                formatter: function (value, row, index) {
                                    return Util.formatIdAndNameSpan(row.event_type_id);
                                }
                            }
                        ]
                    ],
                    data: [],
                    onLoadSuccess: function (data) {
                        angular.element('#' + $scope.gridId).datagrid('setFilterRow', {
                            callback: function () {
                                $compile(angular.element('#' + $scope.gridId).datagrid('getPanel').find('.datagrid-view2 .datagrid-htable'))($scope);
                            }
                        });
                    },
                    rowStyler: function (index, row) {
                        if (row.event_type_id != undefined && row.event_type_id.enable_alert == "true") {
                            return 'background-color:#ffb5b0;';
                        }
                    }
                });
                $scope.$watch(function () {
                    return  angular.element('.panel.datagrid').width();
                }, function (nv, ov) {
                    if (nv !== ov) {
                        angular.element('#' + $scope.gridId).datagrid('resize');
                    }
                }, true);

                angular.element($window).on('resize', function ($event) {
                    var event = $event | window.event;
                    angular.element('#' + $scope.gridId).datagrid('resize');
                });

                $scope.receivedEventCount = 0;

                $scope.$on('EventReceived', function (event, messageObject) {
                        if ($scope.receiveEvents == false) {
                            return;
                        }
                        if (angular.element('#' + $scope.gridId).length <= 0) {
                            return;
                        }
                        if (messageObject.Alert != undefined) {
                            MonitoringAgent.addToTopOfRealtimeEventList(messageObject.Alert.event_id);
                        } else {
                            MonitoringAgent.addToTopOfRealtimeEventList(messageObject.Event);
                        }
                        $scope.receivedEventCount++;
                    }
                );

                startRealtimeEventCheckInterval();

                $scope.receiveEvents = true;
                $scope.playPauseBtnText = MessageProperties.get("button.pause");
                $scope.playPauseBtnImg = Const.REAL_TIME_BUTTON_IMG_URL.PAUSE;
                $scope.playPause = function () {
                    $('#btnPlayPause').addClass('disabled').attr('disabled', 'disabled');
                    if ($scope.receiveEvents == false) {
                        $scope.receiveEvents = true;
                        $scope.playPauseBtnText = MessageProperties.get("button.pause");
                        $scope.playPauseBtnImg = Const.REAL_TIME_BUTTON_IMG_URL.PAUSE;
                        angular.element('#btnPlayPause').removeClass('disabled').removeClass('btnFuncPlay').removeAttr('disabled').addClass('btnFuncDark');
                    } else {
                        $scope.receiveEvents = false;
                        $scope.playPauseBtnText = MessageProperties.get("button.play");
                        $scope.playPauseBtnImg = Const.REAL_TIME_BUTTON_IMG_URL.PLAY;
                        angular.element('#btnPlayPause').removeClass('disabled').removeClass('btnFuncDark').removeAttr('disabled').addClass('btnFuncPlay');
                    }
                };

                $scope.clear = function () {
                    $scope.gridInformation = MessageProperties.get('common.notFound');
                    MonitoringAgent.setRealtimeEventList([]);
                    DatagridFilterService.setOriginalDataList(MonitoringAgent.getRealtimeEventList());
                    angular.element('#' + $scope.gridId).datagrid({data: DatagridFilterService.getOriginalDataList()});
                };

                // TODO BSS-195 BSS-192 print :: server api not implemented
                PrintOptionService.getReportColumnNameFunction = function (gridColumnName) {
                    return gridColumnName;
                };
                $scope.reportUrl = Const.API.BASE_URL + 'monitoring/realtimeevent/report';
                $scope.openPrintOptionPopupDlg = PrintOptionService.openPrintOptionPopupDlg;
                $scope.openColumnSettingPopupDlg = ColumnSettingService.openColumnSettingPopupDlg;

                ColumnSettingService.setGridId($scope.gridId);
                ColumnSettingService.setCallbackFunction($scope.callbackAfterApplyFilter);
                var customColumnSetting = ColumnSettingService.getColumnSettingFromLocalStroage();
                if (customColumnSetting) {
                    $timeout(function () {
                        ColumnSettingService.applyChangedColumnsToGrid(customColumnSetting, false);
                    }, 100);
                }

                $scope.$on(Const.UI_EVENT_TYPE.MONITORING_FILTER_CLICKED, function (event, message) {
                    if (message.viewId == Const.MONITORING_VIEW.REAL_TIME_EVENT) {
//                    console.log("real time event MonitoringFilterClicked");
                        if (message.filterId != undefined) {
                            var filterObject = DatagridFilterService.findFilterObjectByIdFromLocalStorage(message.filterId);
                            DatagridFilterService.setFilterObject(filterObject);
                        } else {
                            DatagridFilterService.clearFilterObject();
                        }
                        $scope.callbackAfterApplyFilter();
//                    $scope.$apply();
                    }
                });
            };
        }]);
});