'use strict';

define([
    'modules/common/datagrid/columnSettingController',
    'modules/common/datagrid/columnSettingService',
    'modules/common/print/printOptionController',
    'modules/common/print/printOptionService',
    'modules/common/services/card-agent',
    'modules/setting/cardBatchEnrollController',
    'modules/setting/searchCardController'
],function() {
    var app = angular.module('biostarWebApp');
    app.register.controller('CardController', ['$scope', '$rootScope', '$q', '$location', 'API', 'CardsAgent', 'MessageProperties', 'ColumnSettingService',  'PrintOptionService', 'Const', '$window', function ($scope, $rootScope, $q, $location, API, CardsAgent, MessageProperties, ColumnSettingService,  PrintOptionService, Const, $window) {
        var
            cardInfo,
            httpTimeOut = undefined,
            canceller = $q.defer(),
            cardStatusName = 0,
            ngIncludeReloadVer,
            cardsQueryStrings = {
                query: undefined,
                offset: 0,
                limit: 15,
                total: undefined,
                group_id: undefined
            },
            cardListDataGridOptions = {
                scrollbarSize: 0,
                view: $.extend({}, $.fn.datagrid.defaults.view, {
                    onBeforeRender: function (target, rows) {
                        $("#cardGrid").datagrid('resize');
                    }, onAfterRender: function (target) {
                        if ($(target).datagrid('getRows').length === 0) {
                            var html =
                                '<div style="width:100%; height:360px; font-size:1.333em; line-height: 360px; ' +
                                'vertical-align: middle; text-align: center;">' +
                                MessageProperties.get('validate.notFound') +
                                '</div>';
                            $("div.datagrid-view2 div.datagrid-body").css('height', '360px').html(html);
                        }
                    }
                }),
                fitColumns: true,
                resize: true,
                singleSelect: true,
                columns: [
                    [
                        {field: 'card_type', title: MessageProperties.get('setting.card.cardType'), width: 80, align: 'left', formatter: function (value, row, index) {

                            return value.name || '-';
                        }},
                        {field: 'card_id', title: MessageProperties.get('setting.card.cardId'), width: 80, align: 'left'},
                        {field: 'status', title: MessageProperties.get('common.status'), width: 80, align: 'left', formatter: function (value, row, index) {
                            return $scope.CARD_STATUS_NAME[row.status];
                        }},
                        {field: 'user_id', title: MessageProperties.get('setting.card.userId'), width: 80, align: 'left', formatter: function (value, row, index) {
                            if (value) {
                                return value.user_id;
                            }
                            return '-';
                        }},
                        {field: 'user_nm', title: MessageProperties.get('setting.card.userName'), width: 80, align: 'left', formatter: function (value, row, index) {
                            if (row.user_id && row.user_id.name) {
                                return row.user_id.name;
                            }
                            return '-';
                        }}
                    ]
                ],
                onClickRow: function (rowIndex, rowData) {
//                    cardAgent.getOnecard(rowData.card_id).then(function success(data) {
//                        console.log(data);
//                        $location.path('/card/card-info/' + rowData.card_id);
//                    });
                }
            };

        function safeApply(fn) {
            $scope.$root.$$phase ? fn() : $scope.$apply(fn);
        }

        $scope.$watch(function () {
            return  angular.element('.panel.datagrid').width();
        }, function (nv, ov) {
            if (nv !== ov) {
                angular.element('#cardGrid').datagrid('resize');
            }
        }, true);
        angular.element($window).on('resize', function ($event) {
            var event = $event | window.event;
            angular.element('#cardGrid').datagrid('resize');
        });

        angular.element('#cardGrid').datagrid(cardListDataGridOptions);

        $scope.doLnbClick = function (group_id) {
            $scope.cardsQueryStrings.offset = 0;
            $scope.currentListCardStatus = group_id;
            doSearch(group_id);
        };

        function doSearch(group_id, deferFunctionName) {
            $scope.detailPageUrl = undefined;
            if (group_id === -1) {
                $scope.cardsQueryStrings.query = undefined;
                $scope.cardsQueryStrings.offset = 0;
                $scope.cardsQueryStrings.limit = 15;
                $scope.cardsQueryStrings.group_id = undefined;
            }
            if(group_id >= 0) {
                $scope.cardsQueryStrings.group_id = group_id;
            }
//            if(group_id !== $scope.CARDS_BLACKLIST) {
//                CardsAgent.getCards(cardsQueryStrings).then(function success(data) {
//                    $scope.CardListData = data.CardCollection.rows || [];
//                    cardsQueryStrings.total = data.CardCollection.total || 0;
//                    angular.element('#cardGrid').datagrid('loadData', $scope.CardListData);
//                });
//            } else if(group_id === $scope.CARDS_BLACKLIST) {
//                cardsQueryStrings.group_id = undefined;
//                CardsAgent.getBlacklist(cardsQueryStrings).then(function success(data) {
//                    $scope.CardListData = data.BlacklistCollection.rows || [];
//                    cardsQueryStrings.total = data.BlacklistCollection.total || 0;
//                    angular.element('#cardGrid').datagrid('loadData', $scope.CardListData);
//                });
//            }
            CardsAgent.getCards($scope.cardsQueryStrings, deferFunctionName).then(function success(data) {
                $scope.CardListData = data.CardCollection.rows || [];
                $scope.cardsQueryStrings.total = data.CardCollection.total || 0;
                angular.element('#cardGrid').datagrid('loadData', $scope.CardListData);
            });
        }

        $scope.ALL_CARDS = -1;
        $scope.UNASSIGNED_CARDS = 0;
        $scope.ASSIGNED_CARDS = 1;
        $scope.CARDS_BLACKLIST = 2;
        $scope.cardListTitle = [
            MessageProperties.get('common.card'),
            MessageProperties.get('setting.card.unassignedCard'),
            MessageProperties.get('setting.card.assignedCard'),
            MessageProperties.get('setting.card.blacklistCard')
        ]
        $scope.currentListCardStatus = $scope.ALL_CARDS;
        $scope.CARD_STATUS_NAME = [MessageProperties.get('setting.card.unassigned'), MessageProperties.get('setting.card.assigned'), MessageProperties.get('setting.card.blacklist')];
        $scope.DETAIL_VIEW_URL = {
            batchEnroll: 'modules/setting/cardBatchEnroll.html',
            searchCard: 'modules/setting/searchCard.html'
        }

        $scope.cardsQueryStrings = cardsQueryStrings;
        doSearch($scope.currentListCardStatus);

        $scope.SearchCallback = {
            fn: doSearch
        };

        $scope.detailPageUrl = undefined;
        $scope.doDetailViewChange = function (url) {
            window.sessionStorage.setItem('cardInfo', '{}');
            ngIncludeReloadVer = new Date();
            $scope.detailPageUrl = url + '?v=' + ngIncludeReloadVer.getTime();
            console.log($scope.detailPageUrl);
        };

        $scope.$on('CloseDetailView', function (event, data) {
            doSearch();
            $scope.detailPageUrl = undefined;
        });

        cardInfo = angular.fromJson(window.sessionStorage.getItem('cardInfo'));

        if(cardInfo) {
            $scope.detailPageUrl = $scope.DETAIL_VIEW_URL.batchEnroll;
        }

        $scope.backSetting = function () {
            $location.path('/setting');
        };






        $scope.dropDownMenu = [
//            {
//                menu: 'Import to CSV',
//                func: function () {
//                    console.info(this.menu);
//                }
//            },
//            {
//                menu: 'Export to CSV',
//                func: function () {
//                    console.info(this.menu);
//                }
//            },
//            {
//                menu: 'Print Preview',
//                func: function () {
//                    Util.beforeUrl($location.url());
//                    $location.path('/alarm/alarms-report');
//                }
//            },
//            {
//                menu: MessageProperties.get('menu.print'),
//                func: function () {
//                    $scope.openPrintOptionPopupDlg();
//                }
//            },
            {
                menu: MessageProperties.get('menu.columnSetting'),
                func: function () {
                    $scope.openColumnSettingPopupDlg();
                }
            }
        ];



        $scope.$watch('alarmListQueryStrings', function (nv, ov) {
            if (nv !== ov) {
                // TODO BSS-195 BSS-192 print :: server api not implemented
//                PrintOptionService.buildGridQueryFunction(Const.REPORT_COLLECTION_NAME.CARD, nv);
            }
        }, true);

        PrintOptionService.getReportColumnNameFunction = function (gridColumnName) {
            var reportColumnName = gridColumnName;
            if (gridColumnName == "card_type"){
                return "card_type.name"
            } else  if (gridColumnName == "card_id"){
                return "card_id"
            } else  if (gridColumnName == "status"){
                return "status"
            } else  if (gridColumnName == "last_event"){
                return ""
            }
            return reportColumnName;
        };
        $scope.reportUrl = Const.API.BASE_URL + 'cards/report';
        $scope.openPrintOptionPopupDlg = PrintOptionService.openPrintOptionPopupDlg;
        $scope.openColumnSettingPopupDlg = ColumnSettingService.openColumnSettingPopupDlg;
        $scope.gridId = "cardGrid";
        ColumnSettingService.setGridId($scope.gridId);
        ColumnSettingService.setCallbackFunction($scope.SearchCallback.fn);
        var customColumnSetting = ColumnSettingService.getColumnSettingFromLocalStroage();
        if (customColumnSetting) {
            ColumnSettingService.applyChangedColumnsToGrid(customColumnSetting, false);
        }


    }]);
});