'use strict';

define([
    'modules/common/datagrid/columnSettingController',
    'modules/common/datagrid/columnSettingService',
    'modules/common/print/printOptionController',
    'modules/setting/wiegandInfoController',
    'modules/common/print/printOptionService'
], function () {
    var app = angular.module('biostarWebApp');
    app.register.controller('CardFormatController', ['$scope', '$rootScope', '$window', '$location', 'API', 'MessageProperties', 'MessageService', 'ColumnSettingService', 'PrintOptionService', 'Const', 'Util', function ($scope, $rootScope, $window, $location, API, MessageProperties, MessageService, ColumnSettingService, PrintOptionService, Const, Util) {

        $scope.gridId = "cardFormatGrid";
        $scope.gridInformation = '';
        $scope.cardFormatList = [];
        $scope.checkItemArray = [];

        $scope.cardFormatQueryStrings = {
            query: undefined,
            offset: 0,
            limit: 10,
            total: undefined,
            group_id: undefined
        };

        $scope.dropDownMenu = [
//            {
//                menu: MessageProperties.get('menu.print'),
//                func: function () {
//                    $scope.openPrintOptionPopupDlg();
//                }
//            },
            {
                menu: MessageProperties.get('menu.columnSetting'),
                func: function () {
                    $scope.openColumnSettingPopupDlg();
                }
            }
        ];

        var cardFormatInfo;
        var cardFormatListDataGridOptions = {
            view: $.extend({}, $.fn.datagrid.defaults.view, {
                onBeforeRender: function (target, rows) {
                    angular.element("#" + $scope.gridId).datagrid('resize');

                },
                onAfterRender: function (target) {
                    var dataGridBodyElement = angular.element("#" + $scope.gridId).parent(".datagrid-view").find(".datagrid-view2").find(".datagrid-body");
                    if (angular.element("#" + $scope.gridId).datagrid("getRows").length === 0) {
                        var html = '<div class="listNotFound">' +
                            $scope.gridInformation + '</div>';
                        dataGridBodyElement.html(html);
                    }
                }
            }),
            scrollbarSize: 0,
            fitColumns: true,
            autoRowHeight: false,
            resize: true,
            singleSelect: true,
            selectOnCheck: false,
            checkOnSelect: false,
            columns: [
                [
                    {field: Const.FIELD_NAME.CHECKBOX, checkbox: true},
                    {field: 'name', title: MessageProperties.get("common.name"), width: 80},
                    {field: 'length', title: MessageProperties.get("setting.cardFormat.totalBits"), width: 80},
                    {field: 'id_fields', title: MessageProperties.get("setting.cardFormat.numberOfIds"), width: 80,
                        formatter: function (value, row, index) {
                            var returnValue = '';
                            if (row.id_fields != undefined) {
                                returnValue = row.id_fields.length;
                            }
                            return returnValue;
                        }
                    },
                    {field: 'parity_fields', title: MessageProperties.get("setting.cardFormat.numberOfParityBits"), width: 80,
                        formatter: function (value, row, index) {
                            var returnValue = '';
                            if (row.parity_fields != undefined) {
                                returnValue = row.parity_fields.length;
                            }
                            return returnValue;
                        }
                    }
                ]
            ],
            data: [],
            onClickCell: function (rowIndex, field) {
                $rootScope.clickCellOnGrid($scope.gridId, rowIndex, field, "id", function (rowData) {
                    clickCardForamt(rowData.id);
                });
            },
            onCheck: function (rowIndex, rowData) {
                checkBoxClicked(rowData);
            },
            onUncheck: function (rowIndex, rowData) {
                if ($scope.SelectAllYN == true) {
                    angular.element('#' + $scope.gridId).datagrid('uncheckAll');
                } else {
                    checkBoxClicked(rowData);
                }
            },
            onCheckAll: function (rows) {
                checkBoxClicked(rows, true);
            },
            onUncheckAll: function (rows) {
                if ($scope.SelectAllYN == true) {
                    $scope.SelectAllYN = false;
                    $scope.checkItemArray = [];
                }
                checkBoxClicked(rows, false);
            }
        };
        $scope.SelectAllYN = false;

        $scope.$watch('SelectAllYN', function (nv) {
            if (nv) {
                angular.element('#cardFormatGrid').datagrid('checkAll');
            } else {
                angular.element('#cardFormatGrid').datagrid('uncheckAll');
                $scope.checkItemArray = [];
            }
        });


        function checkBoxClicked(rowData, checkAllYN) {
            var i, idx, item;
            if (rowData.constructor == Object) {
                idx = $scope.checkItemArray.indexOf(rowData.id);
                if (idx > -1) {
                    $scope.checkItemArray.splice(idx, 1);
                } else {
                        $scope.checkItemArray.push(rowData.id);
                }
            } else {
                if (checkAllYN) {
                    for (i = rowData.length; i--;) {
                        item = rowData[i];
                        idx = $scope.checkItemArray.indexOf(item.id);
                        if (idx == -1) {
                            $scope.checkItemArray.push(item.id);
                        }
                    }
                } else {
                    for (i = rowData.length; i--;) {
                        item = rowData[i];
                        idx = $scope.checkItemArray.indexOf(item.id);
                        if (idx > -1) {
                            $scope.checkItemArray.splice(idx, 1);
                        }
                    }
                }
            }
            if (!$scope.$$phase) {
                $scope.$apply();
            }
//            console.log($scope.checkItemArray);
        }

        function clickCardForamt(id) {
            API.get("setting/wiegand_formats/" + id).then(function (data) {
//                    console.log(data);
                Util.setSessionObject(Const.SESSION_STORAGE_KEY.CARD_FORMAT_INFO, data.WiegandFormat);
                $scope.detailPageUrl = $scope.DETAIL_VIEW_URL.wiegandInfo;
            });
        }

        angular.element('#' + $scope.gridId).datagrid(cardFormatListDataGridOptions);

        function clearInfo() {
            angular.element('#cardFormatGrid').datagrid('uncheckAll');
            $scope.detailPageUrl = undefined;
            $scope.checkItemArray = [];
            Util.removeSessionObject(Const.SESSION_STORAGE_KEY.CARD_FORMAT_INFO);
        }

        function doSearch() {
            $scope.detailPageUrl = undefined;

            API.get('setting/wiegand_formats', undefined, $scope.cardFormatQueryStrings).then(function success(data) {
                $scope.cardFormatList = data.WiegandFormatCollection.rows || [];
//                console.log($scope.cardFormatList);
                $scope.cardFormatQueryStrings.total = data.WiegandFormatCollection.total || 0;
                if (data.WiegandFormatCollection.total > 0) {
                    $scope.gridInformation = '';

//                    angular.element("#wiegandRoot").html('');
//                    angular.element("#wiegandRoot").show();
//                    for (var i = 0; i < data.WiegandFormatCollection.rows.length; i++) {
//                        angular.element("#wiegandRoot").append('<li><a href="javascript:void(null);" ng-click="clickCardForamt('+ data.WiegandFormatCollection.rows[i].id +')">'+data.WiegandFormatCollection.rows[i].name+'</a></li>');
//                    }

                } else {
                    $scope.gridInformation = MessageProperties.get('validate.notFound');
//                    angular.element("#wiegandRoot").html('');
//                    angular.element("#wiegandRoot").hide();
                }
                angular.element('#' + $scope.gridId).datagrid('loadData', $scope.cardFormatList);
            });
        }

        $scope.DETAIL_VIEW_URL = {
            wiegandInfo: 'modules/setting/wiegandInfo.html'
        }

        doSearch();

        $scope.listSelectStatus = Const.CARD_FORMAT_SELECT_STATUS.ALL;
        $scope.SearchCallback = {
            fn: doSearch,
            wiegandFormat: function () {
                $scope.listSelectStatus = Const.CARD_FORMAT_SELECT_STATUS.WIEGAND;
                clearInfo();
                doSearch();
            },
            all: function () {
                $scope.listSelectStatus = Const.CARD_FORMAT_SELECT_STATUS.ALL;
                clearInfo();
                doSearch();
            }
        };

        $scope.detailPageUrl = undefined;

        $scope.doAddNewWiegand = function () {
            clearInfo();
            Util.setSessionObject(Const.SESSION_STORAGE_KEY.CARD_FORMAT_INFO, {});
            $scope.detailPageUrl = $scope.DETAIL_VIEW_URL.wiegandInfo;
            $scope.$broadcast("NewDetailViewClicked");
        };

        $scope.doDeleteSelectedCardFormat = function () {
//            console.log($scope.checkItemArray);
            MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                if (confirmYn) {
                    var paramObject = {
                        WiegandFormatCollection : {
                            rows : [],
                            total : 0
                        }
                    },
                        i = $scope.checkItemArray.length,
                        checkedId,
                        chkPredefined = false;
                    for (; i--;) {
                        checkedId = $scope.checkItemArray[i];
                        if (checkedId == Const.CARD_FORMAT_ID.H10301 ||
                            checkedId == Const.CARD_FORMAT_ID.H10302 ||
                            checkedId == Const.CARD_FORMAT_ID.H10304 ||
                            checkedId == Const.CARD_FORMAT_ID.HCORP1000) {
                            $scope.checkItemArray.splice($scope.checkItemArray.indexOf(checkedId), 1);
//                            MessageService.displayWarningMessage(MessageProperties.get("setting.cardFormat.predefinedFormatCannotBeDeleted"));
                            if (!chkPredefined) {
                                chkPredefined = true;
                            }
                        } else {
                            paramObject.WiegandFormatCollection.rows.push({
                                id: checkedId
                            });
                        }
                    }
                    if (paramObject.WiegandFormatCollection.rows.length > 0) {
                        API.post('setting/wiegand_formats/delete', undefined, paramObject).then(function(data){
                            clearInfo();
                            doSearch();
                            if (chkPredefined) {
                                MessageService.displayWarningMessage(MessageProperties.get("setting.cardFormat.predefinedFormatCannotBeDeleted"));
                            }
                        });
                    } else {
                        if (chkPredefined) {
                            clearInfo();
                            MessageService.displayWarningMessage(MessageProperties.get("setting.cardFormat.predefinedFormatCannotBeDeleted"));
                        }
                    }
                }
            });
        };

        $scope.$on('CloseDetailView', function (event, data) {
            clearInfo();
            doSearch();
        });

        cardFormatInfo = Util.getSessionObject(Const.SESSION_STORAGE_KEY.CARD_FORMAT_INFO);

        if (cardFormatInfo) {
            $scope.detailPageUrl = $scope.DETAIL_VIEW_URL.wiegandInfo;
        }


        $scope.$watch(function () {
            return  angular.element('.panel.datagrid').width();
        }, function (nv, ov) {
            if (nv !== ov) {
                angular.element("#" + $scope.gridId).datagrid('resize');
            }
        }, true);

        angular.element($window).on('resize', function ($event) {
            var event = $event | window.event;
            angular.element("#" + $scope.gridId).datagrid('resize');
        });
        $scope.backSetting = function () {
            $location.path('/setting');
        };

        $scope.reportUrl = Const.API.BASE_URL + 'users/report';
        $scope.openPrintOptionPopupDlg = PrintOptionService.openPrintOptionPopupDlg;
        $scope.openColumnSettingPopupDlg = ColumnSettingService.openColumnSettingPopupDlg;
        ColumnSettingService.setGridId($scope.gridId);
        ColumnSettingService.setCallbackFunction($scope.SearchCallback.fn);
        var customColumnSetting = ColumnSettingService.getColumnSettingFromLocalStroage();
        if (customColumnSetting) {
            ColumnSettingService.applyChangedColumnsToGrid(customColumnSetting, false);
        }

    }]);
});