define([], function () {

    var app = angular.module('biostarWebApp');

    app.controller('InputScheduleController', ['$scope', '$document', 'API', 'Const', 'Util', 'MessageService', 'MessageProperties', function ($scope, $document, API, Const, Util, MessageService, MessageProperties) {
//            console.log($scope.timeSegments);

        var i;

        function TimeSlot(slot) {

            var _slot = {}
            _slot.startTime = slot && slot.startTime ? slot.startTime : {};
            _slot.endTime = slot && slot.endTime ? slot.endTime : {};


            return _slot;
        }

        function minutesToTimeString(minutes) {
            var time = {};

            time.hours = Math.floor(minutes / 60);
            time.minutes = minutes - (Math.floor(minutes / 60) * 60);


            time.hours = paddingZero(time.hours);
            time.minutes = paddingZero(time.minutes);

            return time;
        }

        function paddingZero(time) {
            if (time.toString().length == 1) {
                time = '0' + time;
            }

            return time;
        }


        if ($scope.useDailyIteration == true) {
            $scope.timeSegmentTitle = MessageProperties.get('schedule.day', parseInt($scope.dayIdx) + 1)
        } else if ($scope.useDailyIteration == false) {
            $scope.timeSegmentTitle = Const.FULL_DAYS_TEXT[$scope.dayIdx];
        } else {
            API.get('holiday_groups').then(function success(data) {
                var HolidayGroupList = data.HolidayGroupCollection.rows;
                $scope.timeSegmentTitle = HolidayGroupList[$scope.dayIdx].name;
            });
        }

        $scope.timeSlotList = [];

        $scope.timeSegments.sort(Util.dynamicSort('start_time'));

        for (i = 0; i < Const.TIME_SEGMENT_MAX_COUNT; i++) {
            if (i < $scope.timeSegments.length) {
                $scope.timeSlotList.push(new TimeSlot({startTime: minutesToTimeString($scope.timeSegments[i].start_time), endTime: minutesToTimeString($scope.timeSegments[i].end_time)}));
            } else {
                $scope.timeSlotList.push(new TimeSlot());
            }
        }

        $scope.timeSlotList.sort(function (a, b) {
            if (angular.isDefined(a.startTime.hours) && angular.isDefined(b.startTime.hours)) {
                if (parseInt(a.startTime.hours) != parseInt(b.startTime.hours)) {
                    return parseInt(a.startTime.hours) < parseInt(b.startTime.hours) ? -1 : 1;
                }
            }

        });

        $scope.doClearTimeSlotListInfo = function () {
//                MessageService.displayConfirmMessage(MessageProperties.get('validate.removeAllTimeSlots'), function (confirmYn) {
//                    if(confirmYn) {
            $scope.timeSlotList = [];
            for (i = 0; i < Const.TIME_SEGMENT_MAX_COUNT; i++) {
                $scope.timeSlotList.push(new TimeSlot());
            }


//                    }
//                });
        };

        function checkValidTime(time) {
            if (time.hours == undefined || !time.hours.toString().length) {
                if (time.minutes == undefined || !time.minutes.toString().length) {
                    return 'skip';

                } else {
                    return false;

                }

            } else if (isNaN(time.hours)) {
                return false;

            } else {
                if (time.minutes == undefined || !time.minutes.toString().length) {
                    return true;

                } else {
                    return !isNaN(time.minutes);

                }

            }
        }

        function checkValidTimeSlot(timeSlot, idx) {

            var startTimeValidCheckResult = checkValidTime(timeSlot.startTime),
                endTimeValidCheckResult = checkValidTime(timeSlot.endTime);

            if (startTimeValidCheckResult == 'skip' && endTimeValidCheckResult == 'skip') {
                return 'skip';
            }

            else if (startTimeValidCheckResult == false && endTimeValidCheckResult == false) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.invalidTimeSlot', parseInt(idx) + 1));
                return false;

            } else if ((startTimeValidCheckResult == false || startTimeValidCheckResult == 'skip') && endTimeValidCheckResult == true) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.invalidTimeSlotStartTime', parseInt(idx) + 1));
                return false;

            } else if (startTimeValidCheckResult == true && (endTimeValidCheckResult == false || endTimeValidCheckResult == 'skip')) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.invalidTimeSlotEndTime', parseInt(idx) + 1));
                return false;

            } else if (startTimeValidCheckResult == true && endTimeValidCheckResult == true) {
                var startHours, endHours, startMinutes, endMinutes;

                startHours = timeSlot.startTime.hours ? parseInt(timeSlot.startTime.hours) * 60 : 0;
                startMinutes = timeSlot.startTime.minutes ? parseInt(timeSlot.startTime.minutes) : 0;
                endHours = timeSlot.endTime.hours ? parseInt(timeSlot.endTime.hours) * 60 : 0;
                endMinutes = timeSlot.endTime.minutes ? parseInt(timeSlot.endTime.minutes) : 0;

                var startTime = startHours + startMinutes;
                var endTime = endHours + endMinutes;

                if (startMinutes > 59 || startMinutes < 0 || startHours < 0 || startHours > 1440) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.invalidTimeSlotStartTime', parseInt(idx) + 1));
                    return false;
                }

                if (endMinutes > 59 || endMinutes < 0 || endHours < 0 || endHours > 1440) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.invalidTimeSlotEndTime', parseInt(idx) + 1));
                    return false;
                }

                if (startTime >= endTime) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.invalidTimeSlot', parseInt(idx) + 1));
                    return false;
                } else if (startTime < 0) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.invalidTimeSlotStartTime', parseInt(idx) + 1));
                    return false;
                } else if (endTime > 1440) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.invalidTimeSlotEndTime', parseInt(idx) + 1));
                    return false;
                }

                return {start_time: startTime, end_time: endTime};

            }
        }

        $scope.doApplyTimeSegmentInfo = function () {
            var options = angular.element('#doAddTimeSegmentToTimeLineDlg').dialog('options');
            var timeSegments = [];

            for (i = 0; i < $scope.timeSlotList.length; i++) {
                var timeSlot = $scope.timeSlotList[i];
                var returnValue = checkValidTimeSlot(timeSlot, i);

                if (returnValue == false) {
                    return false;
                } else {
                    if (typeof returnValue == 'object') {
                        timeSegments.push(returnValue);
                    }
                }
            }

            $scope.$parent.$parent.setTimeSegments(timeSegments);
            options.onClose();

        };

        $scope.doCancelEditTimeSegmentInfo = function () {
            var options = angular.element('#doAddTimeSegmentToTimeLineDlg').dialog('options');
            options.onClose();
        };

    }]);
});