'use strict';

define(function () {
    var app = angular.module('biostarWebApp');

    app.register.controller('HolidayInfoController', ['$scope', '$location', 'API', 'Const', '$log', 'UUID', 'MessageService', 'MessageProperties', function ($scope, $location, API, Const, $log, UUID, MessageService, MessageProperties) {
        var currentDate = new Date();
        var page_mode = Const.PAGE_MODE.CREATE,
            holidayInfo = angular.fromJson(window.sessionStorage.getItem('holidayInfo')),
            Holiday = (function () {
                var holiday = {};
                holiday.id = undefined;
                holiday.name = undefined;
                holiday.repeat_interval = Const.DEFAULT_REPEAT_INTERVAL;
                holiday.repeat_type = undefined;
                holiday.start_date = new Date(currentDate.getFullYear(), currentDate.getMonth(), currentDate.getDate()).buildCurrentTimeWithNoTimeZoneInUTCString(),
                    holiday.month = undefined;
                holiday.week = undefined;
                holiday.day_of_week = undefined;
                holiday.day = undefined;
                holiday.duration = undefined;

                return holiday;
            });

        // Page 이전/다음 버튼 영역 -start-
        $scope.currPos = window.sessionStorage.getItem('holidayCurrPos') || undefined;

        $scope.prevHolidayInfoBtn = function () {
            var currPos = window.sessionStorage.getItem('holidayCurrPos');
            var prevIndex = currPos - 2;
            var allHolidayIdList = window.sessionStorage.getItem('getAllHolidayIdList');

            if (allHolidayIdList && currPos) {
                allHolidayIdList = angular.fromJson(allHolidayIdList);
            } else {
                $scope.doCancelHolidayGroupInfo();
                return false;
            }

            if (prevIndex < 0) {
                return false;
            }

            $scope.currPos = parseInt(currPos) - 1;

            window.sessionStorage.setItem('holidayCurrPos', $scope.currPos);

            if (allHolidayIdList[prevIndex] && allHolidayIdList[prevIndex] != null) {
                getHolidayInfo(allHolidayIdList[prevIndex]);
            } else {
                getAllHolidayIdList(prevIndex);
            }
        };

        $scope.nextHolidayInfoBtn = function () {
            var currPos = window.sessionStorage.getItem('holidayCurrPos');
            var nextIndex = currPos;
            var allHolidayIdList = window.sessionStorage.getItem('getAllHolidayIdList');

            if (allHolidayIdList && currPos) {
                allHolidayIdList = angular.fromJson(allHolidayIdList);
            } else {
                $scope.doCancelHolidayGroupInfo();
                return false;
            }

            if (currPos >= allHolidayIdList.length) {
                return false;
            }

            $scope.currPos = parseInt(currPos) + 1;

            window.sessionStorage.setItem('holidayCurrPos', $scope.currPos);

            if (allHolidayIdList[nextIndex] && allHolidayIdList[nextIndex] != null) {
                getHolidayInfo(allHolidayIdList[nextIndex]);
            } else {
                getAllHolidayIdList(nextIndex);
            }
        };

        function getAllHolidayIdList(index) {
            var i, allHolidayIdList, startIdx;
            var limit = angular.copy($scope.holidayListQueryStrings.limit);

            allHolidayIdList = window.sessionStorage.getItem('getAllHolidayIdList');

            if (allHolidayIdList) {
                allHolidayIdList = angular.fromJson(allHolidayIdList);
            } else {
                allHolidayIdList = [];
            }

            $scope.holidayListQueryStrings.offset = Math.floor(index / limit) * limit;

            startIdx = parseInt($scope.holidayListQueryStrings.offset);

            API.get('holiday_groups', 'getAllHolidayIdList', $scope.holidayListQueryStrings).then(function success(data) {
                if (allHolidayIdList.length != data.HolidayGroupCollection.total) {
                    allHolidayIdList = [];
                }

                limit = parseInt($scope.holidayListQueryStrings.limit);

                if (parseInt($scope.holidayListQueryStrings.offset) + parseInt($scope.holidayListQueryStrings.limit) > data.HolidayGroupCollection.total) {
                    limit = parseInt(data.HolidayGroupCollection.total) - $scope.holidayListQueryStrings.offset;
                }

                for (i = 0; i < limit; i++) {
                    console.log(startIdx + i);
                    allHolidayIdList[startIdx + i] = data.HolidayGroupCollection.rows[i].id;
                }

                console.log(allHolidayIdList[index]);
                getHolidayInfo(allHolidayIdList[index]);

                window.sessionStorage.setItem('getAllHolidayIdList', angular.toJson(allHolidayIdList));
            });
        }

        function getHolidayInfo(id) {
            API.get('holiday_groups/' + id).then(function success(data) {
                holidayInfo = data.HolidayGroup;
                window.sessionStorage.setItem('holidayInfo', angular.toJson(data.HolidayGroup));
                $scope.holidayGroup = angular.copy(holidayInfo);
                page_mode = Const.PAGE_MODE.UPDATE;
//                console.log('page_mode::'+page_mode)
                $scope.currHolidayName = $scope.holidayGroup.name;
                API.get('holiday_groups/' + $scope.holidayGroup.id + '/holidays').then(function (data) {
                    $scope.holidayList = data.HolidayCollection.rows;
                });
            });
        };
        // Page 이전/다음 버튼 영역 -end-

        function addHoliday() {
            $scope.holidayGroup = undefined;
            $scope.holidayList = undefined;
            page_mode = Const.PAGE_MODE.CREATE;
//            console.log('page_mode::'+page_mode)

            if (Object.keys(holidayInfo).length > 0) {
                $scope.holidayGroup = angular.copy(holidayInfo);
                page_mode = Const.PAGE_MODE.UPDATE;
//                console.log('page_mode::'+page_mode)
                $scope.currHolidayName = $scope.holidayGroup.name;
                API.get('holiday_groups/' + $scope.holidayGroup.id + '/holidays').then(function (data) {
                    $scope.holidayList = data.HolidayCollection.rows;
                });
            }
        }

        $scope.doDeleteHolidayGroup = function (item) {
            MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                if (confirmYn) {
                    API.del('holiday_groups/' + item.id).then(function (data) {
                        $scope.doCancelHolidayGroupInfo();
                    });
                } else {

                }
            });
        };

        $scope.doAddHolidaysItem = function () {
            if (angular.isUndefined($scope.holidayList)) {
                $scope.holidayList = [];
            }

            console.log($scope.holidayList.length);
            if ($scope.holidayList.length >= Const.HOLIDAY_LIMIT) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.allowMaximum128Holidays'));
                return;
            }

            $scope.holidayList.push(new Holiday());
        };

        $scope.doDeleteHoliday = function ($index) {
            MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                if (confirmYn) {
                    $scope.holidayList.splice($index, 1);
                }
            });
        };

        $scope.doApplyHolidayGroupInfo = function () {
            var call_api = API.post, id = '';
            if (page_mode == Const.PAGE_MODE.UPDATE) {
                call_api = API.put;
                id = '/' + $scope.holidayGroup.id;
            }

            if (!$scope.holidayGroup) {
                MessageService.displayWarningMessage(MessageProperties.get('common.pleaseEnterName'));
                return false;
            }

            if ($scope.holidayGroup.name == '' || $scope.holidayGroup.name == undefined) {
                MessageService.displayWarningMessage(MessageProperties.get('common.pleaseEnterName'));
                return;
            }

            $scope.holidayGroup.holidays = angular.copy($scope.holidayList);

            call_api('holiday_groups' + id, undefined, {'HolidayGroup': $scope.holidayGroup}).then(function (data) {
                $scope.doCancelHolidayGroupInfo();
            });
        };

        $scope.doCancelHolidayGroupInfo = function () {
            $scope.$emit('CloseHolidayInfo');
        };


        addHoliday()

    }]);
});