'use strict';

define([
        'modules/common/services/scheduleAgent',
        'modules/setting/scheduleInfoController',
        'modules/common/datagrid/columnSettingController',
        'modules/common/datagrid/columnSettingService',
        'modules/common/print/printOptionController',
        'modules/common/print/printOptionService'
    ],
    function () {

        var app = angular.module('biostarWebApp');

        app.register.controller('ScheduleController', ['$scope', '$rootScope', '$location', '$compile', '$q', '$window', 'API', 'Util', 'Const', 'ScheduleAgent', 'MessageProperties', 'ColumnSettingService', 'PrintOptionService', 'MessageService', function ($scope, $rootScope, $location, $compile, $q, $window, API, Util, Const, ScheduleAgent, MessageProperties, ColumnSettingService, PrintOptionService, MessageService) {

            $scope.gridId = "scheduleGrid";
            var menuID = "#scheduleContextMenu";

            $scope.clearCheckItemArray = function () {
                $scope.checkItemArray = [];
            };

            var i,
                scheduleInfo = undefined,
                scheduleListQueryStrings = {
                    query: undefined,
                    offset: 0,
                    limit: 15,
                    total: undefined
                },
                scheduleListDataGridOptions = {
                    scrollbarSize: 0,
                    view: $.extend({}, $.fn.datagrid.defaults.view, {
                        onBeforeRender: function (target, rows) {
                            $("#scheduleGrid").datagrid('resize');
                        }, onAfterRender: function (target) {
                            $.fn.datagrid.defaults.view.onAfterRender.call(this, target);
                            var grdPanel = angular.element(target).datagrid('getPanel');
                            var opts = angular.element(target).datagrid('options');
                            var vc = grdPanel.children('div.datagrid-view');
                            grdPanel.find(".datagrid-empty").remove();

                            if (angular.element(target).datagrid('getRows').length === 0) {
                                var html = '<div style="width:100%; height:360px; font-size:1.333em; line-height: 360px; ' +
                                    'vertical-align: middle; text-align: center;">' + MessageProperties.get('validate.notFound') +
                                    '</div>';
                                grdPanel.find('div.datagrid-view2 div.datagrid-body').html(html);
                            }

                            //헤더에 전체선택 버튼이 없다면 추가
                            if (grdPanel.find('.datagrid-header-check.compiled').length <= 0) {
                                grdPanel.find('.datagrid-header-check').attr({
                                    'select-all-yn': 'SelectAllYN',
                                    'datagrid-id': '"' + target.id + '"',
                                    'clear-function': 'clearCheckItemArray'
                                });
                                $compile(grdPanel.find('.datagrid-header-row'))($scope);
                                grdPanel.find('.datagrid-header-check').addClass('compiled');
                            }
                        }
                    }),
                    fitColumns: true,
                    resize: true,
                    singleSelect: true,
                    selectOnCheck: false,
                    checkOnSelect: false,
                    columns: [
                        [
                            {
                                field: Const.FIELD_NAME.CHECKBOX, checkbox: true
                            },
                            {
                                field: 'name', title: MessageProperties.get('common.name'), width: 80, align: 'left'
                            },
                            {
                                field: 'description', title: MessageProperties.get('common.description'), width: 80, align: 'left'
                            },
                            {
                                field: 'use_daily_iteration', title: MessageProperties.get('common.type'), width: 80, align: 'left',
                                formatter: function (value, row, index) {
                                    if (value == 'true') {
                                        return MessageProperties.get('schedule.daily');
                                    }
                                    return MessageProperties.get('schedule.weekly');

                                }},
                            {
                                field: 'days_of_iteration', title: MessageProperties.get('schedule.cycle'), width: 80, align: 'left',
                                formatter: function (value, row, index) {
                                    if (row.use_daily_iteration == 'true') {
                                        return value + ' ' + MessageProperties.get('schedule.days');
                                    }
                                    return '-';
                                }
                            }
                        ]
                    ],
                    onCheck: function (rowIndex, rowData) {
                        var existsYn = $scope.checkItemArray.indexOf(rowData.id);
                        if (existsYn == -1) {
                            if ($scope.checkItemArray.length >= Const.SELECT_LIST_ITEM_LIMIT) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.canNotSelectMoreItem', Const.SELECT_LIST_ITEM_LIMIT));
                                angular.element('#scheduleGrid').datagrid('uncheckRow', rowIndex);
                            } else {
                                $scope.checkItemArray.push(rowData.id);
                                if (!$scope.$root.$$phase) $scope.$apply();
                            }
                        }
                    },
                    onUncheck: function (rowIndex, rowData) {
                        if ($scope.SelectAllYN == true) {
                            $scope.checkItemArray = [];
                            var data = angular.element('#scheduleGrid').datagrid('getChecked');
                            $scope.SelectAllYN = false;
                            if (!$scope.$root.$$phase) $scope.$apply();
                            for (i = 0; i < data.length; i++) {
                                var index = angular.element('#scheduleGrid').datagrid('getRowIndex', data[i]);
                                angular.element('#scheduleGrid').datagrid('checkRow', index);
                            }
                        } else {
                            var idx = $scope.checkItemArray.indexOf(rowData.id);
                            if (idx > -1) {
                                $scope.checkItemArray.splice(idx, 1);

                                if (!$scope.$root.$$phase) $scope.$apply();
                            }
                        }
                    },
                    onCheckAll: function (rows) {
                        var i, uncheckList = [];
                        for (i = 0; i < rows.length; i++) {
                            var item = rows[i];
                            var idx = $scope.checkItemArray.indexOf(item.id);

                            if (idx == -1) {
                                if ($scope.checkItemArray.length < Const.SELECT_LIST_ITEM_LIMIT) {
                                    $scope.checkItemArray.push(item.id);
                                } else {
                                    uncheckList.push(item);
                                }
                            }
                        }

                        if (uncheckList.length > 0) {
                            MessageService.displayWarningMessage(MessageProperties.get('validate.canNotSelectMoreItem', Const.SELECT_LIST_ITEM_LIMIT));
                            for (i = uncheckList.length; i--;) {
                                console.log(uncheckList[i].user_id);
                                angular.element('#scheduleGrid').datagrid('uncheckRow', angular.element('#scheduleGrid').datagrid('getRowIndex', uncheckList[i]));
                            }
                        }


                        console.log($scope.checkItemArray);

                        if (!$scope.$root.$$phase) $scope.$apply();
                    },
                    onUncheckAll: function (rows) {
                        if ($scope.SelectAllYN == true) {
                            $scope.SelectAllYN = false;
                            $scope.checkItemArray = [];
                        }
                        for (var i = rows.length; i--;) {
                            var item = rows[i];
                            var idx = $scope.checkItemArray.indexOf(item.id);
                            if (idx > -1) {
                                $scope.checkItemArray.splice(idx, 1);
                            }
                        }
                        if (!$scope.$root.$$phase) $scope.$apply();
                    },
                    onClickCell: function (rowIndex, field) {
                        $rootScope.clickCellOnGrid($scope.gridId, rowIndex, field, "id", function (rowData) {
                            if (rowData.id == Const.DEFAULT_SCHEDULE.NO_TIME || rowData.id == Const.DEFAULT_SCHEDULE.ALWAYS) {
                                return;
                            }
                            API.get('schedules/' + rowData.id, 'getOneSchedule').then(function success(data) {
                                window.sessionStorage.setItem('scheduleCurrPos', parseInt($scope.scheduleListQueryStrings.offset) + rowIndex + 1);
                                window.sessionStorage.setItem('scheduleInfo', angular.toJson(data.Schedule));
                                setScheduleInfoUrl()
                            });
                        });
                    },
                    onSortColumn: function (sort, order) {
                        var i,
                            orderItemArray,
                            descending = order == 'desc',
                            orderItemObject = {};

                        $scope.scheduleListQueryStrings.order_by = sort + ":" + descending;

                        beSearchScheduleList();
                    }
                };

            function setScheduleInfoUrl() {
                $scope.selectedScheduleInfo = 'modules/setting/scheduleInfo.html';
            }


            function CheckUnCheckOnGrid(rowData, CheckAllYN) {
                var i, idx, item;

                if (rowData.constructor == Object) {
                    if (rowData.id == Const.DEFAULT_SCHEDULE.NO_TIME || rowData.id == Const.DEFAULT_SCHEDULE.ALWAYS) {
                        return;
                    }
                    idx = $scope.checkItemArray.indexOf(rowData.id);
                    if (idx > -1) {
                        $scope.checkItemArray.splice(idx, 1);
                    } else {
                        $scope.checkItemArray.push(rowData.id);
                    }

                } else {
                    if (CheckAllYN) {
                        for (i = rowData.length; i--;) {
                            item = rowData[i];
                            if (item.id == Const.DEFAULT_SCHEDULE.NO_TIME || item.id == Const.DEFAULT_SCHEDULE.ALWAYS) {
                                continue;
                            }

                            idx = $scope.checkItemArray.indexOf(item.id);
                            if (idx == -1) {
                                $scope.checkItemArray.push(item.id);
                            }
                        }
                    } else {
                        for (i = rowData.length; i--;) {
                            item = rowData[i];
                            if (item.id == Const.DEFAULT_SCHEDULE.NO_TIME || item.id == Const.DEFAULT_SCHEDULE.ALWAYS) {
                                continue;
                            }

                            idx = $scope.checkItemArray.indexOf(item.id);
                            if (idx > -1) {
                                $scope.checkItemArray.splice(idx, 1);
                            }
                        }
                    }
                }

                if (!$scope.$root.$$phase) $scope.$apply();

            }

            $scope.onClickScheduleTab = function (url) {
                console.log(url);
                $location.path(url);
            };

            $scope.backSetting = function () {
                $location.path('/setting');
            };

            $scope.doClickScheduleListRow = function (item, $index) {
                AccessControlAgent.getOneSchedule(item.id).then(function success(data) {
                    window.sessionStorage.setItem('scheduleCurrPos', parseInt($scope.scheduleListQueryStrings.offset) + $index + 1);
                    setScheduleInfoUrl()
                });
            };

            $scope.checkItemArray = [];

            $scope.scheduleListQueryStrings = scheduleListQueryStrings;
            $scope.ScheduleListData = [];

            angular.element('#scheduleGrid').datagrid(scheduleListDataGridOptions);

            //헤더에 전체선택 버튼 추가
            $scope.SelectAllYN = false;

            $scope.$watch('SelectAllYN', function (nv) {
                if (nv) {
                    angular.element('#scheduleGrid').datagrid('checkAll');
                } else {
                    angular.element('#scheduleGrid').datagrid('uncheckAll');
                    $scope.checkItemArray = [];
                }
            });

            $scope.cancelSelectAllSchedule = function () {
                angular.element('#scheduleGrid').datagrid('uncheckAll');
            };

            function reloadScheduleLNBInfo() {
                ScheduleAgent.getScheduleList(undefined, 'reloadScheduleLNBInfo').then(function (data) {
                    $scope.scheduleList = data.ScheduleCollection.rows;
                });
            }

            reloadScheduleLNBInfo();

            function beSearchScheduleList(query, deferFunctionName) {
                var i, rowData, allScheduleIdList, startIdx, limit;

                if (query) {
                    scheduleListQueryStrings.query = query;
                }

                ScheduleAgent.getScheduleList($scope.scheduleListQueryStrings).then(function success(data) {
                    $scope.ScheduleListData = data.ScheduleCollection.rows || [];
                    Util.setSessionObject('ScheduleListData', angular.toJson($scope.ScheduleListData));

                    angular.element('#scheduleGrid').datagrid('loadData', $scope.ScheduleListData);

                    allScheduleIdList = new Array(parseInt(data.ScheduleCollection.total));

                    startIdx = parseInt($scope.scheduleListQueryStrings.offset);
                    limit = parseInt($scope.scheduleListQueryStrings.limit);

                    if (parseInt($scope.scheduleListQueryStrings.offset) + parseInt($scope.scheduleListQueryStrings.limit) > data.ScheduleCollection.total) {
                        limit = parseInt(data.ScheduleCollection.total) - $scope.scheduleListQueryStrings.offset;
                    }

                    for (i = 0; i < limit; i++) {
                        allScheduleIdList[startIdx + i] = data.ScheduleCollection.rows[i].id;
                    }

                    window.sessionStorage.setItem('getAllScheduleIdList', angular.toJson(allScheduleIdList));


                    if ($scope.SelectAllYN) {
                        for (i = $scope.ScheduleListData.length; i--;) {
                            rowData = $scope.ScheduleListData[i];
                            if ($scope.checkItemArray.indexOf(rowData.id) < 0) {
                                $scope.checkItemArray.push(rowData.id);
                            }
                        }
                    }

                    scheduleListQueryStrings.total = data.ScheduleCollection.total || 0;

                    if (!$scope.SelectAllYN) {
                        angular.element('.datagrid-header-check').find('input').attr('checked', false);
                    } else {

                    }

                    for (i = $scope.ScheduleListData.length; i--;) {
                        rowData = $scope.ScheduleListData[i];
                        if ($scope.checkItemArray.indexOf(rowData.id) > -1) {
                            angular.element('#scheduleGrid').datagrid('checkRow', i);
                        }
                    }

                });
            }

            beSearchScheduleList();


            $scope.SearchCallback = {
                fn: beSearchScheduleList
            };

            $scope.$on('ReloadSchedulesDataGrid', function (event, data) {
                angular.element('#scheduleGrid').datagrid('uncheckAll');
                scheduleListQueryStrings.offset = 0;
                if (data && data.offset) {
                    scheduleListQueryStrings.offset = data.offset;
                }
                beSearchScheduleList();
                reloadScheduleLNBInfo();
            });

            // Delete Schedule
            $scope.deleteSelectedScheduleInfo = function (itemIdx) {

                angular.element(menuID).menu("hide");

                var param = $scope.checkItemArray.join('+');
                if ($scope.SelectAllYN) {
                    param = Const.SELECTED_ALL;
                }

                if (itemIdx != undefined) {
                    param = [];
                    param.push(itemIdx);
                }

                MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                    if (confirmYn) {
                        API.del('schedules' + '?id=' + param).then(function success(data) {
                            $scope.$emit('ReloadSchedulesDataGrid');
                        }, function error(data) {
                            var idx = $scope.checkItemArray.indexOf(Const.DEFAULT_SCHEDULE.ALWAYS);
                            if (idx > -1 && $scope.checkItemArray.length > 1) {
                                $scope.checkItemArray.splice(idx, 1);

                                param = $scope.checkItemArray.join('+');
                                API.del('schedules' + '?id=' + param).then(function success(data) {
                                    $scope.$emit('ReloadSchedulesDataGrid');
                                }, function error(data) {

                                });
                            }
                        });
                    }
                });


            };

            $scope.$watch(function () {
                return  angular.element('.panel.datagrid').width();
            }, function (nv, ov) {
                if (nv !== ov) {
                    angular.element('#scheduleGrid').datagrid('resize');
                }
            }, true);

            angular.element($window).on('resize', function ($event) {
                var event = $event | window.event;
                angular.element('#scheduleGrid').datagrid('resize');
                Util.debug('div#scheduleGrid resize');
            });

            $scope.dropDownMenu = [
//                    {
//                        menu: MessageProperties.get('menu.print'),
//                        func: function () {
//                            PrintOptionService.openPrintOptionPopupDlg();
//                        }
//                    },
                {
                    menu: MessageProperties.get('menu.columnSetting'),
                    func: function () {
                        ColumnSettingService.openColumnSettingPopupDlg();
                    }
                }
            ];

            $scope.doAddScheduleInfo = function () {
                if ($scope.scheduleListQueryStrings.total < Const.SCHEDULE_LIMIT) {
                    $scope.$emit('OpenScheduleInfo');
                } else {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.only128ItemsAreAllowedForSchedules'));
                }
            };

            $scope.doLoadScheduleInfo = function (item) {
                if (item.id == Const.DEFAULT_SCHEDULE.NO_TIME || item.id == Const.DEFAULT_SCHEDULE.ALWAYS) {
                    return;
                }
                API.get('schedules/' + item.id, 'getOneSchedule').then(function success(data) {
                    window.sessionStorage.setItem('scheduleInfo', angular.toJson(data.Schedule));
                    setScheduleInfoUrl()
                });
            };

            var clickMenuInitUnbind = $rootScope.$on('ClickMenuInit', function (event, data) {
                window.sessionStorage.removeItem('scheduleInfo');
                window.sessionStorage.removeItem('ScheduleListData');
                window.sessionStorage.removeItem('scheduleCurrPos');
                $scope.$emit('ReloadSchedulesDataGrid');
                $scope.selectedScheduleInfo = undefined;
            });

            $scope.$on('$locationChangeStart', clickMenuInitUnbind);

            $scope.$on('CloseScheduleInfo', function (event, data) {
                window.sessionStorage.removeItem('scheduleInfo');
                window.sessionStorage.removeItem('ScheduleListData');
                window.sessionStorage.removeItem('scheduleCurrPos');
                $scope.$emit('ReloadSchedulesDataGrid');
                $scope.selectedScheduleInfo = undefined;
            });

            $scope.$on('OpenScheduleInfo', function (event, data) {
                window.sessionStorage.setItem('scheduleInfo', '{}');
                setScheduleInfoUrl();
            });

            scheduleInfo = window.sessionStorage.getItem('scheduleInfo');

            if (scheduleInfo && scheduleInfo != 'undefined') {
                scheduleInfo = angular.fromJson(scheduleInfo);
                setScheduleInfoUrl()
            }

            $scope.doAddHolidayInfo = function () {
                window.sessionStorage.setItem('holidayInfo', '{}');
                $location.path('/setting/holiday')
            };


            $scope.$watch('scheduleListQueryStrings', function (nv, ov) {
                if (nv !== ov) {
                    // TODO BSS-195 BSS-192 server api not implemented
//                        PrintOptionService.buildGridQueryFunction(Const.REPORT_COLLECTION_NAME.ACCESS_GROUP, nv);
                }
            }, true);

            $scope.selectedLnbItem = undefined;

            function initContextMenu() {
                angular.element(menuID).menu({});
            }

            function clearMenuContext(divId) {
                angular.element(divId).find(".menu-item").remove();
            }

            function buildContextMenuDiv(contextMenuObject, ngIfCondition) {
                var html = '<div';
                if (ngIfCondition != undefined) {
                    html += ' ng-if="' + ngIfCondition + '" ';
                }
                html += ' class="menu-item" style="height:31px;" ng-click="' + contextMenuObject.clickFunctionName + '()" ng-privilege><div class="menu-text">' + contextMenuObject.menuName + '</div></div>';
                return html;
            }

            var ContextMenu = {
                'DELETE_SCHEDULE': {
                    menuName: MessageProperties.get('button.deleteSchedule'), clickFunctionName: 'deleteSelectedScheduleInfo(selectedLnbItem)'
                }
            };

            initContextMenu();

            angular.element('.leftTreeMenu').on('contextmenu', function ($event) {
                var event = $event || window.event,
                    target = event.target;

                var top;

                var idx = angular.element(target).closest('li').attr('data-list-index');

                $scope.selectedLnbItem = $scope.scheduleList[idx].id;

                if (event.preventDefault()) event.preventDefault();
                if (event.returnValue) event.returnValue = false;

                event.cancelBubble = true;

                if ($scope.selectedLnbItem != Const.DEFAULT_SCHEDULE.NO_TIME && $scope.selectedLnbItem != Const.DEFAULT_SCHEDULE.ALWAYS) {
                    angular.element(menuID).menu("hide");
                    clearMenuContext(menuID);

                    angular.element(menuID).append(buildContextMenuDiv(ContextMenu.DELETE_SCHEDULE));

                    angular.element(menuID).css("height", "auto");
                    angular.element(menuID).css("width", "auto");

                    $compile(angular.element(menuID))($scope);

                    angular.element(menuID).menu('show', {
                        left: event.pageX,
                        top: event.pageY
                    });
                }
            });

            PrintOptionService.getReportColumnNameFunction = function (gridColumnName) {
                var reportColumnName = gridColumnName;
                if (gridColumnName == "schedule_group_id") {
                    return "schedule_groups.name"
                } else if (gridColumnName == "access_groups") {
                    return "access_groups.name"
                } else if (gridColumnName == "fingerprint_templates") {
                    return "finger-tempates.count"
                } else if (gridColumnName == "cards") {
                    return "cards.count"
                }
                return reportColumnName;
            };
            $scope.reportUrl = Const.API.BASE_URL + 'schedules/report';
            $scope.openPrintOptionPopupDlg = PrintOptionService.openPrintOptionPopupDlg;
            $scope.openColumnSettingPopupDlg = ColumnSettingService.openColumnSettingPopupDlg;

            ColumnSettingService.setGridId($scope.gridId);
            ColumnSettingService.setCallbackFunction($scope.SearchCallback.fn);
            var customColumnSetting = ColumnSettingService.getColumnSettingFromLocalStroage();
            if (customColumnSetting) {
                ColumnSettingService.applyChangedColumnsToGrid(customColumnSetting, false);
            }

			$scope.$on('$destroy', function() {
				window.sessionStorage.removeItem('scheduleInfo');
				window.sessionStorage.removeItem('ScheduleListData');
				window.sessionStorage.removeItem('scheduleCurrPos');
			});
        }]);
    });