'use strict';

define(['modules/common/services/scheduleAgent'], function () {
    var app = angular.module('biostarWebApp')
    app.register.controller('ScheduleInfoController', ['$scope', '$location', 'API', 'Const', 'Util', '$log', 'UUID', 'ScheduleAgent', 'MessageProperties', 'MessageService', function ($scope, $location, API, Const, Util, $log, UUID, ScheduleAgent, MessageProperties, MessageService) {

        var page_mode = Const.PAGE_MODE.CREATE,
            scheduleInfo = angular.fromJson(window.sessionStorage.getItem('scheduleInfo')),
            keepHolidaySchedulesValue = undefined;

        var currentDate = new Date();

        var Schedules = (function (_schedules) {
            var i,
                id,
                name,
                description,
                schedules = angular.copy(_schedules),
                daily_schedules = [],
                holiday_schedules = [],
                days_of_iteration = 7,
                start_date = new Date(currentDate.getFullYear(), currentDate.getMonth(), currentDate.getDate()).buildCurrentTimeWithNoTimeZoneInUTCString(),
                use_daily_iteration = false;


            if (schedules) {
                id = schedules.id;
                name = schedules.name;
                description = schedules.description;
                daily_schedules = schedules.daily_schedules;
                holiday_schedules = schedules.holiday_schedules;
                days_of_iteration = parseInt(schedules.days_of_iteration);
                start_date = schedules.start_date || start_date;
                use_daily_iteration =
                        schedules.use_daily_iteration.constructor == String ? JSON.parse(schedules.use_daily_iteration) : schedules.use_daily_iteration;

                if (use_daily_iteration == false && daily_schedules && daily_schedules.length < days_of_iteration) {
                    var idx,
                        item,
                        _daily_schedules = [];
                    for (i = days_of_iteration; i--;) {
                        _daily_schedules.push({
                            day_index: i,
                            time_segments: []
                        });
                    }
                    for (i = 0; i < daily_schedules.length; i++) {
                        idx = daily_schedules[i].day_index;
                        if (!daily_schedules[i].time_segments) {
                            daily_schedules[i].time_segments = [];
                        }
                        _daily_schedules[idx] = angular.copy(daily_schedules[i]);
                    }
                    daily_schedules = angular.copy(_daily_schedules);
                } else if (use_daily_iteration == true) {
                    if (!daily_schedules) {
                        daily_schedules = [];
                    }

                    if (daily_schedules && daily_schedules.length > days_of_iteration) {
                        daily_schedules.splice(days_of_iteration, daily_schedules.length);

                    } else if (daily_schedules && daily_schedules.length < days_of_iteration) {

                        for (i = daily_schedules.length; i < days_of_iteration; i++) {
                            daily_schedules.push({
                                day_index: i,
                                time_segments: []
                            });
                        }
                    }
                }
            }

            if (daily_schedules != undefined && !daily_schedules.length) {
                for (i = 0; i < days_of_iteration; i++) {
                    daily_schedules.push({
                        day_index: i,
                        time_segments: []
                    });
                }
            }

            return {
                id: id,
                name: name,
                description: description,
                daily_schedules: daily_schedules,
                holiday_schedules: holiday_schedules,
                days_of_iteration: days_of_iteration,
                start_date: start_date,
                use_daily_iteration: use_daily_iteration
            }
        });

        $scope.doAddSchedule = function () {
            $scope.holidayScheduleYN = false;
            $scope.schedules = new Schedules();

            page_mode = Const.PAGE_MODE.CREATE;
            if (Object.keys(scheduleInfo).length > 0) {
                $scope.schedules = new Schedules(scheduleInfo);
                $scope.currScheduleName = $scope.schedules.name;
                page_mode = Const.PAGE_MODE.UPDATE;

                if ($scope.schedules.holiday_schedules && $scope.schedules.holiday_schedules.length) {
                    $scope.holidayScheduleYN = true;
                }
            }
        };

        $scope.changeUseDailyIterationRadio = function () {
            var _schedules = new Schedules();
            $scope.schedules.daily_schedules = angular.copy(_schedules.daily_schedules);
            $scope.schedules.days_of_iteration = $scope.schedules.daily_schedules.length;
        };

        $scope.changeDailyIterationValue = function () {
            if ($scope.schedules.days_of_iteration <= 90 && $scope.schedules.days_of_iteration >= 1) {
                $scope.schedules = new Schedules($scope.schedules);
            }
        };

        $scope.doCopyYesterdayTimeLine = function ($index) {
            $scope.schedules.daily_schedules[$index].time_segments = angular.copy($scope.schedules.daily_schedules[$index - 1].time_segments);
        };

        $scope.changeHolidayScheduleYNCheckBox = function () {
            if ($scope.holidayScheduleYN) {
//                    if (keepHolidaySchedulesValue) {
//                        $scope.schedules.holiday_schedules = angular.copy(keepHolidaySchedulesValue);
//                    } else {
                $scope.schedules.holiday_schedules = [];
                $scope.schedules.holiday_schedules.push({
                    holiday_group_id: {},
                    time_segments: []
                });
//                    }
            } else {
//                    keepHolidaySchedulesValue = angular.copy($scope.schedules.holiday_schedules);
                $scope.schedules.holiday_schedules = [];
            }
        };

        $scope.addHoliday = function () {
            if ($scope.schedules.holiday_schedules.length < 4) {
                $scope.schedules.holiday_schedules.push({
                    holiday_group_id: {},
                    time_segments: []
                });
            }
        };

        $scope.deleteHoliday = function ($index) {
            MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                if (confirmYn) {
                    $scope.schedules.holiday_schedules.splice($index, 1);
                    if (!$scope.schedules.holiday_schedules.length) {
                        $scope.holidayScheduleYN = false;
                    }
                    if (!$scope.$root.$$phase) $scope.$apply();
                }
            });

        };

        $scope.doApplyScheduleInfo = function () {
            var i,
                holiday_schedules = [],
                call_api = API.post,
                id = '';

            if ($.trim($scope.schedules.name).length <= 0) {
                MessageService.displayWarningMessage(MessageProperties.get('common.pleaseEnterName'));
                return false;
            }

            if ($scope.schedules.holiday_schedules) {

                for (i = 0; i < $scope.schedules.holiday_schedules.length; i++) {
                    var item = $scope.schedules.holiday_schedules[i];
                    if (!item.holiday_group_id || Object.keys(item.holiday_group_id).length == 0) {
                        MessageService.displayWarningMessage(MessageProperties.get('schedule.pleaseInputHolidayGroup'));
                        return false;
                    }

//                        if (!item.time_segments.length) {
//                            MessageService.displayWarningMessage(MessageProperties.get('schedule.pleaseInputTimeSegment'));
//                            return false;
//                        }

                    holiday_schedules.push(item);
                }

                $scope.schedules.holiday_schedules = angular.copy(holiday_schedules);
            }

            if (page_mode == Const.PAGE_MODE.UPDATE) {
                call_api = API.put;
                id = '/' + $scope.schedules.id;
            }

//                Util.debug(angular.toJson($scope.schedules));
            $scope.schedules.days_of_iteration = $scope.schedules.daily_schedules.length;
            call_api('schedules' + id, undefined, {"Schedule": $scope.schedules}).then(function (data) {
                $scope.doCancelScheduleInfo();
            });
        };

        $scope.doCancelScheduleInfo = function () {
            $scope.$emit('CloseScheduleInfo');
        };

        // Page 이전/다음 버튼 영역 -start-
        $scope.currPos = window.sessionStorage.getItem('scheduleCurrPos') || undefined;

        $scope.prevScheduleInfoBtn = function () {
            var currPos = window.sessionStorage.getItem('scheduleCurrPos');
            var prevIndex = currPos - 2;
            var allScheduleIdList = window.sessionStorage.getItem('getAllScheduleIdList');

            if (allScheduleIdList && currPos) {
                allScheduleIdList = angular.fromJson(allScheduleIdList);
            } else {
                $scope.doCancelScheduleInfo();
                return false;
            }

            if (prevIndex < 0) {
                return false;
            }

            if (allScheduleIdList[prevIndex] == Const.DEFAULT_SCHEDULE.NO_TIME || allScheduleIdList[prevIndex] == Const.DEFAULT_SCHEDULE.ALWAYS) {
                return;
            }

            $scope.currPos = parseInt(currPos) - 1;

            window.sessionStorage.setItem('scheduleCurrPos', $scope.currPos);

            if (allScheduleIdList[prevIndex] && allScheduleIdList[prevIndex] != null) {
                getAcGroupInfo(allScheduleIdList[prevIndex]);
            } else {
                getAllScheduleIdList(prevIndex);
            }
        };

        $scope.nextScheduleInfoBtn = function () {
            var currPos = window.sessionStorage.getItem('scheduleCurrPos');
            var nextIndex = currPos;
            var allScheduleIdList = window.sessionStorage.getItem('getAllScheduleIdList');

            if (allScheduleIdList && currPos) {
                allScheduleIdList = angular.fromJson(allScheduleIdList);
            } else {
                $scope.doCancelScheduleInfo();
                return false;
            }

            if (currPos >= allScheduleIdList.length) {
                return false;
            }

            $scope.currPos = parseInt(currPos) + 1;

            window.sessionStorage.setItem('scheduleCurrPos', $scope.currPos);

            if (allScheduleIdList[nextIndex] && allScheduleIdList[nextIndex] != null) {
                getAcGroupInfo(allScheduleIdList[nextIndex]);
            } else {
                getAllScheduleIdList(nextIndex);
            }
        };

        function getAllScheduleIdList(index) {
            var i, allScheduleIdList, startIdx;
            var limit = angular.copy($scope.scheduleListQueryStrings.limit);

            allScheduleIdList = window.sessionStorage.getItem('getAllScheduleIdList');

            if (allScheduleIdList) {
                allScheduleIdList = angular.fromJson(allScheduleIdList);
            } else {
                allScheduleIdList = [];
            }

            $scope.scheduleListQueryStrings.offset = Math.floor(index / limit) * limit;

            startIdx = parseInt($scope.scheduleListQueryStrings.offset);

            ScheduleAgent.getScheduleList($scope.scheduleListQueryStrings).then(function success(data) {
                if (allScheduleIdList.length != data.ScheduleCollection.total) {
                    allScheduleIdList = [];
                }

                limit = parseInt($scope.scheduleListQueryStrings.limit);

                if (parseInt($scope.scheduleListQueryStrings.offset) + parseInt($scope.scheduleListQueryStrings.limit) > data.ScheduleCollection.total) {
                    limit = parseInt(data.ScheduleCollection.total) - $scope.scheduleListQueryStrings.offset;
                }

                for (i = 0; i < limit; i++) {
                    console.log(startIdx + i);
                    allScheduleIdList[startIdx + i] = data.ScheduleCollection.rows[i].id;
                }

                console.log(allScheduleIdList[index]);
                getAcGroupInfo(allScheduleIdList[index]);

                window.sessionStorage.setItem('getAllScheduleIdList', angular.toJson(allScheduleIdList));
            });
        }

        function getAcGroupInfo(id) {
            var _acGroup;
            API.get('schedules/' + id).then(function success(data) {
                $scope.schedules = new Schedules(data.Schedule);
                $scope.currScheduleName = $scope.schedules.name;
                window.sessionStorage.setItem('scheduleInfo', angular.toJson(data.Schedule));
                page_mode = Const.PAGE_MODE.UPDATE
            });
        };
        // Page 이전/다음 버튼 영역 -end-

        $scope.DEFAULT_SCHEDULE = Const.DEFAULT_SCHEDULE;
        $scope.HolidayGroupList = undefined;
        $scope.FULL_DAYS_TEXT = Const.FULL_DAYS_TEXT;

        // Holiday 그룹 정보 조회
        API.get('holiday_groups').then(function success(data) {
            $scope.HolidayGroupList = data.HolidayGroupCollection.rows;
        });

        $scope.$watch('schedules.days_of_iteration', function (newValue, oldValue) {
            if (newValue != oldValue) {
                if (newValue > 90 || newValue < 0) {
                    $scope.schedules.days_of_iteration = oldValue;
                }
            }
            if (newValue == 0 && oldValue == 0) {
                $scope.schedules.days_of_iteration = 1;
            } else if (newValue == '' && oldValue == 0) {

            } else if (newValue == 0 && oldValue == '') {
                $scope.schedules.days_of_iteration = 1;
            }
        });


        $scope.$watch('schedules.holiday_schedules', function (nv, ov) {
            var i, changedIdx;
            if (angular.isDefined(nv) && angular.isDefined(ov) && nv.length == ov.length) {
                for (i = nv.length; i--;) {
                    if (angular.equals(nv[i].holiday_group_id, ov[i].holiday_group_id) == false) {
                        changedIdx = i;
                        break;
                    }
                }

                if (angular.isDefined(changedIdx) && Object.keys(nv[i].holiday_group_id).length == 0) {
                    $scope.schedules.holiday_schedules[changedIdx].time_segments = angular.copy([]);
                }
            }
        }, true);

        $scope.doAddSchedule();
    }])
    ;
});