'use strict';

define([
    'modules/common/datagrid/columnSettingController',
    'modules/common/datagrid/columnSettingService',
    'modules/common/print/printOptionController',
    'modules/common/print/printOptionService',
    'modules/setting/triggerActionInfoController'
    ],
    function () {

    var app = angular.module('biostarWebApp');
    app.register.controller('TriggerActionsController',['$scope', '$rootScope', '$compile', '$document', '$window', '$q', '$http', '$location', 'Const', 'MessageProperties', 'Util', 'ColumnSettingService', 'API',  'PrintOptionService', 'MessageService', function ($scope, $rootScope, $compile, $document, $window, $q, $http, $location, Const, MessageProperties, Util, ColumnSettingService, API,  PrintOptionService, MessageService) {

        $scope.gridId = "triggerActionGrid";

        var TriggerAction = (function (triggerAction) {
            var name = '',
                priority = 0,
                schedule_id,
                trigger = {
                    source_type: [],
                    event_types: [],
                    devices: undefined,
                    doors: undefined
                },
                action = {
                    id: 0,
                    device_action_id: {
                        device_id: undefined,
                        relay_signal_id: undefined
                    },
                    email_action_id: {
                        receipients: []
                    }
                };
            if (triggerAction) {
                name = triggerAction.name;
                priority = triggerAction.priority;
                schedule_id = triggerAction.schedule_id;
                trigger = triggerAction.trigger;
                action = triggerAction.action;
            }
            return {
                name: name,
                priority: priority,
                schedule_id: schedule_id,
                trigger: trigger,
                action: action
            };
        });


        $scope.clearCheckItemArray = function() {
            $scope.checkItemArray = [];
        };

        var triggerActionInfo = undefined,
            httpTimeOut = undefined,
            canceller = $q.defer(),
            triggerActionListDataGridColumns = [
                [
                    {field: Const.FIELD_NAME.CHECKBOX, checkbox: true},
                    {field: 'name', title: MessageProperties.get('common.name'), width: 120, align: 'left'},
                    {field: 'trigger.source_type', title: MessageProperties.get('setting.triggerAction.source'), width: 100, formatter: function (value, row, index) {
                        var typeNum = parseInt(row.trigger.source_type),
                            TRIGGER_TYPE_SELECTOR = ['biostars', 'devices', 'doors'];
                        return Const.TRIGGER_ACTION_TYPE_LABEL[Const.TRIGGER_ACTION_TYPE[typeNum]] + ' + <mark class="number">' + row.trigger[TRIGGER_TYPE_SELECTOR[typeNum]].length + '<mark><span></span></mark></mark>';
                    }},
                    {field: 'trigger.event_types', title: MessageProperties.get('setting.triggerAction.alarmEvent'), width: 80,
                        formatter: function (value, row, index) {
                            var i = 0,
                                rValue = '',
                                eventArr = row.trigger.event_types;
                            for (i = eventArr.length; i--;) {
                                rValue += eventArr[i].name + '<br/>';
                            }

                            return rValue;
                        }},
                    {field: 'schedule_id', title: MessageProperties.get('common.schedule'), width: 80,
                        formatter: function (value, row, index) {
                            return value.name;
                        }}
//                    {field: 'priority', title: 'PRIORITY', width: 40, align: 'center'}
                ]
            ],
            triggerActionListQueryStrings = {
                query: undefined,
                offset: 0,
                limit: 15,
                total: undefined,
                timeout: undefined
            },
            triggerActionListDataGridOptions = {
                scrollbarSize: 0,
                view: $.extend({}, $.fn.datagrid.defaults.view, {
                    onBeforeRender: function (target, rows) {
                        $("#triggerActionGrid").datagrid('resize');
                    },
                    onAfterRender: function (target) {
                        var dataGridBodyElement = angular.element("#" + $scope.gridId).parent(".datagrid-view").find(".datagrid-view2").find(".datagrid-body"),
                        grdPanel = angular.element(target).datagrid('getPanel');
                        if (angular.element("#" + $scope.gridId).datagrid("getRows").length === 0) {
                            var html = '<div class="listNotFound">' +
                                $scope.gridInformation + '</div>';
                            dataGridBodyElement.html(html);
                        }

                        //헤더에 전체선택 버튼이 없다면 추가
                        if (grdPanel.find('.datagrid-header-check.compiled').length <= 0) {
                            grdPanel.find('.datagrid-header-check').attr({
                                'select-all-yn': 'SelectAllYN',
                                'datagrid-id': '"' + target.id + '"',
                                'clear-function': 'clearCheckItemArray'
                            });
                            $compile(grdPanel.find('.datagrid-header-row'))($scope);
                            grdPanel.find('.datagrid-header-check').addClass('compiled');
                        }
                    }
                }),
                onCheck: function (rowIndex, rowData) {
                    CheckUnCheckOnGrid(rowData);

                },
                onUncheck: function (rowIndex, rowData) {
                    CheckUnCheckOnGrid(rowData);
                },
                onCheckAll: function (rows) {
                    CheckUnCheckOnGrid(rows, true);
                },
                onUncheckAll: function (rows) {
                    if($scope.SelectAllYN == true) {
                        $scope.SelectAllYN = false;
                        $scope.checkItemArray = [];
                    }
                    CheckUnCheckOnGrid(rows, false);

                },
                fitColumns: true,
                resize: true,
                singleSelect: true,
                selectOnCheck: false,
                checkOnSelect: false,
                columns: triggerActionListDataGridColumns,
                onClickCell: function (rowIndex, field) {
                    $rootScope.clickCellOnGrid($scope.gridId, rowIndex, field, "id", function (rowData) {
                        API.get('trigger_actions' + '/' + rowData.id).
                            then(function (data) {
                                window.sessionStorage.setItem('triggerActionInfo', angular.toJson(data.TriggerAction));
                                setTriggerActionInfoUrl();
                            });
                    });
                }
            };

        function CheckUnCheckOnGrid(rowData, CheckAllYN) {
            var i, idx, item;
            if (rowData.constructor == Object) {
                idx = $scope.checkItemArray.indexOf(rowData.id);
                if (idx > -1) {
                    $scope.checkItemArray.splice(idx, 1);
                } else {
                    $scope.checkItemArray.push(rowData.id);
                }

            } else {
//                $scope.checkItemArray = [];
                if (CheckAllYN) {
                    for (i = rowData.length; i--; ) {
                        item = rowData[i];
                        idx = $scope.checkItemArray.indexOf(item.id);
                        if (idx == -1) {
                            $scope.checkItemArray.push(item.id);
                        }
                    }
                } else {
                    for (i = rowData.length; i--; ) {
                        item = rowData[i];
                        idx = $scope.checkItemArray.indexOf(item.id);
                        if( idx > -1) {
                            $scope.checkItemArray.splice(idx, 1);
                        }

                    }
                }
            }
            console.log($scope.checkItemArray);
            if(!$scope.$root.$$phase) $scope.$apply();
        }

        $scope.doAddTriggerAction = function () {
            window.sessionStorage.setItem('triggerActionInfo', '{}')
            setTriggerActionInfoUrl();
        };


        //헤더에 전체선택 버튼 추가
        $scope.SelectAllYN = false;
        $scope.$watch('SelectAllYN', function (nv) {
            if (nv) {
                angular.element('#triggerActionGrid').datagrid('checkAll');
            } else {
                angular.element('#triggerActionGrid').datagrid('uncheckAll');
                $scope.checkItemArray = [];
            }
        });

        function safeApply(fn) {
            $scope.$root.$$phase ? fn() : $scope.$apply(fn);
        }

        function setTriggerActionInfoUrl() {
            $scope.selectedTriggerActionInfo = 'modules/setting/triggerActionInfo.html';
//            $scope.$apply();
        }

        function unSetTriggerActionInfoUrl() {
            $scope.selectedTriggerActionInfo = undefined;
//            $scope.$apply();
        }


        $scope.deleteTheSelectedTriggerAction = function () {
            MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                if (confirmYn) {
                    var i,
                        param = 'id=';
                    if($scope.SelectAllYN) {
                        param += '*';
                    } else {
                        param += $scope.checkItemArray.join('+');
                    }
                    API.delete('trigger_actions?' + param).then(function (data) {
                        $scope.checkItemArray = [];
                        beSearchTriggerActionList();
                    });
                }
            });
        };



        $scope.checkItemArray = [];
        $scope.triggerActionListQueryStrings = triggerActionListQueryStrings;
        $scope.TriggerActionListData = [];

        angular.element('#triggerActionGrid').datagrid(triggerActionListDataGridOptions);





        function beSearchTriggerActionList(paramQueryString, deferFunctionName) {
            searchLNBList();

            API.get('trigger_actions', deferFunctionName, $scope.triggerActionListQueryStrings).then(function (data) {
                var i,
                    rowData = data.TriggerActionCollection.rows;
                window.sessionStorage.setItem('TriggerActionListData', angular.toJson(rowData));
                if (rowData != undefined && rowData.length > 0) {
                    $scope.gridInformation = '';
                    angular.element('#triggerActionGrid').datagrid('loadData', rowData);
//                    angular.element('#triggerActionGrid').datagrid('uncheckAll');
                } else {
                    $scope.gridInformation = MessageProperties.get('validate.notFound');
                    angular.element("#" + $scope.gridId).datagrid('loadData', []);
                }
                $scope.triggerActionListQueryStrings.total = data.TriggerActionCollection.total || 0;
                if($scope.SelectAllYN) {
                    for(i = $scope.TriggerActionListData.length; i--; ) {
                        rowData = $scope.TriggerActionListData[i];
                        if($scope.checkItemArray.indexOf(rowData.id) < 0) {
                            $scope.checkItemArray.push(rowData.id);
                        }
                    }
                }
                $scope.TriggerActionListData = rowData;

                if(!$scope.SelectAllYN) {
                    angular.element('.datagrid-header-check').find('input').attr('checked', false);
//                    angular.element('#triggerActionGrid').datagrid('uncheckAll');
                } else {
//                    angular.element('#triggerActionGrid').datagrid('checkAll');
                }

                for(i = $scope.TriggerActionListData.length; i--; ) {
                    rowData = $scope.TriggerActionListData[i];
                    if($scope.checkItemArray.indexOf(rowData.id) > -1) {
                        angular.element('#triggerActionGrid').datagrid('checkRow', i);
                        $scope.checkItemArray.push(rowData.id);
                    }
                }
            });
        }

        $scope.gridInformation = MessageProperties.get('validate.notFound');



        $scope.SearchCallback = {
            fn: beSearchTriggerActionList
        };

        $scope.$on('ReloadTriggerActionsDataGrid', function (event, data) {

            angular.element('#triggerActionGrid').datagrid('uncheckAll');

            if (data && data.id) {
                $scope.triggerActionListQueryStrings.group_id = data.id;
            } else {
                $scope.triggerActionListQueryStrings.group_id = 1;
            }

            $scope.triggerActionListQueryStrings.offset = 0;

            if (data && data.offset) {
                $scope.triggerActionListQueryStrings.offset = data.offset;
            }
            beSearchTriggerActionList();
            $scope.checkItemArray = [];
        });

//      $scope.$on('TriggerActionGroupDataReceiveSuccess', function(event, data) {
//          triggerActionListQueryStrings.offset = 0;
//          if(!TriggerActionListData)
//          beSearchTriggerActionList();
//      });


        function searchLNBList() {
            var lnbQueryOption = angular.copy($scope.triggerActionListQueryStrings);
            lnbQueryOption.limit = Const.INTEGER_MAX;
            API.get('trigger_actions', 'lnbQueryOption', lnbQueryOption).then(function (data) {
                $scope.lnbListData = data.TriggerActionCollection.rows;
            });
        }
        $scope.onClickLnbList = function (listItem) {
            API.get('trigger_actions' + '/' + listItem.id).
                then(function (data) {
                    window.sessionStorage.setItem('triggerActionInfo', angular.toJson(data.TriggerAction));
                    setTriggerActionInfoUrl();
                });
        };

        beSearchTriggerActionList();

        $scope.$watch(function () {
            return  angular.element('.panel.datagrid').width();
        }, function (nv, ov) {
            if (nv !== ov) {
                angular.element('#triggerActionGrid').datagrid('resize');
            }
        }, true);

        angular.element($window).on('resize', function ($event) {
            var event = $event | window.event;
            angular.element('#triggerActionGrid').datagrid('resize');
            Util.debug('div#triggerActionGrid resize');
        });
        $scope.backSetting = function () {
            $location.path('/setting');
        };






        $scope.dropDownMenu = [
//            {
//                menu: 'Import to CSV',
//                func: function () {
//                    console.info(this.menu);
//                }
//            },
//            {
//                menu: 'Export to CSV',
//                func: function () {
//                    console.info(this.menu);
//                }
//            },
//            {
//                menu: 'Print Preview',
//                func: function () {
//                    Util.beforeUrl($location.url());
//                    $location.path('/triggerAction/triggerActions-report');
//                }
//            },
//            {
//                menu: MessageProperties.get('menu.print'),
//                func: function () {
//                    $scope.openPrintOptionPopupDlg();
//                }
//            },
            {
                menu: MessageProperties.get('menu.columnSetting'),
                func: function () {
                    $scope.openColumnSettingPopupDlg();
                }
            }
        ];








        var clickMenuInitUnbind = $rootScope.$on('ClickMenuInit', function (event, data) {
            window.sessionStorage.removeItem('triggerActionInfo');
            window.sessionStorage.removeItem('TriggerActionListData');
            window.sessionStorage.removeItem('triggerActionCurrPos');
            unSetTriggerActionInfoUrl();
        });

        $scope.$on('$locationChangeStart', clickMenuInitUnbind);

        $scope.$on('CloseTriggerActionInfo', function (event, data) {
            window.sessionStorage.removeItem('triggerActionInfo');
            window.sessionStorage.removeItem('TriggerActionListData');
            window.sessionStorage.removeItem('triggerActionCurrPos');
            unSetTriggerActionInfoUrl();
//            angular.element('#triggerActionGrid').datagrid('unselectAll');
        });

        $scope.$on('OpenTriggerActionInfo', function (event, data) {
            window.sessionStorage.setItem('triggerActionInfo', '{}');
            setTriggerActionInfoUrl();
//            angular.element('#triggerActionGrid').datagrid('unselectAll');
        });

        triggerActionInfo = window.sessionStorage.getItem('triggerActionInfo');
        if (triggerActionInfo && triggerActionInfo != 'undefined') {
            triggerActionInfo = angular.fromJson(triggerActionInfo);
            setTriggerActionInfoUrl();
        }

        $scope.$watch('triggerActionListQueryStrings', function (nv, ov) {
            if (nv !== ov) {
                // TODO BSS-195 BSS-192 print :: server api not implemented
//                PrintOptionService.buildGridQueryFunction(Const.REPORT_COLLECTION_NAME.TRIIGER_ACTION, nv);
            }
        }, true);

        PrintOptionService.getReportColumnNameFunction = function (gridColumnName) {
            var reportColumnName = gridColumnName;
            if (gridColumnName == "user_group_id"){
                return "user_groups.name"
            } else  if (gridColumnName == "access_groups"){
                return "access_groups.name"
            } else  if (gridColumnName == "fingerprint_templates"){
                return "finger-tempates.count"
            } else  if (gridColumnName == "cards"){
                return "cards.count"
            }
            return reportColumnName;
        };
        $scope.reportUrl = Const.API.BASE_URL + 'trigger_actions/report';
        $scope.openPrintOptionPopupDlg = PrintOptionService.openPrintOptionPopupDlg;
        $scope.openColumnSettingPopupDlg = ColumnSettingService.openColumnSettingPopupDlg;

        ColumnSettingService.setGridId($scope.gridId);
        ColumnSettingService.setCallbackFunction($scope.SearchCallback.fn);
        var customColumnSetting = ColumnSettingService.getColumnSettingFromLocalStroage();
        if (customColumnSetting) {
            ColumnSettingService.applyChangedColumnsToGrid(customColumnSetting, false);
        }




//        $scope.openColumnSettingPopupDlg = ColumnSettingService.openColumnSettingPopupDlg;
    }]);

});
