'use strict';
define(function () {
    var app = angular.module('biostarWebApp');

    app.register.controller('WiegandInfoController', ['$scope', '$rootScope', '$location', '$compile', 'API', 'Const', 'Util', 'MessageService', 'MessageProperties', function ($scope, $rootScope, $location, $compile, API, Const, Util, MessageService, MessageProperties) {
        function initPage() {
            $scope.cardFormatInfo = Util.getSessionObject(Const.SESSION_STORAGE_KEY.CARD_FORMAT_INFO);
            if ($scope.cardFormatInfo == undefined) {
                $scope.cardFormatInfo = {};
            }

            $scope.currCardFormat = $scope.cardFormatInfo.name;
            $scope.readOnly = false;
            if ($scope.cardFormatInfo.id != undefined) {
                if ($scope.cardFormatInfo.id == Const.CARD_FORMAT_ID.H10301 ||
                    $scope.cardFormatInfo.id == Const.CARD_FORMAT_ID.H10302 ||
                    $scope.cardFormatInfo.id == Const.CARD_FORMAT_ID.H10304 ||
                    $scope.cardFormatInfo.id == Const.CARD_FORMAT_ID.HCORP1000) {
                    $scope.readOnly = true;
                }
            }
            $scope.cardFormatInfo.totalBits = $scope.cardFormatInfo.length;
            $scope.cardFormatInfo.idFieldList = [];
            $scope.cardFormatInfo.parityFieldList = [];

            $scope.cardFormatInfo.parityTypeOptions = {};
            for (var key in Const.WIEGAND_PARITY_TYPE) {
                if (Const.WIEGAND_PARITY_TYPE.hasOwnProperty(key)) {
                    $scope.cardFormatInfo.parityTypeOptions[key] = Const.WIEGAND_PARITY_TYPE[key];
                }
            }
        }

        $scope.addNewWiegandLabel = MessageProperties.get('setting.cardFormat.addNewWiegand');

        initPage();

        if ($scope.cardFormatInfo.id_fields != undefined) {
            for (var i = 0; i < $scope.cardFormatInfo.id_fields.length; i++) {
                var tempString = atob($scope.cardFormatInfo.id_fields[i]);
                var byteArrayStringObject = buildByteArrayStringObject(i, tempString);
                Util.debug(byteArrayStringObject);
                var idField = createIdObject(i, false, byteArrayStringObject.startIndex, byteArrayStringObject.endIndex);
                Util.debug(idField);
                $scope.cardFormatInfo.idFieldList.push(idField);
            }
        }

        function createIdObject(index, isNew, startIndex, endIndex) {
            var idField = {
                id: 'ID' + index,
                isNew: isNew,
                size: 0
            };
            if (startIndex > -1) {
                idField.startBit = startIndex - (Const.WIEGAND_FORMAT_MAX_SIZE - $scope.cardFormatInfo.totalBits );
            }
            if (endIndex > -1) {
                idField.endBit = endIndex - (Const.WIEGAND_FORMAT_MAX_SIZE - $scope.cardFormatInfo.totalBits );
            }
            if (startIndex > -1 && endIndex > -1) {
                idField.size = idField.endBit - idField.startBit + 1;
            }
            return idField;
        }

        function createParityObject(index, isNew, startIndex, endIndex) {
            var parityField = {
                id: 'ID' + $scope.cardFormatInfo.parityFieldList.length,
                isNew: isNew,
                size: 0
            };
//            console.log(index);
            if (isNew == true) {
//                parityField.type = Const.WIEGAND_PARITY_TYPE.NONE;
                parityField.type = '1';
                parityField.typeName = Const.WIEGAND_PARITY_TYPE[parityField.type];
            } else {
                if ($scope.cardFormatInfo.parity_types != undefined) {
                    parityField.type = $scope.cardFormatInfo.parity_types[index];
                    parityField.typeName = Const.WIEGAND_PARITY_TYPE[ $scope.cardFormatInfo.parity_types[index]];
                }
                if ($scope.cardFormatInfo.parity_positions != undefined && $scope.cardFormatInfo.parity_positions.length > index) {
                    parityField.position = $scope.cardFormatInfo.parity_positions[index];
                }
                if (startIndex > -1) {
                    parityField.startBit = startIndex - (Const.WIEGAND_FORMAT_MAX_SIZE - $scope.cardFormatInfo.totalBits );
                }
                if (endIndex > -1) {
                    parityField.endBit = endIndex - (Const.WIEGAND_FORMAT_MAX_SIZE - $scope.cardFormatInfo.totalBits );
                }
                if (startIndex > -1 && endIndex > -1) {
                    parityField.size = parityField.endBit - parityField.startBit + 1;
                }
            }
            Util.debug(parityField);
            return parityField;
        }

        function buildByteArrayStringObject(mainIndex, inputString, createParityBlock) {
            var returnObject = {
                startIndex: -1,
                endIndex: -1,
                bitArrayString: ''
            };
            if (createParityBlock == undefined) {
                createParityBlock = false;
            }
            var currentIndex = 0;
            for (var i = 0; i < inputString.length; i++) {
                // convert to binary
                var tempArray = inputString.charCodeAt(i).toString(2);
                // 1 byte == 8 bits
                var tempPaddedString = Util.leftPad(tempArray, 8);
                for (var tempIndex = 0; tempIndex < 8; tempIndex++) {
                    returnObject.bitArrayString += tempPaddedString[tempIndex];
                    if (returnObject.startIndex < 0 || returnObject.endIndex < 0) {
                        if (tempPaddedString[tempIndex] == "1" && returnObject.startIndex < 0) {
                            returnObject.startIndex = currentIndex;
                        } else if (tempPaddedString[tempIndex] == "0" && returnObject.startIndex > -1) {
                            returnObject.endIndex = currentIndex - 1;
                            if (createParityBlock == true) {
                                var parityField = createParityObject(mainIndex, false, returnObject.startIndex, returnObject.endIndex);
                                $scope.cardFormatInfo.parityFieldList.push(parityField);
                                returnObject.startIndex = -1;
                                returnObject.endIndex = -1;
                            }
                        }
                    }
                    currentIndex++;
                }
            }
            if (returnObject.startIndex > -1) {
                if (returnObject.endIndex == -1) {
                    returnObject.endIndex = Const.WIEGAND_FORMAT_MAX_SIZE - 1;
                }
                if (createParityBlock == true) {
                    var parityField = createParityObject(mainIndex, false, returnObject.startIndex, returnObject.endIndex);
                    $scope.cardFormatInfo.parityFieldList.push(parityField);
                    returnObject.startIndex = -1;
                    returnObject.endIndex = -1;
                }
            }
            return returnObject;
        }

        if ($scope.cardFormatInfo.parity_fields != undefined) {
            for (var i = 0; i < $scope.cardFormatInfo.parity_fields.length; i++) {
                var tempString = atob($scope.cardFormatInfo.parity_fields[i]);
                var byteArrayStringObject = buildByteArrayStringObject(i, tempString, true);
//                console.log(byteArrayStringObject.bitArrayString);
            }
        }

        $scope.doAddId = function () {
            var newIndex = $scope.cardFormatInfo.idFieldList.length;
            var idField = createIdObject(newIndex, true, -1, -1);
            $scope.cardFormatInfo.idFieldList.push(idField);
            $scope.$watch('cardFormatInfo.idFieldList[' + newIndex + ']', function (nv, ov) {
                $scope.changeIdPosition(newIndex);
            }, true);
        };

        $scope.doDeleteId = function (deleteIndex) {
            MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                if (confirmYn) {
                    $scope.cardFormatInfo.idFieldList.splice(deleteIndex, 1);
                }
            });
        };

        $scope.doAddParity = function () {
            var newIndex = $scope.cardFormatInfo.parityFieldList.length;
            var parityField = createParityObject(newIndex, true, -1, -1);
            $scope.cardFormatInfo.parityFieldList.push(parityField);
            $scope.$watch('cardFormatInfo.parityFieldList[' + newIndex + ']', function (nv, ov) {
                $scope.changeParityPosition(newIndex);
            }, true);
        };

        $scope.doDeleteParity = function (deleteIndex) {
            MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                if (confirmYn) {
                    $scope.cardFormatInfo.parityFieldList.splice(deleteIndex, 1);
                }
            });
        };

        $scope.doApply = function () {

            var wiegandFormat = {
                name: $scope.cardFormatInfo.name,
                length: $scope.cardFormatInfo.totalBits,
                id_fields: [],
                parity_fields: [],
                parity_types: [],
                parity_positions: []
            };

            if (!$scope.cardFormatInfo.name) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.mustBeEntered', 'common.name'), function () {
//                    angular.element('#cardFormatName').focus();
                });
                return;
            }
            if (!$scope.cardFormatInfo.totalBits) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.mustBeEntered', 'setting.cardFormat.totalBits'), function () {
//                    angular.element('#cardFormatTotalBits').focus();
                });
                return;
            }
            if ($scope.cardFormatInfo.idFieldList.length === 0) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.mustBeEntered', 'setting.cardFormat.idField'), function () {
                    $scope.doAddId();
                });
                return;
            }
//            if ($scope.cardFormatInfo.parityFieldList.length === 0) {
//                MessageService.displayWarningMessage(MessageProperties.get('validate.mustBeEntered', 'setting.cardFormat.parityBits'), function () {
//                    $scope.doAddParity();
//                });
//                return;
//            }

            for (var i = 0; i < $scope.cardFormatInfo.idFieldList.length; i++) {
                if ($scope.cardFormatInfo.idFieldList[i].startBit == undefined || $scope.cardFormatInfo.idFieldList[i].startBit.length < 1) {
                    MessageService.displayWarningMessage(MessageProperties.get("validate.mustBeEntered", "setting.cardFormat.startBit"));
                    return;
                }
                if ($scope.cardFormatInfo.idFieldList[i].endBit == undefined || $scope.cardFormatInfo.idFieldList[i].endBit.length < 1) {
                    MessageService.displayWarningMessage(MessageProperties.get("validate.mustBeEntered", "setting.cardFormat.endBit"));
                    return;
                }
                if ($scope.cardFormatInfo.idFieldList[i].endBit != undefined && $scope.cardFormatInfo.idFieldList[i].endBit > $scope.cardFormatInfo.totalBits - 1) {
                    MessageService.displayWarningMessage(MessageProperties.get("validate.mustBeLessThan", "setting.cardFormat.endBit", $scope.cardFormatInfo.totalBits));
                    return;
                }
                var stringValue = Util.buildBase64EncodedWiegandFormat($scope.cardFormatInfo.idFieldList[i].startBit, $scope.cardFormatInfo.idFieldList[i].endBit, $scope.cardFormatInfo.totalBits);
                wiegandFormat.id_fields.push(btoa(stringValue));
            }
            var parityFieldMap = {};
            for (var i = 0; i < $scope.cardFormatInfo.parityFieldList.length; i++) {
                if ($scope.cardFormatInfo.parityFieldList[i].position == undefined || $scope.cardFormatInfo.parityFieldList[i].position.length < 1) {
                    MessageService.displayWarningMessage(MessageProperties.get("validate.mustBeEntered", "setting.cardFormat.position"));
                    return;
                }
                if ($scope.cardFormatInfo.parityFieldList[i].startBit == undefined || $scope.cardFormatInfo.parityFieldList[i].startBit.length < 1) {
                    MessageService.displayWarningMessage(MessageProperties.get("validate.mustBeEntered", "setting.cardFormat.startBit"));
                    return;
                }
                if ($scope.cardFormatInfo.parityFieldList[i].endBit == undefined || $scope.cardFormatInfo.parityFieldList[i].endBit.length < 1) {
                    MessageService.displayWarningMessage(MessageProperties.get("validate.mustBeEntered", "setting.cardFormat.endBit"));
                    return;
                }
                if ($scope.cardFormatInfo.parityFieldList[i].endBit != undefined && $scope.cardFormatInfo.parityFieldList[i].endBit > $scope.cardFormatInfo.totalBits - 1) {
                    MessageService.displayWarningMessage(MessageProperties.get("validate.mustBeLessThan", "setting.cardFormat.endBit", $scope.cardFormatInfo.totalBits));
                    return;
                }
//                if ($scope.cardFormatInfo.parityFieldList[i].type == undefined || $scope.cardFormatInfo.parityFieldList[i].type == Const.WIEGAND_PARITY_TYPE.NONE) {
                if ($scope.cardFormatInfo.parityFieldList[i].type == undefined) {
                    MessageService.displayWarningMessage(MessageProperties.get("validate.mustBeEntered", "setting.cardFormat.type"));
                    return;
                }
                var stringValue = Util.buildWiegandFormatString($scope.cardFormatInfo.parityFieldList[i].startBit, $scope.cardFormatInfo.parityFieldList[i].endBit, $scope.cardFormatInfo.totalBits);
//                wiegandFormat.parity_fields.push(btoa(stringValue));
//                console.log($scope.cardFormatInfo.parityFieldList[i]);
//                tempParityFieldList.parity_fields.push(stringValue);
//                console.log(stringValue);
//                wiegandFormat.parity_types.push($scope.cardFormatInfo.parityFieldList[i].type);
//                wiegandFormat.parity_positions.push($scope.cardFormatInfo.parityFieldList[i].position);
                var type = $scope.cardFormatInfo.parityFieldList[i].type;
                var position = $scope.cardFormatInfo.parityFieldList[i].position;
                var existingParity = parityFieldMap[position + "_" + type];
                if (existingParity == undefined || existingParity.parityBitList == undefined) {
                    parityFieldMap[position + "_" + type] = {
                        position: position,
                        type: type,
                        parityBitList: stringValue
                    };
                } else {
//                    console.log(existingParity.parityBitList, stringValue);
                    var newParityBitList = '';
                    for (var index = 0; index < stringValue.length; index++) {
                        var existingBit = existingParity.parityBitList.charAt(index);
                        var newBit = stringValue.charAt(index);
                        var replaceBit = '0';
                        if (existingBit == '0' && newBit == '0') {
                            replaceBit = '0';
                        } else if (existingBit == '1' || newBit == '1') {
                            replaceBit = '1';
                        }
//                        if (index == 0) {
//                            existingParity.parityBitList = replaceBit + existingParity.parityBitList.substring(1);
//                        } else if (index == stringValue.length - 1) {
//                            existingParity.parityBitList = existingParity.parityBitList.substring(0, index - 1) + replaceBit;
//                        } else {
//                            existingParity.parityBitList = existingParity.parityBitList.substring(0, index) + replaceBit + existingParity.parityBitList.substring(index + 1);
//                        }
                        newParityBitList += replaceBit;
                    }
//                    console.log(existingParity.parityBitList , newParityBitList);
                    existingParity.parityBitList = newParityBitList;
//                    console.log(existingParity.parityBitList.length, existingParity.parityBitList);
                }
            }

            for (var key in parityFieldMap) {
                if (parityFieldMap.hasOwnProperty(key)) {
                    var data = parityFieldMap[key];
                    wiegandFormat.parity_fields.push(btoa(Util.convert256BinaryArrayStringToString(data.parityBitList)));
                    wiegandFormat.parity_types.push(data.type);
                    wiegandFormat.parity_positions.push(data.position);
                }
            }
            if (wiegandFormat.parity_types.length > 4) {
                MessageService.displayWarningMessage(MessageProperties.get("validate.onlyFourAllowed", "setting.cardFormat.parityBits"));
                return;
            }

            wiegandFormat.description = $scope.cardFormatInfo.description;
            var params = { WiegandFormat: wiegandFormat};
            if ($scope.cardFormatInfo.id != undefined) {
                wiegandFormat.id = $scope.cardFormatInfo.id;
                API.put('/setting/wiegand_formats/' + wiegandFormat.id, undefined, params).then(function (data) {
                    $scope.doCancel();
                });
            } else {
                API.post('/setting/wiegand_formats', undefined, params).then(function (data) {
                    $scope.doCancel();
                });
            }
        };
        $scope.doCancel = function () {
            Util.removeSessionObject(Const.SESSION_STORAGE_KEY.CARD_FORMAT_INFO);
            $scope.$emit('CloseDetailView');
        };

        $rootScope.$on('$routeChangeStart', function (evt, next, curr) {
            Util.removeSessionObject(Const.SESSION_STORAGE_KEY.CARD_FORMAT_INFO);
        });

        $scope.$on('NewDetailViewClicked', function (event, data) {
            initPage();
        });

        $scope.changeIdPosition = function (index) {
            if ($scope.cardFormatInfo.idFieldList[index].startBit == undefined || $scope.cardFormatInfo.idFieldList[index].endBit == undefined) {
                $scope.cardFormatInfo.idFieldList[index].size = 0;
            } else {
                $scope.cardFormatInfo.idFieldList[index].size = $scope.cardFormatInfo.idFieldList[index].endBit - $scope.cardFormatInfo.idFieldList[index].startBit + 1;
            }
        };
        $scope.changeParityPosition = function (index) {
            if ($scope.cardFormatInfo.parityFieldList[index].startBit == undefined || $scope.cardFormatInfo.parityFieldList[index].endBit == undefined) {
                $scope.cardFormatInfo.parityFieldList[index].size = 0;
            } else {
                $scope.cardFormatInfo.parityFieldList[index].size = $scope.cardFormatInfo.parityFieldList[index].endBit - $scope.cardFormatInfo.parityFieldList[index].startBit + 1;
            }
        };
        function initView() {
            if ($scope.cardFormatInfo.id === undefined) {
                $scope.doAddId();
//                $scope.doAddParity();
            }
        }

        initView();
    }]);
})
;