'use strict';

define([
    'modules/common/services/card-agent',
    'modules/common/services/device-agent',
    'modules/common/widgetModule'
], function () {
    var app = angular.module('biostarWebApp');
    app.register.controller('UserCardRegistDlgCtrl', ['$scope', '$timeout', 'API', 'Const', 'Util', 'DeviceAgent', 'CardsAgent', 'MessageProperties', 'MessageService', function ($scope, $timeout, API, Const, Util, DeviceAgent, CardsAgent, MessageProperties, MessageService) {


        var ASSIGNED_CARDS = 1,
            UNASSIGNED_CARDS = 0;

        function canNotTheReadCard() {
            $scope.cardRegistrationProcessMessage = MessageProperties.get('validate.canNotTheReadCard');
            $timeout(function () {
                $scope.cardRegistrationProcessMessage = undefined;
            }, 1000);
        }

        $scope.cardsQueryStrings = {
            query: undefined,
            offset: 0,
            limit: 15,
            total: undefined,
            group_id: UNASSIGNED_CARDS
        };

        $scope.SearchCallback = {
            fn: getUnassignedCardData
        };

        API.get('cards/types').then(function success(data) {
            $scope.cardTypeListData = data.CardTypeCollection.rows.slice(0, 2); // 미지원 카드 타입 삭제
//            $scope.cardTypeListData = data.CardTypeCollection.rows;
            $scope.card.card_type = $scope.cardTypeListData[0];
        });

        $scope.card = {};
        $scope.cardId = new Array(4);

        $scope.CARD_REGISTRATION_TYPE = {
            "0":MessageProperties.get('common.registerByCardReader'),
            "1":MessageProperties.get('common.assignCard'),
            "2":MessageProperties.get('common.enterManually')
        };

        $scope.READ_CARD = Const.CARD_REGISTRATION_TYPE_ID.READ_CARD;
        $scope.ASSIGN_CARD = Const.CARD_REGISTRATION_TYPE_ID.ASSIGN_CARD;
        $scope.CARD_ID_DIRECT_INPUT = Const.CARD_REGISTRATION_TYPE_ID.MANUAL_CARD;

        function createTreeNode(originalNode, newNode) {
//            newNode.status = originalNode.status;
            newNode.name = originalNode.name;
            newNode.text = originalNode.text;
            newNode.id = originalNode.id;
            newNode.device_type_id = originalNode.device_type_id;
            newNode.isDevice = originalNode.isDevice;
            newNode.iconCls = originalNode.iconCls;
            newNode.rs485 = originalNode.rs485;
//            newNode.status = originalNode.status;
            return newNode;
        }

        function remainingCardScanDevice(newTreeNode, originalTreeNode) {
            var item, newChildren = [];

            newTreeNode = createTreeNode(originalTreeNode, newTreeNode);
            if (originalTreeNode.children != undefined) {
                for (var i = 0; i < originalTreeNode.children.length; i++) {
                    var originalChildNode = originalTreeNode.children[i];
                    var newChildNode = {};

                    if (newTreeNode.children == undefined) {
                        newTreeNode.children = [];
                    }

                    if (angular.isUndefined(originalChildNode.device_type_id)) {
                        newTreeNode.children.push(newChildNode);
                    } else if (Const.CARD_DEVICES.indexOf(originalChildNode.device_type_id.id) > -1) {
                        if ((originalChildNode.rs485.mode == Const.RS485_MODE.SLAVE || originalChildNode.rs485.mode == Const.RS485_MODE.AUTO) &&
                            originalChildNode.parent_node_id != undefined) {
                            // do not display slave device
                        } else {
                            if (originalChildNode.rs485.mode != Const.RS485_MODE.SLAVE) {
                                // do not display slave device by rs485mode
                                newTreeNode.children.push(newChildNode);
                            }
                        }
                    }

                    remainingCardScanDevice(newChildNode, originalChildNode);
//                    newChildNode = createTreeNode(originalChildNode, newChildNode);
                }
            }

        }


        $scope.DeviceTreeData = [
            {}
        ];
        if (DeviceAgent.getDeviceGroupRecursiveData().then) {
            DeviceAgent.getDeviceGroupRecursiveData().then(function success(data) {
                remainingCardScanDevice($scope.DeviceTreeData[0], data);
            })
        } else {
            remainingCardScanDevice($scope.DeviceTreeData[0], DeviceAgent.getDeviceGroupRecursiveData());
        }

        $scope.doClickReadCard = function () {
            if (!$scope.selectedDeviceForRead || !$scope.selectedDeviceForRead.id) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseDeviceSelect'));
                return;
            }

            $scope.cardRegistrationProcessMessage = MessageProperties.get('common.placeCardNearCardReader');

            if ($scope.selectedDeviceForRead && $scope.selectedDeviceForRead.id) {
                API.post('devices/' + $scope.selectedDeviceForRead.id + '/scan_card', 'scanFingerprint',
                    undefined, undefined, undefined, undefined, false).then(function success(data) {

                        if (data.Card) {
                            $scope.cardId[0] = data.Card.card_id;
                            $scope.card.card_type = data.Card.card_type;

                            if ($scope.card.card_type.type == Const.CARD_TYPE_LIST.CSN_WIEGAND) {
                                $scope.card.wiegand_format_id = Const.DEFAULT_WIEGAND_FORMAT_ID;
                            }

                            console.log(DeviceAgent.getAllDevicesMapWithStatus()[$scope.selectedDeviceForRead.id]);

                            $scope.cardRegistrationProcessMessage = undefined;
                        } else {
                            canNotTheReadCard();
                        }

                    }, function error(data) {
                        canNotTheReadCard();
                    });
            } else {
                canNotTheReadCard();
            }


        };

        $scope.doClickUseUserId = function () {
            if ($scope.user.user_id) {
                if (parseInt($scope.user.user_id)) {
                    $scope.cardId[0] = $scope.user.user_id;
                } else {
                    $.messager.alert('Biostar Error', MessageProperties.get('canNotReadUserIdBecuaseNotNumber'));
                }

            } else {
                $scope.cardRegistrationProcessMessage = MessageProperties.get('validate.pleaseEnterUserId');
                $timeout(function () {
                    $scope.cardRegistrationProcessMessage = undefined;
                }, 1000);
            }
        };

        function getUnassignedCardData() {
            CardsAgent.getCards($scope.cardsQueryStrings).then(function success(data) {
                var i, item, popupEle;
                $scope.UnassignedCardData = data.CardCollection.rows || [];
                $scope.cardsQueryStrings.total = data.CardCollection.total || 0;
                $scope.unassignedCardDataCheckBox = new Array($scope.UnassignedCardData.length);
                popupEle = angular.element('#cardRegistrationDlg').parent();
                popupEle.css('top', $(window).height() / 2 - popupEle.outerHeight(true) / 2);
            });
        }

        $scope.selectCardForAssign = function (item) {
            if (!item.user_id) {
                $scope.card = item;
            } else {
                $scope.cardRegistrationProcessMessage = MessageProperties.get('validate.alreadyAssigned');
                $timeout(function () {
                    $scope.cardRegistrationProcessMessage = undefined;
                }, 1000);
            }

        };

        $scope.$watch('cardIdInputType', function (nv, ov) {
            if (nv != ov) {
                var cardType = angular.copy($scope.card.card_type);
                $scope.card = {};
                $scope.card.card_type = cardType;

                $scope.cardId = [];
                console.log(nv, ov);
                if ($scope.cardIdInputType == $scope.READ_CARD) {

                } else if ($scope.cardIdInputType == $scope.ASSIGN_CARD) {
                    getUnassignedCardData();
                }
            }
        });


        $scope.doClearSearchWord = function () {
            $scope.cardsQueryStrings.query = '';
            $scope.cardsQueryStrings.offset = 0;
            getUnassignedCardData();
        };

        $scope.doCardSearch = function () {
            $scope.cardsQueryStrings.offset = 0;
            getUnassignedCardData();
        };

//        $scope.$watch('card.card_type', function (nv, ov) {
//            if (nv != ov && nv) {
//                if ($scope.card.card_type.name.indexOf('CSN') > -1) {
//
//                }
//            }
//        });

        // Combo Box 초기값 설정
        $scope.cardIdInputType = $scope.READ_CARD;

        // 카드 타입별 CSN 갯수
        $scope.numberOfCardId = 'one';
//        $scope.$watch('card.card_type', function (nv, ov) {
//            if (nv !== ov) {
//                if (nv && nv.name.indexOf('EM') > -1) {
//                    $scope.numberOfCardId = 'two';
////                    angular.element('.card-sn-ctrl')
////                        .removeClass('one').addClass('two').removeClass('three').removeClass('four');
//                } else {
//                    $scope.numberOfCardId = 'one';
////                    angular.element('.card-sn-ctrl')
////                        .addClass('one').removeClass('two').removeClass('three').removeClass('four');
//                }
//            }
//        }, true);

        getUnassignedCardData();

        $scope.doApplyCardRegistration = function () {
            if ($scope.card.card_type && $scope.card.card_type.type == Const.CARD_TYPE_LIST.CSN_WIEGAND) {
                $scope.card.wiegand_format_id = {"id": Const.WIEGAND_DEFAULT_FORMAT_ID};
//            } else if($scope.card.card_type && $scope.card.card_type.type == Const.CARD_TYPE_LIST.CSN_WIEGAND) {
//                $scope.card.card_type.id =  Const.CARD_TYPE_LIST.CSN_WIEGAND;
//            } else if($scope.card.card_type && $scope.card.card_type.type == Const.CARD_TYPE_LIST.CSN) {
//                $scope.card.card_type = $scope.cardTypeListData[Const.CARD_TYPE_DEFAULT_INDEX];
            }

            if ($scope.cardIdInputType !== $scope.ASSIGN_CARD) {
                var i;
                $scope.card.card_id = [];
                for (i = 0; i < $scope.cardId.length; i++) {
                    if ($scope.cardId[i] && $scope.cardId[i].length > 0) {
                        $scope.card.card_id.push($scope.cardId[i]);
                    }
                }
                $scope.card.card_id = $scope.card.card_id.join('-');

                if ($scope.cardIdInputType != $scope.ASSIGN_CARD && !$scope.card.card_id) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseInputCardID'));
                    return false;
                }
//                $scope.card.user_id = {user_id: $scope.user.user_id};

                API.post('cards', undefined, {"CardCollection": {rows: [$scope.card]}}, undefined, undefined, undefined).then(function (data) {

                    var i, id;


                    console.log(data.CardCollection.rows);

                    for (i = data.CardCollection.rows.length; i--;) {
                        id = data.CardCollection.rows[i].id;
                    }

                    $scope.card.id = id;

                    Util.debug($scope.card);

                    if ($scope.user.cards) {
                        $scope.user.cards.push($scope.card);
                    } else {
                        $scope.user.cards = [];
                        $scope.user.cards.push($scope.card);
                    }

                    $scope.$emit('closeCardRegistrationDlg');
                }, function error(data) {
//                    if(data.Response.code != 150) {
//                        MessageService.displayWarningMessage(MessageProperties.get('validate.cardIdAlreadyExistsOrAssigned'));
//                        $scope.cardIdInputType = $scope.ASSIGN_CARD;
//                    } else {
//                        MessageService.displayErrorMessage(data, true);
//                    }

                });
            } else {

                if (!$scope.card.card_id) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseSelectCard'));
                    return false;
                }

                var idx = -1;

                if ($scope.user.cards && $scope.user.cards.length > 0) {
                    for (i = $scope.user.cards.length; i--;) {
                        if ($scope.card.card_id == $scope.user.cards[i].card_id) {
                            idx = i;
                        }
                    }
                }

                if (idx > -1) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.alreadyAssigned'));
                    return false;
                }

                if ($scope.user.cards) {
                    $scope.user.cards.push($scope.card);
                } else {
                    $scope.user.cards = [];
                    $scope.user.cards.push($scope.card);
                }

                $scope.$emit('closeCardRegistrationDlg');
            }
        };

        $scope.doCancelCardRegistration = function () {
            $scope.$emit('closeCardRegistrationDlg');
        };

        $scope.UnassignedCardsTableCheckboxAll = function ($event) {
            for (var i = $scope.unassignedCardDataCheckBox.length; i--;) {
                $scope.unassignedCardDataCheckBox[i] = $event.target.checked;
            }
        };

        $scope.$watch('unassignedCardDataCheckBox', function (nv, ov) {
            if (nv !== ov) {
                $scope.unassignedCardsTableCheckboxAll = true;
                for (var i = nv.length; i--;) {
                    if (nv[i] !== true) {
                        $scope.unassignedCardsTableCheckboxAll = false;
                        break;
                    }
                }
            }
        }, true);

        $scope.YouCanNumberOnly = MessageProperties.get('validate.onlyNumbersAreAllowed');


        $scope.doKeyDown = function ($event) {
            Util.debug(angular.element('input[name="card_id1"]').val());
        };
    }]);
});
