'use strict';
define(['modules/common/services/device-agent', 'modules/common/services/user-agent'], function () {
    var app = angular.module('biostarWebApp');
    app.register.controller('UserUserManualImportToDeviceCtrl',
        ['$scope', 'Const', 'UserAgent', 'DeviceAgent', 'MessageService', 'MessageProperties', 'Util',
            function ($scope, Const, UserAgent, DeviceAgent, MessageService, MessageProperties, Util) {

        function createTreeNode(originalNode, newNode){
//            newNode.status = originalNode.status;
            newNode.name = originalNode.name;
            newNode.text = originalNode.text;
            newNode.id = originalNode.id;
            newNode.device_type_id = originalNode.device_type_id;
            newNode.isDevice = originalNode.isDevice;
            newNode.iconCls = originalNode.iconCls;
            newNode.rs485 = originalNode.rs485;
//            newNode.status = originalNode.status;
            return newNode;
        }

        function removeSIODevice(newTreeNode, originalTreeNode) {
            var item, newChildren = [];

            newTreeNode = createTreeNode(originalTreeNode, newTreeNode);

            if (originalTreeNode.children != undefined) {
                for (var i = 0; i < originalTreeNode.children.length; i++) {
                    var originalChildNode = originalTreeNode.children[i];
                    var newChildNode = {};

                    if (newTreeNode.children == undefined) {
                        newTreeNode.children = [];
                    }

                    if (angular.isUndefined(originalChildNode.device_type_id)) {
                        newTreeNode.children.push(newChildNode);
                    } else if(originalChildNode.device_type_id.id != Const.DEVICE_TYPE_ID.SECURE_IO2 && originalChildNode.device_type_id.id != Const.DEVICE_TYPE_ID.SECURE_IO) {
                        if(originalChildNode.rs485.mode != Const.RS485_MODE.SLAVE) {
                            newTreeNode.children.push(newChildNode);
                        }
                    }
                    removeSIODevice(newChildNode, originalChildNode);
                }
            }

        }

        $scope.DeviceTreeData = [{}];
        if (DeviceAgent.getDeviceGroupRecursiveData().then) {
            DeviceAgent.getDeviceGroupRecursiveData().then(function success(data) {
                removeSIODevice($scope.DeviceTreeData[0], data);
            });
        } else {
            removeSIODevice($scope.DeviceTreeData[0], DeviceAgent.getDeviceGroupRecursiveData());
        }

        $scope.import = {};
        $scope.import.devices = [];
        $scope.import.forced_update = false;

        $scope.doUserManualImportToDevice = function () {
            if (angular.element(event.target || event.srcElement).hasClass('disabled')) {
                return;
            }
            // $scope.checkItemArray 는 this Dlg 를 embed 하고 있는 UserController의 Scope 변수입니다.
            var i, item,
                param = $scope.checkItemArray,
                deviceCollection = [];
            // $scope.SelectAllYN 는 this Dlg 를 embed 하고 있는 UserController의 Scope 변수입니다.
            if ($scope.SelectAllYN) {
                param = Const.PARAM_ALL_STRING;
            }

            for(i = $scope.import.devices.length; i--; ) {
                item = $scope.import.devices[i];
                deviceCollection.push({"id":item.id});
            }

            if(deviceCollection.length <= 0) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.thereAreNoDevices'));
                return false;
            }
            if($scope.checkItemArray.length <= 0) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseSelectUser'), function() {
                    $scope.$emit('InitManualImportToDeviceDlgUrl');
                });
                return false;
            }

            UserAgent.exportUsersToDevice({
                "UserIDs": param,
                "query": $scope.userListQueryStrings,
                "overwrite": $scope.import.forced_update,
                "DeviceCollection": {"rows":deviceCollection}
            }).then(
                function success(data) {
//                    console.log("export users to device then");
                    //UserController receives from this event.
                    $scope.$emit('InitManualImportToDeviceDlgUrl');
                    Util.debug("exportUsersToDevice success");
//                    MessageService.displayInformationMessage(MessageProperties.get('user.transferCompletion'));
                }).finally(function() {
                    $scope.$emit('ReloadUsersDataGrid');
                });
        };

        $scope.doCancelUserManualImportToDevice = function () {
            //UserController receives from this event.
            $scope.$emit('InitManualImportToDeviceDlgUrl');
        };
    }]);
});