'use strict';

define(function () {
    var app = angular.module('biostarWebApp');
    app.register.service('FingerprintScanService', ['$q', '$http','API','Const','Util','$location','MessageProperties', function FingerprintScan($q, $http, API, Const, Util, $location, MessageProperties) {
        // AngularJS will instantiate a singleton by calling "new" on this function

        var template_type = '';
        var URL_BY_PROTOCOL = '';

        if(window.location.protocol.indexOf('https') > -1) {
            URL_BY_PROTOCOL = Const.USBAGENT.HTTPS_BASE_URL;
        } else {
            URL_BY_PROTOCOL = Const.USBAGENT.HTTP_BASE_URL;
        }

        var setTemplateType = function() {
            var biostarSetting = Util.getSessionObject(Const.SESSION_STORAGE_KEY.SERVER_SETTING);
            if (biostarSetting == undefined) {
                API.get("setting/biostar").then(function (data) {
                    Util.setSessionObject(Const.SESSION_STORAGE_KEY.SERVER_SETTING, data.BioStar);
                    template_type = Const.FINGERPRINT_TEMPLATE_FORMAT_OPTION[data.BioStar.fingerprint_format];
                    template_type = Const.FINGERPRINT_USB_SCANNER_TEMPLATE_TYPE_FOR_SWITCH[template_type];
                });
            } else {
                template_type = Const.FINGERPRINT_TEMPLATE_FORMAT_OPTION[biostarSetting.fingerprint_format];
                template_type = Const.FINGERPRINT_USB_SCANNER_TEMPLATE_TYPE_FOR_SWITCH[template_type];
                return template_type;
            }
        };

        setTemplateType();

        var getFingerprint = function (options) {
            var deferred = $q.defer();
            options.params.timestamp = new Date().getTime(); // 타임스탬프 부여하여 두번째 요청과 구별함
            //var _bs_session_id = $http.defaults.headers.common['bs-session-id'];
            //delete $http.defaults.headers.common['bs-session-id'];
            options.params.template_type = template_type;
            $.ajax({
                type: "GET",
                url: URL_BY_PROTOCOL + "ScanBiomini",
                data: options.params,
                success: function (data) {
                    var retCode = angular.fromJson(data);
                    if (retCode.retcode == 1) {
                        deferred.resolve(data);
                    } else {
                        deferred.reject(MessageProperties.get('FailedToScanFingerPrint'), retCode.retcode);
                    }
                },
                error: function (xhr, status, error) {
                    deferred.reject(error);
                }
            });
            //$http.defaults.headers.common['bs-session-id'] = _bs_session_id;

            return deferred.promise;
        };

        var getVerifyFingerprint = function (options) {
            var deferred = $q.defer();
            options.timestamp = new Date().getTime(); // 타임스탬프 부여하여 두번째 요청과 구별함
            //var _bs_session_id = $http.defaults.headers.common['bs-session-id'];
            //delete $http.defaults.headers.common['bs-session-id'];
            options.template_type = template_type;
            $.ajax({
                type: "POST",
                url: URL_BY_PROTOCOL + "Verify",
                data: options,
                success: function (data) {
                    var retCode = angular.fromJson(data);
                    if (retCode.retcode == 1) {
                        deferred.resolve(data);
                    } else {
                        deferred.reject(MessageProperties.get('validate.fingerprintsDoNotMatch'), retCode.retcode);
                    }
                },
                error: function (xhr, status, error) {
                    deferred.reject(error);
                }
            });
            //$http.defaults.headers.common['bs-session-id'] = _bs_session_id;

//  		$http.post(URL_BY_PROTOCOL + 'Verify', angular.toJson(options))
//  		.success(function(data) {
//  			deferred.resolve(data);
//  			//$http.defaults.headers.common['bs-session-id'] = _bs_session_id;
//  		})
//  		.error(function(reason) {
//  			deferred.reject(reason);
//  			//$http.defaults.headers.common['bs-session-id'] = _bs_session_id;
//  		});
            return deferred.promise;
        };

        var getTemplateFingerprintImg = function (options) {
            var deferred = $q.defer();
            options.timestamp = new Date().getTime(); // 타임스탬프 부여하여 두번째 요청과 구별함
            //var _bs_session_id = $http.defaults.headers.common['bs-session-id'];
            //delete $http.defaults.headers.common['bs-session-id'];
            options.template_type = template_type;
            $.ajax({
                type: "POST",
                url: URL_BY_PROTOCOL + "GetTemplateImage",
                data: options,
                success: function (data) {
                    var retCode = angular.fromJson(data);
                    if (retCode.retcode == 1) {
                        deferred.resolve(data);
                    } else {
                        deferred.reject(MessageProperties.get('validate.failedToLoadFingerprintTemplate'), retCode.retcode);
                    }
                },
                error: function (xhr, status, error) {
                    deferred.reject(error);
                }
            });
            //$http.defaults.headers.common['bs-session-id'] = _bs_session_id;

//  		$http.post(URL_BY_PROTOCOL + 'GetTemplateImage', angular.toJson(options))
//  		.success(function(data) {
//  			deferred.resolve(data);
//  			//$http.defaults.headers.common['bs-session-id'] = _bs_session_id;
//  		})
//  		.error(function(reason) {
//  			deferred.reject(reason);
//  			//$http.defaults.headers.common['bs-session-id'] = _bs_session_id;
//  		});
            return deferred.promise;
        };


        var connectCheckUSBDevice = function () {
            var deferred = $q.defer();
            var options = {
                timestamp: (new Date()).getTime(),
                'template1': '1',
                'template2': '1'
            }; // 타임스탬프 부여하여 두번째 요청과 구별함
            //var _bs_session_id = $http.defaults.headers.common['bs-session-id'];
            //delete $http.defaults.headers.common['bs-session-id'];
            options.template_type = template_type;
            $.ajax({
                type: "POST",
                url: URL_BY_PROTOCOL + "Verify",
                data: options,
                success: function (data) {
                    deferred.resolve(data);
                },
                error: function (xhr, status, error) {
                    deferred.reject(error);
                }
            });
            //$http.defaults.headers.common['bs-session-id'] = _bs_session_id;
            return deferred.promise;
        };

        return {
            getFingerprint: getFingerprint,
            getVerifyFingerprint: getVerifyFingerprint,
            getTemplateFingerprintImg: getTemplateFingerprintImg,
            connectCheckUSBDevice: connectCheckUSBDevice,
            setTemplateType : setTemplateType
        };

    }]);
});
