﻿'use strict';

define([
    'modules/common/services/user-agent',
    'modules/user/userGroupTreeController',
    'modules/user/dialog/user-info-batch-edit-dlg',
    'modules/user/dialog/user-manual-import-to-device',
    'modules/user/userInfoController',
    'modules/common/datagrid/columnSettingController',
    'modules/common/datagrid/columnSettingService',
    'modules/common/print/printOptionController',
    'modules/common/print/printOptionService'
], function () {
    var app = angular.module('biostarWebApp');
    app.register.controller('UserController', [
        '$scope', '$rootScope', '$compile', '$document', '$window', '$q', '$location', 'Const', 'UserAgent', 'MessageProperties', 'Util', 'ColumnSettingService', 'PrintOptionService', 'MessageService', 'PrivilegeService', 'FileService',
        function ($scope, $rootScope, $compile, $document, $window, $q, $location, Const, UserAgent, MessageProperties, Util, ColumnSettingService, PrintOptionService, MessageService, PrivilegeService, FileService) {

            if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.USER)) {
                $scope.readOnly = true;
            }

            $scope.gridId = "userGrid";
            var userInfo,
                userListDataGridColumns = [
                    [
                        {
                            field: ($scope.readOnly ? 'none' : Const.FIELD_NAME.CHECKBOX),
                            checkbox: !$scope.readOnly,
                            hidden: $scope.readOnly
                        },
                        {
                            field: 'user_id',
                            title: MessageProperties.get('common.id'),
                            width: 70,
                            align: 'left',
                            sortable: true
                        },
                        {
                            field: 'name',
                            title: MessageProperties.get('common.name'),
                            width: 70,
                            align: 'left',
                            sortable: true,
                            formatter: function (value, row, index) {
                                return value || '-';
                            }
                        },
                        {
                            field: 'email',
                            title: MessageProperties.get('common.email'),
                            width: 100,
                            align: 'left',
                            sortable: true,
                            formatter: function (value, row, index) {
                                return value || '-';
                            }
                        },
                        {
                            field: 'user_group_id',
                            title: MessageProperties.get('common.group'),
                            width: 80,
                            align: 'left',
                            sortable: false,
                            formatter: function (value, row, index) {
                                var path = UserAgent.getUserGroupPath(value.id);

                                if (value.id == 1) {
                                    path = value.name;
                                }

                                return '<span title="' + path + '">' + value.name + '</span>';
                            }
                        },
                        {
                            field: 'access_groups',
                            title: MessageProperties.get('common.accessGroup'),
                            width: 100,
                            align: 'left',
                            formatter: function (value, row, index) {
                                var
                                    box,
                                    len,
                                    item,
                                    data = [],
                                    ret_str = '-';

                                if (value) {
                                    for (box = 0, len = value.length; box < len; box++) {
                                        data.push(value[box].name);
                                    }
                                }

                                if (row.access_groups_in_user_group) {
                                    for (box = 0, len = row.access_groups_in_user_group.length; box < len; box++) {
                                        item = row.access_groups_in_user_group[box];
                                        if (data.indexOf(item.name) == -1) {
                                            data.push(item.name);
                                        }
                                    }
                                }

                                if (data.length > 0) {
                                    ret_str = data[0];
                                    if (data.length > 1) {
                                        ret_str += ' + ' + '<mark class="number">' + parseInt(data.length - 1) + '</mark>';
                                    }
                                }
                                return ret_str;
                            }
                        },
                        {
                            field: 'fingerprint_template_count',
                            title: '<span class="iconFinger">Fingerprint</span>',
                            width: 30,
                            formatter: function (value, row, index) {
                                if (angular.isDefined(value))
                                    return value;
                                else {
                                    return '-';
                                }
                            }
                        },
                        {
                            field: 'card_count', title: '<span class="iconCard">Card</span>', width: 30,
                            formatter: function (value, row, index) {
                                if (angular.isDefined(value))
                                    return value;
                                else {
                                    return '-';
                                }
                            }
                        },
                        {
                            field: 'disabled', title: MessageProperties.get('common.status'), width: 30,
                            formatter: function (value, row, index) {
                                if (value != 'true') {
                                    return '-';
                                } else {
                                    return MessageProperties.get("user.status.inactive");
                                }
                            }
                        }
                    ]
                ],
                userListQueryStrings = {
                    query: undefined,
                    offset: 0,
                    limit: 15,
                    group_id: 1,
                    order_by: undefined,
                    total: undefined,
                    timeout: undefined
                },
                userListDataGridOptions = {
                    scrollbarSize: 0,
                    view: $.extend({}, $.fn.datagrid.defaults.view, {
                        onBeforeRender: function (target, rows) {
                            angular.element(target).datagrid('resize');
                        },
                        onAfterRender: function (target) {
                            $.fn.datagrid.defaults.view.onAfterRender.call(this, target);
                            var grdPanel = angular.element(target).datagrid('getPanel');
                            var opts = angular.element(target).datagrid('options');
                            var vc = grdPanel.children('div.datagrid-view');
                            grdPanel.find(".datagrid-empty").remove();

                            if (angular.element(target).datagrid('getRows').length === 0) {
                                var html = '<div style="width:100%; height:360px; font-size:1.333em; line-height: 360px; ' +
                                    'vertical-align: middle; text-align: center;">' + MessageProperties.get('validate.notFound') +
                                    '</div>';
                                grdPanel.find('div.datagrid-view2 div.datagrid-body').html(html);
                            }

                            //헤더에 전체선택 버튼이 없다면 추가
                            if (grdPanel.find('.datagrid-header-check.compiled').length <= 0) {
                                grdPanel.find('.datagrid-header-check').attr({
                                    'select-all-yn': 'SelectAllYN',
                                    'datagrid-id': '"' + target.id + '"',
                                    'clear-function': 'clearCheckItemArray'
                                });
                                $compile(grdPanel.find('.datagrid-header-row'))($scope);
                                grdPanel.find('.datagrid-header-check').addClass('compiled');
                            }
                        }
                    }),
                    fitColumns: true,
                    resize: true,
                    checkbox: !$scope.readOnly,
//                height: 572,
                    singleSelect: true,
                    selectOnCheck: false,
                    checkOnSelect: false,
                    columns: userListDataGridColumns,
//                multiSort: true,
                    onSortColumn: function (sort, order) {
                        var i,
                            orderItemArray,
                            descending = order == 'desc',
                            orderItemObject = {};

                        $scope.userListQueryStrings.order_by = sort + ":" + descending;
//
//
//                    if($scope.userListQueryStrings.order_by && $scope.userListQueryStrings.order_by.split) {
//                        orderItemArray = $scope.userListQueryStrings.order_by.split('+');
//                        for(i = orderItemArray.length; i--; ) {
//                            var items = orderItemArray[i].split(':');
//                            orderItemObject[items[0]] = items[1];
//                        }
//                    } else {
//                        orderItemArray = [];
//                    }
//
//                    delete orderItemObject[sort];
//
//                    orderItemObject[sort] = descending;
//
//                    $scope.$apply(function() {
//                        orderItemArray = [];
//                        for( var key in orderItemObject) {
//                            orderItemArray.push(key+':'+orderItemObject[key]);
//                        }
//
//                        $scope.userListQueryStrings.order_by = orderItemArray.join('+');
//                    });

                        beSearchUserList();
                    },
                    onCheck: function (rowIndex, rowData) {
                        var existsYn = $scope.checkItemArray.indexOf(rowData.user_id);
                        if (existsYn == -1) {
                            if ($scope.checkItemArray.length >= Const.SELECT_LIST_ITEM_LIMIT) {
                                MessageService.displayWarningMessage(MessageProperties.get('validate.canNotSelectMoreItem', Const.SELECT_LIST_ITEM_LIMIT));
                                angular.element('#userGrid').datagrid('uncheckRow', rowIndex);
                            } else {
                                $scope.checkItemArray.push(rowData.user_id);
                                if (!$scope.$root.$$phase) $scope.$apply();
                            }
                        }
                    },
                    onUncheck: function (rowIndex, rowData) {
                        if ($scope.SelectAllYN == true) {
                            $scope.checkItemArray = [];
                            var data = angular.element('#userGrid').datagrid('getChecked');
                            $scope.SelectAllYN = false;
                            if (!$scope.$root.$$phase) $scope.$apply();
                            for (var i = 0; i < data.length; i++) {
                                var index = angular.element('#userGrid').datagrid('getRowIndex', data[i]);
                                angular.element('#userGrid').datagrid('checkRow', index);
                            }
                        } else {
                            var idx = $scope.checkItemArray.indexOf(rowData.user_id);
                            if (idx > -1) {
                                $scope.checkItemArray.splice(idx, 1);

                                if (!$scope.$root.$$phase) $scope.$apply();
                            }
                        }
                    },
                    onCheckAll: function (rows) {
                        var i, uncheckList = [];
                        for (i = 0; i < rows.length; i++) {
                            var item = rows[i];
                            var idx = $scope.checkItemArray.indexOf(item.user_id);

                            if (idx == -1) {
                                if ($scope.checkItemArray.length < Const.SELECT_LIST_ITEM_LIMIT) {
                                    $scope.checkItemArray.push(item.user_id);
                                } else {
                                    uncheckList.push(item);
                                }
                            }
                        }

                        if (uncheckList.length > 0 && $scope.SelectAllYN == false) {
                            MessageService.displayWarningMessage(MessageProperties.get('validate.canNotSelectMoreItem', Const.SELECT_LIST_ITEM_LIMIT));
                            for (i = uncheckList.length; i--;) {
                                angular.element('#userGrid').datagrid('uncheckRow', angular.element('#userGrid').datagrid('getRowIndex', uncheckList[i]));
                            }
                        }

                        if (!$scope.$root.$$phase) $scope.$apply();
                    },
                    onUncheckAll: function (rows) {
                        if ($scope.SelectAllYN == true) {
                            $scope.SelectAllYN = false;
                            $scope.checkItemArray = [];
                        }
                        for (var i = rows.length; i--;) {
                            var item = rows[i];
                            var idx = $scope.checkItemArray.indexOf(item.user_id);
                            if (idx > -1) {
                                $scope.checkItemArray.splice(idx, 1);
                            }
                        }
                        if (!$scope.$root.$$phase) $scope.$apply();

                    },
                    onClickCell: function (rowIndex, field) {
                        $rootScope.clickCellOnGrid($scope.gridId, rowIndex, field, "user_id", function (rowData) {
                            UserAgent.getOneUser(rowData.user_id).then(function success(data) {
                                window.sessionStorage.setItem('userCurrPos', parseInt($scope.userListQueryStrings.offset) + rowIndex + 1);
                                setUserInfoUrl()
                            });
                        });
                    }
                };

            $scope.clearCheckItemArray = function () {
                $scope.checkItemArray = [];
            };

            function setUserInfoUrl() {
                $scope.selectedUserInfo = 'modules/user/userInfo.html';
            }

            function SelectUserGroupAfterAction(user_group) {
                $scope.user_group = user_group
            }

            $scope.deleteTheSelectedUser = function () {
                MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                    if (confirmYn) {
                        var param = $scope.checkItemArray;
                        if ($scope.SelectAllYN) {
                            param = 'ALL';
                        }
                        UserAgent.deleteUsers({
                            "UserIDs": param,
                            "query": $scope.userListQueryStrings
                        }).then(function success(data) {
                        }).finally(function () {
                            $scope.$emit('ReloadUsersDataGrid');
                        })
                    }
                });
            };

            $scope.checkItemArray = [];
            $scope.user_group = {
//            id: 1,
//            name: 'All User Group'
            };

            $scope.$on('CurrentSelectedUserGroupName', function (event, data) {
                $scope.user_group.id = data.id;
                $scope.user_group.name = data.name;
            });

            $scope.userListQueryStrings = userListQueryStrings;
            $scope.UserListData = [];

            angular.element('#userGrid').datagrid(userListDataGridOptions);

            //헤더에 전체선택 버튼 추가
            $scope.SelectAllYN = false;

            $scope.$watch('SelectAllYN', function (nv) {
                if (nv) {
                    angular.element('#userGrid').datagrid('checkAll');
                } else {
                    angular.element('#userGrid').datagrid('uncheckAll');
                    $scope.checkItemArray = [];
                }
            });

            $scope.cancelSelectAllUser = function () {
                angular.element('#userGrid').datagrid('uncheckAll');
            };


            function beSearchUserList(query, deferFunctionName) {
                var i, rowData, allUserIdList, startIdx, limit;

                if (query) {
                    userListQueryStrings.query = query;
                }

                $rootScope.$broadcast('CurrentSelectedUser`sGroup', $scope.user_group);

                UserAgent.getUsers($scope.userListQueryStrings, deferFunctionName).then(function success(data) {
                    var index = 0;
                    if (data.UserCollection.rows != undefined) {
                        for (index = 0; index < data.UserCollection.rows.length; index++) {
                            if (data.UserCollection != undefined && data.UserCollection.rows[index] != undefined &&
                                data.UserCollection.rows[index].user_group_id != undefined && data.UserCollection.rows[index].user_group_id.id == Const.DEFAULT_ALL_GROUP_ID) {
                                data.UserCollection.rows[index].user_group_id.name = MessageProperties.get("menu.user.allUserGroups");
                            }
                        }
                    }
                    $scope.UserListData = data.UserCollection.rows || [];

                    window.sessionStorage.setItem('UserListData', angular.toJson($scope.UserListData));

                    allUserIdList = new Array(parseInt(data.UserCollection.total));

                    startIdx = parseInt($scope.userListQueryStrings.offset);
                    limit = parseInt($scope.userListQueryStrings.limit);

                    if (parseInt($scope.userListQueryStrings.offset) + parseInt($scope.userListQueryStrings.limit) > data.UserCollection.total) {
                        limit = parseInt(data.UserCollection.total) - $scope.userListQueryStrings.offset;
                    }

                    for (i = 0; i < limit; i++) {
                        allUserIdList[startIdx + i] = data.UserCollection.rows[i].user_id;
                    }

                    window.sessionStorage.setItem('getAllUserIdList', angular.toJson(allUserIdList));

                    angular.element('#userGrid').datagrid('loadData', $scope.UserListData);

                    $scope.userListQueryStrings.total = data.UserCollection.total || 0;

                    if ($scope.SelectAllYN) {
                        for (i = $scope.UserListData.length; i--;) {
                            rowData = $scope.UserListData[i];
                            if ($scope.checkItemArray.indexOf(rowData.user_id) < 0) {
                                $scope.checkItemArray.push(rowData.user_id);
                            }
                        }
                    }

                    if (!$scope.SelectAllYN) {
                        angular.element('.datagrid-header-check').find('input').attr('checked', false);
//                    angular.element('#userGrid').datagrid('uncheckAll');
                    } else {
//                    angular.element('#userGrid').datagrid('checkAll');
                    }

                    for (i = $scope.UserListData.length; i--;) {
                        rowData = $scope.UserListData[i];
                        if ($scope.checkItemArray.indexOf(rowData.user_id) > -1) {
                            angular.element('#userGrid').datagrid('checkRow', i);
                        }
                    }

                });
            }


            $scope.SearchCallback = {
                fn: beSearchUserList
            };

            $scope.$on('ReloadUsersDataGrid', function (event, data) {
                var groupId = Util.getSessionObject('selectedUserGroupId');
                Util.debug('ReloadUsersDataGrid');
                angular.element('#userGrid').datagrid('uncheckAll');

                userListQueryStrings.offset = 0;

                if (data && data.offset) {
                    userListQueryStrings.offset = data.offset;
                }

                if (groupId) {
                    userListQueryStrings.group_id = groupId;
                }

                beSearchUserList();
                $scope.checkItemArray = [];
            });

//        if (UserAgent.getUserGroupRecursiveData('inUserList')) {
//            beSearchUserList();
//        }

            $scope.$watch(function () {
                return angular.element('.panel.datagrid').width();
            }, function (nv, ov) {
                if (nv !== ov) {
                    angular.element('#userGrid').datagrid('resize');
                }
            }, true);

            angular.element($window).on('resize', function ($event) {
                var event = $event | window.event;
                angular.element('#userGrid').datagrid('resize');
            });

            $scope.disableSelectedUserInfo = function () {
                MessageService.displayConfirmMessage(MessageProperties.get('validate.inactiveAllSelectedUsers'), function (confirmYN) {
                    if (confirmYN) {
                        var param = $scope.checkItemArray;
                        if ($scope.SelectAllYN) {
                            param = 'ALL';
                        }
                        UserAgent.putUsers({
                            "userIDs": param, "user": {"disabled": "true"}, "query": $scope.userListQueryStrings
                        }).then(function success(data) {
                        }, function error(reason, status) {
                        }).finally(function () {
                            $scope.$emit('ReloadUsersDataGrid');
                        });
                    }
                })
            };

            $scope.userInfoBatchEditDlg = {};
            $scope.doOpenUserInfoBatchEditDlg = function () {
                $scope.userInfoBatchEditDlg.url = 'modules/user/dialog/user-info-batch-edit-dlg.html';
            };

            $scope.$on('InitUserInfoBatchEditDlgUrl', function () {
                $scope.userInfoBatchEditDlg = {};
                angular.element('#userInfoBatchEditDlg').dialog('destroy');
            });

            $scope.doAfterOpenUserInfoBatchEditDlg = function () {
                var options = {};
                options.iconCls = 'icon-save';
                options.modal = true;
                options.shadow = false;
                options.border = false;
                options.title = MessageProperties.get('user.header.batchEdit');
                options.width = 740;
                options.height = 'auto';
                options.top = 200;

                options.onClose = function () {
                    $scope.$apply(function () {
                        $scope.userInfoBatchEditDlg = {url: ''};
                        angular.element('#userInfoBatchEditDlg').dialog('destroy');
                    });
                };

                $("#userInfoBatchEditDlg").dialog(options);
            };

            $scope.userManualImportToDeviceDlg = {};
            $scope.doOpenManualImportToDeviceDlg = function () {
                $scope.userManualImportToDeviceDlg.url = 'modules/user/dialog/user-manual-import-to-device.html';
            };

            $scope.$on('InitManualImportToDeviceDlgUrl', function () {
                $scope.userManualImportToDeviceDlg = {};
                angular.element('#userManualImportToDeviceDlg').dialog('destroy');
            });


            $scope.doAfterOpenManualImportToDeviceDlg = function () {
                var options = {};
                options.iconCls = 'icon-save';
                options.modal = true;
                options.shadow = false;
                options.border = false;
                options.title = MessageProperties.get('user.header.transferToDevice');
                options.width = 570;
                options.height = 'auto';
                options.top = 200;

                options.onClose = function () {
                    $scope.$apply(function () {
                        $scope.userManualImportToDeviceDlg = {url: ''};
                        angular.element('#userManualImportToDeviceDlg').dialog('destroy');
                    });
                };

                $("#userManualImportToDeviceDlg").dialog(options);
            };

            $scope.dropDownMenu = [
//            {
//                menu: 'Import to CSV',
//                func: function () {
//                    console.info(this.menu);
//                }
//            },
//            {
//                menu: 'Export to CSV',
//                func: function () {
//                    console.info(this.menu);
//                }
//            },
//            {
//                menu: 'Print Preview',
//                func: function () {
//                    Util.beforeUrl($location.url());
//                    $location.path('/user/users-report');
//                }
//            },
                {
                    menu: MessageProperties.get('menu.print'),
                    func: function () {
                        PrintOptionService.openPrintOptionPopupDlg();
                    }
                },
                {
                    menu: MessageProperties.get('menu.columnSetting'),
                    func: function () {
                        ColumnSettingService.openColumnSettingPopupDlg();
                    }
                },
                {
                    menu: MessageProperties.get('menu.csvDownload'),
                    func: function () {
                        if ($scope.checkItemArray == undefined || $scope.checkItemArray.length < 1) {
                            MessageService.displayWarningMessage(MessageProperties.get("validate.mustBeSelected", "common.user"))
                            return;
                        }
                        var params = {
                            id_list: $scope.checkItemArray,
                            query: $scope.userListQueryStrings.query,
                            offset: $scope.userListQueryStrings.offset,
                            limit: $scope.userListQueryStrings.limit,
                            group_id: $scope.userListQueryStrings.group_id
                        };
                        if ($scope.SelectAllYN) {
                            params.id_list = Const.BIOSTAR_SERVER_PARAM_ALL;
                        }
                        UserAgent.downloadCsv(params).then(
                            function success(data) {
                                //Util.debug(data);
                                var url = Const.WEBDAV_PREFIX + data.CsvOption.filename;
                                $window.location.href = url;
                                //var downloadWindow = window.open(url, data.CsvOption.filename, 'width=10,height=10,directories=0,titlebar=0,toolbar=0,location=0,status=0,menubar=0,scrollbars=0,resizable=yes');
                                //window.setTimeout(function () {
                                //    if (downloadWindow != undefined) {
                                //        downloadWindow.close();
                                //    }
                                //}, 3000);

                            }, function error(data) {
                                Util.error(data);
                            }).finally(function () {
                            });
                    }
                },
                {
                    menu: MessageProperties.get('menu.csvUpload'),
                    func: function () {
                        angular.element('#csv_file_upload').click();
                    }
                }
            ];

            angular.element('#csv_file_upload').on('change', function (event) {
                var element = this;
                var filename = angular.element(element).val();
                if (filename == undefined || filename == null || filename.length < 1) {
                    return;
                }
                FileService.blobFileReader(event).then(function (data) {

                    var index1 = filename.lastIndexOf('/');
                    if (index1 < 0) {
                        index1 = filename.lastIndexOf('\\');
                    }
                    if (index1 > -1) {
                        filename = filename.substr(index1 + 1);
                    }
                    UserAgent.uploadFileToWebDav(filename, data).then(function (result1) {
                        Util.debug(result1);
                        if (result1.httpResponseStatus == Const.HTTP_STATUS_CODE.CREATED ||
                            result1.httpResponseStatus == Const.HTTP_STATUS_CODE.NO_CONTENT) {
                            var cvsOptionData = {
                                CsvOption: {
                                    filename: filename,
                                    columns: []
                                }
                            };
                            UserAgent.uploadCsv(cvsOptionData).then(function (result2) {
                                MessageService.displayInformationMessage(MessageProperties.get('common.uploadSuccessful'), function () {
                                    angular.element('#csv_file_upload').val('');
                                    $scope.$emit('ReloadUsersDataGrid');
                                });
                            }, function (error3) {
                                Util.error(error3);
                                //MessageService.displayErrorMessage(MessageProperties.get(Const.SERVER_RESPONSE_MESSAGE_PREFIX + "." + error3.Response.code));
                            }).finally(function () {
                            });
                        }
                    }, function (error2) {
                        Util.error(error2);
                    }).finally(function () {
                    });
                }, function (error) {
                    Util.error(error);
                }).finally(function () {
                });
            });

            $scope.$on('ReloadUserGroupData', function (event, data) {
                angular.element('#userGrid').datagrid('uncheckAll');
            });

            function closeUserInfo(event, data) {
                window.sessionStorage.removeItem('userInfo');
                window.sessionStorage.removeItem('UserListData');
                window.sessionStorage.removeItem('userCurrPos');
                $scope.$emit('ReloadUserGroupData');
                $scope.selectedUserInfo = undefined;
                beSearchUserList();
            }

            var clickMenuInitUnbind = $rootScope.$on('ClickMenuInit', closeUserInfo);
            $scope.$on('$locationChangeStart', clickMenuInitUnbind);

            $scope.$on('CloseUserInfo', closeUserInfo);

            $scope.$on('OpenUserInfo', function (event, data) {
                window.sessionStorage.setItem('userInfo', '{}');
                setUserInfoUrl();
//            angular.element('#userGrid').datagrid('unselectAll');
            });

            userInfo = window.sessionStorage.getItem('userInfo');

            if (userInfo && userInfo != 'undefined') {
                userInfo = angular.fromJson(userInfo);
                setUserInfoUrl()
            }

            $scope.$watch('userListQueryStrings', function (nv, ov) {
                if (nv !== ov) {
                    PrintOptionService.buildGridQueryFunction(Const.REPORT_COLLECTION_NAME.USER, nv);
                }
            }, true);

            PrintOptionService.getReportColumnNameFunction = function (gridColumnName) {
                var reportColumnName = gridColumnName;
                if (gridColumnName == "user_group_id") {
                    return "user_group_id.name"
//            } else  if (gridColumnName == "access_groups"){
//                return "access_groups.id"
                } else if (gridColumnName == "fingerprint_templates") {
                    return "fingerprint_templates.count"
                } else if (gridColumnName == "cards") {
                    return "cards.count"
                }
                return reportColumnName;
            };
            PrintOptionService.getReportColumnTypeFunction = function (gridColumnName) {
                if (gridColumnName == "access_groups") {
                    return "objects"
                } else if (gridColumnName == "disabled") {
                    return "enum";
                } else {
                    return "string";
                }
            };
            $scope.reportUrl = Const.API.BASE_URL + 'users/report';
            $scope.openPrintOptionPopupDlg = PrintOptionService.openPrintOptionPopupDlg;
            $scope.openColumnSettingPopupDlg = ColumnSettingService.openColumnSettingPopupDlg;

            ColumnSettingService.setGridId($scope.gridId);
            ColumnSettingService.setCallbackFunction($scope.SearchCallback.fn);
            var customColumnSetting = ColumnSettingService.getColumnSettingFromLocalStroage();
            if (customColumnSetting) {
                ColumnSettingService.applyChangedColumnsToGrid(customColumnSetting, false);
            }
        }]);
});


