﻿'use strict';

define([
    'modules/common/services/user-agent'
], function () {
    var app = angular.module('biostarWebApp');

    app.register.controller('UserGroupTreeController', ['$scope', '$rootScope', '$location', '$compile', '$timeout', 'UserAgent', 'MessageProperties', 'Const', 'Util', 'PrivilegeService', 'MessageService', function ($scope, $rootScope, $location, $compile, $timeout, UserAgent, MessageProperties, Const, Util, PrivilegeService, MessageService) {

        var UserGroupTreeOption,
            contextMenuDiv,
            selectedNode,
            contextNode,
            renameYn,
            body = document.getElementsByTagName('body')[0];

        var isAndroid = (/android/gi).test(navigator.appVersion);

        window.sessionStorage.removeItem('selectedUserGroupId');

        $scope.isUserMenu = function () {
            return ($location.path() == Const.MENU.USER);
        };

        var menuID = "#userGroupTreeContextMenu";

        function initContextMenu() {
            angular.element(menuID).menu({});
        }

        function clearMenuContext(divId) {
            angular.element(divId).find(".menu-item").remove();
        }

        function buildContextMenuDiv(contextMenuObject, ngIfCondition) {
            var html = '<div';
            if (ngIfCondition != undefined) {
                html += ' ng-if="' + ngIfCondition + '" ';
            }
            html += ' class="menu-item" style="height:31px;" ng-click="' + contextMenuObject.clickFunctionName + '()" ng-privilege><div class="menu-text">' + contextMenuObject.menuName + '</div></div>';
            return html;
        }

        initContextMenu();

        var onContextMenu = function (e, node) {

            if (e.preventDefault) e.preventDefault();
            if (e.returnValue) e.returnValue = false;

            if (!PrivilegeService.isUIAuthorizedForUpdate(Const.PERMISSION_MODULE_ID.USER)) {
                return false;
            }

            angular.element(menuID).menu("hide");
            clearMenuContext(menuID);

            angular.element(menuID).append(buildContextMenuDiv(ContextMenu.ADD_USER_GROUP));
            if (node.id != 1) {
                angular.element(menuID).append(buildContextMenuDiv(ContextMenu.RENAME_USER_GROUP));
                angular.element(menuID).append(buildContextMenuDiv(ContextMenu.DELETE_USER_GROUP));
            }

            angular.element(menuID).css("height", "auto");
            angular.element(menuID).css("width", "auto");

            $compile(angular.element(menuID))($scope);

            angular.element(menuID).menu('options').ownerTreeNode = node;
            angular.element(menuID).menu('show', {
                left: e.pageX,
                top: e.pageY
            });

        };

		function userGroupMaxNumbers() {

			var newUserGroupPrefix = MessageProperties.get("menu.user.newUserGroup");
			var newUserGroupsArray =  angular.element('#userGroupsTree').find('span[title*="' + newUserGroupPrefix + '"]');
			var numbers = [];
			var regExp = /[\d]+/gi;

			for(var i = 0; i < newUserGroupsArray.length; i++) {
				var text = angular.element(newUserGroupsArray[i]).html();
				numbers = numbers.concat(text.match(regExp));

			}

			numbers.sort(function (a, b) {
				return parseInt(a) > parseInt(b) ? -1 : 1;
			});

			console.log(numbers);

			return numbers[0] ? parseInt(numbers[0]) + 1: 1;

		}

        $scope.addUserGroup = function addUserGroup (_node) {
            var name;

            var newUserGroupPrefix = MessageProperties.get("menu.user.newUserGroup");
            var newUserGroupsMaxCount = userGroupMaxNumbers();

            angular.element(menuID).menu('hide');

            if(_node == null) {
                _node = angular.element(menuID).menu('options').ownerTreeNode;
                if(_node == null) {
                    return false;
                }
            }

            name = newUserGroupPrefix + ' ' + newUserGroupsMaxCount;

            if (_node.depth == 8) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.userGroupCanNotExceedSteps'));
            } else {
                var newNode = {
                    "parent_id": {
                        "id": _node.id
                    },
                    "depth": parseInt(_node.depth || 0) + 1,
                    "sync_device_groups": _node.sync_device_groups || [],
                    "sync_devices": _node.sync_devices || [],
                    "inherited": true,
                    "text": name,
                    "name": name
                };

                if (_node.state == 'closed') {
                    angular.element('#userGroupsTree').tree('expand', _node.target);
                    angular.element('#userGroupsTree').tree('scrollTo', _node.target);
                }

                writeTreeExpandedState(_node.id, 'open');

                UserAgent.postOneUserGroup(newNode).then(function success(data) {
					newNode.id = data.UserGroup.id;

					angular.element('#userGroupsTree').tree('append', {
						"parent": _node.target,
						"data": newNode
					});

					newNode.target = angular.element('#userGroupsTree').tree('find', newNode.id).target;
					angular.element('#userGroupsTree').tree('beginEdit', newNode.target);
					angular.element(newNode.target).find('input.tree-editor').focus()[0].setSelectionRange(0, newNode.name.length);
                });
            }

            angular.element(contextMenuDiv).addClass('hide');
        };

        $scope.renameUserGroupName = function renameUserGroupName() {
            angular.element(menuID).menu('hide');

			var node = angular.element(menuID).menu('options').ownerTreeNode;
			if(node == null) {
				return false;
			}

            if (node.id != 1) {
                renameYn = true;
                angular.element('#userGroupsTree').tree('beginEdit', node.target);
                angular.element(node.target).find('input.tree-editor').focus()[0].setSelectionRange(0, node.name.length);
                angular.element(contextMenuDiv).addClass('hide');
            }
        };

        $scope.deleteUserGroup = function deleteUserGroup() {
            angular.element(menuID).menu('hide');

			var node = angular.element(menuID).menu('options').ownerTreeNode;
			if(node == null) {
				return false;
			}

            if (node.id != 1) {
                MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                    if (confirmYn) {
                        var delId = node.id;
                        UserAgent.deleteUserGroups(delId);
                    }

                    angular.element(contextMenuDiv).addClass('hide');
                });
            }
        };

        function writeTreeExpandedState(group_id, state) {
            var expandedState = window.sessionStorage.getItem('UserGroupTreeExpandedState');
            if (expandedState) {
                expandedState = angular.fromJson(expandedState);
            } else {
                expandedState = {};
            }
            expandedState[group_id] = state;

            window.sessionStorage.setItem('UserGroupTreeExpandedState', angular.toJson(expandedState));
        }

        var ContextMenu = {
            'ADD_USER_GROUP': {
                menuName: MessageProperties.get('user.addUserGroup'), clickFunctionName: 'addUserGroup'
            },
            'RENAME_USER_GROUP': {
                menuName: MessageProperties.get('user.renameUserGroup'), clickFunctionName: 'renameUserGroupName'
            },
            'DELETE_USER_GROUP': {
                menuName: MessageProperties.get('user.deleteUserGroup'), clickFunctionName: 'deleteUserGroup'
            }
        };

        UserGroupTreeOption = {
            animate: true,
            dnd: !isAndroid,
            formatter: function (node) {
                if (node.id == Const.DEFAULT_ALL_GROUP_ID) {
                    node.name = MessageProperties.get("menu.user.allUserGroups");
                }
                var mans = '<b>' + (node.userTotal || '0') + '</b>';

                if (node.depth === '0') {
                    return '<span class="userGroupTreeNode" data-depth="' + node.depth + '" title="' + node.name + '">' + node.name + mans + '</span>';
                } else {
                    return '<span class="userGroupTreeNode" data-depth="' + node.depth + '" title="' + node.name + '">' + node.name + mans + /*'<mark>' + node.path + '</mark>' +*/ '</span>';
                }
            },
            onExpand: function (node) {
                writeTreeExpandedState(node.id, 'open');
            },
            onCollapse: function (node) {
                writeTreeExpandedState(node.id, 'closed');
            },
            onClick: function (node) {

                Util.setSessionObject('selectedUserGroupId', node.id);
                $rootScope.$broadcast('CurrentSelectedUserGroupName', node);


                if ($scope.contextNode && $scope.contextNode.target) {
                    if (angular.element($scope.contextNode.target).find('input')[0]) {
                        angular.element('#userGroupsTree').tree('endEdit', $scope.contextNode.target);
                    }
                }

                $rootScope.$broadcast('ReloadUsersDataGrid');

            },
            onBeforeDrag: function (node) {
                if (!$scope.isUserMenu()) {
                    return false;
                }
            },
            onBeforeDrop: function (target, source, point) {
                var parent = angular.element('#userGroupsTree').tree('getData', target);

                var depth = 0;

                function findLastChildrenDepth(node) {
                    var i;
                    if (depth < node.depth) {
                        depth = node.depth;
                    }

					if (node.children && node.children.length) {
                        for (i = node.children.length; i--;) {
                            findLastChildrenDepth(node.children[i]);
                        }
                    }
                    return false;
                }

				console.log(source);

                findLastChildrenDepth(source);

                depth = depth - source.depth + 1;

                depth = parseInt(parent.depth) + parseInt(depth);

                if (depth > 8) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.userGroupCanNotExceedSteps'));
                    $rootScope.$broadcast('ReloadUserGroupData');
                    return false;
                } else {

                }
            },
            onDrop: function (target, source, point) {
                angular.element(target).css('top', 'auto');
                var parent = angular.element('#userGroupsTree').tree('getData', target);

                var nodeChangeData = {
                    "id": source.id,
                    "parent_id": {
                        "id": parent.id
                    }
                };

				writeTreeExpandedState(parent.id, 'open');

                UserAgent.putOneUserGroup(nodeChangeData).then(function success(data) {
                }, function error(reason, status) {
                }).finally(function () {
                });
            },
            onBeforeEdit: function (node) {
                node.text = node.name;
            },
            onCancelEdit: function (node) {
                if (node.id == undefined) {
                    angular.element('#userGroupsTree').tree('remove', node.target);
                }
            },
            onAfterEdit: function (node) {

                if (!node.text) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseUserGroupName'));
                    return false;
                } else if (node.text.length == 0) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.pleaseUserGroupName'));
                    return false;
                }

                if (!angular.element(node.target).is(':visible')) {
                    return false;
                }

                if ($.trim(node.text).length <= 0) {
                    return false;
                }

                var nodeChangeData = {
                    "name": node.text.trim()
                };

                // 수정
                nodeChangeData.id = node.id;

                angular.element('#userGroupsTree').tree('update', {target: node.target, name: node.text});

                UserAgent.putOneUserGroup(nodeChangeData).then(function success(data) {
                }, function error(data) {
                }).finally(function (data) {
                    angular.element('#lnbUserGroupTree').scrollLeft(0);
//					$scope.$broadcast('ReloadUserGroupData');
//					$scope.$emit('ReloadUserGroupData');
                });

            },
            onContextMenu: onContextMenu,
			onLoadSuccess: function (node, data) {
				var findNode;
				var nodeId = Util.getSessionObject('selectedUserGroupId');

				if (nodeId) {
					findNode = angular.element('#userGroupsTree').tree('find', nodeId);
				}

				if (!nodeId || !findNode) {
					findNode = angular.element('#userGroupsTree').tree('getRoot');
				}

				if (findNode && findNode.target) {
					angular.element('#userGroupsTree').tree('select', findNode.target);
					UserGroupTreeOption.onClick(findNode);
				}
			}
        };

        angular.element('#userGroupsTree').tree(UserGroupTreeOption);

        $scope.bindUserTreeData = function () {
            UserAgent.getUserGroupRecursiveData('inUserGroupTree', true).then(function success(data) {
                angular.element('#userGroupsTree').tree('loadData', data);
            });
        };

        $scope.$on('ReloadUserGroupData', function () {

            Util.debug('%cReloadUserGroupData', 'background: skyblue; color: green');

            $scope.contextNode = undefined;

            UserAgent.getUserGroupRecursiveData('inUserGroupTreeForReload', true).then(function success(data) {
                angular.element('#userGroupsTree').tree('loadData', data);
            });

			$scope.SelectAllYN = false;
        });

        $scope.doAddUser = function () {
            $scope.$emit('OpenUserInfo');
        };

        $scope.init = function () {
            $scope.bindUserTreeData();


        };

    }]);


});