'use strict';

define([
    'modules/common/services/access-control-agent',
    'modules/common/services/user-agent',
    'modules/common/services/card-agent',
    'modules/common/services/device-agent',
    'modules/user/dialog/finger-print-dlg',
    'modules/common/directives/ngbss-user-auth-mode-list',
    'modules/user/dialog/card-registration-dlg',
    'modules/user/dialog/card-issue-history-dlg'
], function () {
    var app = angular.module('biostarWebApp');
    app.register.controller('UserInfoController', ['$scope', '$rootScope', '$q', '$location', '$timeout', 'API', 'Util', 'Auth', 'UserAgent', 'CardsAgent', 'AccessControlAgent', 'Const', 'MessageProperties', 'MessageService', function ($scope, $rootScope, $q, $location, $timeout, API, Util, Auth, UserAgent, CardsAgent, AccessControlAgent, Const, MessageProperties, MessageService) {
        var
            user = {},
            pageMode = Const.PAGE_MODE.CREATE;


        $scope.PASSWORD_INITSTRING = Const.PASSWORD_INITSTRING;
        $scope.CARD_STATUS = Const.CARD_STATUS;

        //유저 추가를 취소하고 목록페이지로 돌아갑니다.
        function cancelAddUser() {
            window.sessionStorage.removeItem($scope.currUserId);
            $scope.$emit('CloseUserInfo');
        }

        function addUser() {
            var i;

            if ($scope.card && $scope.card.length) {

                API.post('cards', undefined, {"CardCollection": {rows: $scope.card}}).then(function (data) {

                    for (i = $scope.card.length; i--;) {
                        $scope.card[i].id = data.CardCollection.rows[i].id;
                    }

                    if (!$scope.user.cards) {
                        $scope.user.cards = [];
                    }

                    $scope.user.cards = $scope.user.cards.concat($scope.card);
                    $scope.card = undefined;

                    $scope.$emit('SaveUserData');

                });

            } else {
                $scope.$emit('SaveUserData');
            }
        }

        function endUserEdit() {
            $scope.$emit('CloseUserInfo');
            $scope.$emit('UserInfoUpdated', $scope.user);
        }

        function assignData(target, source, field) {
            if (field == 'password' || field == 'pin') {
                if (target[field] != Const.PASSWORD_INITSTRING) {
                    return target[field];
                } else {
                    return undefined;
                }
            } else {
                if (source[field] != undefined && target[field] != undefined) {
                    return compareData(source, target, field);
                } else if (source[field] == undefined && target[field] != undefined) {
                    return target[field];
                } else if (source[field] != undefined && target[field] == undefined) {
                    return undefined;
                }
            }

            return undefined;
        }

        function compareData(source, target, field) {
            if (source[field].constructor != Object && source[field].constructor != Array) {
                if (source[field] != target[field]) {
                    if (target[field].length == 0) {
                        target[field] = '';
                    }
                    return target[field];
                } else {
                    return undefined;
                }
            } else {
                if (angular.equals(source[field], target[field])) {
                    return undefined;
                } else {
                    return target[field];
                }
            }
        }

        function assignUserData() {
            getUserInfo($scope.user.user_id);
        }

        $scope.$on('SaveUserData', function (event, data) {

            var item,
                nameRegExp = /^[a-zA-Z0-9-_]+$/gi,
                saveData = {},
                originalData = Util.getSessionObject('userInfo');

            saveData.name = assignData($scope.user, originalData, 'name');
            saveData.email = assignData($scope.user, originalData, 'email');
            saveData.photo = assignData($scope.user, originalData, 'photo');
            saveData.phone = assignData($scope.user, originalData, 'phone');
            saveData.user_id = assignData($scope.user, originalData, 'user_id');
            saveData.user_group_id = assignData($scope.user, originalData, 'user_group_id');
            saveData.disabled = assignData($scope.user, originalData, 'disabled');
            saveData.start_datetime = assignData($scope.user, originalData, 'start_datetime');
            saveData.expiry_datetime = assignData($scope.user, originalData, 'expiry_datetime');
            if ($scope.pageMode == Const.PAGE_MODE.CREATE) {
                if (saveData.start_datetime == undefined) {
                    saveData.start_datetime = Const.DEFAULT_START_DATE;
                }
                if (saveData.expiry_datetime == undefined) {
                    saveData.expiry_datetime = Const.DEFAULT_EXPIRY_DATE;
                }
            }
            saveData.permission = assignData($scope.user, originalData, 'permission');
            saveData.access_groups = assignData($scope.user, originalData, 'access_groups');
            saveData.login_id = assignData($scope.user, originalData, 'login_id');
            saveData.password = assignData($scope.user, originalData, 'password');
            saveData.pin = assignData($scope.user, originalData, 'pin');
            saveData.security_level = assignData($scope.user, originalData, 'security_level');
            saveData.fingerprint_templates = assignData($scope.user, originalData, 'fingerprint_templates');
            saveData.cards = assignData($scope.user, originalData, 'cards');

            saveData.private_operation_modes = assignData($scope.user, originalData, 'private_operation_modes');
            if (saveData.private_operation_modes) {
                var private_operation_modes = [];
                for (var i = 0; i < saveData.private_operation_modes.length; i++) {
                    item = saveData.private_operation_modes[i];

                    private_operation_modes.push({
                        index: i,
                        user_id: $scope.user.user_id,
                        operation_method: item.operation_method,
                        operation_mode: item.operation_mode
                    });
                }

                if (!private_operation_modes.length && originalData.private_operation_modes && !originalData.private_operation_modes.length) {
                    private_operation_modes = undefined;
                }

                saveData.private_operation_modes = private_operation_modes;
            }


            if (!originalData.private_operation_modes && $scope.user.private_operation_modes && $scope.user.private_operation_modes.length == 0) {
                delete saveData.private_operation_modes;
            }

            if (!originalData.access_groups && $scope.user.access_groups && $scope.user.access_groups.length == 0) {
                delete saveData.access_groups;
            }

            if (!originalData.permission && $scope.user.permission && Object.keys($scope.user.permission).length == 0) {
                delete saveData.permission;
            }

            for (var key in saveData) {
                item = saveData[key];
                if (item == undefined) {
                    delete saveData[key];
                }
            }

            if (saveData.user_id && nameRegExp.test(saveData.user_id) == false) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.invalidUserId'));
                return;
            } else if (!saveData.user_id && !$scope.user.user_id) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.invalidUserId'));
                return;
            }

            if ($scope.userInfoForm.$error.email) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.invalidEmail'));
                return;
            }

            if ($scope.user.password && $scope.user.password != Const.PASSWORD_INITSTRING && $scope.user.password.length > 0 && $scope.user.password != $scope.user.confirm_password) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.invalidPassword'));
                return;
            }

            if ($scope.UsePIN == Const.BOOLEAN.TRUE && $scope.user.pin != Const.PASSWORD_INITSTRING && $scope.user.pin / 1 != $scope.user.pin) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.invalidPin'));
                return;
            }

            if ($scope.UsePIN == Const.BOOLEAN.TRUE && $scope.user.pin && $scope.user.pin != Const.PASSWORD_INITSTRING && $scope.UsePIN == Const.BOOLEAN.TRUE && $scope.user.pin.length > 0 && $scope.user.pin != $scope.user.confirm_pin) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.invalidPin'));
                return;
            }

            if ($scope.user.permission && Object.keys($scope.user.permission).length > 0) {
                if (!$scope.user.login_id || !$scope.user.login_id.length) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.invalidLoginId'));
                    return;
                }
                if (!$scope.user.password || !$scope.user.password.length) {
                    MessageService.displayWarningMessage(MessageProperties.get('validate.invalidPassword'));
                    return;
                }
            }


            if (saveData.fingerprint_templates && saveData.fingerprint_templates.length > 0) {

                for (i = 0; i < saveData.fingerprint_templates.length; i++) {
                    item = saveData.fingerprint_templates[i];

                    delete item.template0Img;
                    delete item.template1Img;
                }
            }

//            if($scope.checkUserIdDuplicationOK == false) {
//                MessageService.displayWarningMessage(MessageProperties.get('validate.invalidUserId'));
//                return;
//            }


            if ($scope.UsePIN == Const.BOOLEAN.FALSE) {
                saveData.pin = '';
            }

            if (saveData.pin && saveData.pin.length < 4) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.invalidPinMinLength'));
                return;
            }

            if (saveData.permission && Object.keys(saveData.permission).length <= 0) {

                saveData.login_id = '';
                saveData.password = '';

                if (pageMode == Const.PAGE_MODE.CREATE) {
                    saveData.login_id = undefined;
                    saveData.password = undefined;
                }
            }

            if (!saveData.permission && pageMode == Const.PAGE_MODE.CREATE) {
                saveData.login_id = undefined;
                saveData.password = undefined;
            }

            if (saveData.fingerprint_templates && saveData.fingerprint_templates.length > 0) {
                for (i = saveData.fingerprint_templates.length; i--;) {
                    item = saveData.fingerprint_templates[i];
                    delete item.template_image0;
                    delete item.template_image1;
                    delete item.raw_image0;
                    delete item.raw_image1;
                }
            }

            if (Object.keys(saveData).length > 0) {
                if (pageMode == Const.PAGE_MODE.CREATE) {

                    if (angular.isNumber(1 * saveData.user_id) == false) {
                        MessageService.displayWarningMessage(MessageProperties.get('validate.onlyNumbersAreAllowed'));
                        return;
                    } else {
                        if (saveData.user_id > Const.USER_ID_MAX_VALUE) {
                            MessageService.displayWarningMessage(MessageProperties.get('validate.userIdMaximumNumber', Const.USER_ID_MAX_VALUE));
                            return;
                        }
                    }

                    UserAgent.postOneUser(saveData).then(function success(data) {
                        endUserEdit();
                    }, function error(data) {

                    });
                } else if (pageMode == Const.PAGE_MODE.UPDATE) {
                    UserAgent.putOneUser({"user_id": $scope.currUserId, "user": saveData}).then(function success(data) {
                        endUserEdit();
                    }, function error(data) {

                    })
                }
            } else {
//               MessageService.displayConfirmMessage(MessageProperties.get('confirm.noChangesContinue'), function (confirmYn) {
//                    if (confirmYn) {
                $scope.$emit('CloseUserInfo');
                if (!$scope.$root.$$phase) $scope.$apply();
//                    }
//                });
            }

        });

        UserAgent.getUserGroupRecursiveData().then(function success(data) {
            $scope.UserTreeData = data;
        });

        $scope.user = Util.getSessionObject('userInfo');

        $scope.currentUser = Auth.getUser();

        // User 정보가 sessionStorage에 있다면..
        if ($scope.user.user_id) {
            pageMode = Const.PAGE_MODE.UPDATE;

            $scope.currUserId = $scope.user.user_id;
            $scope.currLoginId = $scope.user.login_id;
            $scope.currUserName = $scope.user.name;

            $scope.checkUserIdDuplicationOK = true;
            $scope.checkLoginIdDuplicationOK = true;

            if ($scope.user.password_exists == Const.BOOLEAN.TRUE) {
				$scope.user.password = $scope.PASSWORD_INITSTRING;
				$scope.user.confirm_password = $scope.PASSWORD_INITSTRING;
            }

            if ($scope.user.pin_exists == Const.BOOLEAN.TRUE) {
				$scope.user.pin = $scope.PASSWORD_INITSTRING;
				$scope.user.confirm_pin = $scope.PASSWORD_INITSTRING;
                $scope.UsePIN = Const.BOOLEAN.TRUE;
            } else {
                $scope.UsePIN = Const.BOOLEAN.FALSE;
            }

			if ($scope.user.private_operation_modes && $scope.user.private_operation_modes.length > 0) {
				$scope.deviceDefaultOK = Const.BOOLEAN.FALSE;
			} else {
				$scope.deviceDefaultOK = Const.BOOLEAN.TRUE;
			}

            checkOperatorEditPermission(user);
        } else { // userId가 파라미터로 넘어오지 않았다면 추가 모드
            pageMode = Const.PAGE_MODE.CREATE;

            $scope.checkUserIdDuplicationOK = false;
            $scope.checkLoginIdDuplicationOK = false;

            // ID 자동 생성(User ID & Login Id)
//            UserAgent.getAutoGenerateUserId().then(function success(data) {
//                $scope.user.user_id = data;
//                $scope.user.login_id = data;
//            });

			$scope.user.user_group_id = {};
			$scope.user.user_group_id.id = window.sessionStorage.getItem('selectedUserGroupId');

            window.sessionStorage.removeItem('userCurrPos');

            $scope.UsePIN = Const.BOOLEAN.FALSE;
        }

        $scope.pinUseOK = false;

        $scope.pageMode = pageMode;

        if ($scope.user.disabled == undefined) {
            $scope.user.disabled = Const.BOOLEAN.FALSE;
        }

        $scope.FINGERPRINT_SECURITY_LEVEL = Const.FINGERPRINT_SECURITY_LEVEL;

        $scope.addUser = addUser;
        $scope.cancelAddUser = cancelAddUser;

        $scope.PermissionData = undefined;
        API.get('permissions').then(function (data) {
            $scope.PermissionData = data.PermissionCollection.rows;
        });

        $scope.userInfoFormValidStatus = false;
        $scope.$watch('userInfoForm.$invalid', function (nv, ov) {
            if (nv !== ov) {
                $scope.userInfoFormValidStatus = nv;
            }
        });


        $scope.fingerPrintScanDlg = {};


        $scope.doDeleteAllFingerprint = function () {
            MessageService.displayConfirmMessage(MessageProperties.get('validate.deleteAllFingerprints'), function (confirmYn) {
                if (confirmYn) {
                    $scope.user.fingerprint_templates = [];
                    if (!$scope.$root.$$phase) $scope.$apply();
                }
            });
        };

        $scope.doOpenFingerPrintScanDlg = function () {
//
//            if ($scope.userInfoFormValidStatus) {
//                return false;
//            }

            $scope.fingerPrintScanDlg = {};
            $scope.fingerPrintScanDlg = {url: 'modules/user/dialog/finger-print-dlg.html'};
        };

        $scope.$on('closeFingerPrintScanDlg', function () {
            $scope.fingerPrintScanDlg = {url: ''};
            angular.element('#fingerDlg').dialog('destroy');
        });

        $scope.doAfterOpenFingerPrintScanDlg = function () {
            var options = {};
            options.iconCls = 'icon-save';
            options.modal = true;
            options.shadow = false;
            options.border = false;
            options.title = MessageProperties.get('user.header.fingerprint');
            options.width = 580;
            options.height = 'auto';
//            options.top = 200;

            options.onClose = function () {
                $scope.fingerPrintScanDlg = {url: ''};
                angular.element('#fingerDlg').dialog('destroy');

                console.log($scope.user.fingerprint_templates);

                if (!$scope.$root.$$phase) $scope.$apply();
            };

            $("#fingerDlg").dialog(options);

        };

        $scope.card = undefined;
        $scope.cardRegistration = {};
        $scope.doOpenCardRegistrationDlg = function () {

            var cards = 0;
            if ($scope.card && $scope.card.length) {
                cards += $scope.card.length;
            }
            if ($scope.user.cards && $scope.user.cards.length) {
                cards += $scope.user.cards.length;
            }

//            if ($scope.userInfoFormValidStatus) {
//                return false;
//            }

            if (cards >= 8) {
                MessageService.displayWarningMessage(MessageProperties.get('validate.canNotAssignMoreCard'));
                return false;
            }

            $scope.cardRegistration = {};
            $scope.cardRegistration = {url: 'modules/user/dialog/card-registration-dlg.html'};
        };

        $scope.doDeleteCardInfo = function ($index, more) {
            MessageService.displayConfirmMessage(MessageProperties.get('confirm.delete'), function (confirmYn) {
                if (confirmYn) {

                    if (more) {
                        $scope.card.splice($index, 1);
                    } else {
                        $scope.user.cards.splice($index, 1);
                    }


                    if (!$scope.$root.$$phase) $scope.$apply();
                }
            });
        };

        $scope.$on('CopyCardInfoForPost', function (event, data) {
            if (!$scope.card) {
                $scope.card = [];
            }
            $scope.card.push(angular.copy(data));
        });

        $scope.$on('closeCardRegistrationDlg', function () {
            $scope.cardRegistration = {url: ''};
            angular.element('#cardRegistrationDlg').dialog('destroy');
        });

        $scope.doAfterOpenCardRegistrationDlg = function () {
            var options = {};
            options.iconCls = 'icon-save';
            options.modal = true;
            options.shadow = false;
            options.border = false;
            options.title = MessageProperties.get('user.header.card');
            options.width = 580;
            options.height = 'auto';
//            options.top = 200;

            options.onClose = function () {
                $scope.cardRegistration = {url: ''};
                angular.element('#cardRegistrationDlg').dialog('destroy');
                if (!$scope.$root.$$phase) $scope.$apply();
            };

            $("#cardRegistrationDlg").dialog(options);

        };


        $scope.cardIssueHistoryDlg = {};

        $scope.doOpenCardIssueHistoryDlg = function () {
            $scope.cardIssueHistoryDlg = {url: 'modules/user/dialog/card-issue-history-dlg.html'};
        };

        $scope.$on('closeCardIssueHistoryDlg', function () {
            $scope.cardIssueHistoryDlg = {url: ''};
            angular.element('#cardIssueHistoryDlg').dialog('destroy');
        });

        $scope.doAfterOpenCardIssueHistoryDlg = function () {
            var options = {};
            options.iconCls = 'icon-save';
            options.modal = true;
            options.shadow = false;
            options.border = false;
            options.title = MessageProperties.get('user.header.cardHistory');
            options.width = 780;
            options.height = 'auto';
            options.top = 200;

            options.onClose = function () {
                $scope.$emit('closeCardIssueHistoryDlg');
                if (!$scope.$root.$$phase) $scope.$apply();
            };

            $("#cardIssueHistoryDlg").dialog(options);

        };

        function photoFileUpload($event) {
            var event = $event || window.event,
                target = event.target || (event.srcElement || document),
                files = target.files,
                type = "default",
                reader = new FileReader(),
                url = URL.createObjectURL(files[0]),
                image = new Image(),
                canvas = document.getElementById("photoCanvas"),
                ctx = canvas.getContext("2d");

            if (/image/.test(files[0].type)) {
                image.src = url;
                reader.readAsDataURL(files[0]);
            } else {
                MessageService.displayWarningMessage(MessageProperties.get('validate.onlyImageFilesAreSupported'));
            }
            image.onload = function () {
                ctx.drawImage(image, 0, 0, canvas.width, canvas.height);

                $scope.user.photo = canvas.toDataURL("image/jpeg").replace(/(^data:[a-z/;]+base64,)/g, '');
                if (!$scope.$root.$$phase) $scope.$apply();
            };
        }

        angular.element('#photoFile').on('change', photoFileUpload);
        angular.element('#changePhotoFile').on('change', photoFileUpload);
        angular.element('#deletePhotoFile').on('click', function () {
            $scope.user.photo = '';
            if (!$scope.$root.$$phase) $scope.$apply();
        });

        $scope.$on('getUserId', function (event, cbFunction) {
            cbFunction($scope.currUserId);
        });


        $scope.$on('$destroy', function () {
            window.sessionStorage.removeItem('userInfo');
        });

        $scope.currPos = window.sessionStorage.getItem('userCurrPos') || undefined;

        $scope.prevUserInfoBtn = function () {
            var currPos = window.sessionStorage.getItem('userCurrPos');
            var prevIndex = currPos - 2;
            var allUserIdList = window.sessionStorage.getItem('getAllUserIdList');

            if (allUserIdList && currPos) {
                allUserIdList = angular.fromJson(allUserIdList);
            } else {
                cancelAddUser();
                return false;
            }

            if (prevIndex < 0) {
                return false;
            }

            $scope.currPos = parseInt(currPos) - 1;

            window.sessionStorage.setItem('userCurrPos', $scope.currPos);

            if (allUserIdList[prevIndex]) {
                getUserInfo(allUserIdList[prevIndex]);
            } else {
                getAllUserIdList(prevIndex);
            }
        };

        $scope.nextUserInfoBtn = function () {
            var currPos = window.sessionStorage.getItem('userCurrPos');
            var nextIndex = currPos;
            var allUserIdList = window.sessionStorage.getItem('getAllUserIdList');

            if (allUserIdList && currPos) {
                allUserIdList = angular.fromJson(allUserIdList);
            } else {
                cancelAddUser();
                return false;
            }

            if (currPos >= allUserIdList.length) {
                return false;
            }

            $scope.currPos = parseInt(currPos) + 1;

            window.sessionStorage.setItem('userCurrPos', $scope.currPos);

            if (allUserIdList[nextIndex]) {
                getUserInfo(allUserIdList[nextIndex]);
            } else {
                getAllUserIdList(nextIndex);
            }

        };

        function getAllUserIdList(index) {
            var i, allUserIdList, startIdx;
            var limit = angular.copy($scope.userListQueryStrings.limit);

            allUserIdList = window.sessionStorage.getItem('getAllUserIdList');

            if (allUserIdList) {
                allUserIdList = angular.fromJson(allUserIdList);
            } else {
                allUserIdList = [];
            }

            $scope.userListQueryStrings.offset = Math.floor(index / limit) * limit;


            startIdx = parseInt($scope.userListQueryStrings.offset);

            UserAgent.getUsers($scope.userListQueryStrings, 'getAllUserIdList').then(function success(data) {
                if (allUserIdList.length != data.UserCollection.total) {
                    allUserIdList = [];
                }

                limit = parseInt($scope.userListQueryStrings.limit);

                if (parseInt($scope.userListQueryStrings.offset) + parseInt($scope.userListQueryStrings.limit) > data.UserCollection.total) {
                    limit = parseInt(data.UserCollection.total) - $scope.userListQueryStrings.offset;
                }

                for (i = 0; i < limit; i++) {
                    allUserIdList[startIdx + i] = data.UserCollection.rows[i].user_id;
                }

                getUserInfo(allUserIdList[index]);

                window.sessionStorage.setItem('getAllUserIdList', angular.toJson(allUserIdList));
            });
        }

        function getUserInfo(user_id) {
            UserAgent.getOneUser(user_id).then(function success(data) {
                $scope.user = angular.copy(data.User);
                window.sessionStorage.setItem('userInfo', angular.toJson($scope.user));
                $scope.currUserId = $scope.user.user_id;
                $scope.currLoginId = $scope.user.login_id;
                $scope.currUserName = $scope.user.name;

                if ($scope.user.password_exists == Const.BOOLEAN.TRUE) {
                    $scope.user.password = $scope.PASSWORD_INITSTRING;
                    $scope.user.confirm_password = $scope.PASSWORD_INITSTRING;
                }

                if ($scope.user.pin_exists == Const.BOOLEAN.TRUE) {
                    $scope.user.pin = $scope.PASSWORD_INITSTRING;
                    $scope.user.confirm_pin = $scope.PASSWORD_INITSTRING;
                    $scope.UsePIN = Const.BOOLEAN.TRUE;
                } else {
                    $scope.UsePIN = Const.BOOLEAN.FALSE;
                }

				if ($scope.user.private_operation_modes && $scope.user.private_operation_modes.length > 0) {
					$scope.deviceDefaultOK = Const.BOOLEAN.FALSE;
				} else {
					$scope.deviceDefaultOK = Const.BOOLEAN.TRUE;
				}

                checkOperatorEditPermission($scope.user);
            });
        }

        function checkOperatorEditPermission(user) {
            /**
             * 1. 현재 사용자가 Administrator이면 모두 수정 가능
             * 2. 조회중인 사용자가 Administrator면 수정 불가
             * 3. 현재 사용자가 Administrator권한이면 모두 수정가능
             * 4. User Operator는 Administrator 권한 수정 못함.
             */


            if ($scope.currentUser.permission && $scope.currentUser.permission.id == Const.PERMISSION_IDS.ADMINISTRATOR) {
                $scope.adminOnly = $scope.user.user_id == Const.ADMINISTRATOR_DEFAULT_ID
            } else if ($scope.currentUser.permission && $scope.currentUser.permission.id == Const.PERMISSION_IDS.USER) {
//                $scope.adminOnly = user.permission && user.permission.id == Const.PERMISSION_IDS.ADMINISTRATOR;
                $scope.adminOnly = true;
            } else {
                $scope.adminOnly = false;
            }
        }

        $scope.focusPasswordInputBox = function () {
            if ($scope.user.password_exists == Const.BOOLEAN.TRUE && $scope.user.password == $scope.PASSWORD_INITSTRING) {
                $scope.user.password = '';
                $scope.user.confirm_password = '';
                console.log($scope.user.password_exists, $scope.user.password)
            }
        };

        $scope.blurPasswordInputBox = function () {
            if ($scope.user.password_exists == Const.BOOLEAN.TRUE && $scope.user.password.length == 0) {
                $scope.user.password = $scope.PASSWORD_INITSTRING;
                $scope.user.confirm_password = $scope.PASSWORD_INITSTRING;
            }
        };

        $scope.focusPinInputBox = function () {
            if ($scope.user.pin_exists == Const.BOOLEAN.TRUE && $scope.user.pin == $scope.PASSWORD_INITSTRING) {
                $scope.user.pin = '';
                $scope.user.confirm_pin = '';
            }
        };

        $scope.blurPinInputBox = function () {
            if ($scope.user.pin_exists == Const.BOOLEAN.TRUE && $scope.user.pin.length == 0) {
                $scope.user.pin = $scope.PASSWORD_INITSTRING;
                $scope.user.confirm_pin = $scope.PASSWORD_INITSTRING;
            }
        };

        $scope.doToggleDeviceDefault = function () {
            if ($scope.deviceDefaultOK == Const.BOOLEAN.FALSE) {
                $scope.user.private_operation_modes = [];
            } else if ($scope.deviceDefaultOK == Const.BOOLEAN.TRUE) {
                $scope.user.private_operation_modes = [];
            }
        };

        $scope.doCardBlocked = function (blockedYn, card) {
            var blacklist = {};
            blacklist.Blacklist = {};
            blacklist.Blacklist.card_id = {};
            blacklist.Blacklist.card_id.id = card.id;
            if (blockedYn) {
                CardsAgent.postBlacklist(blacklist, assignUserData)
            } else if (!blockedYn) {
                CardsAgent.deleteOneBlacklist(card.id, assignUserData)
            }
        };

        $scope.doChangeUsePINBox = function () {
            if ($scope.UsePIN == Const.BOOLEAN.FALSE) {
                $scope.user.pin = '';
                $scope.user.confirm_pin = '';
            } else {
                if ($scope.user.pin_exists == Const.BOOLEAN.TRUE) {
                    $scope.user.pin = Const.PASSWORD_INITSTRING;
                }
            }
        };

        $scope.$watch('UsePIN', function (nv, ov) {
            console.log(nv, ov);
//            if(nv == Const.BOOLEAN.FALSE) {
//                $scope.user.pin = '';
//                $scope.user.confirm_pin = '';
//            } else if(nv == Const.BOOLEAN.TRUE) {
//                if($scope.user.pin_exists == Const.BOOLEAN.TRUE) {
//                    $scope.user.pin = Const.PASSWORD_INITSTRING;
//                }
//            }
        });

        $scope.$watch('user', function (nv, ov) {

            if (nv != ov) {
                if (angular.equals(nv.permission, ov.permission) == false) {
                    if (nv.permission && Object.keys(nv.permission).length <= 0) {
                        $scope.user.password = '';
                        $scope.user.confirm_password = '';
                    } else {
                        if ($scope.user.password_exists == Const.BOOLEAN.TRUE) {
                            $scope.user.password = Const.PASSWORD_INITSTRING;
                        }
                    }
                }
            }
        }, true);


        $scope.fieldDisableCondition = function() {
            return ($scope.user.permission && $scope.user.permission.id == Const.PERMISSION_IDS.ADMINISTRATOR && $scope.currentUser.permission.id != Const.PERMISSION_IDS.ADMINISTRATOR) || ($scope.user.user_id == Const.ADMINISTRATOR_DEFAULT_ID && $scope.currentUser.user_id != Const.ADMINISTRATOR_DEFAULT_ID)
        }
    }]);
});